@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019 Apple Inc. All rights reserved.
+ * Copyright (C) 2019-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,116 +31,20 @@
 #include "B3BasicBlockInlines.h"
 #include "B3BlockInsertionSet.h"
 #include "B3ConstPtrValue.h"
-#include "B3EnsureLoopPreHeaders.h"
 #include "B3InsertionSet.h"
 #include "B3NaturalLoops.h"
 #include "B3PhaseScope.h"
 #include "B3ProcedureInlines.h"
 #include "B3ValueInlines.h"
-#include "B3Variable.h"
-#include "B3VariableValue.h"
-#include <wtf/GraphNodeWorklist.h>
-#include <wtf/IndexSet.h>
-#include <wtf/SmallPtrSet.h>
+#include <wtf/SmallSet.h>
 #include <wtf/Vector.h>
 
 namespace JSC { namespace B3 {
 
 #if CPU(X86_64)
-/*
- * The semantics of B3 require that loads and stores are not reduced in granularity.
- * If we would use system memcpy, then it is possible that we would get a byte copy loop,
- * and this could cause GC tearing.
- */
-void fastForwardCopy32(uint32_t* dst, const uint32_t* src, size_t size)
+JSC_DEFINE_JIT_OPERATION(operationFastForwardCopy32, void, (uint32_t* dst, const uint32_t* src, size_t size))
 {
-    uint64_t i;
-    int64_t counter;
-    uint64_t tmp, tmp2;
-
-    asm volatile(
-        "test %q[size], %q[size]\t\n"
-        "jz 900f\t\n" // exit
-        "xorq %q[i], %q[i]\t\n"
-        // if size < 8
-        "cmp $8, %q[size]\t\n"
-        "jl 100f\t\n" // backporch
-        // if (dst + size*4 <= src || src + size*4 <= dst)
-        "leaq (%q[src], %q[size], 4), %q[tmp]\t\n"
-        "cmp %q[tmp], %q[dst]\t\n"
-        "jae 500f\t\n" // simd no overlap
-        "leaq (%q[dst], %q[size], 4), %q[tmp]\t\n"
-        "cmp %q[tmp], %q[src]\t\n"
-        "jae 500f\t\n" // simd no overlap
-        "jmp 100f\t\n" // Backporch
-
-        // Backporch (assumes we can write at least one int)
-        "100:\t\n"
-        // counter = (size - i)%4
-        "mov %q[size], %q[counter]\t\n"
-        "subq %q[i], %q[counter]\t\n"
-
-        // If we are a multiple of 4, unroll the loop
-        // We already know we are nonzero
-        "and $3, %q[counter]\t\n"
-        "jz 200f\t\n" // unrolled loop
-
-        "negq %q[counter]\t\n"
-        ".balign 32\t\n"
-        "101:\t\n"
-        "mov (%q[src], %q[i], 4), %k[tmp]\t\n"
-        "mov %k[tmp], (%q[dst], %q[i], 4)\t\n"
-        "incq %q[i]\t\n"
-        "incq %q[counter]\t\n"
-        "jnz 101b\t\n"
-        // Do we still have anything left?
-        "cmpq %q[size], %q[i]\t\n"
-        "je 900f\t\n" // exit
-        // Then go to the unrolled loop
-        "jmp 200f\t\n"
-
-        // Unrolled loop (assumes multiple of 4, can do one iteration)
-        "200:\t\n"
-        "shr $2, %q[counter]\t\n"
-        "negq %q[counter]\t\n"
-        ".balign 32\t\n"
-        "201:\t\n"
-        "mov (%q[src], %q[i], 4), %k[tmp]\t\n"
-        "mov %k[tmp], (%q[dst], %q[i], 4)\t\n"
-        "mov 4(%q[src], %q[i], 4), %k[tmp2]\t\n"
-        "mov %k[tmp2], 4(%q[dst], %q[i], 4)\t\n"
-        "mov 8(%q[src], %q[i], 4), %k[tmp2]\t\n"
-        "mov %k[tmp2], 8(%q[dst], %q[i], 4)\t\n"
-        "mov 12(%q[src], %q[i], 4), %k[tmp]\t\n"
-        "mov %k[tmp], 12(%q[dst], %q[i], 4)\t\n"
-        "addq $4, %q[i]\t\n"
-        "cmpq %q[size], %q[i]\t\n"
-        "jb 201b\t\n"
-        "jmp 900f\t\n" // exit
-
-        // simd no overlap
-        // counter = -(size/8);
-        "500:\t\n"
-        "mov %q[size], %q[counter]\t\n"
-        "shrq $3, %q[counter]\t\n"
-        "negq %q[counter]\t\n"
-        ".balign 32\t\n"
-        "501:\t\n"
-        "movups (%q[src], %q[i], 4), %%xmm0\t\n"
-        "movups 16(%q[src], %q[i], 4), %%xmm1\t\n"
-        "movups %%xmm0, (%q[dst], %q[i], 4)\t\n"
-        "movups %%xmm1, 16(%q[dst], %q[i], 4)\t\n"
-        "addq $8, %q[i]\t\n"
-        "incq %q[counter]\t\n"
-        "jnz 501b\t\n"
-        // if (i == size)
-        "cmpq %q[size], %q[i]\t\n"
-        "je 900f\t\n" // end
-        "jmp 100b\t\n" // backporch
-        "900:\t\n"
-    : [i] "=&c" (i), [counter] "=&r" (counter), [tmp] "=&a" (tmp),  [tmp2] "=&r" (tmp2)
-    : [dst] "D" (dst), [src] "S" (src), [size] "r" (size)
-    : "xmm0", "xmm1");
+    return fastForwardCopy32(dst, src, size);
 }
 #endif
 
@@ -189,10 +93,10 @@ public:
         if (verbose)
             dataLogLn("Found store/load:", *store);
 
-        auto extractArrayInfo = [&] (Value* value) -> Optional<AddrInfo> {
+        auto extractArrayInfo = [&] (Value* value) -> std::optional<AddrInfo> {
             patternMatchedValues.add(value);
 
-            Optional<AddrInfo> addr = { AddrInfo() };
+            std::optional<AddrInfo> addr = { AddrInfo() };
 
             Value* sum = value;
             if (value->opcode() == ZExt32)
@@ -255,7 +159,7 @@ public:
         BasicBlock* loopFoot = nullptr;
         BasicBlock* loopPreheader = nullptr;
         BasicBlock* loopPostfooter = nullptr;
-        SmallPtrSet<BasicBlock*> loopInnerBlocks;
+        SmallSet<BasicBlock*> loopInnerBlocks;
 
         {
             for (unsigned i = 0; i < loop->size(); ++i)
@@ -267,8 +171,8 @@ public:
             if (!loopInnerBlocks.contains(store->owner))
                 return;
 
-            SmallPtrSet<BasicBlock*> loopEntrances;
-            SmallPtrSet<BasicBlock*> loopExits;
+            SmallSet<BasicBlock*> loopEntrances;
+            SmallSet<BasicBlock*> loopExits;
             for (auto* block : loopInnerBlocks) {
                 for (auto successor : block->successors()) {
                     if (!loopInnerBlocks.contains(successor.block()))
@@ -310,8 +214,11 @@ public:
 
             for (BasicBlock* predecessor : loopPostfooter->predecessors()) {
                 if (loopInnerBlocks.contains(predecessor)) {
-                    // There is only one loop exit.
-                    RELEASE_ASSERT(!loopFoot);
+                    if (loopFoot) {
+                        if (verbose)
+                            dataLogLn("Loop has more than one exit point");
+                        return;
+                    }
                     loopFoot = predecessor;
                 }
             }
@@ -444,7 +351,7 @@ public:
 
         Effects effects = Effects::forCall();
         memcpy->appendNew<CCallValue>(m_proc, B3::Void, origin, effects,
-            memcpy->appendNew<ConstPtrValue>(m_proc, origin, tagCFunctionPtr<void*>(fastForwardCopy32, B3CCallPtrTag)),
+            memcpy->appendNew<ConstPtrValue>(m_proc, origin, tagCFunction<OperationPtrTag>(operationFastForwardCopy32)),
             destination->appendAddr(m_proc, memcpy, destination->arrayBase),
             source->appendAddr(m_proc, memcpy, source->arrayBase),
             loopBound);
