@@ -26,32 +26,41 @@
 
 // @internal
 
-@globalPrivate
-function newPromiseReaction(promiseOrCapability, onFulfilled, onRejected)
+@linkTimeConstant
+function pushNewPromiseReaction(thenable, existingReactions, promiseOrCapability, onFulfilled, onRejected)
 {
     "use strict";
 
-    return {
-        @promiseOrCapability: promiseOrCapability,
-        @onFulfilled: onFulfilled,
-        @onRejected: onRejected,
-        @next: @undefined,
-    };
+    if (!existingReactions) {
+        existingReactions = {
+            @promiseOrCapability: promiseOrCapability,
+            @onFulfilled: onFulfilled,
+            @onRejected: onRejected,
+            // This is 3x then number of out of line reactions (promise, fulfill callback, reject callback).
+            @outOfLineReactionCounts: 0,
+        };
+        @putPromiseInternalField(thenable, @promiseFieldReactionsOrResult, existingReactions);
+    } else {
+        var outOfLineReactionCounts = existingReactions.@outOfLineReactionCounts;
+        @putByValDirect(existingReactions, outOfLineReactionCounts++, promiseOrCapability);
+        @putByValDirect(existingReactions, outOfLineReactionCounts++, onFulfilled);
+        @putByValDirect(existingReactions, outOfLineReactionCounts++, onRejected);
+        existingReactions.@outOfLineReactionCounts = outOfLineReactionCounts;
+    }
 }
 
-@globalPrivate
+@linkTimeConstant
 function newPromiseCapabilitySlow(constructor)
 {
+    "use strict";
+
     var promiseCapability = {
         @resolve: @undefined,
         @reject: @undefined,
         @promise: @undefined,
     };
 
-    if (!@isConstructor(constructor))
-        @throwTypeError("promise capability requires a constructor function");
-
-    var promise = new constructor(function (resolve, reject) {
+    var promise = new constructor((resolve, reject) => {
         if (promiseCapability.@resolve !== @undefined)
             @throwTypeError("resolve function is already set");
         if (promiseCapability.@reject !== @undefined)
@@ -61,10 +70,10 @@ function newPromiseCapabilitySlow(constructor)
         promiseCapability.@reject = reject;
     });
 
-    if (typeof promiseCapability.@resolve !== "function")
+    if (!@isCallable(promiseCapability.@resolve))
         @throwTypeError("executor did not take a resolve function");
 
-    if (typeof promiseCapability.@reject !== "function")
+    if (!@isCallable(promiseCapability.@reject))
         @throwTypeError("executor did not take a reject function");
 
     promiseCapability.@promise = promise;
@@ -72,7 +81,7 @@ function newPromiseCapabilitySlow(constructor)
     return promiseCapability;
 }
 
-@globalPrivate
+@linkTimeConstant
 function newPromiseCapability(constructor)
 {
     "use strict";
@@ -92,25 +101,46 @@ function newPromiseCapability(constructor)
     return @newPromiseCapabilitySlow(constructor);
 }
 
-@globalPrivate
+@linkTimeConstant
+function promiseResolve(constructor, value)
+{
+    "use strict";
+
+    if (@isPromise(value) && value.constructor === constructor)
+        return value;
+
+    if (constructor === @Promise) {
+        var promise = @newPromise();
+        @resolvePromiseWithFirstResolvingFunctionCallCheck(promise, value);
+        return promise;
+    }
+
+    return @promiseResolveSlow(constructor, value);
+}
+
+@linkTimeConstant
 function promiseResolveSlow(constructor, value)
 {
+    "use strict";
+
     @assert(constructor !== @Promise);
     var promiseCapability = @newPromiseCapabilitySlow(constructor);
     promiseCapability.@resolve.@call(@undefined, value);
     return promiseCapability.@promise;
 }
 
-@globalPrivate
+@linkTimeConstant
 function promiseRejectSlow(constructor, reason)
 {
+    "use strict";
+
     @assert(constructor !== @Promise);
     var promiseCapability = @newPromiseCapabilitySlow(constructor);
     promiseCapability.@reject.@call(@undefined, reason);
     return promiseCapability.@promise;
 }
 
-@globalPrivate
+@linkTimeConstant
 function newHandledRejectedPromise(error)
 {
     "use strict";
@@ -120,30 +150,27 @@ function newHandledRejectedPromise(error)
     return promise;
 }
 
-@globalPrivate
+@linkTimeConstant
 function triggerPromiseReactions(state, reactions, argument)
 {
     "use strict";
 
-    // Reverse the order of singly-linked-list.
-    var previous = @undefined;
-    var current = reactions;
-    while (current) {
-        var next = current.@next;
-        current.@next = previous;
-        previous = current;
-        current = next;
-    }
-    reactions = previous;
+    if (!reactions)
+        return;
 
-    current = reactions;
-    while (current) {
-        @enqueueJob(@promiseReactionJob, state, current, argument);
-        current = current.@next;
+    var isResolved = state === @promiseStateFulfilled;
+
+    @enqueueJob(@promiseReactionJob, state, reactions.@promiseOrCapability, isResolved ? reactions.@onFulfilled : reactions.@onRejected, argument);
+
+    for (var i = 0, count = reactions.@outOfLineReactionCounts; i < count; i += 3) {
+        var promise = reactions[i];
+        var handler = isResolved ? reactions[i + 1] : reactions[i + 2];
+        @enqueueJob(@promiseReactionJob, state, promise, handler, argument);
     }
+    @assert(i === count);
 }
 
-@globalPrivate
+@linkTimeConstant
 function resolvePromise(promise, resolution)
 {
     "use strict";
@@ -168,13 +195,14 @@ function resolvePromise(promise, resolution)
         return;
     }
 
-    if (typeof then !== 'function')
+    if (!@isCallable(then))
         return @fulfillPromise(promise, resolution);
 
     @enqueueJob(@promiseResolveThenableJob, resolution, then, @createResolvingFunctions(promise));
 }
 
-@globalPrivate
+// Keep in sync with JSPromise::rejectedPromise.
+@linkTimeConstant
 function rejectPromise(promise, reason)
 {
     "use strict";
@@ -187,15 +215,13 @@ function rejectPromise(promise, reason)
     @putPromiseInternalField(promise, @promiseFieldReactionsOrResult, reason);
     @putPromiseInternalField(promise, @promiseFieldFlags, flags | @promiseStateRejected);
 
-    @InspectorInstrumentation.promiseRejected(promise, reason, reactions);
-
     if (!(flags & @promiseFlagsIsHandled))
         @hostPromiseRejectionTracker(promise, @promiseRejectionReject);
 
     @triggerPromiseReactions(@promiseStateRejected, reactions, reason);
 }
 
-@globalPrivate
+@linkTimeConstant
 function fulfillPromise(promise, value)
 {
     "use strict";
@@ -208,14 +234,14 @@ function fulfillPromise(promise, value)
     @putPromiseInternalField(promise, @promiseFieldReactionsOrResult, value);
     @putPromiseInternalField(promise, @promiseFieldFlags, flags | @promiseStateFulfilled);
 
-    @InspectorInstrumentation.promiseFulfilled(promise, value, reactions);
-
     @triggerPromiseReactions(@promiseStateFulfilled, reactions, value);
 }
 
-@globalPrivate
+@linkTimeConstant
 function resolvePromiseWithFirstResolvingFunctionCallCheck(promise, value)
 {
+    "use strict";
+
     @assert(@isPromise(promise));
     var flags = @getPromiseInternalField(promise, @promiseFieldFlags);
     if (flags & @promiseFlagsIsFirstResolvingFunctionCalled)
@@ -224,9 +250,24 @@ function resolvePromiseWithFirstResolvingFunctionCallCheck(promise, value)
     return @resolvePromise(promise, value);
 }
 
-@globalPrivate
+@linkTimeConstant
+function fulfillPromiseWithFirstResolvingFunctionCallCheck(promise, value)
+{
+    "use strict";
+
+    @assert(@isPromise(promise));
+    var flags = @getPromiseInternalField(promise, @promiseFieldFlags);
+    if (flags & @promiseFlagsIsFirstResolvingFunctionCalled)
+        return;
+    @putPromiseInternalField(promise, @promiseFieldFlags, flags | @promiseFlagsIsFirstResolvingFunctionCalled);
+    return @fulfillPromise(promise, value);
+}
+
+@linkTimeConstant
 function rejectPromiseWithFirstResolvingFunctionCallCheck(promise, reason)
 {
+    "use strict";
+
     @assert(@isPromise(promise));
     var flags = @getPromiseInternalField(promise, @promiseFieldFlags);
     if (flags & @promiseFlagsIsFirstResolvingFunctionCalled)
@@ -235,35 +276,34 @@ function rejectPromiseWithFirstResolvingFunctionCallCheck(promise, reason)
     return @rejectPromise(promise, reason);
 }
 
-@globalPrivate
+@linkTimeConstant
 function createResolvingFunctions(promise)
 {
     "use strict";
-
     @assert(@isPromise(promise));
 
     var alreadyResolved = false;
 
-    function @resolve(resolution) {
+    var resolve = (0, /* prevent function name inference */ (resolution) => {
         if (alreadyResolved)
             return @undefined;
         alreadyResolved = true;
 
         return @resolvePromise(promise, resolution);
-    }
+    });
 
-    function @reject(reason) {
+    var reject = (0, /* prevent function name inference */ (reason) => {
         if (alreadyResolved)
             return @undefined;
         alreadyResolved = true;
 
         return @rejectPromise(promise, reason);
-    }
+    });
 
-    return { @resolve, @reject };
+    return { @resolve: resolve, @reject: reject };
 }
 
-@globalPrivate
+@linkTimeConstant
 function promiseReactionJobWithoutPromise(handler, argument)
 {
     "use strict";
@@ -276,7 +316,7 @@ function promiseReactionJobWithoutPromise(handler, argument)
 }
 
 // This function has strong guarantee that each handler function (onFulfilled and onRejected) will be called at most once.
-@globalPrivate
+@linkTimeConstant
 function resolveWithoutPromise(resolution, onFulfilled, onRejected)
 {
     "use strict";
@@ -299,7 +339,7 @@ function resolveWithoutPromise(resolution, onFulfilled, onRejected)
         return;
     }
 
-    if (typeof then !== 'function') {
+    if (!@isCallable(then)) {
         @fulfillWithoutPromise(resolution, onFulfilled, onRejected);
         return;
     }
@@ -309,7 +349,7 @@ function resolveWithoutPromise(resolution, onFulfilled, onRejected)
 }
 
 // This function has strong guarantee that each handler function (onFulfilled and onRejected) will be called at most once.
-@globalPrivate
+@linkTimeConstant
 function rejectWithoutPromise(reason, onFulfilled, onRejected)
 {
     "use strict";
@@ -318,7 +358,7 @@ function rejectWithoutPromise(reason, onFulfilled, onRejected)
 }
 
 // This function has strong guarantee that each handler function (onFulfilled and onRejected) will be called at most once.
-@globalPrivate
+@linkTimeConstant
 function fulfillWithoutPromise(value, onFulfilled, onRejected)
 {
     "use strict";
@@ -326,34 +366,51 @@ function fulfillWithoutPromise(value, onFulfilled, onRejected)
     @enqueueJob(@promiseReactionJobWithoutPromise, onFulfilled, value);
 }
 
-@globalPrivate
+// This function has strong guarantee that each handler function (onFulfilled and onRejected) will be called at most once.
+// This is special version of resolveWithoutPromise which skips resolution's then handling.
+// https://github.com/tc39/ecma262/pull/1250
+@linkTimeConstant
+function resolveWithoutPromiseForAsyncAwait(resolution, onFulfilled, onRejected)
+{
+    "use strict";
+
+    if (@isPromise(resolution)) {
+        var constructor = resolution.constructor;
+        if (constructor === @Promise || constructor === @InternalPromise)
+            return @performPromiseThen(resolution, onFulfilled, onRejected);
+    }
+
+    return @resolveWithoutPromise(resolution, onFulfilled, onRejected);
+}
+
+@linkTimeConstant
 function createResolvingFunctionsWithoutPromise(onFulfilled, onRejected)
 {
     "use strict";
 
     var alreadyResolved = false;
 
-    function @resolve(resolution) {
+    var resolve = (0, /* prevent function name inference */ (resolution) => {
         if (alreadyResolved)
             return @undefined;
         alreadyResolved = true;
 
         @resolveWithoutPromise(resolution, onFulfilled, onRejected);
-    }
+    });
 
-    function @reject(reason) {
+    var reject = (0, /* prevent function name inference */ (reason) => {
         if (alreadyResolved)
             return @undefined;
         alreadyResolved = true;
 
         @rejectWithoutPromise(reason, onFulfilled, onRejected);
-    }
+    });
 
-    return { @resolve, @reject };
+    return { @resolve: resolve, @reject: reject };
 }
 
-@globalPrivate
-function promiseReactionJob(state, reaction, argument)
+@linkTimeConstant
+function promiseReactionJob(state, promiseOrCapability, handler, argument)
 {
     // Promise Reaction has four types.
     // 1. @promiseOrCapability is PromiseCapability, and having handlers.
@@ -366,11 +423,8 @@ function promiseReactionJob(state, reaction, argument)
     //     It does not have promise capability. Just handlers are passed.
     "use strict";
 
-    var promiseOrCapability = reaction.@promiseOrCapability;
-
     // Case (3).
-    if (!reaction.@onRejected) {
-        @assert(!reaction.@onFulfilled);
+    if (@isUndefinedOrNull(handler)) {
         try {
             @assert(@isPromise(promiseOrCapability));
             if (state === @promiseStateFulfilled)
@@ -383,8 +437,6 @@ function promiseReactionJob(state, reaction, argument)
         return;
     }
 
-    var handler = (state === @promiseStateFulfilled) ? reaction.@onFulfilled: reaction.@onRejected;
-
     // Case (4).
     if (!promiseOrCapability) {
         @promiseReactionJobWithoutPromise(handler, argument);
@@ -411,7 +463,7 @@ function promiseReactionJob(state, reaction, argument)
     promiseOrCapability.@resolve.@call(@undefined, result);
 }
 
-@globalPrivate
+@linkTimeConstant
 function promiseResolveThenableJobFast(thenable, promiseToResolve)
 {
     "use strict";
@@ -419,46 +471,59 @@ function promiseResolveThenableJobFast(thenable, promiseToResolve)
     @assert(@isPromise(thenable));
     @assert(@isPromise(promiseToResolve));
 
+    // Even if we are using @defaultPromiseThen, still thenable.constructor access is observable, and if it is not returning @Promise,
+    // we need to call this constructor.
+    var constructor = @speciesConstructor(thenable, @Promise);
+    if (constructor !== @Promise && constructor !== @InternalPromise) {
+        @promiseResolveThenableJobWithDerivedPromise(thenable, constructor, @createResolvingFunctions(promiseToResolve));
+        return;
+    }
+
     var flags = @getPromiseInternalField(thenable, @promiseFieldFlags);
     var state = flags & @promiseStateMask;
-    var reaction = @newPromiseReaction(promiseToResolve, @undefined, @undefined);
-    if (state === @promiseStatePending) {
-        reaction.@next = @getPromiseInternalField(thenable, @promiseFieldReactionsOrResult);
-        @putPromiseInternalField(thenable, @promiseFieldReactionsOrResult, reaction);
-    } else {
+    var reactionsOrResult = @getPromiseInternalField(thenable, @promiseFieldReactionsOrResult);
+    if (state === @promiseStatePending)
+        @pushNewPromiseReaction(thenable, reactionsOrResult, promiseToResolve, @undefined, @undefined);
+    else {
         if (state === @promiseStateRejected && !(flags & @promiseFlagsIsHandled))
             @hostPromiseRejectionTracker(thenable, @promiseRejectionHandle);
-        @enqueueJob(@promiseReactionJob, state, reaction, @getPromiseInternalField(thenable, @promiseFieldReactionsOrResult));
+        @enqueueJob(@promiseReactionJob, state, promiseToResolve, @undefined, reactionsOrResult);
     }
     @putPromiseInternalField(thenable, @promiseFieldFlags, @getPromiseInternalField(thenable, @promiseFieldFlags) | @promiseFlagsIsHandled);
 }
 
-@globalPrivate
+@linkTimeConstant
 function promiseResolveThenableJobWithoutPromiseFast(thenable, onFulfilled, onRejected)
 {
     "use strict";
 
     @assert(@isPromise(thenable));
 
+    // Even if we are using @defaultPromiseThen, still thenable.constructor access is observable, and if it is not returning @Promise,
+    // we need to call this constructor.
+    var constructor = @speciesConstructor(thenable, @Promise);
+    if (constructor !== @Promise && constructor !== @InternalPromise) {
+        @promiseResolveThenableJobWithDerivedPromise(thenable, constructor, @createResolvingFunctionsWithoutPromise(onFulfilled, onRejected));
+        return;
+    }
+
     var flags = @getPromiseInternalField(thenable, @promiseFieldFlags);
     var state = flags & @promiseStateMask;
-    if (state === @promiseStatePending) {
-        var reaction = @newPromiseReaction(@undefined, onFulfilled, onRejected);
-        reaction.@next = @getPromiseInternalField(thenable, @promiseFieldReactionsOrResult);
-        @putPromiseInternalField(thenable, @promiseFieldReactionsOrResult, reaction);
-    } else {
-        var result = @getPromiseInternalField(thenable, @promiseFieldReactionsOrResult);
+    var reactionsOrResult = @getPromiseInternalField(thenable, @promiseFieldReactionsOrResult);
+    if (state === @promiseStatePending)
+        @pushNewPromiseReaction(thenable, reactionsOrResult, @undefined, onFulfilled, onRejected);
+    else {
         if (state === @promiseStateRejected) {
             if (!(flags & @promiseFlagsIsHandled))
                 @hostPromiseRejectionTracker(thenable, @promiseRejectionHandle);
-            @rejectWithoutPromise(result, onFulfilled, onRejected);
+            @rejectWithoutPromise(reactionsOrResult, onFulfilled, onRejected);
         } else
-            @fulfillWithoutPromise(result, onFulfilled, onRejected);
+            @fulfillWithoutPromise(reactionsOrResult, onFulfilled, onRejected);
     }
     @putPromiseInternalField(thenable, @promiseFieldFlags, @getPromiseInternalField(thenable, @promiseFieldFlags) | @promiseFlagsIsHandled);
 }
 
-@globalPrivate
+@linkTimeConstant
 function promiseResolveThenableJob(thenable, then, resolvingFunctions)
 {
     "use strict";
@@ -469,3 +534,63 @@ function promiseResolveThenableJob(thenable, then, resolvingFunctions)
         return resolvingFunctions.@reject.@call(@undefined, error);
     }
 }
+
+@linkTimeConstant
+function promiseResolveThenableJobWithDerivedPromise(thenable, constructor, resolvingFunctions)
+{
+    "use strict";
+
+    try {
+        var promiseOrCapability = @newPromiseCapabilitySlow(constructor);
+        @performPromiseThen(thenable, resolvingFunctions.@resolve, resolvingFunctions.@reject, promiseOrCapability);
+        return promiseOrCapability.@promise;
+    } catch (error) {
+        return resolvingFunctions.@reject.@call(@undefined, error);
+    }
+}
+
+@linkTimeConstant
+function promiseEmptyOnFulfilled(argument)
+{
+    "use strict";
+
+    return argument;
+}
+
+@linkTimeConstant
+function promiseEmptyOnRejected(argument)
+{
+    "use strict";
+
+    throw argument;
+}
+
+@linkTimeConstant
+function performPromiseThen(promise, onFulfilled, onRejected, promiseOrCapability)
+{
+    "use strict";
+
+    if (!@isCallable(onFulfilled))
+        onFulfilled = @promiseEmptyOnFulfilled;
+
+    if (!@isCallable(onRejected))
+        onRejected = @promiseEmptyOnRejected;
+
+    var reactionsOrResult = @getPromiseInternalField(promise, @promiseFieldReactionsOrResult);
+    var flags = @getPromiseInternalField(promise, @promiseFieldFlags);
+    var state = flags & @promiseStateMask;
+    if (state === @promiseStatePending)
+        @pushNewPromiseReaction(promise, reactionsOrResult, promiseOrCapability, onFulfilled, onRejected);
+    else {
+        var handler;
+
+        if (state === @promiseStateRejected) {
+            handler = onRejected;
+            if (!(flags & @promiseFlagsIsHandled))
+                @hostPromiseRejectionTracker(promise, @promiseRejectionHandle);
+        } else
+            handler = onFulfilled;
+        @enqueueJob(@promiseReactionJob, state, promiseOrCapability, handler, reactionsOrResult);
+    }
+    @putPromiseInternalField(promise, @promiseFieldFlags, @getPromiseInternalField(promise, @promiseFieldFlags) | @promiseFlagsIsHandled);
+}
