@@ -23,7 +23,7 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  */
 
-@globalPrivate
+@linkTimeConstant
 @constructor
 function RegExpStringIterator(regExp, string, global, fullUnicode)
 {
@@ -36,7 +36,7 @@ function RegExpStringIterator(regExp, string, global, fullUnicode)
     @putByIdDirectPrivate(this, "regExpStringIteratorDone", false);
 }
 
-@globalPrivate
+@linkTimeConstant
 function advanceStringIndex(string, index, unicode)
 {
     // This function implements AdvanceStringIndex described in ES6 21.2.5.2.3.
@@ -59,14 +59,14 @@ function advanceStringIndex(string, index, unicode)
     return index + 2;
 }
 
-@globalPrivate
+@linkTimeConstant
 function regExpExec(regexp, str)
 {
     "use strict";
 
     var exec = regexp.exec;
     var builtinExec = @regExpBuiltinExec;
-    if (exec !== builtinExec && typeof exec === "function") {
+    if (exec !== builtinExec && @isCallable(exec)) {
         var result = exec.@call(regexp, str);
         if (result !== null && !@isObject(result))
             @throwTypeError("The result of a RegExp exec must be null or an object");
@@ -75,7 +75,7 @@ function regExpExec(regexp, str)
     return builtinExec.@call(regexp, str);
 }
 
-@globalPrivate
+@linkTimeConstant
 function hasObservableSideEffectsForRegExpMatch(regexp)
 {
     "use strict";
@@ -98,7 +98,7 @@ function hasObservableSideEffectsForRegExpMatch(regexp)
     return typeof regexp.lastIndex !== "number";
 }
 
-@globalPrivate
+@linkTimeConstant
 function matchSlow(regexp, str)
 {
     "use strict";
@@ -132,7 +132,7 @@ function matchSlow(regexp, str)
         if (!resultString.length)
             regexp.lastIndex = @advanceStringIndex(str, regexp.lastIndex, unicode);
 
-        resultList.@push(resultString);
+        @arrayPush(resultList, resultString);
     }
 }
 
@@ -174,110 +174,111 @@ function matchAll(strArg)
     return new @RegExpStringIterator(matcher, string, global, fullUnicode);
 }
 
+@linkTimeConstant
+function getSubstitution(matched, str, position, captures, namedCaptures, replacement)
+{
+    "use strict";
+
+    var matchLength = matched.length;
+    var stringLength = str.length;
+    var tailPos = position + matchLength;
+    var m = captures.length;
+    var replacementLength = replacement.length;
+    var result = "";
+    var lastStart = 0;
+
+    for (var start = 0; start = @stringIndexOfInternal.@call(replacement, "$", lastStart), start !== -1; lastStart = start) {
+        if (start - lastStart > 0)
+            result = result + @stringSubstringInternal.@call(replacement, lastStart, start);
+        start++;
+        if (start >= replacementLength)
+            result = result + "$";
+        else {
+            var ch = replacement[start];
+            switch (ch)
+            {
+            case "$":
+                result = result + "$";
+                start++;
+                break;
+            case "&":
+                result = result + matched;
+                start++;
+                break;
+            case "`":
+                if (position > 0)
+                    result = result + @stringSubstringInternal.@call(str, 0, position);
+                start++;
+                break;
+            case "'":
+                if (tailPos < stringLength)
+                    result = result + @stringSubstringInternal.@call(str, tailPos);
+                start++;
+                break;
+            case "<":
+                if (namedCaptures !== @undefined) {
+                    var groupNameStartIndex = start + 1;
+                    var groupNameEndIndex = @stringIndexOfInternal.@call(replacement, ">", groupNameStartIndex);
+                    if (groupNameEndIndex !== -1) {
+                        var groupName = @stringSubstringInternal.@call(replacement, groupNameStartIndex, groupNameEndIndex);
+                        var capture = namedCaptures[groupName];
+                        if (capture !== @undefined)
+                            result = result + @toString(capture);
+
+                        start = groupNameEndIndex + 1;
+                        break;
+                    }
+                }
+
+                result = result + "$<";
+                start++;
+                break;
+            default:
+                var chCode = ch.@charCodeAt(0);
+                if (chCode >= 0x30 && chCode <= 0x39) {
+                    var originalStart = start - 1;
+                    start++;
+
+                    var n = chCode - 0x30;
+                    if (n > m) {
+                        result = result + @stringSubstringInternal.@call(replacement, originalStart, start);
+                        break;
+                    }
+
+                    if (start < replacementLength) {
+                        var nextChCode = replacement.@charCodeAt(start);
+                        if (nextChCode >= 0x30 && nextChCode <= 0x39) {
+                            var nn = 10 * n + nextChCode - 0x30;
+                            if (nn <= m) {
+                                n = nn;
+                                start++;
+                            }
+                        }
+                    }
+
+                    if (n == 0) {
+                        result = result + @stringSubstringInternal.@call(replacement, originalStart, start);
+                        break;
+                    }
+
+                    var capture = captures[n - 1];
+                    if (capture !== @undefined)
+                        result = result + capture;
+                } else
+                    result = result + "$";
+                break;
+            }
+        }
+    }
+
+    return result + @stringSubstringInternal.@call(replacement, lastStart);
+}
+
 @overriddenName="[Symbol.replace]"
 function replace(strArg, replace)
 {
     "use strict";
 
-    function getSubstitution(matched, str, position, captures, namedCaptures, replacement)
-    {
-        "use strict";
-
-        var matchLength = matched.length;
-        var stringLength = str.length;
-        var tailPos = position + matchLength;
-        var m = captures.length;
-        var replacementLength = replacement.length;
-        var result = "";
-        var lastStart = 0;
-
-        for (var start = 0; start = replacement.indexOf("$", lastStart), start !== -1; lastStart = start) {
-            if (start - lastStart > 0)
-                result = result + replacement.substring(lastStart, start);
-            start++;
-            var ch = replacement.charAt(start);
-            if (ch === "")
-                result = result + "$";
-            else {
-                switch (ch)
-                {
-                case "$":
-                    result = result + "$";
-                    start++;
-                    break;
-                case "&":
-                    result = result + matched;
-                    start++;
-                    break;
-                case "`":
-                    if (position > 0)
-                        result = result + str.substring(0, position);
-                    start++;
-                    break;
-                case "'":
-                    if (tailPos < stringLength)
-                        result = result + str.substring(tailPos);
-                    start++;
-                    break;
-                case "<":
-                    if (namedCaptures !== @undefined) {
-                        var groupNameStartIndex = start + 1;
-                        var groupNameEndIndex = replacement.indexOf(">", groupNameStartIndex);
-                        if (groupNameEndIndex !== -1) {
-                            var groupName = replacement.substring(groupNameStartIndex, groupNameEndIndex);
-                            var capture = namedCaptures[groupName];
-                            if (capture !== @undefined)
-                                result = result + @toString(capture);
-
-                            start = groupNameEndIndex + 1;
-                            break;
-                        }
-                    }
-
-                    result = result + "$<";
-                    start++;
-                    break;
-                default:
-                    var chCode = ch.charCodeAt(0);
-                    if (chCode >= 0x30 && chCode <= 0x39) {
-                        var originalStart = start - 1;
-                        start++;
-
-                        var n = chCode - 0x30;
-                        if (n > m) {
-                            result = result + replacement.substring(originalStart, start);
-                            break;
-                        }
-
-                        if (start < replacementLength) {
-                            var nextChCode = replacement.charCodeAt(start);
-                            if (nextChCode >= 0x30 && nextChCode <= 0x39) {
-                                var nn = 10 * n + nextChCode - 0x30;
-                                if (nn <= m) {
-                                    n = nn;
-                                    start++;
-                                }
-                            }
-                        }
-
-                        if (n == 0) {
-                            result = result + replacement.substring(originalStart, start);
-                            break;
-                        }
-
-                        var capture = captures[n - 1];
-                        if (capture !== @undefined)
-                            result = result + capture;
-                    } else
-                        result = result + "$";
-                    break;
-                }
-            }
-        }
-
-        return result + replacement.substring(lastStart);
-    }
-
     if (!@isObject(this))
         @throwTypeError("RegExp.prototype.@@replace requires that |this| be an Object");
 
@@ -285,7 +286,7 @@ function replace(strArg, replace)
 
     var str = @toString(strArg);
     var stringLength = str.length;
-    var functionalReplace = typeof replace === 'function';
+    var functionalReplace = @isCallable(replace);
 
     if (!functionalReplace)
         replace = @toString(replace);
@@ -307,21 +308,22 @@ function replace(strArg, replace)
         if (result === null)
             done = true;
         else {
-            resultList.@push(result);
+            @arrayPush(resultList, result);
             if (!global)
                 done = true;
             else {
                 var matchStr = @toString(result[0]);
 
-                if (!matchStr.length)
-                    regexp.lastIndex = @advanceStringIndex(str, regexp.lastIndex, unicode);
+                if (!matchStr.length) {
+                    var thisIndex = @toLength(regexp.lastIndex);
+                    regexp.lastIndex = @advanceStringIndex(str, thisIndex, unicode);
+                }
             }
         }
     }
 
     var accumulatedResult = "";
     var nextSourcePosition = 0;
-    var lastPosition = 0;
 
     for (var i = 0, resultListLength = resultList.length; i < resultListLength; ++i) {
         var result = resultList[i];
@@ -330,7 +332,7 @@ function replace(strArg, replace)
             nCaptures = 0;
         var matched = @toString(result[0]);
         var matchLength = matched.length;
-        var position = result.index;
+        var position = @toIntegerOrInfinity(result.index);
         position = (position > stringLength) ? stringLength : position;
         position = (position < 0) ? 0 : position;
 
@@ -339,19 +341,22 @@ function replace(strArg, replace)
             var capN = result[n];
             if (capN !== @undefined)
                 capN = @toString(capN);
-            captures.@push(capN);
+            @arrayPush(captures, capN);
         }
 
         var replacement;
         var namedCaptures = result.groups;
 
         if (functionalReplace) {
-            var replacerArgs = [ matched ].concat(captures);
-            replacerArgs.@push(position);
-            replacerArgs.@push(str);
+            var replacerArgs = [ matched ];
+            for (var j = 0; j < captures.length; j++)
+                @arrayPush(replacerArgs, captures[j]);
+
+            @arrayPush(replacerArgs, position);
+            @arrayPush(replacerArgs, str);
 
             if (namedCaptures !== @undefined)
-                replacerArgs.@push(namedCaptures);
+                @arrayPush(replacerArgs, namedCaptures);
 
             var replValue = replace.@apply(@undefined, replacerArgs);
             replacement = @toString(replValue);
@@ -359,20 +364,19 @@ function replace(strArg, replace)
             if (namedCaptures !== @undefined)
                 namedCaptures = @toObject(namedCaptures, "RegExp.prototype[Symbol.replace] requires 'groups' property of a match not be null");
 
-            replacement = getSubstitution(matched, str, position, captures, namedCaptures, replace);
+            replacement = @getSubstitution(matched, str, position, captures, namedCaptures, replace);
         }
 
-        if (position >= nextSourcePosition && position >= lastPosition) {
-            accumulatedResult = accumulatedResult + str.substring(nextSourcePosition, position) + replacement;
+        if (position >= nextSourcePosition) {
+            accumulatedResult = accumulatedResult + @stringSubstringInternal.@call(str, nextSourcePosition, position) + replacement;
             nextSourcePosition = position + matchLength;
-            lastPosition = position;
         }
     }
 
     if (nextSourcePosition >= stringLength)
         return  accumulatedResult;
 
-    return accumulatedResult + str.substring(nextSourcePosition);
+    return accumulatedResult + @stringSubstringInternal.@call(str, nextSourcePosition);
 }
 
 // 21.2.5.9 RegExp.prototype[@@search] (string)
@@ -400,10 +404,9 @@ function search(strArg)
     // 4. Let previousLastIndex be ? Get(rx, "lastIndex").
     var previousLastIndex = regexp.lastIndex;
 
-    // 5.If SameValue(previousLastIndex, 0) is false, then
+    // 5. If SameValue(previousLastIndex, 0) is false, then
     // 5.a. Perform ? Set(rx, "lastIndex", 0, true).
-    // FIXME: Add SameValue support. https://bugs.webkit.org/show_bug.cgi?id=173226
-    if (previousLastIndex !== 0)
+    if (!@sameValue(previousLastIndex, 0))
         regexp.lastIndex = 0;
 
     // 6. Let result be ? RegExpExec(rx, S).
@@ -412,8 +415,7 @@ function search(strArg)
     // 7. Let currentLastIndex be ? Get(rx, "lastIndex").
     // 8. If SameValue(currentLastIndex, previousLastIndex) is false, then
     // 8.a. Perform ? Set(rx, "lastIndex", previousLastIndex, true).
-    // FIXME: Add SameValue support. https://bugs.webkit.org/show_bug.cgi?id=173226
-    if (regexp.lastIndex !== previousLastIndex)
+    if (!@sameValue(regexp.lastIndex, previousLastIndex))
         regexp.lastIndex = previousLastIndex;
 
     // 9. If result is null, return -1.
@@ -424,7 +426,7 @@ function search(strArg)
     return result.index;
 }
 
-@globalPrivate
+@linkTimeConstant
 function hasObservableSideEffectsForRegExpSplit(regexp)
 {
     "use strict";
@@ -459,10 +461,13 @@ function hasObservableSideEffectsForRegExpSplit(regexp)
     if (regexpUnicode !== @regExpProtoUnicodeGetter)
         return true;
     
-    // This is accessed by the RegExp species constructor.
+    // These are accessed by the RegExp species constructor.
     var regexpSource = @tryGetById(regexp, "source");
     if (regexpSource !== @regExpProtoSourceGetter)
         return true;
+    var regexpSymbolMatch = @tryGetByIdWithWellKnownSymbol(regexp, "match");
+    if (regexpSymbolMatch !== @regExpPrototypeSymbolMatch)
+        return true;
 
     return typeof regexp.lastIndex !== "number";
 }
@@ -525,7 +530,7 @@ function split(string, limit)
         // a. Let z be ? RegExpExec(splitter, S).
         var z = @regExpExec(splitter, str);
         // b. If z is not null, return A.
-        if (z != null)
+        if (z !== null)
             return result;
         // c. Perform ! CreateDataProperty(A, "0", S).
         @putByValDirect(result, 0, str);
@@ -559,10 +564,10 @@ function split(string, limit)
             // iv. Else e != p,
             else {
                 // 1. Let T be a String value equal to the substring of S consisting of the elements at indices p (inclusive) through q (exclusive).
-                var subStr = @stringSubstrInternal.@call(str, position, matchPosition - position);
+                var subStr = @stringSubstringInternal.@call(str, position, matchPosition);
                 // 2. Perform ! CreateDataProperty(A, ! ToString(lengthA), T).
                 // 3. Let lengthA be lengthA + 1.
-                @putByValDirect(result, result.length, subStr);
+                @arrayPush(result, subStr);
                 // 4. If lengthA = lim, return A.
                 if (result.length == limit)
                     return result;
@@ -581,7 +586,7 @@ function split(string, limit)
                     var nextCapture = matches[i];
                     // b. Perform ! CreateDataProperty(A, ! ToString(lengthA), nextCapture).
                     // d. Let lengthA be lengthA + 1.
-                    @putByValDirect(result, result.length, nextCapture);
+                    @arrayPush(result, nextCapture);
                     // e. If lengthA = lim, return A.
                     if (result.length == limit)
                         return result;
@@ -594,9 +599,9 @@ function split(string, limit)
         }
     }
     // 20. Let T be a String value equal to the substring of S consisting of the elements at indices p (inclusive) through size (exclusive).
-    var remainingStr = @stringSubstrInternal.@call(str, position, size);
+    var remainingStr = @stringSubstringInternal.@call(str, position, size);
     // 21. Perform ! CreateDataProperty(A, ! ToString(lengthA), T).
-    @putByValDirect(result, result.length, remainingStr);
+    @arrayPush(result, remainingStr);
     // 22. Return A.
     return result;
 }
