@@ -26,13 +26,17 @@ types [
 
     :BasicBlockLocation,
     :BoundLabel,
+    :BaselineCallLinkInfo,
     :DebugHookType,
-    :ErrorType,
+    :ECMAMode,
+    :ErrorTypeWithExtension,
+    :EnumeratorMetadata,
     :GetByIdMode,
     :GetByIdModeMetadata,
     :GetByValHistory,
     :GetPutInfo,
     :IndexingType,
+    :IterationModeMetadata,
     :JSCell,
     :JSGlobalLexicalEnvironment,
     :JSGlobalObject,
@@ -41,9 +45,9 @@ types [
     :JSScope,
     :JSType,
     :JSValue,
-    :LLIntCallLinkInfo,
     :ResultType,
     :OperandTypes,
+    :PrivateFieldPutKind,
     :ProfileTypeBytecodeFlag,
     :PropertyOffset,
     :PutByIdFlags,
@@ -60,8 +64,6 @@ types [
 
     :ValueProfile,
     :ValueProfileAndVirtualRegisterBuffer,
-    :UnaryArithProfile,
-    :BinaryArithProfile,
     :ArrayProfile,
     :ArrayAllocationProfile,
     :ObjectAllocationProfile,
@@ -174,6 +176,7 @@ op :argument_count,
 op :to_this,
     args: {
         srcDst: VirtualRegister,
+        ecmaMode: ECMAMode,
     },
     metadata: {
         cachedStructureID: StructureID,
@@ -284,10 +287,8 @@ op_group :ProfiledBinaryOp,
         dst: VirtualRegister,
         lhs: VirtualRegister,
         rhs: VirtualRegister,
+        profileIndex: unsigned,
         operandTypes: OperandTypes,
-    },
-    metadata: {
-        arithProfile: BinaryArithProfile
     }
 
 op_group :ValueProfiledBinaryOp,
@@ -323,13 +324,16 @@ op_group :UnaryOp,
         :to_string,
         :unsigned,
         :is_empty,
-        :is_undefined,
+        :typeof_is_undefined,
+        :typeof_is_object,
+        :typeof_is_function,
         :is_undefined_or_null,
         :is_boolean,
         :is_number,
+        :is_big_int,
         :is_object,
-        :is_object_or_null,
-        :is_function,
+        :is_callable,
+        :is_constructor,
     ],
     args: {
         dst: VirtualRegister,
@@ -343,9 +347,7 @@ op_group :UnaryInPlaceProfiledOp,
     ],
     args: {
         srcDst: VirtualRegister,
-    },
-    metadata: {
-        arithProfile: UnaryArithProfile
+        profileIndex: unsigned,
     }
 
 op :to_object,
@@ -375,10 +377,8 @@ op :negate,
     args: {
         dst: VirtualRegister,
         operand: VirtualRegister,
+        profileIndex: unsigned,
         resultType: ResultType,
-    },
-    metadata: {
-        arithProfile: UnaryArithProfile,
     }
 
 op :not,
@@ -447,6 +447,20 @@ op :in_by_id,
         property: unsigned,
     }
 
+op :has_private_name,
+    args: {
+        dst: VirtualRegister,
+        base: VirtualRegister,
+        property: VirtualRegister,
+    }
+
+op :has_private_brand,
+    args: {
+        dst: VirtualRegister,
+        base: VirtualRegister,
+        brand: VirtualRegister,
+    }
+
 op :get_by_id,
     args: {
         dst: VirtualRegister,
@@ -492,6 +506,15 @@ op :get_by_id_direct,
         offset: unsigned,
     }
 
+op :get_prototype_of,
+    args: {
+        dst: VirtualRegister,
+        value: VirtualRegister,
+    },
+    metadata: {
+        profile: ValueProfile,
+    }
+
 op :try_get_by_id,
     args: {
         dst: VirtualRegister,
@@ -500,6 +523,8 @@ op :try_get_by_id,
     },
     metadata: {
         profile: ValueProfile,
+        structureID: StructureID,
+        offset: unsigned,
     }
 
 op :put_by_id,
@@ -522,6 +547,7 @@ op :put_by_id_with_this,
         thisValue: VirtualRegister,
         property: unsigned,
         value: VirtualRegister,
+        ecmaMode: ECMAMode,
     }
 
 op :del_by_id,
@@ -529,6 +555,7 @@ op :del_by_id,
         dst: VirtualRegister,
         base: VirtualRegister,
         property: unsigned,
+        ecmaMode: ECMAMode,
     }
 
 op :get_by_val,
@@ -543,11 +570,60 @@ op :get_by_val,
         seenIdentifiers: GetByValHistory,
     }
 
+op :get_private_name,
+    args: {
+        dst: VirtualRegister,
+        base: VirtualRegister,
+        property: VirtualRegister,
+    },
+    metadata: {
+        profile: ValueProfile,
+        structureID: StructureID,
+        offset: unsigned,
+        property: WriteBarrier[JSCell],
+    }
+
+op :put_private_name,
+    args: {
+        base: VirtualRegister,
+        property: VirtualRegister,
+        value: VirtualRegister,
+        putKind: PrivateFieldPutKind,
+    },
+    metadata: {
+        oldStructureID: StructureID,
+        property: WriteBarrier[JSCell],
+        offset: unsigned,
+        newStructureID: StructureID,
+    }
+
+op :set_private_brand,
+    args: {
+        base: VirtualRegister,
+        brand: VirtualRegister,
+    },
+    metadata: {
+        oldStructureID: StructureID,
+        newStructureID: StructureID,
+        brand: WriteBarrier[JSCell],
+    }
+
+op :check_private_brand,
+    args: {
+        base: VirtualRegister,
+        brand: VirtualRegister,
+    },
+    metadata: {
+        structureID: StructureID,
+        brand: WriteBarrier[JSCell],
+    }
+
 op :put_by_val,
     args: {
         base: VirtualRegister,
         property: VirtualRegister,
         value: VirtualRegister,
+        ecmaMode: ECMAMode,
     },
     metadata: {
         arrayProfile: ArrayProfile,
@@ -559,6 +635,7 @@ op :put_by_val_with_this,
         thisValue: VirtualRegister,
         property: VirtualRegister,
         value: VirtualRegister,
+        ecmaMode: ECMAMode,
     }
 
 op :put_by_val_direct,
@@ -566,6 +643,7 @@ op :put_by_val_direct,
         base: VirtualRegister,
         property: VirtualRegister,
         value: VirtualRegister,
+        ecmaMode: ECMAMode,
     },
     metadata: {
         arrayProfile: ArrayProfile,
@@ -576,6 +654,7 @@ op :del_by_val,
         dst: VirtualRegister,
         base: VirtualRegister,
         property: VirtualRegister,
+        ecmaMode: ECMAMode,
     }
 
 op :put_getter_by_id,
@@ -677,6 +756,13 @@ op :jnundefined_or_null,
         targetLabel: BoundLabel,
     }
 
+op :jeq_ptr,
+    args: {
+        value: VirtualRegister,
+        specialPointer: VirtualRegister,
+        targetLabel: BoundLabel,
+    }
+
 op :jneq_ptr,
     args: {
         value: VirtualRegister,
@@ -756,7 +842,8 @@ op :call,
         argv: unsigned,
     },
     metadata: {
-        callLinkInfo: LLIntCallLinkInfo,
+        callLinkInfo: BaselineCallLinkInfo,
+        arrayProfile: ArrayProfile,
         profile: ValueProfile,
     }
 
@@ -768,7 +855,8 @@ op :tail_call,
         argv: unsigned,
     },
     metadata: {
-        callLinkInfo: LLIntCallLinkInfo,
+        callLinkInfo: BaselineCallLinkInfo,
+        arrayProfile: ArrayProfile,
         profile: ValueProfile,
     }
 
@@ -778,9 +866,11 @@ op :call_eval,
         callee: VirtualRegister,
         argc: unsigned,
         argv: unsigned,
+        ecmaMode: ECMAMode,
     },
     metadata: {
-        callLinkInfo: LLIntCallLinkInfo,
+        callLinkInfo: BaselineCallLinkInfo,
+        arrayProfile: ArrayProfile,
         profile: ValueProfile,
     }
 
@@ -794,6 +884,7 @@ op :call_varargs,
         firstVarArg: int,
     },
     metadata: {
+        callLinkInfo: BaselineCallLinkInfo,
         arrayProfile: ArrayProfile,
         profile: ValueProfile,
     },
@@ -815,6 +906,7 @@ op :tail_call_varargs,
         firstVarArg: int,
     },
     metadata: {
+        callLinkInfo: BaselineCallLinkInfo,
         arrayProfile: ArrayProfile,
         profile: ValueProfile,
     },
@@ -836,6 +928,7 @@ op :tail_call_forward_arguments,
         firstVarArg: int,
     },
     metadata: {
+        callLinkInfo: BaselineCallLinkInfo,
         arrayProfile: ArrayProfile,
         profile: ValueProfile,
     }
@@ -848,7 +941,8 @@ op :construct,
         argv: unsigned,
     },
     metadata: {
-        callLinkInfo: LLIntCallLinkInfo,
+        callLinkInfo: BaselineCallLinkInfo,
+        arrayProfile: ArrayProfile,
         profile: ValueProfile,
     }
 
@@ -862,6 +956,7 @@ op :construct_varargs,
         firstVarArg: int,
     },
     metadata: {
+        callLinkInfo: BaselineCallLinkInfo,
         arrayProfile: ArrayProfile,
         profile: ValueProfile,
     },
@@ -1035,7 +1130,7 @@ op :throw,
 op :throw_static_error,
     args: {
         message: VirtualRegister,
-        errorType: ErrorType,
+        errorType: ErrorTypeWithExtension,
     }
 
 op :debug,
@@ -1069,73 +1164,69 @@ op :profile_control_flow,
         basicBlockLocation: BasicBlockLocation.*,
     }
 
-op :get_enumerable_length,
-    args: {
-        dst: VirtualRegister,
-        base: VirtualRegister,
-    }
-
-op :has_indexed_property,
-    args: {
-        dst: VirtualRegister,
-        base: VirtualRegister,
-        property: VirtualRegister,
-    },
-    metadata: {
-        arrayProfile: ArrayProfile,
-    }
-
-op :has_structure_property,
-    args: {
-        dst: VirtualRegister,
-        base: VirtualRegister,
-        property: VirtualRegister,
-        enumerator: VirtualRegister,
-    }
-
-op :has_generic_property,
-    args: {
-        dst: VirtualRegister,
-        base: VirtualRegister,
-        property: VirtualRegister,
-    }
-
-op :get_direct_pname,
-    args: {
-        dst: VirtualRegister,
-        base: VirtualRegister,
-        property: VirtualRegister,
-        index: VirtualRegister,
-        enumerator: VirtualRegister,
-    },
-    metadata: {
-        profile: ValueProfile,
-    }
-
 op :get_property_enumerator,
     args: {
         dst: VirtualRegister,
         base: VirtualRegister,
     }
 
-op :enumerator_structure_pname,
+op :enumerator_next,
     args: {
-        dst: VirtualRegister,
+        # out
+        propertyName: VirtualRegister,
+        # in/out
+        mode: VirtualRegister, # Will always be a JS UInt32 representing a JSForInMode.
+        index: VirtualRegister, # Gets reset to zero every time mode changes.
+        # in
+        base: VirtualRegister,
         enumerator: VirtualRegister,
-        index: VirtualRegister,
+    },
+    metadata: {
+        arrayProfile: ArrayProfile,
+        enumeratorMetadata: EnumeratorMetadata,
     }
 
-op :enumerator_generic_pname,
+op :enumerator_get_by_val,
     args: {
         dst: VirtualRegister,
-        enumerator: VirtualRegister,
+        base: VirtualRegister,
+        mode: VirtualRegister,
+        propertyName: VirtualRegister,
         index: VirtualRegister,
+        enumerator: VirtualRegister,
+    },
+    metadata: {
+        profile: ValueProfile,
+        arrayProfile: ArrayProfile,
+        enumeratorMetadata: EnumeratorMetadata,
     }
 
-op :to_index_string,
+op :enumerator_in_by_val,
     args: {
         dst: VirtualRegister,
+        base: VirtualRegister,
+        mode: VirtualRegister,
+        propertyName: VirtualRegister,
         index: VirtualRegister,
+        enumerator: VirtualRegister,
+    },
+    metadata: {
+        arrayProfile: ArrayProfile,
+        enumeratorMetadata: EnumeratorMetadata,
+    }
+
+op :enumerator_has_own_property,
+    args: {
+        dst: VirtualRegister,
+        base: VirtualRegister,
+        mode: VirtualRegister,
+        propertyName: VirtualRegister,
+        index: VirtualRegister,
+        enumerator: VirtualRegister,
+    },
+    metadata: {
+        arrayProfile: ArrayProfile,
+        enumeratorMetadata: EnumeratorMetadata,
     }
 
 op :unreachable
@@ -1153,9 +1244,64 @@ op :get_rest_length,
         numParametersToSkip: unsigned,
     }
 
+# Semantically, this is iterator = symbolIterator.@call(iterable); next = iterator.next;
+# where symbolIterator the result of iterable[Symbol.iterator] (which is done in a different bytecode).
+# For builtin iterators, however, this has special behavior where next becomes the empty value, which
+# indicates that we are in a known iteration mode to op_iterator_next.
+op :iterator_open,
+    args: {
+        iterator: VirtualRegister,
+        next: VirtualRegister,
+        symbolIterator: VirtualRegister,
+        iterable: VirtualRegister,
+        stackOffset: unsigned,
+    },
+    metadata: {
+        iterableProfile: ValueProfile,
+        callLinkInfo: BaselineCallLinkInfo,
+        iteratorProfile: ValueProfile,
+        modeMetadata: GetByIdModeMetadata,
+        nextProfile: ValueProfile,
+        arrayProfile: ArrayProfile,
+        iterationMetadata: IterationModeMetadata,
+    },
+    checkpoints: {
+        symbolCall: nil,
+        getNext: nil,
+    }
+
+# Semantically, this is nextResult = next.@call(iterator); done = nextResult.done; value = done ? undefined : nextResult.value;
+op :iterator_next,
+    args: {
+        done: VirtualRegister,
+        value: VirtualRegister,
+        iterable: VirtualRegister,
+        next: VirtualRegister,
+        iterator: VirtualRegister,
+        stackOffset: unsigned,
+    },
+    metadata: {
+        callLinkInfo: BaselineCallLinkInfo,
+        nextResultProfile: ValueProfile,
+        doneModeMetadata: GetByIdModeMetadata,
+        doneProfile: ValueProfile,
+        valueModeMetadata: GetByIdModeMetadata,
+        valueProfile: ValueProfile,
+        arrayProfile: ArrayProfile,
+        iterableProfile: ArrayProfile,
+        iterationMetadata: IterationModeMetadata,
+    },
+    tmps: {
+        nextResult: JSValue,
+    },
+    checkpoints: {
+        computeNext: nil,
+        getDone: nil,
+        getValue: nil,
+    }
+
 op :yield,
     args: {
-        generator: VirtualRegister,
         yieldPoint: unsigned,
         argument: VirtualRegister,
     }
@@ -1210,7 +1356,6 @@ begin_section :CLoopHelpers,
     macro_name_component: :CLOOP_BYTECODE_HELPER
 
 op :llint_entry
-op :getHostCallReturnValue
 op :llint_return_to_host
 op :llint_vm_entry_to_javascript
 op :llint_vm_entry_to_native
@@ -1248,6 +1393,21 @@ op :llint_cloop_did_return_from_js_31
 op :llint_cloop_did_return_from_js_32
 op :llint_cloop_did_return_from_js_33
 op :llint_cloop_did_return_from_js_34
+op :llint_cloop_did_return_from_js_35
+op :llint_cloop_did_return_from_js_36
+op :llint_cloop_did_return_from_js_37
+op :llint_cloop_did_return_from_js_38
+op :llint_cloop_did_return_from_js_39
+op :llint_cloop_did_return_from_js_40
+op :llint_cloop_did_return_from_js_41
+op :llint_cloop_did_return_from_js_42
+op :llint_cloop_did_return_from_js_43
+op :llint_cloop_did_return_from_js_44
+op :llint_cloop_did_return_from_js_45
+op :llint_cloop_did_return_from_js_46
+op :llint_cloop_did_return_from_js_47
+op :llint_cloop_did_return_from_js_48
+op :llint_cloop_did_return_from_js_49
 
 end_section :CLoopHelpers
 
@@ -1270,20 +1430,67 @@ op :llint_native_call_trampoline
 op :llint_native_construct_trampoline
 op :llint_internal_function_call_trampoline
 op :llint_internal_function_construct_trampoline
+op :llint_link_call_trampoline
+op :llint_virtual_call_trampoline
+op :llint_virtual_construct_trampoline
+op :llint_virtual_tail_call_trampoline
 op :checkpoint_osr_exit_from_inlined_call_trampoline
 op :checkpoint_osr_exit_trampoline
-op :handleUncaughtException
+op :normal_osr_exit_trampoline
+op :fuzzer_return_early_from_loop_hint
+op :llint_get_host_call_return_value
+op :llint_handle_uncaught_exception
 op :op_call_return_location
 op :op_construct_return_location
+op :op_call_varargs_return_location
+op :op_construct_varargs_return_location
 op :op_call_varargs_slow_return_location
 op :op_construct_varargs_slow_return_location
 op :op_get_by_id_return_location
 op :op_get_by_val_return_location
 op :op_put_by_id_return_location
 op :op_put_by_val_return_location
+op :op_iterator_open_return_location
+op :op_iterator_next_return_location
 op :wasm_function_prologue
 op :wasm_function_prologue_no_tls
 
+op :op_call_slow_return_location
+op :op_construct_slow_return_location
+op :op_iterator_open_slow_return_location
+op :op_iterator_next_slow_return_location
+op :op_tail_call_slow_return_location
+op :op_tail_call_forward_arguments_slow_return_location
+op :op_tail_call_varargs_slow_return_location
+op :op_call_eval_slow_return_location
+
+op :js_trampoline_op_call
+op :js_trampoline_op_construct
+op :js_trampoline_op_call_varargs
+op :js_trampoline_op_construct_varargs
+op :js_trampoline_op_iterator_next
+op :js_trampoline_op_iterator_open
+op :js_trampoline_op_call_slow
+op :js_trampoline_op_tail_call_slow
+op :js_trampoline_op_construct_slow
+op :js_trampoline_op_call_varargs_slow
+op :js_trampoline_op_tail_call_varargs_slow
+op :js_trampoline_op_tail_call_forward_arguments_slow
+op :js_trampoline_op_construct_varargs_slow
+op :js_trampoline_op_call_eval_slow
+op :js_trampoline_op_iterator_next_slow
+op :js_trampoline_op_iterator_open_slow
+op :js_trampoline_llint_function_for_call_arity_check_untag
+op :js_trampoline_llint_function_for_call_arity_check_tag
+op :js_trampoline_llint_function_for_construct_arity_check_untag
+op :js_trampoline_llint_function_for_construct_arity_check_tag
+op :wasm_trampoline_wasm_call
+op :wasm_trampoline_wasm_call_no_tls
+op :wasm_trampoline_wasm_call_indirect
+op :wasm_trampoline_wasm_call_indirect_no_tls
+op :wasm_trampoline_wasm_call_ref
+op :wasm_trampoline_wasm_call_ref_no_tls
+
 end_section :NativeHelpers
 
 begin_section :Wasm,
@@ -1297,6 +1504,15 @@ autogenerate_wasm_opcodes
 # Helpers
 
 op :throw_from_slow_path_trampoline
+op :throw_from_fault_handler_trampoline_fastTLS
+op :throw_from_fault_handler_trampoline_reg_instance
+
+op :call_return_location
+op :call_no_tls_return_location
+op :call_indirect_return_location
+op :call_indirect_no_tls_return_location
+op :call_ref_return_location
+op :call_ref_no_tls_return_location
 
 # FIXME: Wasm and JS LLInt should share common opcodes
 # https://bugs.webkit.org/show_bug.cgi?id=203656
@@ -1411,6 +1627,20 @@ op :table_set,
         tableIndex: unsigned,
     }
 
+op :table_init,
+    args: {
+        dstOffset: VirtualRegister,
+        srcOffset: VirtualRegister,
+        length: VirtualRegister,
+        elementIndex: unsigned,
+        tableIndex: unsigned,
+    }
+
+op :elem_drop,
+    args: {
+        elementIndex: unsigned,
+    }
+
 op :table_size,
     args: {
         dst: VirtualRegister,
@@ -1433,6 +1663,15 @@ op :table_fill,
         tableIndex: unsigned,
     }
 
+op :table_copy,
+    args: {
+        dstOffset: VirtualRegister,
+        srcOffset: VirtualRegister,
+        length: VirtualRegister,
+        dstTableIndex: unsigned,
+        srcTableIndex: unsigned,
+    }
+
 op :call,
     args: {
         functionIndex: unsigned,
@@ -1450,7 +1689,7 @@ op :call_no_tls,
 op :call_indirect,
     args: {
         functionIndex: VirtualRegister,
-        signatureIndex: unsigned,
+        typeIndex: unsigned,
         stackOffset: unsigned,
         numberOfStackArgs: unsigned,
         tableIndex: unsigned,
@@ -1459,12 +1698,28 @@ op :call_indirect,
 op :call_indirect_no_tls,
     args: {
         functionIndex: VirtualRegister,
-        signatureIndex: unsigned,
+        typeIndex: unsigned,
         stackOffset: unsigned,
         numberOfStackArgs: unsigned,
         tableIndex: unsigned,
     }
 
+op :call_ref,
+    args: {
+        functionReference: VirtualRegister,
+        typeIndex: unsigned,
+        stackOffset: unsigned,
+        numberOfStackArgs: unsigned,
+    }
+
+op :call_ref_no_tls,
+    args: {
+        functionReference: VirtualRegister,
+        typeIndex: unsigned,
+        stackOffset: unsigned,
+        numberOfStackArgs: unsigned,
+    }
+
 op :current_memory,
     args: {
         dst: VirtualRegister,
@@ -1476,6 +1731,33 @@ op :grow_memory,
         delta: VirtualRegister
     }
 
+op :memory_fill,
+    args: {
+        dstAddress: VirtualRegister,
+        targetValue: VirtualRegister,
+        count: VirtualRegister,
+    }
+
+op :memory_copy,
+    args: {
+        dstAddress: VirtualRegister,
+        srcAddress: VirtualRegister,
+        count: VirtualRegister,
+    }
+
+op :memory_init,
+    args: {
+        dstAddress: VirtualRegister,
+        srcAddress: VirtualRegister,
+        length: VirtualRegister,
+        dataSegmentIndex: unsigned,
+    }
+
+op :data_drop,
+    args: {
+        dataSegmentIndex: unsigned,
+    }
+
 op :select,
     args: {
         dst: VirtualRegister,
@@ -1515,4 +1797,117 @@ op_group :Store,
         offset: unsigned,
     }
 
+op_group :AtomicBinaryRMW,
+    [
+        "add",
+        "sub",
+        "and",
+        "or",
+        "xor",
+        "xchg",
+    ].flat_map {|op|
+        [
+            "i64_atomic_rmw_#{op}",
+            "i64_atomic_rmw8_#{op}_u",
+            "i64_atomic_rmw16_#{op}_u",
+            "i64_atomic_rmw32_#{op}_u",
+        ]
+    }.map {|op| op.to_sym },
+    args: {
+        dst: VirtualRegister,
+        pointer: VirtualRegister,
+        offset: unsigned,
+        value: VirtualRegister,
+    }
+
+op_group :AtomicCompareExchange,
+    [
+        :i64_atomic_rmw_cmpxchg,
+        :i64_atomic_rmw8_cmpxchg_u,
+        :i64_atomic_rmw16_cmpxchg_u,
+        :i64_atomic_rmw32_cmpxchg_u,
+    ],
+    args: {
+        dst: VirtualRegister,
+        pointer: VirtualRegister,
+        offset: unsigned,
+        expected: VirtualRegister,
+        value: VirtualRegister,
+    }
+
+op_group :AtomicWait,
+    [
+        :memory_atomic_wait32,
+        :memory_atomic_wait64,
+    ],
+    args: {
+        dst: VirtualRegister,
+        pointer: VirtualRegister,
+        offset: unsigned,
+        value: VirtualRegister,
+        timeout: VirtualRegister,
+    }
+
+op :memory_atomic_notify,
+    args: {
+        dst: VirtualRegister,
+        pointer: VirtualRegister,
+        offset: unsigned,
+        count: VirtualRegister,
+    }
+
+op :atomic_fence,
+    args: {
+    }
+
+op :throw,
+    args: {
+        exceptionIndex: unsigned,
+        firstValue: VirtualRegister,
+    }
+
+op :rethrow,
+    args: {
+        exception: VirtualRegister,
+    }
+
+op_group :Catch,
+    [
+        :catch,
+        :catch_no_tls,
+    ],
+    args: {
+        exceptionIndex: unsigned,
+        exception: VirtualRegister,
+        argumentCount: unsigned,
+        startOffset: unsigned,
+    }
+
+op_group :CatchAll,
+    [
+        :catch_all,
+        :catch_all_no_tls,
+    ],
+    args: {
+        exception: VirtualRegister,
+    }
+
+op :i31_new,
+    args: {
+        dst: VirtualRegister,
+        value: VirtualRegister,
+    }
+
+op :i31_get_s,
+    args: {
+        dst: VirtualRegister,
+        ref: VirtualRegister,
+    }
+
+op :i31_get_u,
+    args: {
+        dst: VirtualRegister,
+        ref: VirtualRegister,
+    }
+
 end_section :Wasm
