@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2014-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -40,6 +40,7 @@
 #include "StructureStubClearingWatchpoint.h"
 #include "StructureStubInfo.h"
 #include "SuperSampler.h"
+#include "ThunkGenerators.h"
 #include <wtf/CommaPrinter.h>
 #include <wtf/ListDump.h>
 
@@ -58,10 +59,9 @@ void AccessGenerationResult::dump(PrintStream& out) const
         out.print(":", m_code);
 }
 
-void AccessGenerationState::installWatchpoint(const ObjectPropertyCondition& condition)
+void AccessGenerationState::installWatchpoint(CodeBlock* codeBlock, const ObjectPropertyCondition& condition)
 {
-    WatchpointsOnStructureStubInfo::ensureReferenceAndInstallWatchpoint(
-        watchpoints, jit->codeBlock(), stubInfo, condition);
+    WatchpointsOnStructureStubInfo::ensureReferenceAndInstallWatchpoint(watchpoints, codeBlock, stubInfo, condition);
 }
 
 void AccessGenerationState::restoreScratch()
@@ -72,7 +72,10 @@ void AccessGenerationState::restoreScratch()
 void AccessGenerationState::succeed()
 {
     restoreScratch();
-    success.append(jit->jump());
+    if (jit->codeBlock()->useDataIC())
+        jit->farJump(CCallHelpers::Address(stubInfo->m_stubInfoGPR, StructureStubInfo::offsetOfDoneLocation()), JSInternalPtrTag);
+    else
+        success.append(jit->jump());
 }
 
 const RegisterSet& AccessGenerationState::liveRegistersForCall()
@@ -107,6 +110,8 @@ const RegisterSet& AccessGenerationState::calculateLiveRegistersForCallAndExcept
             RELEASE_ASSERT(JITCode::isOptimizingJIT(jit->codeBlock()->jitType()));
 
         m_liveRegistersForCall = RegisterSet(m_liveRegistersToPreserveAtExceptionHandlingCallSite, allocator->usedRegisters());
+        if (jit->codeBlock()->useDataIC())
+            m_liveRegistersForCall.add(stubInfo->m_stubInfoGPR);
         m_liveRegistersForCall.exclude(calleeSaveRegisters());
     }
     return m_liveRegistersForCall;
@@ -125,6 +130,21 @@ auto AccessGenerationState::preserveLiveRegistersToStackForCall(const RegisterSe
     };
 }
 
+auto AccessGenerationState::preserveLiveRegistersToStackForCallWithoutExceptions() -> SpillState
+{
+    RegisterSet liveRegisters = allocator->usedRegisters();
+    if (jit->codeBlock()->useDataIC())
+        liveRegisters.add(stubInfo->m_stubInfoGPR);
+    liveRegisters.exclude(calleeSaveRegisters());
+
+    constexpr unsigned extraStackPadding = 0;
+    unsigned numberOfStackBytesUsedForRegisterPreservation = ScratchRegisterAllocator::preserveRegistersToStackForCall(*jit, liveRegisters, extraStackPadding);
+    return SpillState {
+        WTFMove(liveRegisters),
+        numberOfStackBytesUsedForRegisterPreservation
+    };
+}
+
 void AccessGenerationState::restoreLiveRegistersFromStackForCallWithThrownException(const SpillState& spillState)
 {
     // Even if we're a getter, we don't want to ignore the result value like we normally do
@@ -213,23 +233,37 @@ void AccessGenerationState::emitExplicitExceptionHandler()
                 linkBuffer.link(jumpToOSRExitExceptionHandler, originalHandler.nativeCode);
             });
     } else {
-        jit->setupArguments<decltype(operationLookupExceptionHandler)>(CCallHelpers::TrustedImmPtr(&m_vm));
-        jit->prepareCallOperation(m_vm);
-        CCallHelpers::Call lookupExceptionHandlerCall = jit->call(OperationPtrTag);
+        CCallHelpers::Jump jumpToExceptionHandler = jit->jump();
+        VM* vm = &m_vm;
         jit->addLinkTask(
             [=] (LinkBuffer& linkBuffer) {
-                linkBuffer.link(lookupExceptionHandlerCall, FunctionPtr<OperationPtrTag>(operationLookupExceptionHandler));
+                linkBuffer.link(jumpToExceptionHandler, CodeLocationLabel(vm->getCTIStub(handleExceptionGenerator).retaggedCode<NoPtrTag>()));
             });
-        jit->jumpToExceptionHandler(m_vm);
     }
 }
 
+ScratchRegisterAllocator AccessGenerationState::makeDefaultScratchAllocator(GPRReg extraToLock)
+{
+    ScratchRegisterAllocator allocator(stubInfo->usedRegisters);
+    allocator.lock(stubInfo->baseRegs());
+    allocator.lock(valueRegs);
+    allocator.lock(extraGPR);
+#if USE(JSVALUE32_64)
+    allocator.lock(stubInfo->m_extraTagGPR);
+#endif
+    allocator.lock(stubInfo->m_stubInfoGPR);
+    allocator.lock(stubInfo->m_arrayProfileGPR);
+    allocator.lock(extraToLock);
+
+    return allocator;
+}
+
 PolymorphicAccess::PolymorphicAccess() { }
 PolymorphicAccess::~PolymorphicAccess() { }
 
 AccessGenerationResult PolymorphicAccess::addCases(
     const GCSafeConcurrentJSLocker& locker, VM& vm, CodeBlock* codeBlock, StructureStubInfo& stubInfo,
-    Vector<std::unique_ptr<AccessCase>, 2> originalCasesToAdd)
+    Vector<RefPtr<AccessCase>, 2> originalCasesToAdd)
 {
     SuperSamplerScope superSamplerScope(false);
     
@@ -245,9 +279,9 @@ AccessGenerationResult PolymorphicAccess::addCases(
     //   add more things after failure.
     
     // First ensure that the originalCasesToAdd doesn't contain duplicates.
-    Vector<std::unique_ptr<AccessCase>> casesToAdd;
+    Vector<RefPtr<AccessCase>> casesToAdd;
     for (unsigned i = 0; i < originalCasesToAdd.size(); ++i) {
-        std::unique_ptr<AccessCase> myCase = WTFMove(originalCasesToAdd[i]);
+        RefPtr<AccessCase> myCase = WTFMove(originalCasesToAdd[i]);
 
         // Add it only if it is not replaced by the subsequent cases in the list.
         bool found = false;
@@ -264,8 +298,7 @@ AccessGenerationResult PolymorphicAccess::addCases(
         casesToAdd.append(WTFMove(myCase));
     }
 
-    if (PolymorphicAccessInternal::verbose)
-        dataLog("casesToAdd: ", listDump(casesToAdd), "\n");
+    dataLogLnIf(PolymorphicAccessInternal::verbose, "casesToAdd: ", listDump(casesToAdd));
 
     // If there aren't any cases to add, then fail on the grounds that there's no point to generating a
     // new stub that will be identical to the old one. Returning null should tell the caller to just
@@ -316,16 +349,15 @@ AccessGenerationResult PolymorphicAccess::addCases(
         m_list.append(WTFMove(caseToAdd));
     }
     
-    if (PolymorphicAccessInternal::verbose)
-        dataLog("After addCases: m_list: ", listDump(m_list), "\n");
+    dataLogLnIf(PolymorphicAccessInternal::verbose, "After addCases: m_list: ", listDump(m_list));
 
     return AccessGenerationResult::Buffered;
 }
 
 AccessGenerationResult PolymorphicAccess::addCase(
-    const GCSafeConcurrentJSLocker& locker, VM& vm, CodeBlock* codeBlock, StructureStubInfo& stubInfo, std::unique_ptr<AccessCase> newAccess)
+    const GCSafeConcurrentJSLocker& locker, VM& vm, CodeBlock* codeBlock, StructureStubInfo& stubInfo, Ref<AccessCase> newAccess)
 {
-    Vector<std::unique_ptr<AccessCase>, 2> newAccesses;
+    Vector<RefPtr<AccessCase>, 2> newAccesses;
     newAccesses.append(WTFMove(newAccess));
     return addCases(locker, vm, codeBlock, stubInfo, WTFMove(newAccesses));
 }
@@ -336,29 +368,35 @@ bool PolymorphicAccess::visitWeak(VM& vm) const
         if (!at(i).visitWeak(vm))
             return false;
     }
-    if (Vector<WriteBarrier<JSCell>>* weakReferences = m_weakReferences.get()) {
-        for (WriteBarrier<JSCell>& weakReference : *weakReferences) {
-            if (!vm.heap.isMarked(weakReference.get()))
+    if (m_stubRoutine) {
+        for (StructureID weakReference : m_stubRoutine->weakStructures()) {
+            Structure* structure = weakReference.decode();
+            if (!vm.heap.isMarked(structure))
                 return false;
         }
     }
     return true;
 }
 
-bool PolymorphicAccess::propagateTransitions(SlotVisitor& visitor) const
+template<typename Visitor>
+void PolymorphicAccess::propagateTransitions(Visitor& visitor) const
 {
-    bool result = true;
     for (unsigned i = 0; i < size(); ++i)
-        result &= at(i).propagateTransitions(visitor);
-    return result;
+        at(i).propagateTransitions(visitor);
 }
 
-void PolymorphicAccess::visitAggregate(SlotVisitor& visitor)
+template void PolymorphicAccess::propagateTransitions(AbstractSlotVisitor&) const;
+template void PolymorphicAccess::propagateTransitions(SlotVisitor&) const;
+
+template<typename Visitor>
+void PolymorphicAccess::visitAggregateImpl(Visitor& visitor)
 {
     for (unsigned i = 0; i < size(); ++i)
         at(i).visitAggregate(visitor);
 }
 
+DEFINE_VISIT_AGGREGATE(PolymorphicAccess);
+
 void PolymorphicAccess::dump(PrintStream& out) const
 {
     out.print(RawPointer(this), ":[");
@@ -391,16 +429,15 @@ AccessGenerationResult PolymorphicAccess::regenerate(const GCSafeConcurrentJSLoc
 {
     SuperSamplerScope superSamplerScope(false);
     
-    if (PolymorphicAccessInternal::verbose)
-        dataLog("Regenerate with m_list: ", listDump(m_list), "\n");
+    dataLogLnIf(PolymorphicAccessInternal::verbose, "Regenerate with m_list: ", listDump(m_list));
 
     AccessGenerationState state(vm, globalObject, ecmaMode);
 
     state.access = this;
     state.stubInfo = &stubInfo;
     
-    state.baseGPR = stubInfo.baseGPR;
-    state.u.thisGPR = stubInfo.regs.thisGPR;
+    state.baseGPR = stubInfo.m_baseGPR;
+    state.extraGPR = stubInfo.m_extraGPR;
     state.valueRegs = stubInfo.valueRegs();
 
     // Regenerating is our opportunity to figure out what our list of cases should look like. We
@@ -412,7 +449,7 @@ AccessGenerationResult PolymorphicAccess::regenerate(const GCSafeConcurrentJSLoc
     unsigned srcIndex = 0;
     unsigned dstIndex = 0;
     while (srcIndex < m_list.size()) {
-        std::unique_ptr<AccessCase> someCase = WTFMove(m_list[srcIndex++]);
+        RefPtr<AccessCase> someCase = WTFMove(m_list[srcIndex++]);
         
         // If the case had been generated, then we have to keep the original in m_list in case we
         // fail to regenerate. That case may have data structures that are used by the code that it
@@ -451,18 +488,8 @@ AccessGenerationResult PolymorphicAccess::regenerate(const GCSafeConcurrentJSLoc
     }
     m_list.resize(dstIndex);
 
-    ScratchRegisterAllocator allocator(stubInfo.usedRegisters);
+    auto allocator = state.makeDefaultScratchAllocator();
     state.allocator = &allocator;
-    allocator.lock(state.baseGPR);
-    if (state.u.thisGPR != InvalidGPRReg)
-        allocator.lock(state.u.thisGPR);
-    allocator.lock(state.valueRegs);
-#if USE(JSVALUE32_64)
-    allocator.lock(stubInfo.baseTagGPR);
-    if (stubInfo.v.thisTagGPR != InvalidGPRReg)
-        allocator.lock(stubInfo.v.thisTagGPR);
-#endif
-
     state.scratchGPR = allocator.allocateScratchGPR();
 
     for (auto& accessCase : cases) {
@@ -471,12 +498,6 @@ AccessGenerationResult PolymorphicAccess::regenerate(const GCSafeConcurrentJSLoc
             break;
         }
     }
-    
-    CCallHelpers jit(codeBlock);
-    state.jit = &jit;
-
-    state.preservedReusedRegisterState =
-        allocator.preserveReusedRegistersByPushing(jit, ScratchRegisterAllocator::ExtraStackSpace::NoExtraSpace);
 
     bool generatedFinalCode = false;
 
@@ -490,14 +511,43 @@ AccessGenerationResult PolymorphicAccess::regenerate(const GCSafeConcurrentJSLoc
         generatedFinalCode = true;
     }
 
-    if (PolymorphicAccessInternal::verbose)
-        dataLog("Optimized cases: ", listDump(cases), "\n");
-    
+    dataLogLnIf(PolymorphicAccessInternal::verbose, "Optimized cases: ", listDump(cases));
+
+    bool doesCalls = false;
+    bool doesJSGetterSetterCalls = false;
+    bool canBeShared = Options::useDataICSharing();
+    Vector<JSCell*> cellsToMark;
+    FixedVector<RefPtr<AccessCase>> keys(cases.size());
+    unsigned index = 0;
+    for (auto& entry : cases) {
+        doesCalls |= entry->doesCalls(vm, &cellsToMark);
+        switch (entry->type()) {
+        case AccessCase::Getter:
+        case AccessCase::Setter:
+            // Getter / Setter relies on stack-pointer adjustment, which is tied to the linked CodeBlock, which makes this code unshareable.
+            canBeShared = false;
+            doesJSGetterSetterCalls = true;
+            break;
+        case AccessCase::CustomValueGetter:
+        case AccessCase::CustomAccessorGetter:
+        case AccessCase::CustomValueSetter:
+        case AccessCase::CustomAccessorSetter:
+            // Custom getter / setter emits JSGlobalObject pointer, which is tied to the linked CodeBlock.
+            canBeShared = false;
+            break;
+        default:
+            break;
+        }
+        keys[index] = entry;
+        ++index;
+    }
+    state.m_doesCalls = doesCalls;
+    state.m_doesJSGetterSetterCalls = doesJSGetterSetterCalls;
+
     // At this point we're convinced that 'cases' contains the cases that we want to JIT now and we
     // won't change that set anymore.
     
     bool allGuardedByStructureCheck = true;
-    bool hasJSGetterSetterCall = false;
     bool needsInt32PropertyCheck = false;
     bool needsStringPropertyCheck = false;
     bool needsSymbolPropertyCheck = false;
@@ -513,10 +563,42 @@ AccessGenerationResult PolymorphicAccess::regenerate(const GCSafeConcurrentJSLoc
         }
         commit(locker, vm, state.watchpoints, codeBlock, stubInfo, *newCase);
         allGuardedByStructureCheck &= newCase->guardedByStructureCheck(stubInfo);
-        if (newCase->type() == AccessCase::Getter || newCase->type() == AccessCase::Setter)
-            hasJSGetterSetterCall = true;
+        if (newCase->usesPolyProto())
+            canBeShared = false;
+    }
+    if (needsSymbolPropertyCheck || needsStringPropertyCheck || needsInt32PropertyCheck)
+        canBeShared = false;
+
+    auto finishCodeGeneration = [&](RefPtr<PolymorphicAccessJITStubRoutine>&& stub) {
+        m_stubRoutine = WTFMove(stub);
+        m_watchpoints = WTFMove(state.watchpoints);
+        dataLogLnIf(PolymorphicAccessInternal::verbose, "Returning: ", m_stubRoutine->code());
+
+        m_list = WTFMove(cases);
+        m_list.shrinkToFit();
+
+        AccessGenerationResult::Kind resultKind;
+        if (m_list.size() >= Options::maxAccessVariantListSize() || generatedFinalCode)
+            resultKind = AccessGenerationResult::GeneratedFinalCode;
+        else
+            resultKind = AccessGenerationResult::GeneratedNewCode;
+
+        return AccessGenerationResult(resultKind, m_stubRoutine->code().code());
+    };
+
+    CCallHelpers jit(codeBlock);
+    state.jit = &jit;
+
+    if (!canBeShared && ASSERT_ENABLED) {
+        jit.addPtr(CCallHelpers::TrustedImm32(codeBlock->stackPointerOffset() * sizeof(Register)), GPRInfo::callFrameRegister, jit.scratchRegister());
+        auto ok = jit.branchPtr(CCallHelpers::Equal, CCallHelpers::stackPointerRegister, jit.scratchRegister());
+        jit.breakpoint();
+        ok.link(&jit);
     }
 
+    state.preservedReusedRegisterState =
+        allocator.preserveReusedRegistersByPushing(jit, ScratchRegisterAllocator::ExtraStackSpace::NoExtraSpace);
+    
     if (cases.isEmpty()) {
         // This is super unlikely, but we make it legal anyway.
         state.failAndRepatch.append(jit.jump());
@@ -531,9 +613,9 @@ AccessGenerationResult PolymorphicAccess::regenerate(const GCSafeConcurrentJSLoc
 
                 if (!stubInfo.propertyIsInt32) {
 #if USE(JSVALUE64) 
-                    notInt32 = jit.branchIfNotInt32(state.u.propertyGPR);
+                    notInt32 = jit.branchIfNotInt32(state.propertyGPR());
 #else
-                    notInt32 = jit.branchIfNotInt32(state.stubInfo->v.propertyTagGPR);
+                    notInt32 = jit.branchIfNotInt32(state.stubInfo->propertyTagGPR());
 #endif
                 }
                 for (unsigned i = cases.size(); i--;) {
@@ -556,10 +638,10 @@ AccessGenerationResult PolymorphicAccess::regenerate(const GCSafeConcurrentJSLoc
 
             if (needsStringPropertyCheck) {
                 CCallHelpers::JumpList notString;
-                GPRReg propertyGPR = state.u.propertyGPR;
+                GPRReg propertyGPR = state.propertyGPR();
                 if (!stubInfo.propertyIsString) {
 #if USE(JSVALUE32_64)
-                    GPRReg propertyTagGPR = state.stubInfo->v.propertyTagGPR;
+                    GPRReg propertyTagGPR = state.stubInfo->propertyTagGPR();
                     notString.append(jit.branchIfNotCell(propertyTagGPR));
 #else
                     notString.append(jit.branchIfNotCell(propertyGPR));
@@ -589,9 +671,9 @@ AccessGenerationResult PolymorphicAccess::regenerate(const GCSafeConcurrentJSLoc
             if (needsSymbolPropertyCheck) {
                 CCallHelpers::JumpList notSymbol;
                 if (!stubInfo.propertyIsSymbol) {
-                    GPRReg propertyGPR = state.u.propertyGPR;
+                    GPRReg propertyGPR = state.propertyGPR();
 #if USE(JSVALUE32_64)
-                    GPRReg propertyTagGPR = state.stubInfo->v.propertyTagGPR;
+                    GPRReg propertyTagGPR = state.stubInfo->propertyTagGPR();
                     notSymbol.append(jit.branchIfNotCell(propertyTagGPR));
 #else
                     notSymbol.append(jit.branchIfNotCell(propertyGPR));
@@ -641,14 +723,24 @@ AccessGenerationResult PolymorphicAccess::regenerate(const GCSafeConcurrentJSLoc
         // of something that isn't patchable. The slow path will decrement "countdown" and will only
         // patch things if the countdown reaches zero. We increment the slow path count here to ensure
         // that the slow path does not try to patch.
-#if CPU(X86) || CPU(X86_64)
-        jit.move(CCallHelpers::TrustedImmPtr(&stubInfo.countdown), state.scratchGPR);
-        jit.add8(CCallHelpers::TrustedImm32(1), CCallHelpers::Address(state.scratchGPR));
+        if (codeBlock->useDataIC()) {
+#if CPU(X86) || CPU(X86_64) || CPU(ARM64)
+            jit.add8(CCallHelpers::TrustedImm32(1), CCallHelpers::Address(stubInfo.m_stubInfoGPR, StructureStubInfo::offsetOfCountdown()));
 #else
-        jit.load8(&stubInfo.countdown, state.scratchGPR);
-        jit.add32(CCallHelpers::TrustedImm32(1), state.scratchGPR);
-        jit.store8(state.scratchGPR, &stubInfo.countdown);
+            jit.load8(CCallHelpers::Address(stubInfo.m_stubInfoGPR, StructureStubInfo::offsetOfCountdown()), state.scratchGPR);
+            jit.add32(CCallHelpers::TrustedImm32(1), state.scratchGPR);
+            jit.store8(state.scratchGPR, CCallHelpers::Address(stubInfo.m_stubInfoGPR, StructureStubInfo::offsetOfCountdown()));
 #endif
+        } else {
+#if CPU(X86) || CPU(X86_64) || CPU(ARM64)
+            jit.move(CCallHelpers::TrustedImmPtr(&stubInfo.countdown), state.scratchGPR);
+            jit.add8(CCallHelpers::TrustedImm32(1), CCallHelpers::Address(state.scratchGPR));
+#else
+            jit.load8(&stubInfo.countdown, state.scratchGPR);
+            jit.add32(CCallHelpers::TrustedImm32(1), state.scratchGPR);
+            jit.store8(state.scratchGPR, &stubInfo.countdown);
+#endif
+        }
     }
 
     CCallHelpers::JumpList failure;
@@ -661,7 +753,7 @@ AccessGenerationResult PolymorphicAccess::regenerate(const GCSafeConcurrentJSLoc
 
     CodeBlock* codeBlockThatOwnsExceptionHandlers = nullptr;
     DisposableCallSiteIndex callSiteIndexForExceptionHandling;
-    if (state.needsToRestoreRegistersIfException() && hasJSGetterSetterCall) {
+    if (state.needsToRestoreRegistersIfException() && doesJSGetterSetterCalls) {
         // Emit the exception handler.
         // Note that this code is only reachable when doing genericUnwind from a pure JS getter/setter .
         // Note also that this is not reachable from custom getter/setter. Custom getter/setters will have 
@@ -701,50 +793,64 @@ AccessGenerationResult PolymorphicAccess::regenerate(const GCSafeConcurrentJSLoc
         callSiteIndexForExceptionHandling = state.callSiteIndexForExceptionHandling();
     }
 
-    LinkBuffer linkBuffer(jit, codeBlock, JITCompilationCanFail);
+    if (codeBlock->useDataIC()) {
+        failure.link(&jit);
+        // In ARM64, we do not push anything on stack specially.
+        // So we can just jump to the slow-path even though this thunk is called (not jumped).
+        // FIXME: We should tail call to the thunk which calls the slow path function.
+        // And we should eliminate IC slow-path generation in BaselineJIT.
+        jit.farJump(CCallHelpers::Address(stubInfo.m_stubInfoGPR, StructureStubInfo::offsetOfSlowPathStartLocation()), JITStubRoutinePtrTag);
+    }
+
+    RefPtr<PolymorphicAccessJITStubRoutine> stub;
+    FixedVector<StructureID> weakStructures(WTFMove(state.weakStructures));
+    if (codeBlock->useDataIC() && canBeShared) {
+        SharedJITStubSet::Searcher searcher {
+            stubInfo.m_baseGPR,
+            stubInfo.m_valueGPR,
+            stubInfo.m_extraGPR,
+            stubInfo.m_stubInfoGPR,
+            stubInfo.m_arrayProfileGPR,
+            stubInfo.usedRegisters,
+            keys,
+            weakStructures,
+        };
+        stub = vm.m_sharedJITStubs->find(searcher);
+        if (stub) {
+            dataLogLnIf(PolymorphicAccessInternal::verbose, "Found existing code stub ", stub->code());
+            return finishCodeGeneration(WTFMove(stub));
+        }
+    }
+
+    LinkBuffer linkBuffer(jit, codeBlock, LinkBuffer::Profile::InlineCache, JITCompilationCanFail);
     if (linkBuffer.didFailToAllocate()) {
-        if (PolymorphicAccessInternal::verbose)
-            dataLog("Did fail to allocate.\n");
+        dataLogLnIf(PolymorphicAccessInternal::verbose, "Did fail to allocate.");
         return AccessGenerationResult::GaveUp;
     }
 
     CodeLocationLabel<JSInternalPtrTag> successLabel = stubInfo.doneLocation;
 
-    linkBuffer.link(state.success, successLabel);
-
-    linkBuffer.link(failure, stubInfo.slowPathStartLocation);
+    if (codeBlock->useDataIC())
+        ASSERT(state.success.empty());
+    else {
+        linkBuffer.link(state.success, successLabel);
+        linkBuffer.link(failure, stubInfo.slowPathStartLocation);
+    }
     
-    if (PolymorphicAccessInternal::verbose)
-        dataLog(FullCodeOrigin(codeBlock, stubInfo.codeOrigin), ": Generating polymorphic access stub for ", listDump(cases), "\n");
+    dataLogLnIf(PolymorphicAccessInternal::verbose, FullCodeOrigin(codeBlock, stubInfo.codeOrigin), ": Generating polymorphic access stub for ", listDump(cases));
 
     MacroAssemblerCodeRef<JITStubRoutinePtrTag> code = FINALIZE_CODE_FOR(
         codeBlock, linkBuffer, JITStubRoutinePtrTag,
         "%s", toCString("Access stub for ", *codeBlock, " ", stubInfo.codeOrigin, " with return point ", successLabel, ": ", listDump(cases)).data());
+    
+    stub = createICJITStubRoutine(code, WTFMove(keys), WTFMove(weakStructures), vm, codeBlock, doesCalls, cellsToMark, WTFMove(state.m_callLinkInfos), codeBlockThatOwnsExceptionHandlers, callSiteIndexForExceptionHandling);
 
-    bool doesCalls = false;
-    Vector<JSCell*> cellsToMark;
-    for (auto& entry : cases)
-        doesCalls |= entry->doesCalls(vm, &cellsToMark);
-    
-    m_stubRoutine = createJITStubRoutine(code, vm, codeBlock, doesCalls, cellsToMark, WTFMove(state.m_callLinkInfos), codeBlockThatOwnsExceptionHandlers, callSiteIndexForExceptionHandling);
-    m_watchpoints = WTFMove(state.watchpoints);
-    if (!state.weakReferences.isEmpty()) {
-        state.weakReferences.shrinkToFit();
-        m_weakReferences = makeUnique<Vector<WriteBarrier<JSCell>>>(WTFMove(state.weakReferences));
+    if (codeBlock->useDataIC()) {
+        if (canBeShared)
+            vm.m_sharedJITStubs->add(SharedJITStubSet::Hash::Key(stubInfo.m_baseGPR, stubInfo.m_valueGPR, stubInfo.m_extraGPR, stubInfo.m_stubInfoGPR, stubInfo.m_arrayProfileGPR, stubInfo.usedRegisters, stub.get()));
     }
-    if (PolymorphicAccessInternal::verbose)
-        dataLog("Returning: ", code.code(), "\n");
-    
-    m_list = WTFMove(cases);
-    m_list.shrinkToFit();
-    
-    AccessGenerationResult::Kind resultKind;
-    if (m_list.size() >= Options::maxAccessVariantListSize() || generatedFinalCode)
-        resultKind = AccessGenerationResult::GeneratedFinalCode;
-    else
-        resultKind = AccessGenerationResult::GeneratedNewCode;
-    
-    return AccessGenerationResult(resultKind, code.code());
+
+    return finishCodeGeneration(WTFMove(stub));
 }
 
 void PolymorphicAccess::aboutToDie()
@@ -839,6 +945,12 @@ void printInternal(PrintStream& out, AccessCase::AccessType type)
     case AccessCase::InMiss:
         out.print("InMiss");
         return;
+    case AccessCase::CheckPrivateBrand:
+        out.print("CheckPrivateBrand");
+        return;
+    case AccessCase::SetPrivateBrand:
+        out.print("SetPrivateBrand");
+        return;
     case AccessCase::ArrayLength:
         out.print("ArrayLength");
         return;
@@ -911,6 +1023,48 @@ void printInternal(PrintStream& out, AccessCase::AccessType type)
     case AccessCase::IndexedStringLoad:
         out.print("IndexedStringLoad");
         return;
+    case AccessCase::IndexedNoIndexingMiss:
+        out.print("IndexedNoIndexingMiss");
+        return;
+    case AccessCase::IndexedInt32Store:
+        out.print("IndexedInt32Store");
+        return;
+    case AccessCase::IndexedDoubleStore:
+        out.print("IndexedDoubleStore");
+        return;
+    case AccessCase::IndexedContiguousStore:
+        out.print("IndexedContiguousStore");
+        return;
+    case AccessCase::IndexedArrayStorageStore:
+        out.print("IndexedArrayStorageStore");
+        return;
+    case AccessCase::IndexedTypedArrayInt8Store:
+        out.print("IndexedTypedArrayInt8Store");
+        return;
+    case AccessCase::IndexedTypedArrayUint8Store:
+        out.print("IndexedTypedArrayUint8Store");
+        return;
+    case AccessCase::IndexedTypedArrayUint8ClampedStore:
+        out.print("IndexedTypedArrayUint8ClampedStore");
+        return;
+    case AccessCase::IndexedTypedArrayInt16Store:
+        out.print("IndexedTypedArrayInt16Store");
+        return;
+    case AccessCase::IndexedTypedArrayUint16Store:
+        out.print("IndexedTypedArrayUint16Store");
+        return;
+    case AccessCase::IndexedTypedArrayInt32Store:
+        out.print("IndexedTypedArrayInt32Store");
+        return;
+    case AccessCase::IndexedTypedArrayUint32Store:
+        out.print("IndexedTypedArrayUint32Store");
+        return;
+    case AccessCase::IndexedTypedArrayFloat32Store:
+        out.print("IndexedTypedArrayFloat32Store");
+        return;
+    case AccessCase::IndexedTypedArrayFloat64Store:
+        out.print("IndexedTypedArrayFloat64Store");
+        return;
     }
 
     RELEASE_ASSERT_NOT_REACHED();
