@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2011-2022 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,12 +29,17 @@
 #include "config.h"
 #include "SpeculatedType.h"
 
+#include "DateInstance.h"
 #include "DirectArguments.h"
+#include "Integrity.h"
 #include "JSArray.h"
 #include "JSBigInt.h"
 #include "JSBoundFunction.h"
-#include "JSCInlines.h"
+#include "JSCJSValueInlines.h"
+#include "JSCellInlines.h"
+#include "JSDataView.h"
 #include "JSFunction.h"
+#include "JSGenericTypedArrayView.h"
 #include "JSMap.h"
 #include "JSSet.h"
 #include "JSWeakMap.h"
@@ -43,7 +48,6 @@
 #include "RegExpObject.h"
 #include "ScopedArguments.h"
 #include "StringObject.h"
-#include "ValueProfile.h"
 #include <wtf/CommaPrinter.h>
 #include <wtf/StringPrintStream.h>
 
@@ -148,7 +152,17 @@ void dumpSpeculation(PrintStream& outStream, SpeculatedType value)
                 strOut.print("Float64Array");
             else
                 isTop = false;
-    
+
+            if (value & SpecBigInt64Array)
+                strOut.print("BigInt64Array");
+            else
+                isTop = false;
+
+            if (value & SpecBigUint64Array)
+                strOut.print("BigUint64Array");
+            else
+                isTop = false;
+
             if (value & SpecFunction)
                 strOut.print("Function");
             else
@@ -174,6 +188,16 @@ void dumpSpeculation(PrintStream& outStream, SpeculatedType value)
             else
                 isTop = false;
 
+            if (value & SpecDateObject)
+                strOut.print("DateObject");
+            else
+                isTop = false;
+
+            if (value & SpecPromiseObject)
+                strOut.print("PromiseObject");
+            else
+                isTop = false;
+
             if (value & SpecMapObject)
                 strOut.print("MapObject");
             else
@@ -229,8 +253,8 @@ void dumpSpeculation(PrintStream& outStream, SpeculatedType value)
         else
             isTop = false;
 
-        if (value & SpecBigInt)
-            strOut.print("BigInt");
+        if (value & SpecHeapBigInt)
+            strOut.print("HeapBigInt");
         else
             isTop = false;
     }
@@ -263,13 +287,20 @@ void dumpSpeculation(PrintStream& outStream, SpeculatedType value)
             isTop = false;
         
         if (value & SpecDoublePureNaN)
-            strOut.print("DoublePureNan");
+            strOut.print("DoublePureNaN");
         else
             isTop = false;
     }
-    
+
+#if USE(BIGINT32)
+    if (value & SpecBigInt32)
+        strOut.print("BigInt32");
+    else
+        isTop = false;
+#endif
+
     if (value & SpecDoubleImpureNaN)
-        out.print("DoubleImpureNan");
+        strOut.print("DoubleImpureNaN");
     
     if (value & SpecBoolean)
         strOut.print("Bool");
@@ -338,6 +369,10 @@ static const char* speculationToAbbreviatedString(SpeculatedType prediction)
         return "<Float32array>";
     if (isFloat64ArraySpeculation(prediction))
         return "<Float64array>";
+    if (isBigInt64ArraySpeculation(prediction))
+        return "<BigInt64array>";
+    if (isBigUint64ArraySpeculation(prediction))
+        return "<BigUint64array>";
     if (isDirectArgumentsSpeculation(prediction))
         return "<DirectArguments>";
     if (isScopedArgumentsSpeculation(prediction))
@@ -403,6 +438,10 @@ SpeculatedType speculationFromTypedArrayType(TypedArrayType type)
         return SpecFloat32Array;
     case TypeFloat64:
         return SpecFloat64Array;
+    case TypeBigInt64:
+        return SpecBigInt64Array;
+    case TypeBigUint64:
+        return SpecBigUint64Array;
     case NotTypedArray:
     case TypeDataView:
         break;
@@ -411,65 +450,87 @@ SpeculatedType speculationFromTypedArrayType(TypedArrayType type)
     return SpecNone;
 }
 
-SpeculatedType speculationFromClassInfo(const ClassInfo* classInfo)
+SpeculatedType speculationFromClassInfoInheritance(const ClassInfo* classInfo)
 {
     if (classInfo == JSString::info())
         return SpecString;
+    ASSERT_WITH_MESSAGE(!classInfo->isSubClassOf(JSString::info()), "Rope strings should still have JSString's ClassInfo");
 
+    static_assert(std::is_final_v<Symbol>);
     if (classInfo == Symbol::info())
         return SpecSymbol;
-    
-    if (classInfo == JSBigInt::info())
-        return SpecBigInt;
 
+    static_assert(std::is_final_v<JSBigInt>);
+    if (classInfo == JSBigInt::info())
+        return SpecHeapBigInt;
+
+    static_assert(std::is_final_v<JSFinalObject>);
     if (classInfo == JSFinalObject::info())
         return SpecFinalObject;
     
-    if (classInfo == JSArray::info())
-        return SpecArray;
-    
+    static_assert(std::is_final_v<DirectArguments>);
     if (classInfo == DirectArguments::info())
         return SpecDirectArguments;
     
+    static_assert(std::is_final_v<ScopedArguments>);
     if (classInfo == ScopedArguments::info())
         return SpecScopedArguments;
-    
-    if (classInfo == StringObject::info())
-        return SpecStringObject;
 
+    static_assert(std::is_final_v<RegExpObject>);
     if (classInfo == RegExpObject::info())
         return SpecRegExpObject;
 
+    static_assert(std::is_final_v<DateInstance>);
+    if (classInfo == DateInstance::info())
+        return SpecDateObject;
+
+    static_assert(std::is_final_v<JSMap>);
     if (classInfo == JSMap::info())
         return SpecMapObject;
 
+    static_assert(std::is_final_v<JSSet>);
     if (classInfo == JSSet::info())
         return SpecSetObject;
 
+    static_assert(std::is_final_v<JSWeakMap>);
     if (classInfo == JSWeakMap::info())
         return SpecWeakMapObject;
 
+    static_assert(std::is_final_v<JSWeakSet>);
     if (classInfo == JSWeakSet::info())
         return SpecWeakSetObject;
 
+    static_assert(std::is_final_v<ProxyObject>);
     if (classInfo == ProxyObject::info())
         return SpecProxyObject;
 
+    static_assert(std::is_final_v<JSDataView>);
     if (classInfo == JSDataView::info())
         return SpecDataViewObject;
-    
+
+    if (classInfo->isSubClassOf(StringObject::info()))
+        return SpecStringObject | SpecObjectOther;
+
+    if (classInfo->isSubClassOf(JSArray::info()))
+        return SpecArray | SpecDerivedArray;
+
     if (classInfo->isSubClassOf(JSFunction::info())) {
         if (classInfo == JSBoundFunction::info())
             return SpecFunctionWithNonDefaultHasInstance;
         return SpecFunctionWithDefaultHasInstance;
     }
-    
-    if (isTypedView(classInfo->typedArrayStorageType))
-        return speculationFromTypedArrayType(classInfo->typedArrayStorageType);
 
-    if (classInfo->isSubClassOf(JSArray::info()))
-        return SpecDerivedArray;
+    if (classInfo->isSubClassOf(JSPromise::info()))
+        return SpecPromiseObject;
     
+#define JSC_TYPED_ARRAY_CHECK(type) do { \
+        static_assert(std::is_final<JS ## type ## Array>::value); \
+        if (classInfo == JS ## type ## Array::info()) \
+            return Spec ## type ## Array; \
+    } while (0);
+    FOR_EACH_TYPED_ARRAY_TYPE_EXCLUDING_DATA_VIEW(JSC_TYPED_ARRAY_CHECK)
+#undef JSC_TYPED_ARRAY_CHECK
+
     if (classInfo->isSubClassOf(JSObject::info()))
         return SpecObjectOther;
     
@@ -478,28 +539,64 @@ SpeculatedType speculationFromClassInfo(const ClassInfo* classInfo)
 
 SpeculatedType speculationFromStructure(Structure* structure)
 {
-    if (structure->typeInfo().type() == StringType)
-        return SpecString;
-    if (structure->typeInfo().type() == SymbolType)
-        return SpecSymbol;
-    if (structure->typeInfo().type() == BigIntType)
-        return SpecBigInt;
-    if (structure->typeInfo().type() == DerivedArrayType)
-        return SpecDerivedArray;
-    return speculationFromClassInfo(structure->classInfo());
+    SpeculatedType filteredResult = SpecNone;
+    switch (structure->typeInfo().type()) {
+    case StringType:
+        filteredResult = SpecString;
+        break;
+    case SymbolType:
+        filteredResult = SpecSymbol;
+        break;
+    case HeapBigIntType:
+        filteredResult = SpecHeapBigInt;
+        break;
+    case DerivedArrayType:
+        filteredResult = SpecDerivedArray;
+        break;
+    case ArrayType:
+        filteredResult = SpecArray;
+        break;
+    case StringObjectType:
+        filteredResult = SpecStringObject;
+        break;
+    // We do not want to accept String.prototype in StringObjectUse, so that we do not include it as SpecStringObject.
+    case DerivedStringObjectType:
+        filteredResult = SpecObjectOther;
+        break;
+    default:
+        return speculationFromClassInfoInheritance(structure->classInfoForCells());
+    }
+    ASSERT(filteredResult);
+    ASSERT(isSubtypeSpeculation(filteredResult, speculationFromClassInfoInheritance(structure->classInfoForCells())));
+    return filteredResult;
 }
 
 SpeculatedType speculationFromCell(JSCell* cell)
 {
+    // FIXME: rdar://69036888: remove isSanePointer checks when no longer needed.
+    if (UNLIKELY(!Integrity::isSanePointer(cell))) {
+        ASSERT_NOT_REACHED();
+        return SpecNone;
+    }
     if (cell->isString()) {
         JSString* string = jsCast<JSString*>(cell);
         if (const StringImpl* impl = string->tryGetValueImpl()) {
+            if (UNLIKELY(!Integrity::isSanePointer(impl))) {
+                ASSERT_NOT_REACHED();
+                return SpecNone;
+            }
             if (impl->isAtom())
                 return SpecStringIdent;
         }
         return SpecString;
     }
-    return speculationFromStructure(cell->structure());
+    // FIXME: rdar://69036888: undo this when no longer needed.
+    auto* structure = cell->structureID().tryDecode();
+    if (UNLIKELY(!Integrity::isSanePointer(structure))) {
+        ASSERT_NOT_REACHED();
+        return SpecNone;
+    }
+    return speculationFromStructure(structure);
 }
 
 SpeculatedType speculationFromValue(JSValue value)
@@ -519,6 +616,8 @@ SpeculatedType speculationFromValue(JSValue value)
             return SpecAnyIntAsDouble;
         return SpecNonIntAsDouble;
     }
+    if (value.isBigInt32())
+        return SpecBigInt32;
     if (value.isCell())
         return speculationFromCell(value.asCell());
     if (value.isBoolean())
@@ -567,27 +666,37 @@ TypedArrayType typedArrayTypeFromSpeculation(SpeculatedType type)
         
     if (isFloat64ArraySpeculation(type))
         return TypeFloat64;
+
+    if (isBigInt64ArraySpeculation(type))
+        return TypeBigInt64;
+
+    if (isBigUint64ArraySpeculation(type))
+        return TypeBigUint64;
     
     return NotTypedArray;
 }
 
-SpeculatedType speculationFromJSType(JSType type)
+std::optional<SpeculatedType> speculationFromJSType(JSType type)
 {
     switch (type) {
     case StringType:
         return SpecString;
     case SymbolType:
         return SpecSymbol;
-    case BigIntType:
-        return SpecBigInt;
+    case HeapBigIntType:
+        return SpecHeapBigInt;
     case ArrayType:
         return SpecArray;
     case DerivedArrayType:
         return SpecDerivedArray;
     case RegExpObjectType:
         return SpecRegExpObject;
+    case JSDateType:
+        return SpecDateObject;
     case ProxyObjectType:
         return SpecProxyObject;
+    case JSPromiseType:
+        return SpecPromiseObject;
     case JSMapType:
         return SpecMapObject;
     case JSSetType:
@@ -599,9 +708,8 @@ SpeculatedType speculationFromJSType(JSType type)
     case DataViewType:
         return SpecDataViewObject;
     default:
-        ASSERT_NOT_REACHED();
+        return std::nullopt;
     }
-    return SpecNone;
 }
 
 SpeculatedType leastUpperBoundOfStrictlyEquivalentSpeculations(SpeculatedType type)
@@ -612,13 +720,28 @@ SpeculatedType leastUpperBoundOfStrictlyEquivalentSpeculations(SpeculatedType ty
 
     if (type & SpecString)
         type |= SpecString;
+
+    if (type & SpecBigInt)
+        type |= SpecBigInt;
+
+    return type;
+}
+
+static inline SpeculatedType leastUpperBoundOfEquivalentSpeculations(SpeculatedType type)
+{
+    type = leastUpperBoundOfStrictlyEquivalentSpeculations(type);
+
+    // Boolean or BigInt can be converted to Number when performing non-strict equal.
+    if (type & (SpecIntAnyFormat | SpecNonIntAsDouble | SpecBoolean | SpecBigInt))
+        type |= (SpecIntAnyFormat | SpecNonIntAsDouble | SpecBoolean | SpecBigInt);
+
     return type;
 }
 
 bool valuesCouldBeEqual(SpeculatedType a, SpeculatedType b)
 {
-    a = leastUpperBoundOfStrictlyEquivalentSpeculations(a);
-    b = leastUpperBoundOfStrictlyEquivalentSpeculations(b);
+    a = leastUpperBoundOfEquivalentSpeculations(a);
+    b = leastUpperBoundOfEquivalentSpeculations(b);
     
     // Anything could be equal to a string.
     if (a & SpecString)
@@ -672,6 +795,17 @@ SpeculatedType typeOfDoubleDifference(SpeculatedType a, SpeculatedType b)
     return typeOfDoubleSumOrDifferenceOrProduct(a, b);
 }
 
+SpeculatedType typeOfDoubleIncOrDec(SpeculatedType t)
+{
+    // Impure NaN could become pure NaN during addition because addition may clear bits.
+    if (t & SpecDoubleImpureNaN)
+        t |= SpecDoublePureNaN;
+    // Values could overflow, or fractions could become integers.
+    if (t & SpecDoubleReal)
+        t |= SpecDoubleReal;
+    return t;
+}
+
 SpeculatedType typeOfDoubleProduct(SpeculatedType a, SpeculatedType b)
 {
     return typeOfDoubleSumOrDifferenceOrProduct(a, b);
@@ -784,6 +918,10 @@ SpeculatedType speculationFromString(const char* speculation)
         return SpecFloat32Array;
     if (!strncmp(speculation, "SpecFloat64Array", strlen("SpecFloat64Array")))
         return SpecFloat64Array;
+    if (!strncmp(speculation, "SpecBigInt64Array", strlen("SpecBigInt64Array")))
+        return SpecBigInt64Array;
+    if (!strncmp(speculation, "SpecBigUint64Array", strlen("SpecBigUint64Array")))
+        return SpecBigUint64Array;
     if (!strncmp(speculation, "SpecTypedArrayView", strlen("SpecTypedArrayView")))
         return SpecTypedArrayView;
     if (!strncmp(speculation, "SpecDirectArguments", strlen("SpecDirectArguments")))
@@ -794,6 +932,10 @@ SpeculatedType speculationFromString(const char* speculation)
         return SpecStringObject;
     if (!strncmp(speculation, "SpecRegExpObject", strlen("SpecRegExpObject")))
         return SpecRegExpObject;
+    if (!strncmp(speculation, "SpecDateObject", strlen("SpecDateObject")))
+        return SpecDateObject;
+    if (!strncmp(speculation, "SpecPromiseObject", strlen("SpecPromiseObject")))
+        return SpecPromiseObject;
     if (!strncmp(speculation, "SpecMapObject", strlen("SpecMapObject")))
         return SpecMapObject;
     if (!strncmp(speculation, "SpecSetObject", strlen("SpecSetObject")))
