@@ -28,29 +28,56 @@
 
 #if ENABLE(DFG_JIT)
 
-#include "DFGBasicBlockInlines.h"
+#include "DFGBlockMapInlines.h"
 #include "DFGGraph.h"
 #include "DFGPhase.h"
-#include "JSCInlines.h"
+#include "JSCJSValueInlines.h"
+#include <wtf/MathExtras.h>
 
 namespace JSC { namespace DFG {
 
-class BackwardsPropagationPhase : public Phase {
+// This phase is run at the end of BytecodeParsing, so the graph isn't in a fully formed state.
+// For example, we can't access the predecessor list of any basic blocks yet.
+
+class BackwardsPropagationPhase {
 public:
     BackwardsPropagationPhase(Graph& graph)
-        : Phase(graph, "backwards propagation")
+        : m_graph(graph)
+        , m_flagsAtHead(graph)
     {
     }
-    
+
     bool run()
     {
-        m_changed = true;
-        while (m_changed) {
-            m_changed = false;
+        for (BasicBlock* block : m_graph.blocksInNaturalOrder()) {
+            m_flagsAtHead[block] = Operands<NodeFlags>(OperandsLike, m_graph.block(0)->variablesAtHead);
+            m_flagsAtHead[block].fill(0);
+        }
+
+        bool changed;
+        do {
+            changed = false;
+
             for (BlockIndex blockIndex = m_graph.numBlocks(); blockIndex--;) {
                 BasicBlock* block = m_graph.block(blockIndex);
                 if (!block)
                     continue;
+
+                {
+                    unsigned numSuccessors = block->numSuccessors();
+                    if (!numSuccessors) {
+                        m_currentFlags = Operands<NodeFlags>(OperandsLike, m_graph.block(0)->variablesAtHead);
+                        m_currentFlags.fill(0);
+                    } else {
+                        m_currentFlags = m_flagsAtHead[block->successor(0)];
+                        for (unsigned i = 1; i < numSuccessors; ++i) {
+                            BasicBlock* successor = block->successor(i);
+                            for (size_t i = 0; i < m_currentFlags.size(); ++i)
+                                m_currentFlags[i] |= m_flagsAtHead[successor][i];
+                        }
+                    }
+                }
+
             
                 // Prevent a tower of overflowing additions from creating a value that is out of the
                 // safe 2^48 range.
@@ -58,8 +85,13 @@ public:
             
                 for (unsigned indexInBlock = block->size(); indexInBlock--;)
                     propagate(block->at(indexInBlock));
+
+                if (m_flagsAtHead[block] != m_currentFlags) {
+                    m_flagsAtHead[block] = m_currentFlags;
+                    changed = true;
+                }
             }
-        }
+        } while (changed);
         
         return true;
     }
@@ -154,6 +186,11 @@ private:
         return isWithinPowerOfTwo<power>(edge.node());
     }
 
+    static bool mergeFlags(NodeFlags& flagsRef, NodeFlags newFlags)
+    {
+        return checkAndSet(flagsRef, flagsRef | newFlags);
+    }
+
     bool mergeDefaultFlags(Node* node)
     {
         bool changed = false;
@@ -178,6 +215,10 @@ private:
         return changed;
     }
     
+    static constexpr NodeFlags VariableIsUsed = 1 << (1 + WTF::getMSBSetConstexpr(NodeBytecodeBackPropMask));
+    static_assert(!(VariableIsUsed & NodeBytecodeBackPropMask));
+    static_assert(VariableIsUsed > NodeBytecodeBackPropMask, "Verify the above doesn't overflow");
+    
     void propagate(Node* node)
     {
         NodeFlags flags = node->flags() & NodeBytecodeBackPropMask;
@@ -185,25 +226,41 @@ private:
         switch (node->op()) {
         case GetLocal: {
             VariableAccessData* variableAccessData = node->variableAccessData();
-            flags &= ~NodeBytecodeUsesAsInt; // We don't care about cross-block uses-as-int.
-            m_changed |= variableAccessData->mergeFlags(flags);
+            flags |= m_currentFlags.operand(variableAccessData->operand());
+            flags |= VariableIsUsed;
+            m_currentFlags.operand(variableAccessData->operand()) = flags;
             break;
         }
             
         case SetLocal: {
             VariableAccessData* variableAccessData = node->variableAccessData();
-            if (!variableAccessData->isLoadedFrom())
+
+            Operand operand = variableAccessData->operand();
+            NodeFlags flags = m_currentFlags.operand(operand);
+            if (!(flags & VariableIsUsed))
                 break;
-            flags = variableAccessData->flags();
-            RELEASE_ASSERT(!(flags & ~NodeBytecodeBackPropMask));
-            flags |= NodeBytecodeUsesAsNumber; // Account for the fact that control flow may cause overflows that our modeling can't handle.
-            node->child1()->mergeFlags(flags);
+
+            flags &= NodeBytecodeBackPropMask;
+            flags &= ~NodeBytecodeUsesAsInt; // We don't care about cross-block uses-as-int.
+
+            variableAccessData->mergeFlags(flags);
+            // We union with NodeBytecodeUsesAsNumber to account for the fact that control flow may cause overflows that our modeling can't handle.
+            // For example, a loop where we always add a constant value.
+            node->child1()->mergeFlags(flags | NodeBytecodeUsesAsNumber); 
+
+            m_currentFlags.operand(operand) = 0;
             break;
         }
             
         case Flush: {
             VariableAccessData* variableAccessData = node->variableAccessData();
-            m_changed |= variableAccessData->mergeFlags(NodeBytecodeUsesAsValue);
+            mergeFlags(m_currentFlags.operand(variableAccessData->operand()), NodeBytecodeUsesAsValue | VariableIsUsed);
+            break;
+        }
+
+        case PhantomLocal: {
+            VariableAccessData* variableAccessData = node->variableAccessData();
+            mergeFlags(m_currentFlags.operand(variableAccessData->operand()), VariableIsUsed);
             break;
         }
             
@@ -391,7 +448,8 @@ private:
             node->child2()->mergeFlags(flags & ~NodeBytecodeNeedsNegZero);
             break;
         }
-            
+
+        case EnumeratorGetByVal:
         case GetByVal: {
             m_graph.varArgChild(node, 0)->mergeFlags(NodeBytecodeUsesAsValue);
             m_graph.varArgChild(node, 1)->mergeFlags(NodeBytecodeUsesAsNumber | NodeBytecodeUsesAsOther | NodeBytecodeUsesAsInt | NodeBytecodeUsesAsArrayIndex);
@@ -415,7 +473,8 @@ private:
             
         case ToPrimitive:
         case ToNumber:
-        case ToNumeric: {
+        case ToNumeric:
+        case CallNumberConstructor: {
             node->child1()->mergeFlags(flags);
             break;
         }
@@ -490,13 +549,16 @@ private:
         }
     }
     
+    Graph& m_graph;
     bool m_allowNestedOverflowingAdditions;
-    bool m_changed;
+
+    BlockMap<Operands<NodeFlags>> m_flagsAtHead;
+    Operands<NodeFlags> m_currentFlags;
 };
 
-bool performBackwardsPropagation(Graph& graph)
+void performBackwardsPropagation(Graph& graph)
 {
-    return runPhase<BackwardsPropagationPhase>(graph);
+    BackwardsPropagationPhase(graph).run();
 }
 
 } } // namespace JSC::DFG
