@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2019 Apple Inc. All rights reserved.
+ * Copyright (C) 2011-2022 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,21 +29,18 @@
 #if ENABLE(DFG_JIT)
 
 #include "CodeBlock.h"
+#include "CodeBlockWithJITType.h"
 #include "DFGFailedFinalizer.h"
 #include "DFGInlineCacheWrapperInlines.h"
 #include "DFGJITCode.h"
 #include "DFGJITFinalizer.h"
 #include "DFGOSRExit.h"
-#include "DFGOperations.h"
-#include "DFGRegisterBank.h"
-#include "DFGSlowPathGenerator.h"
 #include "DFGSpeculativeJIT.h"
 #include "DFGThunks.h"
-#include "JSCInlines.h"
 #include "JSCJSValueInlines.h"
 #include "LinkBuffer.h"
 #include "MaxFrameExtentForSlowPathCall.h"
-#include "StructureStubInfo.h"
+#include "ProbeContext.h"
 #include "ThunkGenerators.h"
 #include "VM.h"
 
@@ -52,7 +49,7 @@ namespace JSC { namespace DFG {
 JITCompiler::JITCompiler(Graph& dfg)
     : CCallHelpers(dfg.m_codeBlock)
     , m_graph(dfg)
-    , m_jitCode(adoptRef(new JITCode()))
+    , m_jitCode(adoptRef(new JITCode(m_graph.m_plan.isUnlinked())))
     , m_blockHeads(dfg.numBlocks())
     , m_pcToCodeOriginMapBuilder(dfg.m_vm)
 {
@@ -71,21 +68,29 @@ JITCompiler::~JITCompiler()
 
 void JITCompiler::linkOSRExits()
 {
-    ASSERT(m_jitCode->osrExit.size() == m_exitCompilationInfo.size());
+    ASSERT(m_osrExit.size() == m_exitCompilationInfo.size());
     if (UNLIKELY(m_graph.compilation())) {
-        for (unsigned i = 0; i < m_jitCode->osrExit.size(); ++i) {
+        for (unsigned i = 0; i < m_osrExit.size(); ++i) {
             OSRExitCompilationInfo& info = m_exitCompilationInfo[i];
             Vector<Label> labels;
+
+            auto appendLabel = [&] (Label label) {
+                RELEASE_ASSERT(label.isSet());
+                labels.append(label);
+            };
+            
             if (!info.m_failureJumps.empty()) {
                 for (unsigned j = 0; j < info.m_failureJumps.jumps().size(); ++j)
-                    labels.append(info.m_failureJumps.jumps()[j].label());
-            } else
-                labels.append(info.m_replacementSource);
+                    appendLabel(info.m_failureJumps.jumps()[j].label());
+            } else if (info.m_replacementSource.isSet())
+                appendLabel(info.m_replacementSource);
             m_exitSiteLabels.append(labels);
         }
     }
     
-    for (unsigned i = 0; i < m_jitCode->osrExit.size(); ++i) {
+    JumpList dispatchCases;
+    JumpList dispatchCasesWithoutLinkedFailures;
+    for (unsigned i = 0; i < m_osrExit.size(); ++i) {
         OSRExitCompilationInfo& info = m_exitCompilationInfo[i];
         JumpList& failureJumps = info.m_failureJumps;
         if (!failureJumps.empty())
@@ -94,9 +99,47 @@ void JITCompiler::linkOSRExits()
             info.m_replacementDestination = label();
 
         jitAssertHasValidCallFrame();
+#if USE(JSVALUE64)
+        if (m_graph.m_plan.isUnlinked()) {
+            move(TrustedImm32(i), GPRInfo::numberTagRegister);
+            if (info.m_replacementDestination.isSet())
+                dispatchCasesWithoutLinkedFailures.append(jump());
+            else
+                dispatchCases.append(jump());
+            continue;
+        }
+#endif
+        UNUSED_VARIABLE(dispatchCases);
+        UNUSED_VARIABLE(dispatchCasesWithoutLinkedFailures);
         store32(TrustedImm32(i), &vm().osrExitIndex);
         info.m_patchableJump = patchableJump();
     }
+
+#if USE(JSVALUE64)
+    if (m_graph.m_plan.isUnlinked()) {
+        // When jumping to OSR exit handler via exception, we do not have proper callFrameRegister and constantsRegister.
+        // We should reload appropriate callFrameRegister from VM::callFrameForCatch to materialize constants buffer register.
+        // FIXME: The following code can be a DFG Thunk.
+        if (!dispatchCasesWithoutLinkedFailures.empty()) {
+            dispatchCasesWithoutLinkedFailures.link(this);
+            loadPtr(vm().addressOfCallFrameForCatch(), GPRInfo::notCellMaskRegister);
+            MacroAssembler::Jump didNotHaveException = branchTestPtr(MacroAssembler::Zero, GPRInfo::notCellMaskRegister);
+            move(GPRInfo::notCellMaskRegister, GPRInfo::constantsRegister);
+            emitGetFromCallFrameHeaderPtr(CallFrameSlot::codeBlock, GPRInfo::constantsRegister, GPRInfo::constantsRegister);
+            loadPtr(Address(GPRInfo::constantsRegister, CodeBlock::offsetOfJITData()), GPRInfo::constantsRegister);
+            didNotHaveException.link(this);
+        }
+        dispatchCases.link(this);
+        store32(GPRInfo::numberTagRegister, &vm().osrExitIndex);
+        loadPtr(Address(GPRInfo::constantsRegister, JITData::offsetOfExits()), GPRInfo::constantsRegister);
+        static_assert(sizeof(JITData::ExitVector::value_type) == 16);
+        ASSERT(!JITData::ExitVector::value_type::offsetOfCodePtr());
+        lshiftPtr(TrustedImm32(4), GPRInfo::numberTagRegister);
+        addPtr(GPRInfo::numberTagRegister, GPRInfo::constantsRegister);
+        emitMaterializeTagCheckRegisters();
+        farJump(Address(GPRInfo::constantsRegister, JITData::ExitVector::Storage::offsetOfData()), OSRExitPtrTag);
+    }
+#endif
 }
 
 void JITCompiler::compileEntry()
@@ -108,13 +151,19 @@ void JITCompiler::compileEntry()
     // check) which will be dependent on stack layout. (We'd need to account for this in
     // both normal return code and when jumping to an exception handler).
     emitFunctionPrologue();
-    emitPutToCallFrameHeader(m_codeBlock, CallFrameSlot::codeBlock);
+    jitAssertCodeBlockOnCallFrameWithType(GPRInfo::regT2, JITType::DFGJIT);
 }
 
 void JITCompiler::compileSetupRegistersForEntry()
 {
     emitSaveCalleeSaves();
     emitMaterializeTagCheckRegisters();    
+#if USE(JSVALUE64)
+    if (m_graph.m_plan.isUnlinked()) {
+        emitGetFromCallFrameHeaderPtr(CallFrameSlot::codeBlock, GPRInfo::constantsRegister);
+        loadPtr(Address(GPRInfo::constantsRegister, CodeBlock::offsetOfJITData()), GPRInfo::constantsRegister);
+    }
+#endif
 }
 
 void JITCompiler::compileEntryExecutionFlag()
@@ -134,38 +183,6 @@ void JITCompiler::compileBody()
     ASSERT_UNUSED(compiledSpeculative, compiledSpeculative);
 }
 
-void JITCompiler::compileExceptionHandlers()
-{
-    if (!m_exceptionChecksWithCallFrameRollback.empty()) {
-        m_exceptionChecksWithCallFrameRollback.link(this);
-
-        copyCalleeSavesToEntryFrameCalleeSavesBuffer(vm().topEntryFrame);
-
-        // operationLookupExceptionHandlerFromCallerFrame is passed one argument, the VM*.
-        move(TrustedImmPtr(&vm()), GPRInfo::argumentGPR0);
-        prepareCallOperation(vm());
-        addPtr(TrustedImm32(m_graph.stackPointerOffset() * sizeof(Register)), GPRInfo::callFrameRegister, stackPointerRegister);
-
-        m_calls.append(CallLinkRecord(call(OperationPtrTag), FunctionPtr<OperationPtrTag>(operationLookupExceptionHandlerFromCallerFrame)));
-
-        jumpToExceptionHandler(vm());
-    }
-
-    if (!m_exceptionChecks.empty()) {
-        m_exceptionChecks.link(this);
-
-        copyCalleeSavesToEntryFrameCalleeSavesBuffer(vm().topEntryFrame);
-
-        // operationLookupExceptionHandler is passed one argument, the VM*.
-        move(TrustedImmPtr(&vm()), GPRInfo::argumentGPR0);
-        prepareCallOperation(vm());
-
-        m_calls.append(CallLinkRecord(call(OperationPtrTag), FunctionPtr<OperationPtrTag>(operationLookupExceptionHandler)));
-
-        jumpToExceptionHandler(vm());
-    }
-}
-
 void JITCompiler::link(LinkBuffer& linkBuffer)
 {
     // Link the code, populate data in CodeBlock data structures.
@@ -181,60 +198,61 @@ void JITCompiler::link(LinkBuffer& linkBuffer)
     
     m_graph.registerFrozenValues();
 
-    BitVector usedJumpTables;
+    ASSERT(m_jitCode->m_stringSwitchJumpTables.isEmpty());
+    ASSERT(m_jitCode->m_switchJumpTables.isEmpty());
+    if (!m_graph.m_stringSwitchJumpTables.isEmpty()) 
+        m_jitCode->m_stringSwitchJumpTables = WTFMove(m_graph.m_stringSwitchJumpTables);
+    if (!m_graph.m_switchJumpTables.isEmpty())
+        m_jitCode->m_switchJumpTables = WTFMove(m_graph.m_switchJumpTables);
+
     for (Bag<SwitchData>::iterator iter = m_graph.m_switchData.begin(); !!iter; ++iter) {
         SwitchData& data = **iter;
-        if (!data.didUseJumpTable)
-            continue;
-        
-        if (data.kind == SwitchString)
-            continue;
-        
-        RELEASE_ASSERT(data.kind == SwitchImm || data.kind == SwitchChar);
-        
-        usedJumpTables.set(data.switchTableIndex);
-        SimpleJumpTable& table = m_codeBlock->switchJumpTable(data.switchTableIndex);
-        table.ctiDefault = linkBuffer.locationOf<JSSwitchPtrTag>(m_blockHeads[data.fallThrough.block->index]);
-        table.ctiOffsets.grow(table.branchOffsets.size());
-        for (unsigned j = table.ctiOffsets.size(); j--;)
-            table.ctiOffsets[j] = table.ctiDefault;
-        for (unsigned j = data.cases.size(); j--;) {
-            SwitchCase& myCase = data.cases[j];
-            table.ctiOffsets[myCase.value.switchLookupValue(data.kind) - table.min] =
-                linkBuffer.locationOf<JSSwitchPtrTag>(m_blockHeads[myCase.target.block->index]);
+        switch (data.kind) {
+        case SwitchChar:
+        case SwitchImm: {
+            if (!data.didUseJumpTable) {
+                ASSERT(m_jitCode->m_switchJumpTables[data.switchTableIndex].isEmpty());
+                continue;
+            }
+
+            const UnlinkedSimpleJumpTable& unlinkedTable = m_graph.unlinkedSwitchJumpTable(data.switchTableIndex);
+            SimpleJumpTable& linkedTable = m_jitCode->m_switchJumpTables[data.switchTableIndex];
+            linkedTable.m_ctiDefault = linkBuffer.locationOf<JSSwitchPtrTag>(m_blockHeads[data.fallThrough.block->index]);
+            RELEASE_ASSERT(linkedTable.m_ctiOffsets.size() == unlinkedTable.m_branchOffsets.size());
+            for (unsigned j = linkedTable.m_ctiOffsets.size(); j--;)
+                linkedTable.m_ctiOffsets[j] = linkedTable.m_ctiDefault;
+            for (unsigned j = data.cases.size(); j--;) {
+                SwitchCase& myCase = data.cases[j];
+                linkedTable.m_ctiOffsets[myCase.value.switchLookupValue(data.kind) - unlinkedTable.m_min] =
+                    linkBuffer.locationOf<JSSwitchPtrTag>(m_blockHeads[myCase.target.block->index]);
+            }
+            break;
         }
-    }
-    
-    for (unsigned i = m_codeBlock->numberOfSwitchJumpTables(); i--;) {
-        if (usedJumpTables.get(i))
-            continue;
-        
-        m_codeBlock->switchJumpTable(i).clear();
-    }
 
-    // NOTE: we cannot clear string switch tables because (1) we're running concurrently
-    // and we cannot deref StringImpl's and (2) it would be weird to deref those
-    // StringImpl's since we refer to them.
-    for (Bag<SwitchData>::iterator switchDataIter = m_graph.m_switchData.begin(); !!switchDataIter; ++switchDataIter) {
-        SwitchData& data = **switchDataIter;
-        if (!data.didUseJumpTable)
-            continue;
-        
-        if (data.kind != SwitchString)
-            continue;
-        
-        StringJumpTable& table = m_codeBlock->stringSwitchJumpTable(data.switchTableIndex);
+        case SwitchString: {
+            if (!data.didUseJumpTable) {
+                ASSERT(m_jitCode->m_stringSwitchJumpTables[data.switchTableIndex].isEmpty());
+                continue;
+            }
 
-        table.ctiDefault = linkBuffer.locationOf<JSSwitchPtrTag>(m_blockHeads[data.fallThrough.block->index]);
-        StringJumpTable::StringOffsetTable::iterator iter;
-        StringJumpTable::StringOffsetTable::iterator end = table.offsetTable.end();
-        for (iter = table.offsetTable.begin(); iter != end; ++iter)
-            iter->value.ctiOffset = table.ctiDefault;
-        for (unsigned j = data.cases.size(); j--;) {
-            SwitchCase& myCase = data.cases[j];
-            iter = table.offsetTable.find(myCase.value.stringImpl());
-            RELEASE_ASSERT(iter != end);
-            iter->value.ctiOffset = linkBuffer.locationOf<JSSwitchPtrTag>(m_blockHeads[myCase.target.block->index]);
+            const UnlinkedStringJumpTable& unlinkedTable = m_graph.unlinkedStringSwitchJumpTable(data.switchTableIndex);
+            StringJumpTable& linkedTable = m_jitCode->m_stringSwitchJumpTables[data.switchTableIndex];
+            auto ctiDefault = linkBuffer.locationOf<JSSwitchPtrTag>(m_blockHeads[data.fallThrough.block->index]);
+            RELEASE_ASSERT(linkedTable.m_ctiOffsets.size() == unlinkedTable.m_offsetTable.size() + 1);
+            for (auto& entry : linkedTable.m_ctiOffsets)
+                entry = ctiDefault;
+            for (unsigned j = data.cases.size(); j--;) {
+                SwitchCase& myCase = data.cases[j];
+                auto iter = unlinkedTable.m_offsetTable.find(myCase.value.stringImpl());
+                RELEASE_ASSERT(iter != unlinkedTable.m_offsetTable.end());
+                linkedTable.m_ctiOffsets[iter->value.m_indexInTable] = linkBuffer.locationOf<JSSwitchPtrTag>(m_blockHeads[myCase.target.block->index]);
+            }
+            break;
+        }
+
+        case SwitchCell:
+            RELEASE_ASSERT_NOT_REACHED();
+            break;
         }
     }
 
@@ -246,54 +264,69 @@ void JITCompiler::link(LinkBuffer& linkBuffer)
     finalizeInlineCaches(m_getByIdsWithThis, linkBuffer);
     finalizeInlineCaches(m_getByVals, linkBuffer);
     finalizeInlineCaches(m_putByIds, linkBuffer);
+    finalizeInlineCaches(m_putByVals, linkBuffer);
+    finalizeInlineCaches(m_delByIds, linkBuffer);
+    finalizeInlineCaches(m_delByVals, linkBuffer);
     finalizeInlineCaches(m_inByIds, linkBuffer);
+    finalizeInlineCaches(m_inByVals, linkBuffer);
     finalizeInlineCaches(m_instanceOfs, linkBuffer);
+    finalizeInlineCaches(m_privateBrandAccesses, linkBuffer);
+
+    if (m_graph.m_plan.isUnlinked()) {
+        m_jitCode->m_unlinkedStubInfos = FixedVector<UnlinkedStructureStubInfo>(m_unlinkedStubInfos.size());
+        if (m_jitCode->m_unlinkedStubInfos.size())
+            std::move(m_unlinkedStubInfos.begin(), m_unlinkedStubInfos.end(), m_jitCode->m_unlinkedStubInfos.begin());
+        ASSERT(m_jitCode->common.m_stubInfos.isEmpty());
+    }
 
-    auto linkCallThunk = FunctionPtr<NoPtrTag>(vm().getCTIStub(linkCallThunkGenerator).retaggedCode<NoPtrTag>());
     for (auto& record : m_jsCalls) {
-        CallLinkInfo& info = *record.info;
-        linkBuffer.link(record.slowCall, linkCallThunk);
-        info.setCallLocations(
-            CodeLocationLabel<JSInternalPtrTag>(linkBuffer.locationOfNearCall<JSInternalPtrTag>(record.slowCall)),
-            CodeLocationLabel<JSInternalPtrTag>(linkBuffer.locationOf<JSInternalPtrTag>(record.targetToCheck)),
-            linkBuffer.locationOfNearCall<JSInternalPtrTag>(record.fastCall));
+        auto& info = *record.info;
+        info.setCodeLocations(
+            linkBuffer.locationOf<JSInternalPtrTag>(record.slowPathStart),
+            linkBuffer.locationOf<JSInternalPtrTag>(record.doneLocation));
     }
     
-    for (JSDirectCallRecord& record : m_jsDirectCalls) {
-        CallLinkInfo& info = *record.info;
-        linkBuffer.link(record.call, linkBuffer.locationOf<NoPtrTag>(record.slowPath));
-        info.setCallLocations(
-            CodeLocationLabel<JSInternalPtrTag>(),
+    for (auto& record : m_jsDirectCalls) {
+        auto& info = *record.info;
+        info.setCodeLocations(
             linkBuffer.locationOf<JSInternalPtrTag>(record.slowPath),
-            linkBuffer.locationOfNearCall<JSInternalPtrTag>(record.call));
+            CodeLocationLabel<JSInternalPtrTag>());
     }
-    
-    for (JSDirectTailCallRecord& record : m_jsDirectTailCalls) {
-        CallLinkInfo& info = *record.info;
-        info.setCallLocations(
-            linkBuffer.locationOf<JSInternalPtrTag>(record.patchableJump),
-            linkBuffer.locationOf<JSInternalPtrTag>(record.slowPath),
-            linkBuffer.locationOfNearCall<JSInternalPtrTag>(record.call));
-    }
-    
-    MacroAssemblerCodeRef<JITThunkPtrTag> osrExitThunk = vm().getCTIStub(osrExitGenerationThunkGenerator);
-    auto target = CodeLocationLabel<JITThunkPtrTag>(osrExitThunk.code());
-    for (unsigned i = 0; i < m_jitCode->osrExit.size(); ++i) {
-        OSRExitCompilationInfo& info = m_exitCompilationInfo[i];
-        if (!Options::useProbeOSRExit()) {
+
+    if (!m_exceptionChecks.empty())
+        linkBuffer.link(m_exceptionChecks, CodeLocationLabel(vm().getCTIStub(handleExceptionGenerator).retaggedCode<NoPtrTag>()));
+    if (!m_exceptionChecksWithCallFrameRollback.empty())
+        linkBuffer.link(m_exceptionChecksWithCallFrameRollback, CodeLocationLabel(vm().getCTIStub(handleExceptionWithCallFrameRollbackGenerator).retaggedCode<NoPtrTag>()));
+
+    if (!m_graph.m_plan.isUnlinked()) {
+        MacroAssemblerCodeRef<JITThunkPtrTag> osrExitThunk = vm().getCTIStub(osrExitGenerationThunkGenerator);
+        auto target = CodeLocationLabel<JITThunkPtrTag>(osrExitThunk.code());
+        Vector<JumpReplacement> jumpReplacements;
+        for (unsigned i = 0; i < m_osrExit.size(); ++i) {
+            OSRExitCompilationInfo& info = m_exitCompilationInfo[i];
             linkBuffer.link(info.m_patchableJump.m_jump, target);
-            OSRExit& exit = m_jitCode->osrExit[i];
+            OSRExit& exit = m_osrExit[i];
             exit.m_patchableJumpLocation = linkBuffer.locationOf<JSInternalPtrTag>(info.m_patchableJump);
+            if (info.m_replacementSource.isSet()) {
+                jumpReplacements.append(JumpReplacement(
+                    linkBuffer.locationOf<JSInternalPtrTag>(info.m_replacementSource),
+                    linkBuffer.locationOf<OSRExitPtrTag>(info.m_replacementDestination)));
+            }
         }
-        if (info.m_replacementSource.isSet()) {
-            m_jitCode->common.jumpReplacements.append(JumpReplacement(
-                linkBuffer.locationOf<JSInternalPtrTag>(info.m_replacementSource),
-                linkBuffer.locationOf<OSRExitPtrTag>(info.m_replacementDestination)));
-        }
+        m_jitCode->common.m_jumpReplacements = WTFMove(jumpReplacements);
     }
+
+#if ASSERT_ENABLED
+    for (auto& info : m_exitCompilationInfo) {
+        if (info.m_replacementSource.isSet())
+            ASSERT(!m_graph.m_plan.isUnlinked());
+    }
+    if (m_graph.m_plan.isUnlinked())
+        ASSERT(m_jitCode->common.m_jumpReplacements.isEmpty());
+#endif
     
     if (UNLIKELY(m_graph.compilation())) {
-        ASSERT(m_exitSiteLabels.size() == m_jitCode->osrExit.size());
+        ASSERT(m_exitSiteLabels.size() == m_osrExit.size());
         for (unsigned i = 0; i < m_exitSiteLabels.size(); ++i) {
             Vector<Label>& labels = m_exitSiteLabels[i];
             Vector<MacroAssemblerCodePtr<JSInternalPtrTag>> addresses;
@@ -305,6 +338,8 @@ void JITCompiler::link(LinkBuffer& linkBuffer)
         ASSERT(!m_exitSiteLabels.size());
 
     m_jitCode->common.compilation = m_graph.compilation();
+    m_jitCode->m_osrExit = WTFMove(m_osrExit);
+    m_jitCode->m_speculationRecovery = WTFMove(m_speculationRecovery);
     
     // Link new DFG exception handlers and remove baseline JIT handlers.
     m_codeBlock->clearExceptionHandlers();
@@ -326,7 +361,9 @@ void JITCompiler::link(LinkBuffer& linkBuffer)
     }
 
     if (m_pcToCodeOriginMapBuilder.didBuildMapping())
-        m_codeBlock->setPCToCodeOriginMap(makeUnique<PCToCodeOriginMap>(WTFMove(m_pcToCodeOriginMapBuilder), linkBuffer));
+        m_jitCode->common.m_pcToCodeOriginMap = makeUnique<PCToCodeOriginMap>(WTFMove(m_pcToCodeOriginMapBuilder), linkBuffer);
+
+    m_jitCode->m_linkerIR = LinkerIR(WTFMove(m_constantPool));
 }
 
 static void emitStackOverflowCheck(JITCompiler& jit, MacroAssembler::JumpList& stackOverflow)
@@ -370,20 +407,20 @@ void JITCompiler::compile()
     if (maxFrameExtentForSlowPathCall)
         addPtr(TrustedImm32(-static_cast<int32_t>(maxFrameExtentForSlowPathCall)), stackPointerRegister);
 
-    m_speculative->callOperationWithCallFrameRollbackOnException(operationThrowStackOverflowError, m_codeBlock);
+    emitGetFromCallFrameHeaderPtr(CallFrameSlot::codeBlock, GPRInfo::argumentGPR0);
+    m_speculative->callOperationWithCallFrameRollbackOnException(operationThrowStackOverflowError, GPRInfo::argumentGPR0);
 
     // Generate slow path code.
     m_speculative->runSlowPathGenerators(m_pcToCodeOriginMapBuilder);
     m_pcToCodeOriginMapBuilder.appendItem(labelIgnoringWatchpoints(), PCToCodeOriginMapBuilder::defaultCodeOrigin());
     
-    compileExceptionHandlers();
     linkOSRExits();
     
     // Create OSR entry trampolines if necessary.
     m_speculative->createOSREntries();
     setEndOfCode();
 
-    auto linkBuffer = makeUnique<LinkBuffer>(*this, m_codeBlock, JITCompilationCanFail);
+    auto linkBuffer = makeUnique<LinkBuffer>(*this, m_codeBlock, LinkBuffer::Profile::DFG, JITCompilationCanFail);
     if (linkBuffer->didFailToAllocate()) {
         m_graph.m_plan.setFinalizer(makeUnique<FailedFinalizer>(m_graph.m_plan));
         return;
@@ -394,8 +431,12 @@ void JITCompiler::compile()
 
     disassemble(*linkBuffer);
 
-    m_graph.m_plan.setFinalizer(makeUnique<JITFinalizer>(
-        m_graph.m_plan, m_jitCode.releaseNonNull(), WTFMove(linkBuffer)));
+    auto codeRef = FINALIZE_DFG_CODE(*linkBuffer, JSEntryPtrTag, "DFG JIT code for %s", toCString(CodeBlockWithJITType(m_codeBlock, JITType::DFGJIT)).data());
+    m_jitCode->initializeCodeRefForDFG(codeRef, codeRef.code());
+    m_jitCode->variableEventStream = m_speculative->finalizeEventStream();
+
+    auto finalizer = makeUnique<JITFinalizer>(m_graph.m_plan, m_jitCode.releaseNonNull(), WTFMove(linkBuffer));
+    m_graph.m_plan.setFinalizer(WTFMove(finalizer));
 }
 
 void JITCompiler::compileFunction()
@@ -441,7 +482,8 @@ void JITCompiler::compileFunction()
     if (maxFrameExtentForSlowPathCall)
         addPtr(TrustedImm32(-static_cast<int32_t>(maxFrameExtentForSlowPathCall)), stackPointerRegister);
 
-    m_speculative->callOperationWithCallFrameRollbackOnException(operationThrowStackOverflowError, m_codeBlock);
+    emitGetFromCallFrameHeaderPtr(CallFrameSlot::codeBlock, GPRInfo::argumentGPR0);
+    m_speculative->callOperationWithCallFrameRollbackOnException(operationThrowStackOverflowError, GPRInfo::argumentGPR0);
     
     // The fast entry point into a function does not check the correct number of arguments
     // have been passed to the call (we only use the fast entry point where we can statically
@@ -460,7 +502,9 @@ void JITCompiler::compileFunction()
         emitStoreCodeOrigin(CodeOrigin(BytecodeIndex(0)));
         if (maxFrameExtentForSlowPathCall)
             addPtr(TrustedImm32(-static_cast<int32_t>(maxFrameExtentForSlowPathCall)), stackPointerRegister);
-        m_speculative->callOperationWithCallFrameRollbackOnException(m_codeBlock->isConstructor() ? operationConstructArityCheck : operationCallArityCheck, GPRInfo::regT0, m_codeBlock->globalObject());
+        emitGetFromCallFrameHeaderPtr(CallFrameSlot::codeBlock, GPRInfo::argumentGPR0);
+        loadPtr(Address(GPRInfo::argumentGPR0, CodeBlock::offsetOfGlobalObject()), GPRInfo::argumentGPR0);
+        m_speculative->callOperationWithCallFrameRollbackOnException(m_codeBlock->isConstructor() ? operationConstructArityCheck : operationCallArityCheck, GPRInfo::returnValueGPR, GPRInfo::argumentGPR0);
         if (maxFrameExtentForSlowPathCall)
             addPtr(TrustedImm32(maxFrameExtentForSlowPathCall), stackPointerRegister);
         branchTest32(Zero, GPRInfo::returnValueGPR).linkTo(fromArityCheck, this);
@@ -475,7 +519,6 @@ void JITCompiler::compileFunction()
     m_speculative->runSlowPathGenerators(m_pcToCodeOriginMapBuilder);
     m_pcToCodeOriginMapBuilder.appendItem(labelIgnoringWatchpoints(), PCToCodeOriginMapBuilder::defaultCodeOrigin());
     
-    compileExceptionHandlers();
     linkOSRExits();
     
     // Create OSR entry trampolines if necessary.
@@ -483,7 +526,7 @@ void JITCompiler::compileFunction()
     setEndOfCode();
 
     // === Link ===
-    auto linkBuffer = makeUnique<LinkBuffer>(*this, m_codeBlock, JITCompilationCanFail);
+    auto linkBuffer = makeUnique<LinkBuffer>(*this, m_codeBlock, LinkBuffer::Profile::DFG, JITCompilationCanFail);
     if (linkBuffer->didFailToAllocate()) {
         m_graph.m_plan.setFinalizer(makeUnique<FailedFinalizer>(m_graph.m_plan));
         return;
@@ -498,8 +541,13 @@ void JITCompiler::compileFunction()
 
     MacroAssemblerCodePtr<JSEntryPtrTag> withArityCheck = linkBuffer->locationOf<JSEntryPtrTag>(arityCheck);
 
-    m_graph.m_plan.setFinalizer(makeUnique<JITFinalizer>(
-        m_graph.m_plan, m_jitCode.releaseNonNull(), WTFMove(linkBuffer), withArityCheck));
+    m_jitCode->initializeCodeRefForDFG(
+        FINALIZE_DFG_CODE(*linkBuffer, JSEntryPtrTag, "DFG JIT code for %s", toCString(CodeBlockWithJITType(m_codeBlock, JITType::DFGJIT)).data()),
+        withArityCheck);
+    m_jitCode->variableEventStream = m_speculative->finalizeEventStream();
+
+    auto finalizer = makeUnique<JITFinalizer>(m_graph.m_plan, m_jitCode.releaseNonNull(), WTFMove(linkBuffer), withArityCheck);
+    m_graph.m_plan.setFinalizer(WTFMove(finalizer));
 }
 
 void JITCompiler::disassemble(LinkBuffer& linkBuffer)
@@ -518,17 +566,11 @@ void* JITCompiler::addressOfDoubleConstant(Node* node)
 {
     double value = node->asNumber();
     int64_t valueBits = bitwise_cast<int64_t>(value);
-    auto it = m_graph.m_doubleConstantsMap.find(valueBits);
-    if (it != m_graph.m_doubleConstantsMap.end())
-        return it->second;
-
-    if (!m_graph.m_doubleConstants)
-        m_graph.m_doubleConstants = makeUnique<Bag<double>>();
-
-    double* addressInConstantPool = m_graph.m_doubleConstants->add();
-    *addressInConstantPool = value;
-    m_graph.m_doubleConstantsMap[valueBits] = addressInConstantPool;
-    return addressInConstantPool;
+    return m_graph.m_doubleConstantsMap.ensure(valueBits, [&]{
+        double* addressInConstantPool = m_graph.m_doubleConstants.add();
+        *addressInConstantPool = value;
+        return addressInConstantPool;
+    }).iterator->value;
 }
 #endif
 
@@ -536,7 +578,7 @@ void JITCompiler::noticeCatchEntrypoint(BasicBlock& basicBlock, JITCompiler::Lab
 {
     RELEASE_ASSERT(basicBlock.isCatchEntrypoint);
     RELEASE_ASSERT(basicBlock.intersectionOfCFAHasVisited); // An entrypoint is reachable by definition.
-    m_jitCode->common.appendCatchEntrypoint(basicBlock.bytecodeBegin, linkBuffer.locationOf<ExceptionHandlerPtrTag>(blockHead), WTFMove(argumentFormats));
+    m_graph.appendCatchEntrypoint(basicBlock.bytecodeBegin, linkBuffer.locationOf<ExceptionHandlerPtrTag>(blockHead), WTFMove(argumentFormats));
 }
 
 void JITCompiler::noticeOSREntry(BasicBlock& basicBlock, JITCompiler::Label blockHead, LinkBuffer& linkBuffer)
@@ -547,32 +589,35 @@ void JITCompiler::noticeOSREntry(BasicBlock& basicBlock, JITCompiler::Label bloc
     if (!basicBlock.intersectionOfCFAHasVisited)
         return;
 
-    OSREntryData* entry = m_jitCode->appendOSREntryData(basicBlock.bytecodeBegin, linkBuffer.locationOf<OSREntryPtrTag>(blockHead));
+    OSREntryData entry;
+    entry.m_bytecodeIndex = basicBlock.bytecodeBegin;
+    entry.m_machineCode = linkBuffer.locationOf<OSREntryPtrTag>(blockHead);
+
+    FixedOperands<AbstractValue> expectedValues(basicBlock.intersectionOfPastValuesAtHead);
+    Vector<OSREntryReshuffling> reshufflings;
 
-    entry->m_expectedValues = basicBlock.intersectionOfPastValuesAtHead;
-        
     // Fix the expected values: in our protocol, a dead variable will have an expected
     // value of (None, []). But the old JIT may stash some values there. So we really
     // need (Top, TOP).
     for (size_t argument = 0; argument < basicBlock.variablesAtHead.numberOfArguments(); ++argument) {
         Node* node = basicBlock.variablesAtHead.argument(argument);
         if (!node || !node->shouldGenerate())
-            entry->m_expectedValues.argument(argument).makeBytecodeTop();
+            expectedValues.argument(argument).makeBytecodeTop();
     }
     for (size_t local = 0; local < basicBlock.variablesAtHead.numberOfLocals(); ++local) {
         Node* node = basicBlock.variablesAtHead.local(local);
         if (!node || !node->shouldGenerate())
-            entry->m_expectedValues.local(local).makeBytecodeTop();
+            expectedValues.local(local).makeBytecodeTop();
         else {
             VariableAccessData* variable = node->variableAccessData();
-            entry->m_machineStackUsed.set(variable->machineLocal().toLocal());
+            entry.m_machineStackUsed.set(variable->machineLocal().toLocal());
                 
             switch (variable->flushFormat()) {
             case FlushedDouble:
-                entry->m_localsForcedDouble.set(local);
+                entry.m_localsForcedDouble.set(local);
                 break;
             case FlushedInt52:
-                entry->m_localsForcedAnyInt.set(local);
+                entry.m_localsForcedAnyInt.set(local);
                 break;
             default:
                 break;
@@ -580,14 +625,16 @@ void JITCompiler::noticeOSREntry(BasicBlock& basicBlock, JITCompiler::Label bloc
 
             ASSERT(!variable->operand().isTmp());
             if (variable->operand().virtualRegister() != variable->machineLocal()) {
-                entry->m_reshufflings.append(
+                reshufflings.append(
                     OSREntryReshuffling(
                         variable->operand().virtualRegister().offset(), variable->machineLocal().offset()));
             }
         }
     }
         
-    entry->m_reshufflings.shrinkToFit();
+    entry.m_expectedValues = WTFMove(expectedValues);
+    entry.m_reshufflings = WTFMove(reshufflings);
+    m_osrEntry.append(WTFMove(entry));
 }
 
 void JITCompiler::appendExceptionHandlingOSRExit(ExitKind kind, unsigned eventStreamIndex, CodeOrigin opCatchOrigin, HandlerInfo* exceptionHandler, CallSiteIndex callSite, MacroAssembler::JumpList jumpsToFail)
@@ -596,7 +643,7 @@ void JITCompiler::appendExceptionHandlingOSRExit(ExitKind kind, unsigned eventSt
     exit.m_codeOrigin = opCatchOrigin;
     exit.m_exceptionHandlerCallSiteIndex = callSite;
     OSRExitCompilationInfo& exitInfo = appendExitInfo(jumpsToFail);
-    jitCode()->appendOSRExit(exit);
+    m_osrExit.append(WTFMove(exit));
     m_exceptionHandlerOSRExitCallSites.append(ExceptionHandlingOSRExitInfo { exitInfo, *exceptionHandler, callSite });
 }
 
@@ -626,10 +673,10 @@ void JITCompiler::exceptionCheck()
     HandlerInfo* exceptionHandler;
     bool willCatchException = m_graph.willCatchExceptionInMachineFrame(m_speculative->m_currentNode->origin.forExit, opCatchOrigin, exceptionHandler); 
     if (willCatchException) {
-        unsigned streamIndex = m_speculative->m_outOfLineStreamIndex ? *m_speculative->m_outOfLineStreamIndex : m_speculative->m_stream->size();
+        unsigned streamIndex = m_speculative->m_outOfLineStreamIndex ? *m_speculative->m_outOfLineStreamIndex : m_speculative->m_stream.size();
         MacroAssembler::Jump hadException = emitNonPatchableExceptionCheck(vm());
         // We assume here that this is called after callOpeartion()/appendCall() is called.
-        appendExceptionHandlingOSRExit(ExceptionCheck, streamIndex, opCatchOrigin, exceptionHandler, m_jitCode->common.lastCallSite(), hadException);
+        appendExceptionHandlingOSRExit(ExceptionCheck, streamIndex, opCatchOrigin, exceptionHandler, m_jitCode->common.codeOrigins->lastCallSite(), hadException);
     } else
         m_exceptionChecks.append(emitExceptionCheck(vm()));
 }
@@ -669,6 +716,89 @@ void JITCompiler::makeCatchOSREntryBuffer()
     }
 }
 
+void JITCompiler::loadConstant(LinkerIR::Constant index, GPRReg dest)
+{
+#if USE(JSVALUE64)
+    loadPtr(Address(GPRInfo::constantsRegister, JITData::offsetOfData() + sizeof(void*) * index), dest);
+#else
+    UNUSED_PARAM(index);
+    UNUSED_PARAM(dest);
+    RELEASE_ASSERT_NOT_REACHED();
+#endif
+}
+
+void JITCompiler::loadLinkableConstant(LinkableConstant constant, GPRReg dest)
+{
+    constant.materialize(*this, dest);
+}
+
+void JITCompiler::storeLinkableConstant(LinkableConstant constant, Address dest)
+{
+    constant.store(*this, dest);
+}
+
+JITCompiler::LinkableConstant::LinkableConstant(JITCompiler& jit, JSCell* cell)
+{
+    jit.m_graph.m_plan.weakReferences().addLazily(cell);
+    if (jit.m_graph.m_plan.isUnlinked()) {
+        m_index = jit.addToConstantPool(LinkerIR::Type::CellPointer, cell);
+        return;
+    }
+    m_pointer = cell;
+}
+
+JITCompiler::LinkableConstant::LinkableConstant(JITCompiler& jit, void* pointer, NonCellTag)
+{
+    if (jit.m_graph.m_plan.isUnlinked()) {
+        m_index = jit.addToConstantPool(LinkerIR::Type::NonCellPointer, pointer);
+        return;
+    }
+    m_pointer = pointer;
+}
+
+void JITCompiler::LinkableConstant::materialize(CCallHelpers& jit, GPRReg resultGPR)
+{
+#if USE(JSVALUE64)
+    if (isUnlinked()) {
+        jit.loadPtr(unlinkedAddress(), resultGPR);
+        return;
+    }
+#endif
+    jit.move(TrustedImmPtr(m_pointer), resultGPR);
+}
+
+void JITCompiler::LinkableConstant::store(CCallHelpers& jit, CCallHelpers::Address address)
+{
+#if USE(JSVALUE64)
+    if (isUnlinked()) {
+        jit.transferPtr(unlinkedAddress(), address);
+        return;
+    }
+#endif
+    jit.storePtr(TrustedImmPtr(m_pointer), address);
+}
+
+LinkerIR::Constant JITCompiler::addToConstantPool(LinkerIR::Type type, void* payload)
+{
+    LinkerIR::Value value { payload, type };
+    auto result = m_constantPoolMap.add(value, m_constantPoolMap.size());
+    if (result.isNewEntry)
+        m_constantPool.append(value);
+    return result.iterator->value;
+}
+
+std::tuple<CompileTimeStructureStubInfo, JITCompiler::LinkableConstant> JITCompiler::addStructureStubInfo()
+{
+    if (m_graph.m_plan.isUnlinked()) {
+        void* unlinkedStubInfoIndex = bitwise_cast<void*>(static_cast<uintptr_t>(m_unlinkedStubInfos.size()));
+        UnlinkedStructureStubInfo* stubInfo = &m_unlinkedStubInfos.alloc();
+        LinkerIR::Constant stubInfoIndex = addToConstantPool(LinkerIR::Type::StructureStubInfo, unlinkedStubInfoIndex);
+        return std::tuple { stubInfo, LinkableConstant(stubInfoIndex) };
+    }
+    StructureStubInfo* stubInfo = jitCode()->common.m_stubInfos.add();
+    return std::tuple { stubInfo, LinkableConstant() };
+}
+
 } } // namespace JSC::DFG
 
 #endif // ENABLE(DFG_JIT)
