@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2019 Apple Inc. All rights reserved.
+ * Copyright (C) 2011-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,28 +28,27 @@
 
 #if ENABLE(DFG_JIT)
 
-#include "AssemblyHelpers.h"
-#include "BytecodeUseDef.h"
+#include "AssemblyHelpersSpoolers.h"
+#include "BytecodeStructs.h"
 #include "CheckpointOSRExitSideState.h"
-#include "ClonedArguments.h"
 #include "DFGGraph.h"
 #include "DFGMayExit.h"
 #include "DFGOSRExitCompilerCommon.h"
 #include "DFGOperations.h"
 #include "DFGSpeculativeJIT.h"
-#include "DirectArguments.h"
 #include "FrameTracers.h"
 #include "InlineCallFrame.h"
-#include "JSCInlines.h"
-#include "JSCJSValue.h"
+#include "JSCJSValueInlines.h"
 #include "OperandsInlines.h"
 #include "ProbeContext.h"
-#include "ProbeFrame.h"
+#include "VMInlines.h"
+
+#include <wtf/Scope.h>
 
 namespace JSC { namespace DFG {
 
 OSRExit::OSRExit(ExitKind kind, JSValueSource jsValueSource, MethodOfGettingAValueProfile valueProfile, SpeculativeJIT* jit, unsigned streamIndex, unsigned recoveryIndex)
-    : OSRExitBase(kind, jit->m_origin.forExit, jit->m_origin.semantic, jit->m_origin.wasHoisted)
+    : OSRExitBase(kind, jit->m_origin.forExit, jit->m_origin.semantic, jit->m_origin.wasHoisted, jit->m_currentNode ? jit->m_currentNode->index() : 0)
     , m_jsValueSource(jsValueSource)
     , m_valueProfile(valueProfile)
     , m_recoveryIndex(recoveryIndex)
@@ -57,15 +56,10 @@ OSRExit::OSRExit(ExitKind kind, JSValueSource jsValueSource, MethodOfGettingAVal
 {
     bool canExit = jit->m_origin.exitOK;
     if (!canExit && jit->m_currentNode) {
-        ExitMode exitMode = mayExit(jit->m_jit.graph(), jit->m_currentNode);
+        ExitMode exitMode = mayExit(jit->m_graph, jit->m_currentNode);
         canExit = exitMode == ExitMode::Exits || exitMode == ExitMode::ExitsForExceptions;
     }
-    DFG_ASSERT(jit->m_jit.graph(), jit->m_currentNode, canExit);
-}
-
-CodeLocationJump<JSInternalPtrTag> OSRExit::codeLocationForRepatch() const
-{
-    return CodeLocationJump<JSInternalPtrTag>(m_patchableJumpLocation);
+    DFG_ASSERT(jit->m_graph, jit->m_currentNode, canExit);
 }
 
 void OSRExit::emitRestoreArguments(CCallHelpers& jit, VM& vm, const Operands<ValueRecovery>& operands)
@@ -126,10 +120,10 @@ void OSRExit::emitRestoreArguments(CCallHelpers& jit, VM& vm, const Operands<Val
         jit.prepareCallOperation(vm);
         switch (recovery.technique()) {
         case DirectArgumentsThatWereNotCreated:
-            jit.move(AssemblyHelpers::TrustedImmPtr(tagCFunctionPtr<OperationPtrTag>(operationCreateDirectArgumentsDuringExit)), GPRInfo::nonArgGPR0);
+            jit.move(AssemblyHelpers::TrustedImmPtr(tagCFunction<OperationPtrTag>(operationCreateDirectArgumentsDuringExit)), GPRInfo::nonArgGPR0);
             break;
         case ClonedArgumentsThatWereNotCreated:
-            jit.move(AssemblyHelpers::TrustedImmPtr(tagCFunctionPtr<OperationPtrTag>(operationCreateClonedArgumentsDuringExit)), GPRInfo::nonArgGPR0);
+            jit.move(AssemblyHelpers::TrustedImmPtr(tagCFunction<OperationPtrTag>(operationCreateClonedArgumentsDuringExit)), GPRInfo::nonArgGPR0);
             break;
         default:
             RELEASE_ASSERT_NOT_REACHED();
@@ -142,15 +136,16 @@ void OSRExit::emitRestoreArguments(CCallHelpers& jit, VM& vm, const Operands<Val
     }
 }
 
-void JIT_OPERATION operationCompileOSRExit(CallFrame* callFrame)
+JSC_DEFINE_JIT_OPERATION(operationCompileOSRExit, void, (CallFrame* callFrame, void* bufferToPreserve))
 {
     VM& vm = callFrame->deprecatedVM();
     auto scope = DECLARE_THROW_SCOPE(vm);
+    ActiveScratchBufferScope activeScratchBufferScope(ScratchBuffer::fromData(bufferToPreserve), GPRInfo::numberOfRegisters + FPRInfo::numberOfRegisters);
 
-    if (validateDFGDoesGC) {
+    if constexpr (validateDFGDoesGC) {
         // We're about to exit optimized code. So, there's no longer any optimized
         // code running that expects no GC.
-        vm.heap.setExpectDoesGC(true);
+        vm.setDoesGCExpectation(true, DoesGCCheck::Special::DFGOSRExit);
     }
 
     if (vm.callFrameForCatch)
@@ -162,10 +157,10 @@ void JIT_OPERATION operationCompileOSRExit(CallFrame* callFrame)
 
     // It's sort of preferable that we don't GC while in here. Anyways, doing so wouldn't
     // really be profitable.
-    DeferGCForAWhile deferGC(vm.heap);
+    DeferGCForAWhile deferGC(vm);
 
     uint32_t exitIndex = vm.osrExitIndex;
-    OSRExit& exit = codeBlock->jitCode()->dfg()->osrExit[exitIndex];
+    OSRExit& exit = codeBlock->jitCode()->dfg()->m_osrExit[exitIndex];
 
     ASSERT(!vm.callFrameForCatch || exit.m_kind == GenericUnwind);
     EXCEPTION_ASSERT_UNUSED(scope, !!scope.exception() || !exit.isExceptionHandler());
@@ -174,10 +169,11 @@ void JIT_OPERATION operationCompileOSRExit(CallFrame* callFrame)
     Operands<ValueRecovery> operands;
     codeBlock->jitCode()->dfg()->variableEventStream.reconstruct(codeBlock, exit.m_codeOrigin, codeBlock->jitCode()->dfg()->minifiedDFG, exit.m_streamIndex, operands);
 
-    SpeculationRecovery* recovery = 0;
+    SpeculationRecovery* recovery = nullptr;
     if (exit.m_recoveryIndex != UINT_MAX)
-        recovery = &codeBlock->jitCode()->dfg()->speculationRecovery[exit.m_recoveryIndex];
+        recovery = &codeBlock->jitCode()->dfg()->m_speculationRecovery[exit.m_recoveryIndex];
 
+    MacroAssemblerCodeRef<OSRExitPtrTag> exitCode;
     {
         CCallHelpers jit(codeBlock);
 
@@ -205,21 +201,67 @@ void JIT_OPERATION operationCompileOSRExit(CallFrame* callFrame)
 
         OSRExit::compileExit(jit, vm, exit, operands, recovery);
 
-        LinkBuffer patchBuffer(jit, codeBlock);
-        exit.m_code = FINALIZE_CODE_IF(
+        LinkBuffer patchBuffer(jit, codeBlock, LinkBuffer::Profile::DFGOSRExit);
+        exitCode = FINALIZE_CODE_IF(
             shouldDumpDisassembly() || Options::verboseOSR() || Options::verboseDFGOSRExit(),
             patchBuffer, OSRExitPtrTag,
-            "DFG OSR exit #%u (%s, %s) from %s, with operands = %s",
-                exitIndex, toCString(exit.m_codeOrigin).data(),
+            "DFG OSR exit #%u (D@%u, %s, %s) from %s, with operands = %s",
+                exitIndex, exit.m_dfgNodeIndex, toCString(exit.m_codeOrigin).data(),
                 exitKindToString(exit.m_kind), toCString(*codeBlock).data(),
                 toCString(ignoringContext<DumpContext>(operands)).data());
+        codeBlock->dfgJITData()->setExitCode(exitIndex, exitCode);
     }
 
-    MacroAssembler::repatchJump(exit.codeLocationForRepatch(), CodeLocationLabel<OSRExitPtrTag>(exit.m_code.code()));
+    if (exit.codeLocationForRepatch())
+        MacroAssembler::repatchJump(exit.codeLocationForRepatch(), CodeLocationLabel<OSRExitPtrTag>(exitCode.code()));
 
-    vm.osrExitJumpDestination = exit.m_code.code().executableAddress();
+    vm.osrExitJumpDestination = exitCode.code().executableAddress();
 }
 
+IGNORE_WARNINGS_BEGIN("frame-address")
+
+JSC_DEFINE_JIT_OPERATION(operationMaterializeOSRExitSideState, void, (VM* vmPointer, const OSRExitBase* exitPointer, EncodedJSValue* tmpScratch))
+{
+    const OSRExitBase& exit = *exitPointer;
+    VM& vm = *vmPointer;
+    CallFrame* callFrame = DECLARE_CALL_FRAME(vm);
+
+    Vector<std::unique_ptr<CheckpointOSRExitSideState>, VM::expectedMaxActiveSideStateCount> sideStates;
+    sideStates.reserveInitialCapacity(exit.m_codeOrigin.inlineDepth());
+    auto sideStateCommitter = makeScopeExit([&] {
+        for (size_t i = sideStates.size(); i--;)
+            vm.pushCheckpointOSRSideState(WTFMove(sideStates[i]));
+    });
+
+    auto addSideState = [&] (CallFrame* frame, BytecodeIndex index, size_t tmpOffset) {
+        std::unique_ptr<CheckpointOSRExitSideState> sideState = WTF::makeUnique<CheckpointOSRExitSideState>(frame);
+
+        sideState->bytecodeIndex = index;
+        for (size_t i = 0; i < maxNumCheckpointTmps; ++i)
+            sideState->tmps[i] = JSValue::decode(tmpScratch[i + tmpOffset]);
+
+        sideStates.append(WTFMove(sideState));
+    };
+
+    const CodeOrigin* codeOrigin;
+    for (codeOrigin = &exit.m_codeOrigin; codeOrigin && codeOrigin->inlineCallFrame(); codeOrigin = codeOrigin->inlineCallFrame()->getCallerSkippingTailCalls()) {
+        BytecodeIndex callBytecodeIndex = codeOrigin->bytecodeIndex();
+        if (!callBytecodeIndex.checkpoint())
+            continue;
+
+        auto* inlineCallFrame = codeOrigin->inlineCallFrame();
+        addSideState(reinterpret_cast_ptr<CallFrame*>(reinterpret_cast<char*>(callFrame) + inlineCallFrame->returnPCOffset() - sizeof(CPURegister)), callBytecodeIndex, inlineCallFrame->tmpOffset);
+    }
+
+    if (!codeOrigin)
+        return;
+
+    if (BytecodeIndex bytecodeIndex = codeOrigin->bytecodeIndex(); bytecodeIndex.checkpoint())
+        addSideState(callFrame, bytecodeIndex, 0);
+}
+
+IGNORE_WARNINGS_END
+
 void OSRExit::compileExit(CCallHelpers& jit, VM& vm, const OSRExit& exit, const Operands<ValueRecovery>& operands, SpeculationRecovery* recovery)
 {
     jit.jitAssertTagsInPlace();
@@ -289,7 +331,7 @@ void OSRExit::compileExit(CCallHelpers& jit, VM& vm, const OSRExit& exit, const
             CodeOrigin codeOrigin = exit.m_codeOriginForExitProfile;
             CodeBlock* codeBlock = jit.baselineCodeBlockFor(codeOrigin);
             if (ArrayProfile* arrayProfile = codeBlock->getArrayProfile(codeOrigin.bytecodeIndex())) {
-                const Instruction* instruction = codeBlock->instructions().at(codeOrigin.bytecodeIndex()).ptr();
+                const auto* instruction = codeBlock->instructions().at(codeOrigin.bytecodeIndex()).ptr();
                 CCallHelpers::Jump skipProfile;
                 if (instruction->is<OpGetById>()) {
                     auto& metadata = instruction->as<OpGetById>().metadata(codeBlock);
@@ -383,10 +425,14 @@ void OSRExit::compileExit(CCallHelpers& jit, VM& vm, const OSRExit& exit, const
                 // We can't be sure that we have a spare register. So use the numberTagRegister,
                 // since we know how to restore it.
                 jit.load64(AssemblyHelpers::Address(exit.m_jsValueSource.asAddress()), GPRInfo::numberTagRegister);
-                profile.emitReportValue(jit, JSValueRegs(GPRInfo::numberTagRegister));
-                jit.move(AssemblyHelpers::TrustedImm64(JSValue::NumberTag), GPRInfo::numberTagRegister);
-            } else
-                profile.emitReportValue(jit, JSValueRegs(exit.m_jsValueSource.gpr()));
+                // We also use the notCellMaskRegister as the scratch register, for the same reason.
+                // FIXME: find a less gross way of doing this, maybe through delaying these operations until we actually have some spare registers around?
+                profile.emitReportValue(jit, jit.codeBlock(), JSValueRegs(GPRInfo::numberTagRegister), GPRInfo::notCellMaskRegister, DoNotHaveTagRegisters);
+                jit.emitMaterializeTagCheckRegisters();
+            } else {
+                profile.emitReportValue(jit, jit.codeBlock(), JSValueRegs(exit.m_jsValueSource.gpr()), GPRInfo::notCellMaskRegister, DoNotHaveTagRegisters);
+                jit.move(AssemblyHelpers::TrustedImm64(JSValue::NotCellMask), GPRInfo::notCellMaskRegister);
+            }
 #else // not USE(JSVALUE64)
             if (exit.m_jsValueSource.isAddress()) {
                 // Save a register so we can use it.
@@ -398,7 +444,7 @@ void OSRExit::compileExit(CCallHelpers& jit, VM& vm, const OSRExit& exit, const
                 JSValueRegs scratch(scratchTag, scratchPayload);
                 
                 jit.loadValue(exit.m_jsValueSource.asAddress(), scratch);
-                profile.emitReportValue(jit, scratch);
+                profile.emitReportValue(jit, jit.codeBlock(), scratch, InvalidGPRReg);
                 
                 jit.popToRestore(scratchTag);
                 jit.popToRestore(scratchPayload);
@@ -407,10 +453,10 @@ void OSRExit::compileExit(CCallHelpers& jit, VM& vm, const OSRExit& exit, const
                 jit.pushToSave(scratchTag);
                 jit.move(AssemblyHelpers::TrustedImm32(exit.m_jsValueSource.tag()), scratchTag);
                 JSValueRegs value(scratchTag, exit.m_jsValueSource.payloadGPR());
-                profile.emitReportValue(jit, value);
+                profile.emitReportValue(jit, jit.codeBlock(), value, InvalidGPRReg);
                 jit.popToRestore(scratchTag);
             } else
-                profile.emitReportValue(jit, exit.m_jsValueSource.regs());
+                profile.emitReportValue(jit, jit.codeBlock(), exit.m_jsValueSource.regs(), InvalidGPRReg);
 #endif // USE(JSVALUE64)
         }
     }
@@ -454,7 +500,7 @@ void OSRExit::compileExit(CCallHelpers& jit, VM& vm, const OSRExit& exit, const
     // Save all state from GPRs into the scratch buffer.
 
     ScratchBuffer* scratchBuffer = vm.scratchBufferForSize(sizeof(EncodedJSValue) * operands.size());
-    EncodedJSValue* scratch = scratchBuffer ? static_cast<EncodedJSValue*>(scratchBuffer->dataBuffer()) : 0;
+    EncodedJSValue* scratch = scratchBuffer ? static_cast<EncodedJSValue*>(scratchBuffer->dataBuffer()) : nullptr;
 
     for (size_t index = 0; index < operands.size(); ++index) {
         const ValueRecovery& recovery = operands[index];
@@ -476,12 +522,7 @@ void OSRExit::compileExit(CCallHelpers& jit, VM& vm, const OSRExit& exit, const
             break;
             
         case InPair:
-            jit.store32(
-                recovery.tagGPR(),
-                &bitwise_cast<EncodedValueDescriptor*>(scratch + index)->asBits.tag);
-            jit.store32(
-                recovery.payloadGPR(),
-                &bitwise_cast<EncodedValueDescriptor*>(scratch + index)->asBits.payload);
+            jit.storeValue(recovery.jsValueRegs(), scratch + index);
             break;
 #endif
 
@@ -515,34 +556,147 @@ void OSRExit::compileExit(CCallHelpers& jit, VM& vm, const OSRExit& exit, const
     // do this even for state that's already in the right place on the stack.
     // It makes things simpler later.
 
+    bool inlineStackContainsActiveCheckpoint = exit.m_codeOrigin.inlineStackContainsActiveCheckpoint();
+    size_t firstTmpToRestoreEarly = operands.size() - operands.numberOfTmps();
+    if (!inlineStackContainsActiveCheckpoint)
+        firstTmpToRestoreEarly = operands.size(); // Don't eagerly restore.
+
+    // The tag registers are needed to materialize recoveries below.
+    jit.emitMaterializeTagCheckRegisters();
+
     for (size_t index = 0; index < operands.size(); ++index) {
         const ValueRecovery& recovery = operands[index];
 
-        switch (recovery.technique()) {
+        auto currentTechnique = recovery.technique();
+        switch (currentTechnique) {
         case DisplacedInJSStack:
+#if USE(JSVALUE64)
         case CellDisplacedInJSStack:
         case BooleanDisplacedInJSStack:
-        case Int32DisplacedInJSStack:
-        case DoubleDisplacedInJSStack:
-#if USE(JSVALUE64)
-        case Int52DisplacedInJSStack:
-        case StrictInt52DisplacedInJSStack:
-            jit.load64(AssemblyHelpers::addressFor(recovery.virtualRegister()), GPRInfo::regT0);
-            jit.store64(GPRInfo::regT0, scratch + index);
+#endif
+            jit.loadValue(AssemblyHelpers::addressFor(recovery.virtualRegister()), JSRInfo::jsRegT10);
+            jit.storeValue(JSRInfo::jsRegT10, scratch + index);
             break;
+
+        case Constant: {
+#if USE(JSVALUE64)
+            if (index >= firstTmpToRestoreEarly) {
+                ASSERT(operands.operandForIndex(index).isTmp());
+                jit.move(AssemblyHelpers::TrustedImm64(JSValue::encode(recovery.constant())), GPRInfo::regT0);
+                jit.store64(GPRInfo::regT0, scratch + index);
+            }
+#else // not USE(JSVALUE64)
+            UNUSED_VARIABLE(firstTmpToRestoreEarly);
+            jit.storeValue(recovery.constant(), scratch + index, JSRInfo::jsRegT10);
+#endif
+            break;
+        }
+
+        case UnboxedInt32InGPR:
+#if USE(JSVALUE64)
+            jit.load64(scratch + index, GPRInfo::regT0);
+            jit.zeroExtend32ToWord(GPRInfo::regT0, GPRInfo::regT0);
+            jit.or64(GPRInfo::numberTagRegister, GPRInfo::regT0);
+            jit.store64(GPRInfo::regT0, scratch + index);
+#else
+            jit.store32(
+                AssemblyHelpers::TrustedImm32(JSValue::Int32Tag),
+                &bitwise_cast<EncodedValueDescriptor*>(scratch + index)->asBits.tag);
+#endif
+            break;
+
+        case Int32DisplacedInJSStack:
+#if USE(JSVALUE64)
+            jit.load64(AssemblyHelpers::addressFor(recovery.virtualRegister()), GPRInfo::regT0);
+            jit.zeroExtend32ToWord(GPRInfo::regT0, GPRInfo::regT0);
+            jit.or64(GPRInfo::numberTagRegister, GPRInfo::regT0);
+            jit.store64(GPRInfo::regT0, scratch + index);
 #else
             jit.load32(
-                AssemblyHelpers::tagFor(recovery.virtualRegister()),
-                GPRInfo::regT0);
+                AssemblyHelpers::payloadFor(recovery.virtualRegister()),
+                JSRInfo::jsRegT10.payloadGPR());
+            jit.move(AssemblyHelpers::TrustedImm32(JSValue::Int32Tag), JSRInfo::jsRegT10.tagGPR());
+            jit.storeValue(JSRInfo::jsRegT10, scratch + index);
+#endif
+            break;
+
+#if USE(JSVALUE32_64)
+        case UnboxedBooleanInGPR:
+            jit.store32(
+                AssemblyHelpers::TrustedImm32(JSValue::BooleanTag),
+                &bitwise_cast<EncodedValueDescriptor*>(scratch + index)->asBits.tag);
+            break;
+
+        case BooleanDisplacedInJSStack:
             jit.load32(
                 AssemblyHelpers::payloadFor(recovery.virtualRegister()),
-                GPRInfo::regT1);
-            jit.store32(
-                GPRInfo::regT0,
+                JSRInfo::jsRegT10.payloadGPR());
+            jit.move(AssemblyHelpers::TrustedImm32(JSValue::BooleanTag), JSRInfo::jsRegT10.tagGPR());
+            jit.storeValue(JSRInfo::jsRegT10, scratch + index);
+            break;
+
+        case UnboxedCellInGPR:
+            jit.storeCell(
                 &bitwise_cast<EncodedValueDescriptor*>(scratch + index)->asBits.tag);
-            jit.store32(
-                GPRInfo::regT1,
-                &bitwise_cast<EncodedValueDescriptor*>(scratch + index)->asBits.payload);
+            break;
+
+        case CellDisplacedInJSStack:
+            jit.load32(
+                AssemblyHelpers::payloadFor(recovery.virtualRegister()),
+                JSRInfo::jsRegT10.payloadGPR());
+            jit.storeCell(JSRInfo::jsRegT10, scratch + index);
+            break;
+#endif
+
+        case UnboxedDoubleInFPR:
+            jit.move(AssemblyHelpers::TrustedImmPtr(scratch + index), GPRInfo::regT1);
+            jit.loadDouble(MacroAssembler::Address(GPRInfo::regT1), FPRInfo::fpRegT0);
+            jit.purifyNaN(FPRInfo::fpRegT0);
+#if USE(JSVALUE64)
+            jit.boxDouble(FPRInfo::fpRegT0, GPRInfo::regT0);
+            jit.store64(GPRInfo::regT0, MacroAssembler::Address(GPRInfo::regT1));
+#else
+            jit.storeDouble(FPRInfo::fpRegT0, MacroAssembler::Address(GPRInfo::regT1));
+#endif
+            break;
+
+        case DoubleDisplacedInJSStack:
+            jit.move(AssemblyHelpers::TrustedImmPtr(scratch + index), GPRInfo::regT1);
+            jit.loadDouble(AssemblyHelpers::addressFor(recovery.virtualRegister()), FPRInfo::fpRegT0);
+            jit.purifyNaN(FPRInfo::fpRegT0);
+#if USE(JSVALUE64)
+            jit.boxDouble(FPRInfo::fpRegT0, GPRInfo::regT0);
+            jit.store64(GPRInfo::regT0, MacroAssembler::Address(GPRInfo::regT1));
+#else
+            jit.storeDouble(FPRInfo::fpRegT0, MacroAssembler::Address(GPRInfo::regT1));
+#endif
+            break;
+
+#if USE(JSVALUE64)
+        case UnboxedInt52InGPR:
+            jit.load64(scratch + index, GPRInfo::regT0);
+            jit.rshift64(AssemblyHelpers::TrustedImm32(JSValue::int52ShiftAmount), GPRInfo::regT0);
+            jit.boxInt52(GPRInfo::regT0, GPRInfo::regT0, GPRInfo::regT1, FPRInfo::fpRegT0);
+            jit.store64(GPRInfo::regT0, scratch + index);
+            break;
+
+        case Int52DisplacedInJSStack:
+            jit.load64(AssemblyHelpers::addressFor(recovery.virtualRegister()), GPRInfo::regT0);
+            jit.rshift64(AssemblyHelpers::TrustedImm32(JSValue::int52ShiftAmount), GPRInfo::regT0);
+            jit.boxInt52(GPRInfo::regT0, GPRInfo::regT0, GPRInfo::regT1, FPRInfo::fpRegT0);
+            jit.store64(GPRInfo::regT0, scratch + index);
+            break;
+
+        case UnboxedStrictInt52InGPR:
+            jit.load64(scratch + index, GPRInfo::regT0);
+            jit.boxInt52(GPRInfo::regT0, GPRInfo::regT0, GPRInfo::regT1, FPRInfo::fpRegT0);
+            jit.store64(GPRInfo::regT0, scratch + index);
+            break;
+
+        case StrictInt52DisplacedInJSStack:
+            jit.load64(AssemblyHelpers::addressFor(recovery.virtualRegister()), GPRInfo::regT0);
+            jit.boxInt52(GPRInfo::regT0, GPRInfo::regT0, GPRInfo::regT1, FPRInfo::fpRegT0);
+            jit.store64(GPRInfo::regT0, scratch + index);
             break;
 #endif
 
@@ -551,18 +705,27 @@ void OSRExit::compileExit(CCallHelpers& jit, VM& vm, const OSRExit& exit, const
         }
     }
 
-    if (validateDFGDoesGC) {
-        // We're about to exit optimized code. So, there's no longer any optimized
-        // code running that expects no GC. We need to set this before arguments
-        // materialization below (see emitRestoreArguments()).
+    if constexpr (validateDFGDoesGC) {
+        if (Options::validateDoesGC()) {
+            // We're about to exit optimized code. So, there's no longer any optimized
+            // code running that expects no GC. We need to set this before arguments
+            // materialization below (see emitRestoreArguments()).
 
-        // Even though we set Heap::m_expectDoesGC in compileOSRExit(), we also need
-        // to set it here because compileOSRExit() is only called on the first time
-        // we exit from this site, but all subsequent exits will take this compiled
-        // ramp without calling compileOSRExit() first.
-        jit.store8(CCallHelpers::TrustedImm32(true), vm.heap.addressOfExpectDoesGC());
+            // Even though we set Heap::m_doesGC in compileOSRExit(), we also need
+            // to set it here because compileOSRExit() is only called on the first time
+            // we exit from this site, but all subsequent exits will take this compiled
+            // ramp without calling compileOSRExit() first.
+            DoesGCCheck check;
+            check.u.encoded = DoesGCCheck::encode(true, DoesGCCheck::Special::DFGOSRExit);
+#if USE(JSVALUE64)
+            jit.store64(CCallHelpers::TrustedImm64(check.u.encoded), vm.addressOfDoesGC());
+#else
+            jit.store32(CCallHelpers::TrustedImm32(check.u.other), &vm.addressOfDoesGC()->u.other);
+            jit.store32(CCallHelpers::TrustedImm32(check.u.nodeIndex), &vm.addressOfDoesGC()->u.nodeIndex);
+#endif
+        }
     }
-
+    
     // Need to ensure that the stack pointer accounts for the worst-case stack usage at exit. This
     // could toast some stack that the DFG used. We need to do it before storing to stack offsets
     // used by baseline.
@@ -572,227 +735,85 @@ void OSRExit::compileExit(CCallHelpers& jit, VM& vm, const OSRExit& exit, const
         CCallHelpers::framePointerRegister, CCallHelpers::stackPointerRegister);
 
     // Restore the DFG callee saves and then save the ones the baseline JIT uses.
-    jit.emitRestoreCalleeSaves();
-    jit.emitSaveCalleeSavesFor(jit.baselineCodeBlock());
+    jit.emitRestoreCalleeSavesFor(jit.codeBlock()->jitCode()->calleeSaveRegisters());
+    jit.emitSaveCalleeSavesFor(jit.baselineCodeBlock()->jitCode()->calleeSaveRegisters());
 
     // The tag registers are needed to materialize recoveries below.
     jit.emitMaterializeTagCheckRegisters();
 
-    if (exit.isExceptionHandler())
-        jit.copyCalleeSavesToEntryFrameCalleeSavesBuffer(vm.topEntryFrame);
-
-    if (exit.m_codeOrigin.inlineStackContainsActiveCheckpoint()) {
-        // FIXME: Maybe we shouldn't use a probe but filling all the side state objects is tricky otherwise...
-        Vector<ValueRecovery> values(operands.numberOfTmps());
-        for (size_t i = 0; i < operands.numberOfTmps(); ++i)
-            values[i] = operands.tmp(i);
-
-        VM* vmPtr = &vm;
-        auto* tmpScratch = scratch + operands.tmpIndex(0);
-        jit.probe([=, values = WTFMove(values)] (Probe::Context& context) {
-            auto addSideState = [&] (CallFrame* frame, BytecodeIndex index, size_t tmpOffset) {
-                std::unique_ptr<CheckpointOSRExitSideState> sideState = WTF::makeUnique<CheckpointOSRExitSideState>();
-
-                sideState->bytecodeIndex = index;
-                for (size_t i = 0; i < maxNumCheckpointTmps; ++i) {
-                    auto& recovery = values[i + tmpOffset];
-                    // FIXME: We should do what the FTL does and materialize all the JSValues into the scratch buffer.
-                    switch (recovery.technique()) {
-                    case Constant:
-                        sideState->tmps[i] = recovery.constant();
-                        break;
-
-                    case UnboxedInt32InGPR:
-                    case Int32DisplacedInJSStack: {
-                        sideState->tmps[i] = jsNumber(static_cast<int32_t>(tmpScratch[i + tmpOffset]));
-                        break;
-                    }
-
-#if USE(JSVALUE32_64)
-                    case InPair:
-#endif
-                    case InGPR:
-                    case BooleanDisplacedInJSStack:
-                    case CellDisplacedInJSStack:
-                    case DisplacedInJSStack: {
-                        sideState->tmps[i] = reinterpret_cast<JSValue*>(tmpScratch)[i + tmpOffset];
-                        break;
-                    }
-
-                    case UnboxedCellInGPR: {
-#if USE(JSVALUE64)
-                        sideState->tmps[i] = reinterpret_cast<JSValue*>(tmpScratch)[i + tmpOffset];
-#else
-                        EncodedValueDescriptor* valueDescriptor = bitwise_cast<EncodedValueDescriptor*>(tmpScratch + i + tmpOffset);
-                        sideState->tmps[i] = JSValue(JSValue::CellTag, valueDescriptor->asBits.payload);
-#endif
-                        break;
-                    }
-
-                    case UnboxedBooleanInGPR: {
-                        sideState->tmps[i] = jsBoolean(static_cast<bool>(tmpScratch[i + tmpOffset]));
-                        break;
-                    }
-
-                    default: 
-                        RELEASE_ASSERT_NOT_REACHED();
-                        break;
-                    }
-                }
-
-                vmPtr->addCheckpointOSRSideState(frame, WTFMove(sideState));
-            };
-
-            const CodeOrigin* codeOrigin;
-            CallFrame* callFrame = context.gpr<CallFrame*>(GPRInfo::callFrameRegister);
-            for (codeOrigin = &exit.m_codeOrigin; codeOrigin && codeOrigin->inlineCallFrame(); codeOrigin = codeOrigin->inlineCallFrame()->getCallerSkippingTailCalls()) {
-                BytecodeIndex callBytecodeIndex = codeOrigin->bytecodeIndex();
-                if (!callBytecodeIndex.checkpoint())
-                    continue;
-
-                auto* inlineCallFrame = codeOrigin->inlineCallFrame();
-                addSideState(reinterpret_cast_ptr<CallFrame*>(reinterpret_cast<char*>(callFrame) + inlineCallFrame->returnPCOffset() - sizeof(CPURegister)), callBytecodeIndex, inlineCallFrame->tmpOffset);
-            }
-
-            if (!codeOrigin)
-                return;
-
-            if (BytecodeIndex bytecodeIndex = codeOrigin->bytecodeIndex(); bytecodeIndex.checkpoint())
-                addSideState(callFrame, bytecodeIndex, 0);
-        });
+    if (inlineStackContainsActiveCheckpoint) {
+        EncodedJSValue* tmpScratch = scratch + operands.tmpIndex(0);
+        jit.setupArguments<decltype(operationMaterializeOSRExitSideState)>(CCallHelpers::TrustedImmPtr(&vm), CCallHelpers::TrustedImmPtr(&exit), CCallHelpers::TrustedImmPtr(tmpScratch));
+        jit.prepareCallOperation(vm);
+        jit.move(AssemblyHelpers::TrustedImmPtr(tagCFunction<OperationPtrTag>(operationMaterializeOSRExitSideState)), GPRInfo::nonArgGPR0);
+        jit.call(GPRInfo::nonArgGPR0, OperationPtrTag);
     }
 
     // Do all data format conversions and store the results into the stack.
 
+#if USE(JSVALUE64)
+    constexpr GPRReg srcBufferGPR = GPRInfo::regT2;
+    constexpr GPRReg destBufferGPR = GPRInfo::regT3;
+    constexpr GPRReg undefinedGPR = GPRInfo::regT4;
+    bool undefinedGPRIsInitialized = false;
+
+    jit.move(CCallHelpers::TrustedImmPtr(scratch), srcBufferGPR);
+    jit.move(CCallHelpers::framePointerRegister, destBufferGPR);
+    CCallHelpers::CopySpooler spooler(CCallHelpers::CopySpooler::BufferRegs::AllowModification, jit, srcBufferGPR, destBufferGPR, GPRInfo::regT0, GPRInfo::regT1);
+#endif
     for (size_t index = 0; index < operands.size(); ++index) {
         const ValueRecovery& recovery = operands[index];
         Operand operand = operands.operandForIndex(index);
         if (operand.isTmp())
             continue;
 
-        if (operand.isLocal() && operand.toLocal() < static_cast<int>(jit.baselineCodeBlock()->calleeSaveSpaceAsVirtualRegisters()))
+        if (operand.isLocal() && operand.toLocal() < static_cast<int>(CodeBlock::calleeSaveSpaceAsVirtualRegisters(*jit.baselineCodeBlock()->jitCode()->calleeSaveRegisters())))
             continue;
 
         switch (recovery.technique()) {
+        case Constant: {
+#if USE(JSVALUE64)
+            EncodedJSValue currentConstant = JSValue::encode(recovery.constant());
+            if (currentConstant == encodedJSUndefined()) {
+                if (UNLIKELY(!undefinedGPRIsInitialized)) {
+                    jit.move(CCallHelpers::TrustedImm64(encodedJSUndefined()), undefinedGPR);
+                    undefinedGPRIsInitialized = true;
+                }
+                spooler.copyGPR(undefinedGPR);
+            } else
+                spooler.moveConstant(currentConstant);
+            spooler.storeGPR(operand.virtualRegister().offset() * sizeof(CPURegister));
+            break;
+#else
+            FALLTHROUGH;
+#endif
+        }
         case DisplacedInJSStack:
+        case BooleanDisplacedInJSStack:
+        case Int32DisplacedInJSStack:
+        case CellDisplacedInJSStack:
+        case DoubleDisplacedInJSStack:
+        case UnboxedBooleanInGPR:
+        case UnboxedInt32InGPR:
+        case UnboxedCellInGPR:
+        case UnboxedDoubleInFPR:
         case InFPR:
 #if USE(JSVALUE64)
         case InGPR:
-        case UnboxedCellInGPR:
-        case CellDisplacedInJSStack:
-        case BooleanDisplacedInJSStack:
-            jit.load64(scratch + index, GPRInfo::regT0);
-            jit.store64(GPRInfo::regT0, AssemblyHelpers::addressFor(operand));
+        case UnboxedInt52InGPR:
+        case Int52DisplacedInJSStack:
+        case UnboxedStrictInt52InGPR:
+        case StrictInt52DisplacedInJSStack:
+            spooler.loadGPR(index * sizeof(CPURegister));
+            spooler.storeGPR(operand.virtualRegister().offset() * sizeof(CPURegister));
             break;
 #else // not USE(JSVALUE64)
         case InPair:
-            jit.load32(
-                &bitwise_cast<EncodedValueDescriptor*>(scratch + index)->asBits.tag,
-                GPRInfo::regT0);
-            jit.load32(
-                &bitwise_cast<EncodedValueDescriptor*>(scratch + index)->asBits.payload,
-                GPRInfo::regT1);
-            jit.store32(
-                GPRInfo::regT0,
-                AssemblyHelpers::tagFor(operand));
-            jit.store32(
-                GPRInfo::regT1,
-                AssemblyHelpers::payloadFor(operand));
-            break;
-
-        case UnboxedCellInGPR:
-        case CellDisplacedInJSStack:
-            jit.load32(
-                &bitwise_cast<EncodedValueDescriptor*>(scratch + index)->asBits.payload,
-                GPRInfo::regT0);
-            jit.store32(
-                AssemblyHelpers::TrustedImm32(JSValue::CellTag),
-                AssemblyHelpers::tagFor(operand));
-            jit.store32(
-                GPRInfo::regT0,
-                AssemblyHelpers::payloadFor(operand));
-            break;
-
-        case UnboxedBooleanInGPR:
-        case BooleanDisplacedInJSStack:
-            jit.load32(
-                &bitwise_cast<EncodedValueDescriptor*>(scratch + index)->asBits.payload,
-                GPRInfo::regT0);
-            jit.store32(
-                AssemblyHelpers::TrustedImm32(JSValue::BooleanTag),
-                AssemblyHelpers::tagFor(operand));
-            jit.store32(
-                GPRInfo::regT0,
-                AssemblyHelpers::payloadFor(operand));
+            jit.loadValue(scratch + index, JSRInfo::jsRegT10);
+            jit.storeValue(JSRInfo::jsRegT10, AssemblyHelpers::addressFor(operand));
             break;
 #endif // USE(JSVALUE64)
 
-        case UnboxedInt32InGPR:
-        case Int32DisplacedInJSStack:
-#if USE(JSVALUE64)
-            jit.load64(scratch + index, GPRInfo::regT0);
-            jit.zeroExtend32ToPtr(GPRInfo::regT0, GPRInfo::regT0);
-            jit.or64(GPRInfo::numberTagRegister, GPRInfo::regT0);
-            jit.store64(GPRInfo::regT0, AssemblyHelpers::addressFor(operand));
-#else
-            jit.load32(
-                &bitwise_cast<EncodedValueDescriptor*>(scratch + index)->asBits.payload,
-                GPRInfo::regT0);
-            jit.store32(
-                AssemblyHelpers::TrustedImm32(JSValue::Int32Tag),
-                AssemblyHelpers::tagFor(operand));
-            jit.store32(
-                GPRInfo::regT0,
-                AssemblyHelpers::payloadFor(operand));
-#endif
-            break;
-
-#if USE(JSVALUE64)
-        case UnboxedInt52InGPR:
-        case Int52DisplacedInJSStack:
-            jit.load64(scratch + index, GPRInfo::regT0);
-            jit.rshift64(
-                AssemblyHelpers::TrustedImm32(JSValue::int52ShiftAmount), GPRInfo::regT0);
-            jit.boxInt52(GPRInfo::regT0, GPRInfo::regT0, GPRInfo::regT1, FPRInfo::fpRegT0);
-            jit.store64(GPRInfo::regT0, AssemblyHelpers::addressFor(operand));
-            break;
-
-        case UnboxedStrictInt52InGPR:
-        case StrictInt52DisplacedInJSStack:
-            jit.load64(scratch + index, GPRInfo::regT0);
-            jit.boxInt52(GPRInfo::regT0, GPRInfo::regT0, GPRInfo::regT1, FPRInfo::fpRegT0);
-            jit.store64(GPRInfo::regT0, AssemblyHelpers::addressFor(operand));
-            break;
-#endif
-
-        case UnboxedDoubleInFPR:
-        case DoubleDisplacedInJSStack:
-            jit.move(AssemblyHelpers::TrustedImmPtr(scratch + index), GPRInfo::regT0);
-            jit.loadDouble(MacroAssembler::Address(GPRInfo::regT0), FPRInfo::fpRegT0);
-            jit.purifyNaN(FPRInfo::fpRegT0);
-#if USE(JSVALUE64)
-            jit.boxDouble(FPRInfo::fpRegT0, GPRInfo::regT0);
-            jit.store64(GPRInfo::regT0, AssemblyHelpers::addressFor(operand));
-#else
-            jit.storeDouble(FPRInfo::fpRegT0, AssemblyHelpers::addressFor(operand));
-#endif
-            break;
-
-        case Constant:
-#if USE(JSVALUE64)
-            jit.store64(
-                AssemblyHelpers::TrustedImm64(JSValue::encode(recovery.constant())),
-                AssemblyHelpers::addressFor(operand));
-#else
-            jit.store32(
-                AssemblyHelpers::TrustedImm32(recovery.constant().tag()),
-                AssemblyHelpers::tagFor(operand));
-            jit.store32(
-                AssemblyHelpers::TrustedImm32(recovery.constant().payload()),
-                AssemblyHelpers::payloadFor(operand));
-#endif
-            break;
-
         case DirectArgumentsThatWereNotCreated:
         case ClonedArgumentsThatWereNotCreated:
             // Don't do this, yet.
@@ -803,6 +824,9 @@ void OSRExit::compileExit(CCallHelpers& jit, VM& vm, const OSRExit& exit, const
             break;
         }
     }
+#if USE(JSVALUE64)
+    spooler.finalizeGPR();
+#endif
 
     // Now that things on the stack are recovered, do the arguments recovery. We assume that arguments
     // recoveries don't recursively refer to each other. But, we don't try to assume that they only
@@ -858,10 +882,11 @@ void OSRExit::compileExit(CCallHelpers& jit, VM& vm, const OSRExit& exit, const
     adjustAndJumpToTarget(vm, jit, exit);
 }
 
-void JIT_OPERATION operationDebugPrintSpeculationFailure(CallFrame* callFrame, void* debugInfoRaw, void* scratch)
+JSC_DEFINE_JIT_OPERATION(operationDebugPrintSpeculationFailure, void, (CallFrame* callFrame, void* debugInfoRaw, void* scratch))
 {
     VM& vm = callFrame->deprecatedVM();
     NativeCallFrameTracer tracer(vm, callFrame);
+    ActiveScratchBufferScope activeScratchBufferScope(ScratchBuffer::fromData(scratch), GPRInfo::numberOfRegisters + FPRInfo::numberOfRegisters);
 
     SpeculationFailureDebugInfo* debugInfo = static_cast<SpeculationFailureDebugInfo*>(debugInfoRaw);
     CodeBlock* codeBlock = debugInfo->codeBlock;
