@@ -34,6 +34,7 @@
 #include "DFGClobberize.h"
 #include "DFGForAllKills.h"
 #include "DFGGraph.h"
+#include "DFGMayExit.h"
 #include "DFGPhase.h"
 #include <wtf/ListDump.h>
 
@@ -195,10 +196,12 @@ private:
             }
                 
             case FilterGetByStatus:
-            case FilterPutByIdStatus:
+            case FilterPutByStatus:
             case FilterCallLinkStatus:
-            case FilterInByIdStatus:
+            case FilterInByStatus:
             case FilterDeleteByStatus:
+            case FilterCheckPrivateBrandStatus:
+            case FilterSetPrivateBrandStatus:
                 break;
 
             case GetByOffset: {
@@ -252,6 +255,9 @@ private:
         }
         if (verbose)
             dataLog("Selected lastUserIndex = ", lastUserIndex, ", ", block->at(lastUserIndex), "\n");
+
+        InlineCallFrame* startingInlineCallFrame = block->at(candidateNodeIndex)->origin.forExit.inlineCallFrame();
+        HashSet<InlineCallFrame*, WTF::DefaultHash<InlineCallFrame*>, WTF::NullableHashTraits<InlineCallFrame*>> seenInlineCallFrames;
         
         // We're still in business. Determine if between the candidate and the last user there is any
         // effect that could interfere with sinking.
@@ -264,7 +270,6 @@ private:
             // what gets read on OSR exit.
             switch (node->op()) {
             case MovHint:
-            case ZombieHint:
             case KillStack:
                 if (argumentsInvolveStackSlot(candidate, node->unlinkedOperand())) {
                     if (verbose)
@@ -282,7 +287,6 @@ private:
                 break;
                 
             case SetLocal:
-            case Flush:
                 if (argumentsInvolveStackSlot(candidate, node->operand())) {
                     if (verbose)
                         dataLog("    Interference at ", node, "\n");
@@ -310,6 +314,24 @@ private:
                     return;
                 }
             } }
+
+
+            if (startingInlineCallFrame) {
+                if (mayExit(m_graph, node) != DoesNotExit)
+                    seenInlineCallFrames.add(node->origin.forExit.inlineCallFrame());
+            }
+        }
+
+        for (InlineCallFrame* inlineCallFrame : seenInlineCallFrames) {
+            ASSERT(startingInlineCallFrame);
+
+            while (1) {
+                if (!inlineCallFrame)
+                    return;
+                if (inlineCallFrame == startingInlineCallFrame)
+                    break;
+                inlineCallFrame = inlineCallFrame->directCaller.inlineCallFrame();
+            }
         }
         
         // We can make this work.
@@ -399,10 +421,12 @@ private:
             }
 
             case FilterGetByStatus:
-            case FilterPutByIdStatus:
+            case FilterPutByStatus:
             case FilterCallLinkStatus:
-            case FilterInByIdStatus:
+            case FilterInByStatus:
             case FilterDeleteByStatus:
+            case FilterCheckPrivateBrandStatus:
+            case FilterSetPrivateBrandStatus:
                 if (node->child1().node() == candidate)
                     node->remove(m_graph);
                 break;
