@@ -33,6 +33,7 @@
 #include "LinkBuffer.h"
 #include <wtf/FastBitVector.h>
 #include <wtf/FileSystem.h>
+#include <wtf/FixedVector.h>
 #include <wtf/IterationStatus.h>
 #include <wtf/PageReservation.h>
 #include <wtf/ProcessID.h>
@@ -85,6 +86,14 @@ namespace JSC {
 
 using namespace WTF;
 
+#if OS(DARWIN) && CPU(ARM64)
+// We already rely on page size being CeilingOnPageSize elsewhere (e.g. MarkedBlock).
+// Just use the constexpr CeilingOnPageSize for better efficiency.
+static inline constexpr size_t executablePageSize() { return CeilingOnPageSize; }
+#else
+static inline size_t executablePageSize() { return WTF::pageSize(); }
+#endif
+
 #if USE(LIBPAS_JIT_HEAP)
 static constexpr size_t minimumPoolSizeForSegregatedHeap = 256 * MB;
 #endif
@@ -94,22 +103,36 @@ static constexpr size_t fixedExecutableMemoryPoolSize = FIXED_EXECUTABLE_MEMORY_
 #elif CPU(ARM64)
 #if ENABLE(JUMP_ISLANDS)
 static constexpr size_t fixedExecutableMemoryPoolSize = 512 * MB;
-// These sizes guarantee that any jump within an island can jump forwards or backwards
-// to the adjacent island in a single instruction.
-static constexpr size_t regionSize = 112 * MB;
-static constexpr size_t islandRegionSize = 16 * MB;
-static constexpr size_t maxNumberOfRegions = fixedExecutableMemoryPoolSize / regionSize;
-static constexpr size_t islandSizeInBytes = 4;
-static constexpr size_t maxIslandsPerRegion = islandRegionSize / islandSizeInBytes;
 #else
 static constexpr size_t fixedExecutableMemoryPoolSize = 128 * MB;
 #endif
+#elif CPU(ARM_THUMB2)
+#if ENABLE(JUMP_ISLANDS)
+static constexpr size_t fixedExecutableMemoryPoolSize = 32 * MB;
+#else
+static constexpr size_t fixedExecutableMemoryPoolSize = 16 * MB;
+#endif
 #elif CPU(X86_64)
 static constexpr size_t fixedExecutableMemoryPoolSize = 1 * GB;
 #else
 static constexpr size_t fixedExecutableMemoryPoolSize = 32 * MB;
 #endif
 
+#if ENABLE(JUMP_ISLANDS)
+#if CPU(ARM64)
+static constexpr double islandRegionSizeFraction = 0.125;
+static constexpr size_t islandSizeInBytes = 4;
+#elif CPU(ARM_THUMB2)
+static constexpr double islandRegionSizeFraction = 0.05;
+static constexpr size_t islandSizeInBytes = 4;
+#endif
+#endif
+
+// Quick sanity check, in case FIXED_EXECUTABLE_MEMORY_POOL_SIZE_IN_MB was set.
+#if !ENABLE(JUMP_ISLANDS)
+static_assert(fixedExecutableMemoryPoolSize <= MacroAssembler::nearJumpRange, "Executable pool size is too large for near jump/call without JUMP_ISLANDS");
+#endif
+
 #if CPU(ARM)
 static constexpr double executablePoolReservationFraction = 0.15;
 #else
@@ -127,7 +150,7 @@ static bool isJITEnabled()
 {
     bool jitEnabled = !g_jscConfig.jitDisabled;
 #if HAVE(IOS_JIT_RESTRICTIONS)
-    return processHasEntitlement("dynamic-codesigning") && jitEnabled;
+    return processHasEntitlement("dynamic-codesigning"_s) && jitEnabled;
 #else
     return jitEnabled;
 #endif
@@ -143,7 +166,7 @@ void ExecutableAllocator::setJITEnabled(bool enabled)
     g_jscConfig.jitDisabled = !enabled;
 
 #if HAVE(IOS_JIT_RESTRICTIONS)
-    if (!enabled && processHasEntitlement("dynamic-codesigning")) {
+    if (!enabled && processHasEntitlement("dynamic-codesigning"_s)) {
         // Because of an OS quirk, even after the JIT region has been unmapped,
         // the OS thinks that region is reserved, and as such, can cause Gigacage
         // allocation to fail. We work around this by initializing the Gigacage
@@ -335,6 +358,10 @@ static ALWAYS_INLINE JITReservation initializeJITPageReservation()
     if (!isJITEnabled())
         return reservation;
 
+#if OS(DARWIN)
+    // Call pageSize() to run its assertions to enforce invariants that executablePageSize() relies on.
+    WTF::pageSize();
+#endif
     reservation.size = fixedExecutableMemoryPoolSize;
 
     if (Options::jitMemoryReservationSize()) {
@@ -343,19 +370,12 @@ static ALWAYS_INLINE JITReservation initializeJITPageReservation()
         if (reservation.size * executablePoolReservationFraction < minimumExecutablePoolReservationSize)
             reservation.size += minimumExecutablePoolReservationSize;
 #endif
-
-#if ENABLE(JUMP_ISLANDS)
-        // If asked for a reservation smaller than island size, assume that we want that size allocation
-        // plus an island. The alternative would be to turn off jump islands, but since we only use
-        // this for testing, this is probably the easier way to do it.
-        //
-        // The main reason for this is that some JSC stress tests run with a 50KB pool. This hack means
-        // we don't have to change anything about those tests.
-        if (reservation.size < islandRegionSize)
-            reservation.size += islandRegionSize;
-#endif // ENABLE(JUMP_ISLANDS)
     }
-    reservation.size = std::max(roundUpToMultipleOf(pageSize(), reservation.size), pageSize() * 2);
+    reservation.size = std::max(roundUpToMultipleOf(executablePageSize(), reservation.size), executablePageSize() * 2);
+
+#if !ENABLE(JUMP_ISLANDS)
+    RELEASE_ASSERT(reservation.size <= MacroAssembler::nearJumpRange, "Executable pool size is too large for near jump/call without JUMP_ISLANDS");
+#endif
 
 #if USE(LIBPAS_JIT_HEAP)
     if (reservation.size < minimumPoolSizeForSegregatedHeap)
@@ -378,7 +398,7 @@ static ALWAYS_INLINE JITReservation initializeJITPageReservation()
     reservation.pageReservation = tryCreatePageReservation(reservation.size);
 
     if (Options::verboseExecutablePoolAllocation())
-        dataLog(getpid(), ": Got executable pool reservation at ", RawPointer(reservation.pageReservation.base()), "...", RawPointer(bitwise_cast<char*>(reservation.pageReservation.base()) + reservation.pageReservation.size()), ", while I'm at ", RawPointer(bitwise_cast<void*>(initializeJITPageReservation)), "\n");
+        dataLog(getpid(), ": Got executable pool reservation at ", RawPointer(reservation.pageReservation.base()), "...", RawPointer(reservation.pageReservation.end()), ", while I'm at ", RawPointer(bitwise_cast<void*>(initializeJITPageReservation)), "\n");
     
     if (reservation.pageReservation) {
         ASSERT(reservation.pageReservation.size() == reservation.size);
@@ -400,10 +420,10 @@ static ALWAYS_INLINE JITReservation initializeJITPageReservation()
 #if ENABLE(SEPARATED_WX_HEAP)
         if (!g_jscConfig.useFastJITPermissions) {
             // First page of our JIT allocation is reserved.
-            ASSERT(reservation.size >= pageSize() * 2);
-            reservation.base = (void*)((uintptr_t)(reservation.base) + pageSize());
-            reservation.size -= pageSize();
-            initializeSeparatedWXHeaps(reservation.pageReservation.base(), pageSize(), reservation.base, reservation.size);
+            ASSERT(reservation.size >= executablePageSize() * 2);
+            reservation.base = (void*)((uintptr_t)(reservation.base) + executablePageSize());
+            reservation.size -= executablePageSize();
+            initializeSeparatedWXHeaps(reservation.pageReservation.base(), executablePageSize(), reservation.base, reservation.size);
         }
 #endif
 
@@ -430,10 +450,7 @@ class FixedVMPoolExecutableAllocator final {
 
 public:
     FixedVMPoolExecutableAllocator()
-#if ENABLE(JUMP_ISLANDS)
-        : m_allocators(constructFixedSizeArrayWithArguments<RegionAllocator, maxNumberOfRegions>(*this))
-        , m_numAllocators(maxNumberOfRegions)
-#else
+#if !ENABLE(JUMP_ISLANDS)
         : m_allocator(*this)
 #endif
     {
@@ -441,31 +458,55 @@ public:
         m_reservation = WTFMove(reservation.pageReservation);
         if (m_reservation) {
 #if ENABLE(JUMP_ISLANDS)
+            // Consider this scenario:
+            //
+            //                                    <------------- nearJumpRange ------------->
+            //    <------------- nearJumpRange -------------->
+            //    [  island 0 ] [ JIT region 1  ] [ island 1 ] [ JIT region 2  ] [ island 2 ] [ JIT region3  ]
+            //
+            //                         C1 ---jump---> I1 --------------jump---------> I2 ---jump---> C3
+            //
+            // In order to jump across a distance that spans multiple nearJumpRanges, we currently
+            // use chaining near jumps. Hence, a near jump in a jump island also needs to be able
+            // to reach its neighboring jump islands in order to form this chain.
+            //
+            // For example, let say we have code in JIT region 1 that needs to jump to code in JIT region 3 in
+            // the illustration above. That jump will be implemented as:
+            //   1. Code C1 in JIT region 1 near jumps to island I1 in island 1.
+            //   2. Island I1 near jumps to island I2 in island 2.
+            //   3. Island I2 near jumps to code C3 in JIT region 3.
+            //
+            // Each of these near jumps need to be within the range of MacroAssembler::nearJumpRange.
+            //
+            // As a result, the maximum size of each JIT region is:
+            //     MacroAssembler::nearJumpRange - 2 * islandRegionSize
+            //
+            // This is why each RegionAllocator tracks a range of m_regionSize instead of
+            // MacroAssembler::nearJumpRange.
+            //
+            // Note: the illustration above shows a jump chain in the forward direction. The jump island
+            // scheme also allows for a jump chain in the backward direction e.g. from C3 to C1.
+            
+            const size_t islandRegionSize = roundUpToMultipleOf(executablePageSize(), static_cast<size_t>(MacroAssembler::nearJumpRange * islandRegionSizeFraction));
+            m_regionSize = MacroAssembler::nearJumpRange - islandRegionSize;
+            RELEASE_ASSERT(isPageAligned(executablePageSize(), islandRegionSize));
+            RELEASE_ASSERT(isPageAligned(executablePageSize(), m_regionSize));
+            const unsigned numAllocators = (reservation.size + m_regionSize - 1) / m_regionSize;
+            m_allocators = FixedVector<RegionAllocator>::createWithSizeAndConstructorArguments(numAllocators, *this);
+
             uintptr_t start = bitwise_cast<uintptr_t>(memoryStart());
             uintptr_t reservationEnd = bitwise_cast<uintptr_t>(memoryEnd());
-            for (size_t i = 0; i < maxNumberOfRegions; ++i) {
-                RELEASE_ASSERT(start < reservationEnd || Options::jitMemoryReservationSize());
-                if (start >= reservationEnd) {
-                    m_numAllocators = i;
-                    break;
-                }
-                m_allocators[i].m_start = tagCodePtr<ExecutableMemoryPtrTag>(bitwise_cast<void*>(start));
-                m_allocators[i].m_end = tagCodePtr<ExecutableMemoryPtrTag>(bitwise_cast<void*>(start + regionSize));
-                if (m_allocators[i].end() > reservationEnd) {
-                    // We may have taken a page for the executable only copy thunk.
-                    RELEASE_ASSERT(i == maxNumberOfRegions - 1 || Options::jitMemoryReservationSize());
-                    m_allocators[i].m_end = tagCodePtr<ExecutableMemoryPtrTag>(bitwise_cast<void*>(reservationEnd));
-                }
-
-                size_t sizeInBytes = m_allocators[i].allocatorSize();
-                m_allocators[i].addFreshFreeSpace(bitwise_cast<void*>(m_allocators[i].start()), sizeInBytes);
-                m_bytesReserved += sizeInBytes;
-
-                RELEASE_ASSERT(m_allocators[i].allocatorSize() < regionSize);
-                RELEASE_ASSERT(m_allocators[i].islandBegin() > m_allocators[i].start());
-                RELEASE_ASSERT(m_allocators[i].islandBegin() < m_allocators[i].end());
-
-                start += regionSize;
+            for (size_t i = 0; i < numAllocators; ++i) {
+                uintptr_t end = start + m_regionSize;
+                uintptr_t islandBegin = end - islandRegionSize;
+                // The island in the very last region is never actually used (everything goes backwards), but we
+                // can't put code there in case they do need to use a backward jump island, so set up accordingly.
+                if (i == numAllocators - 1)
+                    islandBegin = end = std::min(islandBegin, reservationEnd);
+                RELEASE_ASSERT(end <= reservationEnd);
+                m_allocators[i].configure(start, islandBegin, end);
+                m_bytesReserved += m_allocators[i].allocatorSize();
+                start += m_regionSize;
             }
 #else
             m_allocator.addFreshFreeSpace(reservation.base, reservation.size);
@@ -494,22 +535,22 @@ public:
         Locker locker { getLock() };
 
         unsigned start = 0;
-        if (Options::useRandomizingExecutableIslandAllocation())
-            start = cryptographicallyRandomNumber() % m_numAllocators;
+        if (UNLIKELY(Options::useRandomizingExecutableIslandAllocation()))
+            start = cryptographicallyRandomNumber() % m_allocators.size();
 
         unsigned i = start;
         while (true) {
             RegionAllocator& allocator = m_allocators[i];
             if (RefPtr<ExecutableMemoryHandle> result = allocator.allocate(locker, sizeInBytes))
                 return result;
-            i = (i + 1) % m_numAllocators;
+            i = (i + 1) % m_allocators.size();
             if (i == start)
                 break;
         }
         return nullptr;
 #else
         return m_allocator.allocate(sizeInBytes);
-#endif // ENABLE(JUMP_ISLANDS)
+#endif
     }
 
     Lock& getLock() WTF_RETURNS_LOCK(m_lock) { return m_lock; }
@@ -700,7 +741,7 @@ private:
             m_islandsForJumpSourceLocation.insert(islands);
         }
 
-        RegionAllocator* allocator = findRegion(jumpLocation > target ? jumpLocation - regionSize : jumpLocation);
+        RegionAllocator* allocator = findRegion(jumpLocation > target ? jumpLocation - m_regionSize : jumpLocation);
         RELEASE_ASSERT(allocator);
         void* result = allocator->allocateIsland();
         void* currentIsland = result;
@@ -709,10 +750,10 @@ private:
             islands->jumpIslands.append(CodeLocationLabel<ExecutableMemoryPtrTag>(tagCodePtr<ExecutableMemoryPtrTag>(currentIsland)));
 
             auto emitJumpTo = [&] (void* target) {
-                RELEASE_ASSERT(ARM64Assembler::canEmitJump(bitwise_cast<void*>(jumpLocation), target));
+                RELEASE_ASSERT(Assembler::canEmitJump(bitwise_cast<void*>(jumpLocation), target));
 
                 MacroAssembler jit;
-                auto jump = jit.jump();
+                auto nearTailCall = jit.nearTailCall();
                 LinkBuffer linkBuffer(jit, MacroAssemblerCodePtr<NoPtrTag>(currentIsland), islandSizeInBytes, LinkBuffer::Profile::JumpIsland, JITCompilationMustSucceed, false);
                 RELEASE_ASSERT(linkBuffer.isValid());
 
@@ -724,20 +765,20 @@ private:
                 // has a jump linked to this island hasn't finalized yet, they're guaranteed to finalize there code and run an isb.
                 linkBuffer.setIsJumpIsland();
 
-                linkBuffer.link(jump, CodeLocationLabel<NoPtrTag>(target));
+                linkBuffer.link(nearTailCall, CodeLocationLabel<NoPtrTag>(target));
                 FINALIZE_CODE(linkBuffer, NoPtrTag, "Jump Island: %lu", jumpLocation);
             };
 
-            if (ARM64Assembler::canEmitJump(bitwise_cast<void*>(jumpLocation), bitwise_cast<void*>(target))) {
+            if (Assembler::canEmitJump(bitwise_cast<void*>(jumpLocation), bitwise_cast<void*>(target))) {
                 emitJumpTo(bitwise_cast<void*>(target));
                 break;
             }
 
             uintptr_t nextIslandRegion;
             if (jumpLocation > target)
-                nextIslandRegion = jumpLocation - regionSize;
+                nextIslandRegion = jumpLocation - m_regionSize;
             else
-                nextIslandRegion = jumpLocation + regionSize;
+                nextIslandRegion = jumpLocation + m_regionSize;
 
             RegionAllocator* allocator = findRegion(nextIslandRegion);
             RELEASE_ASSERT(allocator);
@@ -763,7 +804,7 @@ private:
     public:
         Allocator(FixedVMPoolExecutableAllocator& allocator)
 #if !USE(LIBPAS_JIT_HEAP)
-            : Base(allocator.getLock(), jitAllocationGranule, pageSize()) // round up all allocations to 32 bytes
+            : Base(allocator.getLock(), jitAllocationGranule, executablePageSize()) // round up all allocations to 32 bytes
             ,
 #else
             :
@@ -798,12 +839,12 @@ private:
 
         void notifyNeedPage(void* page, size_t count) override
         {
-            m_fixedAllocator.m_reservation.commit(page, pageSize() * count);
+            m_fixedAllocator.m_reservation.commit(page, executablePageSize() * count);
         }
 
         void notifyPageIsFree(void* page, size_t count) override
         {
-            m_fixedAllocator.m_reservation.decommit(page, pageSize() * count);
+            m_fixedAllocator.m_reservation.decommit(page, executablePageSize() * count);
         }
 #endif // !USE(LIBPAS_JIT_HEAP)
 
@@ -821,20 +862,30 @@ private:
         RegionAllocator(FixedVMPoolExecutableAllocator& allocator)
             : Base(allocator)
         {
+            RELEASE_ASSERT(!(executablePageSize() % islandSizeInBytes), "Current implementation relies on this");
+        }
+
+        void configure(uintptr_t start, uintptr_t islandBegin, uintptr_t end)
+        {
+            RELEASE_ASSERT(start < islandBegin);
+            RELEASE_ASSERT(islandBegin <= end);
+            m_start = bitwise_cast<void*>(start);
+            m_islandBegin = bitwise_cast<void*>(islandBegin);
+            m_end = bitwise_cast<void*>(end);
+            RELEASE_ASSERT(!((this->islandBegin() - this->start()) % executablePageSize()));
+            RELEASE_ASSERT(!((this->end() - this->islandBegin()) % executablePageSize()));
+            addFreshFreeSpace(bitwise_cast<void*>(this->start()), allocatorSize());
         }
 
         //  ------------------------------------
         //  | jit allocations -->   <-- islands |
         //  -------------------------------------
 
-        uintptr_t start() { return bitwise_cast<uintptr_t>(untagCodePtr<ExecutableMemoryPtrTag>(m_start)); }
-        uintptr_t end() { return bitwise_cast<uintptr_t>(untagCodePtr<ExecutableMemoryPtrTag>(m_end)); }
+        uintptr_t start() { return bitwise_cast<uintptr_t>(m_start); }
+        uintptr_t islandBegin() { return bitwise_cast<uintptr_t>(m_islandBegin); }
+        uintptr_t end() { return bitwise_cast<uintptr_t>(m_end); }
 
-        uintptr_t islandBegin()
-        {
-            // [start, allocatorEnd)
-            return end() - islandRegionSize;
-        }
+        size_t maxIslandsInThisRegion() { return (end() - islandBegin()) / islandSizeInBytes; }
 
         uintptr_t allocatorSize()
         {
@@ -843,8 +894,8 @@ private:
 
         size_t islandsPerPage()
         {
-            size_t islandsPerPage = pageSize() / islandSizeInBytes;
-            ASSERT(islandsPerPage * islandSizeInBytes == pageSize());
+            size_t islandsPerPage = executablePageSize() / islandSizeInBytes;
+            ASSERT(islandsPerPage * islandSizeInBytes == executablePageSize());
             ASSERT(isPowerOfTwo(islandsPerPage));
             return islandsPerPage;
         }
@@ -872,12 +923,18 @@ private:
             if (void* result = findResult())
                 return result;
 
-            islandBits.resize(islandBits.size() + islandsPerPage());
-            if (UNLIKELY(islandBits.size() > maxIslandsPerRegion))
+            const size_t oldSize = islandBits.size();
+            const size_t maxIslandsInThisRegion = this->maxIslandsInThisRegion();
+
+            RELEASE_ASSERT(oldSize <= maxIslandsInThisRegion);
+            if (UNLIKELY(oldSize == maxIslandsInThisRegion))
                 crashOnJumpIslandExhaustion();
 
-            uintptr_t pageBegin = end - (islandBits.size() * islandSizeInBytes); // [islandBegin, end)
-            m_fixedAllocator.m_reservation.commit(bitwise_cast<void*>(pageBegin), pageSize());
+            const size_t newSize = std::min(oldSize + islandsPerPage(), maxIslandsInThisRegion);
+            islandBits.resize(newSize);
+
+            uintptr_t islandsBegin = end - (newSize * islandSizeInBytes); // [islandsBegin, end)
+            m_fixedAllocator.m_reservation.commit(bitwise_cast<void*>(islandsBegin), (newSize - oldSize) * islandSizeInBytes);
 
             void* result = findResult();
             RELEASE_ASSERT(result);
@@ -916,9 +973,14 @@ private:
             return false;
         }
 
+    private:
+#define REGION_ALLOCATOR_CODEPTR(field) \
+    WTF_FUNCPTR_PTRAUTH_STR("RegionAllocator." #field) field
+
         // Range: [start, end)
-        void* m_start;
-        void* m_end;
+        void* REGION_ALLOCATOR_CODEPTR(m_start);
+        void* REGION_ALLOCATOR_CODEPTR(m_islandBegin);
+        void* REGION_ALLOCATOR_CODEPTR(m_end);
         FastBitVector islandBits;
     };
 #endif // ENABLE(JUMP_ISLANDS)
@@ -955,8 +1017,8 @@ private:
     Lock m_lock;
     PageReservation m_reservation;
 #if ENABLE(JUMP_ISLANDS)
-    std::array<RegionAllocator, maxNumberOfRegions> m_allocators;
-    unsigned m_numAllocators;
+    size_t m_regionSize;
+    FixedVector<RegionAllocator> m_allocators;
     RedBlackTree<Islands, void*> m_islandsForJumpSourceLocation;
 #else
     Allocator m_allocator;
@@ -1152,8 +1214,8 @@ void dumpJITMemory(const void* dst, const void* src, size_t size)
         static void flush() WTF_REQUIRES_LOCK(dumpJITMemoryLock)
         {
             if (fd == -1) {
-                String path = Options::dumpJITMemoryPath();
-                path = path.replace("%pid", String::number(getCurrentProcessID()));
+                auto path = String::fromLatin1(Options::dumpJITMemoryPath());
+                path = makeStringByReplacingAll(path, "%pid"_s, String::number(getCurrentProcessID()));
                 fd = open(FileSystem::fileSystemRepresentation(path).data(), O_CREAT | O_TRUNC | O_APPEND | O_WRONLY | O_EXLOCK | O_NONBLOCK, 0666);
                 RELEASE_ASSERT(fd != -1);
             }
@@ -1215,7 +1277,7 @@ void dumpJITMemory(const void* dst, const void* src, size_t size)
 #endif
 }
 
-#if USE(LIBPAS_JIT_HEAP)
+#if USE(LIBPAS_JIT_HEAP) && ENABLE(JIT)
 RefPtr<ExecutableMemoryHandle> ExecutableMemoryHandle::createImpl(size_t sizeInBytes)
 {
     void* key = jit_heap_try_allocate(sizeInBytes);
@@ -1226,6 +1288,7 @@ RefPtr<ExecutableMemoryHandle> ExecutableMemoryHandle::createImpl(size_t sizeInB
 
 ExecutableMemoryHandle::~ExecutableMemoryHandle()
 {
+    AssemblyCommentRegistry::singleton().unregisterCodeRange(start().untaggedPtr(), end().untaggedPtr());
     FixedVMPoolExecutableAllocator* allocator = g_jscConfig.fixedVMPoolExecutableAllocator;
     allocator->handleWillBeReleased(*this, sizeInBytes());
     jit_heap_deallocate(key());
@@ -1241,7 +1304,7 @@ void ExecutableMemoryHandle::shrink(size_t newSizeInBytes)
         allocator->shrinkBytesAllocated(oldSizeInBytes, sizeInBytes());
     }
 }
-#endif
+#endif // USE(LIBPAS_JIT_HEAP) && ENABLE(JIT)
 
 } // namespace JSC
 
