@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 1999-2000 Harri Porten (porten@kde.org)
- * Copyright (C) 2006-2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2006-2021 Apple Inc. All rights reserved.
  * Copyright (C) 2009 Google Inc. All rights reserved.
  * Copyright (C) 2007-2009 Torch Mobile, Inc.
  * Copyright (C) 2010 &yet, LLC. (nate@andyet.net)
@@ -75,48 +75,112 @@
 #include "ExceptionHelpers.h"
 #include "VM.h"
 #include <limits>
+#include <wtf/Language.h>
+#include <wtf/unicode/icu/ICUHelpers.h>
 
+#if U_ICU_VERSION_MAJOR_NUM >= 69 || (U_ICU_VERSION_MAJOR_NUM == 68 && USE(APPLE_INTERNAL_SDK))
+#define HAVE_ICU_C_TIMEZONE_API 1
+#ifdef U_HIDE_DRAFT_API
+#undef U_HIDE_DRAFT_API
+#endif
+#include <unicode/ucal.h>
+#else
 // icu::TimeZone and icu::BasicTimeZone features are only available in ICU C++ APIs.
 // We use these C++ APIs as an exception.
 #undef U_SHOW_CPLUSPLUS_API
 #define U_SHOW_CPLUSPLUS_API 1
 #include <unicode/basictz.h>
+#include <unicode/locid.h>
 #include <unicode/timezone.h>
 #include <unicode/unistr.h>
 #undef U_SHOW_CPLUSPLUS_API
 #define U_SHOW_CPLUSPLUS_API 0
+#endif
 
 namespace JSC {
 
+#if PLATFORM(COCOA)
+std::atomic<uint64_t> lastTimeZoneID { 1 };
+#endif
+
+#if HAVE(ICU_C_TIMEZONE_API)
+class OpaqueICUTimeZone {
+    WTF_MAKE_FAST_ALLOCATED(OpaqueICUTimeZone);
+public:
+    std::unique_ptr<UCalendar, ICUDeleter<ucal_close>> m_calendar;
+    String m_canonicalTimeZoneID;
+};
+#else
+static icu::TimeZone* toICUTimeZone(OpaqueICUTimeZone* timeZone)
+{
+    return bitwise_cast<icu::TimeZone*>(timeZone);
+}
+static OpaqueICUTimeZone* toOpaqueICUTimeZone(icu::TimeZone* timeZone)
+{
+    return bitwise_cast<OpaqueICUTimeZone*>(timeZone);
+}
+#endif
+
 void OpaqueICUTimeZoneDeleter::operator()(OpaqueICUTimeZone* timeZone)
 {
-    if (timeZone)
-        delete bitwise_cast<icu::TimeZone*>(timeZone);
+    if (timeZone) {
+#if HAVE(ICU_C_TIMEZONE_API)
+        delete timeZone;
+#else
+        delete toICUTimeZone(timeZone);
+#endif
+    }
 }
 
 // Get the combined UTC + DST offset for the time passed in.
 //
 // NOTE: The implementation relies on the fact that no time zones have
 // more than one daylight savings offset change per month.
-// If this function is called with NaN it returns NaN.
+// If this function is called with NaN it returns random value.
 LocalTimeOffset DateCache::calculateLocalTimeOffset(double millisecondsFromEpoch, WTF::TimeType inputTimeType)
 {
-    auto& timeZoneCache = *bitwise_cast<icu::TimeZone*>(this->timeZoneCache());
     int32_t rawOffset = 0;
     int32_t dstOffset = 0;
     UErrorCode status = U_ZERO_ERROR;
+
+    // This function can fail input date is invalid: NaN etc.
+    // We can return any values in this case since later we fail when computing non timezone offset part anyway.
+    constexpr LocalTimeOffset failed { false, 0 };
+
+#if HAVE(ICU_C_TIMEZONE_API)
+    auto& timeZoneCache = *this->timeZoneCache();
+    ucal_setMillis(timeZoneCache.m_calendar.get(), millisecondsFromEpoch, &status);
+    if (U_FAILURE(status))
+        return failed;
+
+    if (inputTimeType != WTF::LocalTime) {
+        rawOffset = ucal_get(timeZoneCache.m_calendar.get(), UCAL_ZONE_OFFSET, &status);
+        if (U_FAILURE(status))
+            return failed;
+        dstOffset = ucal_get(timeZoneCache.m_calendar.get(), UCAL_DST_OFFSET, &status);
+        if (U_FAILURE(status))
+            return failed;
+    } else {
+        ucal_getTimeZoneOffsetFromLocal(timeZoneCache.m_calendar.get(), UCAL_TZ_LOCAL_FORMER, UCAL_TZ_LOCAL_FORMER, &rawOffset, &dstOffset, &status);
+        if (U_FAILURE(status))
+            return failed;
+    }
+#else
+    auto& timeZoneCache = *toICUTimeZone(this->timeZoneCache());
     if (inputTimeType != WTF::LocalTime) {
         constexpr bool isLocalTime = false;
         timeZoneCache.getOffset(millisecondsFromEpoch, isLocalTime, rawOffset, dstOffset, status);
+        if (U_FAILURE(status))
+            return failed;
     } else {
         // icu::TimeZone is a timezone instance which inherits icu::BasicTimeZone.
         // https://unicode-org.atlassian.net/browse/ICU-13705 will move getOffsetFromLocal to icu::TimeZone.
         static_cast<const icu::BasicTimeZone&>(timeZoneCache).getOffsetFromLocal(millisecondsFromEpoch, icu::BasicTimeZone::kFormer, icu::BasicTimeZone::kFormer, rawOffset, dstOffset, status);
+        if (U_FAILURE(status))
+            return failed;
     }
-    // The above can fail if input date is invalid: NaN etc.
-    // We can return any values in this case since later we fail when computing non timezone offset part anyway.
-    if (U_FAILURE(status))
-        return { false, 0 };
+#endif
+
     return { !!dstOffset, rawOffset + dstOffset };
 }
 
@@ -219,11 +283,6 @@ LocalTimeOffset DateCache::localTimeOffset(double millisecondsFromEpoch, WTF::Ti
     return offset;
 }
 
-static inline double timeToMS(double hour, double min, double sec, double ms)
-{
-    return (((hour * WTF::minutesPerHour + min) * WTF::secondsPerMinute + sec) * WTF::msPerSecond + ms);
-}
-
 double DateCache::gregorianDateTimeToMS(const GregorianDateTime& t, double milliseconds, WTF::TimeType inputTimeType)
 {
     double day = dateToDaysFrom1970(t.year(), t.month(), t.monthDay());
@@ -235,6 +294,12 @@ double DateCache::gregorianDateTimeToMS(const GregorianDateTime& t, double milli
     return localTimeResult - localToUTCTimeOffset;
 }
 
+double DateCache::localTimeToMS(double milliseconds, WTF::TimeType inputTimeType)
+{
+    double localToUTCTimeOffset = inputTimeType == WTF::LocalTime ? localTimeOffset(milliseconds, inputTimeType).offset : 0;
+    return milliseconds - localToUTCTimeOffset;
+}
+
 // input is UTC
 void DateCache::msToGregorianDateTime(double millisecondsFromEpoch, WTF::TimeType outputTimeType, GregorianDateTime& tm)
 {
@@ -284,9 +349,12 @@ double DateCache::parseDate(JSGlobalObject* globalObject, VM& vm, const String&
 // https://tc39.es/ecma402/#sec-defaulttimezone
 String DateCache::defaultTimeZone()
 {
+#if HAVE(ICU_C_TIMEZONE_API)
+    return timeZoneCache()->m_canonicalTimeZoneID;
+#else
     icu::UnicodeString timeZoneID;
     icu::UnicodeString canonicalTimeZoneID;
-    auto& timeZone = *bitwise_cast<icu::TimeZone*>(timeZoneCache());
+    auto& timeZone = *toICUTimeZone(timeZoneCache());
     timeZone.getID(timeZoneID);
 
     UErrorCode status = U_ZERO_ERROR;
@@ -300,10 +368,67 @@ String DateCache::defaultTimeZone()
         return "UTC"_s;
 
     return canonical;
+#endif
 }
 
+String DateCache::timeZoneDisplayName(bool isDST)
+{
+    if (m_timeZoneStandardDisplayNameCache.isNull()) {
+#if HAVE(ICU_C_TIMEZONE_API)
+        auto& timeZoneCache = *this->timeZoneCache();
+        CString language = defaultLanguage().utf8();
+        {
+            Vector<UChar, 32> standardDisplayNameBuffer;
+            auto status = callBufferProducingFunction(ucal_getTimeZoneDisplayName, timeZoneCache.m_calendar.get(), UCAL_STANDARD, language.data(), standardDisplayNameBuffer);
+            if (U_SUCCESS(status))
+                m_timeZoneStandardDisplayNameCache = String::adopt(WTFMove(standardDisplayNameBuffer));
+        }
+        {
+            Vector<UChar, 32> dstDisplayNameBuffer;
+            auto status = callBufferProducingFunction(ucal_getTimeZoneDisplayName, timeZoneCache.m_calendar.get(), UCAL_DST, language.data(), dstDisplayNameBuffer);
+            if (U_SUCCESS(status))
+                m_timeZoneDSTDisplayNameCache = String::adopt(WTFMove(dstDisplayNameBuffer));
+        }
+#else
+        auto& timeZoneCache = *toICUTimeZone(this->timeZoneCache());
+        String language = defaultLanguage();
+        icu::Locale locale(language.utf8().data());
+        {
+            icu::UnicodeString standardDisplayName;
+            timeZoneCache.getDisplayName(false /* inDaylight */, icu::TimeZone::LONG, locale, standardDisplayName);
+            m_timeZoneStandardDisplayNameCache = String(standardDisplayName.getBuffer(), standardDisplayName.length());
+        }
+        {
+            icu::UnicodeString dstDisplayName;
+            timeZoneCache.getDisplayName(true /* inDaylight */, icu::TimeZone::LONG, locale, dstDisplayName);
+            m_timeZoneDSTDisplayNameCache = String(dstDisplayName.getBuffer(), dstDisplayName.length());
+        }
+#endif
+    }
+    if (isDST)
+        return m_timeZoneDSTDisplayNameCache;
+    return m_timeZoneStandardDisplayNameCache;
+}
+
+#if PLATFORM(COCOA)
+static void timeZoneChangeNotification(CFNotificationCenterRef, void*, CFStringRef, const void*, CFDictionaryRef)
+{
+    ASSERT(isMainThread());
+    ++lastTimeZoneID;
+}
+#endif
+
 // To confine icu::TimeZone destructor invocation in this file.
-DateCache::DateCache() = default;
+DateCache::DateCache()
+{
+#if PLATFORM(COCOA)
+    static std::once_flag onceKey;
+    std::call_once(onceKey, [&] {
+        CFNotificationCenterAddObserver(CFNotificationCenterGetLocalCenter(), nullptr, timeZoneChangeNotification, kCFTimeZoneSystemTimeZoneDidChangeNotification, nullptr, CFNotificationSuspensionBehaviorDeliverImmediately);
+    });
+#endif
+}
+
 DateCache::~DateCache() = default;
 
 Ref<DateInstanceData> DateCache::cachedDateInstanceData(double millisecondsFromEpoch)
@@ -313,19 +438,56 @@ Ref<DateInstanceData> DateCache::cachedDateInstanceData(double millisecondsFromE
 
 void DateCache::timeZoneCacheSlow()
 {
-    // Do not use icu::TimeZone::createDefault. ICU internally has a cache for timezone and createDefault returns this cached value.
     ASSERT(!m_timeZoneCache);
-    m_timeZoneCache = std::unique_ptr<OpaqueICUTimeZone, OpaqueICUTimeZoneDeleter>(bitwise_cast<OpaqueICUTimeZone*>(icu::TimeZone::detectHostTimeZone()));
+
+    Vector<UChar, 32> timeZoneID;
+    getTimeZoneOverride(timeZoneID);
+#if HAVE(ICU_C_TIMEZONE_API)
+    auto* cache = new OpaqueICUTimeZone;
+
+    String canonical;
+    UErrorCode status = U_ZERO_ERROR;
+    if (timeZoneID.isEmpty()) {
+        status = callBufferProducingFunction(ucal_getHostTimeZone, timeZoneID);
+        ASSERT_UNUSED(status, U_SUCCESS(status));
+    }
+    if (U_SUCCESS(status)) {
+        Vector<UChar, 32> canonicalBuffer;
+        auto status = callBufferProducingFunction(ucal_getCanonicalTimeZoneID, timeZoneID.data(), timeZoneID.size(), canonicalBuffer, nullptr);
+        if (U_SUCCESS(status))
+            canonical = String(canonicalBuffer);
+    }
+    if (canonical.isNull() || isUTCEquivalent(canonical))
+        canonical = "UTC"_s;
+    cache->m_canonicalTimeZoneID = WTFMove(canonical);
+
+    status = U_ZERO_ERROR;
+    cache->m_calendar = std::unique_ptr<UCalendar, ICUDeleter<ucal_close>>(ucal_open(timeZoneID.data(), timeZoneID.size(), "", UCAL_DEFAULT, &status));
+    ASSERT_UNUSED(status, U_SUCCESS(status));
+    ucal_setGregorianChange(cache->m_calendar.get(), minECMAScriptTime, &status); // Ignore "unsupported" error.
+    m_timeZoneCache = std::unique_ptr<OpaqueICUTimeZone, OpaqueICUTimeZoneDeleter>(cache);
+#else
+    if (!timeZoneID.isEmpty()) {
+        m_timeZoneCache = std::unique_ptr<OpaqueICUTimeZone, OpaqueICUTimeZoneDeleter>(toOpaqueICUTimeZone(icu::TimeZone::createTimeZone(icu::UnicodeString(timeZoneID.data(), timeZoneID.size()))));
+        return;
+    }
+    // Do not use icu::TimeZone::createDefault. ICU internally has a cache for timezone and createDefault returns this cached value.
+    m_timeZoneCache = std::unique_ptr<OpaqueICUTimeZone, OpaqueICUTimeZoneDeleter>(toOpaqueICUTimeZone(icu::TimeZone::detectHostTimeZone()));
+#endif
 }
 
-void DateCache::reset()
+void DateCache::resetIfNecessarySlow()
 {
+    // FIXME: We should clear it only when we know the timezone has been changed on Non-Cocoa platforms.
+    // https://bugs.webkit.org/show_bug.cgi?id=218365
     m_timeZoneCache.reset();
     m_utcTimeOffsetCache = LocalTimeOffsetCache();
     m_localTimeOffsetCache = LocalTimeOffsetCache();
     m_cachedDateString = String();
     m_cachedDateStringValue = std::numeric_limits<double>::quiet_NaN();
     m_dateInstanceCache.reset();
+    m_timeZoneStandardDisplayNameCache = String();
+    m_timeZoneDSTDisplayNameCache = String();
 }
 
 } // namespace JSC
