@@ -106,8 +106,8 @@ void Structure::dumpStatistics()
     unsigned numberLeaf = 0;
     unsigned numberUsingSingleSlot = 0;
     unsigned numberSingletons = 0;
-    unsigned numberWithPropertyMaps = 0;
-    unsigned totalPropertyMapsSize = 0;
+    unsigned numberWithPropertyTables = 0;
+    unsigned totalPropertyTablesSize = 0;
 
     HashSet<Structure*>::const_iterator end = liveStructureSet.end();
     for (HashSet<Structure*>::const_iterator it = liveStructureSet.begin(); it != end; ++it) {
@@ -126,8 +126,8 @@ void Structure::dumpStatistics()
         }
 
         if (PropertyTable* table = structure->propertyTableOrNull()) {
-            ++numberWithPropertyMaps;
-            totalPropertyMapsSize += table->sizeInMemory();
+            ++numberWithPropertyTables;
+            totalPropertyTablesSize += table->sizeInMemory();
         }
     }
 
@@ -135,11 +135,11 @@ void Structure::dumpStatistics()
     dataLogF("Number of Structures using the single item optimization for transition map: %d\n", numberUsingSingleSlot);
     dataLogF("Number of Structures that are leaf nodes: %d\n", numberLeaf);
     dataLogF("Number of Structures that singletons: %d\n", numberSingletons);
-    dataLogF("Number of Structures with PropertyMaps: %d\n", numberWithPropertyMaps);
+    dataLogF("Number of Structures with PropertyTables: %d\n", numberWithPropertyTables);
 
     dataLogF("Size of a single Structures: %d\n", static_cast<unsigned>(sizeof(Structure)));
-    dataLogF("Size of sum of all property maps: %d\n", totalPropertyMapsSize);
-    dataLogF("Size of average of all property maps: %f\n", static_cast<double>(totalPropertyMapsSize) / static_cast<double>(liveStructureSet.size()));
+    dataLogF("Size of sum of all property maps: %d\n", totalPropertyTablesSize);
+    dataLogF("Size of average of all property maps: %f\n", static_cast<double>(totalPropertyTablesSize) / static_cast<double>(liveStructureSet.size()));
 #else
     dataLogF("Dumping Structure statistics is not enabled.\n");
 #endif
@@ -197,15 +197,15 @@ inline void Structure::validateFlags() { }
 
 Structure::Structure(VM& vm, JSGlobalObject* globalObject, JSValue prototype, const TypeInfo& typeInfo, const ClassInfo* classInfo, IndexingType indexingType, unsigned inlineCapacity)
     : JSCell(vm, vm.structureStructure.get())
-    , m_blob(StructureID::encode(this), indexingType, typeInfo)
+    , m_blob(indexingType, typeInfo)
     , m_outOfLineTypeFlags(typeInfo.outOfLineTypeFlags())
     , m_inlineCapacity(inlineCapacity)
     , m_bitField(0)
+    , m_propertyHash(0)
     , m_globalObject(vm, this, globalObject, WriteBarrier<JSGlobalObject>::MayBeNull)
     , m_prototype(vm, this, prototype)
     , m_classInfo(classInfo)
     , m_transitionWatchpointSet(IsWatched)
-    , m_propertyHash(0)
 {
     setDictionaryKind(NoneDictionaryKind);
     setIsPinnedPropertyTable(false);
@@ -225,25 +225,29 @@ Structure::Structure(VM& vm, JSGlobalObject* globalObject, JSValue prototype, co
     setTransitionOffset(vm, invalidOffset);
     setMaxOffset(vm, invalidOffset);
  
-    ASSERT(inlineCapacity <= JSFinalObject::maxInlineCapacity());
+    ASSERT(inlineCapacity <= JSFinalObject::maxInlineCapacity);
     ASSERT(static_cast<PropertyOffset>(inlineCapacity) < firstOutOfLineOffset);
     ASSERT(!hasRareData());
     ASSERT(hasReadOnlyOrGetterSetterPropertiesExcludingProto() == m_classInfo->hasStaticSetterOrReadonlyProperties());
     ASSERT(hasGetterSetterProperties() == m_classInfo->hasStaticSetterOrReadonlyProperties());
 
     validateFlags();
+
+#if ENABLE(STRUCTURE_ID_WITH_SHIFT)
+    ASSERT(WTF::roundUpToMultipleOf<Structure::atomSize>(this) == this);
+#endif
 }
 
-const ClassInfo Structure::s_info = { "Structure", nullptr, nullptr, nullptr, CREATE_METHOD_TABLE(Structure) };
+const ClassInfo Structure::s_info = { "Structure"_s, nullptr, nullptr, nullptr, CREATE_METHOD_TABLE(Structure) };
 
 Structure::Structure(VM& vm, CreatingEarlyCellTag)
     : JSCell(CreatingEarlyCell)
     , m_inlineCapacity(0)
     , m_bitField(0)
+    , m_propertyHash(0)
     , m_prototype(vm, this, jsNull())
     , m_classInfo(info())
     , m_transitionWatchpointSet(IsWatched)
-    , m_propertyHash(0)
 {
     setDictionaryKind(NoneDictionaryKind);
     setIsPinnedPropertyTable(false);
@@ -264,23 +268,27 @@ Structure::Structure(VM& vm, CreatingEarlyCellTag)
     setMaxOffset(vm, invalidOffset);
  
     TypeInfo typeInfo = TypeInfo(StructureType, StructureFlags);
-    m_blob = StructureIDBlob(StructureID::encode(this), 0, typeInfo);
+    m_blob = TypeInfoBlob(0, typeInfo);
     m_outOfLineTypeFlags = typeInfo.outOfLineTypeFlags();
 
     ASSERT(hasReadOnlyOrGetterSetterPropertiesExcludingProto() || !m_classInfo->hasStaticSetterOrReadonlyProperties());
     ASSERT(hasGetterSetterProperties() || !m_classInfo->hasStaticSetterOrReadonlyProperties());
     ASSERT(!this->typeInfo().overridesGetCallData() || m_classInfo->methodTable.getCallData != &JSCell::getCallData);
+
+#if ENABLE(STRUCTURE_ID_WITH_SHIFT)
+    ASSERT(WTF::roundUpToMultipleOf<Structure::atomSize>(this) == this);
+#endif
 }
 
-Structure::Structure(VM& vm, Structure* previous, DeferredStructureTransitionWatchpointFire* deferred)
+Structure::Structure(VM& vm, Structure* previous)
     : JSCell(vm, vm.structureStructure.get())
     , m_inlineCapacity(previous->m_inlineCapacity)
     , m_bitField(0)
+    , m_propertyHash(previous->m_propertyHash)
+    , m_seenProperties(previous->m_seenProperties)
     , m_prototype(vm, this, previous->m_prototype.get())
     , m_classInfo(previous->m_classInfo)
     , m_transitionWatchpointSet(IsWatched)
-    , m_propertyHash(previous->m_propertyHash)
-    , m_seenProperties(previous->m_seenProperties)
 {
     setDictionaryKind(previous->dictionaryKind());
     setIsPinnedPropertyTable(false);
@@ -301,13 +309,15 @@ Structure::Structure(VM& vm, Structure* previous, DeferredStructureTransitionWat
     setMaxOffset(vm, invalidOffset);
  
     TypeInfo typeInfo = previous->typeInfo();
-    m_blob = StructureIDBlob(StructureID::encode(this), previous->indexingModeIncludingHistory(), typeInfo);
+    m_blob = TypeInfoBlob(previous->indexingModeIncludingHistory(), typeInfo);
     m_outOfLineTypeFlags = typeInfo.outOfLineTypeFlags();
 
     ASSERT(!previous->typeInfo().structureIsImmortal());
     setPreviousID(vm, previous);
 
-    previous->didTransitionFromThisStructure(deferred);
+    // Do not fire watchpoint inside Structure constructor since watchpoint can involve further heap allocations.
+    // We fire watchpoint separately in Structure::finishCreation.
+    previous->didTransitionFromThisStructureWithoutFiringWatchpoint();
     
     // Copy this bit now, in case previous was being watched.
     setTransitionWatchpointIsLikelyToBeFired(previous->transitionWatchpointIsLikelyToBeFired());
@@ -317,6 +327,10 @@ Structure::Structure(VM& vm, Structure* previous, DeferredStructureTransitionWat
     ASSERT(hasReadOnlyOrGetterSetterPropertiesExcludingProto() || !m_classInfo->hasStaticSetterOrReadonlyProperties());
     ASSERT(hasGetterSetterProperties() || !m_classInfo->hasStaticSetterOrReadonlyProperties());
     ASSERT(!this->typeInfo().overridesGetCallData() || m_classInfo->methodTable.getCallData != &JSCell::getCallData);
+
+#if ENABLE(STRUCTURE_ID_WITH_SHIFT)
+    ASSERT(WTF::roundUpToMultipleOf<Structure::atomSize>(this) == this);
+#endif
 }
 
 Structure::~Structure()
@@ -383,7 +397,7 @@ bool Structure::findStructuresAndMapForMaterialization(Vector<Structure*, 8>& st
 
 PropertyTable* Structure::materializePropertyTable(VM& vm, bool setPropertyTable)
 {
-    ASSERT(structure(vm)->classInfo() == info());
+    ASSERT(structure()->classInfoForCells() == info());
     ASSERT(!protectPropertyTableWhileTransitioning());
     
     DeferGC deferGC(vm);
@@ -414,25 +428,25 @@ PropertyTable* Structure::materializePropertyTable(VM& vm, bool setPropertyTable
             continue;
         switch (structure->transitionKind()) {
         case TransitionKind::PropertyAddition: {
-            PropertyMapEntry entry(structure->m_transitionPropertyName.get(), structure->transitionOffset(), structure->transitionPropertyAttributes());
+            PropertyTableEntry entry(structure->m_transitionPropertyName.get(), structure->transitionOffset(), structure->transitionPropertyAttributes());
             auto nextOffset = table->nextOffset(structure->inlineCapacity());
             ASSERT_UNUSED(nextOffset, nextOffset == structure->transitionOffset());
-            auto result = table->add(vm, entry);
-            ASSERT_UNUSED(result, result.second);
-            ASSERT_UNUSED(result, result.first.first->offset == nextOffset);
+            auto [offset, attribute, result] = table->add(vm, entry);
+            ASSERT_UNUSED(result, result);
+            ASSERT_UNUSED(offset, offset == nextOffset);
+            UNUSED_VARIABLE(attribute);
             break;
         }
         case TransitionKind::PropertyDeletion: {
-            auto item = table->find(structure->m_transitionPropertyName.get());
-            ASSERT(item.first);
-            table->remove(vm, item);
+            auto [offset, attributes] = table->take(vm, structure->m_transitionPropertyName.get());
+            ASSERT_UNUSED(offset, offset != invalidOffset);
+            UNUSED_VARIABLE(attributes);
             table->addDeletedOffset(structure->transitionOffset());
             break;
         }
         case TransitionKind::PropertyAttributeChange: {
-            PropertyMapEntry* entry = table->get(structure->m_transitionPropertyName.get());
-            entry->attributes = structure->transitionPropertyAttributes();
-            ASSERT(entry->offset == structure->transitionOffset());
+            PropertyOffset offset = table->updateAttributeIfExists(structure->m_transitionPropertyName.get(), structure->transitionPropertyAttributes());
+            ASSERT_UNUSED(offset, offset == structure->transitionOffset());
             break;
         }
         default:
@@ -456,9 +470,9 @@ PropertyTable* Structure::materializePropertyTable(VM& vm, bool setPropertyTable
     return table;
 }
 
-bool Structure::holesMustForwardToPrototype(VM& vm, JSObject* base) const
+bool Structure::holesMustForwardToPrototype(JSObject* base) const
 {
-    ASSERT(base->structure(vm) == this);
+    ASSERT(base->structure() == this);
 
     if (this->mayInterceptIndexedAccesses())
         return true;
@@ -469,7 +483,7 @@ bool Structure::holesMustForwardToPrototype(VM& vm, JSObject* base) const
     JSObject* object = asObject(prototype);
 
     while (true) {
-        Structure& structure = *object->structure(vm);
+        Structure& structure = *object->structure();
         if (hasIndexedProperties(object->indexingType()) || structure.mayInterceptIndexedAccesses())
             return true;
         prototype = structure.storedPrototype(object);
@@ -829,18 +843,18 @@ Structure* Structure::nonPropertyTransitionSlow(VM& vm, Structure* structure, Tr
         // table, since our logic for walking the property transition chain to rematerialize the
         // table doesn't know how to take into account such wholesale edits.
 
+        ASSERT(transitionKind == TransitionKind::Seal || transitionKind == TransitionKind::Freeze);
+
         PropertyTable* table = structure->copyPropertyTableForPinning(vm);
         transition->pinForCaching(Locker { transition->m_lock }, vm, table);
         transition->setMaxOffset(vm, structure->maxOffset());
         
         table = transition->propertyTableOrNull();
         RELEASE_ASSERT(table);
-        for (auto& entry : *table) {
-            if (setsDontDeleteOnAllProperties(transitionKind))
-                entry.attributes |= static_cast<unsigned>(PropertyAttribute::DontDelete);
-            if (setsReadOnlyOnNonAccessorProperties(transitionKind) && !(entry.attributes & PropertyAttribute::Accessor))
-                entry.attributes |= static_cast<unsigned>(PropertyAttribute::ReadOnly);
-        }
+        if (transitionKind == TransitionKind::Seal)
+            table->seal();
+        else
+            table->freeze();
     } else {
         transition->setPropertyTable(vm, structure->takePropertyTableOrCloneIfPinned(vm));
         transition->setMaxOffset(vm, structure->maxOffset());
@@ -872,13 +886,7 @@ bool Structure::isSealed(VM& vm)
     PropertyTable* table = ensurePropertyTableIfNotEmpty(vm);
     if (!table)
         return true;
-    
-    PropertyTable::iterator end = table->end();
-    for (PropertyTable::iterator iter = table->begin(); iter != end; ++iter) {
-        if ((iter->attributes & PropertyAttribute::DontDelete) != static_cast<unsigned>(PropertyAttribute::DontDelete))
-            return false;
-    }
-    return true;
+    return table->isSealed();
 }
 
 // In future we may want to cache this property.
@@ -890,22 +898,14 @@ bool Structure::isFrozen(VM& vm)
     PropertyTable* table = ensurePropertyTableIfNotEmpty(vm);
     if (!table)
         return true;
-    
-    PropertyTable::iterator end = table->end();
-    for (PropertyTable::iterator iter = table->begin(); iter != end; ++iter) {
-        if (!(iter->attributes & PropertyAttribute::DontDelete))
-            return false;
-        if (!(iter->attributes & (PropertyAttribute::ReadOnly | PropertyAttribute::Accessor)))
-            return false;
-    }
-    return true;
+    return table->isFrozen();
 }
 
 Structure* Structure::flattenDictionaryStructure(VM& vm, JSObject* object)
 {
     checkOffsetConsistency();
     ASSERT(isDictionary());
-    ASSERT(object->structure(vm) == this);
+    ASSERT(object->structure() == this);
     
     GCSafeConcurrentJSLocker locker(m_lock, vm);
     
@@ -919,25 +919,17 @@ Structure* Structure::flattenDictionaryStructure(VM& vm, JSObject* object)
 
         size_t propertyCount = table->size();
 
-        // Holds our values compacted by insertion order.
+        // Holds our values compacted by insertion order. This is OK since GC is deferred.
         Vector<JSValue> values(propertyCount);
 
         // Copies out our values from their hashed locations, compacting property table offsets as we go.
-        unsigned i = 0;
-        PropertyTable::iterator end = table->end();
-        auto offset = invalidOffset;
-        for (PropertyTable::iterator iter = table->begin(); iter != end; ++iter, ++i) {
-            values[i] = object->getDirect(iter->offset);
-            offset = iter->offset = offsetForPropertyNumber(i, m_inlineCapacity);
-        }
+        PropertyOffset offset = table->renumberPropertyOffsets(object, m_inlineCapacity, values);
         setMaxOffset(vm, offset);
         ASSERT(transitionOffset() == invalidOffset);
         
         // Copies in our values to their compacted locations.
         for (unsigned i = 0; i < propertyCount; i++)
-            object->putDirect(vm, offsetForPropertyNumber(i, m_inlineCapacity), values[i]);
-
-        table->clearDeletedOffsets();
+            object->putDirectOffset(vm, offsetForPropertyNumber(i, m_inlineCapacity), values[i]);
 
         // We need to zero our unused property space; otherwise the GC might see a
         // stale pointer when we add properties in the future.
@@ -1065,7 +1057,7 @@ PropertyTableStatisticsExitLogger::~PropertyTableStatisticsExitLogger()
 {
     unsigned finds = propertyTableStats->numFinds;
     unsigned collisions = propertyTableStats->numCollisions;
-    dataLogF("\nJSC::PropertyMap statistics for process %d\n\n", getCurrentProcessID());
+    dataLogF("\nJSC::PropertyTable statistics for process %d\n\n", getCurrentProcessID());
     dataLogF("%d finds\n", finds);
     dataLogF("%d collisions (%.1f%%)\n", collisions, 100.0 * collisions / finds);
     dataLogF("%d lookups\n", propertyTableStats->numLookups.load());
@@ -1135,9 +1127,10 @@ PropertyOffset Structure::getConcurrently(UniquedStringImpl* uid, unsigned& attr
         assertIsHeld(tableStructure->m_lock); // Sadly Clang needs some help here.
         // Because uid is UniquedStringImpl, it is guaranteed that the hash is already computed.
         // So we can use PropertyTable::get even from the concurrent compilers.
-        if (auto* entry = table->get(uid)) {
-            result = entry->offset;
-            attributes = entry->attributes;
+        auto [offset, entryAttributes] = table->get(uid);
+        if (offset != invalidOffset) {
+            result = offset;
+            attributes = entryAttributes;
         }
         tableStructure->m_lock.unlock();
     }
@@ -1145,12 +1138,12 @@ PropertyOffset Structure::getConcurrently(UniquedStringImpl* uid, unsigned& attr
     return result;
 }
 
-Vector<PropertyMapEntry> Structure::getPropertiesConcurrently()
+Vector<PropertyTableEntry> Structure::getPropertiesConcurrently()
 {
-    Vector<PropertyMapEntry> result;
+    Vector<PropertyTableEntry> result;
 
     forEachPropertyConcurrently(
-        [&] (const PropertyMapEntry& entry) -> bool {
+        [&] (const PropertyTableEntry& entry) -> bool {
             result.append(entry);
             return true;
         });
@@ -1192,34 +1185,35 @@ void Structure::getPropertyNamesFromStructure(VM& vm, PropertyNameArray& propert
     bool knownUnique = propertyNames.canAddKnownUniqueForStructure();
     bool foundSymbol = false;
 
-    auto checkDontEnumAndAdd = [&](PropertyTable::iterator iter) {
-        if (mode == DontEnumPropertiesMode::Include || !(iter->attributes & PropertyAttribute::DontEnum)) {
+    auto checkDontEnumAndAdd = [&](const auto& entry) {
+        if (mode == DontEnumPropertiesMode::Include || !(entry.attributes() & PropertyAttribute::DontEnum)) {
             if (knownUnique)
-                propertyNames.addUnchecked(iter->key);
+                propertyNames.addUnchecked(entry.key());
             else
-                propertyNames.add(iter->key);
+                propertyNames.add(entry.key());
         }
     };
     
-    PropertyTable::iterator end = table->end();
-    for (PropertyTable::iterator iter = table->begin(); iter != end; ++iter) {
-        ASSERT(!isQuickPropertyAccessAllowedForEnumeration() || !(iter->attributes & PropertyAttribute::DontEnum));
-        ASSERT(!isQuickPropertyAccessAllowedForEnumeration() || !iter->key->isSymbol());
-        if (iter->key->isSymbol()) {
+    table->forEachProperty([&](const auto& entry) {
+        ASSERT(!isQuickPropertyAccessAllowedForEnumeration() || !(entry.attributes() & PropertyAttribute::DontEnum));
+        ASSERT(!isQuickPropertyAccessAllowedForEnumeration() || !entry.key()->isSymbol());
+        if (entry.key()->isSymbol()) {
             foundSymbol = true;
             if (propertyNames.propertyNameMode() != PropertyNameMode::Symbols)
-                continue;
+                return IterationStatus::Continue;
         }
-        checkDontEnumAndAdd(iter);
-    }
+        checkDontEnumAndAdd(entry);
+        return IterationStatus::Continue;
+    });
 
     if (foundSymbol && propertyNames.propertyNameMode() == PropertyNameMode::StringsAndSymbols) {
         // To ensure the order defined in the spec, we append symbols at the last elements of keys.
         // https://tc39.es/ecma262/#sec-ordinaryownpropertykeys
-        for (PropertyTable::iterator iter = table->begin(); iter != end; ++iter) {
-            if (iter->key->isSymbol())
-                checkDontEnumAndAdd(iter);
-        }
+        table->forEachProperty([&](const auto& entry) {
+            if (entry.key()->isSymbol())
+                checkDontEnumAndAdd(entry);
+            return IterationStatus::Continue;
+        });
     }
 }
 
@@ -1244,14 +1238,17 @@ void DeferredStructureTransitionWatchpointFire::dump(PrintStream& out) const
     out.print("Structure transition from ", *m_structure);
 }
 
-void Structure::didTransitionFromThisStructure(DeferredStructureTransitionWatchpointFire* deferred) const
+void Structure::didTransitionFromThisStructureWithoutFiringWatchpoint() const
 {
     // If the structure is being watched, and this is the kind of structure that the DFG would
     // like to watch, then make sure to note for all future versions of this structure that it's
     // unwise to watch it.
     if (m_transitionWatchpointSet.isBeingWatched())
         const_cast<Structure*>(this)->setTransitionWatchpointIsLikelyToBeFired(true);
+}
 
+void Structure::fireStructureTransitionWatchpoint(DeferredStructureTransitionWatchpointFire* deferred) const
+{
     if (deferred) {
         ASSERT(deferred->structure() == this);
         m_transitionWatchpointSet.fireAll(vm(), deferred);
@@ -1259,6 +1256,12 @@ void Structure::didTransitionFromThisStructure(DeferredStructureTransitionWatchp
         m_transitionWatchpointSet.fireAll(vm(), StructureFireDetail(this));
 }
 
+void Structure::didTransitionFromThisStructure(DeferredStructureTransitionWatchpointFire* deferred) const
+{
+    didTransitionFromThisStructureWithoutFiringWatchpoint();
+    fireStructureTransitionWatchpoint(deferred);
+}
+
 template<typename Visitor>
 void Structure::visitChildrenImpl(JSCell* cell, Visitor& visitor)
 {
@@ -1333,16 +1336,16 @@ Ref<StructureShape> Structure::toStructureShape(JSValue value, bool& sawPolyProt
     while (curStructure) {
         sawPolyProtoStructure |= curStructure->hasPolyProto();
         curStructure->forEachPropertyConcurrently(
-            [&] (const PropertyMapEntry& entry) -> bool {
-                if (!PropertyName(entry.key).isPrivateName())
-                    curShape->addProperty(*entry.key);
+            [&] (const PropertyTableEntry& entry) -> bool {
+                if (!PropertyName(entry.key()).isPrivateName())
+                    curShape->addProperty(*entry.key());
                 return true;
             });
 
         if (JSObject* curObject = curValue.getObject())
             curShape->setConstructorName(JSObject::calculatedClassName(curObject));
         else
-            curShape->setConstructorName(curStructure->classInfo()->className);
+            curShape->setConstructorName(curStructure->classInfoForCells()->className);
 
         if (curStructure->isDictionary())
             curShape->enterDictionaryMode();
@@ -1372,14 +1375,14 @@ void Structure::dump(PrintStream& out) const
     auto* structureID = reinterpret_cast<void*>(id().bits());
     out.print(RawPointer(this), ":[", RawPointer(structureID),
         "/", (uint32_t)(reinterpret_cast<uintptr_t>(structureID)), ", ",
-        classInfo()->className, ", (", inlineSize(), "/", inlineCapacity(), ", ",
+        classInfoForCells()->className, ", (", inlineSize(), "/", inlineCapacity(), ", ",
         outOfLineSize(), "/", outOfLineCapacity(), "){");
 
     CommaPrinter comma;
     
     const_cast<Structure*>(this)->forEachPropertyConcurrently(
-        [&] (const PropertyMapEntry& entry) -> bool {
-            out.print(comma, entry.key, ":", static_cast<int>(entry.offset));
+        [&] (const PropertyTableEntry& entry) -> bool {
+            out.print(comma, entry.key(), ":", static_cast<int>(entry.offset()));
             return true;
         });
     
@@ -1424,7 +1427,7 @@ void Structure::dumpInContext(PrintStream& out, DumpContext* context) const
 
 void Structure::dumpBrief(PrintStream& out, const CString& string) const
 {
-    out.print("%", string, ":", classInfo()->className);
+    out.print("%", string, ":", classInfoForCells()->className);
     if (indexingType() & IndexingShapeMask)
         out.print(",", IndexingTypeDump(indexingType()));
 }
@@ -1508,7 +1511,7 @@ bool Structure::canAccessPropertiesQuicklyForEnumeration() const
 
 auto Structure::findPropertyHashEntry(PropertyName propertyName) const -> std::optional<PropertyHashEntry>
 {
-    for (const ClassInfo* info = classInfo(); info; info = info->parentClass) {
+    for (const ClassInfo* info = classInfoForCells(); info; info = info->parentClass) {
         if (const HashTable* propHashTable = info->staticPropHashTable) {
             if (const HashTableValue* entry = propHashTable->entry(propertyName))
                 return PropertyHashEntry { propHashTable, entry };
