@@ -32,9 +32,7 @@
 #include "YarrCanonicalize.h"
 #include "YarrParser.h"
 #include <wtf/DataLog.h>
-#include <wtf/Optional.h>
-#include <wtf/StackPointer.h>
-#include <wtf/Threading.h>
+#include <wtf/StackCheck.h>
 #include <wtf/Vector.h>
 
 namespace JSC { namespace Yarr {
@@ -436,10 +434,9 @@ private:
 
 class YarrPatternConstructor {
 public:
-    YarrPatternConstructor(YarrPattern& pattern, void* stackLimit)
+    YarrPatternConstructor(YarrPattern& pattern)
         : m_pattern(pattern)
         , m_characterClassConstructor(pattern.ignoreCase(), pattern.unicode() ? CanonicalMode::Unicode : CanonicalMode::UCS2)
-        , m_stackLimit(stackLimit)
     {
         auto body = makeUnique<PatternDisjunction>();
         m_pattern.m_body = body.get();
@@ -462,16 +459,6 @@ public:
         m_pattern.m_disjunctions.append(WTFMove(body));
     }
 
-    void saveUnmatchedNamedForwardReferences()
-    {
-        m_unmatchedNamedForwardReferences.shrink(0);
-        
-        for (auto& entry : m_pattern.m_namedForwardReferences) {
-            if (!m_pattern.m_captureGroupNames.contains(entry))
-                m_unmatchedNamedForwardReferences.append(entry);
-        }
-    }
-
     void assertionBOL()
     {
         if (!m_alternative->m_terms.size() && !m_invertParentheticalAssertion) {
@@ -594,7 +581,7 @@ public:
         m_pattern.m_userCharacterClasses.append(WTFMove(newCharacterClass));
     }
 
-    void atomParenthesesSubpatternBegin(bool capture = true, Optional<String> optGroupName = WTF::nullopt)
+    void atomParenthesesSubpatternBegin(bool capture = true, std::optional<String> optGroupName = std::nullopt)
     {
         unsigned subpatternId = m_pattern.m_numSubpatterns + 1;
         if (capture) {
@@ -609,7 +596,7 @@ public:
             ASSERT(!optGroupName);
 
         auto parenthesesDisjunction = makeUnique<PatternDisjunction>(m_alternative);
-        m_alternative->m_terms.append(PatternTerm(PatternTerm::TypeParenthesesSubpattern, subpatternId, parenthesesDisjunction.get(), capture, false));
+        m_alternative->m_terms.append(PatternTerm(PatternTerm::Type::ParenthesesSubpattern, subpatternId, parenthesesDisjunction.get(), capture, false));
         m_alternative = parenthesesDisjunction->addNewAlternative();
         m_pattern.m_disjunctions.append(WTFMove(parenthesesDisjunction));
     }
@@ -617,7 +604,7 @@ public:
     void atomParentheticalAssertionBegin(bool invert = false)
     {
         auto parenthesesDisjunction = makeUnique<PatternDisjunction>(m_alternative);
-        m_alternative->m_terms.append(PatternTerm(PatternTerm::TypeParentheticalAssertion, m_pattern.m_numSubpatterns + 1, parenthesesDisjunction.get(), false, invert));
+        m_alternative->m_terms.append(PatternTerm(PatternTerm::Type::ParentheticalAssertion, m_pattern.m_numSubpatterns + 1, parenthesesDisjunction.get(), false, invert));
         m_alternative = parenthesesDisjunction->addNewAlternative();
         m_invertParentheticalAssertion = invert;
         m_pattern.m_disjunctions.append(WTFMove(parenthesesDisjunction));
@@ -657,7 +644,6 @@ public:
     {
         ASSERT(subpatternId);
         m_pattern.m_containsBackreferences = true;
-        m_pattern.m_maxBackReference = std::max(m_pattern.m_maxBackReference, subpatternId);
 
         if (subpatternId > m_pattern.m_numSubpatterns) {
             m_alternative->m_terms.append(PatternTerm::ForwardReference());
@@ -670,9 +656,9 @@ public:
         // Note to self: if we waited until the AST was baked, we could also remove forwards refs 
         while ((currentAlternative = currentAlternative->m_parent->m_parent)) {
             PatternTerm& term = currentAlternative->lastTerm();
-            ASSERT((term.type == PatternTerm::TypeParenthesesSubpattern) || (term.type == PatternTerm::TypeParentheticalAssertion));
+            ASSERT((term.type == PatternTerm::Type::ParenthesesSubpattern) || (term.type == PatternTerm::Type::ParentheticalAssertion));
 
-            if ((term.type == PatternTerm::TypeParenthesesSubpattern) && term.capture() && (subpatternId == term.parentheses.subpatternId)) {
+            if ((term.type == PatternTerm::Type::ParenthesesSubpattern) && term.capture() && (subpatternId == term.parentheses.subpatternId)) {
                 m_alternative->m_terms.append(PatternTerm::ForwardReference());
                 return;
             }
@@ -687,14 +673,8 @@ public:
         atomBackReference(m_pattern.m_namedGroupToParenIndex.get(subpatternName));
     }
 
-    bool isValidNamedForwardReference(const String& subpatternName)
+    void atomNamedForwardReference(const String&)
     {
-        return !m_unmatchedNamedForwardReferences.contains(subpatternName);
-    }
-
-    void atomNamedForwardReference(const String& subpatternName)
-    {
-        m_pattern.m_namedForwardReferences.appendIfNotContains(subpatternName);
         m_alternative->m_terms.append(PatternTerm::ForwardReference());
     }
     
@@ -704,7 +684,7 @@ public:
     {
         if (UNLIKELY(!isSafeToRecurse())) {
             m_error = ErrorCode::PatternTooLarge;
-            return 0;
+            return nullptr;
         }
 
         std::unique_ptr<PatternDisjunction> newDisjunction;
@@ -723,12 +703,12 @@ public:
         }
         
         if (hasError(error())) {
-            newDisjunction = 0;
-            return 0;
+            newDisjunction = nullptr;
+            return nullptr;
         }
 
         if (!newDisjunction)
-            return 0;
+            return nullptr;
 
         PatternDisjunction* copiedDisjunction = newDisjunction.get();
         m_pattern.m_disjunctions.append(WTFMove(newDisjunction));
@@ -742,7 +722,7 @@ public:
             return PatternTerm(term);
         }
 
-        if ((term.type != PatternTerm::TypeParenthesesSubpattern) && (term.type != PatternTerm::TypeParentheticalAssertion))
+        if ((term.type != PatternTerm::Type::ParenthesesSubpattern) && (term.type != PatternTerm::Type::ParentheticalAssertion))
             return PatternTerm(term);
         
         PatternTerm termCopy = term;
@@ -762,10 +742,10 @@ public:
         }
 
         PatternTerm& term = m_alternative->lastTerm();
-        ASSERT(term.type > PatternTerm::TypeAssertionWordBoundary);
-        ASSERT(term.quantityMinCount == 1 && term.quantityMaxCount == 1 && term.quantityType == QuantifierFixedCount);
+        ASSERT(term.type > PatternTerm::Type::AssertionWordBoundary);
+        ASSERT(term.quantityMinCount == 1 && term.quantityMaxCount == 1 && term.quantityType == QuantifierType::FixedCount);
 
-        if (term.type == PatternTerm::TypeParentheticalAssertion) {
+        if (term.type == PatternTerm::Type::ParentheticalAssertion) {
             // If an assertion is quantified with a minimum count of zero, it can simply be removed.
             // This arises from the RepeatMatcher behaviour in the spec. Matching an assertion never
             // results in any input being consumed, however the continuation passed to the assertion
@@ -786,15 +766,15 @@ public:
         }
 
         if (min == max)
-            term.quantify(min, max, QuantifierFixedCount);
-        else if (!min || (term.type == PatternTerm::TypeParenthesesSubpattern && m_pattern.m_hasCopiedParenSubexpressions))
-            term.quantify(min, max, greedy ? QuantifierGreedy : QuantifierNonGreedy);
+            term.quantify(min, max, QuantifierType::FixedCount);
+        else if (!min || (term.type == PatternTerm::Type::ParenthesesSubpattern && m_pattern.m_hasCopiedParenSubexpressions))
+            term.quantify(min, max, greedy ? QuantifierType::Greedy : QuantifierType::NonGreedy);
         else {
-            term.quantify(min, min, QuantifierFixedCount);
+            term.quantify(min, min, QuantifierType::FixedCount);
             m_alternative->m_terms.append(copyTerm(term));
             // NOTE: this term is interesting from an analysis perspective, in that it can be ignored.....
-            m_alternative->lastTerm().quantify((max == quantifyInfinite) ? max : max - min, greedy ? QuantifierGreedy : QuantifierNonGreedy);
-            if (m_alternative->lastTerm().type == PatternTerm::TypeParenthesesSubpattern)
+            m_alternative->lastTerm().quantify((max == quantifyInfinite) ? max : max - min, greedy ? QuantifierType::Greedy : QuantifierType::NonGreedy);
+            if (m_alternative->lastTerm().type == PatternTerm::Type::ParenthesesSubpattern)
                 m_alternative->lastTerm().parentheses.isCopy = true;
         }
     }
@@ -811,36 +791,36 @@ public:
 
         ErrorCode error = ErrorCode::NoError;
         alternative->m_hasFixedSize = true;
-        Checked<unsigned, RecordOverflow> currentInputPosition = initialInputPosition;
+        CheckedUint32 currentInputPosition = initialInputPosition;
 
         for (unsigned i = 0; i < alternative->m_terms.size(); ++i) {
             PatternTerm& term = alternative->m_terms[i];
 
             switch (term.type) {
-            case PatternTerm::TypeAssertionBOL:
-            case PatternTerm::TypeAssertionEOL:
-            case PatternTerm::TypeAssertionWordBoundary:
-                term.inputPosition = currentInputPosition.unsafeGet();
+            case PatternTerm::Type::AssertionBOL:
+            case PatternTerm::Type::AssertionEOL:
+            case PatternTerm::Type::AssertionWordBoundary:
+                term.inputPosition = currentInputPosition;
                 break;
 
-            case PatternTerm::TypeBackReference:
-                term.inputPosition = currentInputPosition.unsafeGet();
+            case PatternTerm::Type::BackReference:
+                term.inputPosition = currentInputPosition;
                 term.frameLocation = currentCallFrameSize;
                 currentCallFrameSize += YarrStackSpaceForBackTrackInfoBackReference;
                 alternative->m_hasFixedSize = false;
                 break;
 
-            case PatternTerm::TypeForwardReference:
+            case PatternTerm::Type::ForwardReference:
                 break;
 
-            case PatternTerm::TypePatternCharacter:
-                term.inputPosition = currentInputPosition.unsafeGet();
-                if (term.quantityType != QuantifierFixedCount) {
+            case PatternTerm::Type::PatternCharacter:
+                term.inputPosition = currentInputPosition;
+                if (term.quantityType != QuantifierType::FixedCount) {
                     term.frameLocation = currentCallFrameSize;
                     currentCallFrameSize += YarrStackSpaceForBackTrackInfoPatternCharacter;
                     alternative->m_hasFixedSize = false;
                 } else if (m_pattern.unicode()) {
-                    Checked<unsigned, RecordOverflow> tempCount = term.quantityMaxCount;
+                    CheckedUint32 tempCount = term.quantityMaxCount;
                     tempCount *= U16_LENGTH(term.patternCharacter);
                     if (tempCount.hasOverflowed())
                         return ErrorCode::OffsetTooLarge;
@@ -849,9 +829,9 @@ public:
                     currentInputPosition += term.quantityMaxCount;
                 break;
 
-            case PatternTerm::TypeCharacterClass:
-                term.inputPosition = currentInputPosition.unsafeGet();
-                if (term.quantityType != QuantifierFixedCount) {
+            case PatternTerm::Type::CharacterClass:
+                term.inputPosition = currentInputPosition;
+                if (term.quantityType != QuantifierType::FixedCount) {
                     term.frameLocation = currentCallFrameSize;
                     currentCallFrameSize += YarrStackSpaceForBackTrackInfoCharacterClass;
                     alternative->m_hasFixedSize = false;
@@ -859,7 +839,7 @@ public:
                     term.frameLocation = currentCallFrameSize;
                     currentCallFrameSize += YarrStackSpaceForBackTrackInfoCharacterClass;
                     if (term.characterClass->hasOneCharacterSize() && !term.invert()) {
-                        Checked<unsigned, RecordOverflow> tempCount = term.quantityMaxCount;
+                        CheckedUint32 tempCount = term.quantityMaxCount;
                         tempCount *= term.characterClass->hasNonBMPCharacters() ? 2 : 1;
                         if (tempCount.hasOverflowed())
                             return ErrorCode::OffsetTooLarge;
@@ -872,28 +852,28 @@ public:
                     currentInputPosition += term.quantityMaxCount;
                 break;
 
-            case PatternTerm::TypeParenthesesSubpattern:
+            case PatternTerm::Type::ParenthesesSubpattern:
                 // Note: for fixed once parentheses we will ensure at least the minimum is available; others are on their own.
                 term.frameLocation = currentCallFrameSize;
                 if (term.quantityMaxCount == 1 && !term.parentheses.isCopy) {
                     currentCallFrameSize += YarrStackSpaceForBackTrackInfoParenthesesOnce;
-                    error = setupDisjunctionOffsets(term.parentheses.disjunction, currentCallFrameSize, currentInputPosition.unsafeGet(), currentCallFrameSize);
+                    error = setupDisjunctionOffsets(term.parentheses.disjunction, currentCallFrameSize, currentInputPosition, currentCallFrameSize);
                     if (hasError(error))
                         return error;
                     // If quantity is fixed, then pre-check its minimum size.
-                    if (term.quantityType == QuantifierFixedCount)
+                    if (term.quantityType == QuantifierType::FixedCount)
                         currentInputPosition += term.parentheses.disjunction->m_minimumSize;
-                    term.inputPosition = currentInputPosition.unsafeGet();
+                    term.inputPosition = currentInputPosition;
                 } else if (term.parentheses.isTerminal) {
                     currentCallFrameSize += YarrStackSpaceForBackTrackInfoParenthesesTerminal;
-                    error = setupDisjunctionOffsets(term.parentheses.disjunction, currentCallFrameSize, currentInputPosition.unsafeGet(), currentCallFrameSize);
+                    error = setupDisjunctionOffsets(term.parentheses.disjunction, currentCallFrameSize, currentInputPosition, currentCallFrameSize);
                     if (hasError(error))
                         return error;
-                    term.inputPosition = currentInputPosition.unsafeGet();
+                    term.inputPosition = currentInputPosition;
                 } else {
-                    term.inputPosition = currentInputPosition.unsafeGet();
+                    term.inputPosition = currentInputPosition;
                     currentCallFrameSize += YarrStackSpaceForBackTrackInfoParentheses;
-                    error = setupDisjunctionOffsets(term.parentheses.disjunction, currentCallFrameSize, currentInputPosition.unsafeGet(), currentCallFrameSize);
+                    error = setupDisjunctionOffsets(term.parentheses.disjunction, currentCallFrameSize, currentInputPosition, currentCallFrameSize);
                     if (hasError(error))
                         return error;
                 }
@@ -901,15 +881,15 @@ public:
                 alternative->m_hasFixedSize = false;
                 break;
 
-            case PatternTerm::TypeParentheticalAssertion:
-                term.inputPosition = currentInputPosition.unsafeGet();
+            case PatternTerm::Type::ParentheticalAssertion:
+                term.inputPosition = currentInputPosition;
                 term.frameLocation = currentCallFrameSize;
-                error = setupDisjunctionOffsets(term.parentheses.disjunction, currentCallFrameSize + YarrStackSpaceForBackTrackInfoParentheticalAssertion, currentInputPosition.unsafeGet(), currentCallFrameSize);
+                error = setupDisjunctionOffsets(term.parentheses.disjunction, currentCallFrameSize + YarrStackSpaceForBackTrackInfoParentheticalAssertion, currentInputPosition, currentCallFrameSize);
                 if (hasError(error))
                     return error;
                 break;
 
-            case PatternTerm::TypeDotStarEnclosure:
+            case PatternTerm::Type::DotStarEnclosure:
                 ASSERT(!m_pattern.m_saveInitialStartValue);
                 alternative->m_hasFixedSize = false;
                 term.inputPosition = initialInputPosition;
@@ -922,7 +902,7 @@ public:
                 return ErrorCode::OffsetTooLarge;
         }
 
-        alternative->m_minimumSize = (currentInputPosition - initialInputPosition).unsafeGet();
+        alternative->m_minimumSize = currentInputPosition - initialInputPosition;
         newCallFrameSize = currentCallFrameSize;
         return error;
     }
@@ -953,7 +933,6 @@ public:
                 m_pattern.m_containsUnsignedLengthPattern = true;
         }
         
-        ASSERT(minimumInputSize != UINT_MAX);
         ASSERT(maximumCallFrameSize >= initialCallFrameSize);
 
         disjunction->m_hasFixedSize = hasFixedSize;
@@ -989,8 +968,8 @@ public:
             Vector<PatternTerm>& terms = alternatives[i]->m_terms;
             if (terms.size()) {
                 PatternTerm& term = terms.last();
-                if (term.type == PatternTerm::TypeParenthesesSubpattern
-                    && term.quantityType == QuantifierGreedy
+                if (term.type == PatternTerm::Type::ParenthesesSubpattern
+                    && term.quantityType == QuantifierType::Greedy
                     && term.quantityMinCount == 0
                     && term.quantityMaxCount == quantifyInfinite
                     && !term.capture())
@@ -1037,7 +1016,7 @@ public:
             if (term.m_capture)
                 return true;
 
-            if (term.type == PatternTerm::TypeParenthesesSubpattern) {
+            if (term.type == PatternTerm::Type::ParenthesesSubpattern) {
                 PatternDisjunction* nestedDisjunction = term.parentheses.disjunction;
                 for (unsigned alt = 0; alt < nestedDisjunction->m_alternatives.size(); ++alt) {
                     if (containsCapturingTerms(nestedDisjunction->m_alternatives[alt].get(), 0, nestedDisjunction->m_alternatives[alt]->m_terms.size()))
@@ -1069,13 +1048,13 @@ public:
             size_t termIndex, firstExpressionTerm;
 
             termIndex = 0;
-            if (terms[termIndex].type == PatternTerm::TypeAssertionBOL) {
+            if (terms[termIndex].type == PatternTerm::Type::AssertionBOL) {
                 startsWithBOL = true;
                 ++termIndex;
             }
             
             PatternTerm& firstNonAnchorTerm = terms[termIndex];
-            if (firstNonAnchorTerm.type != PatternTerm::TypeCharacterClass
+            if (firstNonAnchorTerm.type != PatternTerm::Type::CharacterClass
                 || firstNonAnchorTerm.characterClass != dotCharacterClass
                 || firstNonAnchorTerm.quantityMinCount
                 || firstNonAnchorTerm.quantityMaxCount != quantifyInfinite)
@@ -1084,15 +1063,15 @@ public:
             firstExpressionTerm = termIndex + 1;
             
             termIndex = terms.size() - 1;
-            if (terms[termIndex].type == PatternTerm::TypeAssertionEOL) {
+            if (terms[termIndex].type == PatternTerm::Type::AssertionEOL) {
                 endsWithEOL = true;
                 --termIndex;
             }
             
             PatternTerm& lastNonAnchorTerm = terms[termIndex];
-            if (lastNonAnchorTerm.type != PatternTerm::TypeCharacterClass
+            if (lastNonAnchorTerm.type != PatternTerm::Type::CharacterClass
                 || lastNonAnchorTerm.characterClass != dotCharacterClass
-                || lastNonAnchorTerm.quantityType != QuantifierGreedy
+                || lastNonAnchorTerm.quantityType != QuantifierType::Greedy
                 || lastNonAnchorTerm.quantityMinCount
                 || lastNonAnchorTerm.quantityMaxCount != quantifyInfinite)
                 return;
@@ -1118,51 +1097,26 @@ public:
     ErrorCode error() { return m_error; }
 
 private:
-    bool isSafeToRecurse() const
-    {
-        if (!m_stackLimit)
-            return true;
-        int8_t* curr = reinterpret_cast<int8_t*>(currentStackPointer());
-        int8_t* limit = reinterpret_cast<int8_t*>(m_stackLimit);
-        return curr >= limit;
-    }
+    inline bool isSafeToRecurse() { return m_stackCheck.isSafeToRecurse(); }
 
     YarrPattern& m_pattern;
     PatternAlternative* m_alternative;
     CharacterClassConstructor m_characterClassConstructor;
-    Vector<String> m_unmatchedNamedForwardReferences;
-    void* m_stackLimit;
+    StackCheck m_stackCheck;
     ErrorCode m_error { ErrorCode::NoError };
     bool m_invertCharacterClass;
     bool m_invertParentheticalAssertion { false };
 };
 
-ErrorCode YarrPattern::compile(const String& patternString, void* stackLimit)
+ErrorCode YarrPattern::compile(StringView patternString)
 {
-    YarrPatternConstructor constructor(*this, stackLimit);
+    YarrPatternConstructor constructor(*this);
 
     {
         ErrorCode error = parse(constructor, patternString, unicode());
         if (hasError(error))
             return error;
     }
-    
-    // If the pattern contains illegal backreferences reset & reparse.
-    // Quoting Netscape's "What's new in JavaScript 1.2",
-    //      "Note: if the number of left parentheses is less than the number specified
-    //       in \#, the \# is taken as an octal escape as described in the next row."
-    if (containsIllegalBackReference() || containsIllegalNamedForwardReferences()) {
-        if (unicode())
-            return ErrorCode::InvalidBackreference;
-
-        unsigned numSubpatterns = m_numSubpatterns;
-
-        constructor.saveUnmatchedNamedForwardReferences();
-        constructor.resetForReparsing();
-        ErrorCode error = parse(constructor, patternString, unicode(), numSubpatterns);
-        ASSERT_UNUSED(error, !hasError(error));
-        ASSERT(numSubpatterns == m_numSubpatterns);
-    }
 
     constructor.checkForTerminalParentheses();
     constructor.optimizeDotStarWrappedExpressions();
@@ -1183,7 +1137,7 @@ ErrorCode YarrPattern::compile(const String& patternString, void* stackLimit)
     return ErrorCode::NoError;
 }
 
-YarrPattern::YarrPattern(const String& pattern, OptionSet<Flags> flags, ErrorCode& error, void* stackLimit)
+YarrPattern::YarrPattern(StringView pattern, OptionSet<Flags> flags, ErrorCode& error)
     : m_containsBackreferences(false)
     , m_containsBOL(false)
     , m_containsUnsignedLengthPattern(false)
@@ -1192,7 +1146,7 @@ YarrPattern::YarrPattern(const String& pattern, OptionSet<Flags> flags, ErrorCod
     , m_flags(flags)
 {
     ASSERT(m_flags != Flags::DeletedValue);
-    error = compile(pattern, stackLimit);
+    error = compile(pattern);
 }
 
 void indentForNestingLevel(PrintStream& out, unsigned nestingDepth)
@@ -1302,19 +1256,19 @@ void PatternAlternative::dump(PrintStream& out, YarrPattern* thisPattern, unsign
 
 void PatternTerm::dumpQuantifier(PrintStream& out)
 {
-    if (quantityType == QuantifierFixedCount && quantityMinCount == 1 && quantityMaxCount == 1)
+    if (quantityType == QuantifierType::FixedCount && quantityMinCount == 1 && quantityMaxCount == 1)
         return;
-    out.print(" {", quantityMinCount.unsafeGet());
+    out.print(" {", quantityMinCount.value());
     if (quantityMinCount != quantityMaxCount) {
         if (quantityMaxCount == UINT_MAX)
             out.print(",...");
         else
-            out.print(",", quantityMaxCount.unsafeGet());
+            out.print(",", quantityMaxCount.value());
     }
     out.print("}");
-    if (quantityType == QuantifierGreedy)
+    if (quantityType == QuantifierType::Greedy)
         out.print(" greedy");
-    else if (quantityType == QuantifierNonGreedy)
+    else if (quantityType == QuantifierType::NonGreedy)
         out.print(" non-greedy");
 }
 
@@ -1322,22 +1276,22 @@ void PatternTerm::dump(PrintStream& out, YarrPattern* thisPattern, unsigned nest
 {
     indentForNestingLevel(out, nestingDepth);
 
-    if (type != TypeParenthesesSubpattern && type != TypeParentheticalAssertion) {
+    if (type != Type::ParenthesesSubpattern && type != Type::ParentheticalAssertion) {
         if (invert())
             out.print("not ");
     }
 
     switch (type) {
-    case TypeAssertionBOL:
+    case Type::AssertionBOL:
         out.println("BOL");
         break;
-    case TypeAssertionEOL:
+    case Type::AssertionEOL:
         out.println("EOL");
         break;
-    case TypeAssertionWordBoundary:
+    case Type::AssertionWordBoundary:
         out.println("word boundary");
         break;
-    case TypePatternCharacter:
+    case Type::PatternCharacter:
         out.printf("character ");
         out.printf("inputPosition %u ", inputPosition);
         if (thisPattern->ignoreCase() && isASCIIAlpha(patternCharacter)) {
@@ -1347,40 +1301,40 @@ void PatternTerm::dump(PrintStream& out, YarrPattern* thisPattern, unsigned nest
         } else
             dumpUChar32(out, patternCharacter);
         dumpQuantifier(out);
-        if (quantityType != QuantifierFixedCount)
+        if (quantityType != QuantifierType::FixedCount)
             out.print(",frame location ", frameLocation);
         out.println();
         break;
-    case TypeCharacterClass:
+    case Type::CharacterClass:
         out.print("character class ");
         out.printf("inputPosition %u ", inputPosition);
         dumpCharacterClass(out, thisPattern, characterClass);
         dumpQuantifier(out);
-        if (quantityType != QuantifierFixedCount || thisPattern->unicode())
+        if (quantityType != QuantifierType::FixedCount || thisPattern->unicode())
             out.print(",frame location ", frameLocation);
         out.println();
         break;
-    case TypeBackReference:
+    case Type::BackReference:
         out.print("back reference to subpattern #", backReferenceSubpatternId);
         out.println(",frame location ", frameLocation);
         break;
-    case TypeForwardReference:
+    case Type::ForwardReference:
         out.println("forward reference");
         break;
-    case TypeParenthesesSubpattern:
+    case Type::ParenthesesSubpattern:
         if (m_capture)
             out.print("captured ");
         else
             out.print("non-captured ");
 
         FALLTHROUGH;
-    case TypeParentheticalAssertion:
+    case Type::ParentheticalAssertion:
         if (m_invert)
             out.print("inverted ");
 
-        if (type == TypeParenthesesSubpattern)
+        if (type == Type::ParenthesesSubpattern)
             out.print("subpattern");
-        else if (type == TypeParentheticalAssertion)
+        else if (type == Type::ParentheticalAssertion)
             out.print("assertion");
 
         if (m_capture)
@@ -1410,7 +1364,7 @@ void PatternTerm::dump(PrintStream& out, YarrPattern* thisPattern, unsigned nest
 
         parentheses.disjunction->dump(out, thisPattern, nestingDepth + 1);
         break;
-    case TypeDotStarEnclosure:
+    case Type::DotStarEnclosure:
         out.println(".* enclosure,frame location ", thisPattern->m_initialStartValueFrameLocation);
         break;
     }
@@ -1427,7 +1381,7 @@ void PatternDisjunction::dump(PrintStream& out, YarrPattern* thisPattern, unsign
     }
 }
 
-void YarrPattern::dumpPatternString(PrintStream& out, const String& patternString)
+void YarrPattern::dumpPatternString(PrintStream& out, StringView patternString)
 {
     out.print("/", patternString, "/");
 
@@ -1443,12 +1397,12 @@ void YarrPattern::dumpPatternString(PrintStream& out, const String& patternStrin
         out.print("y");
 }
 
-void YarrPattern::dumpPattern(const String& patternString)
+void YarrPattern::dumpPattern(StringView patternString)
 {
     dumpPattern(WTF::dataFile(), patternString);
 }
 
-void YarrPattern::dumpPattern(PrintStream& out, const String& patternString)
+void YarrPattern::dumpPattern(PrintStream& out, StringView patternString)
 {
     out.print("RegExp pattern for ");
     dumpPatternString(out, patternString);
