@@ -7,14 +7,20 @@
 
 import os
 import sys
+if sys.version_info.major != 3 and __name__ == '__main__':
+    # Swarming prepends sys.executable so we get python2 regardless of shebang.
+    # Spawn itself with vpython3 instead.
+    import subprocess
+    sys.exit(subprocess.call(['vpython3', os.path.realpath(__file__)] + sys.argv[1:]))
 
-d = os.path.dirname
-THIS_DIR = d(os.path.abspath(__file__))
-ANGLE_SRC_DIR = d(THIS_DIR)
-sys.path.insert(0, os.path.join(ANGLE_SRC_DIR, 'testing', 'merge_scripts'))
-CHROMIUM_SRC_DIR = d(d(ANGLE_SRC_DIR))
-sys.path.insert(0, os.path.join(CHROMIUM_SRC_DIR, 'testing', 'merge_scripts'))
+import pathlib  # python3
 
+PY_UTILS = str(pathlib.Path(__file__).resolve().parents[1] / 'src' / 'tests' / 'py_utils')
+if PY_UTILS not in sys.path:
+    os.stat(PY_UTILS) and sys.path.insert(0, PY_UTILS)
+import angle_path_util
+
+angle_path_util.AddDepsDirToPath('testing/merge_scripts')
 import merge_api
 import standard_isolated_script_merge
 
