@@ -1118,7 +1118,8 @@ enum TQualifier
     EvqPrimitiveIDIn,  // gl_PrimitiveIDIn
     EvqInvocationID,   // gl_InvocationID
     EvqPrimitiveID,    // gl_PrimitiveID
-    EvqLayer,          // gl_Layer
+    EvqLayerOut,       // gl_Layer (GS output)
+    EvqLayerIn,        // gl_Layer (FS input)
 
     // GLSL ES 3.1 extension EXT_gpu_shader5 qualifiers
     EvqPrecise,
@@ -1134,8 +1135,8 @@ enum TQualifier
     EvqTessLevelOuter,
     EvqTessLevelInner,
 
-    // GLES ES 3.1 extension EXT_primitive_bounding_box
-    EvqBoundingBoxEXT,
+    // GLES ES 3.1 extension EXT_primitive_bounding_box/OES_primitive_bounding_box
+    EvqBoundingBox,
 
     EvqTessEvaluationIn,
     EvqTessEvaluationOut,
@@ -1304,6 +1305,36 @@ enum TLayoutTessEvaluationType
     EtetPointMode
 };
 
+class AdvancedBlendEquations
+{
+  public:
+    // Must have a trivial default constructor since it is used in YYSTYPE.
+    AdvancedBlendEquations() = default;
+    explicit constexpr AdvancedBlendEquations(uint32_t initialState)
+        : mEnabledBlendEquations(initialState)
+    {}
+
+    bool any() const;
+    bool anyHsl() const;
+
+    void setAll();
+    void reset() { mEnabledBlendEquations = 0; }
+
+    // Parameter is gl::BlendEquationType, but PackedEnums.h include is not possible here.
+    void set(uint32_t blendEquation);
+
+    uint32_t bits() const { return mEnabledBlendEquations; }
+
+    AdvancedBlendEquations operator|=(AdvancedBlendEquations other)
+    {
+        mEnabledBlendEquations |= other.mEnabledBlendEquations;
+        return *this;
+    }
+
+  private:
+    uint32_t mEnabledBlendEquations;
+};
+
 struct TLayoutQualifier
 {
     // Must have a trivial default constructor since it is used in YYSTYPE.
@@ -1320,7 +1351,7 @@ struct TLayoutQualifier
                invocations == 0 && maxVertices == -1 && vertices == 0 &&
                tesPrimitiveType == EtetUndefined && tesVertexSpacingType == EtetUndefined &&
                tesOrderingType == EtetUndefined && tesPointType == EtetUndefined && index == -1 &&
-               inputAttachmentIndex == -1 && noncoherent == false;
+               inputAttachmentIndex == -1 && noncoherent == false && !advancedBlendEquations.any();
     }
 
     bool isCombinationValid() const
@@ -1333,6 +1364,7 @@ struct TLayoutQualifier
         bool otherLayoutQualifiersSpecified =
             (location != -1 || binding != -1 || index != -1 || matrixPacking != EmpUnspecified ||
              blockStorage != EbsUnspecified || imageInternalFormat != EiifUnspecified);
+        bool blendEquationSpecified = advancedBlendEquations.any();
 
         // we can have either the work group size specified, or number of views,
         // or yuv layout qualifier, or early_fragment_tests layout qualifier, or the other layout
@@ -1340,7 +1372,7 @@ struct TLayoutQualifier
         return (workGroupSizeSpecified ? 1 : 0) + (numViewsSet ? 1 : 0) + (yuv ? 1 : 0) +
                    (earlyFragmentTests ? 1 : 0) + (otherLayoutQualifiersSpecified ? 1 : 0) +
                    (geometryShaderSpecified ? 1 : 0) + (subpassInputSpecified ? 1 : 0) +
-                   (noncoherent ? 1 : 0) <=
+                   (noncoherent ? 1 : 0) + (blendEquationSpecified ? 1 : 0) <=
                1;
     }
 
@@ -1391,6 +1423,9 @@ struct TLayoutQualifier
     int inputAttachmentIndex;
     bool noncoherent;
 
+    // KHR_blend_equation_advanced layout qualifiers.
+    AdvancedBlendEquations advancedBlendEquations;
+
   private:
     explicit constexpr TLayoutQualifier(int /*placeholder*/)
         : location(-1),
@@ -1414,7 +1449,8 @@ struct TLayoutQualifier
           tesPointType(EtetUndefined),
           index(-1),
           inputAttachmentIndex(-1),
-          noncoherent(false)
+          noncoherent(false),
+          advancedBlendEquations(0)
     {}
 };
 
@@ -1506,7 +1542,8 @@ inline const char *getQualifierString(TQualifier q)
     case EvqSecondaryFragDataEXT:   return "SecondaryFragDataEXT";
     case EvqViewIDOVR:              return "ViewIDOVR";
     case EvqViewportIndex:          return "ViewportIndex";
-    case EvqLayer:                  return "Layer";
+    case EvqLayerOut:               return "LayerOut";
+    case EvqLayerIn:                return "LayerIn";
     case EvqLastFragColor:          return "LastFragColor";
     case EvqLastFragData:           return "LastFragData";
     case EvqFragmentInOut:          return "inout";
@@ -1557,7 +1594,7 @@ inline const char *getQualifierString(TQualifier q)
     case EvqPatchVerticesIn:        return "PatchVerticesIn";
     case EvqTessLevelOuter:         return "TessLevelOuter";
     case EvqTessLevelInner:         return "TessLevelInner";
-    case EvqBoundingBoxEXT:         return "BoundingBoxEXT";
+    case EvqBoundingBox:            return "BoundingBox";
     case EvqTessEvaluationIn:       return "in";
     case EvqTessEvaluationOut:      return "out";
     case EvqTessCoord:              return "TessCoord";
