@@ -1,6 +1,7 @@
 /* A Bison parser, made by GNU Bison 3.3.2.  */
 
-/* Apple Note: For the avoidance of doubt, Apple elects to distribute this file under the terms of the BSD license. */
+/* Apple Note: For the avoidance of doubt, Apple elects to distribute this file under the terms of
+ * the BSD license. */
 
 /* Bison interface for Yacc-like parsers in C
 
@@ -111,129 +112,130 @@ enum yytokentype
     FLAT                      = 310,
     SMOOTH                    = 311,
     NOPERSPECTIVE             = 312,
-    READONLY                  = 313,
-    WRITEONLY                 = 314,
-    COHERENT                  = 315,
-    RESTRICT                  = 316,
-    VOLATILE                  = 317,
-    SHARED                    = 318,
-    STRUCT                    = 319,
-    VOID_TYPE                 = 320,
-    WHILE                     = 321,
-    SAMPLER2D                 = 322,
-    SAMPLERCUBE               = 323,
-    SAMPLER_EXTERNAL_OES      = 324,
-    SAMPLER2DRECT             = 325,
-    SAMPLER2DARRAY            = 326,
-    ISAMPLER2D                = 327,
-    ISAMPLER3D                = 328,
-    ISAMPLERCUBE              = 329,
-    ISAMPLER2DARRAY           = 330,
-    USAMPLER2D                = 331,
-    USAMPLER3D                = 332,
-    USAMPLERCUBE              = 333,
-    USAMPLER2DARRAY           = 334,
-    SAMPLER2DMS               = 335,
-    ISAMPLER2DMS              = 336,
-    USAMPLER2DMS              = 337,
-    SAMPLER2DMSARRAY          = 338,
-    ISAMPLER2DMSARRAY         = 339,
-    USAMPLER2DMSARRAY         = 340,
-    SAMPLER3D                 = 341,
-    SAMPLER3DRECT             = 342,
-    SAMPLER2DSHADOW           = 343,
-    SAMPLERCUBESHADOW         = 344,
-    SAMPLER2DARRAYSHADOW      = 345,
-    SAMPLERVIDEOWEBGL         = 346,
-    SAMPLERCUBEARRAYOES       = 347,
-    SAMPLERCUBEARRAYSHADOWOES = 348,
-    ISAMPLERCUBEARRAYOES      = 349,
-    USAMPLERCUBEARRAYOES      = 350,
-    SAMPLERCUBEARRAYEXT       = 351,
-    SAMPLERCUBEARRAYSHADOWEXT = 352,
-    ISAMPLERCUBEARRAYEXT      = 353,
-    USAMPLERCUBEARRAYEXT      = 354,
-    SAMPLERBUFFER             = 355,
-    ISAMPLERBUFFER            = 356,
-    USAMPLERBUFFER            = 357,
-    SAMPLEREXTERNAL2DY2YEXT   = 358,
-    IMAGE2D                   = 359,
-    IIMAGE2D                  = 360,
-    UIMAGE2D                  = 361,
-    IMAGE3D                   = 362,
-    IIMAGE3D                  = 363,
-    UIMAGE3D                  = 364,
-    IMAGE2DARRAY              = 365,
-    IIMAGE2DARRAY             = 366,
-    UIMAGE2DARRAY             = 367,
-    IMAGECUBE                 = 368,
-    IIMAGECUBE                = 369,
-    UIMAGECUBE                = 370,
-    IMAGECUBEARRAYOES         = 371,
-    IIMAGECUBEARRAYOES        = 372,
-    UIMAGECUBEARRAYOES        = 373,
-    IMAGECUBEARRAYEXT         = 374,
-    IIMAGECUBEARRAYEXT        = 375,
-    UIMAGECUBEARRAYEXT        = 376,
-    IMAGEBUFFER               = 377,
-    IIMAGEBUFFER              = 378,
-    UIMAGEBUFFER              = 379,
-    ATOMICUINT                = 380,
-    LAYOUT                    = 381,
-    YUVCSCSTANDARDEXT         = 382,
-    YUVCSCSTANDARDEXTCONSTANT = 383,
-    IDENTIFIER                = 384,
-    TYPE_NAME                 = 385,
-    FLOATCONSTANT             = 386,
-    INTCONSTANT               = 387,
-    UINTCONSTANT              = 388,
-    BOOLCONSTANT              = 389,
-    FIELD_SELECTION           = 390,
-    LEFT_OP                   = 391,
-    RIGHT_OP                  = 392,
-    INC_OP                    = 393,
-    DEC_OP                    = 394,
-    LE_OP                     = 395,
-    GE_OP                     = 396,
-    EQ_OP                     = 397,
-    NE_OP                     = 398,
-    AND_OP                    = 399,
-    OR_OP                     = 400,
-    XOR_OP                    = 401,
-    MUL_ASSIGN                = 402,
-    DIV_ASSIGN                = 403,
-    ADD_ASSIGN                = 404,
-    MOD_ASSIGN                = 405,
-    LEFT_ASSIGN               = 406,
-    RIGHT_ASSIGN              = 407,
-    AND_ASSIGN                = 408,
-    XOR_ASSIGN                = 409,
-    OR_ASSIGN                 = 410,
-    SUB_ASSIGN                = 411,
-    LEFT_PAREN                = 412,
-    RIGHT_PAREN               = 413,
-    LEFT_BRACKET              = 414,
-    RIGHT_BRACKET             = 415,
-    LEFT_BRACE                = 416,
-    RIGHT_BRACE               = 417,
-    DOT                       = 418,
-    COMMA                     = 419,
-    COLON                     = 420,
-    EQUAL                     = 421,
-    SEMICOLON                 = 422,
-    BANG                      = 423,
-    DASH                      = 424,
-    TILDE                     = 425,
-    PLUS                      = 426,
-    STAR                      = 427,
-    SLASH                     = 428,
-    PERCENT                   = 429,
-    LEFT_ANGLE                = 430,
-    RIGHT_ANGLE               = 431,
-    VERTICAL_BAR              = 432,
-    CARET                     = 433,
-    AMPERSAND                 = 434,
-    QUESTION                  = 435
+    PATCH                     = 313,
+    READONLY                  = 314,
+    WRITEONLY                 = 315,
+    COHERENT                  = 316,
+    RESTRICT                  = 317,
+    VOLATILE                  = 318,
+    SHARED                    = 319,
+    STRUCT                    = 320,
+    VOID_TYPE                 = 321,
+    WHILE                     = 322,
+    SAMPLER2D                 = 323,
+    SAMPLERCUBE               = 324,
+    SAMPLER_EXTERNAL_OES      = 325,
+    SAMPLER2DRECT             = 326,
+    SAMPLER2DARRAY            = 327,
+    ISAMPLER2D                = 328,
+    ISAMPLER3D                = 329,
+    ISAMPLERCUBE              = 330,
+    ISAMPLER2DARRAY           = 331,
+    USAMPLER2D                = 332,
+    USAMPLER3D                = 333,
+    USAMPLERCUBE              = 334,
+    USAMPLER2DARRAY           = 335,
+    SAMPLER2DMS               = 336,
+    ISAMPLER2DMS              = 337,
+    USAMPLER2DMS              = 338,
+    SAMPLER2DMSARRAY          = 339,
+    ISAMPLER2DMSARRAY         = 340,
+    USAMPLER2DMSARRAY         = 341,
+    SAMPLER3D                 = 342,
+    SAMPLER3DRECT             = 343,
+    SAMPLER2DSHADOW           = 344,
+    SAMPLERCUBESHADOW         = 345,
+    SAMPLER2DARRAYSHADOW      = 346,
+    SAMPLERVIDEOWEBGL         = 347,
+    SAMPLERCUBEARRAYOES       = 348,
+    SAMPLERCUBEARRAYSHADOWOES = 349,
+    ISAMPLERCUBEARRAYOES      = 350,
+    USAMPLERCUBEARRAYOES      = 351,
+    SAMPLERCUBEARRAYEXT       = 352,
+    SAMPLERCUBEARRAYSHADOWEXT = 353,
+    ISAMPLERCUBEARRAYEXT      = 354,
+    USAMPLERCUBEARRAYEXT      = 355,
+    SAMPLERBUFFER             = 356,
+    ISAMPLERBUFFER            = 357,
+    USAMPLERBUFFER            = 358,
+    SAMPLEREXTERNAL2DY2YEXT   = 359,
+    IMAGE2D                   = 360,
+    IIMAGE2D                  = 361,
+    UIMAGE2D                  = 362,
+    IMAGE3D                   = 363,
+    IIMAGE3D                  = 364,
+    UIMAGE3D                  = 365,
+    IMAGE2DARRAY              = 366,
+    IIMAGE2DARRAY             = 367,
+    UIMAGE2DARRAY             = 368,
+    IMAGECUBE                 = 369,
+    IIMAGECUBE                = 370,
+    UIMAGECUBE                = 371,
+    IMAGECUBEARRAYOES         = 372,
+    IIMAGECUBEARRAYOES        = 373,
+    UIMAGECUBEARRAYOES        = 374,
+    IMAGECUBEARRAYEXT         = 375,
+    IIMAGECUBEARRAYEXT        = 376,
+    UIMAGECUBEARRAYEXT        = 377,
+    IMAGEBUFFER               = 378,
+    IIMAGEBUFFER              = 379,
+    UIMAGEBUFFER              = 380,
+    ATOMICUINT                = 381,
+    LAYOUT                    = 382,
+    YUVCSCSTANDARDEXT         = 383,
+    YUVCSCSTANDARDEXTCONSTANT = 384,
+    IDENTIFIER                = 385,
+    TYPE_NAME                 = 386,
+    FLOATCONSTANT             = 387,
+    INTCONSTANT               = 388,
+    UINTCONSTANT              = 389,
+    BOOLCONSTANT              = 390,
+    FIELD_SELECTION           = 391,
+    LEFT_OP                   = 392,
+    RIGHT_OP                  = 393,
+    INC_OP                    = 394,
+    DEC_OP                    = 395,
+    LE_OP                     = 396,
+    GE_OP                     = 397,
+    EQ_OP                     = 398,
+    NE_OP                     = 399,
+    AND_OP                    = 400,
+    OR_OP                     = 401,
+    XOR_OP                    = 402,
+    MUL_ASSIGN                = 403,
+    DIV_ASSIGN                = 404,
+    ADD_ASSIGN                = 405,
+    MOD_ASSIGN                = 406,
+    LEFT_ASSIGN               = 407,
+    RIGHT_ASSIGN              = 408,
+    AND_ASSIGN                = 409,
+    XOR_ASSIGN                = 410,
+    OR_ASSIGN                 = 411,
+    SUB_ASSIGN                = 412,
+    LEFT_PAREN                = 413,
+    RIGHT_PAREN               = 414,
+    LEFT_BRACKET              = 415,
+    RIGHT_BRACKET             = 416,
+    LEFT_BRACE                = 417,
+    RIGHT_BRACE               = 418,
+    DOT                       = 419,
+    COMMA                     = 420,
+    COLON                     = 421,
+    EQUAL                     = 422,
+    SEMICOLON                 = 423,
+    BANG                      = 424,
+    DASH                      = 425,
+    TILDE                     = 426,
+    PLUS                      = 427,
+    STAR                      = 428,
+    SLASH                     = 429,
+    PERCENT                   = 430,
+    LEFT_ANGLE                = 431,
+    RIGHT_ANGLE               = 432,
+    VERTICAL_BAR              = 433,
+    CARET                     = 434,
+    AMPERSAND                 = 435,
+    QUESTION                  = 436
 };
 #endif
 
