@@ -52,6 +52,7 @@ class ImageSibling : public gl::FramebufferAttachmentObject
                       GLenum binding,
                       const gl::ImageIndex &imageIndex) const override;
     bool isYUV() const override;
+    bool isCreatedWithAHB() const override;
     bool hasProtectedContent() const override;
 
   protected:
@@ -96,19 +97,23 @@ class ExternalImageSibling : public ImageSibling
     gl::Extents getAttachmentSize(const gl::ImageIndex &imageIndex) const override;
     gl::Format getAttachmentFormat(GLenum binding, const gl::ImageIndex &imageIndex) const override;
     GLsizei getAttachmentSamples(const gl::ImageIndex &imageIndex) const override;
+    GLuint getLevelCount() const;
     bool isRenderable(const gl::Context *context,
                       GLenum binding,
                       const gl::ImageIndex &imageIndex) const override;
     bool isTextureable(const gl::Context *context) const;
     bool isYUV() const override;
+    bool isCubeMap() const;
     bool hasProtectedContent() const override;
 
     void onAttach(const gl::Context *context, rx::Serial framebufferSerial) override;
     void onDetach(const gl::Context *context, rx::Serial framebufferSerial) override;
     GLuint getId() const override;
 
-    gl::InitState initState(const gl::ImageIndex &imageIndex) const override;
-    void setInitState(const gl::ImageIndex &imageIndex, gl::InitState initState) override;
+    gl::InitState initState(GLenum binding, const gl::ImageIndex &imageIndex) const override;
+    void setInitState(GLenum binding,
+                      const gl::ImageIndex &imageIndex,
+                      gl::InitState initState) override;
 
     rx::ExternalImageSiblingImpl *getImplementation() const;
 
@@ -136,8 +141,10 @@ struct ImageState : private angle::NonCopyable
 
     gl::Format format;
     bool yuv;
+    bool cubeMap;
     gl::Extents size;
     size_t samples;
+    GLuint levelCount;
     EGLenum sourceType;
     EGLenum colorspace;
     bool hasProtectedContent;
@@ -162,10 +169,15 @@ class Image final : public RefCountObject, public LabeledObject
     bool isRenderable(const gl::Context *context) const;
     bool isTexturable(const gl::Context *context) const;
     bool isYUV() const;
+    bool isCreatedWithAHB() const;
+    // Returns true only if the eglImage contains a complete cubemap
+    bool isCubeMap() const;
     size_t getWidth() const;
     size_t getHeight() const;
+    const gl::Extents &getExtents() const;
     bool isLayered() const;
     size_t getSamples() const;
+    GLuint getLevelCount() const;
     bool hasProtectedContent() const;
 
     Error initialize(const Display *display);
