@@ -23,6 +23,7 @@
 
 #include <bitset>
 #include <map>
+#include <memory>
 #include <unordered_map>
 
 namespace gl
@@ -124,9 +125,9 @@ enum class ClipSpaceOrigin
 };
 
 // Calculate the intersection of two rectangles.  Returns false if the intersection is empty.
-ANGLE_NO_DISCARD bool ClipRectangle(const Rectangle &source,
-                                    const Rectangle &clip,
-                                    Rectangle *intersection);
+[[nodiscard]] bool ClipRectangle(const Rectangle &source,
+                                 const Rectangle &clip,
+                                 Rectangle *intersection);
 // Calculate the smallest rectangle that covers both rectangles.  This rectangle may cover areas
 // not covered by the two rectangles, for example in this situation:
 //
@@ -167,7 +168,7 @@ struct Extents
     Extents() : width(0), height(0), depth(0) {}
     Extents(int width_, int height_, int depth_) : width(width_), height(height_), depth(depth_) {}
 
-    Extents(const Extents &other) = default;
+    Extents(const Extents &other)            = default;
     Extents &operator=(const Extents &other) = default;
 
     bool empty() const { return (width * height * depth) == 0; }
@@ -195,6 +196,7 @@ struct Box
           height(size.height),
           depth(size.depth)
     {}
+    bool valid() const;
     bool operator==(const Box &other) const;
     bool operator!=(const Box &other) const;
     Rectangle toRect() const;
@@ -202,6 +204,8 @@ struct Box
     // Whether the Box has offset 0 and the same extents as argument.
     bool coversSameExtent(const Extents &size) const;
 
+    bool contains(const Box &other) const;
+
     int x;
     int y;
     int z;
@@ -610,6 +614,9 @@ class BlendStateExt final
     using FactorStorage    = StorageType<BlendFactorType, angle::EnumSize<BlendFactorType>()>;
     using EquationStorage  = StorageType<BlendEquationType, angle::EnumSize<BlendEquationType>()>;
     using ColorMaskStorage = StorageType<uint8_t, 16>;
+    static_assert(std::is_same<FactorStorage::Type, uint64_t>::value &&
+                      std::is_same<EquationStorage::Type, uint64_t>::value,
+                  "Factor and Equation storage must be 64-bit.");
 
     BlendStateExt(const size_t drawBuffers = 1);
 
@@ -666,6 +673,7 @@ class BlendStateExt final
     ///////// Blend Equation /////////
 
     EquationStorage::Type expandEquationValue(const GLenum mode) const;
+    EquationStorage::Type expandEquationValue(const gl::BlendEquationType equation) const;
     EquationStorage::Type expandEquationColorIndexed(const size_t index) const;
     EquationStorage::Type expandEquationAlphaIndexed(const size_t index) const;
     void setEquations(const GLenum modeColor, const GLenum modeAlpha);
@@ -677,6 +685,10 @@ class BlendStateExt final
     GLenum getEquationAlphaIndexed(size_t index) const;
     DrawBufferMask compareEquations(const EquationStorage::Type color,
                                     const EquationStorage::Type alpha) const;
+    DrawBufferMask compareEquations(const BlendStateExt &other) const
+    {
+        return compareEquations(other.mEquationColor, other.mEquationAlpha);
+    }
 
     ///////// Blend Factors /////////
 
@@ -703,31 +715,93 @@ class BlendStateExt final
                                   const FactorStorage::Type dstColor,
                                   const FactorStorage::Type srcAlpha,
                                   const FactorStorage::Type dstAlpha) const;
+    DrawBufferMask compareFactors(const BlendStateExt &other) const
+    {
+        return compareFactors(other.mSrcColor, other.mDstColor, other.mSrcAlpha, other.mDstAlpha);
+    }
+
+    constexpr FactorStorage::Type getSrcColorBits() const { return mSrcColor; }
+    constexpr FactorStorage::Type getSrcAlphaBits() const { return mSrcAlpha; }
+    constexpr FactorStorage::Type getDstColorBits() const { return mDstColor; }
+    constexpr FactorStorage::Type getDstAlphaBits() const { return mDstAlpha; }
+
+    constexpr EquationStorage::Type getEquationColorBits() const { return mEquationColor; }
+    constexpr EquationStorage::Type getEquationAlphaBits() const { return mEquationAlpha; }
+
+    constexpr ColorMaskStorage::Type getAllColorMaskBits() const { return mAllColorMask; }
+    constexpr ColorMaskStorage::Type getColorMaskBits() const { return mColorMask; }
+
+    constexpr DrawBufferMask getAllEnabledMask() const { return mAllEnabledMask; }
+    constexpr DrawBufferMask getEnabledMask() const { return mEnabledMask; }
+
+    constexpr DrawBufferMask getUsesAdvancedBlendEquationMask() const
+    {
+        return mUsesAdvancedBlendEquationMask;
+    }
+
+    constexpr uint8_t getDrawBufferCount() const { return mDrawBufferCount; }
+
+    constexpr void setSrcColorBits(const FactorStorage::Type srcColor) { mSrcColor = srcColor; }
+    constexpr void setSrcAlphaBits(const FactorStorage::Type srcAlpha) { mSrcAlpha = srcAlpha; }
+    constexpr void setDstColorBits(const FactorStorage::Type dstColor) { mDstColor = dstColor; }
+    constexpr void setDstAlphaBits(const FactorStorage::Type dstAlpha) { mDstAlpha = dstAlpha; }
+
+    constexpr void setEquationColorBits(const EquationStorage::Type equationColor)
+    {
+        mEquationColor = equationColor;
+    }
+    constexpr void setEquationAlphaBits(const EquationStorage::Type equationAlpha)
+    {
+        mEquationAlpha = equationAlpha;
+    }
+
+    constexpr void setColorMaskBits(const ColorMaskStorage::Type colorMask)
+    {
+        mColorMask = colorMask;
+    }
+
+    constexpr void setEnabledMask(const DrawBufferMask enabledMask) { mEnabledMask = enabledMask; }
 
     ///////// Data Members /////////
+  private:
+    uint64_t mParameterMask;
 
-    FactorStorage::Type mMaxFactorMask;
     FactorStorage::Type mSrcColor;
     FactorStorage::Type mDstColor;
     FactorStorage::Type mSrcAlpha;
     FactorStorage::Type mDstAlpha;
 
-    EquationStorage::Type mMaxEquationMask;
     EquationStorage::Type mEquationColor;
     EquationStorage::Type mEquationAlpha;
 
-    ColorMaskStorage::Type mMaxColorMask;
+    ColorMaskStorage::Type mAllColorMask;
     ColorMaskStorage::Type mColorMask;
 
-    DrawBufferMask mMaxEnabledMask;
+    DrawBufferMask mAllEnabledMask;
     DrawBufferMask mEnabledMask;
 
-    size_t mMaxDrawBuffers;
+    // Cache of whether the blend equation for each index is from KHR_blend_equation_advanced.
+    DrawBufferMask mUsesAdvancedBlendEquationMask;
+
+    uint8_t mDrawBufferCount;
+
+    [[maybe_unused]] uint32_t kUnused = 0;
 };
 
+static_assert(sizeof(BlendStateExt) == sizeof(uint64_t) +
+                                           (sizeof(BlendStateExt::FactorStorage::Type) * 4 +
+                                            sizeof(BlendStateExt::EquationStorage::Type) * 2 +
+                                            sizeof(BlendStateExt::ColorMaskStorage::Type) * 2 +
+                                            sizeof(DrawBufferMask) * 3 + sizeof(uint8_t)) +
+                                           sizeof(uint32_t),
+              "The BlendStateExt class must not contain gaps.");
+
 // Used in StateCache
 using StorageBuffersMask = angle::BitSet<IMPLEMENTATION_MAX_SHADER_STORAGE_BUFFER_BINDINGS>;
 
+template <typename T>
+using SampleMaskArray = std::array<T, IMPLEMENTATION_MAX_SAMPLE_MASK_WORDS>;
+
 template <typename T>
 using TexLevelArray = std::array<T, IMPLEMENTATION_MAX_TEXTURE_LEVELS>;
 
@@ -901,6 +975,9 @@ using BarrierVector = angle::FastVector<T, kBarrierVectorDefaultSize>;
 
 using BufferBarrierVector = BarrierVector<Buffer *>;
 
+using SamplerBindingVector = std::vector<BindingPointer<Sampler>>;
+using BufferVector         = std::vector<OffsetBindingPointer<Buffer>>;
+
 struct TextureAndLayout
 {
     Texture *texture;
@@ -920,7 +997,7 @@ class LevelIndexWrapper
   public:
     LevelIndexWrapper() = default;
     explicit constexpr LevelIndexWrapper(T levelIndex) : mLevelIndex(levelIndex) {}
-    constexpr LevelIndexWrapper(const LevelIndexWrapper &other) = default;
+    constexpr LevelIndexWrapper(const LevelIndexWrapper &other)            = default;
     constexpr LevelIndexWrapper &operator=(const LevelIndexWrapper &other) = default;
 
     constexpr T get() const { return mLevelIndex; }
@@ -1084,6 +1161,7 @@ template <typename ObjT, typename ContextT>
 class DestroyThenDelete
 {
   public:
+    DestroyThenDelete() = default;
     DestroyThenDelete(const ContextT *context) : mContext(context) {}
 
     void operator()(ObjT *obj)
@@ -1093,57 +1171,11 @@ class DestroyThenDelete
     }
 
   private:
-    const ContextT *mContext;
-};
-
-// Helper class for wrapping an onDestroy function.
-template <typename ObjT, typename DeleterT>
-class UniqueObjectPointerBase : angle::NonCopyable
-{
-  public:
-    template <typename ContextT>
-    UniqueObjectPointerBase(const ContextT *context) : mObject(nullptr), mDeleter(context)
-    {}
-
-    template <typename ContextT>
-    UniqueObjectPointerBase(ObjT *obj, const ContextT *context) : mObject(obj), mDeleter(context)
-    {}
-
-    ~UniqueObjectPointerBase()
-    {
-        if (mObject)
-        {
-            mDeleter(mObject);
-        }
-    }
-
-    ObjT *operator->() const { return mObject; }
-
-    ObjT *release()
-    {
-        auto obj = mObject;
-        mObject  = nullptr;
-        return obj;
-    }
-
-    ObjT *get() const { return mObject; }
-
-    void reset(ObjT *obj)
-    {
-        if (mObject)
-        {
-            mDeleter(mObject);
-        }
-        mObject = obj;
-    }
-
-  private:
-    ObjT *mObject;
-    DeleterT mDeleter;
+    const ContextT *mContext = nullptr;
 };
 
 template <typename ObjT, typename ContextT>
-using UniqueObjectPointer = UniqueObjectPointerBase<ObjT, DestroyThenDelete<ObjT, ContextT>>;
+using UniqueObjectPointer = std::unique_ptr<ObjT, DestroyThenDelete<ObjT, ContextT>>;
 
 }  // namespace angle
 
