@@ -14,25 +14,14 @@
 
 namespace rx
 {
-std::string generateShaderForImage2DBindSignature(
-    const d3d::Context *context,
+std::string GenerateShaderForImage2DBindSignature(
     ProgramD3D &programD3D,
     const gl::ProgramState &programData,
     gl::ShaderType shaderType,
-    std::vector<sh::Uniform> &image2DUniforms,
-    const gl::ImageUnitTextureTypeMap &image2DBindLayout);
-
-inline std::string GenerateComputeShaderForImage2DBindSignature(
-    const d3d::Context *context,
-    ProgramD3D &programD3D,
-    const gl::ProgramState &programData,
-    std::vector<sh::Uniform> &image2DUniforms,
-    const gl::ImageUnitTextureTypeMap &image2DBindLayout)
-{
-    return generateShaderForImage2DBindSignature(context, programD3D, programData,
-                                                 gl::ShaderType::Compute, image2DUniforms,
-                                                 image2DBindLayout);
-}
+    const std::string &shaderHLSL,
+    std::vector<sh::ShaderVariable> &image2DUniforms,
+    const gl::ImageUnitTextureTypeMap &image2DBindLayout,
+    unsigned int baseUAVRegister);
 
 }  // namespace rx
 
