@@ -82,7 +82,7 @@ class TextureStorage11 : public TextureStorage
                                GLint maxLevel,
                                const d3d11::SharedSRV **outSRV);
     angle::Result generateSwizzles(const gl::Context *context,
-                                   const gl::SwizzleState &swizzleTarget);
+                                   const gl::TextureState &textureState);
     void markLevelDirty(int mipLevel);
     void markDirty();
 
@@ -143,7 +143,11 @@ class TextureStorage11 : public TextureStorage
     GLsizei getRenderToTextureSamples() const override;
 
   protected:
-    TextureStorage11(Renderer11 *renderer, UINT bindFlags, UINT miscFlags, GLenum internalFormat);
+    TextureStorage11(Renderer11 *renderer,
+                     UINT bindFlags,
+                     UINT miscFlags,
+                     GLenum internalFormat,
+                     const std::string &label);
     int getLevelWidth(int mipLevel) const;
     int getLevelHeight(int mipLevel) const;
     int getLevelDepth(int mipLevel) const;
@@ -157,9 +161,16 @@ class TextureStorage11 : public TextureStorage
     virtual angle::Result getSwizzleRenderTarget(const gl::Context *context,
                                                  int mipLevel,
                                                  const d3d11::RenderTargetView **outRTV) = 0;
+
+    enum class SRVType
+    {
+        Sample,
+        Blit,
+        Stencil
+    };
     angle::Result getSRVLevel(const gl::Context *context,
                               int mipLevel,
-                              bool blitSRV,
+                              SRVType srvType,
                               const d3d11::SharedSRV **outSRV);
 
     // Get a version of a depth texture with only depth information, not stencil.
@@ -199,6 +210,10 @@ class TextureStorage11 : public TextureStorage
     // Helper for resolving MS shadowed texture
     angle::Result resolveTextureHelper(const gl::Context *context, const TextureHelper11 &texture);
     angle::Result releaseMultisampledTexStorageForLevel(size_t level) override;
+    angle::Result findMultisampledRenderTarget(const gl::Context *context,
+                                               const gl::ImageIndex &index,
+                                               GLsizei samples,
+                                               RenderTargetD3D **outRT) const;
     angle::Result getMultisampledRenderTarget(const gl::Context *context,
                                               const gl::ImageIndex &index,
                                               GLsizei samples,
@@ -268,18 +283,20 @@ class TextureStorage11 : public TextureStorage
 
     gl::TexLevelArray<d3d11::SharedSRV> mLevelSRVs;
     gl::TexLevelArray<d3d11::SharedSRV> mLevelBlitSRVs;
+    gl::TexLevelArray<d3d11::SharedSRV> mLevelStencilSRVs;
 };
 
 class TextureStorage11_2D : public TextureStorage11
 {
   public:
-    TextureStorage11_2D(Renderer11 *renderer, SwapChain11 *swapchain);
+    TextureStorage11_2D(Renderer11 *renderer, SwapChain11 *swapchain, const std::string &label);
     TextureStorage11_2D(Renderer11 *renderer,
                         GLenum internalformat,
                         bool renderTarget,
                         GLsizei width,
                         GLsizei height,
                         int levels,
+                        const std::string &label,
                         bool hintLevelZeroOnly = false);
     ~TextureStorage11_2D() override;
 
@@ -289,6 +306,10 @@ class TextureStorage11_2D : public TextureStorage11
                               const TextureHelper11 **outResource) override;
     angle::Result getMippedResource(const gl::Context *context,
                                     const TextureHelper11 **outResource) override;
+    angle::Result findRenderTarget(const gl::Context *context,
+                                   const gl::ImageIndex &index,
+                                   GLsizei samples,
+                                   RenderTargetD3D **outRT) const override;
     angle::Result getRenderTarget(const gl::Context *context,
                                   const gl::ImageIndex &index,
                                   GLsizei samples,
@@ -305,6 +326,7 @@ class TextureStorage11_2D : public TextureStorage11
 
     angle::Result useLevelZeroWorkaroundTexture(const gl::Context *context,
                                                 bool useLevelZeroTexture) override;
+    void onLabelUpdate() override;
 
   protected:
     angle::Result getSwizzleTexture(const gl::Context *context,
@@ -369,7 +391,8 @@ class TextureStorage11_External : public TextureStorage11
   public:
     TextureStorage11_External(Renderer11 *renderer,
                               egl::Stream *stream,
-                              const egl::Stream::GLTextureDescription &glDesc);
+                              const egl::Stream::GLTextureDescription &glDesc,
+                              const std::string &label);
     ~TextureStorage11_External() override;
 
     angle::Result onDestroy(const gl::Context *context) override;
@@ -378,6 +401,10 @@ class TextureStorage11_External : public TextureStorage11
                               const TextureHelper11 **outResource) override;
     angle::Result getMippedResource(const gl::Context *context,
                                     const TextureHelper11 **outResource) override;
+    angle::Result findRenderTarget(const gl::Context *context,
+                                   const gl::ImageIndex &index,
+                                   GLsizei samples,
+                                   RenderTargetD3D **outRT) const override;
     angle::Result getRenderTarget(const gl::Context *context,
                                   const gl::ImageIndex &index,
                                   GLsizei samples,
@@ -391,6 +418,7 @@ class TextureStorage11_External : public TextureStorage11
     angle::Result releaseAssociatedImage(const gl::Context *context,
                                          const gl::ImageIndex &index,
                                          Image11 *incomingImage) override;
+    void onLabelUpdate() override;
 
   protected:
     angle::Result getSwizzleTexture(const gl::Context *context,
@@ -432,7 +460,8 @@ class TextureStorage11ImmutableBase : public TextureStorage11
     TextureStorage11ImmutableBase(Renderer11 *renderer,
                                   UINT bindFlags,
                                   UINT miscFlags,
-                                  GLenum internalFormat);
+                                  GLenum internalFormat,
+                                  const std::string &label);
 
     void associateImage(Image11 *image, const gl::ImageIndex &index) override;
     void disassociateImage(const gl::ImageIndex &index, Image11 *expectedImage) override;
@@ -458,7 +487,8 @@ class TextureStorage11_EGLImage final : public TextureStorage11ImmutableBase
   public:
     TextureStorage11_EGLImage(Renderer11 *renderer,
                               EGLImageD3D *eglImage,
-                              RenderTarget11 *renderTarget11);
+                              RenderTarget11 *renderTarget11,
+                              const std::string &label);
     ~TextureStorage11_EGLImage() override;
 
     angle::Result getSubresourceIndex(const gl::Context *context,
@@ -472,6 +502,10 @@ class TextureStorage11_EGLImage final : public TextureStorage11ImmutableBase
                                    const d3d11::SharedSRV **outSRV) override;
     angle::Result getMippedResource(const gl::Context *context,
                                     const TextureHelper11 **outResource) override;
+    angle::Result findRenderTarget(const gl::Context *context,
+                                   const gl::ImageIndex &index,
+                                   GLsizei samples,
+                                   RenderTargetD3D **outRT) const override;
     angle::Result getRenderTarget(const gl::Context *context,
                                   const gl::ImageIndex &index,
                                   GLsizei samples,
@@ -481,6 +515,7 @@ class TextureStorage11_EGLImage final : public TextureStorage11ImmutableBase
 
     angle::Result useLevelZeroWorkaroundTexture(const gl::Context *context,
                                                 bool useLevelZeroTexture) override;
+    void onLabelUpdate() override;
 
   protected:
     angle::Result getSwizzleTexture(const gl::Context *context,
@@ -519,7 +554,8 @@ class TextureStorage11_Cube : public TextureStorage11
                           bool renderTarget,
                           int size,
                           int levels,
-                          bool hintLevelZeroOnly);
+                          bool hintLevelZeroOnly,
+                          const std::string &label);
     ~TextureStorage11_Cube() override;
 
     angle::Result onDestroy(const gl::Context *context) override;
@@ -532,6 +568,10 @@ class TextureStorage11_Cube : public TextureStorage11
                               const TextureHelper11 **outResource) override;
     angle::Result getMippedResource(const gl::Context *context,
                                     const TextureHelper11 **outResource) override;
+    angle::Result findRenderTarget(const gl::Context *context,
+                                   const gl::ImageIndex &index,
+                                   GLsizei samples,
+                                   RenderTargetD3D **outRT) const override;
     angle::Result getRenderTarget(const gl::Context *context,
                                   const gl::ImageIndex &index,
                                   GLsizei samples,
@@ -548,6 +588,7 @@ class TextureStorage11_Cube : public TextureStorage11
 
     angle::Result useLevelZeroWorkaroundTexture(const gl::Context *context,
                                                 bool useLevelZeroTexture) override;
+    void onLabelUpdate() override;
 
   protected:
     angle::Result getSwizzleTexture(const gl::Context *context,
@@ -610,7 +651,8 @@ class TextureStorage11_3D : public TextureStorage11
                         GLsizei width,
                         GLsizei height,
                         GLsizei depth,
-                        int levels);
+                        int levels,
+                        const std::string &label);
     ~TextureStorage11_3D() override;
 
     angle::Result onDestroy(const gl::Context *context) override;
@@ -619,6 +661,10 @@ class TextureStorage11_3D : public TextureStorage11
                               const TextureHelper11 **outResource) override;
 
     // Handles both layer and non-layer RTs
+    angle::Result findRenderTarget(const gl::Context *context,
+                                   const gl::ImageIndex &index,
+                                   GLsizei samples,
+                                   RenderTargetD3D **outRT) const override;
     angle::Result getRenderTarget(const gl::Context *context,
                                   const gl::ImageIndex &index,
                                   GLsizei samples,
@@ -630,6 +676,7 @@ class TextureStorage11_3D : public TextureStorage11
     angle::Result releaseAssociatedImage(const gl::Context *context,
                                          const gl::ImageIndex &index,
                                          Image11 *incomingImage) override;
+    void onLabelUpdate() override;
 
   protected:
     angle::Result getSwizzleTexture(const gl::Context *context,
@@ -677,13 +724,18 @@ class TextureStorage11_2DArray : public TextureStorage11
                              GLsizei width,
                              GLsizei height,
                              GLsizei depth,
-                             int levels);
+                             int levels,
+                             const std::string &label);
     ~TextureStorage11_2DArray() override;
 
     angle::Result onDestroy(const gl::Context *context) override;
 
     angle::Result getResource(const gl::Context *context,
                               const TextureHelper11 **outResource) override;
+    angle::Result findRenderTarget(const gl::Context *context,
+                                   const gl::ImageIndex &index,
+                                   GLsizei samples,
+                                   RenderTargetD3D **outRT) const override;
     angle::Result getRenderTarget(const gl::Context *context,
                                   const gl::ImageIndex &index,
                                   GLsizei samples,
@@ -695,6 +747,7 @@ class TextureStorage11_2DArray : public TextureStorage11
     angle::Result releaseAssociatedImage(const gl::Context *context,
                                          const gl::ImageIndex &index,
                                          Image11 *incomingImage) override;
+    void onLabelUpdate() override;
 
     struct LevelLayerRangeKey
     {
@@ -771,19 +824,25 @@ class TextureStorage11_2DMultisample final : public TextureStorage11ImmutableBas
                                    GLsizei height,
                                    int levels,
                                    int samples,
-                                   bool fixedSampleLocations);
+                                   bool fixedSampleLocations,
+                                   const std::string &label);
     ~TextureStorage11_2DMultisample() override;
 
     angle::Result onDestroy(const gl::Context *context) override;
 
     angle::Result getResource(const gl::Context *context,
                               const TextureHelper11 **outResource) override;
+    angle::Result findRenderTarget(const gl::Context *context,
+                                   const gl::ImageIndex &index,
+                                   GLsizei samples,
+                                   RenderTargetD3D **outRT) const override;
     angle::Result getRenderTarget(const gl::Context *context,
                                   const gl::ImageIndex &index,
                                   GLsizei samples,
                                   RenderTargetD3D **outRT) override;
 
     angle::Result copyToStorage(const gl::Context *context, TextureStorage *destStorage) override;
+    void onLabelUpdate() override;
 
   protected:
     angle::Result getSwizzleTexture(const gl::Context *context,
@@ -822,19 +881,25 @@ class TextureStorage11_2DMultisampleArray final : public TextureStorage11Immutab
                                         GLsizei depth,
                                         int levels,
                                         int samples,
-                                        bool fixedSampleLocations);
+                                        bool fixedSampleLocations,
+                                        const std::string &label);
     ~TextureStorage11_2DMultisampleArray() override;
 
     angle::Result onDestroy(const gl::Context *context) override;
 
     angle::Result getResource(const gl::Context *context,
                               const TextureHelper11 **outResource) override;
+    angle::Result findRenderTarget(const gl::Context *context,
+                                   const gl::ImageIndex &index,
+                                   GLsizei samples,
+                                   RenderTargetD3D **outRT) const override;
     angle::Result getRenderTarget(const gl::Context *context,
                                   const gl::ImageIndex &index,
                                   GLsizei samples,
                                   RenderTargetD3D **outRT) override;
 
     angle::Result copyToStorage(const gl::Context *context, TextureStorage *destStorage) override;
+    void onLabelUpdate() override;
 
   protected:
     angle::Result getSwizzleTexture(const gl::Context *context,
@@ -869,6 +934,69 @@ class TextureStorage11_2DMultisampleArray final : public TextureStorage11Immutab
     unsigned int mSamples;
     GLboolean mFixedSampleLocations;
 };
+
+class TextureStorage11_Buffer : public TextureStorage11
+{
+  public:
+    TextureStorage11_Buffer(Renderer11 *renderer,
+                            const gl::OffsetBindingPointer<gl::Buffer> &buffer,
+                            GLenum internalFormat,
+                            const std::string &label);
+    ~TextureStorage11_Buffer() override;
+
+    angle::Result getResource(const gl::Context *context,
+                              const TextureHelper11 **outResource) override;
+    angle::Result getMippedResource(const gl::Context *context,
+                                    const TextureHelper11 **outResource) override;
+    angle::Result findRenderTarget(const gl::Context *context,
+                                   const gl::ImageIndex &index,
+                                   GLsizei samples,
+                                   RenderTargetD3D **outRT) const override;
+    angle::Result getRenderTarget(const gl::Context *context,
+                                  const gl::ImageIndex &index,
+                                  GLsizei samples,
+                                  RenderTargetD3D **outRT) override;
+
+    void onLabelUpdate() override;
+
+    void associateImage(Image11 *image, const gl::ImageIndex &index) override;
+    void disassociateImage(const gl::ImageIndex &index, Image11 *expectedImage) override;
+    void verifyAssociatedImageValid(const gl::ImageIndex &index, Image11 *expectedImage) override;
+    angle::Result releaseAssociatedImage(const gl::Context *context,
+                                         const gl::ImageIndex &index,
+                                         Image11 *incomingImage) override;
+
+  protected:
+    angle::Result getSwizzleTexture(const gl::Context *context,
+                                    const TextureHelper11 **outTexture) override;
+    angle::Result getSwizzleRenderTarget(const gl::Context *context,
+                                         int mipLevel,
+                                         const d3d11::RenderTargetView **outRTV) override;
+
+  private:
+    angle::Result createSRVForSampler(const gl::Context *context,
+                                      int baseLevel,
+                                      int mipLevels,
+                                      DXGI_FORMAT format,
+                                      const TextureHelper11 &texture,
+                                      d3d11::SharedSRV *outSRV) override;
+    angle::Result createSRVForImage(const gl::Context *context,
+                                    int level,
+                                    DXGI_FORMAT format,
+                                    const TextureHelper11 &texture,
+                                    d3d11::SharedSRV *outSRV) override;
+    angle::Result createUAVForImage(const gl::Context *context,
+                                    int level,
+                                    DXGI_FORMAT format,
+                                    const TextureHelper11 &texture,
+                                    d3d11::SharedUAV *outUAV) override;
+
+    angle::Result initTexture(const gl::Context *context);
+
+    TextureHelper11 mTexture;
+    const gl::OffsetBindingPointer<gl::Buffer> &mBuffer;
+    GLint64 mDataSize;
+};
 }  // namespace rx
 
 #endif  // LIBANGLE_RENDERER_D3D_D3D11_TEXTURESTORAGE11_H_
