@@ -21,20 +21,20 @@ enum VendorID : uint32_t
     VENDOR_ID_AMD     = 0x1002,
     VENDOR_ID_APPLE   = 0x106B,
     VENDOR_ID_ARM     = 0x13B5,
-    VENDOR_ID_MICROSOFT = 0x1414,
     // Broadcom devices won't use PCI, but this is their Vulkan vendor id.
-    VENDOR_ID_BROADCOM = 0x14E4,
-    VENDOR_ID_GOOGLE   = 0x1AE0,
-    VENDOR_ID_INTEL    = 0x8086,
-    VENDOR_ID_MESA     = 0x10005,
-    VENDOR_ID_NVIDIA   = 0x10DE,
-    VENDOR_ID_POWERVR  = 0x1010,
+    VENDOR_ID_BROADCOM  = 0x14E4,
+    VENDOR_ID_GOOGLE    = 0x1AE0,
+    VENDOR_ID_INTEL     = 0x8086,
+    VENDOR_ID_MESA      = 0x10005,
+    VENDOR_ID_MICROSOFT = 0x1414,
+    VENDOR_ID_NVIDIA    = 0x10DE,
+    VENDOR_ID_POWERVR   = 0x1010,
     // This is Qualcomm PCI Vendor ID.
     // Android doesn't have a PCI bus, but all we need is a unique id.
     VENDOR_ID_QUALCOMM = 0x5143,
     VENDOR_ID_SAMSUNG  = 0x144D,
-    VENDOR_ID_VMWARE   = 0x15AD,
     VENDOR_ID_VIVANTE  = 0x9999,
+    VENDOR_ID_VMWARE   = 0x15AD,
 };
 
 enum AndroidDeviceID : uint32_t
@@ -77,6 +77,11 @@ inline bool IsGoogle(uint32_t vendorId)
     return vendorId == VENDOR_ID_GOOGLE;
 }
 
+inline bool IsMicrosoft(uint32_t vendorId)
+{
+    return vendorId == VENDOR_ID_MICROSOFT;
+}
+
 inline bool IsNvidia(uint32_t vendorId)
 {
     return vendorId == VENDOR_ID_NVIDIA;
@@ -92,11 +97,6 @@ inline bool IsQualcomm(uint32_t vendorId)
     return vendorId == VENDOR_ID_QUALCOMM;
 }
 
-inline bool IsVMWare(uint32_t vendorId)
-{
-    return vendorId == VENDOR_ID_VMWARE;
-}
-
 inline bool IsSamsung(uint32_t vendorId)
 {
     return vendorId == VENDOR_ID_SAMSUNG;
@@ -107,6 +107,11 @@ inline bool IsVivante(uint32_t vendorId)
     return vendorId == VENDOR_ID_VIVANTE;
 }
 
+inline bool IsVMWare(uint32_t vendorId)
+{
+    return vendorId == VENDOR_ID_VMWARE;
+}
+
 inline bool IsNexus5X(uint32_t vendorId, uint32_t deviceId)
 {
     return IsQualcomm(vendorId) && deviceId == ANDROID_DEVICE_ID_NEXUS5X;
@@ -159,8 +164,12 @@ bool IsBroadwell(uint32_t DeviceId);
 bool IsCherryView(uint32_t DeviceId);
 bool IsSkylake(uint32_t DeviceId);
 bool IsBroxton(uint32_t DeviceId);
-bool IsKabylake(uint32_t DeviceId);
+bool IsKabyLake(uint32_t DeviceId);
+bool IsGeminiLake(uint32_t DeviceId);
+bool IsCoffeeLake(uint32_t DeviceId);
 bool Is9thGenIntel(uint32_t DeviceId);
+bool Is11thGenIntel(uint32_t DeviceId);
+bool Is12thGenIntel(uint32_t DeviceId);
 
 // Platform helpers
 inline bool IsWindows()
@@ -181,6 +190,15 @@ inline bool IsLinux()
 #endif
 }
 
+inline bool IsChromeOS()
+{
+#if defined(ANGLE_PLATFORM_CHROMEOS)
+    return true;
+#else
+    return false;
+#endif
+}
+
 inline bool IsApple()
 {
 #if defined(ANGLE_PLATFORM_APPLE)
