@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2015 The ANGLE Project Authors. All rights reserved.
+// Copyright 2015 The ANGLE Project Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -24,8 +24,7 @@
 #include "libANGLE/renderer/gl/wgl/RendererWGL.h"
 #include "libANGLE/renderer/gl/wgl/WindowSurfaceWGL.h"
 #include "libANGLE/renderer/gl/wgl/wgl_utils.h"
-
-#include "platform/Platform.h"
+#include "platform/PlatformMethods.h"
 
 #include <EGL/eglext.h>
 #include <sstream>
@@ -91,7 +90,7 @@ class FunctionsGLWindows : public FunctionsGL
 DisplayWGL::DisplayWGL(const egl::DisplayState &state)
     : DisplayGL(state),
       mRenderer(nullptr),
-      mCurrentData(),
+      mCurrentNativeContexts(),
       mOpenGLModule(nullptr),
       mFunctionsWGL(nullptr),
       mHasWGLCreateContextRobustness(false),
@@ -106,6 +105,7 @@ DisplayWGL::DisplayWGL(const egl::DisplayState &state)
       mD3d11Module(nullptr),
       mD3D11DeviceHandle(nullptr),
       mD3D11Device(nullptr),
+      mD3D11Device1(nullptr),
       mUseARBShare(true)
 {}
 
@@ -127,7 +127,7 @@ egl::Error DisplayWGL::initializeImpl(egl::Display *display)
 {
     mDisplayAttributes = display->getAttributeMap();
 
-    mOpenGLModule = LoadLibraryA("opengl32.dll");
+    mOpenGLModule = LoadLibraryExA("opengl32.dll", NULL, LOAD_LIBRARY_SEARCH_SYSTEM32);
     if (!mOpenGLModule)
     {
         return egl::EglNotInitialized() << "Failed to load OpenGL library.";
@@ -137,28 +137,29 @@ egl::Error DisplayWGL::initializeImpl(egl::Display *display)
     mFunctionsWGL->initialize(mOpenGLModule, nullptr);
 
     // WGL can't grab extensions until it creates a context because it needs to load the driver's
-    // DLLs first. Create a dummy context to load the driver and determine which GL versions are
+    // DLLs first. Create a stub context to load the driver and determine which GL versions are
     // available.
 
     // Work around compile error from not defining "UNICODE" while Chromium does
-    const LPSTR idcArrow = MAKEINTRESOURCEA(32512);
+    const LPWSTR idcArrow = MAKEINTRESOURCEW(32512);
 
-    std::ostringstream stream;
-    stream << "ANGLE DisplayWGL " << gl::FmtHex(display) << " Intermediate Window Class";
-    std::string className = stream.str();
+    std::wostringstream stream;
+    stream << L"ANGLE DisplayWGL " << gl::FmtHex<egl::Display *, wchar_t>(display)
+           << L" Intermediate Window Class";
+    std::wstring className = stream.str();
 
-    WNDCLASSA intermediateClassDesc     = {0};
+    WNDCLASSW intermediateClassDesc     = {};
     intermediateClassDesc.style         = CS_OWNDC;
-    intermediateClassDesc.lpfnWndProc   = DefWindowProcA;
+    intermediateClassDesc.lpfnWndProc   = DefWindowProcW;
     intermediateClassDesc.cbClsExtra    = 0;
     intermediateClassDesc.cbWndExtra    = 0;
     intermediateClassDesc.hInstance     = GetModuleHandle(nullptr);
     intermediateClassDesc.hIcon         = nullptr;
-    intermediateClassDesc.hCursor       = LoadCursorA(nullptr, idcArrow);
-    intermediateClassDesc.hbrBackground = 0;
+    intermediateClassDesc.hCursor       = LoadCursorW(nullptr, idcArrow);
+    intermediateClassDesc.hbrBackground = nullptr;
     intermediateClassDesc.lpszMenuName  = nullptr;
     intermediateClassDesc.lpszClassName = className.c_str();
-    mWindowClass                        = RegisterClassA(&intermediateClassDesc);
+    mWindowClass                        = RegisterClassW(&intermediateClassDesc);
     if (!mWindowClass)
     {
         return egl::EglNotInitialized()
@@ -166,60 +167,61 @@ egl::Error DisplayWGL::initializeImpl(egl::Display *display)
                << "\":" << gl::FmtErr(HRESULT_CODE(GetLastError()));
     }
 
-    HWND dummyWindow =
-        CreateWindowExA(0, reinterpret_cast<const char *>(mWindowClass), "ANGLE Dummy Window",
+    HWND placeholderWindow =
+        CreateWindowExW(0, reinterpret_cast<LPCWSTR>(mWindowClass), L"ANGLE Placeholder Window",
                         WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
                         CW_USEDEFAULT, nullptr, nullptr, nullptr, nullptr);
-    if (!dummyWindow)
+    if (!placeholderWindow)
     {
-        return egl::EglNotInitialized() << "Failed to create dummy OpenGL window.";
+        return egl::EglNotInitialized() << "Failed to create placeholder OpenGL window.";
     }
 
-    HDC dummyDeviceContext = GetDC(dummyWindow);
-    if (!dummyDeviceContext)
+    HDC placeholderDeviceContext = GetDC(placeholderWindow);
+    if (!placeholderDeviceContext)
     {
         return egl::EglNotInitialized()
-               << "Failed to get the device context of the dummy OpenGL window.";
+               << "Failed to get the device context of the placeholder OpenGL window.";
     }
 
     const PIXELFORMATDESCRIPTOR pixelFormatDescriptor = wgl::GetDefaultPixelFormatDescriptor();
 
-    int dummyPixelFormat = ChoosePixelFormat(dummyDeviceContext, &pixelFormatDescriptor);
-    if (dummyPixelFormat == 0)
+    int placeholderPixelFormat =
+        ChoosePixelFormat(placeholderDeviceContext, &pixelFormatDescriptor);
+    if (placeholderPixelFormat == 0)
     {
         return egl::EglNotInitialized()
-               << "Could not find a compatible pixel format for the dummy OpenGL window.";
+               << "Could not find a compatible pixel format for the placeholder OpenGL window.";
     }
 
-    if (!SetPixelFormat(dummyDeviceContext, dummyPixelFormat, &pixelFormatDescriptor))
+    if (!SetPixelFormat(placeholderDeviceContext, placeholderPixelFormat, &pixelFormatDescriptor))
     {
         return egl::EglNotInitialized()
                << "Failed to set the pixel format on the intermediate OpenGL window.";
     }
 
-    HGLRC dummyWGLContext = mFunctionsWGL->createContext(dummyDeviceContext);
-    if (!dummyDeviceContext)
+    HGLRC placeholderWGLContext = mFunctionsWGL->createContext(placeholderDeviceContext);
+    if (!placeholderDeviceContext)
     {
         return egl::EglNotInitialized()
-               << "Failed to create a WGL context for the dummy OpenGL window.";
+               << "Failed to create a WGL context for the placeholder OpenGL window.";
     }
 
-    if (!mFunctionsWGL->makeCurrent(dummyDeviceContext, dummyWGLContext))
+    if (!mFunctionsWGL->makeCurrent(placeholderDeviceContext, placeholderWGLContext))
     {
-        return egl::EglNotInitialized() << "Failed to make the dummy WGL context current.";
+        return egl::EglNotInitialized() << "Failed to make the placeholder WGL context current.";
     }
 
     // Reinitialize the wgl functions to grab the extensions
-    mFunctionsWGL->initialize(mOpenGLModule, dummyDeviceContext);
+    mFunctionsWGL->initialize(mOpenGLModule, placeholderDeviceContext);
 
     mHasWGLCreateContextRobustness =
         mFunctionsWGL->hasExtension("WGL_ARB_create_context_robustness");
 
-    // Destroy the dummy window and context
-    mFunctionsWGL->makeCurrent(dummyDeviceContext, nullptr);
-    mFunctionsWGL->deleteContext(dummyWGLContext);
-    ReleaseDC(dummyWindow, dummyDeviceContext);
-    DestroyWindow(dummyWindow);
+    // Destroy the placeholder window and context
+    mFunctionsWGL->makeCurrent(placeholderDeviceContext, nullptr);
+    mFunctionsWGL->deleteContext(placeholderWGLContext);
+    ReleaseDC(placeholderWindow, placeholderDeviceContext);
+    DestroyWindow(placeholderWindow);
 
     const egl::AttributeMap &displayAttributes = display->getAttributeMap();
     EGLint requestedDisplayType                = static_cast<EGLint>(displayAttributes.get(
@@ -354,7 +356,7 @@ void DisplayWGL::destroy()
             mFunctionsWGL->makeCurrent(mDeviceContext, nullptr);
         }
     }
-    mCurrentData.clear();
+    mCurrentNativeContexts.clear();
 
     SafeDelete(mFunctionsWGL);
 
@@ -387,6 +389,7 @@ void DisplayWGL::destroy()
     }
 
     SafeRelease(mD3D11Device);
+    SafeRelease(mD3D11Device1);
 
     if (mDxgiModule)
     {
@@ -451,8 +454,8 @@ SurfaceImpl *DisplayWGL::createPbufferFromClientBuffer(const egl::SurfaceState &
     }
 
     return new D3DTextureSurfaceWGL(state, mRenderer->getStateManager(), buftype, clientBuffer,
-                                    this, mDeviceContext, mD3D11Device, mRenderer->getFunctions(),
-                                    mFunctionsWGL);
+                                    this, mDeviceContext, mD3D11Device, mD3D11Device1,
+                                    mRenderer->getFunctions(), mFunctionsWGL);
 }
 
 SurfaceImpl *DisplayWGL::createPixmapSurface(const egl::SurfaceState &state,
@@ -472,12 +475,6 @@ rx::ContextImpl *DisplayWGL::createContext(const gl::State &state,
     return new ContextWGL(state, errorSet, mRenderer);
 }
 
-DeviceImpl *DisplayWGL::createDevice()
-{
-    UNREACHABLE();
-    return nullptr;
-}
-
 egl::ConfigSet DisplayWGL::generateConfigs()
 {
     egl::ConfigSet configs;
@@ -557,11 +554,6 @@ egl::ConfigSet DisplayWGL::generateConfigs()
 
 bool DisplayWGL::testDeviceLost()
 {
-    if (mHasRobustness)
-    {
-        return mRenderer->getResetStatus() != gl::GraphicsResetStatus::NoError;
-    }
-
     return false;
 }
 
@@ -585,20 +577,14 @@ egl::Error DisplayWGL::validateClientBuffer(const egl::Config *configuration,
         case EGL_D3D_TEXTURE_ANGLE:
         case EGL_D3D_TEXTURE_2D_SHARE_HANDLE_ANGLE:
             ANGLE_TRY(const_cast<DisplayWGL *>(this)->initializeD3DDevice());
-            return D3DTextureSurfaceWGL::ValidateD3DTextureClientBuffer(buftype, clientBuffer,
-                                                                        mD3D11Device);
+            return D3DTextureSurfaceWGL::ValidateD3DTextureClientBuffer(
+                buftype, clientBuffer, mD3D11Device, mD3D11Device1);
 
         default:
             return DisplayGL::validateClientBuffer(configuration, buftype, clientBuffer, attribs);
     }
 }
 
-std::string DisplayWGL::getVendorString() const
-{
-    // UNIMPLEMENTED();
-    return "";
-}
-
 egl::Error DisplayWGL::initializeD3DDevice()
 {
     if (mD3D11Device != nullptr)
@@ -633,6 +619,9 @@ egl::Error DisplayWGL::initializeD3DDevice()
         return egl::EglNotInitialized() << "Could not create D3D11 device, " << gl::FmtHR(result);
     }
 
+    mD3D11Device->QueryInterface(__uuidof(ID3D11Device1),
+                                 reinterpret_cast<void **>(&mD3D11Device1));
+
     return registerD3DDevice(mD3D11Device, &mD3D11DeviceHandle);
 }
 
@@ -651,8 +640,9 @@ void DisplayWGL::generateExtensions(egl::DisplayExtensions *outExtensions) const
     outExtensions->querySurfacePointer            = true;
     outExtensions->keyedMutex                     = true;
 
-    // Contexts are virtualized so textures can be shared globally
-    outExtensions->displayTextureShareGroup = true;
+    // Contexts are virtualized so textures and semaphores can be shared globally
+    outExtensions->displayTextureShareGroup   = true;
+    outExtensions->displaySemaphoreShareGroup = true;
 
     outExtensions->surfacelessContext = true;
 
@@ -683,22 +673,19 @@ egl::Error DisplayWGL::waitNative(const gl::Context *context, EGLint engine)
     return egl::NoError();
 }
 
-egl::Error DisplayWGL::makeCurrent(egl::Surface *drawSurface,
+egl::Error DisplayWGL::makeCurrent(egl::Display *display,
+                                   egl::Surface *drawSurface,
                                    egl::Surface *readSurface,
                                    gl::Context *context)
 {
-    CurrentNativeContext &currentContext = mCurrentData[std::this_thread::get_id()];
+    CurrentNativeContext &currentContext = mCurrentNativeContexts[std::this_thread::get_id()];
 
-    HDC newDC = currentContext.dc;
+    HDC newDC = mDeviceContext;
     if (drawSurface)
     {
         SurfaceWGL *drawSurfaceWGL = GetImplAs<SurfaceWGL>(drawSurface);
         newDC                      = drawSurfaceWGL->getDC();
     }
-    else
-    {
-        newDC = mDeviceContext;
-    }
 
     HGLRC newContext = currentContext.glrc;
     if (context)
@@ -706,7 +693,7 @@ egl::Error DisplayWGL::makeCurrent(egl::Surface *drawSurface,
         ContextWGL *contextWGL = GetImplAs<ContextWGL>(context);
         newContext             = contextWGL->getContext();
     }
-    else
+    else if (!mUseDXGISwapChains)
     {
         newContext = 0;
     }
@@ -724,7 +711,7 @@ egl::Error DisplayWGL::makeCurrent(egl::Surface *drawSurface,
         currentContext.glrc = newContext;
     }
 
-    return DisplayGL::makeCurrent(drawSurface, readSurface, context);
+    return DisplayGL::makeCurrent(display, drawSurface, readSurface, context);
 }
 
 egl::Error DisplayWGL::registerD3DDevice(IUnknown *device, HANDLE *outHandle)
@@ -920,7 +907,7 @@ egl::Error DisplayWGL::createRenderer(std::shared_ptr<RendererWGL> *outRenderer)
     {
         return egl::EglNotInitialized() << "Failed to make the intermediate WGL context current.";
     }
-    CurrentNativeContext &currentContext = mCurrentData[std::this_thread::get_id()];
+    CurrentNativeContext &currentContext = mCurrentNativeContexts[std::this_thread::get_id()];
     currentContext.dc                    = mDeviceContext;
     currentContext.glrc                  = context;
 
@@ -1058,4 +1045,19 @@ WorkerContext *DisplayWGL::createWorkerContext(std::string *infoLog,
     return new WorkerContextWGL(mFunctionsWGL, workerPbuffer, workerDeviceContext, workerContext);
 }
 
+void DisplayWGL::initializeFrontendFeatures(angle::FrontendFeatures *features) const
+{
+    mRenderer->initializeFrontendFeatures(features);
+}
+
+void DisplayWGL::populateFeatureList(angle::FeatureList *features)
+{
+    mRenderer->getFeatures().populateFeatureList(features);
+}
+
+RendererGL *DisplayWGL::getRenderer() const
+{
+    return reinterpret_cast<RendererGL *>(mRenderer.get());
+}
+
 }  // namespace rx
