@@ -141,7 +141,6 @@ class Std140BlockLayoutEncoderFactory : public gl::CustomBlockLayoutEncoderFacto
 ProgramVk::ProgramVk(const gl::ProgramState &state) : ProgramImpl(state)
 {
     GlslangWrapperVk::ResetGlslangProgramInterfaceInfo(&mGlslangProgramInterfaceInfo);
-    mExecutable.setProgram(this);
 }
 
 ProgramVk::~ProgramVk() = default;
@@ -154,8 +153,6 @@ void ProgramVk::destroy(const gl::Context *context)
 
 void ProgramVk::reset(ContextVk *contextVk)
 {
-    mOriginalShaderInfo.release(contextVk);
-
     GlslangWrapperVk::ResetGlslangProgramInterfaceInfo(&mGlslangProgramInterfaceInfo);
 
     mExecutable.reset(contextVk);
@@ -166,66 +163,16 @@ std::unique_ptr<rx::LinkEvent> ProgramVk::load(const gl::Context *context,
                                                gl::InfoLog &infoLog)
 {
     ContextVk *contextVk = vk::GetImpl(context);
-    gl::ShaderMap<size_t> requiredBufferSize;
-    requiredBufferSize.fill(0);
 
     reset(contextVk);
 
-    mOriginalShaderInfo.load(stream);
-    mExecutable.load(stream);
-
-    // Deserializes the uniformLayout data of mDefaultUniformBlocks
-    for (gl::ShaderType shaderType : gl::AllShaderTypes())
-    {
-        const size_t uniformCount = stream->readInt<size_t>();
-        for (unsigned int uniformIndex = 0; uniformIndex < uniformCount; ++uniformIndex)
-        {
-            sh::BlockMemberInfo blockInfo;
-            gl::LoadBlockMemberInfo(stream, &blockInfo);
-            mDefaultUniformBlocks[shaderType].uniformLayout.push_back(blockInfo);
-        }
-    }
-
-    // Deserializes required uniform block memory sizes
-    for (gl::ShaderType shaderType : gl::AllShaderTypes())
-    {
-        requiredBufferSize[shaderType] = stream->readInt<size_t>();
-    }
-
-    // Initialize and resize the mDefaultUniformBlocks' memory
-    angle::Result status = resizeUniformBlockMemory(contextVk, requiredBufferSize);
-    if (status != angle::Result::Continue)
-    {
-        return std::make_unique<LinkEventDone>(status);
-    }
-
-    status = mExecutable.createPipelineLayout(contextVk, mState.getExecutable(), nullptr);
-    return std::make_unique<LinkEventDone>(status);
+    return mExecutable.load(contextVk, mState.getExecutable(), mState.isSeparable(), stream);
 }
 
 void ProgramVk::save(const gl::Context *context, gl::BinaryOutputStream *stream)
 {
-    mOriginalShaderInfo.save(stream);
-    mExecutable.save(stream);
-
-    // Serializes the uniformLayout data of mDefaultUniformBlocks
-    for (gl::ShaderType shaderType : gl::AllShaderTypes())
-    {
-        const size_t uniformCount = mDefaultUniformBlocks[shaderType].uniformLayout.size();
-        stream->writeInt(uniformCount);
-        for (unsigned int uniformIndex = 0; uniformIndex < uniformCount; ++uniformIndex)
-        {
-            sh::BlockMemberInfo &blockInfo =
-                mDefaultUniformBlocks[shaderType].uniformLayout[uniformIndex];
-            gl::WriteBlockMemberInfo(stream, blockInfo);
-        }
-    }
-
-    // Serializes required uniform block memory sizes
-    for (gl::ShaderType shaderType : gl::AllShaderTypes())
-    {
-        stream->writeInt(mDefaultUniformBlocks[shaderType].uniformData.size());
-    }
+    ContextVk *contextVk = vk::GetImpl(context);
+    mExecutable.save(contextVk, mState.isSeparable(), stream);
 }
 
 void ProgramVk::setBinaryRetrievableHint(bool retrievable)
@@ -238,20 +185,6 @@ void ProgramVk::setSeparable(bool separable)
     // Nothing to do here yet.
 }
 
-// TODO: http://anglebug.com/3570: Move/Copy all of the necessary information into
-// the ProgramExecutable, so this function can be removed.
-void ProgramVk::fillProgramStateMap(gl::ShaderMap<const gl::ProgramState *> *programStatesOut)
-{
-    for (gl::ShaderType shaderType : gl::AllShaderTypes())
-    {
-        (*programStatesOut)[shaderType] = nullptr;
-        if (mState.getExecutable().hasLinkedShaderStage(shaderType))
-        {
-            (*programStatesOut)[shaderType] = &mState;
-        }
-    }
-}
-
 std::unique_ptr<LinkEvent> ProgramVk::link(const gl::Context *context,
                                            const gl::ProgramLinkedResources &resources,
                                            gl::InfoLog &infoLog,
@@ -273,9 +206,15 @@ std::unique_ptr<LinkEvent> ProgramVk::link(const gl::Context *context,
                                     &mGlslangProgramInterfaceInfo, &spirvBlobs,
                                     &mExecutable.mVariableInfoMap);
 
+    if (contextVk->getFeatures().enablePrecisionQualifiers.enabled)
+    {
+        mExecutable.resolvePrecisionMismatch(mergedVaryings);
+    }
+
     // Compile the shaders.
-    angle::Result status = mOriginalShaderInfo.initShaders(
-        mState.getExecutable().getLinkedShaderStages(), spirvBlobs, mExecutable.mVariableInfoMap);
+    const gl::ProgramExecutable &programExecutable = mState.getExecutable();
+    angle::Result status                           = mExecutable.mOriginalShaderInfo.initShaders(
+                                  programExecutable.getLinkedShaderStages(), spirvBlobs, mExecutable.mVariableInfoMap);
     if (status != angle::Result::Continue)
     {
         return std::make_unique<LinkEventDone>(status);
@@ -287,14 +226,20 @@ std::unique_ptr<LinkEvent> ProgramVk::link(const gl::Context *context,
         return std::make_unique<LinkEventDone>(status);
     }
 
-    if (contextVk->getFeatures().enablePrecisionQualifiers.enabled)
-    {
-        mExecutable.resolvePrecisionMismatch(mergedVaryings);
-    }
-
     // TODO(jie.a.chen@intel.com): Parallelize linking.
     // http://crbug.com/849576
-    status = mExecutable.createPipelineLayout(contextVk, mState.getExecutable(), nullptr);
+    status = mExecutable.createPipelineLayout(contextVk, programExecutable, nullptr);
+    if (status != angle::Result::Continue)
+    {
+        return std::make_unique<LinkEventDone>(status);
+    }
+
+    // Warm up the pipeline cache by creating a few placeholder pipelines.  This is not done for
+    // separable programs, and is deferred to when the program pipeline is finalized.
+    if (!mState.isSeparable())
+    {
+        status = mExecutable.warmUpPipelineCache(contextVk, programExecutable);
+    }
     return std::make_unique<LinkEventDone>(status);
 }
 
@@ -319,7 +264,8 @@ angle::Result ProgramVk::initDefaultUniformBlocks(const gl::Context *glContext)
     initDefaultUniformLayoutMapping(layoutMap);
 
     // All uniform initializations are complete, now resize the buffers accordingly and return
-    return resizeUniformBlockMemory(contextVk, requiredBufferSize);
+    return mExecutable.resizeUniformBlockMemory(contextVk, mState.getExecutable(),
+                                                requiredBufferSize);
 }
 
 void ProgramVk::generateUniformLayoutMapping(gl::ShaderMap<sh::BlockLayoutMap> &layoutMap,
@@ -382,35 +328,12 @@ void ProgramVk::initDefaultUniformLayoutMapping(gl::ShaderMap<sh::BlockLayoutMap
 
         for (const gl::ShaderType shaderType : glExecutable.getLinkedShaderStages())
         {
-            mDefaultUniformBlocks[shaderType].uniformLayout.push_back(layoutInfo[shaderType]);
+            mExecutable.mDefaultUniformBlocks[shaderType]->uniformLayout.push_back(
+                layoutInfo[shaderType]);
         }
     }
 }
 
-angle::Result ProgramVk::resizeUniformBlockMemory(ContextVk *contextVk,
-                                                  gl::ShaderMap<size_t> &requiredBufferSize)
-{
-    const gl::ProgramExecutable &glExecutable = mState.getExecutable();
-
-    for (const gl::ShaderType shaderType : glExecutable.getLinkedShaderStages())
-    {
-        if (requiredBufferSize[shaderType] > 0)
-        {
-            if (!mDefaultUniformBlocks[shaderType].uniformData.resize(
-                    requiredBufferSize[shaderType]))
-            {
-                ANGLE_VK_CHECK(contextVk, false, VK_ERROR_OUT_OF_HOST_MEMORY);
-            }
-
-            // Initialize uniform buffer memory to zero by default.
-            mDefaultUniformBlocks[shaderType].uniformData.fill(0);
-            mDefaultUniformBlocksDirty.set(shaderType);
-        }
-    }
-
-    return angle::Result::Continue;
-}
-
 GLboolean ProgramVk::validate(const gl::Caps &caps, gl::InfoLog *infoLog)
 {
     // No-op. The spec is very vague about the behavior of validation.
@@ -430,7 +353,7 @@ void ProgramVk::setUniformImpl(GLint location, GLsizei count, const T *v, GLenum
     {
         for (const gl::ShaderType shaderType : glExecutable.getLinkedShaderStages())
         {
-            DefaultUniformBlock &uniformBlock     = mDefaultUniformBlocks[shaderType];
+            DefaultUniformBlock &uniformBlock     = *mExecutable.mDefaultUniformBlocks[shaderType];
             const sh::BlockMemberInfo &layoutInfo = uniformBlock.uniformLayout[location];
 
             // Assume an offset of -1 means the block is unused.
@@ -442,14 +365,14 @@ void ProgramVk::setUniformImpl(GLint location, GLsizei count, const T *v, GLenum
             const GLint componentCount = linkedUniform.typeInfo->componentCount;
             UpdateDefaultUniformBlock(count, locationInfo.arrayIndex, componentCount, v, layoutInfo,
                                       &uniformBlock.uniformData);
-            mDefaultUniformBlocksDirty.set(shaderType);
+            mExecutable.mDefaultUniformBlocksDirty.set(shaderType);
         }
     }
     else
     {
         for (const gl::ShaderType shaderType : glExecutable.getLinkedShaderStages())
         {
-            DefaultUniformBlock &uniformBlock     = mDefaultUniformBlocks[shaderType];
+            DefaultUniformBlock &uniformBlock     = *mExecutable.mDefaultUniformBlocks[shaderType];
             const sh::BlockMemberInfo &layoutInfo = uniformBlock.uniformLayout[location];
 
             // Assume an offset of -1 means the block is unused.
@@ -477,7 +400,7 @@ void ProgramVk::setUniformImpl(GLint location, GLsizei count, const T *v, GLenum
                 }
             }
 
-            mDefaultUniformBlocksDirty.set(shaderType);
+            mExecutable.mDefaultUniformBlocksDirty.set(shaderType);
         }
     }
 }
@@ -493,7 +416,7 @@ void ProgramVk::getUniformImpl(GLint location, T *v, GLenum entryPointType) cons
     const gl::ShaderType shaderType = linkedUniform.getFirstShaderTypeWhereActive();
     ASSERT(shaderType != gl::ShaderType::InvalidEnum);
 
-    const DefaultUniformBlock &uniformBlock = mDefaultUniformBlocks[shaderType];
+    const DefaultUniformBlock &uniformBlock = *mExecutable.mDefaultUniformBlocks[shaderType];
     const sh::BlockMemberInfo &layoutInfo   = uniformBlock.uniformLayout[location];
 
     ASSERT(linkedUniform.typeInfo->componentType == entryPointType ||
@@ -593,7 +516,7 @@ void ProgramVk::setUniformMatrixfv(GLint location,
 
     for (const gl::ShaderType shaderType : glExecutable.getLinkedShaderStages())
     {
-        DefaultUniformBlock &uniformBlock     = mDefaultUniformBlocks[shaderType];
+        DefaultUniformBlock &uniformBlock     = *mExecutable.mDefaultUniformBlocks[shaderType];
         const sh::BlockMemberInfo &layoutInfo = uniformBlock.uniformLayout[location];
 
         // Assume an offset of -1 means the block is unused.
@@ -606,7 +529,7 @@ void ProgramVk::setUniformMatrixfv(GLint location,
             locationInfo.arrayIndex, linkedUniform.getArraySizeProduct(), count, transpose, value,
             uniformBlock.uniformData.data() + layoutInfo.offset);
 
-        mDefaultUniformBlocksDirty.set(shaderType);
+        mExecutable.mDefaultUniformBlocksDirty.set(shaderType);
     }
 }
 
@@ -696,134 +619,4 @@ void ProgramVk::getUniformuiv(const gl::Context *context, GLint location, GLuint
 {
     getUniformImpl(location, params, GL_UNSIGNED_INT);
 }
-
-size_t ProgramVk::calcUniformUpdateRequiredSpace(ContextVk *contextVk,
-                                                 const gl::ProgramExecutable &glExecutable,
-                                                 gl::ShaderMap<VkDeviceSize> &uniformOffsets) const
-{
-    size_t requiredSpace = 0;
-    for (const gl::ShaderType shaderType : glExecutable.getLinkedShaderStages())
-    {
-        if (mDefaultUniformBlocksDirty[shaderType])
-        {
-            uniformOffsets[shaderType] = requiredSpace;
-            requiredSpace += getDefaultUniformAlignedSize(contextVk, shaderType);
-        }
-    }
-    return requiredSpace;
-}
-
-angle::Result ProgramVk::updateUniforms(ContextVk *contextVk)
-{
-    ASSERT(hasDirtyUniforms());
-
-    bool anyNewBufferAllocated                = false;
-    uint8_t *bufferData                       = nullptr;
-    VkDeviceSize bufferOffset                 = 0;
-    uint32_t offsetIndex                      = 0;
-    const gl::ProgramExecutable &glExecutable = mState.getExecutable();
-    gl::ShaderMap<VkDeviceSize> offsets;  // offset to the beginning of bufferData
-    size_t requiredSpace;
-
-    // We usually only update uniform data for shader stages that are actually dirty. But when the
-    // buffer for uniform data have switched, because all shader stages are using the same buffer,
-    // we then must update uniform data for all shader stages to keep all shader stages' uniform
-    // data in the same buffer.
-    requiredSpace = calcUniformUpdateRequiredSpace(contextVk, glExecutable, offsets);
-    ASSERT(requiredSpace > 0);
-
-    // Allocate space from dynamicBuffer. Always try to allocate from the current buffer first.
-    // If that failed, we deal with fall out and try again.
-    vk::DynamicBuffer *defaultUniformStorage = contextVk->getDefaultUniformStorage();
-    if (!defaultUniformStorage->allocateFromCurrentBuffer(requiredSpace, &bufferData,
-                                                          &bufferOffset))
-    {
-        for (const gl::ShaderType shaderType : glExecutable.getLinkedShaderStages())
-        {
-            if (!mDefaultUniformBlocks[shaderType].uniformData.empty())
-            {
-                mDefaultUniformBlocksDirty.set(shaderType);
-            }
-        }
-
-        requiredSpace = calcUniformUpdateRequiredSpace(contextVk, glExecutable, offsets);
-        ANGLE_TRY(defaultUniformStorage->allocate(contextVk, requiredSpace, &bufferData, nullptr,
-                                                  &bufferOffset, &anyNewBufferAllocated));
-    }
-
-    // Update buffer memory by immediate mapping. This immediate update only works once.
-    for (const gl::ShaderType shaderType : glExecutable.getLinkedShaderStages())
-    {
-        if (mDefaultUniformBlocksDirty[shaderType])
-        {
-            const angle::MemoryBuffer &uniformData = mDefaultUniformBlocks[shaderType].uniformData;
-            memcpy(&bufferData[offsets[shaderType]], uniformData.data(), uniformData.size());
-            mExecutable.mDynamicUniformDescriptorOffsets[offsetIndex] =
-                static_cast<uint32_t>(bufferOffset + offsets[shaderType]);
-            mDefaultUniformBlocksDirty.reset(shaderType);
-        }
-        ++offsetIndex;
-    }
-    ANGLE_TRY(defaultUniformStorage->flush(contextVk));
-
-    vk::BufferHelper *defaultUniformBuffer = defaultUniformStorage->getCurrentBuffer();
-    if (mExecutable.getCurrentDefaultUniformBufferSerial() !=
-        defaultUniformBuffer->getBufferSerial())
-    {
-        // We need to reinitialize the descriptor sets if we newly allocated buffers since we can't
-        // modify the descriptor sets once initialized.
-        vk::UniformsAndXfbDescriptorDesc defaultUniformsDesc;
-        vk::UniformsAndXfbDescriptorDesc *uniformsAndXfbBufferDesc;
-
-        if (glExecutable.hasTransformFeedbackOutput())
-        {
-            const gl::State &glState = contextVk->getState();
-            TransformFeedbackVk *transformFeedbackVk =
-                vk::GetImpl(glState.getCurrentTransformFeedback());
-            uniformsAndXfbBufferDesc = &transformFeedbackVk->getTransformFeedbackDesc();
-            uniformsAndXfbBufferDesc->updateDefaultUniformBuffer(
-                defaultUniformBuffer->getBufferSerial());
-        }
-        else
-        {
-            defaultUniformsDesc.updateDefaultUniformBuffer(defaultUniformBuffer->getBufferSerial());
-            uniformsAndXfbBufferDesc = &defaultUniformsDesc;
-        }
-
-        bool newDescriptorSetAllocated;
-        ANGLE_TRY(mExecutable.allocUniformAndXfbDescriptorSet(contextVk, *uniformsAndXfbBufferDesc,
-                                                              &newDescriptorSetAllocated));
-        if (newDescriptorSetAllocated)
-        {
-            // Update the descriptor set with the bufferInfo
-            for (const gl::ShaderType shaderType : glExecutable.getLinkedShaderStages())
-            {
-                mExecutable.updateDefaultUniformsDescriptorSet(
-                    shaderType, mDefaultUniformBlocks[shaderType], defaultUniformBuffer, contextVk);
-            }
-            mExecutable.updateTransformFeedbackDescriptorSetImpl(mState, contextVk);
-        }
-    }
-
-    return angle::Result::Continue;
-}
-
-void ProgramVk::setAllDefaultUniformsDirty()
-{
-    const gl::ProgramExecutable &glExecutable = mState.getExecutable();
-    for (const gl::ShaderType shaderType : glExecutable.getLinkedShaderStages())
-    {
-        setShaderUniformDirtyBit(shaderType);
-    }
-}
-
-void ProgramVk::onProgramBind()
-{
-    // Because all programs share default uniform buffers, when we switch programs, we have to
-    // re-update all uniform data. We could do more tracking to avoid update if the context's
-    // current uniform buffer is still the same buffer we last time used and buffer has not been
-    // recycled. But statistics gathered on gfxbench shows that app always update uniform data on
-    // program bind anyway, so not really worth it to add more tracking logic here.
-    setAllDefaultUniformsDirty();
-}
 }  // namespace rx
