@@ -19,21 +19,23 @@ namespace rx
 {
 namespace
 {
-void FillTextureFormatCaps(RendererVk *renderer, VkFormat format, gl::TextureCaps *outTextureCaps)
+void FillTextureFormatCaps(RendererVk *renderer,
+                           angle::FormatID formatID,
+                           gl::TextureCaps *outTextureCaps)
 {
     const VkPhysicalDeviceLimits &physicalDeviceLimits =
         renderer->getPhysicalDeviceProperties().limits;
     bool hasColorAttachmentFeatureBit =
-        renderer->hasImageFormatFeatureBits(format, VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BIT);
-    bool hasDepthAttachmentFeatureBit =
-        renderer->hasImageFormatFeatureBits(format, VK_FORMAT_FEATURE_DEPTH_STENCIL_ATTACHMENT_BIT);
+        renderer->hasImageFormatFeatureBits(formatID, VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BIT);
+    bool hasDepthAttachmentFeatureBit = renderer->hasImageFormatFeatureBits(
+        formatID, VK_FORMAT_FEATURE_DEPTH_STENCIL_ATTACHMENT_BIT);
 
     outTextureCaps->texturable =
-        renderer->hasImageFormatFeatureBits(format, VK_FORMAT_FEATURE_SAMPLED_IMAGE_BIT);
+        renderer->hasImageFormatFeatureBits(formatID, VK_FORMAT_FEATURE_SAMPLED_IMAGE_BIT);
     outTextureCaps->filterable = renderer->hasImageFormatFeatureBits(
-        format, VK_FORMAT_FEATURE_SAMPLED_IMAGE_FILTER_LINEAR_BIT);
+        formatID, VK_FORMAT_FEATURE_SAMPLED_IMAGE_FILTER_LINEAR_BIT);
     outTextureCaps->blendable =
-        renderer->hasImageFormatFeatureBits(format, VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BLEND_BIT);
+        renderer->hasImageFormatFeatureBits(formatID, VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BLEND_BIT);
 
     // For renderbuffer and texture attachments we require transfer and sampling for
     // GLES 2.0 CopyTexImage support. Sampling is also required for other features like
@@ -62,12 +64,18 @@ void FillTextureFormatCaps(RendererVk *renderer, VkFormat format, gl::TextureCap
     }
 }
 
-bool HasFullBufferFormatSupport(RendererVk *renderer, VkFormat vkFormat)
+bool HasFullBufferFormatSupport(RendererVk *renderer, angle::FormatID formatID)
 {
-    return renderer->hasBufferFormatFeatureBits(vkFormat, VK_FORMAT_FEATURE_VERTEX_BUFFER_BIT);
+    // Note: GL_EXT_texture_buffer support uses the same vkBufferFormat that is determined by
+    // Format::initBufferFallback, which uses this function.  That relies on the fact that formats
+    // required for GL_EXT_texture_buffer all have mandatory VERTEX_BUFFER feature support in
+    // Vulkan.  If this function is changed to test for more features in such a way that makes any
+    // of those formats use a fallback format, the implementation of GL_EXT_texture_buffer must be
+    // modified not to use vkBufferFormat.
+    return renderer->hasBufferFormatFeatureBits(formatID, VK_FORMAT_FEATURE_VERTEX_BUFFER_BIT);
 }
 
-using SupportTest = bool (*)(RendererVk *renderer, VkFormat vkFormat);
+using SupportTest = bool (*)(RendererVk *renderer, angle::FormatID formatID);
 
 template <class FormatInitInfo>
 int FindSupportedFormat(RendererVk *renderer,
@@ -82,51 +90,58 @@ int FindSupportedFormat(RendererVk *renderer,
     for (int i = static_cast<int>(skip); i < last; ++i)
     {
         ASSERT(info[i].format != angle::FormatID::NONE);
-        if (hasSupport(renderer, info[i].vkFormat))
+        if (hasSupport(renderer, info[i].format))
             return i;
     }
 
-    if (skip > 0 && !hasSupport(renderer, info[last].vkFormat))
+    if (skip > 0 && !hasSupport(renderer, info[last].format))
     {
         // We couldn't find a valid fallback, try again without skip
         return FindSupportedFormat(renderer, info, 0, numInfo, hasSupport);
     }
 
     ASSERT(info[last].format != angle::FormatID::NONE);
-    ASSERT(hasSupport(renderer, info[last].vkFormat));
+    ASSERT(hasSupport(renderer, info[last].format));
     return last;
 }
 
+bool HasNonFilterableTextureFormatSupport(RendererVk *renderer, angle::FormatID formatID)
+{
+    constexpr uint32_t kBitsColor =
+        VK_FORMAT_FEATURE_SAMPLED_IMAGE_BIT | VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BIT;
+    constexpr uint32_t kBitsDepth = VK_FORMAT_FEATURE_DEPTH_STENCIL_ATTACHMENT_BIT;
+
+    return renderer->hasImageFormatFeatureBits(formatID, kBitsColor) ||
+           renderer->hasImageFormatFeatureBits(formatID, kBitsDepth);
+}
 }  // anonymous namespace
 
 namespace vk
 {
-
 // Format implementation.
 Format::Format()
-    : intendedFormatID(angle::FormatID::NONE),
-      internalFormat(GL_NONE),
-      actualImageFormatID(angle::FormatID::NONE),
-      vkImageFormat(VK_FORMAT_UNDEFINED),
-      actualBufferFormatID(angle::FormatID::NONE),
-      vkBufferFormat(VK_FORMAT_UNDEFINED),
-      actualCompressedBufferFormatID(angle::FormatID::NONE),
-      vkCompressedBufferFormat(VK_FORMAT_UNDEFINED),
-      imageInitializerFunction(nullptr),
-      textureLoadFunctions(),
-      vertexLoadFunction(nullptr),
-      compressedVertexLoadFunction(nullptr),
-      vertexLoadRequiresConversion(false),
-      compressedVertexLoadRequiresConversion(false),
-      vkBufferFormatIsPacked(false),
-      vkFormatIsInt(false),
-      vkFormatIsUnsigned(false)
+    : mIntendedFormatID(angle::FormatID::NONE),
+      mIntendedGLFormat(GL_NONE),
+      mActualSampleOnlyImageFormatID(angle::FormatID::NONE),
+      mActualRenderableImageFormatID(angle::FormatID::NONE),
+      mActualBufferFormatID(angle::FormatID::NONE),
+      mActualCompressedBufferFormatID(angle::FormatID::NONE),
+      mImageInitializerFunction(nullptr),
+      mTextureLoadFunctions(),
+      mRenderableTextureLoadFunctions(),
+      mVertexLoadFunction(nullptr),
+      mCompressedVertexLoadFunction(nullptr),
+      mVertexLoadRequiresConversion(false),
+      mCompressedVertexLoadRequiresConversion(false),
+      mVkBufferFormatIsPacked(false),
+      mVkFormatIsInt(false),
+      mVkFormatIsUnsigned(false)
 {}
 
 void Format::initImageFallback(RendererVk *renderer, const ImageFormatInitInfo *info, int numInfo)
 {
     size_t skip                 = renderer->getFeatures().forceFallbackFormat.enabled ? 1 : 0;
-    SupportTest testFunction    = HasFullTextureFormatSupport;
+    SupportTest testFunction    = HasNonRenderableTextureFormatSupport;
     const angle::Format &format = angle::Format::Get(info[0].format);
     if (format.isInt() || (format.isFloat() && format.redBits >= 32))
     {
@@ -136,18 +151,22 @@ void Format::initImageFallback(RendererVk *renderer, const ImageFormatInitInfo *
         // enabled automatically by examining format capabilities.
         testFunction = HasNonFilterableTextureFormatSupport;
     }
-    if (format.isSnorm() || format.isBlock)
+
+    int i = FindSupportedFormat(renderer, info, skip, static_cast<uint32_t>(numInfo), testFunction);
+    mActualSampleOnlyImageFormatID = info[i].format;
+    mImageInitializerFunction      = info[i].initializer;
+
+    // Set renderable format.
+    if (testFunction != HasNonFilterableTextureFormatSupport && !format.isSnorm() &&
+        !format.isBlock)
     {
         // Rendering to SNORM textures is not supported on Android, and it's
         // enabled by the extension EXT_render_snorm.
         // Compressed textures also need to perform this check.
-        testFunction = HasNonRenderableTextureFormatSupport;
+        testFunction = HasFullTextureFormatSupport;
+        i = FindSupportedFormat(renderer, info, skip, static_cast<uint32_t>(numInfo), testFunction);
+        mActualRenderableImageFormatID = info[i].format;
     }
-    int i = FindSupportedFormat(renderer, info, skip, static_cast<uint32_t>(numInfo), testFunction);
-
-    actualImageFormatID      = info[i].format;
-    vkImageFormat            = info[i].vkFormat;
-    imageInitializerFunction = info[i].initializer;
 }
 
 void Format::initBufferFallback(RendererVk *renderer,
@@ -158,29 +177,122 @@ void Format::initBufferFallback(RendererVk *renderer,
     {
         size_t skip = renderer->getFeatures().forceFallbackFormat.enabled ? 1 : 0;
         int i       = FindSupportedFormat(renderer, info, skip, compressedStartIndex,
-                                    HasFullBufferFormatSupport);
+                                          HasFullBufferFormatSupport);
 
-        actualBufferFormatID         = info[i].format;
-        vkBufferFormat               = info[i].vkFormat;
-        vkBufferFormatIsPacked       = info[i].vkFormatIsPacked;
-        vertexLoadFunction           = info[i].vertexLoadFunction;
-        vertexLoadRequiresConversion = info[i].vertexLoadRequiresConversion;
+        mActualBufferFormatID         = info[i].format;
+        mVkBufferFormatIsPacked       = info[i].vkFormatIsPacked;
+        mVertexLoadFunction           = info[i].vertexLoadFunction;
+        mVertexLoadRequiresConversion = info[i].vertexLoadRequiresConversion;
     }
 
-    if (compressedStartIndex < numInfo)
+    if (renderer->getFeatures().compressVertexData.enabled && compressedStartIndex < numInfo)
     {
         int i = FindSupportedFormat(renderer, info, compressedStartIndex, numInfo,
                                     HasFullBufferFormatSupport);
 
-        actualCompressedBufferFormatID         = info[i].format;
-        vkCompressedBufferFormat               = info[i].vkFormat;
-        vkCompressedBufferFormatIsPacked       = info[i].vkFormatIsPacked;
-        compressedVertexLoadFunction           = info[i].vertexLoadFunction;
-        compressedVertexLoadRequiresConversion = info[i].vertexLoadRequiresConversion;
+        mActualCompressedBufferFormatID         = info[i].format;
+        mVkCompressedBufferFormatIsPacked       = info[i].vkFormatIsPacked;
+        mCompressedVertexLoadFunction           = info[i].vertexLoadFunction;
+        mCompressedVertexLoadRequiresConversion = info[i].vertexLoadRequiresConversion;
     }
 }
 
-size_t Format::getImageCopyBufferAlignment() const
+size_t Format::getVertexInputAlignment(bool compressed) const
+{
+    const angle::Format &bufferFormat = getActualBufferFormat(compressed);
+    size_t pixelBytes                 = bufferFormat.pixelBytes;
+    return mVkBufferFormatIsPacked ? pixelBytes : (pixelBytes / bufferFormat.channelCount);
+}
+
+bool HasEmulatedImageChannels(const angle::Format &intendedFormat,
+                              const angle::Format &actualFormat)
+{
+    return (intendedFormat.alphaBits == 0 && actualFormat.alphaBits > 0) ||
+           (intendedFormat.blueBits == 0 && actualFormat.blueBits > 0) ||
+           (intendedFormat.greenBits == 0 && actualFormat.greenBits > 0) ||
+           (intendedFormat.depthBits == 0 && actualFormat.depthBits > 0) ||
+           (intendedFormat.stencilBits == 0 && actualFormat.stencilBits > 0);
+}
+
+bool HasEmulatedImageFormat(angle::FormatID intendedFormatID, angle::FormatID actualFormatID)
+{
+    return actualFormatID != intendedFormatID;
+}
+
+bool operator==(const Format &lhs, const Format &rhs)
+{
+    return &lhs == &rhs;
+}
+
+bool operator!=(const Format &lhs, const Format &rhs)
+{
+    return &lhs != &rhs;
+}
+
+// FormatTable implementation.
+FormatTable::FormatTable() {}
+
+FormatTable::~FormatTable() {}
+
+void FormatTable::initialize(RendererVk *renderer, gl::TextureCapsMap *outTextureCapsMap)
+{
+    for (size_t formatIndex = 0; formatIndex < angle::kNumANGLEFormats; ++formatIndex)
+    {
+        Format &format                           = mFormatData[formatIndex];
+        const auto intendedFormatID              = static_cast<angle::FormatID>(formatIndex);
+        const angle::Format &intendedAngleFormat = angle::Format::Get(intendedFormatID);
+
+        format.initialize(renderer, intendedAngleFormat);
+        format.mIntendedFormatID = intendedFormatID;
+
+        if (!format.valid())
+        {
+            continue;
+        }
+
+        // No sample-able or render-able formats, so nothing left to do. This includes skipping the
+        // rest of the loop for buffer-only formats, since they are not texturable.
+        if (format.mActualSampleOnlyImageFormatID == angle::FormatID::NONE)
+        {
+            continue;
+        }
+
+        if (format.mActualRenderableImageFormatID == angle::FormatID::NONE)
+        {
+            // If renderable format was not set, it means there is no fallback format for
+            // renderable. We populate this the same formatID as sampleOnly formatID so that
+            // getActualFormatID() will be simpler.
+            format.mActualRenderableImageFormatID = format.mActualSampleOnlyImageFormatID;
+        }
+
+        gl::TextureCaps textureCaps;
+        FillTextureFormatCaps(renderer, format.mActualSampleOnlyImageFormatID, &textureCaps);
+
+        if (textureCaps.texturable)
+        {
+            format.mTextureLoadFunctions = GetLoadFunctionsMap(
+                format.mIntendedGLFormat, format.mActualSampleOnlyImageFormatID);
+        }
+
+        if (format.mActualRenderableImageFormatID == format.mActualSampleOnlyImageFormatID)
+        {
+            outTextureCapsMap->set(intendedFormatID, textureCaps);
+            format.mRenderableTextureLoadFunctions = format.mTextureLoadFunctions;
+        }
+        else
+        {
+            FillTextureFormatCaps(renderer, format.mActualRenderableImageFormatID, &textureCaps);
+            outTextureCapsMap->set(intendedFormatID, textureCaps);
+            if (textureCaps.texturable)
+            {
+                format.mRenderableTextureLoadFunctions = GetLoadFunctionsMap(
+                    format.mIntendedGLFormat, format.mActualRenderableImageFormatID);
+            }
+        }
+    }
+}
+
+size_t GetImageCopyBufferAlignment(angle::FormatID actualFormatID)
 {
     // vkCmdCopyBufferToImage must have an offset that is a multiple of 4 as well as a multiple
     // of the texel size (if uncompressed) or pixel block size (if compressed).
@@ -201,94 +313,33 @@ size_t Format::getImageCopyBufferAlignment() const
     // - else texelSize % 4 != 0 gives a 2x multiplier
     // - else there's no multiplier.
     //
-    const angle::Format &format = actualImageFormat();
+    const angle::Format &actualFormat = angle::Format::Get(actualFormatID);
 
-    ASSERT(format.pixelBytes != 0);
-    const size_t texelSize  = format.pixelBytes;
+    ASSERT(actualFormat.pixelBytes != 0);
+    const size_t texelSize  = actualFormat.pixelBytes;
     const size_t multiplier = texelSize % 2 != 0 ? 4 : texelSize % 4 != 0 ? 2 : 1;
     const size_t alignment  = multiplier * texelSize;
 
     return alignment;
 }
 
-size_t Format::getValidImageCopyBufferAlignment() const
+size_t GetValidImageCopyBufferAlignment(angle::FormatID intendedFormatID,
+                                        angle::FormatID actualFormatID)
 {
     constexpr size_t kMinimumAlignment = 16;
-    return (intendedFormatID == angle::FormatID::NONE) ? kMinimumAlignment
-                                                       : getImageCopyBufferAlignment();
+    return (intendedFormatID == angle::FormatID::NONE)
+               ? kMinimumAlignment
+               : GetImageCopyBufferAlignment(actualFormatID);
 }
 
-bool Format::hasEmulatedImageChannels() const
-{
-    const angle::Format &angleFmt   = intendedFormat();
-    const angle::Format &textureFmt = actualImageFormat();
-
-    return (angleFmt.alphaBits == 0 && textureFmt.alphaBits > 0) ||
-           (angleFmt.blueBits == 0 && textureFmt.blueBits > 0) ||
-           (angleFmt.greenBits == 0 && textureFmt.greenBits > 0) ||
-           (angleFmt.depthBits == 0 && textureFmt.depthBits > 0) ||
-           (angleFmt.stencilBits == 0 && textureFmt.stencilBits > 0);
-}
-
-bool operator==(const Format &lhs, const Format &rhs)
-{
-    return &lhs == &rhs;
-}
-
-bool operator!=(const Format &lhs, const Format &rhs)
-{
-    return &lhs != &rhs;
-}
-
-// FormatTable implementation.
-FormatTable::FormatTable() {}
-
-FormatTable::~FormatTable() {}
-
-void FormatTable::initialize(RendererVk *renderer,
-                             gl::TextureCapsMap *outTextureCapsMap,
-                             std::vector<GLenum> *outCompressedTextureFormats)
-{
-    for (size_t formatIndex = 0; formatIndex < angle::kNumANGLEFormats; ++formatIndex)
-    {
-        vk::Format &format               = mFormatData[formatIndex];
-        const auto formatID              = static_cast<angle::FormatID>(formatIndex);
-        const angle::Format &angleFormat = angle::Format::Get(formatID);
-
-        format.initialize(renderer, angleFormat);
-        const GLenum internalFormat = format.internalFormat;
-        format.intendedFormatID     = formatID;
-
-        if (!format.valid())
-        {
-            continue;
-        }
-
-        gl::TextureCaps textureCaps;
-        FillTextureFormatCaps(renderer, format.vkImageFormat, &textureCaps);
-        outTextureCapsMap->set(formatID, textureCaps);
-
-        if (textureCaps.texturable)
-        {
-            format.textureLoadFunctions =
-                GetLoadFunctionsMap(internalFormat, format.actualImageFormatID);
-        }
-
-        if (angleFormat.isBlock)
-        {
-            outCompressedTextureFormats->push_back(internalFormat);
-        }
-    }
-}
-
-VkImageUsageFlags GetMaximalImageUsageFlags(RendererVk *renderer, VkFormat format)
+VkImageUsageFlags GetMaximalImageUsageFlags(RendererVk *renderer, angle::FormatID formatID)
 {
     constexpr VkFormatFeatureFlags kImageUsageFeatureBits =
         VK_FORMAT_FEATURE_SAMPLED_IMAGE_BIT | VK_FORMAT_FEATURE_STORAGE_IMAGE_BIT |
         VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BIT | VK_FORMAT_FEATURE_DEPTH_STENCIL_ATTACHMENT_BIT |
         VK_FORMAT_FEATURE_TRANSFER_SRC_BIT | VK_FORMAT_FEATURE_TRANSFER_DST_BIT;
     VkFormatFeatureFlags featureBits =
-        renderer->getImageFormatFeatureBits(format, kImageUsageFeatureBits);
+        renderer->getImageFormatFeatureBits(formatID, kImageUsageFeatureBits);
     VkImageUsageFlags imageUsageFlags = 0;
     if (featureBits & VK_FORMAT_FEATURE_SAMPLED_IMAGE_BIT)
         imageUsageFlags |= VK_IMAGE_USAGE_SAMPLED_BIT;
@@ -305,10 +356,9 @@ VkImageUsageFlags GetMaximalImageUsageFlags(RendererVk *renderer, VkFormat forma
     imageUsageFlags |= VK_IMAGE_USAGE_INPUT_ATTACHMENT_BIT;
     return imageUsageFlags;
 }
-
 }  // namespace vk
 
-bool HasFullTextureFormatSupport(RendererVk *renderer, VkFormat vkFormat)
+bool HasFullTextureFormatSupport(RendererVk *renderer, angle::FormatID formatID)
 {
     constexpr uint32_t kBitsColor = VK_FORMAT_FEATURE_SAMPLED_IMAGE_BIT |
                                     VK_FORMAT_FEATURE_SAMPLED_IMAGE_FILTER_LINEAR_BIT |
@@ -317,11 +367,11 @@ bool HasFullTextureFormatSupport(RendererVk *renderer, VkFormat vkFormat)
     // In OpenGL ES, all renderable formats except 32-bit floating-point support blending.
     // 32-bit floating-point case validation is handled by ANGLE's frontend.
     uint32_t kBitsColorFull = kBitsColor;
-    switch (vkFormat)
+    switch (formatID)
     {
-        case VK_FORMAT_R32_SFLOAT:
-        case VK_FORMAT_R32G32_SFLOAT:
-        case VK_FORMAT_R32G32B32A32_SFLOAT:
+        case angle::FormatID::R32_FLOAT:
+        case angle::FormatID::R32G32_FLOAT:
+        case angle::FormatID::R32G32B32A32_FLOAT:
             break;
         default:
             kBitsColorFull |= VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BLEND_BIT;
@@ -330,35 +380,18 @@ bool HasFullTextureFormatSupport(RendererVk *renderer, VkFormat vkFormat)
 
     constexpr uint32_t kBitsDepth = VK_FORMAT_FEATURE_DEPTH_STENCIL_ATTACHMENT_BIT;
 
-    return renderer->hasImageFormatFeatureBits(vkFormat, kBitsColorFull) ||
-           renderer->hasImageFormatFeatureBits(vkFormat, kBitsDepth);
+    return renderer->hasImageFormatFeatureBits(formatID, kBitsColorFull) ||
+           renderer->hasImageFormatFeatureBits(formatID, kBitsDepth);
 }
 
-bool HasNonFilterableTextureFormatSupport(RendererVk *renderer, VkFormat vkFormat)
-{
-    constexpr uint32_t kBitsColor =
-        VK_FORMAT_FEATURE_SAMPLED_IMAGE_BIT | VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BIT;
-    constexpr uint32_t kBitsDepth = VK_FORMAT_FEATURE_DEPTH_STENCIL_ATTACHMENT_BIT;
-
-    return renderer->hasImageFormatFeatureBits(vkFormat, kBitsColor) ||
-           renderer->hasImageFormatFeatureBits(vkFormat, kBitsDepth);
-}
-
-bool HasNonRenderableTextureFormatSupport(RendererVk *renderer, VkFormat vkFormat)
+bool HasNonRenderableTextureFormatSupport(RendererVk *renderer, angle::FormatID formatID)
 {
     constexpr uint32_t kBitsColor =
         VK_FORMAT_FEATURE_SAMPLED_IMAGE_BIT | VK_FORMAT_FEATURE_SAMPLED_IMAGE_FILTER_LINEAR_BIT;
     constexpr uint32_t kBitsDepth = VK_FORMAT_FEATURE_DEPTH_STENCIL_ATTACHMENT_BIT;
 
-    return renderer->hasImageFormatFeatureBits(vkFormat, kBitsColor) ||
-           renderer->hasImageFormatFeatureBits(vkFormat, kBitsDepth);
-}
-
-size_t GetVertexInputAlignment(const vk::Format &format, bool compressed)
-{
-    const angle::Format &bufferFormat = format.actualBufferFormat(compressed);
-    size_t pixelBytes                 = bufferFormat.pixelBytes;
-    return format.vkBufferFormatIsPacked ? pixelBytes : (pixelBytes / bufferFormat.channelCount);
+    return renderer->hasImageFormatFeatureBits(formatID, kBitsColor) ||
+           renderer->hasImageFormatFeatureBits(formatID, kBitsDepth);
 }
 
 GLenum GetSwizzleStateComponent(const gl::SwizzleState &swizzleState, GLenum component)
@@ -391,12 +424,8 @@ gl::SwizzleState ApplySwizzle(const gl::SwizzleState &formatSwizzle,
     return result;
 }
 
-gl::SwizzleState GetFormatSwizzle(const ContextVk *contextVk,
-                                  const vk::Format &format,
-                                  const bool sized)
+gl::SwizzleState GetFormatSwizzle(const angle::Format &angleFormat, const bool sized)
 {
-    const angle::Format &angleFormat = format.intendedFormat();
-
     gl::SwizzleState internalSwizzle;
 
     if (angleFormat.isLUMA())
