@@ -1220,8 +1220,8 @@ TEST_F(GeometryShaderTest, NonArrayInput)
     }
 }
 
-// Verify that it is a compile error to declare an unsized Geometry Shader input before a valid
-// input primitive declaration.
+// Verify that compilation errors do not occur even if declaring an unsized Geometry Shader input
+// before a valid input primitive declaration.
 TEST_F(GeometryShaderTest, DeclareUnsizedInputBeforeInputPrimitive)
 {
     const std::string &shaderString1 =
@@ -1249,9 +1249,9 @@ TEST_F(GeometryShaderTest, DeclareUnsizedInputBeforeInputPrimitive)
             int length = texcoord2.length();
         })";
 
-    if (compile(shaderString1) || compile(shaderString2))
+    if (!compile(shaderString1) || !compile(shaderString2))
     {
-        FAIL() << "Shader compilation succeeded, expecting failure:\n" << mInfoLog;
+        FAIL() << "Shader compilation failed, expecting success:\n" << mInfoLog;
     }
 }
 
