@@ -20,6 +20,7 @@
 #include "common/platform.h"
 #include "common/string_utils.h"
 #include "common/system_utils.h"
+#include "common/third_party/base/anglebase/no_destructor.h"
 #include "platform/PlatformMethods.h"
 #include "tests/test_utils/runner/TestSuite.h"
 #include "util/OSWindow.h"
@@ -53,16 +54,6 @@ void HandlePlatformError(PlatformMethods *platform, const char *errorMessage)
     gGlobalError = true;
 }
 
-std::string DrawElementsToGoogleTestName(const std::string &dEQPName)
-{
-    std::string gTestName = dEQPName.substr(dEQPName.find('.') + 1);
-    std::replace(gTestName.begin(), gTestName.end(), '.', '_');
-
-    // Occurs in some luminance tests
-    gTestName.erase(std::remove(gTestName.begin(), gTestName.end(), '-'), gTestName.end());
-    return gTestName;
-}
-
 // Relative to the ANGLE root folder.
 constexpr char kCTSRootPath[] = "third_party/VK-GL-CTS/src/";
 constexpr char kSupportPath[] = "src/tests/deqp_support/";
@@ -70,20 +61,20 @@ constexpr char kSupportPath[] = "src/tests/deqp_support/";
 #define OPENGL_CTS_DIR(PATH) "external/openglcts/data/mustpass/gles/" PATH
 
 const char *gCaseListFiles[] = {
-    OPENGL_CTS_DIR("aosp_mustpass/master/gles2-master.txt"),
-    OPENGL_CTS_DIR("aosp_mustpass/master/gles3-master.txt"),
-    OPENGL_CTS_DIR("aosp_mustpass/master/gles31-master.txt"),
-    "/android/cts/master/egl-master.txt",
-    OPENGL_CTS_DIR("khronos_mustpass/master/gles2-khr-master.txt"),
-    OPENGL_CTS_DIR("khronos_mustpass/master/gles3-khr-master.txt"),
-    OPENGL_CTS_DIR("khronos_mustpass/master/gles31-khr-master.txt"),
-    OPENGL_CTS_DIR("khronos_mustpass/master/gles32-khr-master.txt"),
-    OPENGL_CTS_DIR("aosp_mustpass/master/gles3-rotate-landscape.txt"),
-    OPENGL_CTS_DIR("aosp_mustpass/master/gles3-rotate-reverse-portrait.txt"),
-    OPENGL_CTS_DIR("aosp_mustpass/master/gles3-rotate-reverse-landscape.txt"),
-    OPENGL_CTS_DIR("aosp_mustpass/master/gles31-rotate-landscape.txt"),
-    OPENGL_CTS_DIR("aosp_mustpass/master/gles31-rotate-reverse-portrait.txt"),
-    OPENGL_CTS_DIR("aosp_mustpass/master/gles31-rotate-reverse-landscape.txt"),
+    OPENGL_CTS_DIR("aosp_mustpass/main/gles2-master.txt"),
+    OPENGL_CTS_DIR("aosp_mustpass/main/gles3-master.txt"),
+    OPENGL_CTS_DIR("aosp_mustpass/main/gles31-master.txt"),
+    "/android/cts/main/egl-master.txt",
+    OPENGL_CTS_DIR("khronos_mustpass/main/gles2-khr-master.txt"),
+    OPENGL_CTS_DIR("khronos_mustpass/main/gles3-khr-master.txt"),
+    OPENGL_CTS_DIR("khronos_mustpass/main/gles31-khr-master.txt"),
+    OPENGL_CTS_DIR("khronos_mustpass/main/gles32-khr-master.txt"),
+    OPENGL_CTS_DIR("aosp_mustpass/main/gles3-rotate-landscape.txt"),
+    OPENGL_CTS_DIR("aosp_mustpass/main/gles3-rotate-reverse-portrait.txt"),
+    OPENGL_CTS_DIR("aosp_mustpass/main/gles3-rotate-reverse-landscape.txt"),
+    OPENGL_CTS_DIR("aosp_mustpass/main/gles31-rotate-landscape.txt"),
+    OPENGL_CTS_DIR("aosp_mustpass/main/gles31-rotate-reverse-portrait.txt"),
+    OPENGL_CTS_DIR("aosp_mustpass/main/gles31-rotate-reverse-landscape.txt"),
 };
 
 #undef OPENGL_CTS_DIR
@@ -120,8 +111,9 @@ constexpr char kVerboseString[]     = "--verbose";
 constexpr char kRenderDocString[]   = "--renderdoc";
 constexpr char kNoRenderDocString[] = "--no-renderdoc";
 constexpr char kdEQPFlagsPrefix[]   = "--deqp-";
+constexpr char kGTestFilter[]       = "--gtest_filter=";
 
-std::array<char, 500> gCaseStringBuffer;
+angle::base::NoDestructor<std::vector<char>> gFilterStringBuffer;
 
 // For angle_deqp_gles3*_rotateN_tests, default gOptions.preRotation to N.
 #if defined(ANGLE_DEQP_GLES3_ROTATE90_TESTS) || defined(ANGLE_DEQP_GLES31_ROTATE90_TESTS)
@@ -142,8 +134,8 @@ constexpr bool kEnableRenderDocCapture = false;
 
 const APIInfo *gInitAPI = nullptr;
 dEQPOptions gOptions    = {
-    kDefaultPreRotation,      // preRotation
-    kEnableRenderDocCapture,  // enableRenderDocCapture
+       kDefaultPreRotation,      // preRotation
+       kEnableRenderDocCapture,  // enableRenderDocCapture
 };
 
 constexpr const char gdEQPEGLConfigNameString[] = "--deqp-gl-config-name=";
@@ -231,13 +223,12 @@ class dEQPCaseList
 
     struct CaseInfo
     {
-        CaseInfo(const std::string &dEQPName, const std::string &gTestName, int expectation)
-            : mDEQPName(dEQPName), mGTestName(gTestName), mExpectation(expectation)
+        CaseInfo(const std::string &testNameIn, int expectationIn)
+            : testName(testNameIn), expectation(expectationIn)
         {}
 
-        std::string mDEQPName;
-        std::string mGTestName;
-        int mExpectation;
+        std::string testName;
+        int expectation;
     };
 
     void initialize();
@@ -328,15 +319,11 @@ void dEQPCaseList::initialize()
         std::string inString;
         std::getline(caseListStream, inString);
 
-        std::string dEQPName = TrimString(inString, kWhitespaceASCII);
-        if (dEQPName.empty())
+        std::string testName = TrimString(inString, kWhitespaceASCII);
+        if (testName.empty())
             continue;
-        std::string gTestName = DrawElementsToGoogleTestName(dEQPName);
-        if (gTestName.empty())
-            continue;
-
-        int expectation = testSuite->getTestExpectation(dEQPName);
-        mCaseInfoList.push_back(CaseInfo(dEQPName, gTestName, expectation));
+        int expectation = testSuite->getTestExpectation(testName);
+        mCaseInfoList.push_back(CaseInfo(testName, expectation));
     }
 
     if (testSuite->logAnyUnusedTestExpectations())
@@ -365,24 +352,23 @@ bool IsPassingResult(dEQPTestResult result)
     }
 }
 
-template <size_t TestModuleIndex>
-class dEQPTest : public testing::TestWithParam<size_t>
+class dEQP : public testing::Test
 {
   public:
-    static testing::internal::ParamGenerator<size_t> GetTestingRange()
+    static testing::internal::ParamGenerator<size_t> GetTestingRange(size_t testModuleIndex)
     {
-        return testing::Range<size_t>(0, GetCaseList().numCases());
+        return testing::Range<size_t>(0, GetCaseList(testModuleIndex).numCases());
     }
 
-    static std::string GetCaseGTestName(size_t caseIndex)
+    static std::string GetTestCaseName(size_t testModuleIndex, size_t caseIndex)
     {
-        const auto &caseInfo = GetCaseList().getCaseInfo(caseIndex);
-        return caseInfo.mGTestName;
+        const auto &caseInfo = GetCaseList(testModuleIndex).getCaseInfo(caseIndex);
+        return caseInfo.testName;
     }
 
-    static const dEQPCaseList &GetCaseList()
+    static const dEQPCaseList &GetCaseList(size_t testModuleIndex)
     {
-        static dEQPCaseList sCaseList(TestModuleIndex);
+        static dEQPCaseList sCaseList(testModuleIndex);
         sCaseList.initialize();
         return sCaseList;
     }
@@ -390,8 +376,12 @@ class dEQPTest : public testing::TestWithParam<size_t>
     static void SetUpTestCase();
     static void TearDownTestCase();
 
+    dEQP(size_t testModuleIndex, size_t caseIndex)
+        : mTestModuleIndex(testModuleIndex), mTestCaseIndex(caseIndex)
+    {}
+
   protected:
-    void runTest() const
+    void TestBody() override
     {
         if (sTestExceptionCount > 1)
         {
@@ -399,14 +389,13 @@ class dEQPTest : public testing::TestWithParam<size_t>
             return;
         }
 
-        const auto &caseInfo = GetCaseList().getCaseInfo(GetParam());
-        std::cout << caseInfo.mDEQPName << std::endl;
+        const auto &caseInfo = GetCaseList(mTestModuleIndex).getCaseInfo(mTestCaseIndex);
 
         // Tests that crash exit the harness before collecting the result. To tally the number of
         // crashed tests we track how many tests we "tried" to run.
         sTestCount++;
 
-        if (caseInfo.mExpectation == GPUTestExpectationsParser::kGpuTestSkip)
+        if (caseInfo.expectation == GPUTestExpectationsParser::kGpuTestSkip)
         {
             sSkippedTestCount++;
             std::cout << "Test skipped.\n";
@@ -414,35 +403,35 @@ class dEQPTest : public testing::TestWithParam<size_t>
         }
 
         TestSuite *testSuite = TestSuite::GetInstance();
-        testSuite->maybeUpdateTestTimeout(caseInfo.mExpectation);
+        testSuite->maybeUpdateTestTimeout(caseInfo.expectation);
 
-        gExpectError          = (caseInfo.mExpectation != GPUTestExpectationsParser::kGpuTestPass);
-        dEQPTestResult result = deqp_libtester_run(caseInfo.mDEQPName.c_str());
+        gExpectError          = (caseInfo.expectation != GPUTestExpectationsParser::kGpuTestPass);
+        dEQPTestResult result = deqp_libtester_run(caseInfo.testName.c_str());
 
         bool testSucceeded = IsPassingResult(result);
 
-        if (!testSucceeded && caseInfo.mExpectation == GPUTestExpectationsParser::kGpuTestFlaky)
+        if (!testSucceeded && caseInfo.expectation == GPUTestExpectationsParser::kGpuTestFlaky)
         {
-            result        = deqp_libtester_run(caseInfo.mDEQPName.c_str());
+            result        = deqp_libtester_run(caseInfo.testName.c_str());
             testSucceeded = IsPassingResult(result);
         }
 
         countTestResult(result);
 
-        if (caseInfo.mExpectation == GPUTestExpectationsParser::kGpuTestPass ||
-            caseInfo.mExpectation == GPUTestExpectationsParser::kGpuTestFlaky)
+        if (caseInfo.expectation == GPUTestExpectationsParser::kGpuTestPass ||
+            caseInfo.expectation == GPUTestExpectationsParser::kGpuTestFlaky)
         {
             EXPECT_TRUE(testSucceeded);
 
             if (!testSucceeded)
             {
-                sUnexpectedFailed.push_back(caseInfo.mDEQPName);
+                sUnexpectedFailed.push_back(caseInfo.testName);
             }
         }
         else if (testSucceeded)
         {
             std::cout << "Test expected to fail but passed!" << std::endl;
-            sUnexpectedPasses.push_back(caseInfo.mDEQPName);
+            sUnexpectedPasses.push_back(caseInfo.testName);
         }
     }
 
@@ -512,28 +501,22 @@ class dEQPTest : public testing::TestWithParam<size_t>
 
     static std::vector<std::string> sUnexpectedFailed;
     static std::vector<std::string> sUnexpectedPasses;
+
+    size_t mTestModuleIndex = 0;
+    size_t mTestCaseIndex   = 0;
 };
 
-template <size_t TestModuleIndex>
-uint32_t dEQPTest<TestModuleIndex>::sTestCount = 0;
-template <size_t TestModuleIndex>
-uint32_t dEQPTest<TestModuleIndex>::sPassedTestCount = 0;
-template <size_t TestModuleIndex>
-uint32_t dEQPTest<TestModuleIndex>::sFailedTestCount = 0;
-template <size_t TestModuleIndex>
-uint32_t dEQPTest<TestModuleIndex>::sTestExceptionCount = 0;
-template <size_t TestModuleIndex>
-uint32_t dEQPTest<TestModuleIndex>::sNotSupportedTestCount = 0;
-template <size_t TestModuleIndex>
-uint32_t dEQPTest<TestModuleIndex>::sSkippedTestCount = 0;
-template <size_t TestModuleIndex>
-std::vector<std::string> dEQPTest<TestModuleIndex>::sUnexpectedFailed;
-template <size_t TestModuleIndex>
-std::vector<std::string> dEQPTest<TestModuleIndex>::sUnexpectedPasses;
+uint32_t dEQP::sTestCount             = 0;
+uint32_t dEQP::sPassedTestCount       = 0;
+uint32_t dEQP::sFailedTestCount       = 0;
+uint32_t dEQP::sTestExceptionCount    = 0;
+uint32_t dEQP::sNotSupportedTestCount = 0;
+uint32_t dEQP::sSkippedTestCount      = 0;
+std::vector<std::string> dEQP::sUnexpectedFailed;
+std::vector<std::string> dEQP::sUnexpectedPasses;
 
 // static
-template <size_t TestModuleIndex>
-void dEQPTest<TestModuleIndex>::SetUpTestCase()
+void dEQP::SetUpTestCase()
 {
     sPassedTestCount       = 0;
     sFailedTestCount       = 0;
@@ -606,79 +589,12 @@ void dEQPTest<TestModuleIndex>::SetUpTestCase()
 }
 
 // static
-template <size_t TestModuleIndex>
-void dEQPTest<TestModuleIndex>::TearDownTestCase()
+void dEQP::TearDownTestCase()
 {
     PrintTestStats();
     deqp_libtester_shutdown_platform();
 }
 
-#define ANGLE_INSTANTIATE_DEQP_TEST_CASE(API, N)                              \
-    class dEQP : public dEQPTest<N>                                           \
-    {};                                                                       \
-    TEST_P(dEQP, API) { runTest(); }                                          \
-                                                                              \
-    INSTANTIATE_TEST_SUITE_P(, dEQP, dEQP::GetTestingRange(),                 \
-                             [](const testing::TestParamInfo<size_t> &info) { \
-                                 return dEQP::GetCaseGTestName(info.param);   \
-                             })
-
-#ifdef ANGLE_DEQP_GLES2_TESTS
-ANGLE_INSTANTIATE_DEQP_TEST_CASE(GLES2, 0);
-#endif
-
-#ifdef ANGLE_DEQP_GLES3_TESTS
-ANGLE_INSTANTIATE_DEQP_TEST_CASE(GLES3, 1);
-#endif
-
-#ifdef ANGLE_DEQP_GLES31_TESTS
-ANGLE_INSTANTIATE_DEQP_TEST_CASE(GLES31, 2);
-#endif
-
-#ifdef ANGLE_DEQP_EGL_TESTS
-ANGLE_INSTANTIATE_DEQP_TEST_CASE(EGL, 3);
-#endif
-
-#ifdef ANGLE_DEQP_KHR_GLES2_TESTS
-ANGLE_INSTANTIATE_DEQP_TEST_CASE(KHR_GLES2, 4);
-#endif
-
-#ifdef ANGLE_DEQP_KHR_GLES3_TESTS
-ANGLE_INSTANTIATE_DEQP_TEST_CASE(KHR_GLES3, 5);
-#endif
-
-#ifdef ANGLE_DEQP_KHR_GLES31_TESTS
-ANGLE_INSTANTIATE_DEQP_TEST_CASE(KHR_GLES31, 6);
-#endif
-
-#ifdef ANGLE_DEQP_KHR_GLES32_TESTS
-ANGLE_INSTANTIATE_DEQP_TEST_CASE(KHR_GLES32, 7);
-#endif
-
-#ifdef ANGLE_DEQP_GLES3_ROTATE90_TESTS
-ANGLE_INSTANTIATE_DEQP_TEST_CASE(GLES3_ROTATE90, 8);
-#endif
-
-#ifdef ANGLE_DEQP_GLES3_ROTATE180_TESTS
-ANGLE_INSTANTIATE_DEQP_TEST_CASE(GLES3_ROTATE180, 9);
-#endif
-
-#ifdef ANGLE_DEQP_GLES3_ROTATE270_TESTS
-ANGLE_INSTANTIATE_DEQP_TEST_CASE(GLES3_ROTATE270, 10);
-#endif
-
-#ifdef ANGLE_DEQP_GLES31_ROTATE90_TESTS
-ANGLE_INSTANTIATE_DEQP_TEST_CASE(GLES31_ROTATE90, 11);
-#endif
-
-#ifdef ANGLE_DEQP_GLES31_ROTATE180_TESTS
-ANGLE_INSTANTIATE_DEQP_TEST_CASE(GLES31_ROTATE180, 12);
-#endif
-
-#ifdef ANGLE_DEQP_GLES31_ROTATE270_TESTS
-ANGLE_INSTANTIATE_DEQP_TEST_CASE(GLES31_ROTATE270, 13);
-#endif
-
 void HandleDisplayType(const char *displayTypeString)
 {
     std::stringstream argStream;
@@ -732,19 +648,22 @@ void HandleEGLConfigName(const char *configNameString)
 // The --deqp-case flag takes a case expression that is parsed into a --gtest_filter. It converts
 // the "dEQP" style names (functional.thing.*) into "GoogleTest" style names (functional_thing_*).
 // Currently it does not handle multiple tests and multiple filters in different arguments.
-void HandleCaseName(const char *caseString, int *argc, int argIndex, char **argv)
+void HandleFilterArg(const char *filterString, int *argc, int argIndex, char **argv)
 {
-    std::string googleTestName = DrawElementsToGoogleTestName(caseString);
-    gCaseStringBuffer.fill(0);
-    int bytesWritten = snprintf(gCaseStringBuffer.data(), gCaseStringBuffer.size() - 1,
-                                "--gtest_filter=*%s", googleTestName.c_str());
-    if (bytesWritten <= 0 || static_cast<size_t>(bytesWritten) >= gCaseStringBuffer.size() - 1)
+    std::string googleTestFilter = ReplaceDashesWithQuestionMark(filterString);
+
+    gFilterStringBuffer->resize(googleTestFilter.size() + 3 + strlen(kGTestFilter), 0);
+    std::fill(gFilterStringBuffer->begin(), gFilterStringBuffer->end(), 0);
+
+    int bytesWritten = snprintf(gFilterStringBuffer->data(), gFilterStringBuffer->size() - 1,
+                                "%s*%s", kGTestFilter, googleTestFilter.c_str());
+    if (bytesWritten <= 0 || static_cast<size_t>(bytesWritten) >= gFilterStringBuffer->size() - 1)
     {
-        std::cout << "Error parsing test case string: " << caseString;
+        std::cout << "Error parsing filter string: " << filterString;
         exit(1);
     }
 
-    argv[argIndex] = gCaseStringBuffer.data();
+    argv[argIndex] = gFilterStringBuffer->data();
 }
 
 void HandleLogImages(const char *logImagesString)
@@ -763,10 +682,91 @@ void HandleLogImages(const char *logImagesString)
         exit(1);
     }
 }
+
+size_t GetTestModuleIndex()
+{
+#ifdef ANGLE_DEQP_GLES2_TESTS
+    return 0;
+#endif
+
+#ifdef ANGLE_DEQP_GLES3_TESTS
+    return 1;
+#endif
+
+#ifdef ANGLE_DEQP_GLES31_TESTS
+    return 2;
+#endif
+
+#ifdef ANGLE_DEQP_EGL_TESTS
+    return 3;
+#endif
+
+#ifdef ANGLE_DEQP_KHR_GLES2_TESTS
+    return 4;
+#endif
+
+#ifdef ANGLE_DEQP_KHR_GLES3_TESTS
+    return 5;
+#endif
+
+#ifdef ANGLE_DEQP_KHR_GLES31_TESTS
+    return 6;
+#endif
+
+#ifdef ANGLE_DEQP_KHR_GLES32_TESTS
+    return 7;
+#endif
+
+#ifdef ANGLE_DEQP_GLES3_ROTATE90_TESTS
+    return 8;
+#endif
+
+#ifdef ANGLE_DEQP_GLES3_ROTATE180_TESTS
+    return 9;
+#endif
+
+#ifdef ANGLE_DEQP_GLES3_ROTATE270_TESTS
+    return 10;
+#endif
+
+#ifdef ANGLE_DEQP_GLES31_ROTATE90_TESTS
+    return 11;
+#endif
+
+#ifdef ANGLE_DEQP_GLES31_ROTATE180_TESTS
+    return 12;
+#endif
+
+#ifdef ANGLE_DEQP_GLES31_ROTATE270_TESTS
+    return 13;
+#endif
+}
+
+void RegisterGLCTSTests()
+{
+    size_t testModuleIndex = GetTestModuleIndex();
+
+    const dEQPCaseList &caseList = dEQP::GetCaseList(testModuleIndex);
+
+    for (size_t caseIndex = 0; caseIndex < caseList.numCases(); ++caseIndex)
+    {
+        auto factory = [testModuleIndex, caseIndex]() {
+            return new dEQP(testModuleIndex, caseIndex);
+        };
+
+        std::string testCaseName = dEQP::GetTestCaseName(testModuleIndex, caseIndex);
+        size_t pos               = testCaseName.find('.');
+        ASSERT(pos != std::string::npos);
+        std::string moduleName = testCaseName.substr(0, pos);
+        std::string testName   = testCaseName.substr(pos + 1);
+        testing::RegisterTest(moduleName.c_str(), testName.c_str(), nullptr, nullptr, __FILE__,
+                              __LINE__, factory);
+    }
+}
 }  // anonymous namespace
 
 // Called from main() to process command-line arguments.
-void InitTestHarness(int *argc, char **argv)
+int RunGLCTSTests(int *argc, char **argv)
 {
     int argIndex = 0;
     while (argIndex < *argc)
@@ -790,7 +790,11 @@ void InitTestHarness(int *argc, char **argv)
         }
         else if (strncmp(argv[argIndex], kdEQPCaseString, strlen(kdEQPCaseString)) == 0)
         {
-            HandleCaseName(argv[argIndex] + strlen(kdEQPCaseString), argc, argIndex, argv);
+            HandleFilterArg(argv[argIndex] + strlen(kdEQPCaseString), argc, argIndex, argv);
+        }
+        else if (strncmp(argv[argIndex], kGTestFilter, strlen(kGTestFilter)) == 0)
+        {
+            HandleFilterArg(argv[argIndex] + strlen(kGTestFilter), argc, argIndex, argv);
         }
         else if (strncmp(argv[argIndex], kVerboseString, strlen(kVerboseString)) == 0 ||
                  strcmp(argv[argIndex], "-v") == 0)
@@ -827,5 +831,8 @@ void InitTestHarness(int *argc, char **argv)
         std::cout << "PreRotation is only supported on Vulkan" << std::endl;
         exit(1);
     }
+
+    angle::TestSuite testSuite(argc, argv, RegisterGLCTSTests);
+    return testSuite.run();
 }
 }  // namespace angle
