@@ -29,7 +29,7 @@ void AddIntegerValue(CFMutableDictionaryRef dictionary, const CFStringRef key, i
     CFRelease(number);
 }
 
-class ScopedIOSurfaceRef : angle::NonCopyable
+class [[nodiscard]] ScopedIOSurfaceRef : angle::NonCopyable
 {
   public:
     explicit ScopedIOSurfaceRef(IOSurfaceRef surface) : mSurface(surface) {}
@@ -71,17 +71,47 @@ class ScopedIOSurfaceRef : angle::NonCopyable
     IOSurfaceRef mSurface = nullptr;
 };
 
-ScopedIOSurfaceRef CreateSinglePlaneIOSurface(int width,
-                                              int height,
-                                              int32_t format,
-                                              int bytesPerElement)
+struct IOSurfacePlaneInfo
 {
+    int width;
+    int height;
+    int bytesPerElement;
+};
+
+ScopedIOSurfaceRef CreateIOSurface(int32_t format, const std::vector<IOSurfacePlaneInfo> &planes)
+{
+    EXPECT_GT(planes.size(), 0u);
+
     CFMutableDictionaryRef dict = CFDictionaryCreateMutable(
         kCFAllocatorDefault, 0, &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
-    AddIntegerValue(dict, kIOSurfaceWidth, width);
-    AddIntegerValue(dict, kIOSurfaceHeight, height);
+    AddIntegerValue(dict, kIOSurfaceWidth, planes[0].width);
+    AddIntegerValue(dict, kIOSurfaceHeight, planes[0].height);
     AddIntegerValue(dict, kIOSurfacePixelFormat, format);
-    AddIntegerValue(dict, kIOSurfaceBytesPerElement, bytesPerElement);
+
+    if (planes.size() > 1)
+    {
+        CFMutableArrayRef planesInfo =
+            CFArrayCreateMutable(kCFAllocatorDefault, planes.size(), &kCFTypeArrayCallBacks);
+        for (const IOSurfacePlaneInfo &plane : planes)
+        {
+            CFMutableDictionaryRef planeInfo =
+                CFDictionaryCreateMutable(kCFAllocatorDefault, 0, &kCFTypeDictionaryKeyCallBacks,
+                                          &kCFTypeDictionaryValueCallBacks);
+            AddIntegerValue(planeInfo, kIOSurfacePlaneWidth, plane.width);
+            AddIntegerValue(planeInfo, kIOSurfacePlaneHeight, plane.height);
+            AddIntegerValue(planeInfo, kIOSurfacePlaneBytesPerElement, plane.bytesPerElement);
+
+            CFArrayAppendValue(planesInfo, planeInfo);
+            CFRelease(planeInfo);
+        }
+
+        CFDictionaryAddValue(dict, kIOSurfacePlaneInfo, planesInfo);
+        CFRelease(planesInfo);
+    }
+    else
+    {
+        AddIntegerValue(dict, kIOSurfaceBytesPerElement, planes[0].bytesPerElement);
+    }
 
     IOSurfaceRef ioSurface = IOSurfaceCreate(dict);
     EXPECT_NE(nullptr, ioSurface);
@@ -90,9 +120,18 @@ ScopedIOSurfaceRef CreateSinglePlaneIOSurface(int width,
     return ScopedIOSurfaceRef(ioSurface);
 }
 
+ScopedIOSurfaceRef CreateSinglePlaneIOSurface(int width,
+                                              int height,
+                                              int32_t format,
+                                              int bytesPerElement)
+{
+    std::vector<IOSurfacePlaneInfo> planes{{width, height, bytesPerElement}};
+    return CreateIOSurface(format, planes);
+}
+
 }  // anonymous namespace
 
-class IOSurfaceClientBufferTest : public ANGLETest
+class IOSurfaceClientBufferTest : public ANGLETest<>
 {
   protected:
     EGLint getTextureTarget() const
@@ -142,9 +181,10 @@ class IOSurfaceClientBufferTest : public ANGLETest
             EGL_HEIGHT,                        height,
             EGL_IOSURFACE_PLANE_ANGLE,         plane,
             EGL_TEXTURE_TARGET,                getTextureTarget(),
-            EGL_TEXTURE_INTERNAL_FORMAT_ANGLE, internalFormat,
+            EGL_TEXTURE_INTERNAL_FORMAT_ANGLE,
+                static_cast<EGLint>(internalFormat),
             EGL_TEXTURE_FORMAT,                EGL_TEXTURE_RGBA,
-            EGL_TEXTURE_TYPE_ANGLE,            type,
+            EGL_TEXTURE_TYPE_ANGLE,            static_cast<EGLint>(type),
             EGL_NONE,                          EGL_NONE,
         };
         // clang-format on
@@ -173,16 +213,22 @@ class IOSurfaceClientBufferTest : public ANGLETest
     }
 
     void doClearTest(const ScopedIOSurfaceRef &ioSurface,
+                     EGLint width,
+                     EGLint height,
+                     EGLint plane,
                      GLenum internalFormat,
                      GLenum type,
                      const GLColor &data)
     {
         std::array<uint8_t, 4> dataArray{data.R, data.G, data.B, data.A};
-        doClearTest(ioSurface, internalFormat, type, dataArray);
+        doClearTest(ioSurface, width, height, plane, internalFormat, type, dataArray);
     }
 
     template <typename T, size_t dataSize>
     void doClearTest(const ScopedIOSurfaceRef &ioSurface,
+                     EGLint width,
+                     EGLint height,
+                     EGLint plane,
                      GLenum internalFormat,
                      GLenum type,
                      const std::array<T, dataSize> &data)
@@ -190,7 +236,8 @@ class IOSurfaceClientBufferTest : public ANGLETest
         // Bind the IOSurface to a texture and clear it.
         EGLSurface pbuffer;
         GLTexture texture;
-        bindIOSurfaceToTexture(ioSurface, 1, 1, 0, internalFormat, type, &pbuffer, &texture);
+        bindIOSurfaceToTexture(ioSurface, width, height, plane, internalFormat, type, &pbuffer,
+                               &texture);
 
         GLFramebuffer fbo;
         glBindFramebuffer(GL_FRAMEBUFFER, fbo);
@@ -198,7 +245,7 @@ class IOSurfaceClientBufferTest : public ANGLETest
         glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, getGLTextureTarget(), texture,
                                0);
         EXPECT_GL_NO_ERROR();
-        EXPECT_GLENUM_EQ(glCheckFramebufferStatus(GL_FRAMEBUFFER), GL_FRAMEBUFFER_COMPLETE);
+        ASSERT_GL_FRAMEBUFFER_COMPLETE(GL_FRAMEBUFFER);
         EXPECT_GL_NO_ERROR();
 
         glClearColor(1.0f / 255.0f, 2.0f / 255.0f, 3.0f / 255.0f, 4.0f / 255.0f);
@@ -211,13 +258,28 @@ class IOSurfaceClientBufferTest : public ANGLETest
         EXPECT_EGL_TRUE(result);
         EXPECT_EGL_SUCCESS();
 
+        // IOSurface client buffer's rendering doesn't automatically finish after
+        // eglReleaseTexImage(). Need to explicitly call glFinish().
+        glFinish();
+
         IOSurfaceLock(ioSurface.get(), kIOSurfaceLockReadOnly, nullptr);
         std::array<T, dataSize> iosurfaceData;
-        memcpy(iosurfaceData.data(), IOSurfaceGetBaseAddress(ioSurface.get()),
+        memcpy(iosurfaceData.data(), IOSurfaceGetBaseAddressOfPlane(ioSurface.get(), plane),
                sizeof(T) * data.size());
         IOSurfaceUnlock(ioSurface.get(), kIOSurfaceLockReadOnly, nullptr);
 
-        ASSERT_EQ(data, iosurfaceData);
+        if (internalFormat == GL_RGB && IsOSX() && IsOpenGL())
+        {
+            // Ignore alpha component for BGRX, the alpha value is undefined
+            for (int i = 0; i < 3; i++)
+            {
+                ASSERT_EQ(data[i], iosurfaceData[i]);
+            }
+        }
+        else
+        {
+            ASSERT_EQ(data, iosurfaceData);
+        }
 
         result = eglDestroySurface(mDisplay, pbuffer);
         EXPECT_EGL_TRUE(result);
@@ -231,16 +293,20 @@ class IOSurfaceClientBufferTest : public ANGLETest
         B = 4,
         A = 8,
     };
-    void doSampleTest(const ScopedIOSurfaceRef &ioSurface,
-                      GLenum internalFormat,
-                      GLenum type,
-                      void *data,
-                      size_t dataSize,
-                      int mask)
+    void doSampleTestWithExtraSteps(const ScopedIOSurfaceRef &ioSurface,
+                                    EGLint width,
+                                    EGLint height,
+                                    EGLint plane,
+                                    GLenum internalFormat,
+                                    GLenum type,
+                                    void *data,
+                                    size_t dataSize,
+                                    int mask,
+                                    const std::function<void()> &extraStepsBeforeSample)
     {
         // Write the data to the IOSurface
         IOSurfaceLock(ioSurface.get(), 0, nullptr);
-        memcpy(IOSurfaceGetBaseAddress(ioSurface.get()), data, dataSize);
+        memcpy(IOSurfaceGetBaseAddressOfPlane(ioSurface.get(), plane), data, dataSize);
         IOSurfaceUnlock(ioSurface.get(), 0, nullptr);
 
         GLTexture texture;
@@ -250,8 +316,37 @@ class IOSurfaceClientBufferTest : public ANGLETest
 
         // Bind the IOSurface to a texture and clear it.
         EGLSurface pbuffer;
-        bindIOSurfaceToTexture(ioSurface, 1, 1, 0, internalFormat, type, &pbuffer, &texture);
+        bindIOSurfaceToTexture(ioSurface, width, height, plane, internalFormat, type, &pbuffer,
+                               &texture);
 
+        if (extraStepsBeforeSample)
+        {
+            extraStepsBeforeSample();
+        }
+
+        doSampleTestWithTexture(texture, mask);
+
+        EGLBoolean result = eglDestroySurface(mDisplay, pbuffer);
+        EXPECT_EGL_TRUE(result);
+        EXPECT_EGL_SUCCESS();
+    }
+
+    void doSampleTest(const ScopedIOSurfaceRef &ioSurface,
+                      EGLint width,
+                      EGLint height,
+                      EGLint plane,
+                      GLenum internalFormat,
+                      GLenum type,
+                      void *data,
+                      size_t dataSize,
+                      int mask)
+    {
+        doSampleTestWithExtraSteps(ioSurface, width, height, plane, internalFormat, type, data,
+                                   dataSize, mask, nullptr);
+    }
+
+    void doSampleTestWithTexture(const GLTexture &texture, int mask)
+    {
         constexpr char kVS[] =
             "attribute vec4 position;\n"
             "void main()\n"
@@ -294,6 +389,11 @@ class IOSurfaceClientBufferTest : public ANGLETest
 
     void doBlitTest(bool ioSurfaceIsSource, int width, int height)
     {
+        if (!hasBlitExt())
+        {
+            return;
+        }
+
         // Create IOSurface and bind it to a texture.
         ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(width, height, 'BGRA', 4);
         EGLSurface pbuffer;
@@ -307,7 +407,7 @@ class IOSurfaceClientBufferTest : public ANGLETest
         glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, getGLTextureTarget(), texture,
                                0);
         EXPECT_GL_NO_ERROR();
-        EXPECT_GLENUM_EQ(glCheckFramebufferStatus(GL_FRAMEBUFFER), GL_FRAMEBUFFER_COMPLETE);
+        ASSERT_GL_FRAMEBUFFER_COMPLETE(GL_FRAMEBUFFER);
         EXPECT_GL_NO_ERROR();
 
         // Create another framebuffer with a regular renderbuffer.
@@ -321,7 +421,7 @@ class IOSurfaceClientBufferTest : public ANGLETest
         EXPECT_GL_NO_ERROR();
         glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_RENDERBUFFER, rbo);
         EXPECT_GL_NO_ERROR();
-        EXPECT_GLENUM_EQ(glCheckFramebufferStatus(GL_FRAMEBUFFER), GL_FRAMEBUFFER_COMPLETE);
+        ASSERT_GL_FRAMEBUFFER_COMPLETE(GL_FRAMEBUFFER);
         EXPECT_GL_NO_ERROR();
 
         glBindRenderbuffer(GL_RENDERBUFFER, 0);
@@ -361,6 +461,10 @@ class IOSurfaceClientBufferTest : public ANGLETest
     }
 
     bool hasIOSurfaceExt() const { return IsEGLDisplayExtensionEnabled(mDisplay, kIOSurfaceExt); }
+    bool hasBlitExt() const
+    {
+        return IsEGLDisplayExtensionEnabled(mDisplay, "ANGLE_framebuffer_blit");
+    }
 
     EGLConfig mConfig;
     EGLDisplay mDisplay;
@@ -374,7 +478,7 @@ TEST_P(IOSurfaceClientBufferTest, RenderToBGRA8888IOSurface)
     ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, 'BGRA', 4);
 
     GLColor color(3, 2, 1, 4);
-    doClearTest(ioSurface, GL_BGRA_EXT, GL_UNSIGNED_BYTE, color);
+    doClearTest(ioSurface, 1, 1, 0, GL_BGRA_EXT, GL_UNSIGNED_BYTE, color);
 }
 
 // Test reading from BGRA8888 IOSurfaces
@@ -382,13 +486,11 @@ TEST_P(IOSurfaceClientBufferTest, ReadFromBGRA8888IOSurface)
 {
     ANGLE_SKIP_TEST_IF(!hasIOSurfaceExt());
 
-    // TODO(http://anglebug.com/4369)
-    ANGLE_SKIP_TEST_IF(isSwiftshader());
-
     ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, 'BGRA', 4);
 
     GLColor color(3, 2, 1, 4);
-    doSampleTest(ioSurface, GL_BGRA_EXT, GL_UNSIGNED_BYTE, &color, sizeof(color), R | G | B | A);
+    doSampleTest(ioSurface, 1, 1, 0, GL_BGRA_EXT, GL_UNSIGNED_BYTE, &color, sizeof(color),
+                 R | G | B | A);
 }
 
 // Test using BGRX8888 IOSurfaces for rendering
@@ -396,13 +498,10 @@ TEST_P(IOSurfaceClientBufferTest, RenderToBGRX8888IOSurface)
 {
     ANGLE_SKIP_TEST_IF(!hasIOSurfaceExt());
 
-    // TODO(http://anglebug.com/4369)
-    ANGLE_SKIP_TEST_IF(isSwiftshader());
-
     ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, 'BGRA', 4);
 
     GLColor color(3, 2, 1, 255);
-    doClearTest(ioSurface, GL_RGB, GL_UNSIGNED_BYTE, color);
+    doClearTest(ioSurface, 1, 1, 0, GL_RGB, GL_UNSIGNED_BYTE, color);
 }
 
 // Test reading from BGRX8888 IOSurfaces
@@ -413,7 +512,7 @@ TEST_P(IOSurfaceClientBufferTest, ReadFromBGRX8888IOSurface)
     ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, 'BGRA', 4);
 
     GLColor color(3, 2, 1, 4);
-    doSampleTest(ioSurface, GL_RGB, GL_UNSIGNED_BYTE, &color, sizeof(color), R | G | B);
+    doSampleTest(ioSurface, 1, 1, 0, GL_RGB, GL_UNSIGNED_BYTE, &color, sizeof(color), R | G | B);
 }
 
 // Test using RG88 IOSurfaces for rendering
@@ -424,7 +523,7 @@ TEST_P(IOSurfaceClientBufferTest, RenderToRG88IOSurface)
     ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, '2C08', 2);
 
     std::array<uint8_t, 2> color{1, 2};
-    doClearTest(ioSurface, GL_RG, GL_UNSIGNED_BYTE, color);
+    doClearTest(ioSurface, 1, 1, 0, GL_RG, GL_UNSIGNED_BYTE, color);
 }
 
 // Test reading from RG88 IOSurfaces
@@ -435,7 +534,7 @@ TEST_P(IOSurfaceClientBufferTest, ReadFromRG88IOSurface)
     ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, '2C08', 2);
 
     uint8_t color[2] = {1, 2};
-    doSampleTest(ioSurface, GL_RG, GL_UNSIGNED_BYTE, &color, sizeof(color), R | G);
+    doSampleTest(ioSurface, 1, 1, 0, GL_RG, GL_UNSIGNED_BYTE, &color, sizeof(color), R | G);
 }
 
 // Test using R8 IOSurfaces for rendering
@@ -446,7 +545,7 @@ TEST_P(IOSurfaceClientBufferTest, RenderToR8IOSurface)
     ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, 'L008', 1);
 
     std::array<uint8_t, 1> color{1};
-    doClearTest(ioSurface, GL_RED, GL_UNSIGNED_BYTE, color);
+    doClearTest(ioSurface, 1, 1, 0, GL_RED, GL_UNSIGNED_BYTE, color);
 }
 
 // Test reading from R8 IOSurfaces
@@ -457,7 +556,7 @@ TEST_P(IOSurfaceClientBufferTest, ReadFromR8IOSurface)
     ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, 'L008', 1);
 
     uint8_t color = 1;
-    doSampleTest(ioSurface, GL_RED, GL_UNSIGNED_BYTE, &color, sizeof(color), R);
+    doSampleTest(ioSurface, 1, 1, 0, GL_RED, GL_UNSIGNED_BYTE, &color, sizeof(color), R);
 }
 
 // Test using R16 IOSurfaces for rendering
@@ -465,15 +564,18 @@ TEST_P(IOSurfaceClientBufferTest, RenderToR16IOSurface)
 {
     ANGLE_SKIP_TEST_IF(!hasIOSurfaceExt());
 
-    // TODO(http://anglebug.com/4369)
-    ANGLE_SKIP_TEST_IF(isSwiftshader());
+    // This test only works on ES3 since it requires an integer texture.
+    ANGLE_SKIP_TEST_IF(getClientMajorVersion() < 3);
+
+    // TODO(http://anglebug.com/7445): Fails with Metal backend.
+    ANGLE_SKIP_TEST_IF(IsMetal());
 
     // HACK(cwallez@chromium.org) 'L016' doesn't seem to be an official pixel format but it works
     // sooooooo let's test using it
     ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, 'L016', 2);
 
     std::array<uint16_t, 1> color{257};
-    doClearTest(ioSurface, GL_R16UI, GL_UNSIGNED_SHORT, color);
+    doClearTest(ioSurface, 1, 1, 0, GL_R16UI, GL_UNSIGNED_SHORT, color);
 }
 // TODO(cwallez@chromium.org): test reading from R16? It returns 0 maybe because samplerRect is
 // only for floating textures?
@@ -483,13 +585,10 @@ TEST_P(IOSurfaceClientBufferTest, RenderToBGRA1010102IOSurface)
 {
     ANGLE_SKIP_TEST_IF(!hasIOSurfaceExt());
 
-    // TODO(http://anglebug.com/4369)
-    ANGLE_SKIP_TEST_IF(isSwiftshader());
-
     ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, 'l10r', 4);
 
     std::array<uint32_t, 1> color{(0 << 30) | (1 << 22) | (2 << 12) | (3 << 2)};
-    doClearTest(ioSurface, GL_RGB10_A2, GL_UNSIGNED_INT_2_10_10_10_REV, color);
+    doClearTest(ioSurface, 1, 1, 0, GL_RGB10_A2, GL_UNSIGNED_INT_2_10_10_10_REV, color);
 }
 
 // Test reading from BGRA_1010102 IOSurfaces
@@ -500,7 +599,8 @@ TEST_P(IOSurfaceClientBufferTest, ReadFromBGRA1010102IOSurface)
     ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, 'l10r', 4);
 
     uint32_t color = (3 << 30) | (1 << 22) | (2 << 12) | (3 << 2);
-    doSampleTest(ioSurface, GL_RGB10_A2, GL_UNSIGNED_INT_2_10_10_10_REV, &color, sizeof(color),
+    doSampleTest(ioSurface, 1, 1, 0, GL_RGB10_A2, GL_UNSIGNED_INT_2_10_10_10_REV, &color,
+                 sizeof(color),
                  R | G | B);  // Don't test alpha, unorm '4' can't be represented with 2 bits.
 }
 
@@ -509,35 +609,111 @@ TEST_P(IOSurfaceClientBufferTest, RenderToRGBA16FIOSurface)
 {
     ANGLE_SKIP_TEST_IF(!hasIOSurfaceExt());
 
-    // TODO(http://anglebug.com/4369)
-    ANGLE_SKIP_TEST_IF(isSwiftshader());
-
     ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, 'RGhA', 8);
 
     std::array<GLushort, 4> color{
         gl::float32ToFloat16(1.0f / 255.0f), gl::float32ToFloat16(2.0f / 255.0f),
         gl::float32ToFloat16(3.0f / 255.0f), gl::float32ToFloat16(4.0f / 255.0f)};
-    doClearTest(ioSurface, GL_RGBA, GL_HALF_FLOAT, color);
+    doClearTest(ioSurface, 1, 1, 0, GL_RGBA, GL_HALF_FLOAT, color);
 }
 
 // Test reading from RGBA_16F IOSurfaces
-TEST_P(IOSurfaceClientBufferTest, ReadFromToRGBA16FIOSurfaceIOSurface)
+TEST_P(IOSurfaceClientBufferTest, ReadFromToRGBA16FIOSurface)
 {
     ANGLE_SKIP_TEST_IF(!hasIOSurfaceExt());
 
-    // TODO(http://anglebug.com/4369)
-    ANGLE_SKIP_TEST_IF(isSwiftshader());
-
     ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, 'RGhA', 8);
 
     std::array<GLushort, 4> color{
         gl::float32ToFloat16(1.0f / 255.0f), gl::float32ToFloat16(2.0f / 255.0f),
         gl::float32ToFloat16(3.0f / 255.0f), gl::float32ToFloat16(4.0f / 255.0f)};
-    doSampleTest(ioSurface, GL_RGBA, GL_HALF_FLOAT, color.data(), sizeof(GLushort) * 4,
+    doSampleTest(ioSurface, 1, 1, 0, GL_RGBA, GL_HALF_FLOAT, color.data(), sizeof(GLushort) * 4,
                  R | G | B | A);
 }
 
-// TODO(cwallez@chromium.org): Test using RGBA half float IOSurfaces ('RGhA')
+// Test using YUV420 IOSurfaces for rendering
+TEST_P(IOSurfaceClientBufferTest, RenderToYUV420IOSurface)
+{
+    ANGLE_SKIP_TEST_IF(!hasIOSurfaceExt());
+
+    std::vector<IOSurfacePlaneInfo> planes{{2, 2, 1}, {1, 1, 2}};
+    ScopedIOSurfaceRef ioSurface = CreateIOSurface('420v', planes);
+
+    {
+        std::array<GLubyte, 1> colors{1};
+        doClearTest(ioSurface, planes[0].width, planes[0].height, 0, GL_RED, GL_UNSIGNED_BYTE,
+                    colors);
+    }
+
+    {
+        std::array<GLubyte, 2> colors{1, 2};
+        doClearTest(ioSurface, planes[1].width, planes[1].height, 1, GL_RG, GL_UNSIGNED_BYTE,
+                    colors);
+    }
+}
+
+// Test reading from YUV420 IOSurfaces
+TEST_P(IOSurfaceClientBufferTest, ReadFromToYUV420IOSurface)
+{
+    ANGLE_SKIP_TEST_IF(!hasIOSurfaceExt());
+
+    std::vector<IOSurfacePlaneInfo> planes{{2, 2, 1}, {1, 1, 2}};
+    ScopedIOSurfaceRef ioSurface = CreateIOSurface('420v', planes);
+
+    {
+        std::array<GLubyte, 1> colors{1};
+        doSampleTest(ioSurface, planes[0].width, planes[0].height, 0, GL_RED, GL_UNSIGNED_BYTE,
+                     colors.data(), sizeof(GLubyte) * colors.size(), R);
+    }
+
+    {
+        std::array<GLubyte, 2> colors{1, 2};
+        doSampleTest(ioSurface, planes[1].width, planes[1].height, 1, GL_RG, GL_UNSIGNED_BYTE,
+                     colors.data(), sizeof(GLubyte) * colors.size(), R | G);
+    }
+}
+
+// Test using P010 IOSurfaces for rendering
+TEST_P(IOSurfaceClientBufferTest, RenderToP010IOSurface)
+{
+    ANGLE_SKIP_TEST_IF(!hasIOSurfaceExt());
+
+    std::vector<IOSurfacePlaneInfo> planes{{2, 2, 2}, {1, 1, 4}};
+    ScopedIOSurfaceRef ioSurface = CreateIOSurface('x420', planes);
+
+    {
+        std::array<GLushort, 1> colors{257};
+        doClearTest(ioSurface, planes[0].width, planes[0].height, 0, GL_RED, GL_UNSIGNED_SHORT,
+                    colors);
+    }
+
+    {
+        std::array<GLushort, 2> colors{257, 514};
+        doClearTest(ioSurface, planes[1].width, planes[1].height, 1, GL_RG, GL_UNSIGNED_SHORT,
+                    colors);
+    }
+}
+
+// Test reading from P010 IOSurfaces
+TEST_P(IOSurfaceClientBufferTest, ReadFromToP010IOSurface)
+{
+    ANGLE_SKIP_TEST_IF(!hasIOSurfaceExt());
+
+    std::vector<IOSurfacePlaneInfo> planes{{2, 2, 2}, {1, 1, 4}};
+    ScopedIOSurfaceRef ioSurface = CreateIOSurface('x420', planes);
+
+    {
+        std::array<GLushort, 1> colors{257};
+        doSampleTest(ioSurface, planes[0].width, planes[0].height, 0, GL_RED, GL_UNSIGNED_SHORT,
+                     colors.data(), sizeof(GLushort) * colors.size(), R);
+    }
+
+    {
+        std::array<GLushort, 2> colors{257, 514};
+        doSampleTest(ioSurface, planes[1].width, planes[1].height, 1, GL_RG, GL_UNSIGNED_SHORT,
+                     colors.data(), sizeof(GLushort) * colors.size(), R | G);
+    }
+}
 
 // Test blitting from IOSurface
 TEST_P(IOSurfaceClientBufferTest, BlitFromIOSurface)
@@ -555,6 +731,35 @@ TEST_P(IOSurfaceClientBufferTest, BlitToIOSurface)
     doBlitTest(false, 2, 2);
 }
 
+// Test using glCopyTexSubImage to copy to BGRX8888 IOSurfaces works.
+TEST_P(IOSurfaceClientBufferTest, CopySubImageToBGRX8888IOSurface)
+{
+    ANGLE_SKIP_TEST_IF(!hasIOSurfaceExt());
+
+    ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, 'BGRA', 4);
+
+    GLTexture texture;
+    glBindTexture(getGLTextureTarget(), texture);
+    glTexParameteri(getGLTextureTarget(), GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+    glTexParameteri(getGLTextureTarget(), GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+
+    // Bind the IOSurface to a texture and clear it.
+    EGLSurface pbuffer;
+    bindIOSurfaceToTexture(ioSurface, 1, 1, 0, GL_RGB, GL_UNSIGNED_BYTE, &pbuffer, &texture);
+
+    // 1. Clear default framebuffer with desired color.
+    GLColor color(1, 2, 3, 4);
+    glClearColor(color.R / 255.f, color.G / 255.f, color.B / 255.f, color.A / 255.f);
+    glClear(GL_COLOR_BUFFER_BIT);
+
+    // 2. Copy color from default framebuffer to iosurface's bound texture.
+    glCopyTexSubImage2D(getGLTextureTarget(), 0, 0, 0, 0, 0, 1, 1);
+    EXPECT_GL_NO_ERROR();
+
+    // 3. Do texture sampling verification.
+    doSampleTestWithTexture(texture, R | G | B);
+}
+
 // Test the validation errors for missing attributes for eglCreatePbufferFromClientBuffer with
 // IOSurface
 TEST_P(IOSurfaceClientBufferTest, NegativeValidationMissingAttributes)
@@ -983,6 +1188,24 @@ TEST_P(IOSurfaceClientBufferTest, MakeCurrent)
     EXPECT_EGL_SUCCESS();
 }
 
+// Test reading from BGRX8888 IOSurfaces with bound texture's base/max level set to zero.
+// This to verify that changing base/level shouldn't delete the binding.
+// bug: https://bugs.chromium.org/p/chromium/issues/detail?id=1337324
+TEST_P(IOSurfaceClientBufferTest, ReadFromBGRX8888IOSurfaceWithTexBaseMaxLevelSetToZero)
+{
+    ANGLE_SKIP_TEST_IF(!hasIOSurfaceExt());
+    ANGLE_SKIP_TEST_IF(getClientMajorVersion() < 3);
+
+    ScopedIOSurfaceRef ioSurface = CreateSinglePlaneIOSurface(1, 1, 'BGRA', 4);
+
+    GLColor color(3, 2, 1, 4);
+    doSampleTestWithExtraSteps(ioSurface, 1, 1, 0, GL_RGB, GL_UNSIGNED_BYTE, &color, sizeof(color),
+                               R | G | B, /* extra steps */ [this] {
+                                   glTexParameteri(getGLTextureTarget(), GL_TEXTURE_BASE_LEVEL, 0);
+                                   glTexParameteri(getGLTextureTarget(), GL_TEXTURE_MAX_LEVEL, 0);
+                               });
+}
+
 // TODO(cwallez@chromium.org): Test setting width and height to less than the IOSurface's work as
 // expected.
 
@@ -990,4 +1213,6 @@ ANGLE_INSTANTIATE_TEST(IOSurfaceClientBufferTest,
                        ES2_OPENGL(),
                        ES3_OPENGL(),
                        ES2_VULKAN_SWIFTSHADER(),
-                       ES3_VULKAN_SWIFTSHADER());
+                       ES3_VULKAN_SWIFTSHADER(),
+                       ES2_METAL(),
+                       ES3_METAL());
