@@ -12,9 +12,11 @@
 import argparse
 import contextlib
 import fnmatch
+import functools
 import json
 import logging
 import os
+import pathlib
 import platform
 import re
 import shutil
@@ -23,24 +25,17 @@ import tempfile
 import time
 import traceback
 
-# Add //src/testing into sys.path for importing xvfb and test_env, and
-# //src/testing/scripts for importing common.
-d = os.path.dirname
-THIS_DIR = d(os.path.abspath(__file__))
-sys.path.insert(0, d(THIS_DIR))
 
+PY_UTILS = str(pathlib.Path(__file__).resolve().parents[1] / 'py_utils')
+if PY_UTILS not in sys.path:
+    os.stat(PY_UTILS) and sys.path.insert(0, PY_UTILS)
+import android_helper
+import angle_path_util
+import angle_test_util
 from skia_gold import angle_skia_gold_properties
 from skia_gold import angle_skia_gold_session_manager
 
-ANGLE_SRC_DIR = d(d(d(THIS_DIR)))
-sys.path.insert(0, os.path.join(ANGLE_SRC_DIR, 'testing'))
-sys.path.insert(0, os.path.join(ANGLE_SRC_DIR, 'testing', 'scripts'))
-# Handle the Chromium-relative directory as well. As long as one directory
-# is valid, Python is happy.
-CHROMIUM_SRC_DIR = d(d(ANGLE_SRC_DIR))
-sys.path.insert(0, os.path.join(CHROMIUM_SRC_DIR, 'testing'))
-sys.path.insert(0, os.path.join(CHROMIUM_SRC_DIR, 'testing', 'scripts'))
-
+angle_path_util.AddDepsDirToPath('testing/scripts')
 import common
 import test_env
 import xvfb
@@ -57,6 +52,7 @@ DEFAULT_SCREENSHOT_PREFIX = 'angle_vulkan_'
 SWIFTSHADER_SCREENSHOT_PREFIX = 'angle_vulkan_swiftshader_'
 DEFAULT_BATCH_SIZE = 5
 DEFAULT_LOG = 'info'
+DEFAULT_GOLD_INSTANCE = 'angle'
 
 # Filters out stuff like: " I   72.572s run_tests_on_device(96071FFAZ00096) "
 ANDROID_LOGGING_PREFIX = r'I +\d+.\d+s \w+\(\w+\)  '
@@ -125,13 +121,35 @@ def add_skia_gold_args(parser):
         'pre-authenticated. Meant for testing locally instead of on the bots.')
 
 
-def run_wrapper(args, cmd, env, stdoutfile=None):
+@functools.lru_cache()
+def _use_adb(test_suite):
+    return android_helper.ApkFileExists(test_suite)
+
+
+def run_wrapper(test_suite, cmd_args, args, env, stdoutfile):
+    if _use_adb(args.test_suite):
+        return android_helper.RunTests(cmd_args, stdoutfile)[0]
+
+    cmd = [get_binary_name(test_suite)] + cmd_args
+
     if args.xvfb:
         return xvfb.run_executable(cmd, env, stdoutfile=stdoutfile)
     else:
         return test_env.run_command_with_output(cmd, env=env, stdoutfile=stdoutfile)
 
 
+def run_angle_system_info_test(sysinfo_args, args, env):
+    with temporary_dir() as temp_dir:
+        tempfile_path = os.path.join(temp_dir, 'stdout')
+        sysinfo_args += ['--render-test-output-dir=' + temp_dir]
+
+        if run_wrapper('angle_system_info_test', sysinfo_args, args, env, tempfile_path):
+            raise Exception('Error getting system info.')
+
+        with open(os.path.join(temp_dir, 'angle_system_info.json')) as f:
+            return json.load(f)
+
+
 def to_hex(num):
     return hex(int(num))
 
@@ -164,55 +182,15 @@ def get_skia_gold_keys(args, env):
         logging.exception('get_skia_gold_keys may only be called once')
     get_skia_gold_keys.called = True
 
-    class Filter:
+    sysinfo_args = ['--vulkan', '-v']
+    if args.swiftshader:
+        sysinfo_args.append('--swiftshader')
 
-        def __init__(self):
-            self.accepting_lines = True
-            self.done_accepting_lines = False
-            self.android_prefix = re.compile(ANDROID_LOGGING_PREFIX)
-            self.lines = []
-            self.is_android = False
-
-        def append(self, line):
-            if self.done_accepting_lines:
-                return
-            if 'Additional test environment' in line or 'android/test_runner.py' in line:
-                self.accepting_lines = False
-                self.is_android = True
-            if ANDROID_BEGIN_SYSTEM_INFO in line:
-                self.accepting_lines = True
-                return
-            if not self.accepting_lines:
-                return
-
-            if self.is_android:
-                line = self.android_prefix.sub('', line)
-
-            if line[0] == '}':
-                self.done_accepting_lines = True
-
-            self.lines.append(line)
-
-        def get(self):
-            return self.lines
-
-    with common.temporary_file() as tempfile_path:
-        binary = get_binary_name('angle_system_info_test')
-        sysinfo_args = [binary, '--vulkan', '-v']
-        if args.swiftshader:
-            sysinfo_args.append('--swiftshader')
-        if run_wrapper(args, sysinfo_args, env, tempfile_path):
-            raise Exception('Error getting system info.')
-
-        filter = Filter()
-
-        with open(tempfile_path) as f:
-            for line in f:
-                filter.append(line)
-
-        str = ''.join(filter.get())
-        logging.info(str)
-        json_data = json.loads(str)
+    if _use_adb(args.test_suite):
+        json_data = android_helper.AngleSystemInfo(sysinfo_args)
+        logging.info(json_data)
+    else:
+        json_data = run_angle_system_info_test(sysinfo_args, args, env)
 
     if len(json_data.get('gpus', [])) == 0 or not 'activeGPUIndex' in json_data:
         raise Exception('Error getting system info.')
@@ -276,8 +254,7 @@ def upload_test_result_to_skia_gold(args, gold_session_manager, gold_session, go
     png_file_name = os.path.join(screenshot_dir, prefix + image_name + '.png')
 
     if not os.path.isfile(png_file_name):
-        logging.info('Screenshot not found, test skipped.')
-        return SKIP
+        raise Exception('Screenshot not found: ' + png_file_name)
 
     status, error = gold_session.RunComparison(
         name=image_name, png_file=png_file_name, use_luci=use_luci)
@@ -335,11 +312,16 @@ def _get_gtest_filter_for_batch(args, batch):
 def _run_tests(args, tests, extra_flags, env, screenshot_dir, results, test_results):
     keys = get_skia_gold_keys(args, env)
 
+    if _use_adb(args.test_suite):
+        android_helper.PrepareTestSuite(args.test_suite)
+        if args.test_suite == DEFAULT_TEST_SUITE:
+            android_helper.RunSmokeTest()
+
     with temporary_dir('angle_skia_gold_') as skia_gold_temp_dir:
         gold_properties = angle_skia_gold_properties.ANGLESkiaGoldProperties(args)
         gold_session_manager = angle_skia_gold_session_manager.ANGLESkiaGoldSessionManager(
             skia_gold_temp_dir, gold_properties)
-        gold_session = gold_session_manager.GetSkiaGoldSession(keys)
+        gold_session = gold_session_manager.GetSkiaGoldSession(keys, instance=args.instance)
 
         traces = [trace.split(' ')[0] for trace in tests]
 
@@ -358,6 +340,9 @@ def _run_tests(args, tests, extra_flags, env, screenshot_dir, results, test_resu
         batches = _get_batches(traces, args.batch_size)
 
         for batch in batches:
+            if _use_adb(args.test_suite):
+                android_helper.PrepareRestrictedTraces(batch)
+
             for iteration in range(0, args.flaky_retries + 1):
                 with common.temporary_file() as tempfile_path:
                     # This is how we signal early exit
@@ -368,27 +353,33 @@ def _run_tests(args, tests, extra_flags, env, screenshot_dir, results, test_resu
                         logging.info('Test run failed, running retry #%d...' % iteration)
 
                     gtest_filter = _get_gtest_filter_for_batch(args, batch)
-                    cmd = [
-                        args.test_suite,
+                    cmd_args = [
                         gtest_filter,
-                        '--render-test-output-dir=%s' % screenshot_dir,
                         '--one-frame-only',
                         '--verbose-logging',
                         '--enable-all-trace-tests',
+                        '--render-test-output-dir=%s' % screenshot_dir,
                     ] + extra_flags
-                    batch_result = PASS if run_wrapper(args, cmd, env,
+                    batch_result = PASS if run_wrapper(args.test_suite, cmd_args, args, env,
                                                        tempfile_path) == 0 else FAIL
 
+                    with open(tempfile_path) as f:
+                        test_output = f.read() + '\n'
+
                     next_batch = []
                     for trace in batch:
                         artifacts = {}
 
                         if batch_result == PASS:
-                            logging.debug('upload test result: %s' % trace)
-                            result = upload_test_result_to_skia_gold(args, gold_session_manager,
-                                                                     gold_session, gold_properties,
-                                                                     screenshot_dir, trace,
-                                                                     artifacts)
+                            test_prefix = SWIFTSHADER_TEST_PREFIX if args.swiftshader else DEFAULT_TEST_PREFIX
+                            trace_skipped_notice = '[  SKIPPED ] ' + test_prefix + trace + '\n'
+                            if trace_skipped_notice in test_output:
+                                result = SKIP
+                            else:
+                                logging.debug('upload test result: %s' % trace)
+                                result = upload_test_result_to_skia_gold(
+                                    args, gold_session_manager, gold_session, gold_properties,
+                                    screenshot_dir, trace, artifacts)
                         else:
                             result = batch_result
 
@@ -442,11 +433,18 @@ def main():
     parser.add_argument(
         '-l', '--log', help='Log output level. Default is %s.' % DEFAULT_LOG, default=DEFAULT_LOG)
     parser.add_argument('--swiftshader', help='Test with SwiftShader.', action='store_true')
+    parser.add_argument(
+        '-i',
+        '--instance',
+        '--gold-instance',
+        '--skia-gold-instance',
+        help='Skia Gold instance. Default is "%s".' % DEFAULT_GOLD_INSTANCE,
+        default=DEFAULT_GOLD_INSTANCE)
 
     add_skia_gold_args(parser)
 
     args, extra_flags = parser.parse_known_args()
-    logging.basicConfig(level=args.log.upper())
+    angle_test_util.setupLogging(args.log.upper())
 
     env = os.environ.copy()
 
@@ -475,14 +473,9 @@ def main():
     rc = 0
 
     try:
-        if IsWindows():
-            args.test_suite = '.\\%s.exe' % args.test_suite
-        else:
-            args.test_suite = './%s' % args.test_suite
-
         # read test set
-        json_name = os.path.join(ANGLE_SRC_DIR, 'src', 'tests', 'restricted_traces',
-                                 'restricted_traces.json')
+        json_name = os.path.join(angle_path_util.ANGLE_ROOT_DIR, 'src', 'tests',
+                                 'restricted_traces', 'restricted_traces.json')
         with open(json_name) as fp:
             tests = json.load(fp)
 
