@@ -9,29 +9,31 @@
 
 import argparse
 import fnmatch
+import functools
 import importlib
 import io
 import json
 import logging
 import time
 import os
+import pathlib
 import re
 import subprocess
 import sys
 
-# Add //src/testing into sys.path for importing xvfb and test_env, and
-# //src/testing/scripts for importing common.
-d = os.path.dirname
-THIS_DIR = d(os.path.abspath(__file__))
-ANGLE_DIR = d(d(THIS_DIR))
-sys.path.append(os.path.join(ANGLE_DIR, 'testing'))
-sys.path.append(os.path.join(ANGLE_DIR, 'testing', 'scripts'))
+PY_UTILS = str(pathlib.Path(__file__).resolve().parent / 'py_utils')
+if PY_UTILS not in sys.path:
+    os.stat(PY_UTILS) and sys.path.insert(0, PY_UTILS)
+import android_helper
+import angle_path_util
+import angle_test_util
 
+angle_path_util.AddDepsDirToPath('testing/scripts')
 import common
 import test_env
 import xvfb
 
-sys.path.append(os.path.join(ANGLE_DIR, 'third_party', 'catapult', 'tracing'))
+angle_path_util.AddDepsDirToPath('third_party/catapult/tracing')
 from tracing.value import histogram
 from tracing.value import histogram_set
 from tracing.value import merge_histograms
@@ -77,24 +79,41 @@ def _popen(*args, **kwargs):
 
 def run_command_with_output(argv, stdoutfile, env=None, cwd=None, log=True):
     assert stdoutfile
-    with io.open(stdoutfile, 'wb') as writer:
+    with io.open(stdoutfile, 'wb') as writer, \
+          io.open(stdoutfile, 'rb') as reader:
         process = _popen(argv, env=env, cwd=cwd, stdout=writer, stderr=subprocess.STDOUT)
         test_env.forward_signals([process])
         while process.poll() is None:
+            if log:
+                sys.stdout.write(reader.read().decode('utf-8'))
             # This sleep is needed for signal propagation. See the
             # wait_with_signals() docstring.
             time.sleep(0.1)
+        if log:
+            sys.stdout.write(reader.read().decode('utf-8'))
         return process.returncode
 
 
-def _run_and_get_output(args, cmd, env):
+@functools.lru_cache()
+def _use_adb(test_suite):
+    return android_helper.ApkFileExists(test_suite)
+
+
+def _run_and_get_output(args, cmd, env, runner_args):
+    if _use_adb(args.test_suite):
+        result, output = android_helper.RunTests(cmd[1:], log_output=args.show_test_stdout)
+        return result, output.decode().split('\n')
+
+    runner_cmd = cmd + runner_args
+
     lines = []
-    logging.debug(' '.join(cmd))
+    logging.debug(' '.join(runner_cmd))
     with common.temporary_file() as tempfile_path:
         if args.xvfb:
-            exit_code = xvfb.run_executable(cmd, env, stdoutfile=tempfile_path)
+            exit_code = xvfb.run_executable(runner_cmd, env, stdoutfile=tempfile_path)
         else:
-            exit_code = run_command_with_output(cmd, env=env, stdoutfile=tempfile_path, log=False)
+            exit_code = run_command_with_output(
+                runner_cmd, env=env, stdoutfile=tempfile_path, log=args.show_test_stdout)
         with open(tempfile_path) as f:
             for line in f:
                 lines.append(line.strip())
@@ -295,11 +314,14 @@ def main():
         % DEFAULT_CALIBRATION_TIME,
         type=int,
         default=DEFAULT_CALIBRATION_TIME)
+    parser.add_argument(
+        '--show-test-stdout', help='Prints all test stdout during execution.', action='store_true')
+    parser.add_argument(
+        '--perf-counters', help='Colon-separated list of extra perf counter metrics.')
 
     args, extra_flags = parser.parse_known_args()
 
-    importlib.reload(logging)
-    logging.basicConfig(level=args.log.upper())
+    angle_test_util.setupLogging(args.log.upper())
 
     start_time = time.time()
 
@@ -320,11 +342,17 @@ def main():
         args.shard_index = int(env.pop('GTEST_SHARD_INDEX'))
 
     # Get test list
-    cmd = [get_binary_name(args.test_suite), '--list-tests', '--verbose']
-    exit_code, lines = _run_and_get_output(args, cmd, env)
-    if exit_code != EXIT_SUCCESS:
-        logging.fatal('Could not find test list from test output:\n%s' % '\n'.join(lines))
-    tests = _get_tests_from_output(lines)
+    if _use_adb(args.test_suite):
+        android_helper.PrepareTestSuite(args.test_suite)
+        tests = android_helper.ListTests()
+        if args.test_suite == DEFAULT_TEST_SUITE:
+            android_helper.RunSmokeTest()
+    else:
+        cmd = [get_binary_name(args.test_suite), '--list-tests', '--verbose']
+        exit_code, lines = _run_and_get_output(args, cmd, env, [])
+        if exit_code != EXIT_SUCCESS:
+            logging.fatal('Could not find test list from test output:\n%s' % '\n'.join(lines))
+        tests = _get_tests_from_output(lines)
 
     if args.filter:
         tests = _filter_tests(tests, args.filter)
@@ -345,18 +373,28 @@ def main():
     histograms = histogram_set.HistogramSet()
     total_errors = 0
 
+    prepared_traces = set()
     for test_index in range(num_tests):
         test = tests[test_index]
+
+        if _use_adb(args.test_suite):
+            trace = android_helper.GetTraceFromTestName(test)
+            if trace and trace not in prepared_traces:
+                android_helper.PrepareRestrictedTraces([trace])
+                prepared_traces.add(trace)
+
         cmd = [
             get_binary_name(args.test_suite),
             '--gtest_filter=%s' % test,
+            '--verbose',
+            '--calibration-time',
+            str(args.calibration_time),
+        ]
+        runner_args = [
             '--extract-test-list-from-filter',
             '--enable-device-cache',
             '--skip-clear-data',
             '--use-existing-test-data',
-            '--verbose',
-            '--calibration-time',
-            str(args.calibration_time),
         ]
         if args.steps_per_trial:
             steps_per_trial = args.steps_per_trial
@@ -366,7 +404,8 @@ def main():
                 '--warmup-loops',
                 str(args.warmup_loops),
             ]
-            exit_code, calibrate_output = _run_and_get_output(args, cmd_calibrate, env)
+            exit_code, calibrate_output = _run_and_get_output(args, cmd_calibrate, env,
+                                                              runner_args)
             if exit_code != EXIT_SUCCESS:
                 logging.fatal('%s failed. Output:\n%s' %
                               (cmd_calibrate[0], '\n'.join(calibrate_output)))
@@ -376,6 +415,7 @@ def main():
             steps_per_trial = _get_results_from_output(calibrate_output, 'steps_to_run')
             if not steps_per_trial:
                 logging.warning('Skipping test %s' % test)
+                results.result_skip(test)
                 continue
             assert (len(steps_per_trial) == 1)
             steps_per_trial = int(steps_per_trial[0])
@@ -395,13 +435,18 @@ def main():
                 '--trials',
                 str(args.trials_per_sample),
             ]
+
             if args.smoke_test_mode:
                 cmd_run += ['--no-warmup']
             else:
                 cmd_run += ['--warmup-loops', str(args.warmup_loops)]
+
+            if args.perf_counters:
+                cmd_run += ['--perf-counters', args.perf_counters]
+
             with common.temporary_file() as histogram_file_path:
                 cmd_run += ['--isolated-script-test-perf-output=%s' % histogram_file_path]
-                exit_code, output = _run_and_get_output(args, cmd_run, env)
+                exit_code, output = _run_and_get_output(args, cmd_run, env, runner_args)
                 if exit_code != EXIT_SUCCESS:
                     logging.error('%s failed. Output:\n%s' % (cmd_run[0], '\n'.join(output)))
                     results.result_fail(test)
@@ -454,6 +499,10 @@ def main():
                     histograms.Merge(merged_histogram)
             else:
                 logging.error('Test %s failed to record some samples' % test)
+                results.result_fail(test)
+
+    for test in tests:
+        assert results.has_result(test)
 
     if args.isolated_script_test_output:
         results.save_to_output_file(args.test_suite, args.isolated_script_test_output)
