@@ -130,6 +130,7 @@ struct GLColor
 {
     constexpr GLColor() : R(0), G(0), B(0), A(0) {}
     constexpr GLColor(GLubyte r, GLubyte g, GLubyte b, GLubyte a) : R(r), G(g), B(b), A(a) {}
+    GLColor(const angle::Vector3 &floatColor);
     GLColor(const angle::Vector4 &floatColor);
     GLColor(GLuint colorValue);
 
@@ -159,22 +160,19 @@ struct GLColor
     static const GLColor magenta;
 };
 
-struct GLColor16UI
+template <typename T>
+struct GLColorT
 {
-    constexpr GLColor16UI() : GLColor16UI(0, 0, 0, 0) {}
-    constexpr GLColor16UI(GLushort r, GLushort g, GLushort b, GLushort a) : R(r), G(g), B(b), A(a)
-    {}
+    constexpr GLColorT() : GLColorT(0, 0, 0, 0) {}
+    constexpr GLColorT(T r, T g, T b, T a) : R(r), G(g), B(b), A(a) {}
 
-    GLushort R, G, B, A;
+    T R, G, B, A;
 };
 
-struct GLColor32F
-{
-    constexpr GLColor32F() : GLColor32F(0.0f, 0.0f, 0.0f, 0.0f) {}
-    constexpr GLColor32F(GLfloat r, GLfloat g, GLfloat b, GLfloat a) : R(r), G(g), B(b), A(a) {}
-
-    GLfloat R, G, B, A;
-};
+using GLColor16UI = GLColorT<uint16_t>;
+using GLColor32F  = GLColorT<float>;
+using GLColor32I  = GLColorT<int32_t>;
+using GLColor32UI = GLColorT<uint32_t>;
 
 static constexpr GLColor32F kFloatBlack = {0.0f, 0.0f, 0.0f, 1.0f};
 static constexpr GLColor32F kFloatRed   = {1.0f, 0.0f, 0.0f, 1.0f};
@@ -216,7 +214,12 @@ GLColor32F MakeGLColor32F(TR r, TG g, TB b, TA a)
                       static_cast<GLfloat>(a));
 }
 
-bool operator==(const GLColor32F &a, const GLColor32F &b);
+template <typename T>
+bool operator==(const GLColorT<T> &a, const GLColorT<T> &b)
+{
+    return a.R == b.R && a.G == b.G && a.B == b.B && a.A == b.A;
+}
+
 std::ostream &operator<<(std::ostream &ostream, const GLColor32F &color);
 GLColor32F ReadColor32F(GLint x, GLint y);
 
@@ -227,6 +230,7 @@ constexpr std::array<GLenum, 6> kCubeFaces = {
 
 void LoadEntryPointsWithUtilLoader(angle::GLESDriverType driver);
 
+bool IsFormatEmulated(GLenum target);
 }  // namespace angle
 
 #define EXPECT_PIXEL_EQ(x, y, r, g, b, a) \
@@ -249,15 +253,28 @@ void LoadEntryPointsWithUtilLoader(angle::GLESDriverType driver);
 
 #define EXPECT_PIXEL_COLOR32F_EQ(x, y, angleColor) EXPECT_EQ(angleColor, angle::ReadColor32F(x, y))
 
-#define EXPECT_PIXEL_RECT_EQ(x, y, width, height, color)                                           \
-    do                                                                                             \
-    {                                                                                              \
-        std::vector<GLColor> actualColors((width) * (height));                                     \
-        glReadPixels((x), (y), (width), (height), GL_RGBA, GL_UNSIGNED_BYTE, actualColors.data()); \
-        std::vector<GLColor> expectedColors((width) * (height), color);                            \
-        EXPECT_EQ(expectedColors, actualColors);                                                   \
+#define EXPECT_PIXEL_RECT_T_EQ(T, x, y, width, height, format, type, color)           \
+    do                                                                                \
+    {                                                                                 \
+        std::vector<T> actualColors((width) * (height));                              \
+        glReadPixels((x), (y), (width), (height), format, type, actualColors.data()); \
+        std::vector<T> expectedColors((width) * (height), color);                     \
+        EXPECT_EQ(expectedColors, actualColors);                                      \
     } while (0)
 
+#define EXPECT_PIXEL_RECT_EQ(x, y, width, height, color) \
+    EXPECT_PIXEL_RECT_T_EQ(GLColor, x, y, width, height, GL_RGBA, GL_UNSIGNED_BYTE, color)
+
+#define EXPECT_PIXEL_RECT32F_EQ(x, y, width, height, color) \
+    EXPECT_PIXEL_RECT_T_EQ(GLColor32F, x, y, width, height, GL_RGBA, GL_FLOAT, color)
+
+#define EXPECT_PIXEL_RECT32I_EQ(x, y, width, height, color) \
+    EXPECT_PIXEL_RECT_T_EQ(GLColor32I, x, y, width, height, GL_RGBA_INTEGER, GL_INT, color)
+
+#define EXPECT_PIXEL_RECT32UI_EQ(x, y, width, height, color)                                   \
+    EXPECT_PIXEL_RECT_T_EQ(GLColor32UI, x, y, width, height, GL_RGBA_INTEGER, GL_UNSIGNED_INT, \
+                           color)
+
 #define EXPECT_PIXEL_NEAR_HELPER(x, y, r, g, b, a, abs_error, ctype, format, type) \
     do                                                                             \
     {                                                                              \
@@ -361,9 +378,6 @@ class ANGLETestBase
   public:
     void setWindowVisible(OSWindow *osWindow, bool isVisible);
 
-    virtual void overrideWorkaroundsD3D(angle::FeaturesD3D *featuresD3D) {}
-    virtual void overrideFeaturesVk(angle::FeaturesVk *featuresVulkan) {}
-
     static void ReleaseFixtures();
 
     bool isSwiftshader() const
@@ -468,6 +482,7 @@ class ANGLETestBase
     void setBindGeneratesResource(bool bindGeneratesResource);
     void setClientArraysEnabled(bool enabled);
     void setRobustResourceInit(bool enabled);
+    void setMutableRenderBuffer(bool enabled);
     void setContextProgramCacheEnabled(bool enabled);
     void setContextResetStrategy(EGLenum resetStrategy);
     void forceNewDisplay();
@@ -482,7 +497,6 @@ class ANGLETestBase
     EGLWindow *getEGLWindow() const;
     int getWindowWidth() const;
     int getWindowHeight() const;
-    bool isEmulatedPrerotation() const;
 
     EGLint getPlatformRenderer() const;
 
@@ -498,7 +512,7 @@ class ANGLETestBase
     // Has a float uniform "u_layer" to choose the 3D texture layer.
     GLuint get3DTexturedQuadProgram();
 
-    class ScopedIgnorePlatformMessages : angle::NonCopyable
+    class [[nodiscard]] ScopedIgnorePlatformMessages : angle::NonCopyable
     {
       public:
         ScopedIgnorePlatformMessages();
@@ -532,18 +546,8 @@ class ANGLETestBase
                mCurrentParams->isSwiftshader();
     }
 
-    bool isAsyncCommandQueueFeatureEnabled() const
-    {
-        return mCurrentParams->eglParameters.asyncCommandQueueFeatureVulkan == EGL_TRUE;
-    }
-
     bool platformSupportsMultithreading() const;
 
-    bool isAllocateNonZeroMemoryEnabled() const
-    {
-        return mCurrentParams->getAllocateNonZeroMemoryFeature() == EGL_TRUE;
-    }
-
     bool mIsSetUp = false;
 
   private:
@@ -614,10 +618,10 @@ class ANGLETestBase
 };
 
 template <typename Params = angle::PlatformParameters>
-class ANGLETestWithParam : public ANGLETestBase, public ::testing::TestWithParam<Params>
+class ANGLETest : public ANGLETestBase, public ::testing::TestWithParam<Params>
 {
   protected:
-    ANGLETestWithParam();
+    ANGLETest();
 
     virtual void testSetUp() {}
     virtual void testTearDown() {}
@@ -650,19 +654,14 @@ class ANGLETestWithParam : public ANGLETestBase, public ::testing::TestWithParam
 };
 
 template <typename Params>
-ANGLETestWithParam<Params>::ANGLETestWithParam()
+ANGLETest<Params>::ANGLETest()
     : ANGLETestBase(std::get<angle::PlatformParameters>(this->GetParam()))
 {}
 
 template <>
-inline ANGLETestWithParam<angle::PlatformParameters>::ANGLETestWithParam()
-    : ANGLETestBase(this->GetParam())
+inline ANGLETest<angle::PlatformParameters>::ANGLETest() : ANGLETestBase(this->GetParam())
 {}
 
-// Note: this hack is not necessary in C++17.  Once we switch to C++17, we can just rename
-// ANGLETestWithParam to ANGLETest.
-using ANGLETest = ANGLETestWithParam<>;
-
 class ANGLETestEnvironment : public testing::Environment
 {
   public:
