@@ -7,10 +7,36 @@
 #include "test_utils/angle_test_configs.h"
 
 #include "common/platform.h"
+#include "common/string_utils.h"
 #include "util/util_gl.h"
 
+#include <algorithm>
+#include <cctype>
+
 namespace angle
 {
+namespace
+{
+void AppendCapitalizedFeature(std::ostream &stream, Feature feature)
+{
+    const char *name = GetFeatureName(feature);
+
+    if (name == nullptr)
+    {
+        stream << "InternalError";
+        return;
+    }
+
+    const std::string camelCase = angle::ToCamelCase(name);
+
+    stream << static_cast<char>(std::toupper(camelCase[0])) << (camelCase.c_str() + 1);
+}
+
+bool HasFeatureOverride(const std::vector<Feature> &overrides, Feature feature)
+{
+    return std::find(overrides.begin(), overrides.end(), feature) != overrides.end();
+}
+}  // namespace
 
 PlatformParameters::PlatformParameters() : PlatformParameters(2, 0, GLESDriverType::AngleEGL) {}
 
@@ -44,12 +70,37 @@ EGLint PlatformParameters::getDeviceType() const
     return eglParameters.deviceType;
 }
 
+bool PlatformParameters::isSwiftshader() const
+{
+    return eglParameters.deviceType == EGL_PLATFORM_ANGLE_DEVICE_TYPE_SWIFTSHADER_ANGLE;
+}
+
+bool PlatformParameters::isVulkan() const
+{
+    return eglParameters.renderer == EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE;
+}
+
+bool PlatformParameters::isANGLE() const
+{
+    return driver == GLESDriverType::AngleEGL;
+}
+
 void PlatformParameters::initDefaultParameters()
 {
     // Default debug layers to enabled in tests.
     eglParameters.debugLayersEnabled = EGL_TRUE;
 }
 
+bool PlatformParameters::isEnabled(Feature feature) const
+{
+    return HasFeatureOverride(eglParameters.enabledFeatureOverrides, feature);
+}
+
+bool PlatformParameters::isDisabled(Feature feature) const
+{
+    return HasFeatureOverride(eglParameters.disabledFeatureOverrides, feature);
+}
+
 bool operator<(const PlatformParameters &a, const PlatformParameters &b)
 {
     return a.tie() < b.tie();
@@ -65,6 +116,31 @@ bool operator!=(const PlatformParameters &a, const PlatformParameters &b)
     return a.tie() != b.tie();
 }
 
+const char *GetRendererName(EGLint renderer)
+{
+    switch (renderer)
+    {
+        case EGL_PLATFORM_ANGLE_TYPE_DEFAULT_ANGLE:
+            return "Default";
+        case EGL_PLATFORM_ANGLE_TYPE_D3D9_ANGLE:
+            return "D3D9";
+        case EGL_PLATFORM_ANGLE_TYPE_D3D11_ANGLE:
+            return "D3D11";
+        case EGL_PLATFORM_ANGLE_TYPE_METAL_ANGLE:
+            return "Metal";
+        case EGL_PLATFORM_ANGLE_TYPE_NULL_ANGLE:
+            return "Null";
+        case EGL_PLATFORM_ANGLE_TYPE_OPENGL_ANGLE:
+            return "OpenGL";
+        case EGL_PLATFORM_ANGLE_TYPE_OPENGLES_ANGLE:
+            return "OpenGLES";
+        case EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE:
+            return "Vulkan";
+        default:
+            return "Undefined";
+    }
+}
+
 std::ostream &operator<<(std::ostream &stream, const PlatformParameters &pp)
 {
     stream << "ES" << pp.majorVersion << "_";
@@ -76,44 +152,13 @@ std::ostream &operator<<(std::ostream &stream, const PlatformParameters &pp)
     switch (pp.driver)
     {
         case GLESDriverType::AngleEGL:
-        {
-            switch (pp.eglParameters.renderer)
-            {
-                case EGL_PLATFORM_ANGLE_TYPE_DEFAULT_ANGLE:
-                    stream << "Default";
-                    break;
-                case EGL_PLATFORM_ANGLE_TYPE_D3D9_ANGLE:
-                    stream << "D3D9";
-                    break;
-                case EGL_PLATFORM_ANGLE_TYPE_D3D11_ANGLE:
-                    stream << "D3D11";
-                    break;
-                case EGL_PLATFORM_ANGLE_TYPE_METAL_ANGLE:
-                    stream << "Metal";
-                    break;
-                case EGL_PLATFORM_ANGLE_TYPE_NULL_ANGLE:
-                    stream << "Null";
-                    break;
-                case EGL_PLATFORM_ANGLE_TYPE_OPENGL_ANGLE:
-                    stream << "OpenGL";
-                    break;
-                case EGL_PLATFORM_ANGLE_TYPE_OPENGLES_ANGLE:
-                    stream << "OpenGLES";
-                    break;
-                case EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE:
-                    stream << "Vulkan";
-                    break;
-                default:
-                    stream << "Undefined";
-                    break;
-            }
+            stream << GetRendererName(pp.eglParameters.renderer);
             break;
-        }
         case GLESDriverType::SystemWGL:
             stream << "WGL";
             break;
         case GLESDriverType::SystemEGL:
-            stream << "GLES";
+            stream << "EGL";
             break;
         default:
             stream << "Error";
@@ -182,27 +227,25 @@ std::ostream &operator<<(std::ostream &stream, const PlatformParameters &pp)
         stream << "_NoFixture";
     }
 
-    if (pp.eglParameters.contextVirtualization == EGL_FALSE)
+    if (pp.eglParameters.displayPowerPreference == EGL_LOW_POWER_ANGLE)
     {
-        stream << "_NoVirtual";
+        stream << "_LowPowerGPU";
     }
 
-    if (pp.eglParameters.commandGraphFeature == EGL_FALSE)
+    if (pp.eglParameters.displayPowerPreference == EGL_HIGH_POWER_ANGLE)
     {
-        stream << "_NoCommandGraph";
-    }
-    else if (pp.eglParameters.commandGraphFeature == EGL_TRUE)
-    {
-        stream << "_CommandGraph";
+        stream << "_HighPowerGPU";
     }
 
-    if (pp.eglParameters.transformFeedbackFeature == EGL_FALSE)
+    for (Feature feature : pp.eglParameters.enabledFeatureOverrides)
     {
-        stream << "_NoTransformFeedback";
+        stream << "_";
+        AppendCapitalizedFeature(stream, feature);
     }
-    else if (pp.eglParameters.transformFeedbackFeature == EGL_TRUE)
+    for (Feature feature : pp.eglParameters.disabledFeatureOverrides)
     {
-        stream << "_TransformFeedback";
+        stream << "_No";
+        AppendCapitalizedFeature(stream, feature);
     }
 
     return stream;
@@ -278,12 +321,6 @@ EGLPlatformParameters D3D11_FL10_0()
                                  EGL_PLATFORM_ANGLE_DEVICE_TYPE_HARDWARE_ANGLE);
 }
 
-EGLPlatformParameters D3D11_FL9_3()
-{
-    return EGLPlatformParameters(EGL_PLATFORM_ANGLE_TYPE_D3D11_ANGLE, 9, 3,
-                                 EGL_PLATFORM_ANGLE_DEVICE_TYPE_HARDWARE_ANGLE);
-}
-
 EGLPlatformParameters D3D11_NULL()
 {
     return EGLPlatformParameters(EGL_PLATFORM_ANGLE_TYPE_D3D11_ANGLE, EGL_DONT_CARE, EGL_DONT_CARE,
@@ -320,12 +357,6 @@ EGLPlatformParameters D3D11_FL10_0_WARP()
                                  EGL_PLATFORM_ANGLE_DEVICE_TYPE_D3D_WARP_ANGLE);
 }
 
-EGLPlatformParameters D3D11_FL9_3_WARP()
-{
-    return EGLPlatformParameters(EGL_PLATFORM_ANGLE_TYPE_D3D11_ANGLE, 9, 3,
-                                 EGL_PLATFORM_ANGLE_DEVICE_TYPE_D3D_WARP_ANGLE);
-}
-
 EGLPlatformParameters D3D11_REFERENCE()
 {
     return EGLPlatformParameters(EGL_PLATFORM_ANGLE_TYPE_D3D11_ANGLE, EGL_DONT_CARE, EGL_DONT_CARE,
@@ -356,12 +387,6 @@ EGLPlatformParameters D3D11_FL10_0_REFERENCE()
                                  EGL_PLATFORM_ANGLE_DEVICE_TYPE_D3D_REFERENCE_ANGLE);
 }
 
-EGLPlatformParameters D3D11_FL9_3_REFERENCE()
-{
-    return EGLPlatformParameters(EGL_PLATFORM_ANGLE_TYPE_D3D11_ANGLE, 9, 3,
-                                 EGL_PLATFORM_ANGLE_DEVICE_TYPE_D3D_REFERENCE_ANGLE);
-}
-
 EGLPlatformParameters OPENGL()
 {
     return EGLPlatformParameters(EGL_PLATFORM_ANGLE_TYPE_OPENGL_ANGLE);
@@ -404,6 +429,15 @@ EGLPlatformParameters OPENGL_OR_GLES()
 #endif
 }
 
+EGLPlatformParameters OPENGL_OR_GLES(EGLint major, EGLint minor)
+{
+#if defined(ANGLE_PLATFORM_ANDROID)
+    return OPENGLES(major, minor);
+#else
+    return OPENGL(major, minor);
+#endif
+}
+
 EGLPlatformParameters OPENGL_OR_GLES_NULL()
 {
 #if defined(ANGLE_PLATFORM_ANDROID)
@@ -477,11 +511,6 @@ PlatformParameters ES2_D3D11_FL10_0()
     return PlatformParameters(2, 0, egl_platform::D3D11_FL10_0());
 }
 
-PlatformParameters ES2_D3D11_FL9_3()
-{
-    return PlatformParameters(2, 0, egl_platform::D3D11_FL9_3());
-}
-
 PlatformParameters ES2_D3D11_WARP()
 {
     return PlatformParameters(2, 0, egl_platform::D3D11_WARP());
@@ -502,11 +531,6 @@ PlatformParameters ES2_D3D11_FL10_0_WARP()
     return PlatformParameters(2, 0, egl_platform::D3D11_FL10_0_WARP());
 }
 
-PlatformParameters ES2_D3D11_FL9_3_WARP()
-{
-    return PlatformParameters(2, 0, egl_platform::D3D11_FL9_3_WARP());
-}
-
 PlatformParameters ES2_D3D11_REFERENCE()
 {
     return PlatformParameters(2, 0, egl_platform::D3D11_REFERENCE());
@@ -527,11 +551,6 @@ PlatformParameters ES2_D3D11_FL10_0_REFERENCE()
     return PlatformParameters(2, 0, egl_platform::D3D11_FL10_0_REFERENCE());
 }
 
-PlatformParameters ES2_D3D11_FL9_3_REFERENCE()
-{
-    return PlatformParameters(2, 0, egl_platform::D3D11_FL9_3_REFERENCE());
-}
-
 PlatformParameters ES3_D3D11()
 {
     return PlatformParameters(3, 0, egl_platform::D3D11());
@@ -737,6 +756,21 @@ PlatformParameters ES31_VULKAN_SWIFTSHADER()
     return PlatformParameters(3, 1, egl_platform::VULKAN_SWIFTSHADER());
 }
 
+PlatformParameters ES32_VULKAN()
+{
+    return PlatformParameters(3, 2, egl_platform::VULKAN());
+}
+
+PlatformParameters ES32_VULKAN_NULL()
+{
+    return PlatformParameters(3, 2, egl_platform::VULKAN_NULL());
+}
+
+PlatformParameters ES32_VULKAN_SWIFTSHADER()
+{
+    return PlatformParameters(3, 2, egl_platform::VULKAN_SWIFTSHADER());
+}
+
 PlatformParameters ES1_METAL()
 {
     return PlatformParameters(1, 0, egl_platform::METAL());
@@ -761,4 +795,14 @@ PlatformParameters ES3_WGL()
 {
     return PlatformParameters(3, 0, GLESDriverType::SystemWGL);
 }
+
+PlatformParameters ES2_EGL()
+{
+    return PlatformParameters(2, 0, GLESDriverType::SystemEGL);
+}
+
+PlatformParameters ES3_EGL()
+{
+    return PlatformParameters(3, 0, GLESDriverType::SystemEGL);
+}
 }  // namespace angle
