@@ -362,6 +362,11 @@ bool IsIntelUHD630Mobile()
     return HasSystemDeviceID(kVendorID_Intel, kDeviceID_UHD630Mobile);
 }
 
+bool IsIntelHD630Mobile()
+{
+    return HasSystemDeviceID(kVendorID_Intel, kDeviceID_HD630Mobile);
+}
+
 bool IsAMD()
 {
     return HasSystemVendorID(kVendorID_AMD);
@@ -572,7 +577,8 @@ bool IsConfigAllowlisted(const SystemInfo &systemInfo, const PlatformParameters
                 return true;
             case EGL_PLATFORM_ANGLE_TYPE_VULKAN_ANGLE:
                 // http://issuetracker.google.com/173004081
-                return !IsIntel() || param.eglParameters.asyncCommandQueueFeatureVulkan != EGL_TRUE;
+                return !IsIntel() || !param.isEnabled(Feature::AsyncCommandQueue) ||
+                       param.isDisabled(Feature::AsyncCommandQueue);
             default:
                 return false;
         }
@@ -606,7 +612,7 @@ bool IsConfigAllowlisted(const SystemInfo &systemInfo, const PlatformParameters
                 {
                     return false;
                 }
-                if (param.eglParameters.supportsVulkanViewportFlip == EGL_FALSE)
+                if (param.isDisabled(Feature::SupportsNegativeViewport))
                 {
                     return false;
                 }
