@@ -389,6 +389,11 @@ bool OSWindow::takeScreenshot(uint8_t *pixelData)
     return false;
 }
 
+void *OSWindow::getPlatformExtension()
+{
+    return reinterpret_cast<void *>(getNativeWindow());
+}
+
 bool OSWindow::popEvent(Event *event)
 {
     if (mEvents.size() > 0 && event)
