@@ -17,7 +17,7 @@ class AndroidWindow : public OSWindow
     AndroidWindow();
     ~AndroidWindow() override;
 
-    bool initialize(const std::string &name, int width, int height) override;
+    void disableErrorMessageDialog() override;
     void destroy() override;
 
     void resetNativeWindow() override;
@@ -27,11 +27,17 @@ class AndroidWindow : public OSWindow
     void messageLoop() override;
 
     void setMousePosition(int x, int y) override;
+    bool setOrientation(int width, int height) override;
     bool setPosition(int x, int y) override;
     bool resize(int width, int height) override;
     void setVisible(bool isVisible) override;
 
     void signalTestEvent() override;
+
+    ANGLE_UTIL_EXPORT static std::string GetExternalStorageDirectory();
+
+  private:
+    bool initializeImpl(const std::string &name, int width, int height) override;
 };
 
 #endif /* UTIL_ANDROID_WINDOW_H_ */
