@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2019 Apple Inc.  All rights reserved.
+ * Copyright (C) 2003-2022 Apple Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -51,7 +51,7 @@
 #include <os/log.h>
 #endif
 
-#if USE(JOURNALD)
+#if ENABLE(JOURNALD_LOG)
 #define SD_JOURNAL_SUPPRESS_LOCATION
 #include <systemd/sd-journal.h>
 #endif
@@ -95,12 +95,14 @@ extern "C" void _ReadWriteBarrier(void);
 #define LOG_DISABLED !ASSERT_ENABLED
 #endif
 
-#ifndef RELEASE_LOG_DISABLED
-#define RELEASE_LOG_DISABLED !(USE(OS_LOG) || USE(JOURNALD))
+#if ENABLE(RELEASE_LOG)
+#define RELEASE_LOG_DISABLED 0
+#else
+#define RELEASE_LOG_DISABLED !(USE(OS_LOG) || ENABLE(JOURNALD_LOG))
 #endif
 
 #ifndef VERBOSE_RELEASE_LOG
-#define VERBOSE_RELEASE_LOG USE(JOURNALD)
+#define VERBOSE_RELEASE_LOG ENABLE(JOURNALD_LOG)
 #endif
 
 #if COMPILER(GCC_COMPATIBLE)
@@ -152,6 +154,9 @@ extern "C" {
 enum class WTFLogChannelState : uint8_t { Off, On, OnWithAccumulation };
 #undef Always
 enum class WTFLogLevel : uint8_t { Always, Error, Warning, Info, Debug };
+namespace WTF {
+class PrintStream;
+}
 #else
 typedef uint8_t WTFLogChannelState;
 typedef uint8_t WTFLogLevel;
@@ -163,10 +168,10 @@ typedef struct {
     WTFLogLevel level;
 #if !RELEASE_LOG_DISABLED
     const char* subsystem;
-#endif
-#if USE(OS_LOG) && !RELEASE_LOG_DISABLED
+#if USE(OS_LOG)
     __unsafe_unretained os_log_t osLogChannel;
 #endif
+#endif
 } WTFLogChannel;
 
 #define LOG_CHANNEL(name) JOIN_LOG_CHANNEL_WITH_PREFIX(LOG_CHANNEL_PREFIX, name)
@@ -178,6 +183,8 @@ typedef struct {
 #define LOG_CHANNEL_WEBKIT_SUBSYSTEM "WebKitGTK"
 #elif PLATFORM(WPE)
 #define LOG_CHANNEL_WEBKIT_SUBSYSTEM "WPEWebKit"
+#elif PLATFORM(PLAYSTATION)
+#define LOG_CHANNEL_WEBKIT_SUBSYSTEM "SceNKWebKit"
 #else
 #define LOG_CHANNEL_WEBKIT_SUBSYSTEM "com.apple.WebKit"
 #endif
@@ -187,19 +194,23 @@ typedef struct {
 
 #if !defined(DEFINE_LOG_CHANNEL)
 #if RELEASE_LOG_DISABLED
-#define DEFINE_LOG_CHANNEL(name, subsystem) \
-    WTFLogChannel LOG_CHANNEL(name) = { (WTFLogChannelState)0, #name, (WTFLogLevel)1 };
-#endif
-#if USE(OS_LOG) && !RELEASE_LOG_DISABLED
-#define DEFINE_LOG_CHANNEL(name, subsystem) \
-    WTFLogChannel LOG_CHANNEL(name) = { (WTFLogChannelState)0, #name, (WTFLogLevel)1, subsystem, OS_LOG_DEFAULT };
-#endif
-#if USE(JOURNALD) && !RELEASE_LOG_DISABLED
-#define DEFINE_LOG_CHANNEL(name, subsystem)                             \
-    WTFLogChannel LOG_CHANNEL(name) = { (WTFLogChannelState)0, #name, (WTFLogLevel)1, subsystem };
+#define DEFINE_LOG_CHANNEL_WITH_DETAILS(name, initialState, level, subsystem) \
+    WTFLogChannel LOG_CHANNEL(name) = { initialState, #name, level };
+#elif USE(OS_LOG)
+#define DEFINE_LOG_CHANNEL_WITH_DETAILS(name, initialState, level, subsystem) \
+    WTFLogChannel LOG_CHANNEL(name) = { initialState, #name, level, subsystem, OS_LOG_DEFAULT };
+#else
+#define DEFINE_LOG_CHANNEL_WITH_DETAILS(name, initialState, level, subsystem) \
+    WTFLogChannel LOG_CHANNEL(name) = { initialState, #name, level, subsystem };
 #endif
 #endif
 
+// This file is included from C (not C++) files, so we can't say things like WTFLogChannelState::Off.
+static const WTFLogChannelState logChannelStateOff = (WTFLogChannelState)0;
+static const WTFLogChannelState logChannelStateOn = (WTFLogChannelState)1;
+static const WTFLogLevel logLevelError = (WTFLogLevel)1;
+#define DEFINE_LOG_CHANNEL(name, subsystem) DEFINE_LOG_CHANNEL_WITH_DETAILS(name, logChannelStateOff, logLevelError, subsystem);
+
 WTF_EXPORT_PRIVATE void WTFReportNotImplementedYet(const char* file, int line, const char* function);
 WTF_EXPORT_PRIVATE void WTFReportAssertionFailure(const char* file, int line, const char* function, const char* assertion);
 WTF_EXPORT_PRIVATE void WTFReportAssertionFailureWithMessage(const char* file, int line, const char* function, const char* assertion, const char* format, ...) WTF_ATTRIBUTE_PRINTF(5, 6);
@@ -218,7 +229,12 @@ WTF_EXPORT_PRIVATE void WTFSetLogChannelLevel(WTFLogChannel*, WTFLogLevel);
 WTF_EXPORT_PRIVATE bool WTFWillLogWithLevel(WTFLogChannel*, WTFLogLevel);
 
 WTF_EXPORT_PRIVATE void WTFGetBacktrace(void** stack, int* size);
+WTF_EXPORT_PRIVATE void WTFReportBacktraceWithPrefix(const char*);
 WTF_EXPORT_PRIVATE void WTFReportBacktrace(void);
+#ifdef __cplusplus
+WTF_EXPORT_PRIVATE void WTFReportBacktraceWithPrefixAndPrintStream(WTF::PrintStream&, const char*);
+void WTFPrintBacktraceWithPrefixAndPrintStream(WTF::PrintStream&, void** stack, int size, const char* prefix);
+#endif
 WTF_EXPORT_PRIVATE void WTFPrintBacktrace(void** stack, int size);
 #if !RELEASE_LOG_DISABLED
 WTF_EXPORT_PRIVATE void WTFReleaseLogStackTrace(WTFLogChannel*);
@@ -235,7 +251,7 @@ WTF_EXPORT_PRIVATE bool WTFIsDebuggerAttached(void);
 #elif CPU(ARM_THUMB2)
 #define WTFBreakpointTrap()  asm volatile ("bkpt #0")
 #elif CPU(ARM64)
-#define WTFBreakpointTrap()  asm volatile ("brk #0")
+#define WTFBreakpointTrap()  asm volatile ("brk #0xc471")
 #else
 #define WTFBreakpointTrap() WTFCrash() // Not implemented.
 #endif
@@ -263,14 +279,14 @@ WTF_EXPORT_PRIVATE bool WTFIsDebuggerAttached(void);
 #elif !ENABLE(DEVELOPER_MODE) && !OS(DARWIN)
 #ifdef __cplusplus
 #define CRASH() std::abort()
-#define CRASH_UNDER_CONSTEXPR_CONTEXT() std::abort()
+#define CRASH_UNDER_CONSTEXPR_CONTEXT() WTFBreakpointTrapUnderConstexprContext()
 #else
 #define CRASH() abort()
-#define CRASH_UNDER_CONSTEXPR_CONTEXT() abort()
+#define CRASH_UNDER_CONSTEXPR_CONTEXT() WTFBreakpointTrapUnderConstexprContext()
 #endif // __cplusplus
 #else
 #define CRASH() WTFCrash()
-#define CRASH_UNDER_CONSTEXPR_CONTEXT() WTFCrash()
+#define CRASH_UNDER_CONSTEXPR_CONTEXT() WTFBreakpointTrapUnderConstexprContext()
 #endif
 
 #endif // !defined(CRASH)
@@ -321,6 +337,7 @@ WTF_EXPORT_PRIVATE NO_RETURN_DUE_TO_CRASH void WTFCrashWithSecurityImplication(v
 #define ASSERT_UNDER_CONSTEXPR_CONTEXT(assertion) ((void)0)
 #define ASSERT_AT(assertion, file, line, function) ((void)0)
 #define ASSERT_NOT_REACHED(...) ((void)0)
+#define ASSERT_NOT_REACHED_UNDER_CONSTEXPR_CONTEXT(...) ((void)0)
 #define ASSERT_NOT_IMPLEMENTED_YET() ((void)0)
 #define ASSERT_IMPLIES(condition, assertion) ((void)0)
 #define NO_RETURN_DUE_TO_ASSERT
@@ -368,6 +385,10 @@ WTF_EXPORT_PRIVATE NO_RETURN_DUE_TO_CRASH void WTFCrashWithSecurityImplication(v
     CRASH_WITH_INFO(__VA_ARGS__); \
 } while (0)
 
+#define ASSERT_NOT_REACHED_UNDER_CONSTEXPR_CONTEXT(...) do { \
+    CRASH_UNDER_CONSTEXPR_CONTEXT(); \
+} while (0)
+
 #define ASSERT_NOT_IMPLEMENTED_YET() do { \
     WTFReportNotImplementedYet(__FILE__, __LINE__, WTF_PRETTY_FUNCTION); \
     CRASH(); \
@@ -484,7 +505,10 @@ constexpr bool assertionFailureDueToUnreachableCode = false;
 #if LOG_DISABLED
 #define LOG(channel, ...) ((void)0)
 #else
-#define LOG(channel, ...) WTFLog(&LOG_CHANNEL(channel), __VA_ARGS__)
+#define LOG(channel, ...) do { \
+        if (LOG_CHANNEL(channel).state != logChannelStateOff) \
+            WTFLog(&LOG_CHANNEL(channel), __VA_ARGS__); \
+    } while (0)
 #endif
 
 /* LOG_VERBOSE */
@@ -492,7 +516,10 @@ constexpr bool assertionFailureDueToUnreachableCode = false;
 #if LOG_DISABLED
 #define LOG_VERBOSE(channel, ...) ((void)0)
 #else
-#define LOG_VERBOSE(channel, ...) WTFLogVerbose(__FILE__, __LINE__, WTF_PRETTY_FUNCTION, &LOG_CHANNEL(channel), __VA_ARGS__)
+#define LOG_VERBOSE(channel, ...) do { \
+        if (LOG_CHANNEL(channel).state != logChannelStateOff) \
+            WTFLogVerbose(__FILE__, __LINE__, WTF_PRETTY_FUNCTION, &LOG_CHANNEL(channel), __VA_ARGS__); \
+    } while (0)
 #endif
 
 /* LOG_WITH_LEVEL */
@@ -500,13 +527,32 @@ constexpr bool assertionFailureDueToUnreachableCode = false;
 #if LOG_DISABLED
 #define LOG_WITH_LEVEL(channel, level, ...) ((void)0)
 #else
-#define LOG_WITH_LEVEL(channel, level, ...) WTFLogWithLevel(&LOG_CHANNEL(channel), level, __VA_ARGS__)
+#define LOG_WITH_LEVEL(channel, level, ...) do { \
+        if  (LOG_CHANNEL(channel).state != logChannelStateOff && channel->level >= (level)) \
+            WTFLogWithLevel(&LOG_CHANNEL(channel), level, __VA_ARGS__); \
+    } while (0)
+#endif
+
+/* LOG_WITH_STREAM */
+
+#if LOG_DISABLED
+#define LOG_WITH_STREAM(channel, commands) ((void)0)
+#else
+#define LOG_WITH_STREAM(channel, commands) do { \
+        if (LOG_CHANNEL(channel).state != logChannelStateOff) { \
+            WTF::TextStream stream(WTF::TextStream::LineMode::SingleLine); \
+            commands; \
+            WTFLog(&LOG_CHANNEL(channel), "%s", stream.release().utf8().data()); \
+        } \
+    } while (0)
 #endif
 
 /* RELEASE_LOG */
 
 #if RELEASE_LOG_DISABLED
+
 #define PUBLIC_LOG_STRING "s"
+#define PRIVATE_LOG_STRING "s"
 #define RELEASE_LOG(channel, ...) ((void)0)
 #define RELEASE_LOG_ERROR(channel, ...) LOG_ERROR(__VA_ARGS__)
 #define RELEASE_LOG_FAULT(channel, ...) LOG_ERROR(__VA_ARGS__)
@@ -520,10 +566,11 @@ constexpr bool assertionFailureDueToUnreachableCode = false;
 #define RELEASE_LOG_WITH_LEVEL_IF(isAllowed, channel, level, ...) do { if (isAllowed) RELEASE_LOG_WITH_LEVEL(channel, level, __VA_ARGS__); } while (0)
 
 #define RELEASE_LOG_STACKTRACE(channel) ((void)0)
-#endif
 
-#if USE(OS_LOG) && !RELEASE_LOG_DISABLED
+#elif USE(OS_LOG)
+
 #define PUBLIC_LOG_STRING "{public}s"
+#define PRIVATE_LOG_STRING "{private}s"
 #define RELEASE_LOG(channel, ...) os_log(LOG_CHANNEL(channel).osLogChannel, __VA_ARGS__)
 #define RELEASE_LOG_ERROR(channel, ...) os_log_error(LOG_CHANNEL(channel).osLogChannel, __VA_ARGS__)
 #define RELEASE_LOG_FAULT(channel, ...) os_log_fault(LOG_CHANNEL(channel).osLogChannel, __VA_ARGS__)
@@ -537,10 +584,11 @@ constexpr bool assertionFailureDueToUnreachableCode = false;
     if ((isAllowed) && LOG_CHANNEL(channel).level >= (logLevel)) \
         os_log(LOG_CHANNEL(channel).osLogChannel, __VA_ARGS__); \
 } while (0)
-#endif
 
-#if USE(JOURNALD) && !RELEASE_LOG_DISABLED
+#elif ENABLE(JOURNALD_LOG)
+
 #define PUBLIC_LOG_STRING "s"
+#define PRIVATE_LOG_STRING "s"
 #define SD_JOURNAL_SEND(channel, priority, file, line, function, ...) do { \
     if (LOG_CHANNEL(channel).state != WTFLogChannelState::Off) \
         sd_journal_send_with_location("CODE_FILE=" file, "CODE_LINE=" line, function, "WEBKIT_SUBSYSTEM=%s", LOG_CHANNEL(channel).subsystem, "WEBKIT_CHANNEL=%s", LOG_CHANNEL(channel).name, "PRIORITY=%i", priority, "MESSAGE=" __VA_ARGS__, nullptr); \
@@ -562,6 +610,32 @@ constexpr bool assertionFailureDueToUnreachableCode = false;
     if ((isAllowed) && LOG_CHANNEL(channel).level >= (logLevel)) \
         SD_JOURNAL_SEND(channel, LOG_INFO, __FILE__, _STRINGIFY(__LINE__), __func__, __VA_ARGS__); \
 } while (0)
+
+#else
+
+#define PUBLIC_LOG_STRING "s"
+#define PRIVATE_LOG_STRING "s"
+#define LOGF(channel, priority, fmt, ...) do { \
+    auto& logChannel = LOG_CHANNEL(channel); \
+    if (logChannel.state != WTFLogChannelState::Off) \
+        fprintf(stderr, "[%s:%s:%i] " fmt "\n", logChannel.subsystem, logChannel.name, priority, ##__VA_ARGS__); \
+} while (0)
+
+#define RELEASE_LOG(channel, ...) LOGF(channel, 4, __VA_ARGS__)
+#define RELEASE_LOG_ERROR(channel, ...) LOGF(channel, 1, __VA_ARGS__)
+#define RELEASE_LOG_FAULT(channel, ...) LOGF(channel, 2, __VA_ARGS__)
+#define RELEASE_LOG_INFO(channel, ...) LOGF(channel, 3, __VA_ARGS__)
+
+#define RELEASE_LOG_WITH_LEVEL(channel, logLevel, ...) do { \
+    if (LOG_CHANNEL(channel).level >= (logLevel)) \
+        LOGF(channel, logLevel, __VA_ARGS__); \
+} while (0)
+
+#define RELEASE_LOG_WITH_LEVEL_IF(isAllowed, channel, logLevel, ...) do { \
+    if ((isAllowed) && LOG_CHANNEL(channel).level >= (logLevel)) \
+        LOGF(channel, logLevel, __VA_ARGS__); \
+} while (0)
+
 #endif
 
 #if !RELEASE_LOG_DISABLED
@@ -573,6 +647,13 @@ constexpr bool assertionFailureDueToUnreachableCode = false;
 #define RELEASE_LOG_STACKTRACE(channel) WTFReleaseLogStackTrace(&LOG_CHANNEL(channel))
 #endif
 
+/* ALWAYS_LOG */
+
+#define ALWAYS_LOG_WITH_STREAM(commands) do { \
+        WTF::TextStream stream(WTF::TextStream::LineMode::SingleLine); \
+        commands; \
+        WTFLogAlways("%s", stream.release().utf8().data()); \
+    } while (0)
 
 /* RELEASE_ASSERT */
 
@@ -585,6 +666,7 @@ constexpr bool assertionFailureDueToUnreachableCode = false;
 #define RELEASE_ASSERT_WITH_MESSAGE(assertion, ...) RELEASE_ASSERT(assertion)
 #define RELEASE_ASSERT_WITH_SECURITY_IMPLICATION(assertion) RELEASE_ASSERT(assertion)
 #define RELEASE_ASSERT_NOT_REACHED(...) CRASH_WITH_INFO(__VA_ARGS__)
+#define RELEASE_ASSERT_NOT_REACHED_UNDER_CONSTEXPR_CONTEXT() CRASH_UNDER_CONSTEXPR_CONTEXT();
 #define RELEASE_ASSERT_UNDER_CONSTEXPR_CONTEXT(assertion) do { \
     if (UNLIKELY(!(assertion))) { \
         CRASH_UNDER_CONSTEXPR_CONTEXT(); \
@@ -597,6 +679,7 @@ constexpr bool assertionFailureDueToUnreachableCode = false;
 #define RELEASE_ASSERT_WITH_MESSAGE(assertion, ...) ASSERT_WITH_MESSAGE(assertion, __VA_ARGS__)
 #define RELEASE_ASSERT_WITH_SECURITY_IMPLICATION(assertion) ASSERT_WITH_SECURITY_IMPLICATION(assertion)
 #define RELEASE_ASSERT_NOT_REACHED() ASSERT_NOT_REACHED()
+#define RELEASE_ASSERT_NOT_REACHED_UNDER_CONSTEXPR_CONTEXT() ASSERT_NOT_REACHED_UNDER_CONSTEXPR_CONTEXT()
 #define RELEASE_ASSERT_UNDER_CONSTEXPR_CONTEXT(assertion) ASSERT_UNDER_CONSTEXPR_CONTEXT(assertion)
 
 #endif /* ASSERT_ENABLED */
@@ -680,7 +763,12 @@ void isIntegralOrPointerType(T, Types... types)
     static_assert(std::is_integral<T>::value || std::is_enum<T>::value || std::is_pointer<T>::value, "All types need to be bitwise_cast-able to integral type for logging");
     isIntegralOrPointerType(types...);
 }
-}
+
+#if PLATFORM(COCOA)
+WTF_EXPORT_PRIVATE void disableForwardingVPrintfStdErrToOSLog();
+#endif
+
+} // namespace WTF
 
 inline void compilerFenceForCrash()
 {
@@ -692,33 +780,27 @@ inline void compilerFenceForCrash()
 }
 
 #ifndef CRASH_WITH_INFO
+
+#define PP_THIRD_ARG(a,b,c,...) c
+#define VA_OPT_SUPPORTED_I(...) PP_THIRD_ARG(__VA_OPT__(,),true,false,)
+#define VA_OPT_SUPPORTED VA_OPT_SUPPORTED_I(?)
+
 // This is useful if you are going to stuff data into registers before crashing, like the
 // crashWithInfo functions below.
-#if COMPILER(CLANG) || COMPILER(MSVC)
+#if COMPILER(MSVC) || !VA_OPT_SUPPORTED
+// FIXME: Re-check whether MSVC 2020 supports __VA_OPT__ and remove the special
+//        casing once older versions of the compiler are no longer supported.
 #define CRASH_WITH_INFO(...) do { \
         WTF::isIntegralOrPointerType(__VA_ARGS__); \
         compilerFenceForCrash(); \
         WTFCrashWithInfo(__LINE__, __FILE__, WTF_PRETTY_FUNCTION, __COUNTER__, ##__VA_ARGS__); \
     } while (false)
 #else
-// GCC does not allow ##__VA_ARGS__ unless GNU extensions are enabled (--std=gnu++NN instead of
-// --std=c++NN) and I think we don't want that, so we'll have a fallback path for GCC. Obviously
-// this will not actually succeed at getting the desired info into registers before crashing, but
-// it's just a fallback anyway.
-//
-// FIXME: When we enable C++20, we should replace ##__VA_ARGS__ with format __VA_OPT__(,) __VA_ARGS__
-// so that we can remove this fallback.
-inline NO_RETURN_DUE_TO_CRASH void CRASH_WITH_INFO(...)
-{
-    CRASH();
-}
-
-// We must define this here because CRASH_WITH_INFO() is not defined as a macro.
-// FIXME: Remove this when upgrading to C++20.
-inline NO_RETURN_DUE_TO_CRASH void CRASH_WITH_SECURITY_IMPLICATION_AND_INFO(...)
-{
-    CRASH();
-}
+#define CRASH_WITH_INFO(...) do { \
+        WTF::isIntegralOrPointerType(__VA_ARGS__); \
+        compilerFenceForCrash(); \
+        WTFCrashWithInfo(__LINE__, __FILE__, WTF_PRETTY_FUNCTION, __COUNTER__ __VA_OPT__(,) __VA_ARGS__); \
+    } while (false)
 #endif
 #endif // CRASH_WITH_INFO
 
@@ -740,7 +822,7 @@ inline NO_RETURN_DUE_TO_CRASH void CRASH_WITH_SECURITY_IMPLICATION_AND_INFO(...)
 // This would be a macro except that its use of #pragma works best around
 // a function. Hence it uses macro naming convention.
 IGNORE_WARNINGS_BEGIN("missing-noreturn")
-static inline void UNREACHABLE_FOR_PLATFORM()
+static inline void UNREACHABLE_FOR_PLATFORM(void)
 {
     // This *MUST* be a release assert. We use it in places where it's better to crash than to keep
     // going.
