@@ -29,12 +29,13 @@
 #if ENABLE(APPLE_PAY)
 
 #include "ApplePayCancelEvent.h"
+#include "ApplePayCouponCodeChangedEvent.h"
+#include "ApplePayCouponCodeUpdate.h"
 #include "ApplePayErrorCode.h"
 #include "ApplePayErrorContactField.h"
 #include "ApplePayLineItem.h"
 #include "ApplePayPaymentAuthorizationResult.h"
 #include "ApplePayPaymentAuthorizedEvent.h"
-#include "ApplePayPaymentMethodModeUpdate.h"
 #include "ApplePayPaymentMethodSelectedEvent.h"
 #include "ApplePayPaymentMethodUpdate.h"
 #include "ApplePayPaymentRequest.h"
@@ -54,11 +55,11 @@
 #include "LinkIconType.h"
 #include "Page.h"
 #include "PageConsoleClient.h"
-#include "PaymentAuthorizationStatus.h"
 #include "PaymentContact.h"
 #include "PaymentCoordinator.h"
 #include "PaymentMerchantSession.h"
 #include "PaymentMethod.h"
+#include "PaymentRequestUtilities.h"
 #include "PaymentRequestValidator.h"
 #include "SecurityOrigin.h"
 #include "Settings.h"
@@ -66,84 +67,13 @@
 #include <wtf/IsoMallocInlines.h>
 #include <wtf/RunLoop.h>
 
-#if USE(APPLE_INTERNAL_SDK)
-#include <WebKitAdditions/ApplePaySessionAdditions.cpp>
-#endif
-
 namespace WebCore {
 
 WTF_MAKE_ISO_ALLOCATED_IMPL(ApplePaySession);
 
-// The amount follows the regular expression -?[0-9]+(\.[0-9][0-9])?.
-static bool validateAmount(const String& amountString)
-{
-    enum class State {
-        Start,
-        Sign,
-        Digit,
-        Dot,
-        DotDigit,
-        End,
-    };
-
-    State state = State::Start;
-
-    for (unsigned i = 0; i < amountString.length(); ++i) {
-        UChar c = amountString[i];
-
-        switch (state) {
-        case State::Start:
-            if (c == '-') {
-                state = State::Sign;
-                break;
-            }
-
-            if (!isASCIIDigit(c))
-                return false;
-            state = State::Digit;
-            break;
-
-        case State::Sign:
-            if (!isASCIIDigit(c))
-                return false;
-            state = State::Digit;
-            break;
-
-        case State::Digit:
-            if (c == '.') {
-                state = State::Dot;
-                break;
-            }
-
-            if (!isASCIIDigit(c))
-                return false;
-            break;
-
-        case State::Dot:
-            if (!isASCIIDigit(c))
-                return false;
-
-            state = State::DotDigit;
-            break;
-
-        case State::DotDigit:
-            if (!isASCIIDigit(c))
-                return false;
-
-            state = State::End;
-            break;
-
-        case State::End:
-            return false;
-        }
-    }
-
-    return state == State::Digit || state == State::DotDigit || state == State::End;
-}
-
 static ExceptionOr<ApplePayLineItem> convertAndValidateTotal(ApplePayLineItem&& lineItem)
 {
-    if (!validateAmount(lineItem.amount))
+    if (!isValidDecimalMonetaryValue(lineItem.amount))
         return Exception { TypeError, makeString("\"" + lineItem.amount, "\" is not a valid amount.") };
 
     auto validatedTotal = PaymentRequestValidator::validateTotal(lineItem);
@@ -159,14 +89,14 @@ static ExceptionOr<ApplePayLineItem> convertAndValidate(ApplePayLineItem&& lineI
         // It is OK for pending types to not have an amount.
         lineItem.amount = nullString();
     } else {
-        if (!validateAmount(lineItem.amount))
+        if (!isValidDecimalMonetaryValue(lineItem.amount))
             return Exception { TypeError, makeString("\"" + lineItem.amount, "\" is not a valid amount.") };
     }
 
     return WTFMove(lineItem);
 }
 
-static ExceptionOr<Vector<ApplePayLineItem>> convertAndValidate(Optional<Vector<ApplePayLineItem>>&& lineItems)
+static ExceptionOr<Vector<ApplePayLineItem>> convertAndValidate(std::optional<Vector<ApplePayLineItem>>&& lineItems)
 {
     Vector<ApplePayLineItem> result;
     if (!lineItems)
@@ -186,7 +116,7 @@ static ExceptionOr<Vector<ApplePayLineItem>> convertAndValidate(Optional<Vector<
 
 static ExceptionOr<ApplePayShippingMethod> convertAndValidate(ApplePayShippingMethod&& shippingMethod)
 {
-    if (!validateAmount(shippingMethod.amount))
+    if (!isValidDecimalMonetaryValue(shippingMethod.amount))
         return Exception { TypeError, makeString("\"" + shippingMethod.amount, "\" is not a valid amount.") };
 
     return WTFMove(shippingMethod);
@@ -207,9 +137,89 @@ static ExceptionOr<Vector<ApplePayShippingMethod>> convertAndValidate(Vector<App
     return WTFMove(result);
 }
 
-#if defined(ApplePaySessionAdditions_convertAndValidate)
-ApplePaySessionAdditions_convertAndValidate
-#endif
+#if ENABLE(APPLE_PAY_RECURRING_PAYMENTS)
+
+static ExceptionOr<ApplePayRecurringPaymentRequest> convertAndValidate(ApplePayRecurringPaymentRequest&& recurringPaymentRequest)
+{
+    auto& regularBilling = recurringPaymentRequest.regularBilling;
+    if (regularBilling.paymentTiming != ApplePayPaymentTiming::Recurring)
+        return Exception(TypeError, "'regularBilling' must be a 'recurring' line item."_s);
+    if (!regularBilling.label)
+        return Exception(TypeError, "Missing label for 'regularBilling'."_s);
+    if (!isValidDecimalMonetaryValue(regularBilling.amount) && regularBilling.type != ApplePayLineItem::Type::Pending)
+        return Exception(TypeError, makeString('"', regularBilling.amount, "\" is not a valid amount."));
+
+    if (auto& trialBilling = recurringPaymentRequest.trialBilling) {
+        if (trialBilling->paymentTiming != ApplePayPaymentTiming::Recurring)
+            return Exception(TypeError, "'trialBilling' must be a 'recurring' line item."_s);
+        if (!trialBilling->label)
+            return Exception(TypeError, "Missing label for 'trialBilling'."_s);
+        if (!isValidDecimalMonetaryValue(trialBilling->amount) && trialBilling->type != ApplePayLineItem::Type::Pending)
+            return Exception(TypeError, makeString('"', trialBilling->amount, "\" is not a valid amount."));
+    }
+
+    if (auto& managementURL = recurringPaymentRequest.managementURL; !URL { managementURL }.isValid())
+        return Exception(TypeError, makeString('"', managementURL, "\" is not a valid URL."));
+
+    if (auto& tokenNotificationURL = recurringPaymentRequest.tokenNotificationURL; !tokenNotificationURL.isNull() && !URL { tokenNotificationURL }.isValid())
+        return Exception(TypeError, makeString('"', tokenNotificationURL, "\" is not a valid URL."));
+
+    return WTFMove(recurringPaymentRequest);
+}
+
+#endif // ENABLE(APPLE_PAY_RECURRING_PAYMENTS)
+
+#if ENABLE(APPLE_PAY_AUTOMATIC_RELOAD_PAYMENTS)
+
+static ExceptionOr<ApplePayAutomaticReloadPaymentRequest> convertAndValidate(ApplePayAutomaticReloadPaymentRequest&& automaticReloadPaymentRequest)
+{
+    auto& automaticReloadBilling = automaticReloadPaymentRequest.automaticReloadBilling;
+    if (automaticReloadBilling.paymentTiming != ApplePayPaymentTiming::AutomaticReload)
+        return Exception(TypeError, "'automaticReloadBilling' must be an 'automaticReload' line item."_s);
+    if (!automaticReloadBilling.label)
+        return Exception(TypeError, "Missing label for 'automaticReloadBilling'."_s);
+    if (!isValidDecimalMonetaryValue(automaticReloadBilling.amount) && automaticReloadBilling.type != ApplePayLineItem::Type::Pending)
+        return Exception(TypeError, makeString('"', automaticReloadBilling.amount, "\" is not a valid amount."));
+    if (!isValidDecimalMonetaryValue(automaticReloadBilling.automaticReloadPaymentThresholdAmount))
+        return Exception(TypeError, makeString('"', automaticReloadBilling.automaticReloadPaymentThresholdAmount, "\" is not a valid automaticReloadPaymentThresholdAmount."));
+
+    if (auto& managementURL = automaticReloadPaymentRequest.managementURL; !URL { managementURL }.isValid())
+        return Exception(TypeError, makeString('"', managementURL, "\" is not a valid URL."));
+
+    if (auto& tokenNotificationURL = automaticReloadPaymentRequest.tokenNotificationURL; !tokenNotificationURL.isNull() && !URL { tokenNotificationURL }.isValid())
+        return Exception(TypeError, makeString('"', tokenNotificationURL, "\" is not a valid URL."));
+
+    return WTFMove(automaticReloadPaymentRequest);
+}
+
+#endif // ENABLE(APPLE_PAY_AUTOMATIC_RELOAD_PAYMENTS)
+
+#if ENABLE(APPLE_PAY_MULTI_MERCHANT_PAYMENTS)
+
+static ExceptionOr<ApplePayPaymentTokenContext> convertAndValidate(ApplePayPaymentTokenContext&& tokenContext)
+{
+    if (!isValidDecimalMonetaryValue(tokenContext.amount))
+        return Exception { TypeError, makeString("\"" + tokenContext.amount, "\" is not a valid amount.") };
+
+    return WTFMove(tokenContext);
+}
+
+static ExceptionOr<Vector<ApplePayPaymentTokenContext>> convertAndValidate(Vector<ApplePayPaymentTokenContext>&& tokenContexts)
+{
+    Vector<ApplePayPaymentTokenContext> result;
+    result.reserveInitialCapacity(tokenContexts.size());
+
+    for (auto& tokenContext : tokenContexts) {
+        auto convertedTokenContext = convertAndValidate(WTFMove(tokenContext));
+        if (convertedTokenContext.hasException())
+            return convertedTokenContext.releaseException();
+        result.uncheckedAppend(convertedTokenContext.releaseReturnValue());
+    }
+
+    return WTFMove(result);
+}
+
+#endif // ENABLE(APPLE_PAY_MULTI_MERCHANT_PAYMENTS)
 
 static ExceptionOr<ApplePaySessionPaymentRequest> convertAndValidate(Document& document, unsigned version, ApplePayPaymentRequest&& paymentRequest, const PaymentCoordinator& paymentCoordinator)
 {
@@ -240,65 +250,100 @@ static ExceptionOr<ApplePaySessionPaymentRequest> convertAndValidate(Document& d
         result.setShippingMethods(shippingMethods.releaseReturnValue());
     }
 
-#if defined(ApplePaySessionAdditions_convertAndValidate_request)
-    ApplePaySessionAdditions_convertAndValidate_request
+#if ENABLE(APPLE_PAY_RECURRING_PAYMENTS)
+    if (paymentRequest.recurringPaymentRequest) {
+        auto recurringPaymentRequest = convertAndValidate(WTFMove(*paymentRequest.recurringPaymentRequest));
+        if (recurringPaymentRequest.hasException())
+            return recurringPaymentRequest.releaseException();
+        result.setRecurringPaymentRequest(recurringPaymentRequest.releaseReturnValue());
+    }
+#endif
+
+#if ENABLE(APPLE_PAY_AUTOMATIC_RELOAD_PAYMENTS)
+    if (paymentRequest.automaticReloadPaymentRequest) {
+        auto automaticReloadPaymentRequest = convertAndValidate(WTFMove(*paymentRequest.automaticReloadPaymentRequest));
+        if (automaticReloadPaymentRequest.hasException())
+            return automaticReloadPaymentRequest.releaseException();
+        result.setAutomaticReloadPaymentRequest(automaticReloadPaymentRequest.releaseReturnValue());
+    }
+#endif
+
+#if ENABLE(APPLE_PAY_MULTI_MERCHANT_PAYMENTS)
+    if (paymentRequest.multiTokenContexts) {
+        auto multiTokenContexts = convertAndValidate(WTFMove(*paymentRequest.multiTokenContexts));
+        if (multiTokenContexts.hasException())
+            return multiTokenContexts.releaseException();
+        result.setMultiTokenContexts(multiTokenContexts.releaseReturnValue());
+    }
 #endif
 
     // FIXME: Merge this validation into the validation we are doing above.
-    auto validatedPaymentRequest = PaymentRequestValidator::validate(result);
+    constexpr OptionSet fieldsToValidate = {
+        PaymentRequestValidator::Field::MerchantCapabilities,
+        PaymentRequestValidator::Field::SupportedNetworks,
+        PaymentRequestValidator::Field::CountryCode,
+        PaymentRequestValidator::Field::CurrencyCode,
+        PaymentRequestValidator::Field::Total,
+        PaymentRequestValidator::Field::ShippingMethods,
+    };
+    auto validatedPaymentRequest = PaymentRequestValidator::validate(result, fieldsToValidate);
     if (validatedPaymentRequest.hasException())
         return validatedPaymentRequest.releaseException();
 
     return WTFMove(result);
 }
 
-static ExceptionOr<PaymentAuthorizationResult> convertAndValidate(ApplePayPaymentAuthorizationResult&& result)
+#if HAVE(APPLE_PAY_PAYMENT_ORDER_DETAILS)
+
+static ExceptionOr<ApplePayPaymentOrderDetails> convertAndValidate(ApplePayPaymentOrderDetails&& orderDetails)
 {
-    PaymentAuthorizationResult convertedResult;
+    if (auto& webServiceURL = orderDetails.webServiceURL; !URL { webServiceURL }.isValid())
+        return Exception(TypeError, makeString('"', webServiceURL, "\" is not a valid URL."));
 
+    return WTFMove(orderDetails);
+}
+
+#endif // HAVE(APPLE_PAY_PAYMENT_ORDER_DETAILS)
+
+static ExceptionOr<ApplePayPaymentAuthorizationResult> convertAndValidate(ApplePayPaymentAuthorizationResult&& result)
+{
     switch (result.status) {
-    case ApplePaySession::STATUS_SUCCESS:
-        convertedResult.status = PaymentAuthorizationStatus::Success;
+    case ApplePayPaymentAuthorizationResult::Success:
+    case ApplePayPaymentAuthorizationResult::Failure:
+    case ApplePayPaymentAuthorizationResult::PINRequired:
+    case ApplePayPaymentAuthorizationResult::PINIncorrect:
+    case ApplePayPaymentAuthorizationResult::PINLockout:
         break;
 
-    case ApplePaySession::STATUS_FAILURE:
-        convertedResult.status = PaymentAuthorizationStatus::Failure;
+    case ApplePayPaymentAuthorizationResult::InvalidBillingPostalAddress:
+        result.status = ApplePayPaymentAuthorizationResult::Failure;
+        result.errors.insert(0, ApplePayError::create(ApplePayErrorCode::BillingContactInvalid, std::nullopt, nullString()));
         break;
 
-    case ApplePaySession::STATUS_INVALID_BILLING_POSTAL_ADDRESS:
-        convertedResult.status = PaymentAuthorizationStatus::Failure;
-        convertedResult.errors.append(ApplePayError::create(ApplePayErrorCode::BillingContactInvalid, WTF::nullopt, nullString()));
+    case ApplePayPaymentAuthorizationResult::InvalidShippingPostalAddress:
+        result.status = ApplePayPaymentAuthorizationResult::Failure;
+        result.errors.insert(0, ApplePayError::create(ApplePayErrorCode::ShippingContactInvalid, ApplePayErrorContactField::PostalAddress, nullString()));
         break;
 
-    case ApplePaySession::STATUS_INVALID_SHIPPING_POSTAL_ADDRESS:
-        convertedResult.status = PaymentAuthorizationStatus::Failure;
-        convertedResult.errors.append(ApplePayError::create(ApplePayErrorCode::ShippingContactInvalid, ApplePayErrorContactField::PostalAddress, nullString()));
-        break;
-
-    case ApplePaySession::STATUS_INVALID_SHIPPING_CONTACT:
-        convertedResult.status = PaymentAuthorizationStatus::Failure;
-        convertedResult.errors.append(ApplePayError::create(ApplePayErrorCode::ShippingContactInvalid, WTF::nullopt, nullString()));
-        break;
-
-    case ApplePaySession::STATUS_PIN_REQUIRED:
-        convertedResult.status = PaymentAuthorizationStatus::PINRequired;
-        break;
-
-    case ApplePaySession::STATUS_PIN_INCORRECT:
-        convertedResult.status = PaymentAuthorizationStatus::PINIncorrect;
-        break;
-
-    case ApplePaySession::STATUS_PIN_LOCKOUT:
-        convertedResult.status = PaymentAuthorizationStatus::PINLockout;
+    case ApplePayPaymentAuthorizationResult::InvalidShippingContact:
+        result.status = ApplePayPaymentAuthorizationResult::Failure;
+        result.errors.insert(0, ApplePayError::create(ApplePayErrorCode::ShippingContactInvalid, std::nullopt, nullString()));
         break;
 
     default:
         return Exception { InvalidAccessError };
     }
 
-    convertedResult.errors.appendVector(WTFMove(result.errors));
+#if HAVE(APPLE_PAY_PAYMENT_ORDER_DETAILS)
+    if (auto orderDetails = WTFMove(result.orderDetails)) {
+        auto convertedOrderDetails = convertAndValidate(WTFMove(*orderDetails));
+        if (convertedOrderDetails.hasException())
+            return convertedOrderDetails.releaseException();
+        result.orderDetails = convertedOrderDetails.releaseReturnValue();
+    }
+#endif
 
-    return WTFMove(convertedResult);
+    return WTFMove(result);
 }
 
 static ExceptionOr<ApplePayPaymentMethodUpdate> convertAndValidate(ApplePayPaymentMethodUpdate&& update)
@@ -351,9 +396,9 @@ static ExceptionOr<ApplePayShippingMethodUpdate> convertAndValidate(ApplePayShip
     return WTFMove(update);
 }
 
-#if ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#if ENABLE(APPLE_PAY_COUPON_CODE)
 
-static ExceptionOr<ApplePayPaymentMethodModeUpdate> convertAndValidate(ApplePayPaymentMethodModeUpdate&& update)
+static ExceptionOr<ApplePayCouponCodeUpdate> convertAndValidate(ApplePayCouponCodeUpdate&& update)
 {
     auto convertedNewShippingMethods = convertAndValidate(WTFMove(update.newShippingMethods));
     if (convertedNewShippingMethods.hasException())
@@ -373,7 +418,7 @@ static ExceptionOr<ApplePayPaymentMethodModeUpdate> convertAndValidate(ApplePayP
     return WTFMove(update);
 }
 
-#endif // ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#endif // ENABLE(APPLE_PAY_COUPON_CODE)
 
 ExceptionOr<Ref<ApplePaySession>> ApplePaySession::create(Document& document, unsigned version, ApplePayPaymentRequest&& paymentRequest)
 {
@@ -382,16 +427,18 @@ ExceptionOr<Ref<ApplePaySession>> ApplePaySession::create(Document& document, un
         return canCall.releaseException();
 
     if (!UserGestureIndicator::processingUserGesture())
-        return Exception { InvalidAccessError, "Must create a new ApplePaySession from a user gesture handler." };
+        return Exception { InvalidAccessError, "Must create a new ApplePaySession from a user gesture handler."_s };
 
     if (!document.page())
-        return Exception { InvalidAccessError, "Frame is detached" };
+        return Exception { InvalidAccessError, "Frame is detached"_s };
 
     auto convertedPaymentRequest = convertAndValidate(document, version, WTFMove(paymentRequest), document.page()->paymentCoordinator());
     if (convertedPaymentRequest.hasException())
         return convertedPaymentRequest.releaseException();
 
-    return adoptRef(*new ApplePaySession(document, version, convertedPaymentRequest.releaseReturnValue()));
+    auto session = adoptRef(*new ApplePaySession(document, version, convertedPaymentRequest.releaseReturnValue()));
+    session->suspendIfNeeded();
+    return session;
 }
 
 ApplePaySession::ApplePaySession(Document& document, unsigned version, ApplePaySessionPaymentRequest&& paymentRequest)
@@ -400,7 +447,6 @@ ApplePaySession::ApplePaySession(Document& document, unsigned version, ApplePayS
     , m_version { version }
 {
     ASSERT(document.page()->paymentCoordinator().supportsVersion(document, version));
-    suspendIfNeeded();
 }
 
 ApplePaySession::~ApplePaySession() = default;
@@ -440,7 +486,7 @@ ExceptionOr<bool> ApplePaySession::canMakePayments(Document& document)
     if (!page)
         return Exception { InvalidAccessError };
 
-    return page->paymentCoordinator().canMakePayments(document);
+    return page->paymentCoordinator().canMakePayments();
 }
 
 ExceptionOr<void> ApplePaySession::canMakePaymentsWithActiveCard(Document& document, const String& merchantIdentifier, Ref<DeferredPromise>&& passedPromise)
@@ -456,7 +502,7 @@ ExceptionOr<void> ApplePaySession::canMakePaymentsWithActiveCard(Document& docum
             return Exception { InvalidAccessError };
 
         auto& paymentCoordinator = page->paymentCoordinator();
-        bool canMakePayments = paymentCoordinator.canMakePayments(document);
+        bool canMakePayments = paymentCoordinator.canMakePayments();
 
         RunLoop::main().dispatch([promise, canMakePayments]() mutable {
             promise->resolve<IDLBoolean>(canMakePayments);
@@ -483,7 +529,7 @@ ExceptionOr<void> ApplePaySession::openPaymentSetup(Document& document, const St
         return canCall.releaseException();
 
     if (!UserGestureIndicator::processingUserGesture())
-        return Exception { InvalidAccessError, "Must call ApplePaySession.openPaymentSetup from a user gesture handler." };
+        return Exception { InvalidAccessError, "Must call ApplePaySession.openPaymentSetup from a user gesture handler."_s };
 
     auto* page = document.page();
     if (!page)
@@ -502,18 +548,16 @@ ExceptionOr<void> ApplePaySession::openPaymentSetup(Document& document, const St
 ExceptionOr<void> ApplePaySession::begin(Document& document)
 {
     if (!canBegin())
-        return Exception { InvalidAccessError, "Payment session is already active." };
+        return Exception { InvalidAccessError, "Payment session is already active."_s };
 
     if (paymentCoordinator().hasActiveSession())
-        return Exception { InvalidAccessError, "Page already has an active payment session." };
+        return Exception { InvalidAccessError, "Page already has an active payment session."_s };
 
     if (!paymentCoordinator().beginPaymentSession(document, *this, m_paymentRequest))
-        return Exception { InvalidAccessError, "There is already has an active payment session." };
+        return Exception { InvalidAccessError, "There is already has an active payment session."_s };
 
     m_state = State::Active;
 
-    setPendingActivity(*this);
-
     return { };
 }
 
@@ -525,8 +569,6 @@ ExceptionOr<void> ApplePaySession::abort()
     m_state = State::Aborted;
     paymentCoordinator().abortPaymentSession();
 
-    didReachFinalState();
-
     return { };
 }
 
@@ -599,6 +641,25 @@ ExceptionOr<void> ApplePaySession::completePaymentMethodSelection(ApplePayPaymen
     return { };
 }
 
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+
+ExceptionOr<void> ApplePaySession::completeCouponCodeChange(ApplePayCouponCodeUpdate&& update)
+{
+    if (!canCompleteCouponCodeChange())
+        return Exception { InvalidAccessError };
+
+    auto convertedUpdate = convertAndValidate(WTFMove(update));
+    if (convertedUpdate.hasException())
+        return convertedUpdate.releaseException();
+
+    m_state = State::Active;
+    paymentCoordinator().completeCouponCodeChange(convertedUpdate.releaseReturnValue());
+
+    return { };
+}
+
+#endif // ENABLE(APPLE_PAY_COUPON_CODE)
+
 ExceptionOr<void> ApplePaySession::completePayment(ApplePayPaymentAuthorizationResult&& result)
 {
     if (!canCompletePayment())
@@ -609,7 +670,7 @@ ExceptionOr<void> ApplePaySession::completePayment(ApplePayPaymentAuthorizationR
         return convertedResultOrException.releaseException();
 
     auto&& convertedResult = convertedResultOrException.releaseReturnValue();
-    bool isFinalState = isFinalStateResult(convertedResult);
+    bool isFinalState = convertedResult.isFinalState();
 
     paymentCoordinator().completePaymentSession(WTFMove(convertedResult));
 
@@ -619,7 +680,6 @@ ExceptionOr<void> ApplePaySession::completePayment(ApplePayPaymentAuthorizationR
     }
 
     m_state = State::Completed;
-    unsetPendingActivity(*this);
 
     return { };
 }
@@ -658,8 +718,8 @@ ExceptionOr<void> ApplePaySession::completeShippingContactSelection(unsigned sho
 {
     ApplePayShippingContactUpdate update;
 
-    Optional<ApplePayErrorCode> errorCode;
-    Optional<ApplePayErrorContactField> contactField;
+    std::optional<ApplePayErrorCode> errorCode;
+    std::optional<ApplePayErrorContactField> contactField;
 
     switch (status) {
     case ApplePaySession::STATUS_SUCCESS:
@@ -796,18 +856,23 @@ void ApplePaySession::didSelectPaymentMethod(const PaymentMethod& paymentMethod)
     dispatchEvent(event.get());
 }
 
-#if ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#if ENABLE(APPLE_PAY_COUPON_CODE)
 
-void ApplePaySession::didChangePaymentMethodMode(String&& paymentMethodMode)
+void ApplePaySession::didChangeCouponCode(String&& couponCode)
 {
     ASSERT(m_state == State::Active);
 
-#if defined(ApplePaySessionAdditions_didChangePaymentMethodMode)
-    ApplePaySessionAdditions_didChangePaymentMethodMode
-#endif
+    if (!hasEventListeners(eventNames().couponcodechangedEvent)) {
+        paymentCoordinator().completeCouponCodeChange({ });
+        return;
+    }
+
+    m_state = State::CouponCodeChanged;
+    auto event = ApplePayCouponCodeChangedEvent::create(eventNames().couponcodechangedEvent, WTFMove(couponCode));
+    dispatchEvent(event.get());
 }
 
-#endif // ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#endif // ENABLE(APPLE_PAY_COUPON_CODE)
 
 void ApplePaySession::didCancelPaymentSession(PaymentSessionError&& error)
 {
@@ -817,8 +882,6 @@ void ApplePaySession::didCancelPaymentSession(PaymentSessionError&& error)
 
     auto event = ApplePayCancelEvent::create(eventNames().cancelEvent, WTFMove(error));
     dispatchEvent(event.get());
-
-    didReachFinalState();
 }
 
 const char* ApplePaySession::activeDOMObjectName() const
@@ -840,9 +903,9 @@ bool ApplePaySession::canSuspendWithoutCanceling() const
     case State::ShippingMethodSelected:
     case State::ShippingContactSelected:
     case State::PaymentMethodSelected:
-#if ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
-    case State::PaymentMethodModeChanged:
-#endif // ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return false;
     }
@@ -855,8 +918,6 @@ void ApplePaySession::stop()
 
     m_state = State::Aborted;
     paymentCoordinator().abortPaymentSession();
-
-    didReachFinalState();
 }
 
 void ApplePaySession::suspend(ReasonForSuspension reason)
@@ -867,11 +928,10 @@ void ApplePaySession::suspend(ReasonForSuspension reason)
     if (canSuspendWithoutCanceling())
         return;
 
+    auto jsWrapperProtector = makePendingActivity(*this);
     m_state = State::Canceled;
     paymentCoordinator().abortPaymentSession();
     queueTaskToDispatchEvent(*this, TaskSource::UserInteraction, ApplePayCancelEvent::create(eventNames().cancelEvent, { }));
-
-    didReachFinalState();
 }
 
 PaymentCoordinator& ApplePaySession::paymentCoordinator() const
@@ -893,9 +953,9 @@ bool ApplePaySession::canBegin() const
     case State::ShippingMethodSelected:
     case State::ShippingContactSelected:
     case State::PaymentMethodSelected:
-#if ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
-    case State::PaymentMethodModeChanged:
-#endif // ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return false;
     }
@@ -915,9 +975,9 @@ bool ApplePaySession::canAbort() const
     case State::ShippingMethodSelected:
     case State::ShippingContactSelected:
     case State::PaymentMethodSelected:
-#if ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
-    case State::PaymentMethodModeChanged:
-#endif // ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return true;
     }
@@ -937,9 +997,9 @@ bool ApplePaySession::canCancel() const
     case State::ShippingMethodSelected:
     case State::ShippingContactSelected:
     case State::PaymentMethodSelected:
-#if ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
-    case State::PaymentMethodModeChanged:
-#endif // ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return true;
     }
@@ -967,9 +1027,9 @@ bool ApplePaySession::canCompleteShippingMethodSelection() const
     case State::Authorized:
     case State::PaymentMethodSelected:
     case State::ShippingContactSelected:
-#if ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
-    case State::PaymentMethodModeChanged:
-#endif // ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return false;
 
@@ -989,9 +1049,9 @@ bool ApplePaySession::canCompleteShippingContactSelection() const
     case State::Authorized:
     case State::PaymentMethodSelected:
     case State::ShippingMethodSelected:
-#if ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
-    case State::PaymentMethodModeChanged:
-#endif // ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return false;
 
@@ -1011,9 +1071,9 @@ bool ApplePaySession::canCompletePaymentMethodSelection() const
     case State::Authorized:
     case State::ShippingMethodSelected:
     case State::ShippingContactSelected:
-#if ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
-    case State::PaymentMethodModeChanged:
-#endif // ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return false;
 
@@ -1022,9 +1082,9 @@ bool ApplePaySession::canCompletePaymentMethodSelection() const
     }
 }
 
-#if ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#if ENABLE(APPLE_PAY_COUPON_CODE)
 
-bool ApplePaySession::canCompletePaymentMethodModeChange() const
+bool ApplePaySession::canCompleteCouponCodeChange() const
 {
     switch (m_state) {
     case State::Idle:
@@ -1039,12 +1099,12 @@ bool ApplePaySession::canCompletePaymentMethodModeChange() const
     case State::PaymentMethodSelected:
         return false;
 
-    case State::PaymentMethodModeChanged:
+    case State::CouponCodeChanged:
         return true;
     }
 }
 
-#endif // ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#endif // ENABLE(APPLE_PAY_COUPON_CODE)
 
 bool ApplePaySession::canCompletePayment() const
 {
@@ -1057,9 +1117,9 @@ bool ApplePaySession::canCompletePayment() const
     case State::ShippingMethodSelected:
     case State::ShippingContactSelected:
     case State::PaymentMethodSelected:
-#if ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
-    case State::PaymentMethodModeChanged:
-#endif // ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::CancelRequested:
         return false;
 
@@ -1076,9 +1136,9 @@ bool ApplePaySession::isFinalState() const
     case State::ShippingMethodSelected:
     case State::ShippingContactSelected:
     case State::PaymentMethodSelected:
-#if ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
-    case State::PaymentMethodModeChanged:
-#endif // ENABLE(APPLE_PAY_PAYMENT_METHOD_MODE)
+#if ENABLE(APPLE_PAY_COUPON_CODE)
+    case State::CouponCodeChanged:
+#endif
     case State::Authorized:
     case State::CancelRequested:
         return false;
@@ -1090,16 +1150,11 @@ bool ApplePaySession::isFinalState() const
     }
 }
 
-void ApplePaySession::didReachFinalState()
+bool ApplePaySession::virtualHasPendingActivity() const
 {
-    ASSERT(isFinalState());
-    unsetPendingActivity(*this);
+    return m_state != State::Idle && !isFinalState();
 }
 
-#if defined(ApplePaySessionAdditions_definitions)
-ApplePaySessionAdditions_definitions
-#endif
+} // namespace WebCore
 
-}
-
-#endif
+#endif // ENABLE(APPLE_PAY)
