@@ -26,10 +26,9 @@
 #include "config.h"
 #include "IDBConnectionToServer.h"
 
-#if ENABLE(INDEXED_DATABASE)
-
 #include "IDBConnectionProxy.h"
 #include "IDBDatabase.h"
+#include "IDBDatabaseNameAndVersion.h"
 #include "IDBGetRecordData.h"
 #include "IDBKeyRangeData.h"
 #include "IDBOpenDBRequest.h"
@@ -38,18 +37,21 @@
 #include "Logging.h"
 #include "SecurityOrigin.h"
 #include "TransactionOperation.h"
+#include <wtf/IsoMallocInlines.h>
 #include <wtf/MainThread.h>
 
 namespace WebCore {
 namespace IDBClient {
 
+WTF_MAKE_ISO_ALLOCATED_IMPL(IDBConnectionToServer);
+
 Ref<IDBConnectionToServer> IDBConnectionToServer::create(IDBConnectionToServerDelegate& delegate)
 {
     return adoptRef(*new IDBConnectionToServer(delegate));
 }
 
 IDBConnectionToServer::IDBConnectionToServer(IDBConnectionToServerDelegate& delegate)
-    : m_delegate(makeWeakPtr(delegate))
+    : m_delegate(delegate)
     , m_proxy(makeUnique<IDBConnectionProxy>(*this))
 {
 }
@@ -67,7 +69,7 @@ IDBConnectionProxy& IDBConnectionToServer::proxy()
 
 void IDBConnectionToServer::callResultFunctionWithErrorLater(ResultFunction function, const IDBResourceIdentifier& requestIdentifier)
 {
-    callOnMainThread([this, protectedThis = makeRef(*this), function, requestIdentifier]() {
+    callOnMainThread([this, protectedThis = Ref { *this }, function, requestIdentifier]() {
         (this->*function)(IDBResultData::error(requestIdentifier, IDBError::serverConnectionLostError()));
     });
 }
@@ -354,15 +356,15 @@ void IDBConnectionToServer::establishTransaction(uint64_t databaseConnectionIden
         m_delegate->establishTransaction(databaseConnectionIdentifier, info);
 }
 
-void IDBConnectionToServer::commitTransaction(const IDBResourceIdentifier& transactionIdentifier)
+void IDBConnectionToServer::commitTransaction(const IDBResourceIdentifier& transactionIdentifier, uint64_t pendingRequestCount)
 {
     LOG(IndexedDB, "IDBConnectionToServer::commitTransaction");
     ASSERT(isMainThread());
 
     if (m_serverConnectionIsValid)
-        m_delegate->commitTransaction(transactionIdentifier);
+        m_delegate->commitTransaction(transactionIdentifier, pendingRequestCount);
     else {
-        callOnMainThread([this, protectedThis = makeRef(*this), transactionIdentifier] {
+        callOnMainThread([this, protectedThis = Ref { *this }, transactionIdentifier] {
             didCommitTransaction(transactionIdentifier, IDBError::serverConnectionLostError());
         });
     }
@@ -393,7 +395,7 @@ void IDBConnectionToServer::abortTransaction(const IDBResourceIdentifier& transa
     if (m_serverConnectionIsValid)
         m_delegate->abortTransaction(transactionIdentifier);
     else {
-        callOnMainThread([this, protectedThis = makeRef(*this), transactionIdentifier] {
+        callOnMainThread([this, protectedThis = Ref { *this }, transactionIdentifier] {
             didAbortTransaction(transactionIdentifier, IDBError::serverConnectionLostError());
         });
     }
@@ -485,7 +487,7 @@ void IDBConnectionToServer::databaseConnectionClosed(uint64_t databaseConnection
         m_delegate->databaseConnectionClosed(databaseConnectionIdentifier);
 }
 
-void IDBConnectionToServer::abortOpenAndUpgradeNeeded(uint64_t databaseConnectionIdentifier, const IDBResourceIdentifier& transactionIdentifier)
+void IDBConnectionToServer::abortOpenAndUpgradeNeeded(uint64_t databaseConnectionIdentifier, const std::optional<IDBResourceIdentifier>& transactionIdentifier)
 {
     LOG(IndexedDB, "IDBConnectionToServer::abortOpenAndUpgradeNeeded");
     ASSERT(isMainThread());
@@ -494,30 +496,28 @@ void IDBConnectionToServer::abortOpenAndUpgradeNeeded(uint64_t databaseConnectio
         m_delegate->abortOpenAndUpgradeNeeded(databaseConnectionIdentifier, transactionIdentifier);
 }
 
-void IDBConnectionToServer::getAllDatabaseNames(const SecurityOrigin& mainFrameOrigin, const SecurityOrigin& openingOrigin, Function<void (const Vector<String>&)>&& callback)
+void IDBConnectionToServer::getAllDatabaseNamesAndVersions(const IDBResourceIdentifier& requestIdentifier, const ClientOrigin& origin)
 {
-    static uint64_t callbackID = 0;
+    LOG(IndexedDB, "IDBConnectionToServer::getAllDatabaseNamesAndVersions");
+    ASSERT(isMainThread());
 
-    m_getAllDatabaseNamesCallbacks.add(++callbackID, WTFMove(callback));
-
-    if (m_serverConnectionIsValid)
-        m_delegate->getAllDatabaseNames(mainFrameOrigin.data(), openingOrigin.data(), callbackID);
-    else {
-        callOnMainThread([this, protectedThis = makeRef(*this), callbackID = callbackID] {
-            didGetAllDatabaseNames(callbackID, { });
-        });
+    if (m_serverConnectionIsValid) {
+        m_delegate->getAllDatabaseNamesAndVersions(requestIdentifier, origin);
+        return;
     }
+
+    callOnMainThread([this, protectedThis = Ref { *this }, requestIdentifier] {
+        didGetAllDatabaseNamesAndVersions(requestIdentifier, { });
+    });
 }
 
-void IDBConnectionToServer::didGetAllDatabaseNames(uint64_t callbackID, const Vector<String>& databaseNames)
+void IDBConnectionToServer::didGetAllDatabaseNamesAndVersions(const IDBResourceIdentifier& requestIdentifier, Vector<IDBDatabaseNameAndVersion>&& databases)
 {
-    auto callback = m_getAllDatabaseNamesCallbacks.take(callbackID);
-    ASSERT(callback);
+    LOG(IndexedDB, "IDBConnectionToServer::didGetAllDatabaseNamesAndVersions");
+    ASSERT(isMainThread());
 
-    callback(databaseNames);
+    m_proxy->didGetAllDatabaseNamesAndVersions(requestIdentifier, WTFMove(databases));
 }
 
 } // namespace IDBClient
 } // namespace WebCore
-
-#endif // ENABLE(INDEXED_DATABASE)
