@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2017-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,34 +28,32 @@
 
 #include "WebCoreJSClientData.h"
 #include <JavaScriptCore/CatchScope.h>
+#include <JavaScriptCore/HashMapImplInlines.h>
 #include <JavaScriptCore/JSMap.h>
+#include <JavaScriptCore/VMTrapsInlines.h>
 
 namespace WebCore {
 
 std::pair<bool, std::reference_wrapper<JSC::JSObject>> getBackingMap(JSC::JSGlobalObject& lexicalGlobalObject, JSC::JSObject& mapLike)
 {
     auto& vm = lexicalGlobalObject.vm();
-    auto backingMap = mapLike.get(&lexicalGlobalObject, static_cast<JSVMClientData*>(vm.clientData)->builtinNames().backingMapPrivateName());
-    if (!backingMap.isUndefined())
+    auto backingMap = mapLike.getDirect(vm, builtinNames(vm).backingMapPrivateName());
+    if (backingMap)
         return { false, *JSC::asObject(backingMap) };
 
-    auto scope = DECLARE_CATCH_SCOPE(vm);
-
-    backingMap = JSC::JSMap::create(&lexicalGlobalObject, vm, lexicalGlobalObject.mapStructure());
-    scope.releaseAssertNoException();
-
-    mapLike.putDirect(vm, static_cast<JSVMClientData*>(vm.clientData)->builtinNames().backingMapPrivateName(), backingMap, static_cast<unsigned>(JSC::PropertyAttribute::DontEnum));
+    backingMap = JSC::JSMap::create(vm, lexicalGlobalObject.mapStructure());
+    mapLike.putDirect(vm, builtinNames(vm).backingMapPrivateName(), backingMap, static_cast<unsigned>(JSC::PropertyAttribute::DontEnum));
     return { true, *JSC::asObject(backingMap) };
 }
 
 void clearBackingMap(JSC::JSGlobalObject& lexicalGlobalObject, JSC::JSObject& backingMap)
 {
     auto& vm = JSC::getVM(&lexicalGlobalObject);
-    auto function = backingMap.get(&lexicalGlobalObject, vm.propertyNames->clear);
+    auto function = lexicalGlobalObject.mapPrototype()->getDirect(vm, vm.propertyNames->builtinNames().clearPrivateName());
+    ASSERT(function);
 
-    auto callData = JSC::getCallData(vm, function);
-    if (callData.type == JSC::CallData::Type::None)
-        return;
+    auto callData = JSC::getCallData(function);
+    ASSERT(callData.type != JSC::CallData::Type::None);
 
     JSC::MarkedArgumentBuffer arguments;
     JSC::call(&lexicalGlobalObject, function, callData, &backingMap, arguments);
@@ -64,11 +62,11 @@ void clearBackingMap(JSC::JSGlobalObject& lexicalGlobalObject, JSC::JSObject& ba
 void setToBackingMap(JSC::JSGlobalObject& lexicalGlobalObject, JSC::JSObject& backingMap, JSC::JSValue key, JSC::JSValue value)
 {
     auto& vm = JSC::getVM(&lexicalGlobalObject);
-    auto function = backingMap.get(&lexicalGlobalObject, vm.propertyNames->set);
+    auto function = lexicalGlobalObject.mapPrototype()->getDirect(vm, vm.propertyNames->builtinNames().setPrivateName());
+    ASSERT(function);
 
-    auto callData = JSC::getCallData(vm, function);
-    if (callData.type == JSC::CallData::Type::None)
-        return;
+    auto callData = JSC::getCallData(function);
+    ASSERT(callData.type != JSC::CallData::Type::None);
 
     JSC::MarkedArgumentBuffer arguments;
     arguments.append(key);
@@ -83,11 +81,12 @@ JSC::JSValue forwardAttributeGetterToBackingMap(JSC::JSGlobalObject& lexicalGlob
 
 JSC::JSValue forwardFunctionCallToBackingMap(JSC::JSGlobalObject& lexicalGlobalObject, JSC::CallFrame& callFrame, JSC::JSObject& backingMap, const JSC::Identifier& functionName)
 {
-    auto function = backingMap.get(&lexicalGlobalObject, functionName);
+    auto& vm = JSC::getVM(&lexicalGlobalObject);
+    auto function = lexicalGlobalObject.mapPrototype()->getDirect(vm, functionName);
+    ASSERT(function);
 
-    auto callData = JSC::getCallData(lexicalGlobalObject.vm(), function);
-    if (callData.type == JSC::CallData::Type::None)
-        return JSC::jsUndefined();
+    auto callData = JSC::getCallData(function);
+    ASSERT(callData.type != JSC::CallData::Type::None);
 
     JSC::MarkedArgumentBuffer arguments;
     for (size_t cptr = 0; cptr < callFrame.argumentCount(); ++cptr)
@@ -104,7 +103,7 @@ JSC::JSValue forwardForEachCallToBackingMap(JSDOMGlobalObject& globalObject, JSC
     auto* function = globalObject.builtinInternalFunctions().jsDOMBindingInternals().m_forEachWrapperFunction.get();
     ASSERT(function);
 
-    auto callData = JSC::getCallData(globalObject.vm(), function);
+    auto callData = JSC::getCallData(function);
     ASSERT(callData.type != JSC::CallData::Type::None);
 
     JSC::MarkedArgumentBuffer arguments;
