@@ -50,7 +50,7 @@ namespace WebCore {
 
 using namespace JSC;
 
-const ClassInfo JSWindowProxy::s_info = { "JSWindowProxy", &Base::s_info, nullptr, nullptr, CREATE_METHOD_TABLE(JSWindowProxy) };
+const ClassInfo JSWindowProxy::s_info = { "JSWindowProxy"_s, &Base::s_info, nullptr, nullptr, CREATE_METHOD_TABLE(JSWindowProxy) };
 
 inline JSWindowProxy::JSWindowProxy(VM& vm, Structure& structure, DOMWrapperWorld& world)
     : Base(vm, &structure)
@@ -61,7 +61,7 @@ inline JSWindowProxy::JSWindowProxy(VM& vm, Structure& structure, DOMWrapperWorl
 void JSWindowProxy::finishCreation(VM& vm, AbstractDOMWindow& window)
 {
     Base::finishCreation(vm);
-    ASSERT(inherits(vm, info()));
+    ASSERT(inherits(info()));
     setWindow(window);
 }
 
@@ -80,9 +80,9 @@ void JSWindowProxy::destroy(JSCell* cell)
 
 void JSWindowProxy::setWindow(VM& vm, JSDOMGlobalObject& window)
 {
-    ASSERT(window.classInfo(vm) == JSDOMWindow::info() || window.classInfo(vm) == JSRemoteDOMWindow::info());
+    ASSERT(window.classInfo() == JSDOMWindow::info() || window.classInfo() == JSRemoteDOMWindow::info());
     setTarget(vm, &window);
-    structure(vm)->setGlobalObject(vm, &window);
+    structure()->setGlobalObject(vm, &window);
     GCController::singleton().garbageCollectSoon();
 }
 
@@ -112,18 +112,18 @@ void JSWindowProxy::setWindow(AbstractDOMWindow& domWindow)
         window = JSDOMWindow::create(vm, &windowStructure, localWindow, this);
         bool linkedWithNewSDK = true;
 #if PLATFORM(COCOA)
-        linkedWithNewSDK = linkedOnOrAfter(SDKVersion::FirstWithDOMWindowReuseRestriction);
+        linkedWithNewSDK = linkedOnOrAfterSDKWithBehavior(SDKAlignedBehavior::DOMWindowReuseRestriction);
 #endif
         if (!localWindow.document()->haveInitializedSecurityOrigin() && linkedWithNewSDK)
             localWindow.setAsWrappedWithoutInitializedSecurityOrigin();
     }
 
-    prototype->structure(vm)->setGlobalObject(vm, window);
+    prototype->structure()->setGlobalObject(vm, window);
 
     auto& propertiesStructure = *JSDOMWindowProperties::createStructure(vm, window, JSEventTarget::prototype(vm, *window));
     auto& properties = *JSDOMWindowProperties::create(&propertiesStructure, *window);
     properties.didBecomePrototype();
-    prototype->structure(vm)->setPrototypeWithoutTransition(vm, &properties);
+    prototype->structure()->setPrototypeWithoutTransition(vm, &properties);
 
     setWindow(vm, *window);
 
@@ -151,7 +151,7 @@ void JSWindowProxy::attachDebugger(JSC::Debugger* debugger)
 AbstractDOMWindow& JSWindowProxy::wrapped() const
 {
     auto* window = this->window();
-    if (auto* jsWindow = jsDynamicCast<JSRemoteDOMWindowBase*>(window->vm(), window))
+    if (auto* jsWindow = jsDynamicCast<JSRemoteDOMWindowBase*>(window))
         return jsWindow->wrapped();
     return jsCast<JSDOMWindowBase*>(window)->wrapped();
 }
@@ -167,12 +167,12 @@ JSWindowProxy* toJSWindowProxy(WindowProxy& windowProxy, DOMWrapperWorld& world)
     return windowProxy.jsWindowProxy(world);
 }
 
-WindowProxy* JSWindowProxy::toWrapped(VM& vm, JSValue value)
+WindowProxy* JSWindowProxy::toWrapped(VM&, JSValue value)
 {
     if (!value.isObject())
         return nullptr;
     JSObject* object = asObject(value);
-    if (object->inherits<JSWindowProxy>(vm))
+    if (object->inherits<JSWindowProxy>())
         return jsCast<JSWindowProxy*>(object)->windowProxy();
     return nullptr;
 }
