@@ -30,6 +30,7 @@
 
 #include "Animation.h"
 #include "CSSBorderImageSliceValue.h"
+#include "CSSBorderImageWidthValue.h"
 #include "CSSImageGeneratorValue.h"
 #include "CSSImageSetValue.h"
 #include "CSSImageValue.h"
@@ -37,6 +38,7 @@
 #include "CSSPrimitiveValueMappings.h"
 #include "CSSTimingFunctionValue.h"
 #include "CSSValueKeywords.h"
+#include "CompositeOperation.h"
 #include "FillLayer.h"
 #include "Pair.h"
 #include "Rect.h"
@@ -264,11 +266,11 @@ void CSSToStyleMap::mapFillYPosition(CSSPropertyID propertyID, FillLayer& layer,
         layer.setBackgroundYOrigin(*pair->first());
 }
 
-void CSSToStyleMap::mapFillMaskSourceType(CSSPropertyID propertyID, FillLayer& layer, const CSSValue& value)
+void CSSToStyleMap::mapFillMaskMode(CSSPropertyID propertyID, FillLayer& layer, const CSSValue& value)
 {
-    MaskSourceType type = FillLayer::initialFillMaskSourceType(layer.type());
+    MaskMode maskMode = FillLayer::initialFillMaskMode(layer.type());
     if (value.treatAsInitialValue(propertyID)) {
-        layer.setMaskSourceType(type);
+        layer.setMaskMode(maskMode);
         return;
     }
 
@@ -277,18 +279,23 @@ void CSSToStyleMap::mapFillMaskSourceType(CSSPropertyID propertyID, FillLayer& l
 
     switch (downcast<CSSPrimitiveValue>(value).valueID()) {
     case CSSValueAlpha:
-        type = MaskSourceType::Alpha;
+        maskMode = MaskMode::Alpha;
         break;
     case CSSValueLuminance:
-        type = MaskSourceType::Luminance;
+        maskMode = MaskMode::Luminance;
+        break;
+    case CSSValueMatchSource:
+        ASSERT(propertyID == CSSPropertyMaskMode);
+        maskMode = MaskMode::MatchSource;
         break;
     case CSSValueAuto:
+        ASSERT(propertyID == CSSPropertyWebkitMaskSourceType);
         break;
     default:
         ASSERT_NOT_REACHED();
     }
 
-    layer.setMaskSourceType(type);
+    layer.setMaskMode(maskMode);
 }
 
 void CSSToStyleMap::mapAnimationDelay(Animation& animation, const CSSValue& value)
@@ -404,7 +411,7 @@ void CSSToStyleMap::mapAnimationName(Animation& layer, const CSSValue& value)
     if (primitiveValue.valueID() == CSSValueNone)
         layer.setIsNoneAnimation(true);
     else
-        layer.setName(primitiveValue.stringValue(), m_builderState.styleScopeOrdinal());
+        layer.setName({ primitiveValue.stringValue(), primitiveValue.isCustomIdent() }, m_builderState.styleScopeOrdinal());
 }
 
 void CSSToStyleMap::mapAnimationPlayState(Animation& layer, const CSSValue& value)
@@ -451,53 +458,21 @@ void CSSToStyleMap::mapAnimationProperty(Animation& animation, const CSSValue& v
 
 void CSSToStyleMap::mapAnimationTimingFunction(Animation& animation, const CSSValue& value)
 {
-    if (value.treatAsInitialValue(CSSPropertyAnimationTimingFunction)) {
+    if (value.treatAsInitialValue(CSSPropertyAnimationTimingFunction))
         animation.setTimingFunction(Animation::initialTimingFunction());
-        return;
-    }
-
-    if (is<CSSPrimitiveValue>(value)) {
-        switch (downcast<CSSPrimitiveValue>(value).valueID()) {
-        case CSSValueLinear:
-            animation.setTimingFunction(LinearTimingFunction::create());
-            break;
-        case CSSValueEase:
-            animation.setTimingFunction(CubicBezierTimingFunction::create());
-            break;
-        case CSSValueEaseIn:
-            animation.setTimingFunction(CubicBezierTimingFunction::create(CubicBezierTimingFunction::EaseIn));
-            break;
-        case CSSValueEaseOut:
-            animation.setTimingFunction(CubicBezierTimingFunction::create(CubicBezierTimingFunction::EaseOut));
-            break;
-        case CSSValueEaseInOut:
-            animation.setTimingFunction(CubicBezierTimingFunction::create(CubicBezierTimingFunction::EaseInOut));
-            break;
-        case CSSValueStepStart:
-            animation.setTimingFunction(StepsTimingFunction::create(1, StepsTimingFunction::StepPosition::Start));
-            break;
-        case CSSValueStepEnd:
-            animation.setTimingFunction(StepsTimingFunction::create(1, StepsTimingFunction::StepPosition::End));
-            break;
-        default:
-            break;
-        }
-        return;
-    }
-
-    if (is<CSSCubicBezierTimingFunctionValue>(value)) {
-        auto& cubicTimingFunction = downcast<CSSCubicBezierTimingFunctionValue>(value);
-        animation.setTimingFunction(CubicBezierTimingFunction::create(cubicTimingFunction.x1(), cubicTimingFunction.y1(), cubicTimingFunction.x2(), cubicTimingFunction.y2()));
-    } else if (is<CSSStepsTimingFunctionValue>(value)) {
-        auto& stepsTimingFunction = downcast<CSSStepsTimingFunctionValue>(value);
-        animation.setTimingFunction(StepsTimingFunction::create(stepsTimingFunction.numberOfSteps(), stepsTimingFunction.stepPosition()));
-    } else if (is<CSSSpringTimingFunctionValue>(value)) {
-        auto& springTimingFunction = downcast<CSSSpringTimingFunctionValue>(value);
-        animation.setTimingFunction(SpringTimingFunction::create(springTimingFunction.mass(), springTimingFunction.stiffness(), springTimingFunction.damping(), springTimingFunction.initialVelocity()));
-    }
+    else if (auto timingFunction = TimingFunction::createFromCSSValue(value))
+        animation.setTimingFunction(WTFMove(timingFunction));
 }
 
-void CSSToStyleMap::mapNinePieceImage(CSSPropertyID property, CSSValue* value, NinePieceImage& image)
+void CSSToStyleMap::mapAnimationCompositeOperation(Animation& animation, const CSSValue& value)
+{
+    if (value.treatAsInitialValue(CSSPropertyAnimationComposition))
+        animation.setCompositeOperation(Animation::initialCompositeOperation());
+    else if (auto compositeOperation = toCompositeOperation(value))
+        animation.setCompositeOperation(*compositeOperation);
+}
+
+void CSSToStyleMap::mapNinePieceImage(CSSValue* value, NinePieceImage& image)
 {
     // If we're not a value list, then we are "none" and don't need to alter the empty image at all.
     if (!is<CSSValueList>(value))
@@ -518,8 +493,8 @@ void CSSToStyleMap::mapNinePieceImage(CSSPropertyID property, CSSValue* value, N
                 mapNinePieceImageSlice(*slashList.item(0), image);
 
             // Map in the border slices.
-            if (slashList.item(1))
-                image.setBorderSlices(mapNinePieceImageQuad(*slashList.item(1)));
+            if (is<CSSBorderImageWidthValue>(slashList.item(1)))
+                mapNinePieceImageWidth(*slashList.item(1), image);
 
             // Map in the outset.
             if (slashList.item(2))
@@ -529,20 +504,6 @@ void CSSToStyleMap::mapNinePieceImage(CSSPropertyID property, CSSValue* value, N
             mapNinePieceImageRepeat(current, image);
         }
     }
-
-    if (property == CSSPropertyWebkitBorderImage) {
-        // We have to preserve the legacy behavior of -webkit-border-image and make the border slices
-        // also set the border widths. We don't need to worry about percentages, since we don't even support
-        // those on real borders yet.
-        if (image.borderSlices().top().isFixed())
-            style()->setBorderTopWidth(image.borderSlices().top().value());
-        if (image.borderSlices().right().isFixed())
-            style()->setBorderRightWidth(image.borderSlices().right().value());
-        if (image.borderSlices().bottom().isFixed())
-            style()->setBorderBottomWidth(image.borderSlices().bottom().value());
-        if (image.borderSlices().left().isFixed())
-            style()->setBorderLeftWidth(image.borderSlices().left().value());
-    }
 }
 
 void CSSToStyleMap::mapNinePieceImageSlice(CSSValue& value, NinePieceImage& image)
@@ -578,47 +539,70 @@ void CSSToStyleMap::mapNinePieceImageSlice(CSSValue& value, NinePieceImage& imag
     image.setFill(borderImageSlice.m_fill);
 }
 
+void CSSToStyleMap::mapNinePieceImageWidth(CSSValue& value, NinePieceImage& image)
+{
+    if (!is<CSSBorderImageWidthValue>(value))
+        return;
+
+    // Retrieve the border image value.
+    auto& borderImageWidth = downcast<CSSBorderImageWidthValue>(value);
+    image.setBorderSlices(mapNinePieceImageQuad(borderImageWidth.widths()));
+    image.setOverridesBorderWidths(borderImageWidth.m_overridesBorderWidths);
+}
+
 LengthBox CSSToStyleMap::mapNinePieceImageQuad(CSSValue& value)
 {
     if (!is<CSSPrimitiveValue>(value))
         return LengthBox();
 
-    // Get our zoom value.
-    CSSToLengthConversionData conversionData = useSVGZoomRules() ? m_builderState.cssToLengthConversionData().copyWithAdjustedZoom(1.0f) : m_builderState.cssToLengthConversionData();
-
     // Retrieve the primitive value.
     auto& borderWidths = downcast<CSSPrimitiveValue>(value);
 
+    return mapNinePieceImageQuad(borderWidths.quadValue());
+}
+
+LengthBox CSSToStyleMap::mapNinePieceImageQuad(Quad* quad)
+{
+    // Get our zoom value.
+    CSSToLengthConversionData conversionData = useSVGZoomRules() ? m_builderState.cssToLengthConversionData().copyWithAdjustedZoom(1.0f) : m_builderState.cssToLengthConversionData();
+
     // Set up a length box to represent our image slices.
     LengthBox box; // Defaults to 'auto' so we don't have to handle that explicitly below.
-    Quad* slices = borderWidths.quadValue();
-    if (slices->top()->isNumber())
-        box.top() = Length(slices->top()->floatValue(), LengthType::Relative);
-    else if (slices->top()->isPercentage())
-        box.top() = Length(slices->top()->doubleValue(CSSUnitType::CSS_PERCENTAGE), LengthType::Percent);
-    else if (slices->top()->valueID() != CSSValueAuto)
-        box.top() = slices->top()->computeLength<Length>(conversionData);
+    if (quad->top()->isNumber())
+        box.top() = Length(quad->top()->floatValue(), LengthType::Relative);
+    else if (quad->top()->isPercentage())
+        box.top() = Length(quad->top()->doubleValue(CSSUnitType::CSS_PERCENTAGE), LengthType::Percent);
+    else if (quad->top()->isCalculatedPercentageWithLength())
+        box.top() = Length(quad->top()->cssCalcValue()->createCalculationValue(conversionData));
+    else if (quad->top()->valueID() != CSSValueAuto)
+        box.top() = quad->top()->computeLength<Length>(conversionData);
 
-    if (slices->right()->isNumber())
-        box.right() = Length(slices->right()->floatValue(), LengthType::Relative);
-    else if (slices->right()->isPercentage())
-        box.right() = Length(slices->right()->doubleValue(CSSUnitType::CSS_PERCENTAGE), LengthType::Percent);
-    else if (slices->right()->valueID() != CSSValueAuto)
-        box.right() = slices->right()->computeLength<Length>(conversionData);
+    if (quad->right()->isNumber())
+        box.right() = Length(quad->right()->floatValue(), LengthType::Relative);
+    else if (quad->right()->isPercentage())
+        box.right() = Length(quad->right()->doubleValue(CSSUnitType::CSS_PERCENTAGE), LengthType::Percent);
+    else if (quad->right()->isCalculatedPercentageWithLength())
+        box.right() = Length(quad->right()->cssCalcValue()->createCalculationValue(conversionData));
+    else if (quad->right()->valueID() != CSSValueAuto)
+        box.right() = quad->right()->computeLength<Length>(conversionData);
 
-    if (slices->bottom()->isNumber())
-        box.bottom() = Length(slices->bottom()->floatValue(), LengthType::Relative);
-    else if (slices->bottom()->isPercentage())
-        box.bottom() = Length(slices->bottom()->doubleValue(CSSUnitType::CSS_PERCENTAGE), LengthType::Percent);
-    else if (slices->bottom()->valueID() != CSSValueAuto)
-        box.bottom() = slices->bottom()->computeLength<Length>(conversionData);
+    if (quad->bottom()->isNumber())
+        box.bottom() = Length(quad->bottom()->floatValue(), LengthType::Relative);
+    else if (quad->bottom()->isPercentage())
+        box.bottom() = Length(quad->bottom()->doubleValue(CSSUnitType::CSS_PERCENTAGE), LengthType::Percent);
+    else if (quad->bottom()->isCalculatedPercentageWithLength())
+        box.bottom() = Length(quad->bottom()->cssCalcValue()->createCalculationValue(conversionData));
+    else if (quad->bottom()->valueID() != CSSValueAuto)
+        box.bottom() = quad->bottom()->computeLength<Length>(conversionData);
 
-    if (slices->left()->isNumber())
-        box.left() = Length(slices->left()->floatValue(), LengthType::Relative);
-    else if (slices->left()->isPercentage())
-        box.left() = Length(slices->left()->doubleValue(CSSUnitType::CSS_PERCENTAGE), LengthType::Percent);
-    else if (slices->left()->valueID() != CSSValueAuto)
-        box.left() = slices->left()->computeLength<Length>(conversionData);
+    if (quad->left()->isNumber())
+        box.left() = Length(quad->left()->floatValue(), LengthType::Relative);
+    else if (quad->left()->isPercentage())
+        box.left() = Length(quad->left()->doubleValue(CSSUnitType::CSS_PERCENTAGE), LengthType::Percent);
+    else if (quad->left()->isCalculatedPercentageWithLength())
+        box.left() = Length(quad->left()->cssCalcValue()->createCalculationValue(conversionData));
+    else if (quad->left()->valueID() != CSSValueAuto)
+        box.left() = quad->left()->computeLength<Length>(conversionData);
 
     return box;
 }
