@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007, 2008, 2013 Apple Inc. All rights reserved.
+ * Copyright (C) 2006-2022 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,19 +28,22 @@
 
 #include "CachedImage.h"
 #include "CachedImageClient.h"
+#include "CommonAtomStrings.h"
 #include "DataTransferItem.h"
 #include "DataTransferItemList.h"
+#include "DeprecatedGlobalSettings.h"
 #include "DocumentFragment.h"
 #include "DragData.h"
 #include "Editor.h"
 #include "FileList.h"
 #include "Frame.h"
+#include "FrameDestructionObserverInlines.h"
 #include "FrameLoader.h"
 #include "HTMLImageElement.h"
 #include "HTMLParserIdioms.h"
 #include "Image.h"
+#include "PagePasteboardContext.h"
 #include "Pasteboard.h"
-#include "RuntimeEnabledFeatures.h"
 #include "Settings.h"
 #include "StaticPasteboard.h"
 #include "WebContentReader.h"
@@ -68,17 +71,18 @@ private:
 
 #endif
 
-DataTransfer::DataTransfer(StoreMode mode, std::unique_ptr<Pasteboard> pasteboard, Type type)
+DataTransfer::DataTransfer(StoreMode mode, std::unique_ptr<Pasteboard> pasteboard, Type type, String&& effectAllowed)
     : m_storeMode(mode)
     , m_pasteboard(WTFMove(pasteboard))
 #if ENABLE(DRAG_SUPPORT)
     , m_type(type)
     , m_dropEffect("uninitialized"_s)
-    , m_effectAllowed("uninitialized"_s)
+    , m_effectAllowed(WTFMove(effectAllowed))
     , m_shouldUpdateDragImage(false)
 #endif
 {
 #if !ENABLE(DRAG_SUPPORT)
+    UNUSED_PARAM(effectAllowed);
     ASSERT_UNUSED(type, type != Type::DragAndDropData && type != Type::DragAndDropFiles);
 #endif
 }
@@ -90,6 +94,11 @@ Ref<DataTransfer> DataTransfer::createForCopyAndPaste(const Document& document,
     return dataTransfer;
 }
 
+Ref<DataTransfer> DataTransfer::create()
+{
+    return adoptRef(*new DataTransfer(StoreMode::ReadWrite, makeUnique<StaticPasteboard>(), Type::CopyAndPaste, "none"_s));
+}
+
 DataTransfer::~DataTransfer()
 {
 #if ENABLE(DRAG_SUPPORT)
@@ -119,12 +128,12 @@ static String normalizeType(const String& type)
         return type;
 
     String lowercaseType = stripLeadingAndTrailingHTMLSpaces(type).convertToASCIILowercase();
-    if (lowercaseType == "text" || lowercaseType.startsWith("text/plain;"))
-        return "text/plain";
-    if (lowercaseType == "url" || lowercaseType.startsWith("text/uri-list;"))
-        return "text/uri-list";
-    if (lowercaseType.startsWith("text/html;"))
-        return "text/html";
+    if (lowercaseType == "text"_s || lowercaseType.startsWith("text/plain;"_s))
+        return textPlainContentTypeAtom();
+    if (lowercaseType == "url"_s || lowercaseType.startsWith("text/uri-list;"_s))
+        return "text/uri-list"_s;
+    if (lowercaseType.startsWith("text/html;"_s))
+        return "text/html"_s;
 
     return lowercaseType;
 }
@@ -163,13 +172,13 @@ String DataTransfer::getDataForItem(Document& document, const String& type) cons
 
     auto lowercaseType = stripLeadingAndTrailingHTMLSpaces(type).convertToASCIILowercase();
     if (shouldSuppressGetAndSetDataToAvoidExposingFilePaths()) {
-        if (lowercaseType == "text/uri-list") {
+        if (lowercaseType == "text/uri-list"_s) {
             return readURLsFromPasteboardAsString(*m_pasteboard, [] (auto& urlString) {
                 return Pasteboard::canExposeURLToDOMWhenPasteboardContainsFiles(urlString);
             });
         }
 
-        if (lowercaseType == "text/html" && RuntimeEnabledFeatures::sharedFeatures().customPasteboardDataEnabled()) {
+        if (lowercaseType == "text/html"_s && DeprecatedGlobalSettings::customPasteboardDataEnabled()) {
             // If the pasteboard contains files and the page requests 'text/html', we only read from rich text types to prevent file
             // paths from leaking (e.g. from plain text data on the pasteboard) since we sanitize cross-origin markup. However, if
             // custom pasteboard data is disabled, then we can't ensure that the markup we deliver is sanitized, so we fall back to
@@ -185,7 +194,7 @@ String DataTransfer::getDataForItem(Document& document, const String& type) cons
 
 String DataTransfer::readStringFromPasteboard(Document& document, const String& lowercaseType, WebContentReadingPolicy policy) const
 {
-    if (!RuntimeEnabledFeatures::sharedFeatures().customPasteboardDataEnabled())
+    if (!DeprecatedGlobalSettings::customPasteboardDataEnabled())
         return m_pasteboard->readString(lowercaseType);
 
     // StaticPasteboard is only used to stage data written by websites before being committed to the system pasteboard.
@@ -198,7 +207,7 @@ String DataTransfer::readStringFromPasteboard(Document& document, const String&
     if (!Pasteboard::isSafeTypeForDOMToReadAndWrite(lowercaseType))
         return { };
 
-    if (!is<StaticPasteboard>(*m_pasteboard) && lowercaseType == "text/html") {
+    if (!is<StaticPasteboard>(*m_pasteboard) && lowercaseType == "text/html"_s) {
         if (!document.frame())
             return { };
         WebContentMarkupReader reader { *document.frame() };
@@ -206,7 +215,7 @@ String DataTransfer::readStringFromPasteboard(Document& document, const String&
         return reader.markup;
     }
 
-    if (!is<StaticPasteboard>(*m_pasteboard) && lowercaseType == "text/uri-list") {
+    if (!is<StaticPasteboard>(*m_pasteboard) && lowercaseType == "text/uri-list"_s) {
         return readURLsFromPasteboardAsString(*m_pasteboard, [] (auto&) {
             return true;
         });
@@ -222,7 +231,7 @@ String DataTransfer::getData(Document& document, const String& type) const
 
 bool DataTransfer::shouldSuppressGetAndSetDataToAvoidExposingFilePaths() const
 {
-    if (!forFileDrag() && !RuntimeEnabledFeatures::sharedFeatures().customPasteboardDataEnabled())
+    if (!forFileDrag() && !DeprecatedGlobalSettings::customPasteboardDataEnabled())
         return false;
     return m_pasteboard->fileContentState() == Pasteboard::FileContentState::MayContainFilePaths;
 }
@@ -246,19 +255,19 @@ void DataTransfer::setDataFromItemList(const String& type, const String& data)
     ASSERT(canWriteData());
     RELEASE_ASSERT(is<StaticPasteboard>(*m_pasteboard));
 
-    if (!RuntimeEnabledFeatures::sharedFeatures().customPasteboardDataEnabled()) {
+    if (!DeprecatedGlobalSettings::customPasteboardDataEnabled()) {
         m_pasteboard->writeString(type, data);
         return;
     }
 
     String sanitizedData;
-    if (type == "text/html")
+    if (type == "text/html"_s)
         sanitizedData = sanitizeMarkup(data);
-    else if (type == "text/uri-list") {
+    else if (type == "text/uri-list"_s) {
         auto url = URL({ }, data);
         if (url.isValid())
             sanitizedData = url.string();
-    } else if (type == "text/plain")
+    } else if (type == textPlainContentTypeAtom())
         sanitizedData = data; // Nothing to sanitize.
 
     if (sanitizedData != data)
@@ -268,11 +277,11 @@ void DataTransfer::setDataFromItemList(const String& type, const String& data)
         m_pasteboard->writeString(type, sanitizedData);
 }
 
-void DataTransfer::updateFileList()
+void DataTransfer::updateFileList(ScriptExecutionContext* context)
 {
     ASSERT(canWriteData());
 
-    m_fileList->m_files = filesFromPasteboardAndItemList();
+    m_fileList->m_files = filesFromPasteboardAndItemList(context);
 }
 
 void DataTransfer::didAddFileToItemList()
@@ -286,10 +295,10 @@ void DataTransfer::didAddFileToItemList()
     m_fileList->append(*newItem->file());
 }
 
-DataTransferItemList& DataTransfer::items()
+DataTransferItemList& DataTransfer::items(Document& document)
 {
     if (!m_itemList)
-        m_itemList = makeUnique<DataTransferItemList>(*this);
+        m_itemList = makeUnique<DataTransferItemList>(document, *this);
     return *m_itemList;
 }
 
@@ -308,23 +317,24 @@ Vector<String> DataTransfer::types(AddFilesType addFilesType) const
     if (!canReadTypes())
         return { };
     
-    if (!RuntimeEnabledFeatures::sharedFeatures().customPasteboardDataEnabled()) {
+    bool hideFilesType = !canWriteData() && !allowsFileAccess();
+    if (!DeprecatedGlobalSettings::customPasteboardDataEnabled()) {
         auto types = m_pasteboard->typesForLegacyUnsafeBindings();
-        ASSERT(!types.contains("Files"));
-        if (m_pasteboard->fileContentState() != Pasteboard::FileContentState::NoFileOrImageData && addFilesType == AddFilesType::Yes)
-            types.append("Files");
+        ASSERT(!types.contains("Files"_s));
+        if (!hideFilesType && m_pasteboard->fileContentState() != Pasteboard::FileContentState::NoFileOrImageData && addFilesType == AddFilesType::Yes)
+            types.append("Files"_s);
         return types;
     }
 
     auto safeTypes = m_pasteboard->typesSafeForBindings(m_originIdentifier);
-    bool hasFileBackedItem = m_itemList && m_itemList->hasItems() && notFound != m_itemList->items().findMatching([] (const auto& item) {
+    bool hasFileBackedItem = m_itemList && m_itemList->hasItems() && notFound != m_itemList->items().findIf([] (const auto& item) {
         return item->isFile();
     });
 
     auto fileContentState = m_pasteboard->fileContentState();
     if (hasFileBackedItem || fileContentState != Pasteboard::FileContentState::NoFileOrImageData) {
         Vector<String> types;
-        if (addFilesType == AddFilesType::Yes)
+        if (!hideFilesType && addFilesType == AddFilesType::Yes)
             types.append("Files"_s);
 
         if (fileContentState != Pasteboard::FileContentState::MayContainFilePaths) {
@@ -332,23 +342,23 @@ Vector<String> DataTransfer::types(AddFilesType addFilesType) const
             return types;
         }
 
-        if (safeTypes.contains("text/uri-list"))
+        if (safeTypes.contains("text/uri-list"_s))
             types.append("text/uri-list"_s);
-        if (safeTypes.contains("text/html") && RuntimeEnabledFeatures::sharedFeatures().customPasteboardDataEnabled())
+        if (safeTypes.contains("text/html"_s) && DeprecatedGlobalSettings::customPasteboardDataEnabled())
             types.append("text/html"_s);
         return types;
     }
 
-    ASSERT(!safeTypes.contains("Files"));
+    ASSERT(!safeTypes.contains("Files"_s));
     return safeTypes;
 }
 
-Vector<Ref<File>> DataTransfer::filesFromPasteboardAndItemList() const
+Vector<Ref<File>> DataTransfer::filesFromPasteboardAndItemList(ScriptExecutionContext* context) const
 {
     bool addedFilesFromPasteboard = false;
     Vector<Ref<File>> files;
-    if ((!forDrag() || forFileDrag()) && m_pasteboard->fileContentState() != Pasteboard::FileContentState::NoFileOrImageData) {
-        WebCorePasteboardFileReader reader;
+    if (allowsFileAccess() && m_pasteboard->fileContentState() != Pasteboard::FileContentState::NoFileOrImageData) {
+        WebCorePasteboardFileReader reader(context);
         m_pasteboard->read(reader);
         files = WTFMove(reader.files);
         addedFilesFromPasteboard = !files.isEmpty();
@@ -368,7 +378,7 @@ Vector<Ref<File>> DataTransfer::filesFromPasteboardAndItemList() const
     return files;
 }
 
-FileList& DataTransfer::files() const
+FileList& DataTransfer::files(Document* document) const
 {
     if (!canReadData()) {
         if (m_fileList)
@@ -379,11 +389,16 @@ FileList& DataTransfer::files() const
     }
 
     if (!m_fileList)
-        m_fileList = FileList::create(filesFromPasteboardAndItemList());
+        m_fileList = FileList::create(filesFromPasteboardAndItemList(document));
 
     return *m_fileList;
 }
 
+FileList& DataTransfer::files(Document& document) const
+{
+    return files(&document);
+}
+
 struct PasteboardFileTypeReader final : PasteboardFileReader {
     void readFilename(const String& filename)
     {
@@ -416,7 +431,7 @@ bool DataTransfer::hasStringOfType(const String& type)
 Ref<DataTransfer> DataTransfer::createForInputEvent(const String& plainText, const String& htmlText)
 {
     auto pasteboard = makeUnique<StaticPasteboard>();
-    pasteboard->writeString("text/plain"_s, plainText);
+    pasteboard->writeString(textPlainContentTypeAtom(), plainText);
     pasteboard->writeString("text/html"_s, htmlText);
     return adoptRef(*new DataTransfer(StoreMode::Readonly, WTFMove(pasteboard), Type::InputEvent));
 }
@@ -424,16 +439,24 @@ Ref<DataTransfer> DataTransfer::createForInputEvent(const String& plainText, con
 void DataTransfer::commitToPasteboard(Pasteboard& nativePasteboard)
 {
     ASSERT(is<StaticPasteboard>(*m_pasteboard) && !is<StaticPasteboard>(nativePasteboard));
-    PasteboardCustomData customData = downcast<StaticPasteboard>(*m_pasteboard).takeCustomData();
-    if (!customData.hasData())
+    auto& staticPasteboard = downcast<StaticPasteboard>(*m_pasteboard);
+    if (!staticPasteboard.hasNonDefaultData()) {
+        // We clear the platform pasteboard here to ensure that the pasteboard doesn't contain any data
+        // that may have been written before starting the drag or copying, and after ending the last
+        // drag session or paste. After pushing the static pasteboard's contents to the platform, the
+        // pasteboard should only contain data that was in the static pasteboard.
+        nativePasteboard.clear();
         return;
+    }
 
-    if (RuntimeEnabledFeatures::sharedFeatures().customPasteboardDataEnabled()) {
+    auto customData = staticPasteboard.takeCustomData();
+    if (DeprecatedGlobalSettings::customPasteboardDataEnabled()) {
         customData.setOrigin(m_originIdentifier);
         nativePasteboard.writeCustomData({ customData });
         return;
     }
 
+    nativePasteboard.clear();
     customData.forEachPlatformString([&] (auto& type, auto& string) {
         nativePasteboard.writeString(type, string);
     });
@@ -447,7 +470,7 @@ void DataTransfer::commitToPasteboard(Pasteboard& nativePasteboard)
 
 String DataTransfer::dropEffect() const
 {
-    return "none"_s;
+    return noneAtom();
 }
 
 void DataTransfer::setDropEffect(const String&)
@@ -463,15 +486,15 @@ void DataTransfer::setEffectAllowed(const String&)
 {
 }
 
-void DataTransfer::setDragImage(Element*, int, int)
+void DataTransfer::setDragImage(Element&, int, int)
 {
 }
 
 #else
 
-Ref<DataTransfer> DataTransfer::createForDrag()
+Ref<DataTransfer> DataTransfer::createForDrag(const Document& document)
 {
-    return adoptRef(*new DataTransfer(StoreMode::ReadWrite, Pasteboard::createForDragAndDrop(), Type::DragAndDropData));
+    return adoptRef(*new DataTransfer(StoreMode::ReadWrite, Pasteboard::createForDragAndDrop(PagePasteboardContext::create(document.pageID())), Type::DragAndDropData));
 }
 
 Ref<DataTransfer> DataTransfer::createForDragStartEvent(const Document& document)
@@ -481,30 +504,30 @@ Ref<DataTransfer> DataTransfer::createForDragStartEvent(const Document& document
     return dataTransfer;
 }
 
-Ref<DataTransfer> DataTransfer::createForDrop(const Document& document, std::unique_ptr<Pasteboard>&& pasteboard, DragOperation sourceOperation, bool draggingFiles)
+Ref<DataTransfer> DataTransfer::createForDrop(const Document& document, std::unique_ptr<Pasteboard>&& pasteboard, OptionSet<DragOperation> sourceOperationMask, bool draggingFiles)
 {
     auto dataTransfer = adoptRef(*new DataTransfer(DataTransfer::StoreMode::Readonly, WTFMove(pasteboard), draggingFiles ? Type::DragAndDropFiles : Type::DragAndDropData));
-    dataTransfer->setSourceOperation(sourceOperation);
+    dataTransfer->setSourceOperationMask(sourceOperationMask);
     dataTransfer->m_originIdentifier = document.originIdentifierForPasteboard();
     return dataTransfer;
 }
 
-Ref<DataTransfer> DataTransfer::createForUpdatingDropTarget(const Document& document, std::unique_ptr<Pasteboard>&& pasteboard, DragOperation sourceOperation, bool draggingFiles)
+Ref<DataTransfer> DataTransfer::createForUpdatingDropTarget(const Document& document, std::unique_ptr<Pasteboard>&& pasteboard, OptionSet<DragOperation> sourceOperationMask, bool draggingFiles)
 {
     auto dataTransfer = adoptRef(*new DataTransfer(DataTransfer::StoreMode::Protected, WTFMove(pasteboard), draggingFiles ? Type::DragAndDropFiles : Type::DragAndDropData));
-    dataTransfer->setSourceOperation(sourceOperation);
+    dataTransfer->setSourceOperationMask(sourceOperationMask);
     dataTransfer->m_originIdentifier = document.originIdentifierForPasteboard();
     return dataTransfer;
 }
 
-void DataTransfer::setDragImage(Element* element, int x, int y)
+void DataTransfer::setDragImage(Element& element, int x, int y)
 {
     if (!forDrag() || !canWriteData())
         return;
 
     CachedImage* image = nullptr;
-    if (is<HTMLImageElement>(element) && !element->isConnected())
-        image = downcast<HTMLImageElement>(*element).cachedImage();
+    if (is<HTMLImageElement>(element) && !element.isConnected())
+        image = downcast<HTMLImageElement>(element).cachedImage();
 
     m_dragLocation = IntPoint(x, y);
 
@@ -517,7 +540,7 @@ void DataTransfer::setDragImage(Element* element, int x, int y)
         m_dragImageLoader->startLoading(m_dragImage);
     }
 
-    m_dragImageElement = image ? nullptr : element;
+    m_dragImageElement = image ? nullptr : &element;
 
     updateDragImage();
 }
@@ -588,79 +611,79 @@ void DragImageLoader::imageChanged(CachedImage*, const IntRect*)
     m_dataTransfer->updateDragImage();
 }
 
-static DragOperation dragOpFromIEOp(const String& operation)
+static OptionSet<DragOperation> dragOpFromIEOp(const String& operation)
 {
-    if (operation == "uninitialized")
-        return DragOperationEvery;
-    if (operation == "none")
-        return DragOperationNone;
-    if (operation == "copy")
-        return DragOperationCopy;
-    if (operation == "link")
-        return DragOperationLink;
-    if (operation == "move")
-        return (DragOperation)(DragOperationGeneric | DragOperationMove);
-    if (operation == "copyLink")
-        return (DragOperation)(DragOperationCopy | DragOperationLink);
-    if (operation == "copyMove")
-        return (DragOperation)(DragOperationCopy | DragOperationGeneric | DragOperationMove);
-    if (operation == "linkMove")
-        return (DragOperation)(DragOperationLink | DragOperationGeneric | DragOperationMove);
-    if (operation == "all")
-        return DragOperationEvery;
-    return DragOperationPrivate; // really a marker for "no conversion"
+    if (operation == "uninitialized"_s)
+        return anyDragOperation();
+    if (operation == "none"_s)
+        return { };
+    if (operation == "copy"_s)
+        return { DragOperation::Copy };
+    if (operation == "link"_s)
+        return { DragOperation::Link };
+    if (operation == "move"_s)
+        return { DragOperation::Generic, DragOperation::Move };
+    if (operation == "copyLink"_s)
+        return { DragOperation::Copy, DragOperation::Link };
+    if (operation == "copyMove"_s)
+        return { DragOperation::Copy, DragOperation::Generic, DragOperation::Move };
+    if (operation == "linkMove"_s)
+        return { DragOperation::Link, DragOperation::Generic, DragOperation::Move };
+    if (operation == "all"_s)
+        return anyDragOperation();
+    return { DragOperation::Private }; // Really a marker for "no conversion".
 }
 
-static const char* IEOpFromDragOp(DragOperation operation)
+static ASCIILiteral IEOpFromDragOp(OptionSet<DragOperation> operationMask)
 {
-    bool isGenericMove = operation & (DragOperationGeneric | DragOperationMove);
+    bool isGenericMove = operationMask.containsAny({ DragOperation::Generic, DragOperation::Move });
 
-    if ((isGenericMove && (operation & DragOperationCopy) && (operation & DragOperationLink)) || operation == DragOperationEvery)
-        return "all";
-    if (isGenericMove && (operation & DragOperationCopy))
-        return "copyMove";
-    if (isGenericMove && (operation & DragOperationLink))
-        return "linkMove";
-    if ((operation & DragOperationCopy) && (operation & DragOperationLink))
-        return "copyLink";
+    if ((isGenericMove && operationMask.containsAll({ DragOperation::Copy, DragOperation::Link })) || operationMask.containsAll({ DragOperation::Copy, DragOperation::Link, DragOperation::Generic, DragOperation::Private, DragOperation::Move, DragOperation::Delete }))
+        return "all"_s;
+    if (isGenericMove && operationMask.contains(DragOperation::Copy))
+        return "copyMove"_s;
+    if (isGenericMove && operationMask.contains(DragOperation::Link))
+        return "linkMove"_s;
+    if (operationMask.containsAll({ DragOperation::Copy, DragOperation::Link }))
+        return "copyLink"_s;
     if (isGenericMove)
-        return "move";
-    if (operation & DragOperationCopy)
-        return "copy";
-    if (operation & DragOperationLink)
-        return "link";
-    return "none";
+        return "move"_s;
+    if (operationMask.contains(DragOperation::Copy))
+        return "copy"_s;
+    if (operationMask.contains(DragOperation::Link))
+        return "link"_s;
+    return "none"_s;
 }
 
-DragOperation DataTransfer::sourceOperation() const
+OptionSet<DragOperation> DataTransfer::sourceOperationMask() const
 {
-    DragOperation operation = dragOpFromIEOp(m_effectAllowed);
-    ASSERT(operation != DragOperationPrivate);
-    return operation;
+    auto operationMask = dragOpFromIEOp(m_effectAllowed);
+    ASSERT(operationMask != DragOperation::Private);
+    return operationMask;
 }
 
-DragOperation DataTransfer::destinationOperation() const
+OptionSet<DragOperation> DataTransfer::destinationOperationMask() const
 {
-    DragOperation operation = dragOpFromIEOp(m_dropEffect);
-    ASSERT(operation == DragOperationCopy || operation == DragOperationNone || operation == DragOperationLink || operation == (DragOperation)(DragOperationGeneric | DragOperationMove) || operation == DragOperationEvery);
-    return operation;
+    auto operationMask = dragOpFromIEOp(m_dropEffect);
+    ASSERT(operationMask == DragOperation::Copy || operationMask.isEmpty() || operationMask == DragOperation::Link || operationMask == OptionSet<DragOperation>({ DragOperation::Generic, DragOperation::Move }) || operationMask.containsAll({ DragOperation::Copy, DragOperation::Link, DragOperation::Generic, DragOperation::Private, DragOperation::Move, DragOperation::Delete }));
+    return operationMask;
 }
 
-void DataTransfer::setSourceOperation(DragOperation operation)
+void DataTransfer::setSourceOperationMask(OptionSet<DragOperation> operationMask)
 {
-    ASSERT_ARG(operation, operation != DragOperationPrivate);
-    m_effectAllowed = IEOpFromDragOp(operation);
+    ASSERT_ARG(operationMask, operationMask != DragOperation::Private);
+    m_effectAllowed = IEOpFromDragOp(operationMask);
 }
 
-void DataTransfer::setDestinationOperation(DragOperation operation)
+void DataTransfer::setDestinationOperationMask(OptionSet<DragOperation> operationMask)
 {
-    ASSERT_ARG(operation, operation == DragOperationCopy || operation == DragOperationNone || operation == DragOperationLink || operation == DragOperationGeneric || operation == DragOperationMove || operation == (DragOperation)(DragOperationGeneric | DragOperationMove));
-    m_dropEffect = IEOpFromDragOp(operation);
+    ASSERT_ARG(operationMask, operationMask == DragOperation::Copy || operationMask.isEmpty() || operationMask == DragOperation::Link || operationMask == DragOperation::Generic || operationMask == DragOperation::Move || operationMask == OptionSet<DragOperation>({ DragOperation::Generic, DragOperation::Move }));
+    m_dropEffect = IEOpFromDragOp(operationMask);
 }
 
 String DataTransfer::dropEffect() const
 {
-    return m_dropEffect == "uninitialized" ? "none"_s : m_dropEffect;
+    return m_dropEffect == "uninitialized"_s ? "none"_s : m_dropEffect;
 }
 
 void DataTransfer::setDropEffect(const String& effect)
@@ -668,7 +691,7 @@ void DataTransfer::setDropEffect(const String& effect)
     if (!forDrag())
         return;
 
-    if (effect != "none" && effect != "copy" && effect != "link" && effect != "move")
+    if (effect != "none"_s && effect != "copy"_s && effect != "link"_s && effect != "move"_s)
         return;
 
     // FIXME: The spec allows this in all circumstances. There is probably no value
@@ -690,7 +713,7 @@ void DataTransfer::setEffectAllowed(const String& effect)
         return;
 
     // Ignore any attempts to set it to an unknown value.
-    if (dragOpFromIEOp(effect) == DragOperationPrivate)
+    if (dragOpFromIEOp(effect) == DragOperation::Private)
         return;
 
     if (!canWriteData())
@@ -702,11 +725,6 @@ void DataTransfer::setEffectAllowed(const String& effect)
 void DataTransfer::moveDragState(Ref<DataTransfer>&& other)
 {
     RELEASE_ASSERT(is<StaticPasteboard>(other->pasteboard()));
-    // We clear the platform pasteboard here to ensure that the pasteboard doesn't contain any data
-    // that may have been written before starting the drag, and after ending the last drag session.
-    // After pushing the static pasteboard's contents to the platform, the pasteboard should only
-    // contain data that was in the static pasteboard.
-    m_pasteboard->clear();
     other->commitToPasteboard(*m_pasteboard);
 
     m_dropEffect = other->m_dropEffect;
