@@ -80,7 +80,7 @@ static bool canCacheFrame(Frame& frame, DiagnosticLoggingClient& diagnosticLoggi
 
     // Prevent page caching if a subframe is still in provisional load stage.
     // We only do this check for subframes because the main frame is reused when navigating to a new page.
-    if (!frame.isMainFrame() && frameLoader.state() == FrameStateProvisional) {
+    if (!frame.isMainFrame() && frameLoader.state() == FrameState::Provisional) {
         PCLOG("   -Frame is in provisional load stage");
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::provisionalLoadKey());
         return false;
@@ -135,12 +135,7 @@ static bool canCacheFrame(Frame& frame, DiagnosticLoggingClient& diagnosticLoggi
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::isErrorPageKey());
         isCacheable = false;
     }
-    if (frameLoader.subframeLoader().containsPlugins() && !frame.page()->settings().backForwardCacheSupportsPlugins()) {
-        PCLOG("   -Frame contains plugins");
-        logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::hasPluginsKey());
-        isCacheable = false;
-    }
-    if (frame.isMainFrame() && frame.document() && frame.document()->url().protocolIs("https") && documentLoader->response().cacheControlContainsNoStore()) {
+    if (frame.isMainFrame() && frame.document() && frame.document()->url().protocolIs("https"_s) && documentLoader->response().cacheControlContainsNoStore()) {
         PCLOG("   -Frame is HTTPS, and cache control prohibits storing");
         logBackForwardCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::httpsNoStoreKey());
         isCacheable = false;
@@ -290,7 +285,7 @@ BackForwardCache::BackForwardCache()
 {
     static std::once_flag onceFlag;
     std::call_once(onceFlag, [] {
-        PAL::registerNotifyCallback("com.apple.WebKit.showBackForwardCache", [] {
+        PAL::registerNotifyCallback("com.apple.WebKit.showBackForwardCache"_s, [] {
             BackForwardCache::singleton().dump();
         });
     });
@@ -322,7 +317,7 @@ bool BackForwardCache::canCache(Page& page) const
 
 void BackForwardCache::pruneToSizeNow(unsigned size, PruningReason pruningReason)
 {
-    SetForScope<unsigned> change(m_maxSize, size);
+    SetForScope change(m_maxSize, size);
     prune(pruningReason);
 }
 
@@ -389,10 +384,9 @@ static String pruningReasonToDiagnosticLoggingKey(PruningReason pruningReason)
 
 static void setBackForwardCacheState(Page& page, Document::BackForwardCacheState BackForwardCacheState)
 {
-    for (Frame* frame = &page.mainFrame(); frame; frame = frame->tree().traverseNext()) {
-        if (auto* document = frame->document())
-            document->setBackForwardCacheState(BackForwardCacheState);
-    }
+    page.forEachDocument([&] (Document& document) {
+        document.setBackForwardCacheState(BackForwardCacheState);
+    });
 }
 
 // When entering back/forward cache, tear down the render tree before setting the in-cache flag.
@@ -421,12 +415,51 @@ static void firePageHideEventRecursively(Frame& frame)
     // https://html.spec.whatwg.org/multipage/browsers.html#unload-a-document
     IgnoreOpensDuringUnloadCountIncrementer ignoreOpensDuringUnloadCountIncrementer(document);
 
-    frame.loader().stopLoading(UnloadEventPolicyUnloadAndPageHide);
+    frame.loader().stopLoading(UnloadEventPolicy::UnloadAndPageHide);
 
     for (RefPtr<Frame> child = frame.tree().firstChild(); child; child = child->tree().nextSibling())
         firePageHideEventRecursively(*child);
 }
 
+std::unique_ptr<CachedPage> BackForwardCache::trySuspendPage(Page& page, ForceSuspension forceSuspension)
+{
+    page.mainFrame().loader().stopForBackForwardCache();
+
+    if (forceSuspension == ForceSuspension::No && !canCache(page))
+        return nullptr;
+
+    ASSERT_WITH_MESSAGE(!page.isUtilityPage(), "Utility pages such as SVGImage pages should never go into BackForwardCache");
+
+    setBackForwardCacheState(page, Document::AboutToEnterBackForwardCache);
+
+    // Focus the main frame, defocusing a focused subframe (if we have one). We do this here,
+    // before the page enters the back/forward cache, while we still can dispatch DOM blur/focus events.
+    if (CheckedRef focusController { page.focusController() }; focusController->focusedFrame())
+        focusController->setFocusedFrame(&page.mainFrame());
+
+    // Fire the pagehide event in all frames.
+    firePageHideEventRecursively(page.mainFrame());
+
+    destroyRenderTree(page.mainFrame());
+
+    // Stop all loads again before checking if we can still cache the page after firing the pagehide
+    // event, since the page may have started ping loads in its pagehide event handler.
+    page.mainFrame().loader().stopForBackForwardCache();
+
+    // Check that the page is still page-cacheable after firing the pagehide event. The JS event handlers
+    // could have altered the page in a way that could prevent caching.
+    if (forceSuspension == ForceSuspension::No && !canCache(page)) {
+        setBackForwardCacheState(page, Document::NotInBackForwardCache);
+        return nullptr;
+    }
+
+    setBackForwardCacheState(page, Document::InBackForwardCache);
+
+    // Make sure we don't fire any JS events in this scope.
+    ScriptDisallowedScope::InMainThread scriptDisallowedScope;
+    return makeUnique<CachedPage>(page);
+}
+
 bool BackForwardCache::addIfCacheable(HistoryItem& item, Page* page)
 {
     if (item.isInBackForwardCache())
@@ -435,43 +468,15 @@ bool BackForwardCache::addIfCacheable(HistoryItem& item, Page* page)
     if (!page)
         return false;
 
-    page->mainFrame().loader().stopForBackForwardCache();
-
-    if (!canCache(*page))
+    auto cachedPage = trySuspendPage(*page, ForceSuspension::No);
+    if (!cachedPage)
         return false;
 
-    ASSERT_WITH_MESSAGE(!page->isUtilityPage(), "Utility pages such as SVGImage pages should never go into BackForwardCache");
-
-    setBackForwardCacheState(*page, Document::AboutToEnterBackForwardCache);
-
-    // Focus the main frame, defocusing a focused subframe (if we have one). We do this here,
-    // before the page enters the back/forward cache, while we still can dispatch DOM blur/focus events.
-    if (page->focusController().focusedFrame())
-        page->focusController().setFocusedFrame(&page->mainFrame());
-
-    // Fire the pagehide event in all frames.
-    firePageHideEventRecursively(page->mainFrame());
-
-    destroyRenderTree(page->mainFrame());
-
-    // Stop all loads again before checking if we can still cache the page after firing the pagehide
-    // event, since the page may have started ping loads in its pagehide event handler.
-    page->mainFrame().loader().stopForBackForwardCache();
-
-    // Check that the page is still page-cacheable after firing the pagehide event. The JS event handlers
-    // could have altered the page in a way that could prevent caching.
-    if (!canCache(*page)) {
-        setBackForwardCacheState(*page, Document::NotInBackForwardCache);
-        return false;
-    }
-
-    setBackForwardCacheState(*page, Document::InBackForwardCache);
-
     {
         // Make sure we don't fire any JS events in this scope.
         ScriptDisallowedScope::InMainThread scriptDisallowedScope;
 
-        item.setCachedPage(makeUnique<CachedPage>(*page));
+        item.setCachedPage(WTFMove(cachedPage));
         item.m_pruningReason = PruningReason::None;
         m_items.add(&item);
     }
@@ -482,6 +487,13 @@ bool BackForwardCache::addIfCacheable(HistoryItem& item, Page* page)
     return true;
 }
 
+std::unique_ptr<CachedPage> BackForwardCache::suspendPage(Page& page)
+{
+    RELEASE_LOG(BackForwardCache, "BackForwardCache::suspendPage()");
+
+    return trySuspendPage(page, ForceSuspension::Yes);
+}
+
 std::unique_ptr<CachedPage> BackForwardCache::take(HistoryItem& item, Page* page)
 {
     if (!item.m_cachedPage) {
@@ -508,7 +520,7 @@ void BackForwardCache::removeAllItemsForPage(Page& page)
 {
 #if ASSERT_ENABLED
     ASSERT_WITH_MESSAGE(!m_isInRemoveAllItemsForPage, "We should not reenter this method");
-    SetForScope<bool> inRemoveAllItemsForPageScope { m_isInRemoveAllItemsForPage, true };
+    SetForScope inRemoveAllItemsForPageScope { m_isInRemoveAllItemsForPage, true };
 #endif
 
     for (auto it = m_items.begin(); it != m_items.end();) {
