@@ -47,11 +47,17 @@ public:
 
     void queueCancelTask();
 
+    void runFocusingSteps();
+
 private:
     HTMLDialogElement(const QualifiedName&, Document&);
 
+    void removedFromAncestor(RemovalType, ContainerNode& oldParentOfRemovedTree) final;
+    void setIsModal(bool newValue);
+
     String m_returnValue;
     bool m_isModal { false };
+    WeakPtr<Element> m_previouslyFocusedElement;
 };
 
 } // namespace WebCore
