@@ -32,6 +32,7 @@
 #include "FormAssociatedElement.h"
 #include "HTMLFormControlElement.h"
 #include "HTMLNames.h"
+#include "SelectionRestorationMode.h"
 #include <wtf/IsoMallocInlines.h>
 
 namespace WebCore {
@@ -42,7 +43,7 @@ using namespace HTMLNames;
 
 static LabelableElement* firstElementWithIdIfLabelable(TreeScope& treeScope, const AtomString& id)
 {
-    auto element = makeRefPtr(treeScope.getElementById(id));
+    RefPtr element = treeScope.getElementById(id);
     if (!is<LabelableElement>(element))
         return nullptr;
 
@@ -85,26 +86,26 @@ HTMLFormElement* HTMLLabelElement::form() const
     return downcast<HTMLFormControlElement>(control.get())->form();
 }
 
-void HTMLLabelElement::setActive(bool down, bool pause)
+void HTMLLabelElement::setActive(bool down, Style::InvalidationScope invalidationScope)
 {
     if (down == active())
         return;
 
     // Update our status first.
-    HTMLElement::setActive(down, pause);
+    HTMLElement::setActive(down, invalidationScope);
 
     // Also update our corresponding control.
     if (auto element = control())
-        element->setActive(down, pause);
+        element->setActive(down);
 }
 
-void HTMLLabelElement::setHovered(bool over)
+void HTMLLabelElement::setHovered(bool over, Style::InvalidationScope invalidationScope, HitTestRequest request)
 {
     if (over == hovered())
         return;
         
     // Update our status first.
-    HTMLElement::setHovered(over);
+    HTMLElement::setHovered(over, invalidationScope, request);
 
     // Also update our corresponding control.
     if (auto element = control())
@@ -156,7 +157,7 @@ void HTMLLabelElement::defaultEventHandler(Event& event)
 
         document().updateLayoutIgnorePendingStylesheets();
         if (control->isMouseFocusable())
-            control->focus();
+            control->focus({ { }, { }, { }, FocusTrigger::Click, { } });
 
         processingClick = false;
 
@@ -166,27 +167,27 @@ void HTMLLabelElement::defaultEventHandler(Event& event)
     HTMLElement::defaultEventHandler(event);
 }
 
-bool HTMLLabelElement::willRespondToMouseClickEvents()
+bool HTMLLabelElement::willRespondToMouseClickEventsWithEditability(Editability editability) const
 {
     auto element = control();
-    return (element && element->willRespondToMouseClickEvents()) || HTMLElement::willRespondToMouseClickEvents();
+    return (element && element->willRespondToMouseClickEventsWithEditability(editability)) || HTMLElement::willRespondToMouseClickEventsWithEditability(editability);
 }
 
-void HTMLLabelElement::focus(bool restorePreviousSelection, FocusDirection direction)
+void HTMLLabelElement::focus(const FocusOptions& options)
 {
     Ref<HTMLLabelElement> protectedThis(*this);
     if (document().haveStylesheetsLoaded()) {
         document().updateLayout();
         if (isFocusable()) {
-            // The value of restorePreviousSelection is not used for label elements as it doesn't override updateFocusAppearance.
-            Element::focus(restorePreviousSelection, direction);
+            // The value of restorationMode is not used for label elements as it doesn't override updateFocusAppearance.
+            Element::focus(options);
             return;
         }
     }
 
     // To match other browsers, always restore previous selection.
     if (auto element = control())
-        element->focus(true, direction);
+        element->focus({ SelectionRestorationMode::RestoreOrSelectAll, options.direction });
 }
 
 bool HTMLLabelElement::accessKeyAction(bool sendMouseEvents)
@@ -197,4 +198,28 @@ bool HTMLLabelElement::accessKeyAction(bool sendMouseEvents)
     return HTMLElement::accessKeyAction(sendMouseEvents);
 }
 
+auto HTMLLabelElement::insertedIntoAncestor(InsertionType insertionType, ContainerNode& parentOfInsertedTree) -> InsertedIntoAncestorResult
+{
+    auto result = HTMLElement::insertedIntoAncestor(insertionType, parentOfInsertedTree);
+
+    if (parentOfInsertedTree.isInTreeScope() && insertionType.treeScopeChanged) {
+        auto& newScope = parentOfInsertedTree.treeScope();
+        if (newScope.shouldCacheLabelsByForAttribute())
+            updateLabel(newScope, nullAtom(), attributeWithoutSynchronization(forAttr));
+    }
+
+    return result;
+}
+
+void HTMLLabelElement::removedFromAncestor(RemovalType removalType, ContainerNode& oldParentOfRemovedTree)
+{
+    if (oldParentOfRemovedTree.isInTreeScope() && removalType.treeScopeChanged) {
+        auto& oldScope = oldParentOfRemovedTree.treeScope();
+        if (oldScope.shouldCacheLabelsByForAttribute())
+            updateLabel(oldScope, attributeWithoutSynchronization(forAttr), nullAtom());
+    }
+
+    HTMLElement::removedFromAncestor(removalType, oldParentOfRemovedTree);
+}
+
 } // namespace
