@@ -111,13 +111,14 @@ int HTMLSummaryElement::defaultTabIndex() const
 
 bool HTMLSummaryElement::supportsFocus() const
 {
-    return isActiveSummary();
+    return isActiveSummary() || HTMLElement::supportsFocus();
 }
 
 void HTMLSummaryElement::defaultEventHandler(Event& event)
 {
     if (isActiveSummary() && renderer()) {
-        if (event.type() == eventNames().DOMActivateEvent && !isClickableControl(event.target())) {
+        auto& eventNames = WebCore::eventNames();
+        if (event.type() == eventNames.DOMActivateEvent && !isClickableControl(event.target())) {
             if (RefPtr<HTMLDetailsElement> details = detailsElement())
                 details->toggleOpen();
             event.setDefaultHandled();
@@ -126,12 +127,12 @@ void HTMLSummaryElement::defaultEventHandler(Event& event)
 
         if (is<KeyboardEvent>(event)) {
             KeyboardEvent& keyboardEvent = downcast<KeyboardEvent>(event);
-            if (keyboardEvent.type() == eventNames().keydownEvent && keyboardEvent.keyIdentifier() == "U+0020") {
-                setActive(true, true);
+            if (keyboardEvent.type() == eventNames.keydownEvent && keyboardEvent.keyIdentifier() == "U+0020"_s) {
+                setActive(true);
                 // No setDefaultHandled() - IE dispatches a keypress in this case.
                 return;
             }
-            if (keyboardEvent.type() == eventNames().keypressEvent) {
+            if (keyboardEvent.type() == eventNames.keypressEvent) {
                 switch (keyboardEvent.charCode()) {
                 case '\r':
                     dispatchSimulatedClick(&event);
@@ -143,7 +144,7 @@ void HTMLSummaryElement::defaultEventHandler(Event& event)
                     return;
                 }
             }
-            if (keyboardEvent.type() == eventNames().keyupEvent && keyboardEvent.keyIdentifier() == "U+0020") {
+            if (keyboardEvent.type() == eventNames.keyupEvent && keyboardEvent.keyIdentifier() == "U+0020"_s) {
                 if (active())
                     dispatchSimulatedClick(&event);
                 keyboardEvent.setDefaultHandled();
@@ -155,12 +156,12 @@ void HTMLSummaryElement::defaultEventHandler(Event& event)
     HTMLElement::defaultEventHandler(event);
 }
 
-bool HTMLSummaryElement::willRespondToMouseClickEvents()
+bool HTMLSummaryElement::willRespondToMouseClickEventsWithEditability(Editability editability) const
 {
     if (isActiveSummary() && renderer())
         return true;
 
-    return HTMLElement::willRespondToMouseClickEvents();
+    return HTMLElement::willRespondToMouseClickEventsWithEditability(editability);
 }
 
 }
