@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Apple Inc. All rights reserved.
+ * Copyright (C) 2009-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,24 +30,33 @@
 
 #include "InspectorInstrumentation.h"
 #include "ScriptExecutionContext.h"
+#include "WebCoreOpaqueRoot.h"
 #include "WebGLContextGroup.h"
 #include "WebGLRenderingContextBase.h"
 #include "WebGLShader.h"
+#include <JavaScriptCore/SlotVisitor.h>
+#include <JavaScriptCore/SlotVisitorInlines.h>
 #include <wtf/Lock.h>
+#include <wtf/Locker.h>
 #include <wtf/NeverDestroyed.h>
 
+#if !USE(ANGLE)
+#include "GraphicsContextGLOpenGL.h"
+#endif
+
 namespace WebCore {
 
-HashMap<WebGLProgram*, WebGLRenderingContextBase*>& WebGLProgram::instances(const LockHolder&)
+Lock WebGLProgram::s_instancesLock;
+
+HashMap<WebGLProgram*, WebGLRenderingContextBase*>& WebGLProgram::instances()
 {
     static NeverDestroyed<HashMap<WebGLProgram*, WebGLRenderingContextBase*>> instances;
     return instances;
 }
 
-Lock& WebGLProgram::instancesMutex()
+Lock& WebGLProgram::instancesLock()
 {
-    static Lock mutex;
-    return mutex;
+    return s_instancesLock;
 }
 
 Ref<WebGLProgram> WebGLProgram::create(WebGLRenderingContextBase& ctx)
@@ -62,8 +71,8 @@ WebGLProgram::WebGLProgram(WebGLRenderingContextBase& ctx)
     ASSERT(scriptExecutionContext());
 
     {
-        LockHolder lock(instancesMutex());
-        instances(lock).add(this, &ctx);
+        Locker locker { instancesLock() };
+        instances().add(this, &ctx);
     }
 
     setObject(ctx.graphicsContextGL()->createProgram());
@@ -73,13 +82,16 @@ WebGLProgram::~WebGLProgram()
 {
     InspectorInstrumentation::willDestroyWebGLProgram(*this);
 
-    deleteObject(0);
-
     {
-        LockHolder lock(instancesMutex());
-        ASSERT(instances(lock).contains(this));
-        instances(lock).remove(this);
+        Locker locker { instancesLock() };
+        ASSERT(instances().contains(this));
+        instances().remove(this);
     }
+
+    if (!hasGroupOrContext())
+        return;
+
+    runDestructor();
 }
 
 void WebGLProgram::contextDestroyed()
@@ -89,15 +101,15 @@ void WebGLProgram::contextDestroyed()
     ContextDestructionObserver::contextDestroyed();
 }
 
-void WebGLProgram::deleteObjectImpl(GraphicsContextGLOpenGL* context3d, PlatformGLObject obj)
+void WebGLProgram::deleteObjectImpl(const AbstractLocker& locker, GraphicsContextGL* context3d, PlatformGLObject obj)
 {
     context3d->deleteProgram(obj);
     if (m_vertexShader) {
-        m_vertexShader->onDetached(context3d);
+        m_vertexShader->onDetached(locker, context3d);
         m_vertexShader = nullptr;
     }
     if (m_fragmentShader) {
-        m_fragmentShader->onDetached(context3d);
+        m_fragmentShader->onDetached(locker, context3d);
         m_fragmentShader = nullptr;
     }
 }
@@ -156,7 +168,7 @@ WebGLShader* WebGLProgram::getAttachedShader(GCGLenum type)
     }
 }
 
-bool WebGLProgram::attachShader(WebGLShader* shader)
+bool WebGLProgram::attachShader(const AbstractLocker&, WebGLShader* shader)
 {
     if (!shader || !shader->object())
         return false;
@@ -176,7 +188,7 @@ bool WebGLProgram::attachShader(WebGLShader* shader)
     }
 }
 
-bool WebGLProgram::detachShader(WebGLShader* shader)
+bool WebGLProgram::detachShader(const AbstractLocker&, WebGLShader* shader)
 {
     if (!shader || !shader->object())
         return false;
@@ -196,16 +208,25 @@ bool WebGLProgram::detachShader(WebGLShader* shader)
     }
 }
 
-void WebGLProgram::cacheActiveAttribLocations(GraphicsContextGLOpenGL* context3d)
+void WebGLProgram::addMembersToOpaqueRoots(const AbstractLocker&, JSC::AbstractSlotVisitor& visitor)
+{
+    addWebCoreOpaqueRoot(visitor, m_vertexShader.get());
+    addWebCoreOpaqueRoot(visitor, m_fragmentShader.get());
+}
+
+void WebGLProgram::cacheActiveAttribLocations(GraphicsContextGL* context3d)
 {
     m_activeAttribLocations.clear();
 
-    GCGLint numAttribs = 0;
-    context3d->getProgramiv(object(), GraphicsContextGL::ACTIVE_ATTRIBUTES, &numAttribs);
+    GCGLint numAttribs = context3d->getProgrami(object(), GraphicsContextGL::ACTIVE_ATTRIBUTES);
     m_activeAttribLocations.resize(static_cast<size_t>(numAttribs));
     for (int i = 0; i < numAttribs; ++i) {
         GraphicsContextGL::ActiveInfo info;
-        context3d->getActiveAttribImpl(object(), i, info);
+#if USE(ANGLE)
+        context3d->getActiveAttrib(object(), i, info);
+#else
+        static_cast<GraphicsContextGLOpenGL*>(context3d)->getActiveAttribImpl(object(), i, info);
+#endif
         m_activeAttribLocations[i] = context3d->getAttribLocation(object(), info.name);
     }
 }
@@ -218,11 +239,10 @@ void WebGLProgram::cacheInfoIfNeeded()
     if (!object())
         return;
 
-    GraphicsContextGLOpenGL* context = getAGraphicsContextGL();
+    GraphicsContextGL* context = getAGraphicsContextGL();
     if (!context)
         return;
-    GCGLint linkStatus = 0;
-    context->getProgramiv(object(), GraphicsContextGL::LINK_STATUS, &linkStatus);
+    GCGLint linkStatus = context->getProgrami(object(), GraphicsContextGL::LINK_STATUS);
     m_linkStatus = linkStatus;
     if (m_linkStatus) {
         cacheActiveAttribLocations(context);
