@@ -31,11 +31,12 @@
  */
 
 #include "config.h"
-
-#if ENABLE(VIDEO_TRACK)
-
 #include "WebVTTParser.h"
 
+#if ENABLE(VIDEO)
+
+#include "CommonAtomStrings.h"
+#include "Document.h"
 #include "HTMLParserIdioms.h"
 #include "ISOVTTCue.h"
 #include "ProcessingInstruction.h"
@@ -49,13 +50,13 @@
 
 namespace WebCore {
 
-const double secondsPerHour = 3600;
-const double secondsPerMinute = 60;
-const double secondsPerMillisecond = 0.001;
-const char* fileIdentifier = "WEBVTT";
-const unsigned fileIdentifierLength = 6;
-const unsigned regionIdentifierLength = 6;
-const unsigned styleIdentifierLength = 5;
+constexpr double secondsPerHour = 3600;
+constexpr double secondsPerMinute = 60;
+constexpr double secondsPerMillisecond = 0.001;
+constexpr auto fileIdentifier = "WEBVTT"_s;
+constexpr unsigned fileIdentifierLength = 6;
+constexpr unsigned regionIdentifierLength = 6;
+constexpr unsigned styleIdentifierLength = 5;
 
 bool WebVTTParser::parseFloatPercentageValue(VTTScanner& valueScanner, float& percentage)
 {
@@ -90,25 +91,24 @@ bool WebVTTParser::parseFloatPercentageValuePair(VTTScanner& valueScanner, char
     return true;
 }
 
-WebVTTParser::WebVTTParser(WebVTTParserClient* client, ScriptExecutionContext* context)
-    : m_scriptExecutionContext(context)
-    , m_state(Initial)
-    , m_decoder(TextResourceDecoder::create("text/plain", UTF8Encoding()))
+WebVTTParser::WebVTTParser(WebVTTParserClient& client, Document& document)
+    : m_document(document)
+    , m_decoder(TextResourceDecoder::create(textPlainContentTypeAtom(), PAL::UTF8Encoding()))
     , m_client(client)
 {
 }
 
-void WebVTTParser::getNewCues(Vector<RefPtr<WebVTTCueData>>& outputCues)
+Vector<Ref<WebVTTCueData>> WebVTTParser::takeCues()
 {
-    outputCues = WTFMove(m_cuelist);
+    return WTFMove(m_cuelist);
 }
 
-void WebVTTParser::getNewRegions(Vector<RefPtr<VTTRegion>>& outputRegions)
+Vector<Ref<VTTRegion>> WebVTTParser::takeRegions()
 {
-    outputRegions = WTFMove(m_regionList);
+    return WTFMove(m_regionList);
 }
 
-Vector<String> WebVTTParser::getStyleSheets()
+Vector<String> WebVTTParser::takeStyleSheets()
 {
     return WTFMove(m_styleSheets);
 }
@@ -121,7 +121,7 @@ void WebVTTParser::parseFileHeader(String&& data)
     parse();
 }
 
-void WebVTTParser::parseBytes(const char* data, unsigned length)
+void WebVTTParser::parseBytes(const uint8_t* data, unsigned length)
 {
     m_lineReader.append(m_decoder->decode(data, length));
     parse();
@@ -144,8 +144,7 @@ void WebVTTParser::parseCueData(const ISOWebVTTCue& data)
         cue->setOriginalStartTime(originalStartTime);
 
     m_cuelist.append(WTFMove(cue));
-    if (m_client)
-        m_client->newCuesParsed();
+    m_client.newCuesParsed();
 }
 
 void WebVTTParser::flush()
@@ -165,8 +164,7 @@ void WebVTTParser::parse()
         case Initial:
             // Steps 4 - 9 - Check for a valid WebVTT signature.
             if (!hasRequiredFileIdentifier(*line)) {
-                if (m_client)
-                    m_client->fileFailedToParse();
+                m_client.fileFailedToParse();
                 return;
             }
 
@@ -229,7 +227,8 @@ void WebVTTParser::parse()
 void WebVTTParser::fileFinished()
 {
     ASSERT(m_state != Finished);
-    parseBytes("\n\n", 2);
+    constexpr uint8_t endLines[] = { '\n', '\n' };
+    parseBytes(endLines, 2);
     m_state = Finished;
 }
 
@@ -276,14 +275,12 @@ WebVTTParser::ParseState WebVTTParser::collectWebVTTBlock(const String& line)
     // Handle cue block.
     ParseState state = checkAndRecoverCue(line);
     if (state != Header) {
-        if (m_client) {
-            if (!m_regionList.isEmpty())
-                m_client->newRegionsParsed();
-            if (!m_styleSheets.isEmpty())
-                m_client->newStyleSheetsParsed();
-        }
-        if (!m_previousLine.isEmpty() && !m_previousLine.contains("-->"))
-            m_currentId = m_previousLine;
+        if (!m_regionList.isEmpty())
+            m_client.newRegionsParsed();
+        if (!m_styleSheets.isEmpty())
+            m_client.newStyleSheetsParsed();
+        if (!m_previousLine.isEmpty() && !m_previousLine.contains("-->"_s))
+            m_currentId = AtomString { m_previousLine };
         
         return state;
     }
@@ -301,7 +298,7 @@ WebVTTParser::ParseState WebVTTParser::collectWebVTTBlock(const String& line)
 WebVTTParser::ParseState WebVTTParser::checkAndRecoverCue(const String& line)
 {
     // parse cue timings and settings
-    if (line.contains("-->")) {
+    if (line.contains("-->"_s)) {
         ParseState state = recoverCue(line);
         if (state != BadCue)
             return state;
@@ -319,59 +316,55 @@ WebVTTParser::ParseState WebVTTParser::collectStyleSheet(const String& line)
     return Style;
 }
 
-bool WebVTTParser::checkAndCreateRegion(const String& line)
+bool WebVTTParser::checkAndCreateRegion(StringView line)
 {
-    if (m_previousLine.contains("-->"))
+    if (m_previousLine.contains("-->"_s))
         return false;
     // line starts with the substring "REGION" and remaining characters
     // zero or more U+0020 SPACE characters or U+0009 CHARACTER TABULATION
     // (tab) characters expected other than these charecters it is invalid.
-    if (line.startsWith("REGION") && line.substring(regionIdentifierLength).isAllSpecialCharacters<isASpace>()) {
-        m_currentRegion = VTTRegion::create(*m_scriptExecutionContext);
+    if (line.startsWith("REGION"_s) && line.substring(regionIdentifierLength).isAllSpecialCharacters<isASpace>()) {
+        m_currentRegion = VTTRegion::create(m_document);
         return true;
     }
     return false;
 }
 
-bool WebVTTParser::checkAndStoreRegion(const String& line)
+bool WebVTTParser::checkAndStoreRegion(StringView line)
 {
-    if (!line.isEmpty() && !line.contains("-->"))
+    if (!line.isEmpty() && !line.contains("-->"_s))
         return false;
-    
+
     if (!m_currentRegion->id().isEmpty()) {
-        // If the text track list of regions regions contains a region
-        // with the same region identifier value as region, remove that region.
-        for (const auto& region : m_regionList) {
-            if (region->id() == m_currentRegion->id()) {
-                m_regionList.removeFirst(region);
-                break;
-            }
-        }
-        m_regionList.append(m_currentRegion);
+        m_regionList.removeFirstMatching([this] (auto& region) {
+            return region->id() == m_currentRegion->id();
+        });
+        m_regionList.append(m_currentRegion.releaseNonNull());
     }
     m_currentRegion = nullptr;
     return true;
 }
 
-bool WebVTTParser::checkStyleSheet(const String& line)
+bool WebVTTParser::checkStyleSheet(StringView line)
 {
-    if (m_previousLine.contains("-->"))
+    if (m_previousLine.contains("-->"_s))
         return false;
     // line starts with the substring "STYLE" and remaining characters
     // zero or more U+0020 SPACE characters or U+0009 CHARACTER TABULATION
     // (tab) characters expected other than these charecters it is invalid.
-    if (line.startsWith("STYLE") && line.substring(styleIdentifierLength).isAllSpecialCharacters<isASpace>())
+    if (line.startsWith("STYLE"_s) && line.substring(styleIdentifierLength).isAllSpecialCharacters<isASpace>())
         return true;
 
     return false;
 }
 
-bool WebVTTParser::checkAndStoreStyleSheet(const String& line)
+bool WebVTTParser::checkAndStoreStyleSheet(StringView line)
 {
-    if (!line.isEmpty() && !line.contains("-->"))
+    if (!line.isEmpty() && !line.contains("-->"_s))
         return false;
     
-    auto styleSheetText = WTFMove(m_currentSourceStyleSheet);
+    auto styleSheetText = m_currentSourceStyleSheet.toString();
+    m_currentSourceStyleSheet.clear();
 
     // WebVTTMode disallows non-data URLs.
     auto contents = StyleSheetContents::create(CSSParserContext(WebVTTMode));
@@ -403,7 +396,7 @@ bool WebVTTParser::checkAndStoreStyleSheet(const String& line)
         auto selector = selectorList.selectorAt(0);
         auto selectorText = selector->selectorText();
         
-        bool isCue = selectorText == "::cue" || selectorText.startsWith("::cue(");
+        bool isCue = selectorText == "::cue"_s || selectorText.startsWith("::cue("_s);
         if (!isCue)
             return true;
 
@@ -422,9 +415,9 @@ bool WebVTTParser::checkAndStoreStyleSheet(const String& line)
 
 WebVTTParser::ParseState WebVTTParser::collectCueId(const String& line)
 {
-    if (line.contains("-->"))
+    if (line.contains("-->"_s))
         return collectTimingsAndSettings(line);
-    m_currentId = line;
+    m_currentId = AtomString { line };
     return TimingsAndSettings;
 }
 
@@ -470,7 +463,7 @@ WebVTTParser::ParseState WebVTTParser::collectCueText(const String& line)
         return Id;
     }
     // Step 35.
-    if (line.contains("-->")) {
+    if (line.contains("-->"_s)) {
         // Step 39-40.
         createNewCue();
 
@@ -497,7 +490,7 @@ WebVTTParser::ParseState WebVTTParser::ignoreBadCue(const String& line)
 {
     if (line.isEmpty())
         return Id;
-    if (line.contains("-->"))
+    if (line.contains("-->"_s))
         return recoverCue(line);
     return BadCue;
 }
@@ -527,7 +520,7 @@ Ref<DocumentFragment> WebVTTTreeBuilder::buildFromString(const String& cueText)
     auto fragment = DocumentFragment::create(m_document);
 
     if (cueText.isEmpty()) {
-        fragment->parserAppendChild(Text::create(m_document, emptyString()));
+        fragment->parserAppendChild(Text::create(m_document, String { emptyString() }));
         return fragment;
     }
 
@@ -558,13 +551,12 @@ void WebVTTParser::createNewCue()
     cue->setSettings(m_currentSettings);
 
     m_cuelist.append(WTFMove(cue));
-    if (m_client)
-        m_client->newCuesParsed();
+    m_client.newCuesParsed();
 }
 
 void WebVTTParser::resetCueValues()
 {
-    m_currentId = emptyString();
+    m_currentId = emptyAtom();
     m_currentSettings = emptyString();
     m_currentStartTime = MediaTime::zeroTime();
     m_currentEndTime = MediaTime::zeroTime();
@@ -659,7 +651,7 @@ void WebVTTTreeBuilder::constructTreeFromToken(Document& document)
 
     switch (m_token.type()) {
     case WebVTTTokenTypes::Character: {
-        m_currentNode->parserAppendChild(Text::create(document, m_token.characters()));
+        m_currentNode->parserAppendChild(Text::create(document, String { m_token.characters() }));
         break;
     }
     case WebVTTTokenTypes::StartTag: {
@@ -718,7 +710,7 @@ void WebVTTTreeBuilder::constructTreeFromToken(Document& document)
         String charactersString = m_token.characters();
         MediaTime parsedTimeStamp;
         if (WebVTTParser::collectTimeStamp(charactersString, parsedTimeStamp))
-            m_currentNode->parserAppendChild(ProcessingInstruction::create(document, "timestamp", charactersString));
+            m_currentNode->parserAppendChild(ProcessingInstruction::create(document, "timestamp"_s, WTFMove(charactersString)));
         break;
     }
     default:
