@@ -197,7 +197,7 @@ String InspectorPageAgent::sourceMapURLForResource(CachedResource* cachedResourc
     return String();
 }
 
-CachedResource* InspectorPageAgent::cachedResource(Frame* frame, const URL& url)
+CachedResource* InspectorPageAgent::cachedResource(const Frame* frame, const URL& url)
 {
     if (url.isNull())
         return nullptr;
@@ -235,6 +235,8 @@ Protocol::Page::ResourceType InspectorPageAgent::resourceTypeJSON(InspectorPageA
         return Protocol::Page::ResourceType::Beacon;
     case WebSocketResource:
         return Protocol::Page::ResourceType::WebSocket;
+    case EventSourceResource:
+        return Protocol::Page::ResourceType::EventSource;
     case OtherResource:
         return Protocol::Page::ResourceType::Other;
 #if ENABLE(APPLICATION_MANIFEST)
@@ -286,6 +288,8 @@ InspectorPageAgent::ResourceType InspectorPageAgent::inspectorResourceType(const
             return InspectorPageAgent::FetchResource;
         case ResourceRequest::Requester::Main:
             return InspectorPageAgent::DocumentResource;
+        case ResourceRequest::Requester::EventSource:
+            return InspectorPageAgent::EventSourceResource;
         default:
             return InspectorPageAgent::XHRResource;
         }
@@ -409,9 +413,10 @@ Protocol::ErrorStringOr<void> InspectorPageAgent::reload(std::optional<bool>&& i
 
 Protocol::ErrorStringOr<void> InspectorPageAgent::navigate(const String& url)
 {
-    UserGestureIndicator indicator { ProcessingUserGesture };
     Frame& frame = m_inspectedPage.mainFrame();
 
+    UserGestureIndicator indicator { ProcessingUserGesture, frame.document() };
+
     ResourceRequest resourceRequest { frame.document()->completeURL(url) };
     FrameLoadRequest frameLoadRequest { *frame.document(), frame.document()->securityOrigin(), WTFMove(resourceRequest), selfTargetFrameName(), InitiatedByMainFrame::Unknown };
     frameLoadRequest.disableNavigationToInvalidURL();
@@ -569,31 +574,31 @@ static std::optional<Cookie> parseCookieObject(Protocol::ErrorString& errorStrin
 
     cookie.name = cookieObject->getString(Protocol::Page::Cookie::nameKey);
     if (!cookie.name) {
-        errorString = "Invalid value for key name in given cookie";
+        errorString = "Invalid value for key name in given cookie"_s;
         return std::nullopt;
     }
 
     cookie.value = cookieObject->getString(Protocol::Page::Cookie::valueKey);
     if (!cookie.value) {
-        errorString = "Invalid value for key value in given cookie";
+        errorString = "Invalid value for key value in given cookie"_s;
         return std::nullopt;
     }
 
     cookie.domain = cookieObject->getString(Protocol::Page::Cookie::domainKey);
     if (!cookie.domain) {
-        errorString = "Invalid value for key domain in given cookie";
+        errorString = "Invalid value for key domain in given cookie"_s;
         return std::nullopt;
     }
 
     cookie.path = cookieObject->getString(Protocol::Page::Cookie::pathKey);
     if (!cookie.path) {
-        errorString = "Invalid value for key path in given cookie";
+        errorString = "Invalid value for key path in given cookie"_s;
         return std::nullopt;
     }
 
     auto httpOnly = cookieObject->getBoolean(Protocol::Page::Cookie::httpOnlyKey);
     if (!httpOnly) {
-        errorString = "Invalid value for key httpOnly in given cookie";
+        errorString = "Invalid value for key httpOnly in given cookie"_s;
         return std::nullopt;
     }
 
@@ -601,7 +606,7 @@ static std::optional<Cookie> parseCookieObject(Protocol::ErrorString& errorStrin
 
     auto secure = cookieObject->getBoolean(Protocol::Page::Cookie::secureKey);
     if (!secure) {
-        errorString = "Invalid value for key secure in given cookie";
+        errorString = "Invalid value for key secure in given cookie"_s;
         return std::nullopt;
     }
 
@@ -610,7 +615,7 @@ static std::optional<Cookie> parseCookieObject(Protocol::ErrorString& errorStrin
     auto session = cookieObject->getBoolean(Protocol::Page::Cookie::sessionKey);
     cookie.expires = cookieObject->getDouble(Protocol::Page::Cookie::expiresKey);
     if (!session && !cookie.expires) {
-        errorString = "Invalid value for key expires in given cookie";
+        errorString = "Invalid value for key expires in given cookie"_s;
         return std::nullopt;
     }
 
@@ -618,13 +623,13 @@ static std::optional<Cookie> parseCookieObject(Protocol::ErrorString& errorStrin
 
     auto sameSiteString = cookieObject->getString(Protocol::Page::Cookie::sameSiteKey);
     if (!sameSiteString) {
-        errorString = "Invalid value for key sameSite in given cookie";
+        errorString = "Invalid value for key sameSite in given cookie"_s;
         return std::nullopt;
     }
 
     auto sameSite = Protocol::Helpers::parseEnumValueFromString<Protocol::Page::CookieSameSitePolicy>(sameSiteString);
     if (!sameSite) {
-        errorString = "Invalid value for key sameSite in given cookie";
+        errorString = "Invalid value for key sameSite in given cookie"_s;
         return std::nullopt;
     }
 
@@ -669,7 +674,7 @@ Protocol::ErrorStringOr<void> InspectorPageAgent::deleteCookie(const String& coo
     for (Frame* frame = &m_inspectedPage.mainFrame(); frame; frame = frame->tree().traverseNext()) {
         if (auto* document = frame->document()) {
             if (auto* page = document->page())
-                page->cookieJar().deleteCookie(*document, parsedURL, cookieName);
+                page->cookieJar().deleteCookie(*document, parsedURL, cookieName, [] { });
         }
     }
 
@@ -900,7 +905,7 @@ void InspectorPageAgent::didClearWindowObjectInWorld(Frame& frame, DOMWrapperWor
     if (m_bootstrapScript.isEmpty())
         return;
 
-    frame.script().evaluateIgnoringException(ScriptSourceCode(m_bootstrapScript, URL { URL(), "web-inspector://bootstrap.js"_s }));
+    frame.script().evaluateIgnoringException(ScriptSourceCode(m_bootstrapScript, URL { "web-inspector://bootstrap.js"_str }));
 }
 
 void InspectorPageAgent::didPaint(RenderObject& renderer, const LayoutRect& rect)
