@@ -30,6 +30,7 @@
 #include "Document.h"
 #include "DocumentLoader.h"
 #include "Frame.h"
+#include "FrameDestructionObserverInlines.h"
 #include "FrameLoader.h"
 #include "HTTPCookieAcceptPolicy.h"
 #include "NetworkStorageSession.h"
@@ -55,7 +56,7 @@ Ref<CookieJar> CookieJar::create(Ref<StorageSessionProvider>&& storageSessionPro
 
 IncludeSecureCookies CookieJar::shouldIncludeSecureCookies(const Document& document, const URL& url)
 {
-    return (url.protocolIs("https") && !document.foundMixedContent().contains(SecurityContext::MixedContentType::Active)) ? IncludeSecureCookies::Yes : IncludeSecureCookies::No;
+    return (url.protocolIs("https"_s) && !document.foundMixedContent().contains(SecurityContext::MixedContentType::Active)) ? IncludeSecureCookies::Yes : IncludeSecureCookies::No;
 }
 
 SameSiteInfo CookieJar::sameSiteInfo(const Document& document, IsForDOMCookieAccess isAccessForDOM)
@@ -185,12 +186,14 @@ void CookieJar::setRawCookie(const Document&, const Cookie& cookie)
         ASSERT_NOT_REACHED();
 }
 
-void CookieJar::deleteCookie(const Document&, const URL& url, const String& cookieName)
+void CookieJar::deleteCookie(const Document&, const URL& url, const String& cookieName, CompletionHandler<void()>&& completionHandler)
 {
     if (auto* session = m_storageSessionProvider->storageSession())
-        session->deleteCookie(url, cookieName);
-    else
+        session->deleteCookie(url, cookieName, WTFMove(completionHandler));
+    else {
         ASSERT_NOT_REACHED();
+        completionHandler();
+    }
 }
 
 }
