@@ -51,9 +51,11 @@ public:
     const NetworkLoadMetrics& networkLoadMetrics() const { return m_networkLoadMetrics; }
     NetworkLoadMetrics& networkLoadMetrics() { return m_networkLoadMetrics; }
     Vector<Ref<PerformanceServerTiming>> populateServerTiming() const;
-    ResourceTiming isolatedCopy() const;
+    ResourceTiming isolatedCopy() const &;
+    ResourceTiming isolatedCopy() &&;
 
     void overrideInitiatorName(const String& name) { m_initiator = name; }
+    bool isLoadedFromServiceWorker() const { return m_isLoadedFromServiceWorker; }
 
 private:
     ResourceTiming(const URL&, const String& initiator, const ResourceLoadTiming&, const NetworkLoadMetrics&, const ResourceResponse&, const SecurityOrigin&);
@@ -71,6 +73,7 @@ private:
     ResourceLoadTiming m_resourceLoadTiming;
     NetworkLoadMetrics m_networkLoadMetrics;
     Vector<ServerTiming> m_serverTiming;
+    bool m_isLoadedFromServiceWorker { false };
 };
 
 } // namespace WebCore
