@@ -75,13 +75,14 @@ bool CachedSVGFont::ensureCustomFontData(const AtomString& remoteURI)
         {
             // We may get here during render tree updates when events are forbidden.
             // Frameless document can't run scripts or call back to the client so this is safe.
-            m_externalSVGDocument = SVGDocument::create(nullptr, m_settings, URL());
-            auto decoder = TextResourceDecoder::create("application/xml");
+            auto externalSVGDocument = SVGDocument::create(nullptr, m_settings, URL());
+            auto decoder = TextResourceDecoder::create("application/xml"_s);
 
             ScriptDisallowedScope::DisableAssertionsInScope disabledScope;
 
-            m_externalSVGDocument->setContent(decoder->decodeAndFlush(m_data->makeContiguous()->data(), m_data->size()));
+            externalSVGDocument->setContent(decoder->decodeAndFlush(m_data->makeContiguous()->data(), m_data->size()));
             sawError = decoder->sawError();
+            m_externalSVGDocument = WTFMove(externalSVGDocument);
         }
 
         if (sawError)
@@ -102,7 +103,7 @@ bool CachedSVGFont::ensureCustomFontData(const AtomString& remoteURI)
     return m_externalSVGDocument && CachedFont::ensureCustomFontData(m_convertedFont.get());
 }
 
-SVGFontElement* CachedSVGFont::getSVGFontById(const String& fontName) const
+SVGFontElement* CachedSVGFont::getSVGFontById(const AtomString& fontName) const
 {
     ASSERT(m_externalSVGDocument);
     auto elements = descendantsOfType<SVGFontElement>(*m_externalSVGDocument);
@@ -121,10 +122,10 @@ SVGFontElement* CachedSVGFont::maybeInitializeExternalSVGFontElement(const AtomS
 {
     if (m_externalSVGFontElement)
         return m_externalSVGFontElement;
-    String fragmentIdentifier;
+    AtomString fragmentIdentifier;
     size_t start = remoteURI.find('#');
     if (start != notFound)
-        fragmentIdentifier = remoteURI.string().substring(start + 1);
+        fragmentIdentifier = StringView(remoteURI).substring(start + 1).toAtomString();
     m_externalSVGFontElement = getSVGFontById(fragmentIdentifier);
     return m_externalSVGFontElement;
 }
