@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2022 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -47,6 +47,10 @@ bool PlatformMediaSessionManager::m_vorbisDecoderEnabled;
 bool PlatformMediaSessionManager::m_opusDecoderEnabled;
 #endif
 
+#if ENABLE(ALTERNATE_WEBM_PLAYER)
+bool PlatformMediaSessionManager::m_alternateWebMPlayerEnabled;
+#endif
+
 #if ENABLE(VP9)
 bool PlatformMediaSessionManager::m_vp9DecoderEnabled;
 bool PlatformMediaSessionManager::m_vp8DecoderEnabled;
@@ -156,7 +160,7 @@ void PlatformMediaSessionManager::beginInterruption(PlatformMediaSession::Interr
 {
     ALWAYS_LOG(LOGIDENTIFIER);
 
-    m_interrupted = true;
+    m_currentInterruption = type;
     forEachSession([type] (auto& session) {
         session.beginInterruption(type);
     });
@@ -167,7 +171,7 @@ void PlatformMediaSessionManager::endInterruption(PlatformMediaSession::EndInter
 {
     ALWAYS_LOG(LOGIDENTIFIER);
 
-    m_interrupted = false;
+    m_currentInterruption = { };
     forEachSession([flags] (auto& session) {
         session.endInterruption(flags);
     });
@@ -177,8 +181,8 @@ void PlatformMediaSessionManager::addSession(PlatformMediaSession& session)
 {
     ALWAYS_LOG(LOGIDENTIFIER, session.logIdentifier());
     m_sessions.append(session);
-    if (m_interrupted)
-        session.setState(PlatformMediaSession::Interrupted);
+    if (m_currentInterruption)
+        session.beginInterruption(*m_currentInterruption);
 
 #if !RELEASE_LOG_DISABLED
     m_logger->addLogger(session.logger());
@@ -243,7 +247,7 @@ bool PlatformMediaSessionManager::sessionWillBeginPlayback(PlatformMediaSession&
         return false;
     }
 
-    if (m_interrupted)
+    if (m_currentInterruption)
         endInterruption(PlatformMediaSession::NoFlags);
 
     if (restrictions & ConcurrentPlaybackNotPermitted) {
@@ -290,9 +294,14 @@ void PlatformMediaSessionManager::sessionWillEndPlayback(PlatformMediaSession& s
     ALWAYS_LOG(LOGIDENTIFIER, "session moved from index ", pausingSessionIndex, " to ", lastPlayingSessionIndex);
 }
 
-void PlatformMediaSessionManager::sessionStateChanged(PlatformMediaSession&)
+void PlatformMediaSessionManager::sessionStateChanged(PlatformMediaSession& session)
 {
-    scheduleUpdateSessionState();
+    // Call updateSessionState() synchronously if the new state is Playing to ensure
+    // the audio session is active and has the correct category before playback starts.
+    if (session.state() == PlatformMediaSession::Playing)
+        updateSessionState();
+    else
+        scheduleUpdateSessionState();
 }
 
 void PlatformMediaSessionManager::setCurrentSession(PlatformMediaSession& session)
@@ -450,7 +459,7 @@ bool PlatformMediaSessionManager::computeSupportsSeeking() const
 
 void PlatformMediaSessionManager::processSystemWillSleep()
 {
-    if (m_interrupted)
+    if (m_currentInterruption)
         return;
 
     forEachSession([] (auto& session) {
@@ -460,7 +469,7 @@ void PlatformMediaSessionManager::processSystemWillSleep()
 
 void PlatformMediaSessionManager::processSystemDidWake()
 {
-    if (m_interrupted)
+    if (m_currentInterruption)
         return;
 
     forEachSession([] (auto& session) {
@@ -690,6 +699,24 @@ void PlatformMediaSessionManager::setOpusDecoderEnabled(bool enabled)
 #endif
 }
 
+void PlatformMediaSessionManager::setAlternateWebMPlayerEnabled(bool enabled)
+{
+#if ENABLE(ALTERNATE_WEBM_PLAYER)
+    m_alternateWebMPlayerEnabled = enabled;
+#else
+    UNUSED_PARAM(enabled);
+#endif
+}
+
+bool PlatformMediaSessionManager::alternateWebMPlayerEnabled()
+{
+#if ENABLE(ALTERNATE_WEBM_PLAYER)
+    return m_alternateWebMPlayerEnabled;
+#else
+    return false;
+#endif
+}
+
 #if ENABLE(VP9)
 void PlatformMediaSessionManager::setShouldEnableVP9Decoder(bool vp9DecoderEnabled)
 {
