@@ -100,10 +100,60 @@ AudioSourceProviderGStreamer::AudioSourceProviderGStreamer(MediaStreamTrackPriva
 
     m_audioSinkBin = gst_parse_bin_from_description("tee name=audioTee", true, nullptr);
 
-    gst_bin_add_many(GST_BIN_CAST(m_pipeline.get()), src, m_audioSinkBin.get(), nullptr);
-    gst_element_link(src, m_audioSinkBin.get());
+    auto* decodebin = makeGStreamerElement("decodebin3", nullptr);
+    g_signal_connect_swapped(decodebin, "pad-added", G_CALLBACK(+[](AudioSourceProviderGStreamer* provider, GstPad* pad) {
+        auto padCaps = adoptGRef(gst_pad_query_caps(pad, nullptr));
+        bool isAudio = doCapsHaveType(padCaps.get(), "audio");
+        if (webkitGstCheckVersion(1, 18, 0))
+            RELEASE_ASSERT(isAudio);
+        else if (!isAudio)
+            return;
 
-    connectSimpleBusMessageCallback(m_pipeline.get());
+        auto sinkPad = adoptGRef(gst_element_get_static_pad(provider->m_audioSinkBin.get(), "sink"));
+        gst_pad_link(pad, sinkPad.get());
+        gst_element_sync_state_with_parent(provider->m_audioSinkBin.get());
+    }), this);
+
+    gst_bin_add_many(GST_BIN_CAST(m_pipeline.get()), src, decodebin, m_audioSinkBin.get(), nullptr);
+    gst_element_link(src, decodebin);
+
+    auto bus = adoptGRef(gst_pipeline_get_bus(GST_PIPELINE(m_pipeline.get())));
+    ASSERT(bus);
+
+    gst_bus_set_sync_handler(bus.get(), [](GstBus*, GstMessage* messageRef, gpointer userData) -> GstBusSyncReply {
+        auto message = adoptGRef(messageRef);
+        auto* decodebin = GST_ELEMENT_CAST(userData);
+        if (GST_MESSAGE_TYPE(message.get()) == GST_MESSAGE_LATENCY) {
+            auto pipeline = adoptGRef(gst_element_get_parent(decodebin));
+            gst_bin_recalculate_latency(GST_BIN_CAST(pipeline.get()));
+            return GST_BUS_DROP;
+        }
+
+        if (GST_MESSAGE_TYPE(message.get()) != GST_MESSAGE_STREAM_COLLECTION || GST_MESSAGE_SRC(message.get()) != GST_OBJECT_CAST(decodebin))
+            return GST_BUS_DROP;
+
+        GRefPtr<GstStreamCollection> collection;
+        gst_message_parse_stream_collection(message.get(), &collection.outPtr());
+        if (!collection)
+            return GST_BUS_DROP;
+
+        unsigned size = gst_stream_collection_get_size(collection.get());
+        GList* streams = nullptr;
+        for (unsigned i = 0; i < size; i++) {
+            auto* stream = gst_stream_collection_get_stream(collection.get(), i);
+            auto streamType = gst_stream_get_stream_type(stream);
+            if (streamType == GST_STREAM_TYPE_AUDIO) {
+                streams = g_list_append(streams, const_cast<char*>(gst_stream_get_stream_id(stream)));
+                break;
+            }
+        }
+        if (streams) {
+            gst_element_send_event(decodebin, gst_event_new_select_streams(streams));
+            g_list_free(streams);
+        }
+
+        return GST_BUS_DROP;
+    }, gst_object_ref(decodebin), gst_object_unref);
 }
 #endif
 
@@ -120,10 +170,8 @@ AudioSourceProviderGStreamer::~AudioSourceProviderGStreamer()
 
     setClient(nullptr);
 #if ENABLE(MEDIA_STREAM)
-    if (m_pipeline) {
-        disconnectSimpleBusMessageCallback(m_pipeline.get());
+    if (m_pipeline)
         gst_element_set_state(m_pipeline.get(), GST_STATE_NULL);
-    }
 #endif
 }
 
@@ -345,7 +393,6 @@ void AudioSourceProviderGStreamer::handleNewDeinterleavePad(GstPad* pad)
 
     GQuark quark = g_quark_from_static_string("peer");
     g_object_set_qdata(G_OBJECT(pad), quark, sinkPad.get());
-
     m_deinterleaveSourcePads++;
     GQuark channelIdQuark = g_quark_from_static_string("channel-id");
     g_object_set_qdata(G_OBJECT(sink), channelIdQuark, GINT_TO_POINTER(m_deinterleaveSourcePads));
