@@ -80,7 +80,7 @@
 #include <wtf/NeverDestroyed.h>
 #endif
 
-#if USE(ATSPI) || USE(ATK)
+#if USE(ATSPI)
 #include <wtf/glib/GUniquePtr.h>
 #endif
 
@@ -173,16 +173,21 @@ PlatformDisplay::PlatformDisplay()
 #if PLATFORM(GTK)
 PlatformDisplay::PlatformDisplay(GdkDisplay* display)
     : m_sharedDisplay(display)
+#if USE(EGL)
+    , m_eglDisplay(EGL_NO_DISPLAY)
+#endif
 {
+    if (m_sharedDisplay) {
 #if USE(ATSPI) && USE(GTK4)
-    if (const char* atspiBusAddress = static_cast<const char*>(g_object_get_data(G_OBJECT(display), "-gtk-atspi-bus-address")))
-        m_accessibilityBusAddress = String::fromUTF8(atspiBusAddress);
+        if (const char* atspiBusAddress = static_cast<const char*>(g_object_get_data(G_OBJECT(m_sharedDisplay.get()), "-gtk-atspi-bus-address")))
+            m_accessibilityBusAddress = String::fromUTF8(atspiBusAddress);
 #endif
 
-    g_signal_connect(m_sharedDisplay.get(), "closed", G_CALLBACK(+[](GdkDisplay*, gboolean, gpointer userData) {
-        auto& platformDisplay = *static_cast<PlatformDisplay*>(userData);
-        platformDisplay.sharedDisplayDidClose();
-    }), this);
+        g_signal_connect(m_sharedDisplay.get(), "closed", G_CALLBACK(+[](GdkDisplay*, gboolean, gpointer userData) {
+            auto& platformDisplay = *static_cast<PlatformDisplay*>(userData);
+            platformDisplay.sharedDisplayDidClose();
+        }), this);
+    }
 }
 
 void PlatformDisplay::sharedDisplayDidClose()
@@ -242,6 +247,13 @@ bool PlatformDisplay::eglCheckVersion(int major, int minor) const
     return (m_eglMajorVersion > major) || ((m_eglMajorVersion == major) && (m_eglMinorVersion >= minor));
 }
 
+const PlatformDisplay::EGLExtensions& PlatformDisplay::eglExtensions() const
+{
+    if (!m_eglDisplayInitialized)
+        const_cast<PlatformDisplay*>(this)->initializeEGLDisplay();
+    return m_eglExtensions;
+}
+
 void PlatformDisplay::initializeEGLDisplay()
 {
     m_eglDisplayInitialized = true;
@@ -264,6 +276,22 @@ void PlatformDisplay::initializeEGLDisplay()
     m_eglMajorVersion = majorVersion;
     m_eglMinorVersion = minorVersion;
 
+    {
+        const char* extensionsString = eglQueryString(m_eglDisplay, EGL_EXTENSIONS);
+        auto displayExtensions = StringView::fromLatin1(extensionsString).split(' ');
+        auto findExtension =
+            [&](auto extensionName) {
+                return std::any_of(displayExtensions.begin(), displayExtensions.end(),
+                    [&](auto extensionEntry) {
+                        return extensionEntry == extensionName;
+                    });
+            };
+
+        m_eglExtensions.KHR_image_base = findExtension("EGL_KHR_image_base"_s);
+        m_eglExtensions.EXT_image_dma_buf_import = findExtension("EGL_EXT_image_dma_buf_import"_s);
+        m_eglExtensions.EXT_image_dma_buf_import_modifiers = findExtension("EGL_EXT_image_dma_buf_import_modifiers"_s);
+    }
+
     eglDisplays().add(this);
 
 #if !PLATFORM(WIN)
@@ -314,7 +342,7 @@ cmsHPROFILE PlatformDisplay::colorProfile() const
 }
 #endif
 
-#if USE(ATSPI) || USE(ATK)
+#if USE(ATSPI)
 const String& PlatformDisplay::accessibilityBusAddress() const
 {
     if (m_accessibilityBusAddress)
