@@ -51,11 +51,21 @@
 #include "VideoTrackPrivateGStreamer.h"
 #include "WebKitMediaSourceGStreamer.h"
 
+#if PLATFORM(WPE)
+#include <wtf/text/StringToIntegerConversion.h>
+#endif
+
 GST_DEBUG_CATEGORY_EXTERN(webkit_mse_debug);
 #define GST_CAT_DEFAULT webkit_mse_debug
 
 namespace WebCore {
 
+bool SourceBufferPrivateGStreamer::isContentTypeSupported(const ContentType& type)
+{
+    const auto& containerType = type.containerType();
+    return containerType == "audio/mpeg"_s || containerType.endsWith("mp4"_s) || containerType.endsWith("aac"_s) || containerType.endsWith("webm"_s);
+}
+
 Ref<SourceBufferPrivateGStreamer> SourceBufferPrivateGStreamer::create(MediaSourcePrivateGStreamer* mediaSource, const ContentType& contentType, MediaPlayerPrivateGStreamerMSE& playerPrivate)
 {
     return adoptRef(*new SourceBufferPrivateGStreamer(mediaSource, contentType, playerPrivate));
@@ -116,6 +126,8 @@ void SourceBufferPrivateGStreamer::removedFromMediaSource()
     for (auto& track : m_tracks.values())
         track->remove();
     m_hasBeenRemovedFromMediaSource = true;
+
+    m_appendPipeline->stopParser();
 }
 
 MediaPlayer::ReadyState SourceBufferPrivateGStreamer::readyState() const
@@ -135,13 +147,13 @@ void SourceBufferPrivateGStreamer::flush(const AtomString& trackId)
     // This is only for on-the-fly reenqueues after appends. When seeking, the seek will do its own flush.
 
     if (!m_playerPrivate.hasAllTracks()) {
-        // Source element has not emitted tracks yet, so we only need to clear the queue.
+        GST_DEBUG("Source element has not emitted tracks yet, so we only need to clear the queue. trackId = '%s'", trackId.string().utf8().data());
         MediaSourceTrackGStreamer* track = m_tracks.get(trackId);
         track->clearQueue();
         return;
     }
 
-    // Source element has emitted tracks, let it handle the flush, which may cause a pipeline flush as well.
+    GST_DEBUG("Source element has emitted tracks, let it handle the flush, which may cause a pipeline flush as well. trackId = '%s'", trackId.string().utf8().data());
     webKitMediaSrcFlush(m_playerPrivate.webKitMediaSrc(), trackId);
 }
 
@@ -176,7 +188,7 @@ void SourceBufferPrivateGStreamer::notifyClientWhenReadyForMoreSamples(const Ato
 {
     ASSERT(isMainThread());
     MediaSourceTrackGStreamer* track = m_tracks.get(trackId);
-    track->notifyWhenReadyForMoreSamples([protector = makeRef(*this), this, trackId]() mutable {
+    track->notifyWhenReadyForMoreSamples([protector = Ref { *this }, this, trackId]() mutable {
         RunLoop::main().dispatch([protector = WTFMove(protector), this, trackId]() {
             if (!m_hasBeenRemovedFromMediaSource)
                 provideMediaData(trackId);
@@ -203,7 +215,7 @@ bool SourceBufferPrivateGStreamer::isActive() const
     return m_isActive;
 }
 
-void SourceBufferPrivateGStreamer::didReceiveInitializationSegment(SourceBufferPrivateClient::InitializationSegment&& initializationSegment, CompletionHandler<void()>&& completionHandler)
+void SourceBufferPrivateGStreamer::didReceiveInitializationSegment(SourceBufferPrivateClient::InitializationSegment&& initializationSegment, CompletionHandler<void(SourceBufferPrivateClient::ReceiveResult)>&& completionHandler)
 {
     m_hasReceivedInitializationSegment = true;
     for (auto& trackInfo : initializationSegment.videoTracks) {
@@ -273,5 +285,104 @@ WTFLogChannel& SourceBufferPrivateGStreamer::logChannel() const
 }
 #endif
 
+size_t SourceBufferPrivateGStreamer::platformMaximumBufferSize() const
+{
+#if PLATFORM(WPE)
+    if (!m_client)
+        return 0;
+
+    static size_t maxBufferSizeVideo = 0;
+    static size_t maxBufferSizeAudio = 0;
+    static size_t maxBufferSizeText = 0;
+
+    static std::once_flag once;
+    std::call_once(once, []() {
+        // Syntax: Case insensitive, full type (audio, video, text), compact type (a, v, t),
+        //         wildcard (*), unit multipliers (M=Mb, K=Kb, <empty>=bytes).
+        // Examples: MSE_MAX_BUFFER_SIZE='V:50M,audio:12k,TeXT:500K'
+        //           MSE_MAX_BUFFER_SIZE='*:100M'
+        //           MSE_MAX_BUFFER_SIZE='video:90M,T:100000'
+
+        auto s = String::fromLatin1(std::getenv("MSE_MAX_BUFFER_SIZE"));
+        if (!s.isEmpty()) {
+            Vector<String> entries = s.split(',');
+            for (const String& entry : entries) {
+                Vector<String> keyvalue = entry.split(':');
+                if (keyvalue.size() != 2)
+                    continue;
+                String key = keyvalue[0].stripWhiteSpace().convertToLowercaseWithoutLocale();
+                String value = keyvalue[1].stripWhiteSpace().convertToLowercaseWithoutLocale();
+                size_t units = 1;
+                if (value.endsWith('k'))
+                    units = 1024;
+                else if (value.endsWith('m'))
+                    units = 1024 * 1024;
+                if (units != 1)
+                    value = value.left(value.length()-1);
+                auto parsedSize = parseInteger<size_t>(value);
+                if (!parsedSize)
+                    continue;
+                size_t size = *parsedSize;
+
+                if (key == "a"_s || key == "audio"_s || key == "*"_s)
+                    maxBufferSizeAudio = size * units;
+                if (key == "v"_s || key == "video"_s || key == "*"_s)
+                    maxBufferSizeVideo = size * units;
+                if (key == "t"_s || key == "text"_s || key == "*"_s)
+                    maxBufferSizeText = size * units;
+            }
+        }
+    });
+
+    // If any track type size isn't specified, we consider that it has no limit and the values from the
+    // element have to be used. Otherwise, the track limits are accumulative.
+    do {
+        bool hasVideo = false;
+        bool hasAudio = false;
+        bool hasText = false;
+        size_t bufferSize = 0;
+
+        for (auto track : m_tracks.values()) {
+            switch (track->type()) {
+            case TrackPrivateBaseGStreamer::Video:
+                hasVideo = true;
+                break;
+            case TrackPrivateBaseGStreamer::Audio:
+                hasAudio = true;
+                break;
+            case TrackPrivateBaseGStreamer::Text:
+                hasText = true;
+                break;
+            default:
+                break;
+            }
+        }
+
+        if (hasVideo) {
+            if (maxBufferSizeVideo)
+                bufferSize += maxBufferSizeVideo;
+            else
+                break;
+        }
+        if (hasAudio) {
+            if (maxBufferSizeAudio)
+                bufferSize += maxBufferSizeAudio;
+            else
+                break;
+        }
+        if (hasText) {
+            if (maxBufferSizeText)
+                bufferSize += maxBufferSizeText;
+            else
+                break;
+        }
+        if (bufferSize)
+            return bufferSize;
+    } while (false);
+#endif
+
+    return 0;
+}
+
 }
 #endif
