@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013 Google Inc. All rights reserved.
- * Copyright (C) 2013-2018 Apple Inc.  All rights reserved.
+ * Copyright (C) 2013-2022 Apple Inc.  All rights reserved.
  * Copyright (C) 2013 Nokia Corporation and/or its subsidiary(-ies).
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,6 +47,10 @@
 #include "MockRealtimeVideoSourceMac.h"
 #endif
 
+#if USE(GSTREAMER)
+#include "MockRealtimeVideoSourceGStreamer.h"
+#endif
+
 namespace WebCore {
 
 static inline Vector<MockMediaDevice> defaultDevices()
@@ -55,6 +59,10 @@ static inline Vector<MockMediaDevice> defaultDevices()
         MockMediaDevice { "239c24b0-2b15-11e3-8224-0800200c9a66"_s, "Mock audio device 1"_s, MockMicrophoneProperties { 44100 } },
         MockMediaDevice { "239c24b1-2b15-11e3-8224-0800200c9a66"_s, "Mock audio device 2"_s, MockMicrophoneProperties { 48000 } },
 
+        MockMediaDevice { "239c24b0-2b15-11e3-8224-0800200c9a67"_s, "Mock speaker device 1"_s, MockSpeakerProperties { "239c24b0-2b15-11e3-8224-0800200c9a66"_s, 44100 } },
+        MockMediaDevice { "239c24b1-2b15-11e3-8224-0800200c9a67"_s, "Mock speaker device 2"_s, MockSpeakerProperties { "239c24b1-2b15-11e3-8224-0800200c9a66"_s, 48000 } },
+        MockMediaDevice { "239c24b2-2b15-11e3-8224-0800200c9a67"_s, "Mock speaker device 3"_s, MockSpeakerProperties { String { }, 48000 } },
+
         MockMediaDevice { "239c24b2-2b15-11e3-8224-0800200c9a66"_s, "Mock video device 1"_s,
             MockCameraProperties {
                 30,
@@ -83,23 +91,23 @@ static inline Vector<MockMediaDevice> defaultDevices()
                 Color::darkGray,
             } },
 
-        MockMediaDevice { "SCREEN-1"_s, "Mock screen device 1"_s, MockDisplayProperties { CaptureDevice::DeviceType::Screen, Color::lightGray, { 3840, 2160 } } },
-        MockMediaDevice { "SCREEN-2"_s, "Mock screen device 2"_s, MockDisplayProperties { CaptureDevice::DeviceType::Screen, Color::yellow, { 1920, 1080 } } },
+        MockMediaDevice { "SCREEN-1"_s, "Mock screen device 1"_s, MockDisplayProperties { CaptureDevice::DeviceType::Screen, Color::lightGray, { 1920, 1080 } } },
+        MockMediaDevice { "SCREEN-2"_s, "Mock screen device 2"_s, MockDisplayProperties { CaptureDevice::DeviceType::Screen, Color::yellow, { 3840, 2160 } } },
 
-        MockMediaDevice { "WINDOW-2"_s, "Mock window 1"_s, MockDisplayProperties { CaptureDevice::DeviceType::Screen, SRGBA<uint8_t> { 255, 241, 181 }, { 640, 480 } } },
-        MockMediaDevice { "WINDOW-2"_s, "Mock window 2"_s, MockDisplayProperties { CaptureDevice::DeviceType::Screen, SRGBA<uint8_t> { 255, 208, 181 }, { 1280, 600 } } },
+        MockMediaDevice { "WINDOW-1"_s, "Mock window device 1"_s, MockDisplayProperties { CaptureDevice::DeviceType::Window, SRGBA<uint8_t> { 255, 241, 181 }, { 640, 480 } } },
+        MockMediaDevice { "WINDOW-2"_s, "Mock window device 2"_s, MockDisplayProperties { CaptureDevice::DeviceType::Window, SRGBA<uint8_t> { 255, 208, 181 }, { 1280, 600 } } },
     };
 }
 
 class MockRealtimeVideoSourceFactory : public VideoCaptureFactory {
 public:
-    CaptureSourceOrError createVideoCaptureSource(const CaptureDevice& device, String&& hashSalt, const MediaConstraints* constraints) final
+    CaptureSourceOrError createVideoCaptureSource(const CaptureDevice& device, String&& hashSalt, const MediaConstraints* constraints, PageIdentifier pageIdentifier) final
     {
         ASSERT(device.type() == CaptureDevice::DeviceType::Camera);
         if (!MockRealtimeMediaSourceCenter::captureDeviceWithPersistentID(CaptureDevice::DeviceType::Camera, device.persistentId()))
             return { "Unable to find mock camera device with given persistentID"_s };
 
-        return MockRealtimeVideoSource::create(String { device.persistentId() }, String { device.label() }, WTFMove(hashSalt), constraints);
+        return MockRealtimeVideoSource::create(String { device.persistentId() }, AtomString { device.label() }, WTFMove(hashSalt), constraints, pageIdentifier);
     }
 
 private:
@@ -109,44 +117,67 @@ private:
 #if PLATFORM(MAC)
 class MockDisplayCapturer final : public DisplayCaptureSourceCocoa::Capturer {
 public:
-    explicit MockDisplayCapturer(const CaptureDevice&);
+    MockDisplayCapturer(const CaptureDevice&, PageIdentifier);
 
 private:
-    bool start(float) final;
+    bool start() final;
     void stop() final  { m_source->stop(); }
     DisplayCaptureSourceCocoa::DisplayFrameType generateFrame() final;
     RealtimeMediaSourceSettings::DisplaySurfaceType surfaceType() const final { return RealtimeMediaSourceSettings::DisplaySurfaceType::Monitor; }
-    void commitConfiguration(float) final { }
+    void commitConfiguration(const RealtimeMediaSourceSettings&) final;
     CaptureDevice::DeviceType deviceType() const final { return CaptureDevice::DeviceType::Screen; }
+    IntSize intrinsicSize() const final;
 #if !RELEASE_LOG_DISABLED
     const char* logClassName() const final { return "MockDisplayCapturer"; }
 #endif
-    
     Ref<MockRealtimeVideoSource> m_source;
+    RealtimeMediaSourceSettings m_settings;
 };
 
-MockDisplayCapturer::MockDisplayCapturer(const CaptureDevice& device)
-    : m_source(MockRealtimeVideoSourceMac::createForMockDisplayCapturer(String { device.persistentId() }, String { device.label() }, String { }))
+MockDisplayCapturer::MockDisplayCapturer(const CaptureDevice& device, PageIdentifier pageIdentifier)
+    : m_source(MockRealtimeVideoSourceMac::createForMockDisplayCapturer(String { device.persistentId() }, AtomString { device.label() }, String { }, pageIdentifier))
 {
 }
 
-bool MockDisplayCapturer::start(float)
+bool MockDisplayCapturer::start()
 {
+    ASSERT(m_settings.frameRate());
     m_source->start();
     return true;
 }
 
+void MockDisplayCapturer::commitConfiguration(const RealtimeMediaSourceSettings& settings)
+{
+    // FIXME: Update m_source width, height and frameRate according settings
+    m_settings = settings;
+}
+
 DisplayCaptureSourceCocoa::DisplayFrameType MockDisplayCapturer::generateFrame()
 {
     if (auto* imageBuffer = m_source->imageBuffer())
         return imageBuffer->copyNativeImage();
     return { };
 }
-#endif
+
+IntSize MockDisplayCapturer::intrinsicSize() const
+{
+    auto device = MockRealtimeMediaSourceCenter::mockDeviceWithPersistentID(m_source->persistentID());
+    ASSERT(device);
+    if (!device)
+        return { };
+
+    ASSERT(device->isDisplay());
+    if (!device->isDisplay())
+        return { };
+
+    auto& properties = std::get<MockDisplayProperties>(device->properties);
+    return properties.defaultSize;
+}
+#endif // PLATFORM(MAC)
 
 class MockRealtimeDisplaySourceFactory : public DisplayCaptureFactory {
 public:
-    CaptureSourceOrError createDisplayCaptureSource(const CaptureDevice& device, const MediaConstraints* constraints) final
+    CaptureSourceOrError createDisplayCaptureSource(const CaptureDevice& device, String&& hashSalt, const MediaConstraints* constraints, PageIdentifier pageIdentifier) final
     {
         if (!MockRealtimeMediaSourceCenter::captureDeviceWithPersistentID(device.type(), device.persistentId()))
             return { "Unable to find mock display device with given persistentID"_s };
@@ -155,13 +186,18 @@ public:
         case CaptureDevice::DeviceType::Screen:
         case CaptureDevice::DeviceType::Window:
 #if PLATFORM(MAC)
-            return DisplayCaptureSourceCocoa::create(UniqueRef<DisplayCaptureSourceCocoa::Capturer>(makeUniqueRef<MockDisplayCapturer>(device)), device, constraints);
+            return DisplayCaptureSourceCocoa::create(UniqueRef<DisplayCaptureSourceCocoa::Capturer>(makeUniqueRef<MockDisplayCapturer>(device, pageIdentifier)), device, WTFMove(hashSalt), constraints, pageIdentifier);
+#elif USE(GSTREAMER)
+            UNUSED_PARAM(pageIdentifier);
+            return MockDisplayCaptureSourceGStreamer::create(device, WTFMove(hashSalt), constraints);
 #else
-            return MockRealtimeVideoSource::create(String { device.persistentId() }, String { device.label() }, String { }, constraints);
+            return MockRealtimeVideoSource::create(String { device.persistentId() }, AtomString { device.label() }, WTFMove(hashSalt), constraints, pageIdentifier);
 #endif
             break;
         case CaptureDevice::DeviceType::Microphone:
+        case CaptureDevice::DeviceType::Speaker:
         case CaptureDevice::DeviceType::Camera:
+        case CaptureDevice::DeviceType::SystemAudio:
         case CaptureDevice::DeviceType::Unknown:
             ASSERT_NOT_REACHED();
             break;
@@ -169,53 +205,49 @@ public:
 
         return { };
     }
+
 private:
-    CaptureDeviceManager& displayCaptureDeviceManager() final { return MockRealtimeMediaSourceCenter::singleton().displayCaptureDeviceManager(); }
+    DisplayCaptureManager& displayCaptureDeviceManager() final { return MockRealtimeMediaSourceCenter::singleton().displayCaptureDeviceManager(); }
 };
 
 class MockRealtimeAudioSourceFactory final : public AudioCaptureFactory {
 public:
-    CaptureSourceOrError createAudioCaptureSource(const CaptureDevice& device, String&& hashSalt, const MediaConstraints* constraints) final
+    CaptureSourceOrError createAudioCaptureSource(const CaptureDevice& device, String&& hashSalt, const MediaConstraints* constraints, PageIdentifier pageIdentifier) final
     {
         ASSERT(device.type() == CaptureDevice::DeviceType::Microphone);
         if (!MockRealtimeMediaSourceCenter::captureDeviceWithPersistentID(CaptureDevice::DeviceType::Microphone, device.persistentId()))
             return { "Unable to find mock microphone device with given persistentID"_s };
 
-        return MockRealtimeAudioSource::create(String { device.persistentId() }, String { device.label() }, WTFMove(hashSalt), constraints);
+        return MockRealtimeAudioSource::create(String { device.persistentId() }, AtomString { device.label() }, WTFMove(hashSalt), constraints, pageIdentifier);
     }
 private:
-#if PLATFORM(IOS_FAMILY)
-    void setActiveSource(RealtimeMediaSource& source) final { CoreAudioCaptureSourceFactory::singleton().setActiveSource(source); }
-    void unsetActiveSource(RealtimeMediaSource& source) final { CoreAudioCaptureSourceFactory::singleton().unsetActiveSource(source); }
-    RealtimeMediaSource* activeSource() final { return CoreAudioCaptureSourceFactory::singleton().activeSource(); }
-#endif
     CaptureDeviceManager& audioCaptureDeviceManager() final { return MockRealtimeMediaSourceCenter::singleton().audioCaptureDeviceManager(); }
+    const Vector<CaptureDevice>& speakerDevices() const final { return MockRealtimeMediaSourceCenter::speakerDevices(); }
 };
 
 static Vector<MockMediaDevice>& devices()
 {
-    static auto devices = makeNeverDestroyed([] {
-        return defaultDevices();
-    }());
+    static NeverDestroyed devices = defaultDevices();
     return devices;
 }
 
 static HashMap<String, MockMediaDevice>& deviceMap()
 {
-    static auto map = makeNeverDestroyed([] {
+    static NeverDestroyed map = [] {
         HashMap<String, MockMediaDevice> map;
         for (auto& device : devices())
             map.add(device.persistentId, device);
-
         return map;
-    }());
+    }();
     return map;
 }
 
 static inline Vector<CaptureDevice>& deviceListForDevice(const MockMediaDevice& device)
 {
     if (device.isMicrophone())
-        return MockRealtimeMediaSourceCenter::audioDevices();
+        return MockRealtimeMediaSourceCenter::microphoneDevices();
+    if (device.isSpeaker())
+        return MockRealtimeMediaSourceCenter::speakerDevices();
     if (device.isCamera())
         return MockRealtimeMediaSourceCenter::videoDevices();
 
@@ -262,9 +294,17 @@ bool MockRealtimeMediaSourceCenter::mockRealtimeMediaSourceCenterEnabled()
     return singleton().m_isEnabled;
 }
 
-static void createCaptureDevice(const MockMediaDevice& device)
+static CaptureDevice toCaptureDevice(const MockMediaDevice& device)
 {
-    deviceListForDevice(device).append(MockRealtimeMediaSourceCenter::captureDeviceWithPersistentID(device.type(), device.persistentId).value());
+    auto captureDevice = device.captureDevice();
+    captureDevice.setEnabled(true);
+    captureDevice.setIsMockDevice(true);
+    return captureDevice;
+}
+
+static void createMockDevice(const MockMediaDevice& device)
+{
+    deviceListForDevice(device).append(toCaptureDevice(device));
 }
 
 void MockRealtimeMediaSourceCenter::resetDevices()
@@ -273,9 +313,16 @@ void MockRealtimeMediaSourceCenter::resetDevices()
     RealtimeMediaSourceCenter::singleton().captureDevicesChanged();
 }
 
+void MockRealtimeMediaSourceCenter::setMockCaptureDevicesInterrupted(bool isCameraInterrupted, bool isMicrophoneInterrupted)
+{
+    MockRealtimeVideoSource::setIsInterrupted(isCameraInterrupted);
+    MockRealtimeAudioSource::setIsInterrupted(isMicrophoneInterrupted);
+}
+
 void MockRealtimeMediaSourceCenter::setDevices(Vector<MockMediaDevice>&& newMockDevices)
 {
-    audioDevices().clear();
+    microphoneDevices().clear();
+    speakerDevices().clear();
     videoDevices().clear();
     displayDevices().clear();
 
@@ -287,7 +334,7 @@ void MockRealtimeMediaSourceCenter::setDevices(Vector<MockMediaDevice>&& newMock
 
     for (const auto& device : mockDevices) {
         map.add(device.persistentId, device);
-        createCaptureDevice(device);
+        createMockDevice(device);
     }
     RealtimeMediaSourceCenter::singleton().captureDevicesChanged();
 }
@@ -296,7 +343,7 @@ void MockRealtimeMediaSourceCenter::addDevice(const MockMediaDevice& device)
 {
     devices().append(device);
     deviceMap().set(device.persistentId, device);
-    createCaptureDevice(device);
+    createMockDevice(device);
     RealtimeMediaSourceCenter::singleton().captureDevicesChanged();
 }
 
@@ -319,71 +366,79 @@ void MockRealtimeMediaSourceCenter::removeDevice(const String& persistentId)
     RealtimeMediaSourceCenter::singleton().captureDevicesChanged();
 }
 
-Optional<MockMediaDevice> MockRealtimeMediaSourceCenter::mockDeviceWithPersistentID(const String& id)
+std::optional<MockMediaDevice> MockRealtimeMediaSourceCenter::mockDeviceWithPersistentID(const String& id)
 {
     ASSERT(!id.isEmpty());
 
     auto& map = deviceMap();
     auto iterator = map.find(id);
     if (iterator == map.end())
-        return WTF::nullopt;
+        return std::nullopt;
 
     return iterator->value;
 }
 
-Optional<CaptureDevice> MockRealtimeMediaSourceCenter::captureDeviceWithPersistentID(CaptureDevice::DeviceType type, const String& id)
+std::optional<CaptureDevice> MockRealtimeMediaSourceCenter::captureDeviceWithPersistentID(CaptureDevice::DeviceType type, const String& id)
 {
     ASSERT(!id.isEmpty());
 
     auto& map = deviceMap();
     auto iterator = map.find(id);
     if (iterator == map.end() || iterator->value.type() != type)
-        return WTF::nullopt;
+        return std::nullopt;
 
-    CaptureDevice device { iterator->value.persistentId, type, iterator->value.label };
-    device.setEnabled(true);
-    return device;
+    return toCaptureDevice(iterator->value);
 }
 
-Vector<CaptureDevice>& MockRealtimeMediaSourceCenter::audioDevices()
+Vector<CaptureDevice>& MockRealtimeMediaSourceCenter::microphoneDevices()
 {
-    static auto audioDevices = makeNeverDestroyed([] {
-        Vector<CaptureDevice> audioDevices;
+    static NeverDestroyed microphoneDevices = [] {
+        Vector<CaptureDevice> microphoneDevices;
         for (const auto& device : devices()) {
             if (device.isMicrophone())
-                audioDevices.append(captureDeviceWithPersistentID(CaptureDevice::DeviceType::Microphone, device.persistentId).value());
+                microphoneDevices.append(toCaptureDevice(device));
         }
-        return audioDevices;
-    }());
+        return microphoneDevices;
+    }();
+    return microphoneDevices;
+}
 
-    return audioDevices;
+Vector<CaptureDevice>& MockRealtimeMediaSourceCenter::speakerDevices()
+{
+    static NeverDestroyed speakerDevices = [] {
+        Vector<CaptureDevice> speakerDevices;
+        for (const auto& device : devices()) {
+            if (device.isSpeaker())
+                speakerDevices.append(toCaptureDevice(device));
+        }
+        return speakerDevices;
+    }();
+    return speakerDevices;
 }
 
 Vector<CaptureDevice>& MockRealtimeMediaSourceCenter::videoDevices()
 {
-    static auto videoDevices = makeNeverDestroyed([] {
+    static NeverDestroyed videoDevices = [] {
         Vector<CaptureDevice> videoDevices;
         for (const auto& device : devices()) {
             if (device.isCamera())
-                videoDevices.append(captureDeviceWithPersistentID(CaptureDevice::DeviceType::Camera, device.persistentId).value());
+                videoDevices.append(toCaptureDevice(device));
         }
         return videoDevices;
-    }());
-
+    }();
     return videoDevices;
 }
 
 Vector<CaptureDevice>& MockRealtimeMediaSourceCenter::displayDevices()
 {
-    static auto displayDevices = makeNeverDestroyed([] {
+    static NeverDestroyed displayDevices = [] {
         Vector<CaptureDevice> displayDevices;
         for (const auto& device : devices()) {
             if (device.isDisplay())
-                displayDevices.append(captureDeviceWithPersistentID(CaptureDevice::DeviceType::Screen, device.persistentId).value());
+                displayDevices.append(device.captureDevice());
         }
         return displayDevices;
-    }());
-
+    }();
     return displayDevices;
 }
 
@@ -405,6 +460,17 @@ DisplayCaptureFactory& MockRealtimeMediaSourceCenter::displayCaptureFactory()
     return factory.get();
 }
 
+void MockRealtimeMediaSourceCenter::MockDisplayCaptureDeviceManager::windowDevices(Vector<DisplayCaptureManager::WindowCaptureDevice>& windowDevices)
+{
+    auto devices = MockRealtimeMediaSourceCenter::displayDevices();
+    for (auto device : devices) {
+        if (!device.enabled() || device.type() != CaptureDevice::DeviceType::Window)
+            continue;
+
+        windowDevices.append({ WTFMove(device), "Mock Application"_s });
+    }
+}
+
 } // namespace WebCore
 
 #endif // ENABLE(MEDIA_STREAM)
