@@ -24,28 +24,19 @@
 
 #if !LOG_DISABLED || !RELEASE_LOG_DISABLED
 
-#include <string.h>
+#include <wtf/LogInitialization.h>
 #include <wtf/text/WTFString.h>
 
 namespace WebCore {
 
 String logLevelString()
 {
-#if !LOG_DISABLED
-    char* logEnv = getenv("WEBKIT_DEBUG");
-    if (!logEnv)
-        return emptyString();
-
-    // We set up the logs anyway because some of our logging, such as Soup's is available in release builds.
-#if defined(NDEBUG)
-    WTFLogAlways("WEBKIT_DEBUG is not empty, but this is a release build. Notice that many log messages will only appear in a debug build.");
-#endif
+    auto logLevel = WTF::logLevelString();
+    if (logLevel == "-all"_s)
+        return logLevel;
 
     // To disable logging notImplemented set the DISABLE_NI_WARNING environment variable to 1.
-    return String("NotYetImplemented,") + logEnv;
-#else
-    return String();
-#endif
+    return makeString("NotYetImplemented,"_s, logLevel);
 }
 
 } // namespace WebCore
