@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2005-2022 Apple Inc. All rights reserved.
  * Copyright (C) 2014 Google Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -25,6 +25,7 @@
 #include "ColorBlending.h"
 #include "ColorLuminance.h"
 #include "ControlStates.h"
+#include "DeprecatedGlobalSettings.h"
 #include "Document.h"
 #include "FileList.h"
 #include "FloatConversion.h"
@@ -48,8 +49,8 @@
 #include "RenderMeter.h"
 #include "RenderStyle.h"
 #include "RenderView.h"
-#include "RuntimeEnabledFeatures.h"
 #include "ShadowPseudoIds.h"
+#include "SliderThumbElement.h"
 #include "SpinButtonElement.h"
 #include "StringTruncator.h"
 #include "TextControlInnerElements.h"
@@ -84,36 +85,66 @@ RenderTheme::RenderTheme()
 {
 }
 
-ControlPart RenderTheme::adjustAppearanceForElement(RenderStyle& style, const Element* element) const
+ControlPart RenderTheme::adjustAppearanceForElement(RenderStyle& style, const Element* element, ControlPart autoAppearance) const
 {
-    if (!element)
+    if (!element) {
+        style.setEffectiveAppearance(NoControlPart);
         return NoControlPart;
+    }
 
     ControlPart part = style.effectiveAppearance();
-    ControlPart autoAppearance = autoAppearanceForElement(element);
     if (part == autoAppearance)
         return part;
 
     // Aliases of 'auto'.
     // https://drafts.csswg.org/css-ui-4/#typedef-appearance-compat-auto
-    if (part == AutoPart || part == SearchFieldPart || part == TextAreaPart || part == CheckboxPart || part == RadioPart || part == ListboxPart || part == MeterPart || part == ProgressBarPart) {
+    if (part == AutoPart || part == SearchFieldPart || part == TextAreaPart || part == CheckboxPart || part == RadioPart || part == ListboxPart || part == MeterPart || part == ProgressBarPart
+        || part == SquareButtonPart || part == PushButtonPart || part == SliderHorizontalPart || part == MenulistPart) {
         style.setEffectiveAppearance(autoAppearance);
         return autoAppearance;
     }
 
     // The following keywords should work well for some element types
     // even if their default appearances are different from the keywords.
-    if (part == MenulistButtonPart && autoAppearance != MenulistPart) {
+    if (part == ButtonPart) {
+        if (autoAppearance == PushButtonPart || autoAppearance == SquareButtonPart)
+            return part;
         style.setEffectiveAppearance(autoAppearance);
-        part = autoAppearance;
+        return autoAppearance;
+    }
+
+    if (part == MenulistButtonPart) {
+        if (autoAppearance == MenulistPart)
+            return part;
+        style.setEffectiveAppearance(autoAppearance);
+        return autoAppearance;
+    }
+
+    if (part == TextFieldPart) {
+        if (is<HTMLInputElement>(*element) && downcast<HTMLInputElement>(*element).isSearchField())
+            return part;
+        style.setEffectiveAppearance(autoAppearance);
+        return autoAppearance;
     }
 
     return part;
 }
 
+static bool isAppearanceAllowedForAllElements(ControlPart part)
+{
+#if ENABLE(APPLE_PAY)
+    if (part == ApplePayButtonPart)
+        return true;
+#endif
+
+    UNUSED_PARAM(part);
+    return false;
+}
+
 void RenderTheme::adjustStyle(RenderStyle& style, const Element* element, const RenderStyle* userAgentAppearanceStyle)
 {
-    auto part = adjustAppearanceForElement(style, element);
+    ControlPart autoAppearance = autoAppearanceForElement(element);
+    auto part = adjustAppearanceForElement(style, element, autoAppearance);
 
     if (part == NoControlPart)
         return;
@@ -140,12 +171,18 @@ void RenderTheme::adjustStyle(RenderStyle& style, const Element* element, const
         style.setEffectiveAppearance(part);
     }
 
+    if (!isAppearanceAllowedForAllElements(part)
+        && !userAgentAppearanceStyle
+        && autoAppearance == NoControlPart
+        && !style.borderAndBackgroundEqual(RenderStyle::defaultStyle()))
+        style.setEffectiveAppearance(NoControlPart);
+
     if (!style.hasEffectiveAppearance())
         return;
 
     if (!supportsBoxShadow(style))
         style.setBoxShadow(nullptr);
-    
+
 #if USE(NEW_THEME)
     switch (part) {
     case CheckboxPart:
@@ -258,17 +295,6 @@ void RenderTheme::adjustStyle(RenderStyle& style, const Element* element, const
         return adjustMenuListStyle(style, element);
     case MenulistButtonPart:
         return adjustMenuListButtonStyle(style, element);
-    case MediaPlayButtonPart:
-    case MediaCurrentTimePart:
-    case MediaTimeRemainingPart:
-    case MediaEnterFullscreenButtonPart:
-    case MediaExitFullscreenButtonPart:
-    case MediaMuteButtonPart:
-    case MediaVolumeSliderContainerPart:
-        return adjustMediaControlStyle(style, element);
-    case MediaSliderPart:
-    case MediaVolumeSliderPart:
-    case MediaFullScreenVolumeSliderPart:
     case SliderHorizontalPart:
     case SliderVerticalPart:
         return adjustSliderTrackStyle(style, element);
@@ -280,16 +306,14 @@ void RenderTheme::adjustStyle(RenderStyle& style, const Element* element, const
     case SearchFieldCancelButtonPart:
         return adjustSearchFieldCancelButtonStyle(style, element);
     case SearchFieldDecorationPart:
+        return adjustSearchFieldDecorationPartStyle(style, element);
     case SearchFieldResultsDecorationPart:
+        return adjustSearchFieldResultsDecorationPartStyle(style, element);
     case SearchFieldResultsButtonPart:
-        return adjustSearchFieldDecorationStyle(style, element);
+        return adjustSearchFieldResultsButtonStyle(style, element);
     case ProgressBarPart:
         return adjustProgressBarStyle(style, element);
     case MeterPart:
-    case RelevancyLevelIndicatorPart:
-    case ContinuousCapacityLevelIndicatorPart:
-    case DiscreteCapacityLevelIndicatorPart:
-    case RatingLevelIndicatorPart:
         return adjustMeterStyle(style, element);
 #if ENABLE(SERVICE_CONTROLS)
     case ImageControlsButtonPart:
@@ -301,11 +325,6 @@ void RenderTheme::adjustStyle(RenderStyle& style, const Element* element, const
     case ApplePayButtonPart:
         return adjustApplePayButtonStyle(style, element);
 #endif
-#if ENABLE(ATTACHMENT_ELEMENT)
-    case AttachmentPart:
-    case BorderlessAttachmentPart:
-        return adjustAttachmentStyle(style, element);
-#endif
 #if ENABLE(DATALIST_ELEMENT)
     case ListButtonPart:
         return adjustListButtonStyle(style, element);
@@ -324,7 +343,7 @@ ControlPart RenderTheme::autoAppearanceForElement(const Element* elementPtr) con
     if (isImageControl(*elementPtr))
         return ImageControlsButtonPart;
 #endif
-    
+
     Ref element = *elementPtr;
 
     if (is<HTMLInputElement>(element)) {
@@ -405,16 +424,21 @@ ControlPart RenderTheme::autoAppearanceForElement(const Element* elementPtr) con
         if (pseudo == ShadowPseudoIds::webkitSearchCancelButton())
             return SearchFieldCancelButtonPart;
 
-        if (pseudo == ShadowPseudoIds::webkitSearchDecoration())
-            return SearchFieldDecorationPart;
+        if (is<SearchFieldResultsButtonElement>(element)) {
+            if (!downcast<SearchFieldResultsButtonElement>(element.get()).canAdjustStyleForAppearance())
+                return NoControlPart;
 
-        if (pseudo == ShadowPseudoIds::webkitSearchResultsDecoration())
-            return SearchFieldResultsDecorationPart;
+            if (pseudo == ShadowPseudoIds::webkitSearchDecoration())
+                return SearchFieldDecorationPart;
 
-        if (pseudo == ShadowPseudoIds::webkitSearchResultsButton())
-            return SearchFieldResultsButtonPart;
+            if (pseudo == ShadowPseudoIds::webkitSearchResultsDecoration())
+                return SearchFieldResultsDecorationPart;
 
-        if (pseudo == ShadowPseudoIds::webkitSliderThumb() || pseudo == ShadowPseudoIds::webkitMediaSliderThumb())
+            if (pseudo == ShadowPseudoIds::webkitSearchResultsButton())
+                return SearchFieldResultsButtonPart;
+        }
+
+        if (pseudo == ShadowPseudoIds::webkitSliderThumb())
             return SliderThumbHorizontalPart;
 
         if (pseudo == ShadowPseudoIds::webkitInnerSpinButton())
@@ -424,25 +448,6 @@ ControlPart RenderTheme::autoAppearanceForElement(const Element* elementPtr) con
     return NoControlPart;
 }
 
-void RenderTheme::adjustSearchFieldDecorationStyle(RenderStyle& style, const Element* element) const
-{
-    if (is<SearchFieldResultsButtonElement>(element) && !downcast<SearchFieldResultsButtonElement>(*element).canAdjustStyleForAppearance()) {
-        style.setEffectiveAppearance(NoControlPart);
-        return;
-    }
-
-    switch (style.effectiveAppearance()) {
-    case SearchFieldDecorationPart:
-        return adjustSearchFieldDecorationPartStyle(style, element);
-    case SearchFieldResultsDecorationPart:
-        return adjustSearchFieldResultsDecorationPartStyle(style, element);
-    case SearchFieldResultsButtonPart:
-        return adjustSearchFieldResultsButtonStyle(style, element);
-    default:
-        break;
-    }
-}
-
 bool RenderTheme::paint(const RenderBox& box, ControlStates& controlStates, const PaintInfo& paintInfo, const LayoutRect& rect)
 {
     // If painting is disabled, but we aren't updating control tints, then just bail.
@@ -463,7 +468,7 @@ bool RenderTheme::paint(const RenderBox& box, ControlStates& controlStates, cons
     IntRect integralSnappedRect = snappedIntRect(rect);
     float deviceScaleFactor = box.document().deviceScaleFactor();
     FloatRect devicePixelSnappedRect = snapRectToDevicePixels(rect, deviceScaleFactor);
-    
+
 #if USE(NEW_THEME)
     float pageScaleFactor = box.page().pageScaleFactor();
 
@@ -510,10 +515,6 @@ bool RenderTheme::paint(const RenderBox& box, ControlStates& controlStates, cons
     case MenulistPart:
         return paintMenuList(box, paintInfo, devicePixelSnappedRect);
     case MeterPart:
-    case RelevancyLevelIndicatorPart:
-    case ContinuousCapacityLevelIndicatorPart:
-    case DiscreteCapacityLevelIndicatorPart:
-    case RatingLevelIndicatorPart:
         return paintMeter(box, paintInfo, integralSnappedRect);
     case ProgressBarPart:
         return paintProgressBar(box, paintInfo, integralSnappedRect);
@@ -523,47 +524,6 @@ bool RenderTheme::paint(const RenderBox& box, ControlStates& controlStates, cons
     case SliderThumbHorizontalPart:
     case SliderThumbVerticalPart:
         return paintSliderThumb(box, paintInfo, integralSnappedRect);
-    case MediaEnterFullscreenButtonPart:
-    case MediaExitFullscreenButtonPart:
-        return paintMediaFullscreenButton(box, paintInfo, integralSnappedRect);
-    case MediaPlayButtonPart:
-        return paintMediaPlayButton(box, paintInfo, integralSnappedRect);
-    case MediaOverlayPlayButtonPart:
-        return paintMediaOverlayPlayButton(box, paintInfo, integralSnappedRect);
-    case MediaMuteButtonPart:
-        return paintMediaMuteButton(box, paintInfo, integralSnappedRect);
-    case MediaSeekBackButtonPart:
-        return paintMediaSeekBackButton(box, paintInfo, integralSnappedRect);
-    case MediaSeekForwardButtonPart:
-        return paintMediaSeekForwardButton(box, paintInfo, integralSnappedRect);
-    case MediaRewindButtonPart:
-        return paintMediaRewindButton(box, paintInfo, integralSnappedRect);
-    case MediaReturnToRealtimeButtonPart:
-        return paintMediaReturnToRealtimeButton(box, paintInfo, integralSnappedRect);
-    case MediaToggleClosedCaptionsButtonPart:
-        return paintMediaToggleClosedCaptionsButton(box, paintInfo, integralSnappedRect);
-    case MediaSliderPart:
-        return paintMediaSliderTrack(box, paintInfo, integralSnappedRect);
-    case MediaSliderThumbPart:
-        return paintMediaSliderThumb(box, paintInfo, integralSnappedRect);
-    case MediaVolumeSliderMuteButtonPart:
-        return paintMediaMuteButton(box, paintInfo, integralSnappedRect);
-    case MediaVolumeSliderContainerPart:
-        return paintMediaVolumeSliderContainer(box, paintInfo, integralSnappedRect);
-    case MediaVolumeSliderPart:
-        return paintMediaVolumeSliderTrack(box, paintInfo, integralSnappedRect);
-    case MediaVolumeSliderThumbPart:
-        return paintMediaVolumeSliderThumb(box, paintInfo, integralSnappedRect);
-    case MediaFullScreenVolumeSliderPart:
-        return paintMediaFullScreenVolumeSliderTrack(box, paintInfo, integralSnappedRect);
-    case MediaFullScreenVolumeSliderThumbPart:
-        return paintMediaFullScreenVolumeSliderThumb(box, paintInfo, integralSnappedRect);
-    case MediaTimeRemainingPart:
-        return paintMediaTimeRemaining(box, paintInfo, integralSnappedRect);
-    case MediaCurrentTimePart:
-        return paintMediaCurrentTime(box, paintInfo, integralSnappedRect);
-    case MediaControlsBackgroundPart:
-        return paintMediaControlsBackground(box, paintInfo, integralSnappedRect);
     case MenulistButtonPart:
     case TextFieldPart:
     case TextAreaPart:
@@ -636,10 +596,6 @@ bool RenderTheme::paintBorderOnly(const RenderBox& box, const PaintInfo& paintIn
     case ButtonPart:
     case MenulistPart:
     case MeterPart:
-    case RelevancyLevelIndicatorPart:
-    case ContinuousCapacityLevelIndicatorPart:
-    case DiscreteCapacityLevelIndicatorPart:
-    case RatingLevelIndicatorPart:
     case ProgressBarPart:
     case SliderHorizontalPart:
     case SliderVerticalPart:
@@ -713,10 +669,6 @@ void RenderTheme::paintDecorations(const RenderBox& box, const PaintInfo& paintI
         paintSearchFieldDecorations(box, paintInfo, integralSnappedRect);
         break;
     case MeterPart:
-    case RelevancyLevelIndicatorPart:
-    case ContinuousCapacityLevelIndicatorPart:
-    case DiscreteCapacityLevelIndicatorPart:
-    case RatingLevelIndicatorPart:
     case ProgressBarPart:
     case SliderHorizontalPart:
     case SliderVerticalPart:
@@ -734,43 +686,6 @@ void RenderTheme::paintDecorations(const RenderBox& box, const PaintInfo& paintI
     }
 }
 
-#if ENABLE(VIDEO)
-
-String RenderTheme::formatMediaControlsTime(float time) const
-{
-    if (!std::isfinite(time))
-        time = 0;
-    // FIXME: Seems like it would be better to use std::lround here.
-    int seconds = static_cast<int>(std::abs(time));
-    int hours = seconds / (60 * 60);
-    int minutes = (seconds / 60) % 60;
-    seconds %= 60;
-    if (hours)
-        return makeString((time < 0 ? "-" : ""), hours, ':', pad('0', 2, minutes), ':', pad('0', 2, seconds));
-    return makeString((time < 0 ? "-" : ""), pad('0', 2, minutes), ':', pad('0', 2, seconds));
-}
-
-String RenderTheme::formatMediaControlsCurrentTime(float currentTime, float /*duration*/) const
-{
-    return formatMediaControlsTime(currentTime);
-}
-
-String RenderTheme::formatMediaControlsRemainingTime(float currentTime, float duration) const
-{
-    return formatMediaControlsTime(currentTime - duration);
-}
-
-LayoutPoint RenderTheme::volumeSliderOffsetFromMuteButton(const RenderBox& muteButtonBox, const LayoutSize& size) const
-{
-    LayoutUnit y = -size.height();
-    FloatPoint absPoint = muteButtonBox.localToAbsolute(FloatPoint(muteButtonBox.offsetLeft(), y), { IsFixed, UseTransforms });
-    if (absPoint.y() < 0)
-        y = muteButtonBox.height();
-    return LayoutPoint(0_lu, y);
-}
-
-#endif
-
 Color RenderTheme::activeSelectionBackgroundColor(OptionSet<StyleColorOptions> options) const
 {
     auto& cache = colorCache(options);
@@ -915,17 +830,11 @@ bool RenderTheme::isControlStyled(const RenderStyle& style, const RenderStyle& u
     case MenulistPart:
     case ProgressBarPart:
     case MeterPart:
-    case RelevancyLevelIndicatorPart:
-    case ContinuousCapacityLevelIndicatorPart:
-    case DiscreteCapacityLevelIndicatorPart:
-    case RatingLevelIndicatorPart:
     // FIXME: SearchFieldPart should be included here when making search fields style-able.
     case TextFieldPart:
     case TextAreaPart:
         // Test the style to see if the UA border and background match.
-        return style.border() != userAgentStyle.border()
-            || style.backgroundLayers() != userAgentStyle.backgroundLayers()
-            || !style.backgroundColorEqualsToColorIgnoringVisited(userAgentStyle.backgroundColor());
+        return !style.borderAndBackgroundEqual(userAgentStyle);
     default:
         return false;
     }
@@ -1025,14 +934,17 @@ bool RenderTheme::isEnabled(const RenderObject& renderer) const
 
 bool RenderTheme::isFocused(const RenderObject& renderer) const
 {
-    Node* node = renderer.node();
-    if (!is<Element>(node))
+    auto element = dynamicDowncast<Element>(renderer.node());
+    if (!element)
         return false;
 
-    auto focusDelegate = downcast<Element>(*node).focusDelegate();
-    Document& document = focusDelegate->document();
+    RefPtr delegate = element;
+    if (is<SliderThumbElement>(element))
+        delegate = downcast<SliderThumbElement>(*element).hostInput();
+
+    Document& document = delegate->document();
     Frame* frame = document.frame();
-    return focusDelegate == document.focusedElement() && frame && frame->selection().isFocusedAndActive();
+    return delegate == document.focusedElement() && frame && frame->selection().isFocusedAndActive();
 }
 
 bool RenderTheme::isPressed(const RenderObject& renderer) const
@@ -1163,8 +1075,10 @@ void RenderTheme::adjustTextAreaStyle(RenderStyle&, const Element*) const
 {
 }
 
-void RenderTheme::adjustMenuListStyle(RenderStyle&, const Element*) const
+void RenderTheme::adjustMenuListStyle(RenderStyle& style, const Element*) const
 {
+    style.setOverflowX(Overflow::Visible);
+    style.setOverflowY(Overflow::Visible);
 }
 
 void RenderTheme::adjustMeterStyle(RenderStyle& style, const Element*) const
@@ -1205,8 +1119,10 @@ bool RenderTheme::paintCapsLockIndicator(const RenderObject&, const PaintInfo&,
 
 #if ENABLE(ATTACHMENT_ELEMENT)
 
-void RenderTheme::adjustAttachmentStyle(RenderStyle&, const Element*) const
+String RenderTheme::attachmentStyleSheet() const
 {
+    ASSERT(DeprecatedGlobalSettings::attachmentElementEnabled());
+    return "attachment { appearance: auto; }"_s;
 }
 
 bool RenderTheme::paintAttachment(const RenderObject&, const PaintInfo&, const IntRect&)
@@ -1220,7 +1136,7 @@ bool RenderTheme::paintAttachment(const RenderObject&, const PaintInfo&, const I
 
 String RenderTheme::colorInputStyleSheet(const Settings&) const
 {
-    return "input[type=\"color\"] { -webkit-appearance: color-well; width: 44px; height: 23px; outline: none; } "_s;
+    return "input[type=\"color\"] { appearance: auto; width: 44px; height: 23px; box-sizing: border-box; outline: none; } "_s;
 }
 
 #endif // ENABLE(INPUT_TYPE_COLOR)
@@ -1229,7 +1145,7 @@ String RenderTheme::colorInputStyleSheet(const Settings&) const
 
 String RenderTheme::dataListStyleSheet() const
 {
-    ASSERT(RuntimeEnabledFeatures::sharedFeatures().dataListElementEnabled());
+    ASSERT(DeprecatedGlobalSettings::dataListElementEnabled());
     return "datalist { display: none; }"_s;
 }
 
@@ -1359,10 +1275,6 @@ void RenderTheme::adjustMenuListButtonStyle(RenderStyle&, const Element*) const
 {
 }
 
-void RenderTheme::adjustMediaControlStyle(RenderStyle&, const Element*) const
-{
-}
-
 void RenderTheme::adjustSliderTrackStyle(RenderStyle&, const Element*) const
 {
 }
@@ -1418,45 +1330,6 @@ auto RenderTheme::colorCache(OptionSet<StyleColorOptions> options) const -> Colo
     }).iterator->value;
 }
 
-FontCascadeDescription& RenderTheme::cachedSystemFontDescription(CSSValueID systemFontID) const
-{
-    static NeverDestroyed<std::array<FontCascadeDescription, 10>> fontDescriptions;
-    switch (systemFontID) {
-    case CSSValueCaption:
-        return fontDescriptions.get()[0];
-    case CSSValueIcon:
-        return fontDescriptions.get()[1];
-    case CSSValueMenu:
-        return fontDescriptions.get()[2];
-    case CSSValueMessageBox:
-        return fontDescriptions.get()[3];
-    case CSSValueSmallCaption:
-        return fontDescriptions.get()[4];
-    case CSSValueStatusBar:
-        return fontDescriptions.get()[5];
-    case CSSValueWebkitMiniControl:
-        return fontDescriptions.get()[6];
-    case CSSValueWebkitSmallControl:
-        return fontDescriptions.get()[7];
-    case CSSValueWebkitControl:
-        return fontDescriptions.get()[8];
-    case CSSValueNone:
-        return fontDescriptions.get()[9];
-    default:
-        ASSERT_NOT_REACHED();
-        return fontDescriptions.get()[9];
-    }
-}
-
-void RenderTheme::systemFont(CSSValueID systemFontID, FontCascadeDescription& fontDescription) const
-{
-    fontDescription = cachedSystemFontDescription(systemFontID);
-    if (fontDescription.isAbsoluteSize())
-        return;
-
-    updateCachedSystemFontDescription(systemFontID, fontDescription);
-}
-
 Color RenderTheme::systemColor(CSSValueID cssValueId, OptionSet<StyleColorOptions> options) const
 {
     switch (cssValueId) {
@@ -1605,7 +1478,7 @@ Color RenderTheme::disabledTextColor(const Color& textColor, const Color& backgr
         disabledColor = textColor.lightened();
     else
         disabledColor = textColor.darkened();
-    
+
     // If there's not very much contrast between the disabled color and the background color,
     // just leave the text color alone. We don't want to change a good contrast color scheme so that it has really bad contrast.
     // If the contrast was already poor, then it doesn't do any good to change it to a different poor contrast color scheme.
