@@ -30,6 +30,7 @@
 #include "config.h"
 #include "BasicShapes.h"
 
+#include "AnimationUtilities.h"
 #include "BasicShapeFunctions.h"
 #include "CalculationValue.h"
 #include "FloatRect.h"
@@ -49,37 +50,38 @@ namespace WebCore {
 void BasicShapeCenterCoordinate::updateComputedLength()
 {
     if (m_direction == TopLeft) {
-        m_computedLength = m_length.isUndefined() ? Length(0, Fixed) : m_length;
+        m_computedLength = m_length.isUndefined() ? Length(0, LengthType::Fixed) : m_length;
         return;
     }
 
     if (m_length.isUndefined()) {
-        m_computedLength = Length(100, Percent);
+        m_computedLength = Length(100, LengthType::Percent);
         return;
     }
     
     m_computedLength = convertTo100PercentMinusLength(m_length);
 }
 
-struct SVGPathTranslatedByteStream {
-    SVGPathTranslatedByteStream(const FloatPoint& offset, const SVGPathByteStream& rawStream)
-        : m_offset(offset)
-        , m_rawStream(rawStream)
-    { }
-
-    bool operator==(const SVGPathTranslatedByteStream& other) const { return other.m_offset == m_offset && other.m_rawStream == m_rawStream; }
-    bool operator!=(const SVGPathTranslatedByteStream& other) const { return !(*this == other); }
-    bool isEmpty() const { return m_rawStream.isEmpty(); }
+struct SVGPathTransformedByteStream {
+    bool operator==(const SVGPathTransformedByteStream& other) const 
+    { 
+        return other.offset == offset && other.zoom == zoom && other.rawStream == rawStream;
+    }
+    bool operator!=(const SVGPathTransformedByteStream& other) const { return !(*this == other); }
+    bool isEmpty() const { return rawStream.isEmpty(); }
 
     Path path() const
     {
-        Path path = buildPathFromByteStream(m_rawStream);
-        path.translate(toFloatSize(m_offset));
+        Path path = buildPathFromByteStream(rawStream);
+        if (zoom != 1)
+            path.transform(AffineTransform().scale(zoom));
+        path.translate(toFloatSize(offset));
         return path;
     }
     
-    FloatPoint m_offset;
-    SVGPathByteStream m_rawStream;
+    SVGPathByteStream rawStream;
+    float zoom;
+    FloatPoint offset;
 };
 
 struct EllipsePathPolicy : public TinyLRUCachePolicy<FloatRect, Path> {
@@ -106,18 +108,17 @@ public:
     }
 };
 
-struct PolygonPathPolicy : public TinyLRUCachePolicy<Vector<FloatPoint>, Path> {
+struct PolygonPathPolicy : TinyLRUCachePolicy<Vector<FloatPoint>, Path> {
 public:
     static bool isKeyNull(const Vector<FloatPoint>& points) { return !points.size(); }
 
     static Path createValueForKey(const Vector<FloatPoint>& points) { return Path::polygonPathFromPoints(points); }
 };
 
-struct TranslatedByteStreamPathPolicy : public TinyLRUCachePolicy<SVGPathTranslatedByteStream, Path> {
-public:
-    static bool isKeyNull(const SVGPathTranslatedByteStream& stream) { return stream.isEmpty(); }
+struct TransformedByteStreamPathPolicy : TinyLRUCachePolicy<SVGPathTransformedByteStream, Path> {
+    static bool isKeyNull(const SVGPathTransformedByteStream& stream) { return stream.isEmpty(); }
 
-    static Path createValueForKey(const SVGPathTranslatedByteStream& stream) { return stream.path(); }
+    static Path createValueForKey(const SVGPathTransformedByteStream& stream) { return stream.path(); }
 };
 
 static const Path& cachedEllipsePath(const FloatRect& rect)
@@ -138,10 +139,10 @@ static const Path& cachedPolygonPath(const Vector<FloatPoint>& points)
     return cache.get().get(points);
 }
 
-static const Path& cachedTranslatedByteStreamPath(const SVGPathByteStream& stream, const FloatPoint& offset)
+static const Path& cachedTransformedByteStreamPath(const SVGPathByteStream& stream, float zoom, const FloatPoint& offset)
 {
-    static NeverDestroyed<TinyLRUCache<SVGPathTranslatedByteStream, Path, 4, TranslatedByteStreamPathPolicy>> cache;
-    return cache.get().get(SVGPathTranslatedByteStream(offset, stream));
+    static NeverDestroyed<TinyLRUCache<SVGPathTransformedByteStream, Path, 4, TransformedByteStreamPathPolicy>> cache;
+    return cache.get().get(SVGPathTransformedByteStream { stream, zoom, offset });
 }
 
 bool BasicShapeCircle::operator==(const BasicShape& other) const
@@ -189,15 +190,15 @@ bool BasicShapeCircle::canBlend(const BasicShape& other) const
     return radius().canBlend(downcast<BasicShapeCircle>(other).radius());
 }
 
-Ref<BasicShape> BasicShapeCircle::blend(const BasicShape& other, double progress) const
+Ref<BasicShape> BasicShapeCircle::blend(const BasicShape& other, const BlendingContext& context) const
 {
     ASSERT(type() == other.type());
     auto& otherCircle = downcast<BasicShapeCircle>(other);
     auto result =  BasicShapeCircle::create();
 
-    result->setCenterX(m_centerX.blend(otherCircle.centerX(), progress));
-    result->setCenterY(m_centerY.blend(otherCircle.centerY(), progress));
-    result->setRadius(m_radius.blend(otherCircle.radius(), progress));
+    result->setCenterX(m_centerX.blend(otherCircle.centerX(), context));
+    result->setCenterY(m_centerY.blend(otherCircle.centerY(), context));
+    result->setRadius(m_radius.blend(otherCircle.radius(), context));
     return result;
 }
 
@@ -252,7 +253,7 @@ bool BasicShapeEllipse::canBlend(const BasicShape& other) const
     return radiusX().canBlend(otherEllipse.radiusX()) && radiusY().canBlend(otherEllipse.radiusY());
 }
 
-Ref<BasicShape> BasicShapeEllipse::blend(const BasicShape& other, double progress) const
+Ref<BasicShape> BasicShapeEllipse::blend(const BasicShape& other, const BlendingContext& context) const
 {
     ASSERT(type() == other.type());
     auto& otherEllipse = downcast<BasicShapeEllipse>(other);
@@ -267,10 +268,10 @@ Ref<BasicShape> BasicShapeEllipse::blend(const BasicShape& other, double progres
         return result;
     }
 
-    result->setCenterX(m_centerX.blend(otherEllipse.centerX(), progress));
-    result->setCenterY(m_centerY.blend(otherEllipse.centerY(), progress));
-    result->setRadiusX(m_radiusX.blend(otherEllipse.radiusX(), progress));
-    result->setRadiusY(m_radiusY.blend(otherEllipse.radiusY(), progress));
+    result->setCenterX(m_centerX.blend(otherEllipse.centerX(), context));
+    result->setCenterY(m_centerY.blend(otherEllipse.centerY(), context));
+    result->setRadiusX(m_radiusX.blend(otherEllipse.radiusX(), context));
+    result->setRadiusY(m_radiusY.blend(otherEllipse.radiusY(), context));
     return result;
 }
 
@@ -315,7 +316,7 @@ bool BasicShapePolygon::canBlend(const BasicShape& other) const
     return values().size() == otherPolygon.values().size() && windRule() == otherPolygon.windRule();
 }
 
-Ref<BasicShape> BasicShapePolygon::blend(const BasicShape& other, double progress) const
+Ref<BasicShape> BasicShapePolygon::blend(const BasicShape& other, const BlendingContext& context) const
 {
     ASSERT(type() == other.type());
 
@@ -332,8 +333,8 @@ Ref<BasicShape> BasicShapePolygon::blend(const BasicShape& other, double progres
 
     for (size_t i = 0; i < length; i = i + 2) {
         result->appendPoint(
-            WebCore::blend(otherPolygon.values().at(i), m_values.at(i), progress),
-            WebCore::blend(otherPolygon.values().at(i + 1), m_values.at(i + 1), progress));
+            WebCore::blend(otherPolygon.values().at(i), m_values.at(i), context),
+            WebCore::blend(otherPolygon.values().at(i + 1), m_values.at(i + 1), context));
     }
 
     return result;
@@ -352,7 +353,7 @@ BasicShapePath::BasicShapePath(std::unique_ptr<SVGPathByteStream>&& byteStream)
 
 const Path& BasicShapePath::path(const FloatRect& boundingBox)
 {
-    return cachedTranslatedByteStreamPath(*m_byteStream, boundingBox.location());
+    return cachedTransformedByteStreamPath(*m_byteStream, m_zoom, boundingBox.location());
 }
 
 bool BasicShapePath::operator==(const BasicShape& other) const
@@ -361,7 +362,7 @@ bool BasicShapePath::operator==(const BasicShape& other) const
         return false;
 
     auto& otherPath = downcast<BasicShapePath>(other);
-    return m_windRule == otherPath.m_windRule && *m_byteStream == *otherPath.m_byteStream;
+    return m_zoom == otherPath.m_zoom && m_windRule == otherPath.m_windRule && *m_byteStream == *otherPath.m_byteStream;
 }
 
 bool BasicShapePath::canBlend(const BasicShape& other) const
@@ -373,17 +374,17 @@ bool BasicShapePath::canBlend(const BasicShape& other) const
     return windRule() == otherPath.windRule() && canBlendSVGPathByteStreams(*m_byteStream, *otherPath.pathData());
 }
 
-Ref<BasicShape> BasicShapePath::blend(const BasicShape& from, double progress) const
+Ref<BasicShape> BasicShapePath::blend(const BasicShape& from, const BlendingContext& context) const
 {
     ASSERT(type() == from.type());
 
     auto& fromPath = downcast<BasicShapePath>(from);
 
     auto resultingPathBytes = makeUnique<SVGPathByteStream>();
-    buildAnimatedSVGPathByteStream(*fromPath.m_byteStream, *m_byteStream, *resultingPathBytes, progress);
-
+    buildAnimatedSVGPathByteStream(*fromPath.m_byteStream, *m_byteStream, *resultingPathBytes, context.progress);
     auto result = BasicShapePath::create(WTFMove(resultingPathBytes));
     result->setWindRule(windRule());
+    result->setZoom(m_zoom);
     return result;
 }
 
@@ -430,21 +431,21 @@ bool BasicShapeInset::canBlend(const BasicShape& other) const
     return type() == other.type();
 }
 
-Ref<BasicShape> BasicShapeInset::blend(const BasicShape& from, double progress) const
+Ref<BasicShape> BasicShapeInset::blend(const BasicShape& from, const BlendingContext& context) const
 {
     ASSERT(type() == from.type());
 
     auto& fromInset = downcast<BasicShapeInset>(from);
     auto result =  BasicShapeInset::create();
-    result->setTop(WebCore::blend(fromInset.top(), top(), progress));
-    result->setRight(WebCore::blend(fromInset.right(), right(), progress));
-    result->setBottom(WebCore::blend(fromInset.bottom(), bottom(), progress));
-    result->setLeft(WebCore::blend(fromInset.left(), left(), progress));
+    result->setTop(WebCore::blend(fromInset.top(), top(), context));
+    result->setRight(WebCore::blend(fromInset.right(), right(), context));
+    result->setBottom(WebCore::blend(fromInset.bottom(), bottom(), context));
+    result->setLeft(WebCore::blend(fromInset.left(), left(), context));
 
-    result->setTopLeftRadius(WebCore::blend(fromInset.topLeftRadius(), topLeftRadius(), progress));
-    result->setTopRightRadius(WebCore::blend(fromInset.topRightRadius(), topRightRadius(), progress));
-    result->setBottomRightRadius(WebCore::blend(fromInset.bottomRightRadius(), bottomRightRadius(), progress));
-    result->setBottomLeftRadius(WebCore::blend(fromInset.bottomLeftRadius(), bottomLeftRadius(), progress));
+    result->setTopLeftRadius(WebCore::blend(fromInset.topLeftRadius(), topLeftRadius(), context, ValueRange::NonNegative));
+    result->setTopRightRadius(WebCore::blend(fromInset.topRightRadius(), topRightRadius(), context, ValueRange::NonNegative));
+    result->setBottomRightRadius(WebCore::blend(fromInset.bottomRightRadius(), bottomRightRadius(), context, ValueRange::NonNegative));
+    result->setBottomLeftRadius(WebCore::blend(fromInset.bottomLeftRadius(), bottomLeftRadius(), context, ValueRange::NonNegative));
 
     return result;
 }
@@ -493,4 +494,3 @@ TextStream& operator<<(TextStream& ts, const BasicShape& shape)
 }
 
 } // namespace WebCore
-
