@@ -25,13 +25,15 @@
 #pragma once
 
 #include "CSSPropertyNames.h"
+#include "CompositeOperation.h"
 #include <wtf/Vector.h>
 #include <wtf/HashSet.h>
 #include <wtf/text/AtomString.h>
+#include <wtf/text/AtomStringHash.h>
 
 namespace WebCore {
 
-class Element;
+class KeyframeEffect;
 class RenderStyle;
 class TimingFunction;
 
@@ -51,6 +53,10 @@ public:
     bool containsProperty(CSSPropertyID prop) const { return m_properties.contains(prop); }
     const HashSet<CSSPropertyID>& properties() const { return m_properties; }
 
+    void addCustomProperty(const AtomString& customProperty) { m_customProperties.add(customProperty); }
+    bool containsCustomProperty(const AtomString& customProperty) const { return m_customProperties.contains(customProperty); }
+    const HashSet<AtomString>& customProperties() const { return m_customProperties; }
+
     double key() const { return m_key; }
     void setKey(double key) { m_key = key; }
 
@@ -60,11 +66,16 @@ public:
     TimingFunction* timingFunction() const { return m_timingFunction.get(); }
     void setTimingFunction(const RefPtr<TimingFunction>& timingFunction) { m_timingFunction = timingFunction; }
 
+    std::optional<CompositeOperation> compositeOperation() const { return m_compositeOperation; }
+    void setCompositeOperation(std::optional<CompositeOperation> op) { m_compositeOperation = op; }
+
 private:
     double m_key;
     HashSet<CSSPropertyID> m_properties; // The properties specified in this keyframe.
+    HashSet<AtomString> m_customProperties; // The custom properties being animated.
     std::unique_ptr<RenderStyle> m_style;
     RefPtr<TimingFunction> m_timingFunction;
+    std::optional<CompositeOperation> m_compositeOperation;
 };
 
 class KeyframeList {
@@ -83,24 +94,33 @@ public:
     
     void insert(KeyframeValue&&);
     
-    void addProperty(CSSPropertyID prop) { m_properties.add(prop); }
     bool containsProperty(CSSPropertyID prop) const { return m_properties.contains(prop); }
     const HashSet<CSSPropertyID>& properties() const { return m_properties; }
-    
+    bool containsAnimatableProperty() const;
+
+    void addCustomProperty(const AtomString& customProperty) { m_customProperties.add(customProperty); }
+    bool containsCustomProperty(const AtomString& customProperty) const { return m_customProperties.contains(customProperty); }
+    const HashSet<AtomString>& customProperties() const { return m_customProperties; }
+
     void clear();
     bool isEmpty() const { return m_keyframes.isEmpty(); }
     size_t size() const { return m_keyframes.size(); }
     const KeyframeValue& operator[](size_t index) const { return m_keyframes[index]; }
-    const Vector<KeyframeValue>& keyframes() const { return m_keyframes; }
 
     void copyKeyframes(KeyframeList&);
     bool hasImplicitKeyframes() const;
-    void fillImplicitKeyframes(const Element&, Style::Resolver&, const RenderStyle*);
+    void fillImplicitKeyframes(const KeyframeEffect&, const RenderStyle& elementStyle);
+
+    auto begin() const { return m_keyframes.begin(); }
+    auto end() const { return m_keyframes.end(); }
+
+    bool usesContainerUnits() const;
 
 private:
     AtomString m_animationName;
     Vector<KeyframeValue> m_keyframes; // Kept sorted by key.
     HashSet<CSSPropertyID> m_properties; // The properties being animated.
+    HashSet<AtomString> m_customProperties; // The custom properties being animated.
 };
 
 } // namespace WebCore
