@@ -6,6 +6,7 @@
  * Copyright (C) 2009 Dirk Schulze <krit@webkit.org>
  * Copyright (C) Research In Motion Limited 2009-2010. All rights reserved.
  * Copyright (C) 2018 Adobe Systems Incorporated. All rights reserved.
+ * Copyright (C) 2020 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,6 +27,11 @@
 #include "config.h"
 #include "SVGRenderSupport.h"
 
+#include "ElementAncestorIterator.h"
+#include "LegacyRenderSVGRoot.h"
+#include "LegacyRenderSVGShape.h"
+#include "LegacyRenderSVGTransformableContainer.h"
+#include "LegacyRenderSVGViewportContainer.h"
 #include "NodeRenderStyle.h"
 #include "RenderChildIterator.h"
 #include "RenderElement.h"
@@ -38,9 +44,9 @@
 #include "RenderSVGResourceMarker.h"
 #include "RenderSVGResourceMasker.h"
 #include "RenderSVGRoot.h"
+#include "RenderSVGShape.h"
 #include "RenderSVGText.h"
-#include "RenderSVGTransformableContainer.h"
-#include "RenderSVGViewportContainer.h"
+#include "SVGElementTypeHelpers.h"
 #include "SVGGeometryElement.h"
 #include "SVGResources.h"
 #include "SVGResourcesCache.h"
@@ -57,13 +63,10 @@ LayoutRect SVGRenderSupport::clippedOverflowRectForRepaint(const RenderElement&
     // Pass our local paint rect to computeFloatVisibleRectInContainer() which will
     // map to parent coords and recurse up the parent chain.
     FloatRect repaintRect = renderer.repaintRectInLocalCoordinates();
-    const SVGRenderStyle& svgStyle = renderer.style().svgStyle();
-    if (const ShadowData* shadow = svgStyle.shadow())
-        shadow->adjustRectForShadow(repaintRect);
     return enclosingLayoutRect(renderer.computeFloatRectForRepaint(repaintRect, repaintContainer));
 }
 
-Optional<FloatRect> SVGRenderSupport::computeFloatVisibleRectInContainer(const RenderElement& renderer, const FloatRect& rect, const RenderLayerModelObject* container, RenderObject::VisibleRectContext context)
+std::optional<FloatRect> SVGRenderSupport::computeFloatVisibleRectInContainer(const RenderElement& renderer, const FloatRect& rect, const RenderLayerModelObject* container, RenderObject::VisibleRectContext context)
 {
     // Ensure our parent is an SVG object.
     ASSERT(renderer.parent());
@@ -72,9 +75,6 @@ Optional<FloatRect> SVGRenderSupport::computeFloatVisibleRectInContainer(const R
         return FloatRect();
 
     FloatRect adjustedRect = rect;
-    const SVGRenderStyle& svgStyle = renderer.style().svgStyle();
-    if (const ShadowData* shadow = svgStyle.shadow())
-        shadow->adjustRectForShadow(adjustedRect);
     adjustedRect.inflate(renderer.style().outlineWidth());
 
     // Translate to coords in our parent renderer, and then call computeFloatVisibleRectInContainer() on our parent.
@@ -88,25 +88,25 @@ const RenderElement& SVGRenderSupport::localToParentTransform(const RenderElemen
     ASSERT(renderer.parent());
     auto& parent = *renderer.parent();
 
-    // At the SVG/HTML boundary (aka RenderSVGRoot), we apply the localToBorderBoxTransform
+    // At the SVG/HTML boundary (aka LegacyRenderSVGRoot), we apply the localToBorderBoxTransform
     // to map an element from SVG viewport coordinates to CSS box coordinates.
-    if (is<RenderSVGRoot>(parent))
-        transform = downcast<RenderSVGRoot>(parent).localToBorderBoxTransform() * renderer.localToParentTransform();
+    if (is<LegacyRenderSVGRoot>(parent))
+        transform = downcast<LegacyRenderSVGRoot>(parent).localToBorderBoxTransform() * renderer.localToParentTransform();
     else
         transform = renderer.localToParentTransform();
 
     return parent;
 }
 
-void SVGRenderSupport::mapLocalToContainer(const RenderElement& renderer, const RenderLayerModelObject* repaintContainer, TransformState& transformState, bool* wasFixed)
+void SVGRenderSupport::mapLocalToContainer(const RenderElement& renderer, const RenderLayerModelObject* ancestorContainer, TransformState& transformState, bool* wasFixed)
 {
     AffineTransform transform;
     auto& parent = localToParentTransform(renderer, transform);
 
     transformState.applyTransform(transform);
 
-    MapCoordinatesFlags mode = UseTransforms;
-    parent.mapLocalToContainer(repaintContainer, transformState, mode, wasFixed);
+    OptionSet<MapCoordinatesMode> mode = UseTransforms;
+    parent.mapLocalToContainer(ancestorContainer, transformState, mode, wasFixed);
 }
 
 const RenderElement* SVGRenderSupport::pushMappingToContainer(const RenderElement& renderer, const RenderLayerModelObject* ancestorToStopAt, RenderGeometryMap& geometryMap)
@@ -127,13 +127,13 @@ bool SVGRenderSupport::checkForSVGRepaintDuringLayout(const RenderElement& rende
     // When a parent container is transformed in SVG, all children will be painted automatically
     // so we are able to skip redundant repaint checks.
     auto parent = renderer.parent();
-    return !(is<RenderSVGContainer>(parent) && downcast<RenderSVGContainer>(*parent).didTransformToRootUpdate());
+    return !(is<LegacyRenderSVGContainer>(parent) && downcast<LegacyRenderSVGContainer>(*parent).didTransformToRootUpdate());
 }
 
 // Update a bounding box taking into account the validity of the other bounding box.
 static inline void updateObjectBoundingBox(FloatRect& objectBoundingBox, bool& objectBoundingBoxValid, const RenderObject* other, FloatRect otherBoundingBox)
 {
-    bool otherValid = is<RenderSVGContainer>(*other) ? downcast<RenderSVGContainer>(*other).isObjectBoundingBoxValid() : true;
+    bool otherValid = is<LegacyRenderSVGContainer>(*other) ? downcast<LegacyRenderSVGContainer>(*other).isObjectBoundingBoxValid() : true;
     if (!otherValid)
         return;
 
@@ -160,7 +160,7 @@ void SVGRenderSupport::computeContainerBoundingBoxes(const RenderElement& contai
             continue;
 
         // Don't include elements in the union that do not render.
-        if (is<RenderSVGShape>(current) && downcast<RenderSVGShape>(current).isRenderingDisabled())
+        if (is<LegacyRenderSVGShape>(current) && downcast<LegacyRenderSVGShape>(current).isRenderingDisabled())
             continue;
 
         const AffineTransform& transform = current.localToParentTransform();
@@ -184,14 +184,14 @@ bool SVGRenderSupport::paintInfoIntersectsRepaintRect(const FloatRect& localRepa
     return localTransform.mapRect(localRepaintRect).intersects(paintInfo.rect);
 }
 
-RenderSVGRoot* SVGRenderSupport::findTreeRootObject(RenderElement& start)
+LegacyRenderSVGRoot* SVGRenderSupport::findTreeRootObject(RenderElement& start)
 {
-    return lineageOfType<RenderSVGRoot>(start).first();
+    return lineageOfType<LegacyRenderSVGRoot>(start).first();
 }
 
-const RenderSVGRoot* SVGRenderSupport::findTreeRootObject(const RenderElement& start)
+const LegacyRenderSVGRoot* SVGRenderSupport::findTreeRootObject(const RenderElement& start)
 {
-    return lineageOfType<RenderSVGRoot>(start).first();
+    return lineageOfType<LegacyRenderSVGRoot>(start).first();
 }
 
 static inline void invalidateResourcesOfChildren(RenderElement& renderer)
@@ -207,23 +207,23 @@ static inline void invalidateResourcesOfChildren(RenderElement& renderer)
 static inline bool layoutSizeOfNearestViewportChanged(const RenderElement& renderer)
 {
     const RenderElement* start = &renderer;
-    while (start && !is<RenderSVGRoot>(*start) && !is<RenderSVGViewportContainer>(*start))
+    while (start && !is<LegacyRenderSVGRoot>(*start) && !is<LegacyRenderSVGViewportContainer>(*start))
         start = start->parent();
 
     ASSERT(start);
-    if (is<RenderSVGViewportContainer>(*start))
-        return downcast<RenderSVGViewportContainer>(*start).isLayoutSizeChanged();
+    if (is<LegacyRenderSVGViewportContainer>(*start))
+        return downcast<LegacyRenderSVGViewportContainer>(*start).isLayoutSizeChanged();
 
-    return downcast<RenderSVGRoot>(*start).isLayoutSizeChanged();
+    return downcast<LegacyRenderSVGRoot>(*start).isLayoutSizeChanged();
 }
 
 bool SVGRenderSupport::transformToRootChanged(RenderElement* ancestor)
 {
-    while (ancestor && !is<RenderSVGRoot>(*ancestor)) {
-        if (is<RenderSVGTransformableContainer>(*ancestor))
-            return downcast<RenderSVGTransformableContainer>(*ancestor).didTransformToRootUpdate();
-        if (is<RenderSVGViewportContainer>(*ancestor))
-            return downcast<RenderSVGViewportContainer>(*ancestor).didTransformToRootUpdate();
+    while (ancestor && !ancestor->isSVGRootOrLegacySVGRoot()) {
+        if (is<LegacyRenderSVGTransformableContainer>(*ancestor))
+            return downcast<LegacyRenderSVGTransformableContainer>(*ancestor).didTransformToRootUpdate();
+        if (is<LegacyRenderSVGViewportContainer>(*ancestor))
+            return downcast<LegacyRenderSVGViewportContainer>(*ancestor).didTransformToRootUpdate();
         ancestor = ancestor->parent();
     }
 
@@ -260,15 +260,15 @@ void SVGRenderSupport::layoutChildren(RenderElement& start, bool selfNeedsLayout
             // When selfNeedsLayout is false and the layout size changed, we have to check whether this child uses relative lengths
             auto& element = downcast<SVGElement>(*child.node());
             if (element.hasRelativeLengths()) {
-                // When the layout size changed and when using relative values tell the RenderSVGShape to update its shape object
-                if (is<RenderSVGShape>(child))
-                    downcast<RenderSVGShape>(child).setNeedsShapeUpdate();
+                // When the layout size changed and when using relative values tell the LegacyRenderSVGShape to update its shape object
+                if (is<LegacyRenderSVGShape>(child))
+                    downcast<LegacyRenderSVGShape>(child).setNeedsShapeUpdate();
                 else if (is<RenderSVGText>(child)) {
                     auto& svgText = downcast<RenderSVGText>(child);
                     svgText.setNeedsTextMetricsUpdate();
                     svgText.setNeedsPositioningValuesUpdate();
                 }
-
+                child.setNeedsTransformUpdate();
                 needsLayout = true;
             }
         }
@@ -303,7 +303,7 @@ void SVGRenderSupport::layoutChildren(RenderElement& start, bool selfNeedsLayout
 
 bool SVGRenderSupport::isOverflowHidden(const RenderElement& renderer)
 {
-    // RenderSVGRoot should never query for overflow state - it should always clip itself to the initial viewport size.
+    // LegacyRenderSVGRoot should never query for overflow state - it should always clip itself to the initial viewport size.
     ASSERT(!renderer.isDocumentElementRenderer());
 
     return renderer.style().overflowX() == Overflow::Hidden || renderer.style().overflowX() == Overflow::Scroll;
@@ -351,9 +351,9 @@ inline FloatRect clipPathReferenceBox(const RenderElement& renderer, CSSBoxType
         break;
     case CSSBoxType::ViewBox:
         if (renderer.element()) {
-            FloatSize viewportSize;
-            SVGLengthContext(downcast<SVGElement>(renderer.element())).determineViewport(viewportSize);
-            referenceBox.setSize(viewportSize);
+            auto viewportSize = SVGLengthContext(downcast<SVGElement>(renderer.element())).viewportSize();
+            if (viewportSize)
+                referenceBox.setSize(*viewportSize);
             break;
         }
         FALLTHROUGH;
@@ -369,16 +369,16 @@ inline FloatRect clipPathReferenceBox(const RenderElement& renderer, CSSBoxType
 
 inline bool isPointInCSSClippingArea(const RenderElement& renderer, const FloatPoint& point)
 {
-    ClipPathOperation* clipPathOperation = renderer.style().clipPath();
-    if (is<ShapeClipPathOperation>(clipPathOperation)) {
-        auto& clipPath = downcast<ShapeClipPathOperation>(*clipPathOperation);
+    PathOperation* clipPathOperation = renderer.style().clipPath();
+    if (is<ShapePathOperation>(clipPathOperation)) {
+        auto& clipPath = downcast<ShapePathOperation>(*clipPathOperation);
         FloatRect referenceBox = clipPathReferenceBox(renderer, clipPath.referenceBox());
         if (!referenceBox.contains(point))
             return false;
         return clipPath.pathForReferenceRect(referenceBox).contains(point, clipPath.windRule());
     }
-    if (is<BoxClipPathOperation>(clipPathOperation)) {
-        auto& clipPath = downcast<BoxClipPathOperation>(*clipPathOperation);
+    if (is<BoxPathOperation>(clipPathOperation)) {
+        auto& clipPath = downcast<BoxPathOperation>(*clipPathOperation);
         FloatRect referenceBox = clipPathReferenceBox(renderer, clipPath.referenceBox());
         if (!referenceBox.contains(point))
             return false;
@@ -390,14 +390,21 @@ inline bool isPointInCSSClippingArea(const RenderElement& renderer, const FloatP
 
 void SVGRenderSupport::clipContextToCSSClippingArea(GraphicsContext& context, const RenderElement& renderer)
 {
-    ClipPathOperation* clipPathOperation = renderer.style().clipPath();
-    if (is<ShapeClipPathOperation>(clipPathOperation)) {
-        auto& clipPath = downcast<ShapeClipPathOperation>(*clipPathOperation);
-        FloatRect referenceBox = clipPathReferenceBox(renderer, clipPath.referenceBox());
-        context.clipPath(clipPath.pathForReferenceRect(referenceBox), clipPath.windRule());
+    PathOperation* clipPathOperation = renderer.style().clipPath();
+    if (is<ShapePathOperation>(clipPathOperation)) {
+        auto& clipPath = downcast<ShapePathOperation>(*clipPathOperation);
+        auto localToParentTransform = renderer.localToParentTransform();
+
+        auto referenceBox = clipPathReferenceBox(renderer, clipPath.referenceBox());
+        referenceBox = localToParentTransform.mapRect(referenceBox);
+
+        auto path = clipPath.pathForReferenceRect(referenceBox);
+        path.transform(valueOrDefault(localToParentTransform.inverse()));
+
+        context.clipPath(path, clipPath.windRule());
     }
-    if (is<BoxClipPathOperation>(clipPathOperation)) {
-        auto& clipPath = downcast<BoxClipPathOperation>(*clipPathOperation);
+    if (is<BoxPathOperation>(clipPathOperation)) {
+        auto& clipPath = downcast<BoxPathOperation>(*clipPathOperation);
         FloatRect referenceBox = clipPathReferenceBox(renderer, clipPath.referenceBox());
         context.clipPath(clipPath.pathForReferenceRect(FloatRoundedRect {referenceBox}));
     }
@@ -405,8 +412,11 @@ void SVGRenderSupport::clipContextToCSSClippingArea(GraphicsContext& context, co
 
 bool SVGRenderSupport::pointInClippingArea(const RenderElement& renderer, const FloatPoint& point)
 {
-    ClipPathOperation* clipPathOperation = renderer.style().clipPath();
-    if (is<ShapeClipPathOperation>(clipPathOperation) || is<BoxClipPathOperation>(clipPathOperation))
+    if (SVGHitTestCycleDetectionScope::isVisiting(renderer))
+        return false;
+
+    PathOperation* clipPathOperation = renderer.style().clipPath();
+    if (is<ShapePathOperation>(clipPathOperation) || is<BoxPathOperation>(clipPathOperation))
         return isPointInCSSClippingArea(renderer, point);
 
     // We just take clippers into account to determine if a point is on the node. The Specification may
@@ -421,10 +431,8 @@ bool SVGRenderSupport::pointInClippingArea(const RenderElement& renderer, const
     return true;
 }
 
-void SVGRenderSupport::applyStrokeStyleToContext(GraphicsContext* context, const RenderStyle& style, const RenderElement& renderer)
+void SVGRenderSupport::applyStrokeStyleToContext(GraphicsContext& context, const RenderStyle& style, const RenderElement& renderer)
 {
-    ASSERT(context);
-    
     Element* element = renderer.element();
     if (!is<SVGElement>(element)) {
         ASSERT_NOT_REACHED();
@@ -434,15 +442,15 @@ void SVGRenderSupport::applyStrokeStyleToContext(GraphicsContext* context, const
     const SVGRenderStyle& svgStyle = style.svgStyle();
 
     SVGLengthContext lengthContext(downcast<SVGElement>(renderer.element()));
-    context->setStrokeThickness(lengthContext.valueForLength(style.strokeWidth()));
-    context->setLineCap(style.capStyle());
-    context->setLineJoin(style.joinStyle());
-    if (style.joinStyle() == MiterJoin)
-        context->setMiterLimit(style.strokeMiterLimit());
+    context.setStrokeThickness(lengthContext.valueForLength(style.strokeWidth()));
+    context.setLineCap(style.capStyle());
+    context.setLineJoin(style.joinStyle());
+    if (style.joinStyle() == LineJoin::Miter)
+        context.setMiterLimit(style.strokeMiterLimit());
 
     const Vector<SVGLengthValue>& dashes = svgStyle.strokeDashArray();
     if (dashes.isEmpty())
-        context->setStrokeStyle(SolidStroke);
+        context.setStrokeStyle(SolidStroke);
     else {
         DashArray dashArray;
         dashArray.reserveInitialCapacity(dashes.size());
@@ -450,10 +458,16 @@ void SVGRenderSupport::applyStrokeStyleToContext(GraphicsContext* context, const
         float scaleFactor = 1;
 
         if (is<SVGGeometryElement>(element)) {
-            ASSERT(renderer.isSVGShape());
+            ASSERT(renderer.isSVGShapeOrLegacySVGShape());
             // FIXME: A value of zero is valid. Need to differentiate this case from being unspecified.
-            if (float pathLength = downcast<SVGGeometryElement>(element)->pathLength())
-                scaleFactor = downcast<RenderSVGShape>(renderer).getTotalLength() / pathLength;
+            if (float pathLength = downcast<SVGGeometryElement>(element)->pathLength()) {
+                if (is<LegacyRenderSVGShape>(renderer))
+                    scaleFactor = downcast<LegacyRenderSVGShape>(renderer).getTotalLength() / pathLength;
+#if ENABLE(LAYER_BASED_SVG_ENGINE)
+                else if (is<RenderSVGShape>(renderer))
+                    scaleFactor = downcast<RenderSVGShape>(renderer).getTotalLength() / pathLength;
+#endif
+            }
         }
         
         for (auto& dash : dashes) {
@@ -463,9 +477,9 @@ void SVGRenderSupport::applyStrokeStyleToContext(GraphicsContext* context, const
         }
 
         if (canSetLineDash)
-            context->setLineDash(dashArray, lengthContext.valueForLength(svgStyle.strokeDashOffset()) * scaleFactor);
+            context.setLineDash(dashArray, lengthContext.valueForLength(svgStyle.strokeDashOffset()) * scaleFactor);
         else
-            context->setStrokeStyle(SolidStroke);
+            context.setStrokeStyle(SolidStroke);
     }
 }
 
@@ -481,9 +495,10 @@ void SVGRenderSupport::styleChanged(RenderElement& renderer, const RenderStyle*
 }
 
 #if ENABLE(CSS_COMPOSITING)
+
 bool SVGRenderSupport::isolatesBlending(const RenderStyle& style)
 {
-    return style.svgStyle().isolatesBlending() || style.hasFilter() || style.hasBlendMode() || style.opacity() < 1.0f;
+    return style.hasPositionedMask() || style.hasFilter() || style.hasBlendMode() || style.opacity() < 1.0f;
 }
 
 void SVGRenderSupport::updateMaskedAncestorShouldIsolateBlending(const RenderElement& renderer)
@@ -491,20 +506,45 @@ void SVGRenderSupport::updateMaskedAncestorShouldIsolateBlending(const RenderEle
     ASSERT(renderer.element());
     ASSERT(renderer.element()->isSVGElement());
 
-    bool maskedAncestorShouldIsolateBlending = renderer.style().hasBlendMode();
-    for (auto* ancestor = renderer.element()->parentElement(); ancestor && ancestor->isSVGElement(); ancestor = ancestor->parentElement()) {
-        if (!downcast<SVGElement>(*ancestor).isSVGGraphicsElement())
-            continue;
-
-        const auto* style = ancestor->computedStyle();
+    for (auto& ancestor : ancestorsOfType<SVGGraphicsElement>(*renderer.element())) {
+        auto* style = ancestor.computedStyle();
         if (!style || !isolatesBlending(*style))
             continue;
-
-        if (ancestor->computedStyle()->svgStyle().hasMasker())
-            downcast<SVGGraphicsElement>(*ancestor).setShouldIsolateBlending(maskedAncestorShouldIsolateBlending);
-
+        if (style->hasPositionedMask())
+            ancestor.setShouldIsolateBlending(renderer.style().hasBlendMode());
         return;
     }
 }
+
 #endif
+
+SVGHitTestCycleDetectionScope::SVGHitTestCycleDetectionScope(const RenderElement& element)
+{
+    m_element = element;
+    auto result = visitedElements().add(*m_element);
+    ASSERT_UNUSED(result, result.isNewEntry);
+}
+
+SVGHitTestCycleDetectionScope::~SVGHitTestCycleDetectionScope()
+{
+    bool result = visitedElements().remove(*m_element);
+    ASSERT_UNUSED(result, result);
+}
+
+WeakHashSet<RenderElement>& SVGHitTestCycleDetectionScope::visitedElements()
+{
+    static NeverDestroyed<WeakHashSet<RenderElement>> s_visitedElements;
+    return s_visitedElements;
+}
+
+bool SVGHitTestCycleDetectionScope::isEmpty()
+{
+    return visitedElements().computesEmpty();
+}
+
+bool SVGHitTestCycleDetectionScope::isVisiting(const RenderElement& element)
+{
+    return visitedElements().contains(element);
+}
+
 }
