@@ -2,7 +2,7 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 2004-2005 Allan Sandfeld Jensen (kde@carewolf.com)
  * Copyright (C) 2006, 2007 Nicholas Shanks (webkit@nickshanks.com)
- * Copyright (C) 2005-2019 Apple Inc. All rights reserved.
+ * Copyright (C) 2005-2021 Apple Inc. All rights reserved.
  * Copyright (C) 2007 Alexey Proskuryakov <ap@webkit.org>
  * Copyright (C) 2007, 2008 Eric Seidel <eric@webkit.org>
  * Copyright (C) 2008, 2009 Torch Mobile Inc. All rights reserved. (http://www.torchmobile.com/)
@@ -33,8 +33,11 @@
 #include "CSSKeyframesRule.h"
 #include "CSSSelector.h"
 #include "CSSSelectorList.h"
+#include "CommonAtomStrings.h"
 #include "HTMLNames.h"
 #include "MediaQueryEvaluator.h"
+#include "RuleSetBuilder.h"
+#include "SVGElement.h"
 #include "SecurityOrigin.h"
 #include "SelectorChecker.h"
 #include "SelectorFilter.h"
@@ -42,11 +45,6 @@
 #include "StyleRule.h"
 #include "StyleRuleImport.h"
 #include "StyleSheetContents.h"
-#include "ViewportStyleResolver.h"
-
-#if ENABLE(VIDEO_TRACK)
-#include "TextTrackCue.h"
-#endif
 
 namespace WebCore {
 namespace Style {
@@ -76,38 +74,69 @@ static unsigned rulesCountForName(const RuleSet::AtomRuleMap& map, const AtomStr
 
 static bool isHostSelectorMatchingInShadowTree(const CSSSelector& startSelector)
 {
-    auto* leftmostSelector = &startSelector;
-    bool hasDescendantOrChildRelation = false;
-    while (auto* previous = leftmostSelector->tagHistory()) {
-        hasDescendantOrChildRelation = leftmostSelector->hasDescendantOrChildRelation();
-        leftmostSelector = previous;
+    bool hasOnlyOneCompound = true;
+    bool hasHostInLastCompound = false;
+    for (auto* selector = &startSelector; selector; selector = selector->tagHistory()) {
+        if (selector->match() == CSSSelector::PseudoClass && selector->pseudoClassType() == CSSSelector::PseudoClassHost)
+            hasHostInLastCompound = true;
+        if (selector->tagHistory() && selector->relation() != CSSSelector::Subselector) {
+            hasOnlyOneCompound = false;
+            hasHostInLastCompound = false;
+        }
     }
-    if (!hasDescendantOrChildRelation)
-        return false;
-
-    return leftmostSelector->match() == CSSSelector::PseudoClass && leftmostSelector->pseudoClassType() == CSSSelector::PseudoClassHost;
+    return !hasOnlyOneCompound && hasHostInLastCompound;
 }
 
-void RuleSet::addRule(const StyleRule& rule, unsigned selectorIndex, unsigned selectorListIndex, MediaQueryCollector* mediaQueryCollector)
+static bool shouldHaveBucketForAttributeName(const CSSSelector& attributeSelector)
 {
-    RuleData ruleData(rule, selectorIndex, selectorListIndex, m_ruleCount++);
+    // Don't make buckets for lazy attributes since we don't want to synchronize.
+    if (attributeSelector.attributeCanonicalLocalName() == HTMLNames::styleAttr->localName())
+        return false;
+    if (SVGElement::animatableAttributeForName(attributeSelector.attribute().localName()) != nullQName())
+        return false;
+    return true;
+}
+
+void RuleSet::addRule(const StyleRule& rule, unsigned selectorIndex, unsigned selectorListIndex)
+{
+    RuleData ruleData(rule, selectorIndex, selectorListIndex, m_ruleCount);
+    addRule(WTFMove(ruleData), 0, 0);
+}
+
+void RuleSet::addRule(RuleData&& ruleData, CascadeLayerIdentifier cascadeLayerIdentifier, ContainerQueryIdentifier containerQueryIdentifier)
+{
+    ASSERT(ruleData.position() == m_ruleCount);
+
+    ++m_ruleCount;
+
+    if (cascadeLayerIdentifier) {
+        auto oldSize = m_cascadeLayerIdentifierForRulePosition.size();
+        m_cascadeLayerIdentifierForRulePosition.grow(m_ruleCount);
+        std::fill(m_cascadeLayerIdentifierForRulePosition.begin() + oldSize, m_cascadeLayerIdentifierForRulePosition.end(), 0);
+        m_cascadeLayerIdentifierForRulePosition.last() = cascadeLayerIdentifier;
+    }
+
+    if (containerQueryIdentifier) {
+        auto oldSize = m_containerQueryIdentifierForRulePosition.size();
+        m_containerQueryIdentifierForRulePosition.grow(m_ruleCount);
+        std::fill(m_containerQueryIdentifierForRulePosition.begin() + oldSize, m_containerQueryIdentifierForRulePosition.end(), 0);
+        m_containerQueryIdentifierForRulePosition.last() = containerQueryIdentifier;
+    }
 
     m_features.collectFeatures(ruleData);
 
-    if (mediaQueryCollector)
-        mediaQueryCollector->addRuleIfNeeded(ruleData);
-
     unsigned classBucketSize = 0;
     const CSSSelector* idSelector = nullptr;
     const CSSSelector* tagSelector = nullptr;
     const CSSSelector* classSelector = nullptr;
+    const CSSSelector* attributeSelector = nullptr;
     const CSSSelector* linkSelector = nullptr;
     const CSSSelector* focusSelector = nullptr;
     const CSSSelector* hostPseudoClassSelector = nullptr;
     const CSSSelector* customPseudoElementSelector = nullptr;
     const CSSSelector* slottedPseudoElementSelector = nullptr;
     const CSSSelector* partPseudoElementSelector = nullptr;
-#if ENABLE(VIDEO_TRACK)
+#if ENABLE(VIDEO)
     const CSSSelector* cuePseudoElementSelector = nullptr;
 #endif
     const CSSSelector* selector = ruleData.selector();
@@ -130,6 +159,16 @@ void RuleSet::addRule(const StyleRule& rule, unsigned selectorIndex, unsigned se
             }
             break;
         }
+        case CSSSelector::Exact:
+        case CSSSelector::Set:
+        case CSSSelector::List:
+        case CSSSelector::Hyphen:
+        case CSSSelector::Contain:
+        case CSSSelector::Begin:
+        case CSSSelector::End:
+            if (shouldHaveBucketForAttributeName(*selector))
+                attributeSelector = selector;
+            break;
         case CSSSelector::Tag:
             if (selector->tagQName().localName() != starAtom())
                 tagSelector = selector;
@@ -146,7 +185,7 @@ void RuleSet::addRule(const StyleRule& rule, unsigned selectorIndex, unsigned se
             case CSSSelector::PseudoElementPart:
                 partPseudoElementSelector = selector;
                 break;
-#if ENABLE(VIDEO_TRACK)
+#if ENABLE(VIDEO)
             case CSSSelector::PseudoElementCue:
                 cuePseudoElementSelector = selector;
                 break;
@@ -163,8 +202,8 @@ void RuleSet::addRule(const StyleRule& rule, unsigned selectorIndex, unsigned se
             case CSSSelector::PseudoClassAnyLinkDeprecated:
                 linkSelector = selector;
                 break;
-            case CSSSelector::PseudoClassDirectFocus:
             case CSSSelector::PseudoClassFocus:
+            case CSSSelector::PseudoClassFocusVisible:
                 focusSelector = selector;
                 break;
             case CSSSelector::PseudoClassHost:
@@ -175,13 +214,6 @@ void RuleSet::addRule(const StyleRule& rule, unsigned selectorIndex, unsigned se
             }
             break;
         case CSSSelector::Unknown:
-        case CSSSelector::Exact:
-        case CSSSelector::Set:
-        case CSSSelector::List:
-        case CSSSelector::Hyphen:
-        case CSSSelector::Contain:
-        case CSSSelector::Begin:
-        case CSSSelector::End:
         case CSSSelector::PagePseudoClass:
             break;
         }
@@ -190,7 +222,10 @@ void RuleSet::addRule(const StyleRule& rule, unsigned selectorIndex, unsigned se
         selector = selector->tagHistory();
     } while (selector);
 
-#if ENABLE(VIDEO_TRACK)
+    if (!m_hasHostPseudoClassRulesMatchingInShadowTree)
+        m_hasHostPseudoClassRulesMatchingInShadowTree = isHostSelectorMatchingInShadowTree(*ruleData.selector());
+
+#if ENABLE(VIDEO)
     if (cuePseudoElementSelector) {
         m_cuePseudoRules.append(ruleData);
         return;
@@ -226,9 +261,6 @@ void RuleSet::addRule(const StyleRule& rule, unsigned selectorIndex, unsigned se
         return;
     }
 
-    if (!m_hasHostPseudoClassRulesMatchingInShadowTree)
-        m_hasHostPseudoClassRulesMatchingInShadowTree = isHostSelectorMatchingInShadowTree(*ruleData.selector());
-
     if (hostPseudoClassSelector) {
         m_hostPseudoClassRules.append(ruleData);
         return;
@@ -244,6 +276,12 @@ void RuleSet::addRule(const StyleRule& rule, unsigned selectorIndex, unsigned se
         return;
     }
 
+    if (attributeSelector) {
+        addToRuleSet(attributeSelector->attribute().localName(), m_attributeLocalNameRules, ruleData);
+        addToRuleSet(attributeSelector->attributeCanonicalLocalName(), m_attributeCanonicalLocalNameRules, ruleData);
+        return;
+    }
+
     if (linkSelector) {
         m_linkPseudoClassRules.append(ruleData);
         return;
@@ -269,119 +307,6 @@ void RuleSet::addPageRule(StyleRulePage& rule)
     m_pageRules.append(&rule);
 }
 
-void RuleSet::addChildRules(const Vector<RefPtr<StyleRuleBase>>& rules, MediaQueryCollector& mediaQueryCollector, Resolver* resolver, AddRulesMode mode)
-{
-    for (auto& rule : rules) {
-        if (mode == AddRulesMode::ResolverMutationScan && mediaQueryCollector.didMutateResolverWithinDynamicMediaQuery)
-            break;
-
-        if (is<StyleRule>(*rule)) {
-            if (mode == AddRulesMode::Normal)
-                addStyleRule(downcast<StyleRule>(*rule), mediaQueryCollector);
-            continue;
-        }
-        if (is<StyleRulePage>(*rule)) {
-            if (mode == AddRulesMode::Normal)
-                addPageRule(downcast<StyleRulePage>(*rule));
-            continue;
-        }
-        if (is<StyleRuleMedia>(*rule)) {
-            auto& mediaRule = downcast<StyleRuleMedia>(*rule);
-            if (mediaQueryCollector.pushAndEvaluate(mediaRule.mediaQueries())) {
-                addChildRules(mediaRule.childRules(), mediaQueryCollector, resolver, mode);
-                mediaQueryCollector.pop(mediaRule.mediaQueries());
-            }
-            continue;
-        }
-        if (is<StyleRuleFontFace>(*rule)) {
-            // Add this font face to our set.
-            if (resolver) {
-                resolver->document().fontSelector().addFontFaceRule(downcast<StyleRuleFontFace>(*rule.get()), false);
-                resolver->invalidateMatchedDeclarationsCache();
-            }
-            mediaQueryCollector.didMutateResolver();
-            continue;
-        }
-        if (is<StyleRuleKeyframes>(*rule)) {
-            if (resolver)
-                resolver->addKeyframeStyle(downcast<StyleRuleKeyframes>(*rule));
-            mediaQueryCollector.didMutateResolver();
-            continue;
-        }
-        if (is<StyleRuleSupports>(*rule) && downcast<StyleRuleSupports>(*rule).conditionIsSupported()) {
-            addChildRules(downcast<StyleRuleSupports>(*rule).childRules(), mediaQueryCollector, resolver, mode);
-            continue;
-        }
-#if ENABLE(CSS_DEVICE_ADAPTATION)
-        if (is<StyleRuleViewport>(*rule)) {
-            if (resolver)
-                resolver->viewportStyleResolver()->addViewportRule(downcast<StyleRuleViewport>(rule.get()));
-            mediaQueryCollector.didMutateResolver();
-            continue;
-        }
-#endif
-    }
-}
-
-void RuleSet::addRulesFromSheet(StyleSheetContents& sheet, const MediaQueryEvaluator& evaluator)
-{
-    auto mediaQueryCollector = MediaQueryCollector { evaluator };
-    addRulesFromSheet(sheet, mediaQueryCollector, nullptr, AddRulesMode::Normal);
-}
-
-void RuleSet::addRulesFromSheet(StyleSheetContents& sheet, MediaQuerySet* sheetQuery, const MediaQueryEvaluator& evaluator, Style::Resolver& resolver)
-{
-    auto canUseDynamicMediaQueryResolution = [&] {
-        auto mediaQueryCollector = MediaQueryCollector { evaluator, true };
-        if (mediaQueryCollector.pushAndEvaluate(sheetQuery))
-            addRulesFromSheet(sheet, mediaQueryCollector, nullptr, AddRulesMode::ResolverMutationScan);
-        return !mediaQueryCollector.didMutateResolverWithinDynamicMediaQuery;
-    }();
-
-    auto mediaQueryCollector = MediaQueryCollector { evaluator, canUseDynamicMediaQueryResolution };
-
-    if (mediaQueryCollector.pushAndEvaluate(sheetQuery)) {
-        addRulesFromSheet(sheet, mediaQueryCollector, &resolver, AddRulesMode::Normal);
-        mediaQueryCollector.pop(sheetQuery);
-    }
-
-    m_hasViewportDependentMediaQueries = mediaQueryCollector.hasViewportDependentMediaQueries;
-
-    if (mediaQueryCollector.dynamicMediaQueryRules.isEmpty())
-        return;
-
-    auto firstNewIndex = m_dynamicMediaQueryRules.size();
-    m_dynamicMediaQueryRules.appendVector(WTFMove(mediaQueryCollector.dynamicMediaQueryRules));
-
-    // Set the initial values.
-    evaluteDynamicMediaQueryRules(evaluator, firstNewIndex);
-}
-
-void RuleSet::addRulesFromSheet(StyleSheetContents& sheet, MediaQueryCollector& mediaQueryCollector, Resolver* resolver, AddRulesMode mode)
-{
-    for (auto& rule : sheet.importRules()) {
-        if (!rule->styleSheet())
-            continue;
-
-        if (mediaQueryCollector.pushAndEvaluate(rule->mediaQueries())) {
-            addRulesFromSheet(*rule->styleSheet(), mediaQueryCollector, resolver, mode);
-            mediaQueryCollector.pop(rule->mediaQueries());
-        }
-    }
-
-    addChildRules(sheet.childRules(), mediaQueryCollector, resolver, mode);
-
-    if (m_autoShrinkToFitEnabled && mode == AddRulesMode::Normal)
-        shrinkToFit();
-}
-
-void RuleSet::addStyleRule(const StyleRule& rule, MediaQueryCollector& mediaQueryCollector)
-{
-    unsigned selectorListIndex = 0;
-    for (size_t selectorIndex = 0; selectorIndex != notFound; selectorIndex = rule.selectorList().indexOfNextSelectorAfter(selectorIndex))
-        addRule(rule, selectorIndex, selectorListIndex++, &mediaQueryCollector);
-}
-
 template<typename Function>
 void RuleSet::traverseRuleDatas(Function&& function)
 {
@@ -397,11 +322,13 @@ void RuleSet::traverseRuleDatas(Function&& function)
 
     traverseMap(m_idRules);
     traverseMap(m_classRules);
+    traverseMap(m_attributeLocalNameRules);
+    traverseMap(m_attributeCanonicalLocalNameRules);
     traverseMap(m_tagLocalNameRules);
     traverseMap(m_tagLowercaseLocalNameRules);
     traverseMap(m_shadowPseudoElementRules);
     traverseVector(m_linkPseudoClassRules);
-#if ENABLE(VIDEO_TRACK)
+#if ENABLE(VIDEO)
     traverseVector(m_cuePseudoRules);
 #endif
     traverseVector(m_hostPseudoClassRules);
@@ -411,9 +338,9 @@ void RuleSet::traverseRuleDatas(Function&& function)
     traverseVector(m_universalRules);
 }
 
-Optional<DynamicMediaQueryEvaluationChanges> RuleSet::evaluteDynamicMediaQueryRules(const MediaQueryEvaluator& evaluator)
+std::optional<DynamicMediaQueryEvaluationChanges> RuleSet::evaluateDynamicMediaQueryRules(const MediaQueryEvaluator& evaluator)
 {
-    auto collectedChanges = evaluteDynamicMediaQueryRules(evaluator, 0);
+    auto collectedChanges = evaluateDynamicMediaQueryRules(evaluator, 0);
 
     if (collectedChanges.requiredFullReset)
         return { { DynamicMediaQueryEvaluationChanges::Type::ResetStyle } };
@@ -423,9 +350,10 @@ Optional<DynamicMediaQueryEvaluationChanges> RuleSet::evaluteDynamicMediaQueryRu
 
     auto& ruleSet = m_mediaQueryInvalidationRuleSetCache.ensure(collectedChanges.changedQueryIndexes, [&] {
         auto ruleSet = RuleSet::create();
-        for (auto* featureVector : collectedChanges.ruleFeatures) {
-            for (auto& feature : *featureVector)
-                ruleSet->addRule(*feature.styleRule, feature.selectorIndex, feature.selectorListIndex);
+        RuleSetBuilder builder(ruleSet, MediaQueryEvaluator(true));
+        for (auto* rules : collectedChanges.affectedRules) {
+            for (auto& rule : *rules)
+                builder.addStyleRule(rule);
         }
         return ruleSet;
     }).iterator->value;
@@ -433,10 +361,12 @@ Optional<DynamicMediaQueryEvaluationChanges> RuleSet::evaluteDynamicMediaQueryRu
     return { { DynamicMediaQueryEvaluationChanges::Type::InvalidateStyle, { ruleSet.copyRef() } } };
 }
 
-RuleSet::CollectedMediaQueryChanges RuleSet::evaluteDynamicMediaQueryRules(const MediaQueryEvaluator& evaluator, size_t startIndex)
+RuleSet::CollectedMediaQueryChanges RuleSet::evaluateDynamicMediaQueryRules(const MediaQueryEvaluator& evaluator, size_t startIndex)
 {
     CollectedMediaQueryChanges collectedChanges;
 
+    HashMap<size_t, bool, DefaultHash<size_t>, WTF::UnsignedWithZeroKeyHashTraits<size_t>> affectedRulePositionsAndResults;
+
     for (size_t i = startIndex; i < m_dynamicMediaQueryRules.size(); ++i) {
         auto& dynamicRules = m_dynamicMediaQueryRules[i];
         bool result = true;
@@ -452,32 +382,28 @@ RuleSet::CollectedMediaQueryChanges RuleSet::evaluteDynamicMediaQueryRules(const
 
             if (dynamicRules.requiresFullReset) {
                 collectedChanges.requiredFullReset = true;
-                return collectedChanges;
+                continue;
             }
 
-            traverseRuleDatas([&](RuleData& ruleData) {
-                if (!dynamicRules.affectedRulePositions.contains(ruleData.position()))
-                    return;
-                ruleData.setEnabled(result);
-            });
+            for (auto position : dynamicRules.affectedRulePositions)
+                affectedRulePositionsAndResults.add(position, result);
 
             collectedChanges.changedQueryIndexes.append(i);
-            collectedChanges.ruleFeatures.append(&dynamicRules.ruleFeatures);
+            collectedChanges.affectedRules.append(&dynamicRules.affectedRules);
         }
     }
 
-    return collectedChanges;
-}
+    if (affectedRulePositionsAndResults.isEmpty())
+        return collectedChanges;
 
-bool RuleSet::hasShadowPseudoElementRules() const
-{
-    if (!m_shadowPseudoElementRules.isEmpty())
-        return true;
-#if ENABLE(VIDEO_TRACK)
-    if (!m_cuePseudoRules.isEmpty())
-        return true;
-#endif
-    return false;
+    traverseRuleDatas([&](RuleData& ruleData) {
+        auto it = affectedRulePositionsAndResults.find(ruleData.position());
+        if (it == affectedRulePositionsAndResults.end())
+            return;
+        ruleData.setEnabled(it->value);
+    });
+
+    return collectedChanges;
 }
 
 static inline void shrinkMapVectorsToFit(RuleSet::AtomRuleMap& map)
@@ -490,88 +416,35 @@ void RuleSet::shrinkToFit()
 {
     shrinkMapVectorsToFit(m_idRules);
     shrinkMapVectorsToFit(m_classRules);
+    shrinkMapVectorsToFit(m_attributeLocalNameRules);
+    shrinkMapVectorsToFit(m_attributeCanonicalLocalNameRules);
     shrinkMapVectorsToFit(m_tagLocalNameRules);
     shrinkMapVectorsToFit(m_tagLowercaseLocalNameRules);
     shrinkMapVectorsToFit(m_shadowPseudoElementRules);
+
     m_linkPseudoClassRules.shrinkToFit();
-#if ENABLE(VIDEO_TRACK)
+#if ENABLE(VIDEO)
     m_cuePseudoRules.shrinkToFit();
 #endif
     m_hostPseudoClassRules.shrinkToFit();
     m_slottedPseudoElementRules.shrinkToFit();
+    m_partPseudoElementRules.shrinkToFit();
     m_focusPseudoClassRules.shrinkToFit();
     m_universalRules.shrinkToFit();
+
     m_pageRules.shrinkToFit();
     m_features.shrinkToFit();
+
+    for (auto& rule : m_dynamicMediaQueryRules)
+        rule.shrinkToFit();
+    m_dynamicMediaQueryRules.shrinkToFit();
+
+    m_cascadeLayers.shrinkToFit();
+    m_cascadeLayerIdentifierForRulePosition.shrinkToFit();
+    m_containerQueries.shrinkToFit();
+    m_containerQueryIdentifierForRulePosition.shrinkToFit();
+    m_resolverMutatingRulesInLayers.shrinkToFit();
 }
 
-RuleSet::MediaQueryCollector::~MediaQueryCollector() = default;
-
-bool RuleSet::MediaQueryCollector::pushAndEvaluate(const MediaQuerySet* set)
-{
-    if (!set)
-        return true;
-
-    // Only evaluate static expressions that require style rebuild.
-    MediaQueryDynamicResults dynamicResults;
-    auto mode = collectDynamic ? MediaQueryEvaluator::Mode::AlwaysMatchDynamic : MediaQueryEvaluator::Mode::Normal;
-
-    bool result = evaluator.evaluate(*set, &dynamicResults, mode);
-
-    if (!dynamicResults.viewport.isEmpty())
-        hasViewportDependentMediaQueries = true;
-
-    if (!result)
-        return false;
-
-    if (!dynamicResults.isEmpty())
-        dynamicContextStack.append({ *set });
-
-    return true;
-}
-
-void RuleSet::MediaQueryCollector::pop(const MediaQuerySet* set)
-{
-    if (!set || dynamicContextStack.isEmpty() || set != &dynamicContextStack.last().set.get())
-        return;
-
-    if (!dynamicContextStack.last().affectedRulePositions.isEmpty()) {
-        DynamicMediaQueryRules rules;
-        for (auto& context : dynamicContextStack)
-            rules.mediaQuerySets.append(context.set.get());
-
-        if (collectDynamic) {
-            auto& toAdd = dynamicContextStack.last().affectedRulePositions;
-            rules.affectedRulePositions.add(toAdd.begin(), toAdd.end());
-
-            rules.ruleFeatures = WTFMove(dynamicContextStack.last().ruleFeatures);
-            rules.ruleFeatures.shrinkToFit();
-        } else
-            rules.requiresFullReset = true;
-
-        dynamicMediaQueryRules.append(WTFMove(rules));
-    }
-
-    dynamicContextStack.removeLast();
-}
-
-void RuleSet::MediaQueryCollector::didMutateResolver()
-{
-    if (dynamicContextStack.isEmpty())
-        return;
-    didMutateResolverWithinDynamicMediaQuery = true;
-}
-
-void RuleSet::MediaQueryCollector::addRuleIfNeeded(const RuleData& ruleData)
-{
-    if (dynamicContextStack.isEmpty())
-        return;
-
-    auto& context = dynamicContextStack.last();
-    context.affectedRulePositions.append(ruleData.position());
-    context.ruleFeatures.append({ ruleData });
-}
-
-
 } // namespace Style
 } // namespace WebCore
