@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2019 Apple Inc. All rights reserved.
+ * Copyright (C) 2018-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -41,13 +41,16 @@ using namespace JSC;
 
 WTF_MAKE_ISO_ALLOCATED_IMPL(PaintWorkletGlobalScope);
 
-Ref<PaintWorkletGlobalScope> PaintWorkletGlobalScope::create(Document& document, ScriptSourceCode&& code)
+RefPtr<PaintWorkletGlobalScope> PaintWorkletGlobalScope::tryCreate(Document& document, ScriptSourceCode&& code)
 {
-    return adoptRef(*new PaintWorkletGlobalScope(document, WTFMove(code)));
+    RefPtr<VM> vm = VM::tryCreate();
+    if (!vm)
+        return nullptr;
+    return adoptRef(*new PaintWorkletGlobalScope(document, vm.releaseNonNull(), WTFMove(code)));
 }
 
-PaintWorkletGlobalScope::PaintWorkletGlobalScope(Document& document, ScriptSourceCode&& code)
-    : WorkletGlobalScope(document, WTFMove(code))
+PaintWorkletGlobalScope::PaintWorkletGlobalScope(Document& document, Ref<VM>&& vm, ScriptSourceCode&& code)
+    : WorkletGlobalScope(document, WTFMove(vm), WTFMove(code))
 {
 }
 
@@ -58,7 +61,7 @@ double PaintWorkletGlobalScope::devicePixelRatio() const
     return responsibleDocument()->domWindow()->devicePixelRatio();
 }
 
-PaintWorkletGlobalScope::PaintDefinition::PaintDefinition(const AtomString& name, JSC::JSObject* paintConstructor, Ref<CSSPaintCallback>&& paintCallback, Vector<String>&& inputProperties, Vector<String>&& inputArguments)
+PaintWorkletGlobalScope::PaintDefinition::PaintDefinition(const AtomString& name, JSC::JSObject* paintConstructor, Ref<CSSPaintCallback>&& paintCallback, Vector<AtomString>&& inputProperties, Vector<String>&& inputArguments)
     : name(name)
     , paintConstructor(paintConstructor)
     , paintCallback(WTFMove(paintCallback))
@@ -68,40 +71,39 @@ PaintWorkletGlobalScope::PaintDefinition::PaintDefinition(const AtomString& name
 }
 
 // https://drafts.css-houdini.org/css-paint-api/#registering-custom-paint
-ExceptionOr<void> PaintWorkletGlobalScope::registerPaint(JSC::JSGlobalObject& globalObject, const String& name, Strong<JSObject> paintConstructor)
+ExceptionOr<void> PaintWorkletGlobalScope::registerPaint(JSC::JSGlobalObject& globalObject, const AtomString& name, Strong<JSObject> paintConstructor)
 {
     auto& vm = paintConstructor->vm();
     JSC::JSLockHolder lock(vm);
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     // Validate that paintConstructor is a VoidFunction
-    CallData callData;
-    if (JSC::getCallData(vm, paintConstructor.get(), callData) == JSC::CallType::None)
-        return Exception { TypeError, "paintConstructor must be callable" };
+    if (!paintConstructor->isCallable())
+        return Exception { TypeError, "paintConstructor must be callable"_s };
 
     if (name.isEmpty())
-        return Exception { TypeError, "The first argument must not be the empty string" };
+        return Exception { TypeError, "The first argument must not be the empty string"_s };
 
     {
-        auto locker = holdLock(paintDefinitionLock());
+        Locker locker { paintDefinitionLock() };
 
         if (paintDefinitionMap().contains(name))
-            return Exception { InvalidModificationError, "This name has already been registered" };
+            return Exception { InvalidModificationError, "This name has already been registered"_s };
 
-        Vector<String> inputProperties;
+        Vector<AtomString> inputProperties;
 
-        JSValue inputPropertiesIterableValue = paintConstructor->get(&globalObject, Identifier::fromString(vm, "inputProperties"));
+        JSValue inputPropertiesIterableValue = paintConstructor->get(&globalObject, Identifier::fromString(vm, "inputProperties"_s));
         RETURN_IF_EXCEPTION(scope, Exception { ExistingExceptionError });
 
         if (!inputPropertiesIterableValue.isUndefined())
-            inputProperties = convert<IDLSequence<IDLDOMString>>(globalObject, inputPropertiesIterableValue);
+            inputProperties = convert<IDLSequence<IDLAtomStringAdaptor<IDLDOMString>>>(globalObject, inputPropertiesIterableValue);
         RETURN_IF_EXCEPTION(scope, Exception { ExistingExceptionError });
 
         // FIXME: Validate input properties here (step 7).
 
         Vector<String> inputArguments;
 
-        JSValue inputArgumentsIterableValue = paintConstructor->get(&globalObject, Identifier::fromString(vm, "inputArguments"));
+        JSValue inputArgumentsIterableValue = paintConstructor->get(&globalObject, Identifier::fromString(vm, "inputArguments"_s));
         RETURN_IF_EXCEPTION(scope, Exception { ExistingExceptionError });
 
         if (!inputArgumentsIterableValue.isUndefined())
@@ -110,26 +112,26 @@ ExceptionOr<void> PaintWorkletGlobalScope::registerPaint(JSC::JSGlobalObject& gl
 
         // FIXME: Parse syntax for inputArguments here (steps 11 and 12).
 
-        JSValue contextOptionsValue = paintConstructor->get(&globalObject, Identifier::fromString(vm, "contextOptions"));
+        JSValue contextOptionsValue = paintConstructor->get(&globalObject, Identifier::fromString(vm, "contextOptions"_s));
         RETURN_IF_EXCEPTION(scope, Exception { ExistingExceptionError });
         UNUSED_PARAM(contextOptionsValue);
 
         // FIXME: Convert to PaintRenderingContext2DSettings here (step 14).
 
-        if (!paintConstructor->isConstructor(vm))
-            return Exception { TypeError, "The second argument must be a constructor" };
+        if (!paintConstructor->isConstructor())
+            return Exception { TypeError, "The second argument must be a constructor"_s };
 
         JSValue prototypeValue = paintConstructor->get(&globalObject, vm.propertyNames->prototype);
         RETURN_IF_EXCEPTION(scope, Exception { ExistingExceptionError });
 
         if (!prototypeValue.isObject())
-            return Exception { TypeError, "The second argument must have a prototype that is an object" };
+            return Exception { TypeError, "The second argument must have a prototype that is an object"_s };
 
-        JSValue paintValue = prototypeValue.get(&globalObject, Identifier::fromString(vm, "paint"));
+        JSValue paintValue = prototypeValue.get(&globalObject, Identifier::fromString(vm, "paint"_s));
         RETURN_IF_EXCEPTION(scope, Exception { ExistingExceptionError });
 
         if (paintValue.isUndefined())
-            return Exception { TypeError, "The class must have a paint method" };
+            return Exception { TypeError, "The class must have a paint method"_s };
 
         RefPtr<JSCSSPaintCallback> paint = convert<IDLCallbackFunction<JSCSSPaintCallback>>(globalObject, paintValue, *jsCast<JSDOMGlobalObject*>(&globalObject));
         RETURN_IF_EXCEPTION(scope, Exception { ExistingExceptionError });
@@ -139,7 +141,7 @@ ExceptionOr<void> PaintWorkletGlobalScope::registerPaint(JSC::JSGlobalObject& gl
     }
 
     // This is for the case when we have already visited the paint definition map, and the GC is currently running in the background.
-    vm.heap.writeBarrier(&globalObject);
+    vm.writeBarrier(&globalObject);
 
     // FIXME: construct documentDefinition (step 22).
 
