@@ -53,6 +53,10 @@ WI.DOMNode = class DOMNode extends WI.Object
         this._computedRole = null;
         this._contentSecurityPolicyHash = payload.contentSecurityPolicyHash;
 
+        this._layoutFlags = [];
+        this._layoutOverlayShowing = false;
+        this._layoutOverlayColorSetting = null;
+
         if (this._nodeType === Node.DOCUMENT_NODE)
             this.ownerDocument = this;
         else
@@ -153,10 +157,26 @@ WI.DOMNode = class DOMNode extends WI.Object
 
         if (this.isMediaElement())
             WI.DOMNode.addEventListener(WI.DOMNode.Event.DidFireEvent, this._handleDOMNodeDidFireEvent, this);
+
+        // COMPATIBILITY (macOS 13.0, iOS 16.0): CSS.LayoutContextType was renamed/expanded to CSS.LayoutFlag.
+        if (!InspectorBackend.Enum.CSS.LayoutFlag) {
+            let layoutFlags = [WI.DOMNode.LayoutFlag.Rendered];
+            if (payload.layoutContextType)
+                layoutFlags.append(payload.layoutContextType);
+            this.layoutFlags = layoutFlags;
+        } else
+            this.layoutFlags = payload.layoutFlags;
     }
 
     // Static
 
+    static resetDefaultLayoutOverlayConfiguration()
+    {
+        let configuration = WI.DOMNode._defaultLayoutOverlayConfiguration;
+        configuration.nextFlexColorIndex = 0;
+        configuration.nextGridColorIndex = 0;
+    }
+
     static getFullscreenDOMEvents(domEvents)
     {
         return domEvents.reduce((accumulator, current) => {
@@ -195,6 +215,7 @@ WI.DOMNode = class DOMNode extends WI.Object
     get children() { return this._children; }
     get domEvents() { return this._domEvents; }
     get powerEfficientPlaybackRanges() { return this._powerEfficientPlaybackRanges; }
+    get layoutOverlayShowing() { return this._layoutOverlayShowing; }
 
     get attached()
     {
@@ -242,10 +263,59 @@ WI.DOMNode = class DOMNode extends WI.Object
         this._childNodeCount = count;
     }
 
+    get layoutFlags()
+    {
+        return this._layoutFlags;
+    }
+
+    set layoutFlags(layoutFlags)
+    {
+        layoutFlags ||= [];
+        console.assert(Array.isArray(layoutFlags), layoutFlags);
+        console.assert(layoutFlags.every((layoutFlag) => Object.values(WI.DOMNode.LayoutFlag).includes(layoutFlag)), layoutFlags);
+        console.assert(layoutFlags.filter((layoutFlag) => WI.DOMNode._LayoutContextTypes.includes(layoutFlag)).length <= 1, this._layoutFlags);
+
+        if (Array.shallowEqual(layoutFlags, this._layoutFlags))
+            return;
+
+        let oldLayoutContextType = this.layoutContextType;
+
+        this._layoutFlags = layoutFlags;
+
+        this.dispatchEventToListeners(WI.DOMNode.Event.LayoutFlagsChanged);
+
+        if (!this._layoutOverlayShowing)
+            return;
+
+        // The overlay is automatically hidden on the backend when the context type changes.
+        this.dispatchEventToListeners(WI.DOMNode.Event.LayoutOverlayHidden);
+
+        switch (oldLayoutContextType) {
+        case WI.DOMNode.LayoutFlag.Flex:
+            WI.settings.flexOverlayShowOrderNumbers.removeEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+            break;
+
+        case WI.DOMNode.LayoutFlag.Grid:
+            WI.settings.gridOverlayShowExtendedGridLines.removeEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+            WI.settings.gridOverlayShowLineNames.removeEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+            WI.settings.gridOverlayShowLineNumbers.removeEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+            WI.settings.gridOverlayShowTrackSizes.removeEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+            WI.settings.gridOverlayShowAreaNames.removeEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+            break;
+        }
+    }
+
+    get layoutContextType()
+    {
+        return this._layoutFlags.find((layoutFlag) => WI.DOMNode._LayoutContextTypes.includes(layoutFlag)) || null;
+    }
+
     markDestroyed()
     {
         console.assert(!this._destroyed, this);
         this._destroyed = true;
+
+        this.layoutFlags = [];
     }
 
     computedRole()
@@ -436,11 +506,21 @@ WI.DOMNode = class DOMNode extends WI.Object
     {
         console.assert(!this._destroyed, this);
         if (this._destroyed) {
+            if (!callback)
+                return Promise.reject("ERROR: node is destroyed");
+
             callback("ERROR: node is destroyed");
             return;
         }
 
         let target = WI.assumingMainTarget();
+
+        if (!callback) {
+            return target.DOMAgent.setAttributeValue(this.id, name, value).then(() => {
+                this._markUndoableState();
+            });
+        }
+
         target.DOMAgent.setAttributeValue(this.id, name, value, this._makeUndoableCallback(callback));
     }
 
@@ -553,6 +633,114 @@ WI.DOMNode = class DOMNode extends WI.Object
         target.DOMAgent.highlightNode(WI.DOMManager.buildHighlightConfig(mode), this.id);
     }
 
+    showLayoutOverlay({color} = {})
+    {
+        console.assert(!this._destroyed, this);
+        if (this._destroyed)
+            return Promise.reject("ERROR: node is destroyed");
+
+        console.assert(Object.values(WI.DOMNode._LayoutContextTypes).includes(this.layoutContextType), this);
+
+        console.assert(!color || color instanceof WI.Color, color);
+        color ||= this.layoutOverlayColor;
+
+        let target = WI.assumingMainTarget();
+        let agentCommandFunction = null;
+        let agentCommandArguments = {nodeId: this.id};
+
+        switch (this.layoutContextType) {
+        case WI.DOMNode.LayoutFlag.Grid:
+            agentCommandArguments.gridColor = color.toProtocol();
+            agentCommandArguments.showLineNames = WI.settings.gridOverlayShowLineNames.value;
+            agentCommandArguments.showLineNumbers = WI.settings.gridOverlayShowLineNumbers.value;
+            agentCommandArguments.showExtendedGridLines = WI.settings.gridOverlayShowExtendedGridLines.value;
+            agentCommandArguments.showTrackSizes = WI.settings.gridOverlayShowTrackSizes.value;
+            agentCommandArguments.showAreaNames = WI.settings.gridOverlayShowAreaNames.value;
+            agentCommandFunction = target.DOMAgent.showGridOverlay;
+
+            if (!this._layoutOverlayShowing) {
+                WI.settings.gridOverlayShowExtendedGridLines.addEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+                WI.settings.gridOverlayShowLineNames.addEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+                WI.settings.gridOverlayShowLineNumbers.addEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+                WI.settings.gridOverlayShowTrackSizes.addEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+                WI.settings.gridOverlayShowAreaNames.addEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+            }
+            break;
+
+        case WI.DOMNode.LayoutFlag.Flex:
+            agentCommandArguments.flexColor = color.toProtocol();
+            agentCommandArguments.showOrderNumbers = WI.settings.flexOverlayShowOrderNumbers.value;
+            agentCommandFunction = target.DOMAgent.showFlexOverlay;
+
+            if (!this._layoutOverlayShowing)
+                WI.settings.flexOverlayShowOrderNumbers.addEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+            break;
+        }
+
+        this._layoutOverlayShowing = true;
+
+        this.dispatchEventToListeners(WI.DOMNode.Event.LayoutOverlayShown);
+
+        console.assert(agentCommandFunction);
+        return agentCommandFunction.invoke(agentCommandArguments);
+    }
+
+    hideLayoutOverlay()
+    {
+        console.assert(!this._destroyed, this);
+        if (this._destroyed)
+            return Promise.reject("ERROR: node is destroyed");
+
+        console.assert(Object.values(WI.DOMNode._LayoutContextTypes).includes(this.layoutContextType), this);
+
+        let target = WI.assumingMainTarget();
+        let agentCommandFunction;
+        let agentCommandArguments = {nodeId: this.id};
+
+        switch (this.layoutContextType) {
+        case WI.DOMNode.LayoutFlag.Grid:
+            WI.settings.gridOverlayShowExtendedGridLines.removeEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+            WI.settings.gridOverlayShowLineNames.removeEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+            WI.settings.gridOverlayShowLineNumbers.removeEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+            WI.settings.gridOverlayShowTrackSizes.removeEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+            WI.settings.gridOverlayShowAreaNames.removeEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+
+            agentCommandFunction = target.DOMAgent.hideGridOverlay;
+            break;
+
+        case WI.DOMNode.LayoutFlag.Flex:
+            WI.settings.flexOverlayShowOrderNumbers.removeEventListener(WI.Setting.Event.Changed, this._handleLayoutOverlaySettingChanged, this);
+
+            agentCommandFunction = target.DOMAgent.hideFlexOverlay;
+            break;
+        }
+
+        console.assert(this._layoutOverlayShowing, this);
+        this._layoutOverlayShowing = false;
+
+        this.dispatchEventToListeners(WI.DOMNode.Event.LayoutOverlayHidden);
+
+        console.assert(agentCommandFunction);
+        return agentCommandFunction.invoke(agentCommandArguments);
+    }
+
+    get layoutOverlayColor()
+    {
+        this._createLayoutOverlayColorSettingIfNeeded();
+        return new WI.Color(WI.Color.Format.HSL, this._layoutOverlayColorSetting.value);
+    }
+
+    set layoutOverlayColor(color)
+    {
+        console.assert(color instanceof WI.Color, color);
+
+        this._createLayoutOverlayColorSettingIfNeeded();
+        this._layoutOverlayColorSetting.value = color.hsl;
+
+        if (this._layoutOverlayShowing)
+            this.showLayoutOverlay({color});
+    }
+
     scrollIntoView()
     {
         WI.RemoteObject.resolveNode(this).then((object) => {
@@ -757,39 +945,12 @@ WI.DOMNode = class DOMNode extends WI.Object
 
     get escapedIdSelector()
     {
-        let id = this.getAttribute("id");
-        if (!id)
-            return "";
-
-        id = id.trim();
-        if (!id.length)
-            return "";
-
-        id = CSS.escape(id);
-        if (/[\s'"]/.test(id))
-            return `[id="${id}"]`;
-
-        return `#${id}`;
+        return this._idSelector(true);
     }
 
     get escapedClassSelector()
     {
-        let classes = this.getAttribute("class");
-        if (!classes)
-            return "";
-
-        classes = classes.trim();
-        if (!classes.length)
-            return "";
-
-        let foundClasses = new Set;
-        return classes.split(/\s+/).reduce((selector, className) => {
-            if (!className.length || foundClasses.has(className))
-                return selector;
-
-            foundClasses.add(className);
-            return `${selector}.${CSS.escape(className)}`;
-        }, "");
+        return this._classSelector(true);
     }
 
     get displayName()
@@ -799,6 +960,15 @@ WI.DOMNode = class DOMNode extends WI.Object
         return this.nodeNameInCorrectCase() + this.escapedIdSelector + this.escapedClassSelector;
     }
 
+    get unescapedSelector()
+    {
+        if (this.isPseudoElement())
+            return "::" + this._pseudoType;
+
+        const shouldEscape = false;
+        return this.nodeNameInCorrectCase() + this._idSelector(shouldEscape) + this._classSelector(shouldEscape);
+    }
+
     appropriateSelectorFor(justSelector)
     {
         if (this.isPseudoElement())
@@ -1053,19 +1223,103 @@ WI.DOMNode = class DOMNode extends WI.Object
         target.CSSAgent.forcePseudoState(this.id, pseudoClasses, changed.bind(this));
     }
 
+    _markUndoableState()
+    {
+        let target = WI.assumingMainTarget();
+        if (target.hasCommand("DOM.markUndoableState"))
+            target.DOMAgent.markUndoableState();
+    }
+
     _makeUndoableCallback(callback)
     {
         return (...args) => {
-            if (!args[0]) { // error
-                let target = WI.assumingMainTarget();
-                if (target.hasCommand("DOM.markUndoableState"))
-                    target.DOMAgent.markUndoableState();
-            }
+            if (!args[0]) // error
+                this._markUndoableState();
 
             if (callback)
                 callback.apply(null, args);
         };
     }
+
+    _idSelector(shouldEscape)
+    {
+        let id = this.getAttribute("id");
+        if (!id)
+            return "";
+
+        id = id.trim();
+        if (!id.length)
+            return "";
+
+        if (shouldEscape)
+            id = CSS.escape(id);
+        if (/[\s'"]/.test(id))
+            return `[id="${id}"]`;
+
+        return `#${id}`;
+    }
+
+    _classSelector(shouldEscape) {
+        let classes = this.getAttribute("class");
+        if (!classes)
+            return "";
+
+        classes = classes.trim();
+        if (!classes.length)
+            return "";
+
+        let foundClasses = new Set;
+        return classes.split(/\s+/).reduce((selector, className) => {
+            if (!className.length || foundClasses.has(className))
+                return selector;
+
+            foundClasses.add(className);
+            return `${selector}.${(shouldEscape ? CSS.escape(className) : className)}`;
+        }, "");
+    }
+
+    _createLayoutOverlayColorSettingIfNeeded()
+    {
+        if (this._layoutOverlayColorSetting)
+            return;
+
+        let defaultConfiguration = WI.DOMNode._defaultLayoutOverlayConfiguration;
+
+        let url = this.ownerDocument.documentURL || WI.networkManager.mainFrame.url;
+
+        let nextColorIndex;
+        switch (this.layoutContextType) {
+        case WI.DOMNode.LayoutFlag.Grid:
+            nextColorIndex = defaultConfiguration.nextGridColorIndex;
+            defaultConfiguration.nextGridColorIndex = (nextColorIndex + 1) % defaultConfiguration.colors.length;
+            break;
+
+        case WI.DOMNode.LayoutFlag.Flex:
+            nextColorIndex = defaultConfiguration.nextFlexColorIndex;
+            defaultConfiguration.nextFlexColorIndex = (nextColorIndex + 1) % defaultConfiguration.colors.length;
+            break;
+        }
+
+        this._layoutOverlayColorSetting = new WI.Setting(`overlay-color-${url.hash}-${this.path().hash}`, defaultConfiguration.colors[nextColorIndex]);
+    }
+
+    _handleLayoutOverlaySettingChanged(event)
+    {
+        if (this._layoutOverlayShowing)
+            this.showLayoutOverlay();
+    }
+};
+
+WI.DOMNode._defaultLayoutOverlayConfiguration = {
+    colors: [
+        [329, 91, 70],
+        [207, 96, 69],
+        [92, 90, 64],
+        [291, 73, 68],
+        [40, 97, 57],
+    ],
+    nextFlexColorIndex: 0,
+    nextGridColorIndex: 0,
 };
 
 WI.DOMNode.Event = {
@@ -1075,6 +1329,9 @@ WI.DOMNode.Event = {
     EventListenersChanged: "dom-node-event-listeners-changed",
     DidFireEvent: "dom-node-did-fire-event",
     PowerEfficientPlaybackStateChanged: "dom-node-power-efficient-playback-state-changed",
+    LayoutFlagsChanged: "dom-node-layout-flags-changed",
+    LayoutOverlayShown: "dom-node-layout-overlay-shown",
+    LayoutOverlayHidden: "dom-node-layout-overlay-hidden",
 };
 
 WI.DOMNode.PseudoElementType = {
@@ -1094,3 +1351,17 @@ WI.DOMNode.CustomElementState = {
     Waiting: "waiting",
     Failed: "failed",
 };
+
+// Corresponds to `CSS.LayoutFlag`.
+WI.DOMNode.LayoutFlag = {
+    Rendered: "rendered",
+
+    // These are mutually exclusive.
+    Flex: "flex",
+    Grid: "grid",
+};
+
+WI.DOMNode._LayoutContextTypes = [
+    WI.DOMNode.LayoutFlag.Flex,
+    WI.DOMNode.LayoutFlag.Grid,
+];
