@@ -27,14 +27,13 @@
 
 #if PLATFORM(COCOA) && ENABLE(GPU_PROCESS) && ENABLE(MEDIA_STREAM)
 
-#include "MessageReceiver.h"
+#include "Connection.h"
 #include "RemoteSampleBufferDisplayLayerManagerMessagesReplies.h"
 #include "SampleBufferDisplayLayerIdentifier.h"
 #include <WebCore/IntSize.h>
 #include <wtf/HashMap.h>
 
 namespace IPC {
-class Connection;
 class Decoder;
 }
 
@@ -44,26 +43,40 @@ class IntSize;
 
 namespace WebKit {
 
+class GPUConnectionToWebProcess;
 class RemoteSampleBufferDisplayLayer;
 
-class RemoteSampleBufferDisplayLayerManager final : private IPC::MessageReceiver {
+class RemoteSampleBufferDisplayLayerManager final : public IPC::Connection::WorkQueueMessageReceiver {
     WTF_MAKE_FAST_ALLOCATED;
 public:
-    explicit RemoteSampleBufferDisplayLayerManager(Ref<IPC::Connection>&&);
+    static Ref<RemoteSampleBufferDisplayLayerManager> create(GPUConnectionToWebProcess& connection)
+    {
+        auto instance = adoptRef(*new RemoteSampleBufferDisplayLayerManager(connection));
+        instance->startListeningForIPC();
+        return instance;
+    }
     ~RemoteSampleBufferDisplayLayerManager();
 
-    void didReceiveLayerMessage(IPC::Connection&, IPC::Decoder&);
-    void didReceiveMessageFromWebProcess(IPC::Connection& connection, IPC::Decoder& decoder) { didReceiveMessage(connection, decoder); }
+    void close();
+
+    bool allowsExitUnderMemoryPressure() const;
 
 private:
-    // IPC::MessageReceiver
+    explicit RemoteSampleBufferDisplayLayerManager(GPUConnectionToWebProcess&);
+    void startListeningForIPC();
+
+    // IPC::Connection::WorkQueueMessageReceiver overrides.
     void didReceiveMessage(IPC::Connection&, IPC::Decoder&) final;
 
-    using LayerCreationCallback = CompletionHandler<void(Optional<LayerHostingContextID>)>&&;
+    bool dispatchMessage(IPC::Connection&, IPC::Decoder&);
+
+    using LayerCreationCallback = CompletionHandler<void(std::optional<LayerHostingContextID>)>&&;
     void createLayer(SampleBufferDisplayLayerIdentifier, bool hideRootLayer, WebCore::IntSize, LayerCreationCallback);
     void releaseLayer(SampleBufferDisplayLayerIdentifier);
 
+    GPUConnectionToWebProcess& m_connectionToWebProcess;
     Ref<IPC::Connection> m_connection;
+    Ref<WorkQueue> m_queue;
     HashMap<SampleBufferDisplayLayerIdentifier, std::unique_ptr<RemoteSampleBufferDisplayLayer>> m_layers;
 };
 
