@@ -61,7 +61,9 @@ void LoadParameters::encode(IPC::Encoder& encoder) const
     encoder << existingNetworkResourceLoadIdentifierToResume;
     encoder << isServiceWorkerLoad;
     encoder << sessionHistoryVisibility;
-
+#if ENABLE(PUBLIC_SUFFIX_LIST)
+    encoder << topPrivatelyControlledDomain;
+#endif
     platformEncode(encoder);
 }
 
@@ -155,7 +157,12 @@ bool LoadParameters::decode(IPC::Decoder& decoder, LoadParameters& data)
     
     if (!decoder.decode(data.sessionHistoryVisibility))
         return false;
-    
+
+#if ENABLE(PUBLIC_SUFFIX_LIST)
+    if (!decoder.decode(data.topPrivatelyControlledDomain))
+        return false;
+#endif
+
     if (!platformDecode(decoder, data))
         return false;
 
