@@ -52,11 +52,6 @@ void WebProcessCreationParameters::encode(IPC::Encoder& encoder) const
     encoder << injectedBundlePathExtensionHandle;
     encoder << additionalSandboxExtensionHandles;
     encoder << initializationUserData;
-#if PLATFORM(IOS_FAMILY)
-    encoder << cookieStorageDirectoryExtensionHandle;
-    encoder << containerCachesDirectoryExtensionHandle;
-    encoder << containerTemporaryDirectoryExtensionHandle;
-#endif
 #if PLATFORM(COCOA) && ENABLE(REMOTE_INSPECTOR)
     encoder << enableRemoteWebInspectorExtensionHandle;
 #endif
@@ -98,6 +93,7 @@ void WebProcessCreationParameters::encode(IPC::Encoder& encoder) const
     encoder << uiProcessBundleIdentifier;
     encoder << latencyQOS;
     encoder << throughputQOS;
+    encoder << presentingApplicationBundleIdentifier;
 #endif
     encoder << presentingApplicationPID;
 #if PLATFORM(COCOA)
@@ -166,9 +162,10 @@ void WebProcessCreationParameters::encode(IPC::Encoder& encoder) const
 
 #if HAVE(VIDEO_RESTRICTED_DECODING)
 #if PLATFORM(MAC)
-    encoder << videoDecoderExtensionHandles;
+    encoder << trustdExtensionHandle;
 #endif
-    encoder << restrictImageAndVideoDecoders;
+    encoder << enableDecodingHEIC;
+    encoder << enableDecodingAVIF;
 #endif
 
 #if PLATFORM(IOS_FAMILY)
@@ -204,6 +201,9 @@ void WebProcessCreationParameters::encode(IPC::Encoder& encoder) const
 #endif
 
     encoder << accessibilityPreferences;
+#if PLATFORM(IOS_FAMILY)
+    encoder << applicationAccessibilityEnabled;
+#endif
 
 #if PLATFORM(GTK) || PLATFORM(WPE)
     encoder << memoryPressureHandlerConfiguration;
@@ -212,11 +212,16 @@ void WebProcessCreationParameters::encode(IPC::Encoder& encoder) const
 #if USE(GLIB)
     encoder << applicationID;
     encoder << applicationName;
+#if ENABLE(REMOTE_INSPECTOR)
+    encoder << inspectorServerAddress;
+#endif
 #endif
 
 #if USE(ATSPI)
     encoder << accessibilityBusAddress;
 #endif
+
+    encoder << timeZoneOverride;
 }
 
 bool WebProcessCreationParameters::decode(IPC::Decoder& decoder, WebProcessCreationParameters& parameters)
@@ -239,28 +244,6 @@ bool WebProcessCreationParameters::decode(IPC::Decoder& decoder, WebProcessCreat
     parameters.additionalSandboxExtensionHandles = WTFMove(*additionalSandboxExtensionHandles);
     if (!decoder.decode(parameters.initializationUserData))
         return false;
-
-#if PLATFORM(IOS_FAMILY)
-    
-    std::optional<SandboxExtension::Handle> cookieStorageDirectoryExtensionHandle;
-    decoder >> cookieStorageDirectoryExtensionHandle;
-    if (!cookieStorageDirectoryExtensionHandle)
-        return false;
-    parameters.cookieStorageDirectoryExtensionHandle = WTFMove(*cookieStorageDirectoryExtensionHandle);
-
-    std::optional<SandboxExtension::Handle> containerCachesDirectoryExtensionHandle;
-    decoder >> containerCachesDirectoryExtensionHandle;
-    if (!containerCachesDirectoryExtensionHandle)
-        return false;
-    parameters.containerCachesDirectoryExtensionHandle = WTFMove(*containerCachesDirectoryExtensionHandle);
-
-    std::optional<SandboxExtension::Handle> containerTemporaryDirectoryExtensionHandle;
-    decoder >> containerTemporaryDirectoryExtensionHandle;
-    if (!containerTemporaryDirectoryExtensionHandle)
-        return false;
-    parameters.containerTemporaryDirectoryExtensionHandle = WTFMove(*containerTemporaryDirectoryExtensionHandle);
-
-#endif
 #if PLATFORM(COCOA) && ENABLE(REMOTE_INSPECTOR)
     std::optional<SandboxExtension::Handle> enableRemoteWebInspectorExtensionHandle;
     decoder >> enableRemoteWebInspectorExtensionHandle;
@@ -338,6 +321,8 @@ bool WebProcessCreationParameters::decode(IPC::Decoder& decoder, WebProcessCreat
         return false;
     if (!decoder.decode(parameters.throughputQOS))
         return false;
+    if (!decoder.decode(parameters.presentingApplicationBundleIdentifier))
+        return false;
 #endif
     if (!decoder.decode(parameters.presentingApplicationPID))
         return false;
@@ -466,17 +451,23 @@ bool WebProcessCreationParameters::decode(IPC::Decoder& decoder, WebProcessCreat
 
 #if HAVE(VIDEO_RESTRICTED_DECODING)
 #if PLATFORM(MAC)
-    std::optional<Vector<SandboxExtension::Handle>> videoDecoderExtensionHandles;
-    decoder >> videoDecoderExtensionHandles;
-    if (!videoDecoderExtensionHandles)
+    std::optional<SandboxExtension::Handle> trustdExtensionHandle;
+    decoder >> trustdExtensionHandle;
+    if (!trustdExtensionHandle)
         return false;
-    parameters.videoDecoderExtensionHandles = WTFMove(*videoDecoderExtensionHandles);
+    parameters.trustdExtensionHandle = WTFMove(*trustdExtensionHandle);
 #endif
-    std::optional<bool> restrictImageAndVideoDecoders;
-    decoder >> restrictImageAndVideoDecoders;
-    if (!restrictImageAndVideoDecoders)
+    std::optional<bool> enableDecodingHEIC;
+    decoder >> enableDecodingHEIC;
+    if (!enableDecodingHEIC)
         return false;
-    parameters.restrictImageAndVideoDecoders = *restrictImageAndVideoDecoders;
+    parameters.enableDecodingHEIC = *enableDecodingHEIC;
+    
+    std::optional<bool> enableDecodingAVIF;
+    decoder >> enableDecodingAVIF;
+    if (!enableDecodingAVIF)
+        return false;
+    parameters.enableDecodingAVIF = *enableDecodingAVIF;
 #endif
 
 #if PLATFORM(IOS_FAMILY)
@@ -561,6 +552,11 @@ bool WebProcessCreationParameters::decode(IPC::Decoder& decoder, WebProcessCreat
         return false;
     parameters.accessibilityPreferences = WTFMove(*accessibilityPreferences);
 
+#if PLATFORM(IOS_FAMILY)
+    if (!decoder.decode(parameters.applicationAccessibilityEnabled))
+        return false;
+#endif
+
 #if PLATFORM(GTK) || PLATFORM(WPE)
     std::optional<std::optional<MemoryPressureHandler::Configuration>> memoryPressureHandlerConfiguration;
     decoder >> memoryPressureHandlerConfiguration;
@@ -574,6 +570,14 @@ bool WebProcessCreationParameters::decode(IPC::Decoder& decoder, WebProcessCreat
         return false;
     if (!decoder.decode(parameters.applicationName))
         return false;
+
+#if ENABLE(REMOTE_INSPECTOR)
+    std::optional<CString> inspectorServerAddress;
+    decoder >> inspectorServerAddress;
+    if (!inspectorServerAddress)
+        return false;
+    parameters.inspectorServerAddress = WTFMove(*inspectorServerAddress);
+#endif
 #endif
 
 #if USE(ATSPI)
@@ -584,6 +588,12 @@ bool WebProcessCreationParameters::decode(IPC::Decoder& decoder, WebProcessCreat
     parameters.accessibilityBusAddress = WTFMove(*accessibilityBusAddress);
 #endif
 
+    std::optional<String> timeZoneOverride;
+    decoder >> timeZoneOverride;
+    if (!timeZoneOverride)
+        return false;
+    parameters.timeZoneOverride = WTFMove(*timeZoneOverride);
+
     return true;
 }
 
