@@ -27,6 +27,14 @@
 
 using namespace WebKit;
 
+/**
+ * WebKitWebViewSessionState: (ref-func webkit_web_view_session_state_ref) (unref-func webkit_web_view_session_state_unref)
+ *
+ * Handles serialization of a web view's browsing state.
+ *
+ * Since: 2.12
+ */
+
 struct _WebKitWebViewSessionState {
     _WebKitWebViewSessionState(SessionState&& state)
         : sessionState(WTFMove(state))
@@ -60,7 +68,7 @@ static const guint16 g_sessionStateVersion = 2;
 // Use our own enum types to ensure the serialized format even if the core enums change.
 enum ExternalURLsPolicy {
     Allow,
-    AllowExternalSchemes,
+    AllowExternalSchemesButNotAppLinks,
     NotAllow
 };
 
@@ -69,8 +77,8 @@ static inline unsigned toExternalURLsPolicy(WebCore::ShouldOpenExternalURLsPolic
     switch (policy) {
     case WebCore::ShouldOpenExternalURLsPolicy::ShouldAllow:
         return ExternalURLsPolicy::Allow;
-    case WebCore::ShouldOpenExternalURLsPolicy::ShouldAllowExternalSchemes:
-        return ExternalURLsPolicy::AllowExternalSchemes;
+    case WebCore::ShouldOpenExternalURLsPolicy::ShouldAllowExternalSchemesButNotAppLinks:
+        return ExternalURLsPolicy::AllowExternalSchemesButNotAppLinks;
     case WebCore::ShouldOpenExternalURLsPolicy::ShouldNotAllow:
         return ExternalURLsPolicy::NotAllow;
     }
@@ -83,8 +91,8 @@ static inline WebCore::ShouldOpenExternalURLsPolicy toWebCoreExternalURLsPolicy(
     switch (policy) {
     case ExternalURLsPolicy::Allow:
         return WebCore::ShouldOpenExternalURLsPolicy::ShouldAllow;
-    case ExternalURLsPolicy::AllowExternalSchemes:
-        return WebCore::ShouldOpenExternalURLsPolicy::ShouldAllowExternalSchemes;
+    case ExternalURLsPolicy::AllowExternalSchemesButNotAppLinks:
+        return WebCore::ShouldOpenExternalURLsPolicy::ShouldAllowExternalSchemesButNotAppLinks;
     case ExternalURLsPolicy::NotAllow:
         return WebCore::ShouldOpenExternalURLsPolicy::ShouldNotAllow;
     }
@@ -160,10 +168,10 @@ static inline void encodeFrameState(GVariantBuilder* sessionBuilder, const Frame
     g_variant_builder_add(sessionBuilder, "s", frameState.urlString.utf8().data());
     g_variant_builder_add(sessionBuilder, "s", frameState.originalURLString.utf8().data());
     g_variant_builder_add(sessionBuilder, "s", frameState.referrer.utf8().data());
-    g_variant_builder_add(sessionBuilder, "s", frameState.target.utf8().data());
+    g_variant_builder_add(sessionBuilder, "s", frameState.target.string().utf8().data());
     g_variant_builder_open(sessionBuilder, G_VARIANT_TYPE("as"));
-    for (const auto& state : frameState.documentState)
-        g_variant_builder_add(sessionBuilder, "s", state.utf8().data());
+    for (const auto& state : frameState.documentState())
+        g_variant_builder_add(sessionBuilder, "s", state.string().utf8().data());
     g_variant_builder_close(sessionBuilder);
     if (!frameState.stateObjectData)
         g_variant_builder_add(sessionBuilder, "may", FALSE);
@@ -303,12 +311,14 @@ static inline void decodeFrameState(GVariant* frameStateVariant, FrameState& fra
     // send an empty Referer header. Bug #159606.
     if (strlen(referrer))
         frameState.referrer = String::fromUTF8(referrer);
-    frameState.target = String::fromUTF8(target);
+    frameState.target = AtomString::fromUTF8(target);
     if (gsize documentStateLength = g_variant_iter_n_children(documentStateIter.get())) {
-        frameState.documentState.reserveInitialCapacity(documentStateLength);
+        Vector<AtomString> documentState;
+        documentState.reserveInitialCapacity(documentStateLength);
         const char* documentStateString;
         while (g_variant_iter_next(documentStateIter.get(), "&s", &documentStateString))
-            frameState.documentState.uncheckedAppend(String::fromUTF8(documentStateString));
+            documentState.uncheckedAppend(AtomString::fromUTF8(documentStateString));
+        frameState.setDocumentState(documentState);
     }
     if (stateObjectDataIter) {
         Vector<uint8_t> stateObjectVector;
@@ -447,7 +457,9 @@ WebKitWebViewSessionState* webkit_web_view_session_state_new(GBytes* data)
  * webkit_web_view_session_state_ref:
  * @state: a #WebKitWebViewSessionState
  *
- * Atomically increments the reference count of @state by one. This
+ * Atomically increments the reference count of @state by one.
+ *
+ * This
  * function is MT-safe and may be called from any thread.
  *
  * Returns: The passed in #WebKitWebViewSessionState
@@ -465,7 +477,9 @@ WebKitWebViewSessionState* webkit_web_view_session_state_ref(WebKitWebViewSessio
  * webkit_web_view_session_state_unref:
  * @state: a #WebKitWebViewSessionState
  *
- * Atomically decrements the reference count of @state by one. If the
+ * Atomically decrements the reference count of @state by one.
+ *
+ * If the
  * reference count drops to 0, all memory allocated by the #WebKitWebViewSessionState is
  * released. This function is MT-safe and may be called from any thread.
  *
