@@ -21,9 +21,14 @@
 #include "WebKitWebsiteDataManager.h"
 
 #include "WebKitCookieManagerPrivate.h"
+#include "WebKitInitialize.h"
+#include "WebKitMemoryPressureSettings.h"
+#include "WebKitMemoryPressureSettingsPrivate.h"
+#include "WebKitNetworkProxySettingsPrivate.h"
 #include "WebKitPrivate.h"
 #include "WebKitWebsiteDataManagerPrivate.h"
 #include "WebKitWebsiteDataPrivate.h"
+#include "WebProcessPool.h"
 #include "WebsiteDataFetchOption.h"
 #include "WebsiteDataStore.h"
 #include <glib/gi18n-lib.h>
@@ -35,11 +40,11 @@
 using namespace WebKit;
 
 /**
- * SECTION: WebKitWebsiteDataManager
- * @Short_description: Website data manager
- * @Title: WebKitWebsiteDataManager
+ * WebKitWebsiteDataManager:
  * @See_also: #WebKitWebContext, #WebKitWebsiteData
  *
+ * Manages data stored locally by web sites.
+ *
  * WebKitWebsiteDataManager allows you to manage the data that websites
  * can store in the client file system like databases or caches.
  * You can use WebKitWebsiteDataManager to configure the local directories
@@ -93,11 +98,6 @@ enum {
 };
 
 struct _WebKitWebsiteDataManagerPrivate {
-    ~_WebKitWebsiteDataManagerPrivate()
-    {
-        ASSERT(processPools.isEmpty());
-    }
-
     RefPtr<WebKit::WebsiteDataStore> websiteDataStore;
     GUniquePtr<char> baseDataDirectory;
     GUniquePtr<char> baseCacheDirectory;
@@ -110,9 +110,9 @@ struct _WebKitWebsiteDataManagerPrivate {
     GUniquePtr<char> itpDirectory;
     GUniquePtr<char> swRegistrationsDirectory;
     GUniquePtr<char> domCacheDirectory;
+    WebKitTLSErrorsPolicy tlsErrorsPolicy;
 
     GRefPtr<WebKitCookieManager> cookieManager;
-    Vector<WebProcessPool*> processPools;
 };
 
 WEBKIT_DEFINE_TYPE(WebKitWebsiteDataManager, webkit_website_data_manager, G_TYPE_OBJECT)
@@ -240,10 +240,16 @@ static void webkitWebsiteDataManagerConstructed(GObject* object)
         if (!priv->domCacheDirectory)
             priv->domCacheDirectory.reset(g_build_filename(priv->baseCacheDirectory.get(), "CacheStorage", nullptr));
     }
+
+    priv->tlsErrorsPolicy = WEBKIT_TLS_ERRORS_POLICY_FAIL;
+    if (priv->websiteDataStore)
+        priv->websiteDataStore->setIgnoreTLSErrors(false);
 }
 
 static void webkit_website_data_manager_class_init(WebKitWebsiteDataManagerClass* findClass)
 {
+    webkitInitialize();
+
     GObjectClass* gObjectClass = G_OBJECT_CLASS(findClass);
 
     gObjectClass->get_property = webkitWebsiteDataManagerGetProperty;
@@ -485,36 +491,21 @@ WebKit::WebsiteDataStore& webkitWebsiteDataManagerGetDataStore(WebKitWebsiteData
             configuration->setServiceWorkerRegistrationDirectory(FileSystem::stringFromFileSystemRepresentation(priv->swRegistrationsDirectory.get()));
         if (priv->domCacheDirectory)
             configuration->setCacheStorageDirectory(FileSystem::stringFromFileSystemRepresentation(priv->domCacheDirectory.get()));
-        priv->websiteDataStore = WebKit::WebsiteDataStore::create(WTFMove(configuration), PAL::SessionID::defaultSessionID());
+        priv->websiteDataStore = WebKit::WebsiteDataStore::create(WTFMove(configuration), PAL::SessionID::generatePersistentSessionID());
+        priv->websiteDataStore->setIgnoreTLSErrors(priv->tlsErrorsPolicy == WEBKIT_TLS_ERRORS_POLICY_IGNORE);
     }
 
     return *priv->websiteDataStore;
 }
 
-void webkitWebsiteDataManagerAddProcessPool(WebKitWebsiteDataManager* manager, WebProcessPool& processPool)
-{
-    ASSERT(!manager->priv->processPools.contains(&processPool));
-    manager->priv->processPools.append(&processPool);
-}
-
-void webkitWebsiteDataManagerRemoveProcessPool(WebKitWebsiteDataManager* manager, WebProcessPool& processPool)
-{
-    ASSERT(manager->priv->processPools.contains(&processPool));
-    manager->priv->processPools.removeFirst(&processPool);
-}
-
-const Vector<WebProcessPool*>& webkitWebsiteDataManagerGetProcessPools(WebKitWebsiteDataManager* manager)
-{
-    return manager->priv->processPools;
-}
-
 /**
  * webkit_website_data_manager_new:
  * @first_option_name: name of the first option to set
  * @...: value of first option, followed by more options, %NULL-terminated
  *
- * Creates a new #WebKitWebsiteDataManager with the given options. It must
- * be passed as construction parameter of a #WebKitWebContext.
+ * Creates a new #WebKitWebsiteDataManager with the given options.
+ *
+ * It must be passed as construction parameter of a #WebKitWebContext.
  *
  * Returns: (transfer full): the newly created #WebKitWebsiteDataManager
  *
@@ -533,7 +524,9 @@ WebKitWebsiteDataManager* webkit_website_data_manager_new(const gchar* firstOpti
 /**
  * webkit_website_data_manager_new_ephemeral:
  *
- * Creates an ephemeral #WebKitWebsiteDataManager. See #WebKitWebsiteDataManager:is-ephemeral for more details.
+ * Creates an ephemeral #WebKitWebsiteDataManager.
+ *
+ * See #WebKitWebsiteDataManager:is-ephemeral for more details.
  *
  * Returns: (transfer full): a new ephemeral #WebKitWebsiteDataManager.
  *
@@ -548,7 +541,9 @@ WebKitWebsiteDataManager* webkit_website_data_manager_new_ephemeral()
  * webkit_website_data_manager_is_ephemeral:
  * @manager: a #WebKitWebsiteDataManager
  *
- * Get whether a #WebKitWebsiteDataManager is ephemeral. See #WebKitWebsiteDataManager:is-ephemeral for more details.
+ * Get whether a #WebKitWebsiteDataManager is ephemeral.
+ *
+ * See #WebKitWebsiteDataManager:is-ephemeral for more details.
  *
  * Returns: %TRUE if @manager is ephemeral or %FALSE otherwise.
  *
@@ -646,7 +641,7 @@ const gchar* webkit_website_data_manager_get_disk_cache_directory(WebKitWebsiteD
 
     if (!priv->diskCacheDirectory) {
         // The default directory already has the subdirectory.
-        priv->diskCacheDirectory.reset(g_strdup(FileSystem::directoryName(WebKit::WebsiteDataStore::defaultNetworkCacheDirectory()).utf8().data()));
+        priv->diskCacheDirectory.reset(g_strdup(FileSystem::parentPath(WebKit::WebsiteDataStore::defaultNetworkCacheDirectory()).utf8().data()));
     }
     return priv->diskCacheDirectory.get();
 }
@@ -839,7 +834,9 @@ WebKitCookieManager* webkit_website_data_manager_get_cookie_manager(WebKitWebsit
  * @manager: a #WebKitWebsiteDataManager
  * @enabled: value to set
  *
- * Enable or disable Intelligent Tracking Prevention (ITP). When ITP is enabled resource load statistics
+ * Enable or disable Intelligent Tracking Prevention (ITP).
+ *
+ * When ITP is enabled resource load statistics
  * are collected and used to decide whether to allow or block third-party cookies and prevent user tracking.
  * Note that while ITP is enabled the accept policy %WEBKIT_COOKIE_POLICY_ACCEPT_NO_THIRD_PARTY is ignored and
  * %WEBKIT_COOKIE_POLICY_ACCEPT_ALWAYS is used instead. See also webkit_cookie_manager_set_accept_policy().
@@ -875,7 +872,9 @@ gboolean webkit_website_data_manager_get_itp_enabled(WebKitWebsiteDataManager* m
  * @manager: a #WebKitWebsiteDataManager
  * @enabled: value to set
  *
- * Enable or disable persistent credential storage. When enabled, which is the default for
+ * Enable or disable persistent credential storage.
+ *
+ * When enabled, which is the default for
  * non-ephemeral sessions, the network process will try to read and write HTTP authentiacation
  * credentials from persistent storage.
  *
@@ -893,6 +892,7 @@ void webkit_website_data_manager_set_persistent_credential_storage_enabled(WebKi
  * @manager: a #WebKitWebsiteDataManager
  *
  * Get whether persistent credential storage is enabled or not.
+ *
  * See also webkit_website_data_manager_set_persistent_credential_storage_enabled().
  *
  * Returns: %TRUE if persistent credential storage is enabled, or %FALSE otherwise.
@@ -906,6 +906,85 @@ gboolean webkit_website_data_manager_get_persistent_credential_storage_enabled(W
     return webkitWebsiteDataManagerGetDataStore(manager).persistentCredentialStorageEnabled();
 }
 
+/**
+ * webkit_website_data_manager_set_tls_errors_policy:
+ * @manager: a #WebKitWebsiteDataManager
+ * @policy: a #WebKitTLSErrorsPolicy
+ *
+ * Set the TLS errors policy of @manager as @policy.
+ *
+ * Since: 2.32
+ */
+void webkit_website_data_manager_set_tls_errors_policy(WebKitWebsiteDataManager* manager, WebKitTLSErrorsPolicy policy)
+{
+    g_return_if_fail(WEBKIT_IS_WEBSITE_DATA_MANAGER(manager));
+
+    if (manager->priv->tlsErrorsPolicy == policy)
+        return;
+
+    manager->priv->tlsErrorsPolicy = policy;
+    webkitWebsiteDataManagerGetDataStore(manager).setIgnoreTLSErrors(policy == WEBKIT_TLS_ERRORS_POLICY_IGNORE);
+}
+
+/**
+ * webkit_website_data_manager_get_tls_errors_policy:
+ * @manager: a #WebKitWebsiteDataManager
+ *
+ * Get the TLS errors policy of @manager.
+ *
+ * Returns: a #WebKitTLSErrorsPolicy
+ *
+ * Since: 2.32
+ */
+WebKitTLSErrorsPolicy webkit_website_data_manager_get_tls_errors_policy(WebKitWebsiteDataManager* manager)
+{
+    g_return_val_if_fail(WEBKIT_IS_WEBSITE_DATA_MANAGER(manager), WEBKIT_TLS_ERRORS_POLICY_FAIL);
+
+    return manager->priv->tlsErrorsPolicy;
+}
+
+/**
+ * webkit_website_data_manager_set_network_proxy_settings:
+ * @manager: a #WebKitWebsiteDataManager
+ * @proxy_mode: a #WebKitNetworkProxyMode
+ * @proxy_settings: (allow-none): a #WebKitNetworkProxySettings, or %NULL
+ *
+ * Set the network proxy settings to be used by connections started in @manager session.
+ *
+ * By default %WEBKIT_NETWORK_PROXY_MODE_DEFAULT is used, which means that the
+ * system settings will be used (g_proxy_resolver_get_default()).
+ * If you want to override the system default settings, you can either use
+ * %WEBKIT_NETWORK_PROXY_MODE_NO_PROXY to make sure no proxies are used at all,
+ * or %WEBKIT_NETWORK_PROXY_MODE_CUSTOM to provide your own proxy settings.
+ * When @proxy_mode is %WEBKIT_NETWORK_PROXY_MODE_CUSTOM @proxy_settings must be
+ * a valid #WebKitNetworkProxySettings; otherwise, @proxy_settings must be %NULL.
+ *
+ * Since: 2.32
+ */
+void webkit_website_data_manager_set_network_proxy_settings(WebKitWebsiteDataManager* manager, WebKitNetworkProxyMode proxyMode, WebKitNetworkProxySettings* proxySettings)
+{
+    g_return_if_fail(WEBKIT_IS_WEBSITE_DATA_MANAGER(manager));
+    g_return_if_fail((proxyMode != WEBKIT_NETWORK_PROXY_MODE_CUSTOM && !proxySettings) || (proxyMode == WEBKIT_NETWORK_PROXY_MODE_CUSTOM && proxySettings));
+
+    auto& dataStore = webkitWebsiteDataManagerGetDataStore(manager);
+    switch (proxyMode) {
+    case WEBKIT_NETWORK_PROXY_MODE_DEFAULT:
+        dataStore.setNetworkProxySettings({ });
+        break;
+    case WEBKIT_NETWORK_PROXY_MODE_NO_PROXY:
+        dataStore.setNetworkProxySettings(WebCore::SoupNetworkProxySettings(WebCore::SoupNetworkProxySettings::Mode::NoProxy));
+        break;
+    case WEBKIT_NETWORK_PROXY_MODE_CUSTOM:
+        auto settings = webkitNetworkProxySettingsGetNetworkProxySettings(proxySettings);
+        if (settings.isEmpty()) {
+            g_warning("Invalid attempt to set custom network proxy settings with an empty WebKitNetworkProxySettings. Use "
+                "WEBKIT_NETWORK_PROXY_MODE_NO_PROXY to not use any proxy or WEBKIT_NETWORK_PROXY_MODE_DEFAULT to use the default system settings");
+        } else
+            dataStore.setNetworkProxySettings(WTFMove(settings));
+        break;
+    }
+}
+
 static OptionSet<WebsiteDataType> toWebsiteDataTypes(WebKitWebsiteDataTypes types)
 {
     OptionSet<WebsiteDataType> returnValue;
@@ -925,18 +1004,16 @@ static OptionSet<WebsiteDataType> toWebsiteDataTypes(WebKitWebsiteDataTypes type
         returnValue.add(WebsiteDataType::IndexedDBDatabases);
     if (types & WEBKIT_WEBSITE_DATA_HSTS_CACHE)
         returnValue.add(WebsiteDataType::HSTSCache);
-#if ENABLE(NETSCAPE_PLUGIN_API)
-    if (types & WEBKIT_WEBSITE_DATA_PLUGIN_DATA)
-        returnValue.add(WebsiteDataType::PlugInData);
-#endif
     if (types & WEBKIT_WEBSITE_DATA_COOKIES)
         returnValue.add(WebsiteDataType::Cookies);
     if (types & WEBKIT_WEBSITE_DATA_DEVICE_ID_HASH_SALT)
         returnValue.add(WebsiteDataType::DeviceIdHashSalt);
     if (types & WEBKIT_WEBSITE_DATA_ITP)
         returnValue.add(WebsiteDataType::ResourceLoadStatistics);
+#if ENABLE(SERVICE_WORKER)
     if (types & WEBKIT_WEBSITE_DATA_SERVICE_WORKER_REGISTRATIONS)
         returnValue.add(WebsiteDataType::ServiceWorkerRegistrations);
+#endif
     if (types & WEBKIT_WEBSITE_DATA_DOM_CACHE)
         returnValue.add(WebsiteDataType::DOMCache);
     return returnValue;
@@ -1005,7 +1082,9 @@ GList* webkit_website_data_manager_fetch_finish(WebKitWebsiteDataManager* manage
  * @callback: (scope async): a #GAsyncReadyCallback to call when the request is satisfied
  * @user_data: (closure): the data to pass to callback function
  *
- * Asynchronously removes the website data of the for the given @types for websites in the given @website_data list.
+ * Asynchronously removes the website data in the given @website_data list.
+ *
+ * Asynchronously removes the website data of the given @types for websites in the given @website_data list.
  * Use webkit_website_data_manager_clear() if you want to remove the website data for all sites.
  *
  * When the operation is finished, @callback will be called. You can then call
@@ -1071,6 +1150,7 @@ gboolean webkit_website_data_manager_remove_finish(WebKitWebsiteDataManager* man
  * @user_data: (closure): the data to pass to callback function
  *
  * Asynchronously clear the website data of the given @types modified in the past @timespan.
+ *
  * If @timespan is 0, all website data will be removed.
  *
  * When the operation is finished, @callback will be called. You can then call
@@ -1113,6 +1193,13 @@ gboolean webkit_website_data_manager_clear_finish(WebKitWebsiteDataManager* mana
     return g_task_propagate_boolean(G_TASK(result), error);
 }
 
+/**
+ * WebKitITPFirstParty: (ref-func webkit_itp_first_party_ref) (unref-func webkit_itp_first_party_unref)
+ *
+ * Describes a first party origin.
+ *
+ * Since: 2.30
+ */
 struct _WebKitITPFirstParty {
     explicit _WebKitITPFirstParty(WebResourceLoadStatisticsStore::ThirdPartyDataForSpecificFirstParty&& data)
         : domain(data.firstPartyDomain.string().utf8())
@@ -1127,6 +1214,32 @@ struct _WebKitITPFirstParty {
     int referenceCount { 1 };
 };
 
+/**
+ * webkit_website_data_manager_set_memory_pressure_settings:
+ * @settings: a WebKitMemoryPressureSettings.
+ *
+ * Sets @settings as the #WebKitMemoryPressureSettings.
+ *
+ * Sets @settings as the #WebKitMemoryPressureSettings to be used by all the network
+ * processes created by any instance of #WebKitWebsiteDataManager after this function
+ * is called.
+ *
+ * Be sure to call this function before creating any #WebKitWebsiteDataManager, as network
+ * processes of existing instances are not guaranteed to receive the passed settings.
+ *
+ * The periodic check for used memory is disabled by default on network processes. This will
+ * be enabled only if custom settings have been set using this function. After that, in order
+ * to remove the custom settings and disable the periodic check, this function must be called
+ * passing %NULL as the value of @settings.
+ *
+ * Since: 2.34
+ */
+void webkit_website_data_manager_set_memory_pressure_settings(WebKitMemoryPressureSettings* settings)
+{
+    std::optional<MemoryPressureHandler::Configuration> config = settings ? std::make_optional(webkitMemoryPressureSettingsGetMemoryPressureHandlerConfiguration(settings)) : std::nullopt;
+    WebProcessPool::setNetworkProcessMemoryPressureHandlerConfiguration(config);
+}
+
 G_DEFINE_BOXED_TYPE(WebKitITPFirstParty, webkit_itp_first_party, webkit_itp_first_party_ref, webkit_itp_first_party_unref)
 
 static WebKitITPFirstParty* webkitITPFirstPartyCreate(WebResourceLoadStatisticsStore::ThirdPartyDataForSpecificFirstParty&& data)
@@ -1142,6 +1255,7 @@ static WebKitITPFirstParty* webkitITPFirstPartyCreate(WebResourceLoadStatisticsS
  * @itp_first_party: a #WebKitITPFirstParty
  *
  * Atomically increments the reference count of @itp_first_party by one.
+ *
  * This function is MT-safe and may be called from any thread.
  *
  * Returns: The passed #WebKitITPFirstParty
@@ -1161,6 +1275,7 @@ WebKitITPFirstParty* webkit_itp_first_party_ref(WebKitITPFirstParty* firstParty)
  * @itp_first_party: a #WebKitITPFirstParty
  *
  * Atomically decrements the reference count of @itp_first_party by one.
+ *
  * If the reference count drops to 0, all memory allocated by
  * #WebKitITPFirstParty is released. This function is MT-safe and may be
  * called from any thread.
@@ -1181,7 +1296,7 @@ void webkit_itp_first_party_unref(WebKitITPFirstParty* firstParty)
  * webkit_itp_first_party_get_domain:
  * @itp_first_party: a #WebKitITPFirstParty
  *
- * Get the domain name of @itp_first_party
+ * Get the domain name of @itp_first_party.
  *
  * Returns: the domain name
  *
@@ -1199,6 +1314,7 @@ const char* webkit_itp_first_party_get_domain(WebKitITPFirstParty* firstParty)
  * @itp_first_party: a #WebKitITPFirstParty
  *
  * Get whether @itp_first_party has granted website data access to its #WebKitITPThirdParty.
+ *
  * Each @WebKitITPFirstParty is created by webkit_itp_third_party_get_first_parties() and
  * therefore corresponds to exactly one #WebKitITPThirdParty.
  *
@@ -1218,6 +1334,7 @@ gboolean webkit_itp_first_party_get_website_data_access_allowed(WebKitITPFirstPa
  * @itp_first_party: a #WebKitITPFirstParty
  *
  * Get the last time a #WebKitITPThirdParty has been seen under @itp_first_party.
+ *
  * Each @WebKitITPFirstParty is created by webkit_itp_third_party_get_first_parties() and
  * therefore corresponds to exactly one #WebKitITPThirdParty.
  *
@@ -1232,6 +1349,14 @@ GDateTime* webkit_itp_first_party_get_last_update_time(WebKitITPFirstParty* firs
     return firstParty->lastUpdated.get();
 }
 
+/**
+ * WebKitITPThirdParty: (ref-func webkit_itp_first_party_ref) (unref-func webkit_itp_first_party_unref)
+ *
+ * Describes a third party origin.
+ *
+ * Since: 2.30
+ */
+
 struct _WebKitITPThirdParty {
     explicit _WebKitITPThirdParty(WebResourceLoadStatisticsStore::ThirdPartyData&& data)
         : domain(data.thirdPartyDomain.string().utf8())
@@ -1264,6 +1389,7 @@ static WebKitITPThirdParty* webkitITPThirdPartyCreate(WebResourceLoadStatisticsS
  * @itp_third_party: a #WebKitITPThirdParty
  *
  * Atomically increments the reference count of @itp_third_party by one.
+ *
  * This function is MT-safe and may be called from any thread.
  *
  * Returns: The passed #WebKitITPThirdParty
@@ -1283,6 +1409,7 @@ WebKitITPThirdParty* webkit_itp_third_party_ref(WebKitITPThirdParty* thirdParty)
  * @itp_third_party: a #WebKitITPThirdParty
  *
  * Atomically decrements the reference count of @itp_third_party by one.
+ *
  * If the reference count drops to 0, all memory allocated by
  * #WebKitITPThirdParty is released. This function is MT-safe and may be
  * called from any thread.
@@ -1303,7 +1430,7 @@ void webkit_itp_third_party_unref(WebKitITPThirdParty* thirdParty)
  * webkit_itp_third_party_get_domain:
  * @itp_third_party: a #WebKitITPThirdParty
  *
- * Get the domain name of @itp_third_party
+ * Get the domain name of @itp_third_party.
  *
  * Returns: the domain name
  *
@@ -1340,7 +1467,9 @@ GList* webkit_itp_third_party_get_first_parties(WebKitITPThirdParty* thirdParty)
  * @callback: (scope async): a #GAsyncReadyCallback to call when the request is satisfied
  * @user_data: (closure): the data to pass to callback function
  *
- * Asynchronously get the list of #WebKitITPThirdParty seen for @manager. Every #WebKitITPThirdParty
+ * Asynchronously get the list of #WebKitITPThirdParty seen for @manager.
+ *
+ * Every #WebKitITPThirdParty
  * contains the list of #WebKitITPFirstParty under which it has been seen.
  *
  * When the operation is finished, @callback will be called. You can then call
