@@ -85,7 +85,7 @@ typedef struct _WebKitWebViewPrivate WebKitWebViewPrivate;
  *   a #WebKitNavigationPolicyDecision. These decisions are useful for implementing
  *   special actions for new windows, such as forcing the new window to open
  *   in a tab when a keyboard modifier is active or handling a special
- *   target attribute on &lt;a&gt; elements.
+ *   target attribute on <a> elements.
  * @WEBKIT_POLICY_DECISION_TYPE_RESPONSE: This type of decision is used when WebKit has
  *   received a response for a network resource and is about to start the load.
  *   Note that these resources include all subresources of a page such as images
@@ -202,6 +202,23 @@ typedef enum {
     WEBKIT_MEDIA_CAPTURE_STATE_MUTED,
 } WebKitMediaCaptureState;
 
+/**
+ * WebKitWebExtensionMode:
+ * @WEBKIT_WEB_EXTENSION_MODE_NONE: Not for an extension.
+ * @WEBKIT_WEB_EXTENSION_MODE_MANIFESTV2: For a ManifestV2 extension.
+ * @WEBKIT_WEB_EXTENSION_MODE_MANIFESTV3: For a ManifestV3 extension.
+ *
+ * Enum values used for setting if a #WebKitWebView is intended for
+ * WebExtensions.
+ *
+ * Since: 2.38
+ */
+typedef enum {
+    WEBKIT_WEB_EXTENSION_MODE_NONE,
+    WEBKIT_WEB_EXTENSION_MODE_MANIFESTV2,
+    WEBKIT_WEB_EXTENSION_MODE_MANIFESTV3,
+} WebKitWebExtensionMode;
+
 struct _WebKitWebView {
     GObject parent;
 
@@ -212,6 +229,7 @@ struct _WebKitWebView {
 struct _WebKitWebViewClass {
     GObjectClass parent;
 
+    /*< public >*/
     void           (* load_changed)                (WebKitWebView               *web_view,
                                                     WebKitLoadEvent              load_event);
     gboolean       (* load_failed)                 (WebKitWebView               *web_view,
@@ -270,6 +288,7 @@ struct _WebKitWebViewClass {
                                                     WebKitOptionMenu            *menu,
                                                     WebKitRectangle             *rectangle);
 
+    /*< private >*/
     void (*_webkit_reserved0) (void);
     void (*_webkit_reserved1) (void);
     void (*_webkit_reserved2) (void);
@@ -471,6 +490,15 @@ webkit_web_view_run_javascript_in_world_finish       (WebKitWebView
                                                       GAsyncResult              *result,
                                                       GError                   **error);
 
+WEBKIT_API void
+webkit_web_view_run_async_javascript_function_in_world (WebKitWebView            *web_view,
+                                                        const gchar              *body,
+                                                        GVariant                 *arguments,
+                                                        const char               *world_name,
+                                                        GCancellable*             cancellable,
+                                                        GAsyncReadyCallback       callback,
+                                                        gpointer                  user_data);
+
 WEBKIT_API void
 webkit_web_view_run_javascript_from_gresource        (WebKitWebView             *web_view,
                                                       const gchar               *resource,
@@ -614,6 +642,12 @@ WEBKIT_API void
 webkit_web_view_set_display_capture_state            (WebKitWebView             *web_view,
                                                       WebKitMediaCaptureState    state);
 
+WEBKIT_API WebKitWebExtensionMode
+webkit_web_view_get_web_extension_mode               (WebKitWebView             *web_view);
+
+WEBKIT_API const gchar*
+webkit_web_view_get_default_content_security_policy  (WebKitWebView             *web_view);
+
 G_END_DECLS
 
 #endif
