@@ -20,16 +20,17 @@
 
 #if ENABLE(BUBBLEWRAP_SANDBOX)
 
+#include "XDGDBusProxy.h"
 #include <WebCore/PlatformDisplay.h>
 #include <fcntl.h>
 #include <glib.h>
 #include <seccomp.h>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
-#include <unistd.h>
 #include <wtf/FileSystem.h>
+#include <wtf/NeverDestroyed.h>
+#include <wtf/UUID.h>
 #include <wtf/UniStdExtras.h>
-#include <wtf/glib/GLibUtilities.h>
 #include <wtf/glib/GRefPtr.h>
 #include <wtf/glib/GUniquePtr.h>
 
@@ -43,12 +44,6 @@
 #include "WaylandCompositor.h"
 #endif
 
-#if PLATFORM(GTK)
-#define BASE_DIRECTORY "webkitgtk"
-#elif PLATFORM(WPE)
-#define BASE_DIRECTORY "wpe"
-#endif
-
 #if !defined(MFD_ALLOW_SEALING) && HAVE(LINUX_MEMFD_H)
 
 // These defines were added in glibc 2.27, the same release that added memfd_create.
@@ -70,6 +65,12 @@ static int memfd_create(const char* name, unsigned flags)
 }
 #endif // #if !defined(MFD_ALLOW_SEALING) && HAVE(LINUX_MEMFD_H)
 
+#if PLATFORM(GTK)
+#define BASE_DIRECTORY "webkitgtk"
+#elif PLATFORM(WPE)
+#define BASE_DIRECTORY "wpe"
+#endif
+
 namespace WebKit {
 using namespace WebCore;
 
@@ -109,8 +110,7 @@ static int createSealedMemFdWithData(const char* name, gconstpointer data, size_
     return fd;
 }
 
-static int
-argsToFd(const Vector<CString>& args, const char *name)
+int argumentsToFileDescriptor(const Vector<CString>& args, const char* name)
 {
     GString* buffer = g_string_new(nullptr);
 
@@ -129,20 +129,31 @@ argsToFd(const Vector<CString>& args, const char *name)
     return memfd;
 }
 
-static const char* applicationId(GError** error)
+static String effectiveApplicationId()
 {
-    GApplication* app = g_application_get_default();
-    if (!app) {
-        g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_INVALID_DATA, "GApplication is required.");
-        return nullptr;
+    if (auto* app = g_application_get_default()) {
+        if (const char* appID = g_application_get_application_id(app))
+            return String::fromUTF8(appID);
     }
 
-    const char* appID = g_application_get_application_id(app);
-    if (!appID) {
-        g_set_error_literal(error, G_IO_ERROR, G_IO_ERROR_INVALID_DATA, "GApplication must have a valid ID.");
-        return nullptr;
+    const char* programName = g_get_prgname();
+    if (programName && g_application_id_is_valid(programName))
+        return String::fromUTF8(programName);
+
+    // There must be some id for xdg-desktop-portal to function.
+    // xdg-desktop-portal uses this id for permissions.
+    // This creates a somewhat reliable id based on the executable path
+    // which will avoid potentially gaining permissions from another app
+    // and won't flood xdg-desktop-portal with new ids.
+    if (auto executablePath = FileSystem::currentExecutablePath(); !executablePath.isNull()) {
+        GUniquePtr<char> digest(g_compute_checksum_for_data(G_CHECKSUM_SHA256, reinterpret_cast<const uint8_t*>(executablePath.data()), executablePath.length()));
+        return makeString("org.webkit.app-", digest.get());
     }
-    return appID;
+
+    // If it is not possible to obtain the executable path, generate
+    // a random identifier as a fallback.
+    auto uuid = UUID::createVersion4Weak();
+    return makeString("org.webkit.app-", uuid.toString());
 }
 
 static int createFlatpakInfo()
@@ -151,159 +162,29 @@ static int createFlatpakInfo()
     static size_t size;
 
     if (!data.get()) {
-        // xdg-desktop-portal relates your name to certain permissions so we want
-        // them to be application unique which is best done via GApplication.
-        GUniqueOutPtr<GError> error;
-        const char* appID = applicationId(&error.outPtr());
-        if (!appID)
-            g_error("Unable to configure xdg-desktop-portal access in the WebKit sandbox: %s", error->message);
-
         GUniquePtr<GKeyFile> keyFile(g_key_file_new());
-        g_key_file_set_string(keyFile.get(), "Application", "name", appID);
+        g_key_file_set_string(keyFile.get(), "Application", "name", effectiveApplicationId().utf8().data());
         data->reset(g_key_file_to_data(keyFile.get(), &size, nullptr));
     }
 
     return createSealedMemFdWithData("flatpak-info", data->get(), size);
 }
 
-class XDGDBusProxyLauncher {
-public:
-    void setAddress(const char* dbusAddress)
-    {
-        if (!dbusAddress || !g_str_has_prefix(dbusAddress, "unix:"))
-            return;
-
-        GUniquePtr<char> appRunDir(g_build_filename(g_get_user_runtime_dir(), BASE_DIRECTORY, nullptr));
-        m_proxyPath = makeProxyPath(appRunDir.get());
-
-        m_socket = dbusAddress;
-        m_path = dbusAddressToPath(dbusAddress);
-    }
-
-    bool isRunning() const { return m_isRunning; };
-    const CString& path() const { return m_path; };
-    const CString& proxyPath() const { return m_proxyPath; };
-
-    void setPermissions(Vector<CString>&& permissions)
-    {
-        RELEASE_ASSERT_WITH_SECURITY_IMPLICATION(!isRunning());
-        m_permissions = WTFMove(permissions);
-    };
-
-    void launch(bool enableLogging)
-    {
-        RELEASE_ASSERT_WITH_SECURITY_IMPLICATION(!isRunning());
-
-        if (m_socket.isNull() || m_proxyPath.isNull())
-            return;
-
-        int syncFds[2];
-        if (pipe(syncFds) == -1)
-            g_error("Failed to make syncfds for dbus-proxy: %s", g_strerror(errno));
-        setCloseOnExec(syncFds[0]);
-
-        GUniquePtr<char> syncFdStr(g_strdup_printf("--fd=%d", syncFds[1]));
-
-        Vector<CString> proxyArgs = {
-            m_socket, m_proxyPath,
-            "--filter",
-            syncFdStr.get(),
-        };
-
-        if (enableLogging)
-            proxyArgs.append("--log");
-
-        proxyArgs.appendVector(m_permissions);
-
-        int proxyFd = argsToFd(proxyArgs, "dbus-proxy");
-        GUniquePtr<char> proxyArgsStr(g_strdup_printf("--args=%d", proxyFd));
-
-        // We have to run xdg-dbus-proxy under bubblewrap because we need /.flatpak-info to exist in
-        // xdg-dbus-proxy's mount namespace. Portals may use this as a trusted way to get the
-        // sandboxed process's application ID, and will break if it's missing.
-        Vector<CString> args = {
-            DBUS_PROXY_EXECUTABLE,
-            proxyArgsStr.get(),
-        };
-
-        int nargs = args.size() + 1;
-        int i = 0;
-        char** argv = g_newa(char*, nargs);
-        for (const auto& arg : args)
-            argv[i++] = const_cast<char*>(arg.data());
-        argv[i] = nullptr;
-
-        GRefPtr<GSubprocessLauncher> launcher = adoptGRef(g_subprocess_launcher_new(G_SUBPROCESS_FLAGS_INHERIT_FDS));
-        g_subprocess_launcher_take_fd(launcher.get(), proxyFd, proxyFd);
-        g_subprocess_launcher_take_fd(launcher.get(), syncFds[1], syncFds[1]);
-
-        // We are purposefully leaving syncFds[0] open here.
-        // xdg-dbus-proxy will exit() itself once that is closed on our exit
-
-        ProcessLauncher::LaunchOptions launchOptions;
-        launchOptions.processType = ProcessLauncher::ProcessType::DBusProxy;
-        if (!m_path.isNull())
-            launchOptions.extraSandboxPaths.add(m_path, SandboxPermission::ReadOnly);
-        GUniqueOutPtr<GError> error;
-        GRefPtr<GSubprocess> process = bubblewrapSpawn(launcher.get(), launchOptions, argv, &error.outPtr());
-        if (!process.get())
-            g_error("Failed to start dbus proxy: %s", error->message);
-
-        char out;
-        // We need to ensure the proxy has created the socket.
-        // FIXME: This is more blocking IO.
-        if (read(syncFds[0], &out, 1) != 1)
-            g_error("Failed to fully launch dbus-proxy %s", g_strerror(errno));
-
-        m_isRunning = true;
-    };
-
-private:
-    static CString makeProxyPath(const char* appRunDir)
-    {
-        if (g_mkdir_with_parents(appRunDir, 0700) == -1) {
-            g_warning("Failed to mkdir for dbus proxy (%s): %s", appRunDir, g_strerror(errno));
-            return { };
-        }
-
-        GUniquePtr<char> proxySocketTemplate(g_build_filename(appRunDir, "dbus-proxy-XXXXXX", nullptr));
-        int fd;
-        if ((fd = g_mkstemp(proxySocketTemplate.get())) == -1) {
-            g_warning("Failed to make socket file for dbus proxy: %s", g_strerror(errno));
-            return { };
-        }
-
-        close(fd);
-        return CString(proxySocketTemplate.get());
-    };
-
-    static CString dbusAddressToPath(const char* address)
-    {
-        const char* path = strstr(address, "path=");
-        if (!path)
-            return { };
-
-        path += strlen("path=");
-        const char* pathEnd = path;
-        while (*pathEnd && *pathEnd != ',')
-            pathEnd++;
-
-        return CString(path, pathEnd - path);
-    }
-
-    CString m_socket;
-    CString m_path;
-    CString m_proxyPath;
-    bool m_isRunning { false };
-    Vector<CString> m_permissions;
-};
-
 enum class BindFlags {
     ReadOnly,
     ReadWrite,
     Device,
 };
 
+static void bindSymlinksRealPath(Vector<CString>& args, const String& path, const char* bindOption = "--ro-bind")
+{
+    auto realPath = FileSystem::realPath(path);
+    if (path != realPath) {
+        CString rpath = realPath.utf8();
+        args.appendVector(Vector<CString>({ bindOption, rpath.data(), rpath.data() }));
+    }
+}
+
 static void bindIfExists(Vector<CString>& args, const char* path, BindFlags bindFlags = BindFlags::ReadOnly)
 {
     if (!path || path[0] == '\0')
@@ -316,19 +197,36 @@ static void bindIfExists(Vector<CString>& args, const char* path, BindFlags bind
         bindType = "--ro-bind-try";
     else
         bindType = "--bind-try";
-    args.appendVector(Vector<CString>({ bindType, path, path }));
+
+    // Canonicalize the source path, otherwise a symbolic link could
+    // point to a location outside of the namespace.
+    bindSymlinksRealPath(args, String::fromUTF8(path), bindType);
+
+    // As /etc is exposed wholesale, do not layer extraneous bind
+    // directives on top, which could fail in the presence of symbolic
+    // links.
+    if (!g_str_has_prefix(path, "/etc/"))
+        args.appendVector(Vector<CString>({ bindType, path, path }));
 }
 
-static void bindDBusSession(Vector<CString>& args, XDGDBusProxyLauncher& proxy)
+static const char* dbusProxyDirectory()
 {
-    if (!proxy.isRunning())
-        proxy.setAddress(g_getenv("DBUS_SESSION_BUS_ADDRESS"));
+    static GUniquePtr<char> path(g_build_filename(g_get_user_runtime_dir(), BASE_DIRECTORY, nullptr));
+    return path.get();
+}
 
-    if (!proxy.proxyPath().isNull() && !proxy.path().isNull()) {
-        args.appendVector(Vector<CString>({
-            "--bind", proxy.proxyPath(), proxy.path(),
-        }));
-    }
+static void bindDBusSession(Vector<CString>& args, XDGDBusProxy& dbusProxy, bool allowPortals)
+{
+    auto dbusSessionProxyPath = dbusProxy.dbusSessionProxy(BASE_DIRECTORY, allowPortals ? XDGDBusProxy::AllowPortals::Yes : XDGDBusProxy::AllowPortals::No);
+    if (!dbusSessionProxyPath)
+        return;
+
+    GUniquePtr<char> sandboxedSessionBusPath(g_build_filename(dbusProxyDirectory(), "bus", nullptr));
+    GUniquePtr<char> proxyAddress(g_strdup_printf("unix:path=%s", sandboxedSessionBusPath.get()));
+    args.appendVector(Vector<CString> {
+        "--ro-bind", *dbusSessionProxyPath, sandboxedSessionBusPath.get(),
+        "--setenv", "DBUS_SESSION_BUS_ADDRESS", proxyAddress.get()
+    });
 }
 
 #if PLATFORM(X11)
@@ -428,6 +326,7 @@ static void bindFonts(Vector<CString>& args)
     const char* homeDir = g_get_home_dir();
     const char* dataDir = g_get_user_data_dir();
     const char* cacheDir = g_get_user_cache_dir();
+    const char* const * dataDirs = g_get_system_data_dirs();
 
     // Configs can include custom dirs but then we have to parse them...
     GUniquePtr<char> fontConfig(g_build_filename(configDir, "fontconfig", nullptr));
@@ -444,6 +343,10 @@ static void bindFonts(Vector<CString>& args)
     bindIfExists(args, fontHomeConfigDir.get());
     bindIfExists(args, fontData.get());
     bindIfExists(args, fontHomeData.get());
+    for (auto* dataDir = dataDirs; dataDir && *dataDir; dataDir++) {
+        GUniquePtr<char> fontDataDir(g_build_filename(*dataDir, "fonts", nullptr));
+        bindIfExists(args, fontDataDir.get());
+    }
     bindIfExists(args, "/var/cache/fontconfig"); // Used by Debian.
 }
 
@@ -463,61 +366,21 @@ static void bindGtkData(Vector<CString>& args)
     bindIfExists(args, themeHomeData.get());
     bindIfExists(args, iconHomeData.get());
 }
+#endif
 
-static void bindA11y(Vector<CString>& args)
+#if ENABLE(ACCESSIBILITY)
+static void bindA11y(Vector<CString>& args, XDGDBusProxy& dbusProxy)
 {
-    static XDGDBusProxyLauncher proxy;
+    GUniquePtr<char> sandboxedAccessibilityBusPath(g_build_filename(dbusProxyDirectory(), "at-spi-bus", nullptr));
+    auto accessibilityProxyPath = dbusProxy.accessibilityProxy(BASE_DIRECTORY, sandboxedAccessibilityBusPath.get());
+    if (!accessibilityProxyPath)
+        return;
 
-    if (!proxy.isRunning()) {
-        // FIXME: Avoid blocking IO... (It is at least a one-time cost)
-        GRefPtr<GDBusConnection> sessionBus = adoptGRef(g_bus_get_sync(G_BUS_TYPE_SESSION, nullptr, nullptr));
-        if (!sessionBus.get())
-            return;
-
-        GRefPtr<GDBusMessage> msg = adoptGRef(g_dbus_message_new_method_call(
-            "org.a11y.Bus", "/org/a11y/bus", "org.a11y.Bus", "GetAddress"));
-        g_dbus_message_set_body(msg.get(), g_variant_new("()"));
-        GRefPtr<GDBusMessage> reply = adoptGRef(g_dbus_connection_send_message_with_reply_sync(
-            sessionBus.get(), msg.get(),
-            G_DBUS_SEND_MESSAGE_FLAGS_NONE,
-            30000,
-            nullptr,
-            nullptr,
-            nullptr));
-
-        if (reply.get()) {
-            GUniqueOutPtr<GError> error;
-            if (g_dbus_message_to_gerror(reply.get(), &error.outPtr())) {
-                if (!g_error_matches(error.get(), G_DBUS_ERROR, G_DBUS_ERROR_SERVICE_UNKNOWN))
-                    g_warning("Can't find a11y bus: %s", error->message);
-            } else {
-                GUniqueOutPtr<char> a11yAddress;
-                g_variant_get(g_dbus_message_get_body(reply.get()), "(s)", &a11yAddress.outPtr());
-                proxy.setAddress(a11yAddress.get());
-            }
-        }
-
-        proxy.setPermissions({
-            "--sloppy-names",
-            "--call=org.a11y.atspi.Registry=org.a11y.atspi.Socket.Embed@/org/a11y/atspi/accessible/root",
-            "--call=org.a11y.atspi.Registry=org.a11y.atspi.Socket.Unembed@/org/a11y/atspi/accessible/root",
-            "--call=org.a11y.atspi.Registry=org.a11y.atspi.Registry.GetRegisteredEvents@/org/a11y/atspi/registry",
-            "--call=org.a11y.atspi.Registry=org.a11y.atspi.DeviceEventController.GetKeystrokeListeners@/org/a11y/atspi/registry/deviceeventcontroller",
-            "--call=org.a11y.atspi.Registry=org.a11y.atspi.DeviceEventController.GetDeviceEventListeners@/org/a11y/atspi/registry/deviceeventcontroller",
-            "--call=org.a11y.atspi.Registry=org.a11y.atspi.DeviceEventController.NotifyListenersSync@/org/a11y/atspi/registry/deviceeventcontroller",
-            "--call=org.a11y.atspi.Registry=org.a11y.atspi.DeviceEventController.NotifyListenersAsync@/org/a11y/atspi/registry/deviceeventcontroller",
-        });
-
-        proxy.launch(!g_strcmp0(g_getenv("WEBKIT_ENABLE_A11Y_DBUS_PROXY_LOGGING"), "1"));
-    }
-
-    if (!proxy.proxyPath().isNull()) {
-        GUniquePtr<char> proxyAddress(g_strdup_printf("unix:path=%s", proxy.proxyPath().data()));
-        args.appendVector(Vector<CString>({
-            "--ro-bind", proxy.proxyPath(), proxy.proxyPath(),
-            "--setenv", "AT_SPI_BUS_ADDRESS", proxyAddress.get(),
-        }));
-    }
+    GUniquePtr<char> proxyAddress(g_strdup_printf("unix:path=%s", sandboxedAccessibilityBusPath.get()));
+    args.appendVector(Vector<CString> {
+        "--ro-bind", *accessibilityProxyPath, sandboxedAccessibilityBusPath.get(),
+        "--setenv", "AT_SPI_BUS_ADDRESS", proxyAddress.get()
+    });
 }
 #endif
 
@@ -591,17 +454,6 @@ static void bindV4l(Vector<CString>& args)
     }));
 }
 
-static void bindSymlinksRealPath(Vector<CString>& args, const char* path)
-{
-    char realPath[PATH_MAX];
-
-    if (realpath(path, realPath) && strcmp(path, realPath)) {
-        args.appendVector(Vector<CString>({
-            "--ro-bind", realPath, realPath,
-        }));
-    }
-}
-
 // Translate a libseccomp error code into an error message. libseccomp
 // mostly returns negative errno values such as -ENOMEM, but some
 // standard errno values are used for non-standard purposes where their
@@ -792,6 +644,30 @@ static bool shouldUnshareNetwork(ProcessLauncher::ProcessType processType)
     return true;
 }
 
+static std::optional<CString> directoryContainingDBusSocket(const char* dbusAddress)
+{
+    if (!dbusAddress || !g_str_has_prefix(dbusAddress, "unix:"))
+        return std::nullopt;
+
+    if (const char* pathStart = strstr(dbusAddress, "path=")) {
+        pathStart += strlen("path=");
+
+        const char* pathEnd = pathStart;
+        while (*pathEnd && *pathEnd != ',')
+            pathEnd++;
+
+        CString path(pathStart, pathEnd - pathStart);
+        GRefPtr<GFile> file = adoptGRef(g_file_new_for_path(path.data()));
+        GRefPtr<GFile> parent = adoptGRef(g_file_get_parent(file.get()));
+        if (!parent)
+            return std::nullopt;
+
+        return { g_file_peek_path(parent.get()) };
+    }
+
+    return std::nullopt;
+}
+
 static void addExtraPaths(const HashMap<CString, SandboxPermission>& paths, Vector<CString>& args)
 {
     for (const auto& pathAndPermission : paths) {
@@ -858,16 +734,31 @@ GRefPtr<GSubprocess> bubblewrapSpawn(GSubprocessLauncher* launcher, const Proces
         "--ro-bind-try", PKGLIBEXECDIR, PKGLIBEXECDIR,
     };
 
+    addExtraPaths(launchOptions.extraSandboxPaths, sandboxArgs);
+
     if (launchOptions.processType == ProcessLauncher::ProcessType::DBusProxy) {
         sandboxArgs.appendVector(Vector<CString>({
             "--ro-bind", DBUS_PROXY_EXECUTABLE, DBUS_PROXY_EXECUTABLE,
-            // This is a lot of access, but xdg-dbus-proxy is trusted so that's OK. It's sandboxed
-            // only because we have to mount .flatpak-info in its mount namespace. The user rundir
-            // is where we mount our proxy socket.
-            "--bind", runDir, runDir,
+            "--bind", dbusProxyDirectory(), dbusProxyDirectory(),
         }));
 
-        addExtraPaths(launchOptions.extraSandboxPaths, sandboxArgs);
+        // xdg-dbus-proxy is trusted, so it's OK to mount the directories that contain the session
+        // bus and a11y bus sockets wherever they may be. xdg-dbus-proxy is sandboxed only because
+        // we have to mount .flatpak-info in its mount namespace so that portals may use it as a
+        // trusted way to get the app ID of the process that is using it.
+        if (auto sessionBusDirectory = directoryContainingDBusSocket(g_getenv("DBUS_SESSION_BUS_ADDRESS"))) {
+            sandboxArgs.appendVector(Vector<CString>({
+                "--bind", *sessionBusDirectory, *sessionBusDirectory,
+            }));
+        }
+
+#if ENABLE(ACCESSIBILITY)
+        if (auto a11yBusDirectory = directoryContainingDBusSocket(PlatformDisplay::sharedDisplay().accessibilityBusAddress().utf8().data())) {
+            sandboxArgs.appendVector(Vector<CString>({
+                "--bind", *a11yBusDirectory, *a11yBusDirectory,
+            }));
+        }
+#endif
     }
 
     if (shouldUnshareNetwork(launchOptions.processType))
@@ -885,8 +776,8 @@ GRefPtr<GSubprocess> bubblewrapSpawn(GSubprocessLauncher* launcher, const Proces
         }));
     }
 
-    bindSymlinksRealPath(sandboxArgs, "/etc/resolv.conf");
-    bindSymlinksRealPath(sandboxArgs, "/etc/localtime");
+    bindSymlinksRealPath(sandboxArgs, "/etc/resolv.conf"_s);
+    bindSymlinksRealPath(sandboxArgs, "/etc/localtime"_s);
 
     // xdg-desktop-portal defaults to assuming you are host application with
     // full permissions unless it can identify you as a snap or flatpak.
@@ -903,8 +794,6 @@ GRefPtr<GSubprocess> bubblewrapSpawn(GSubprocessLauncher* launcher, const Proces
     }
 
     if (launchOptions.processType == ProcessLauncher::ProcessType::Web) {
-        static XDGDBusProxyLauncher proxy;
-
 #if PLATFORM(WAYLAND) && USE(EGL)
         if (PlatformDisplay::sharedDisplay().type() == PlatformDisplay::Type::Wayland) {
             bindWayland(sandboxArgs);
@@ -916,16 +805,16 @@ GRefPtr<GSubprocess> bubblewrapSpawn(GSubprocessLauncher* launcher, const Proces
             bindX11(sandboxArgs);
 #endif
 
-        addExtraPaths(launchOptions.extraSandboxPaths, sandboxArgs);
-
-        Vector<String> extraPaths = { "applicationCacheDirectory", "mediaKeysDirectory", "waylandSocket", "webSQLDatabaseDirectory" };
+        Vector<String> extraPaths = { "applicationCacheDirectory"_s, "mediaKeysDirectory"_s, "waylandSocket"_s };
         for (const auto& path : extraPaths) {
             String extraPath = launchOptions.extraInitializationData.get(path);
             if (!extraPath.isEmpty())
                 sandboxArgs.appendVector(Vector<CString>({ "--bind-try", extraPath.utf8(), extraPath.utf8() }));
         }
 
-        bindDBusSession(sandboxArgs, proxy);
+        static std::unique_ptr<XDGDBusProxy> dbusProxy = makeUnique<XDGDBusProxy>();
+        if (dbusProxy)
+            bindDBusSession(sandboxArgs, *dbusProxy, flatpakInfoFd != -1);
         // FIXME: We should move to Pipewire as soon as viable, Pulse doesn't restrict clients atm.
         bindPulse(sandboxArgs);
         bindSndio(sandboxArgs);
@@ -934,23 +823,15 @@ GRefPtr<GSubprocess> bubblewrapSpawn(GSubprocessLauncher* launcher, const Proces
         bindOpenGL(sandboxArgs);
         // FIXME: This is also fixed by Pipewire once in use.
         bindV4l(sandboxArgs);
+#if ENABLE(ACCESSIBILITY)
+        if (dbusProxy)
+            bindA11y(sandboxArgs, *dbusProxy);
+#endif
 #if PLATFORM(GTK)
-        bindA11y(sandboxArgs);
         bindGtkData(sandboxArgs);
 #endif
-
-        if (!proxy.isRunning()) {
-            Vector<CString> permissions = {
-                // GStreamers plugin install helper.
-                "--call=org.freedesktop.PackageKit=org.freedesktop.PackageKit.Modify2.InstallGStreamerResources@/org/freedesktop/PackageKit"
-            };
-            if (flatpakInfoFd != -1) {
-                // xdg-desktop-portal used by GTK and us.
-                permissions.append("--talk=org.freedesktop.portal.Desktop");
-            }
-            proxy.setPermissions(WTFMove(permissions));
-            proxy.launch(!g_strcmp0(g_getenv("WEBKIT_ENABLE_DBUS_PROXY_LOGGING"), "1"));
-        }
+        if (dbusProxy && !dbusProxy->launch())
+            dbusProxy = nullptr;
     } else {
         // Only X11 users need this for XShm which is only the Web process.
         sandboxArgs.append("--unshare-ipc");
@@ -963,7 +844,7 @@ GRefPtr<GSubprocess> bubblewrapSpawn(GSubprocessLauncher* launcher, const Proces
         bindIfExists(sandboxArgs, parentDir.utf8().data());
     }
 
-    CString executablePath = getCurrentExecutablePath();
+    CString executablePath = FileSystem::currentExecutablePath();
     if (!executablePath.isNull()) {
         // Our executable is `/foo/bar/bin/Process`, we want `/foo/bar` as a usable prefix
         String parentDir = FileSystem::parentPath(FileSystem::parentPath(FileSystem::stringFromFileSystemRepresentation(executablePath.data())));
@@ -976,7 +857,7 @@ GRefPtr<GSubprocess> bubblewrapSpawn(GSubprocessLauncher* launcher, const Proces
     g_subprocess_launcher_take_fd(launcher, seccompFd, seccompFd);
     sandboxArgs.appendVector(Vector<CString>({ "--seccomp", fdStr.get() }));
 
-    int bwrapFd = argsToFd(sandboxArgs, "bwrap");
+    int bwrapFd = argumentsToFileDescriptor(sandboxArgs, "bwrap");
     GUniquePtr<char> bwrapFdStr(g_strdup_printf("%d", bwrapFd));
     g_subprocess_launcher_take_fd(launcher, bwrapFd, bwrapFd);
 
