@@ -55,17 +55,24 @@ RemoteCDM::RemoteCDM(WeakPtr<RemoteCDMFactory>&& factory, RemoteCDMIdentifier&&
 {
 }
 
+#if !RELEASE_LOG_DISABLED
+void RemoteCDM::setLogIdentifier(const void* logIdentifier)
+{
+    if (m_factory)
+        m_factory->gpuProcessConnection().connection().send(Messages::RemoteCDMProxy::SetLogIdentifier(reinterpret_cast<uint64_t>(logIdentifier)), m_identifier);
+}
+#endif
+
 void RemoteCDM::getSupportedConfiguration(CDMKeySystemConfiguration&& configuration, LocalStorageAccess access, SupportedConfigurationCallback&& callback)
 {
     if (!m_factory) {
-        callback(WTF::nullopt);
+        callback(std::nullopt);
         return;
     }
 
     m_factory->gpuProcessConnection().connection().sendWithAsyncReply(Messages::RemoteCDMProxy::GetSupportedConfiguration(WTFMove(configuration), access), WTFMove(callback), m_identifier);
 }
 
-
 bool RemoteCDM::supportsConfiguration(const CDMKeySystemConfiguration&) const
 {
     ASSERT_NOT_REACHED();
@@ -118,7 +125,7 @@ RefPtr<CDMInstance> RemoteCDM::createInstance()
     m_factory->gpuProcessConnection().connection().sendSync(Messages::RemoteCDMProxy::CreateInstance(), Messages::RemoteCDMProxy::CreateInstance::Reply(identifier, configuration), m_identifier);
     if (!identifier)
         return nullptr;
-    return RemoteCDMInstance::create(makeWeakPtr(m_factory.get()), WTFMove(identifier), WTFMove(configuration));
+    return RemoteCDMInstance::create(m_factory.get(), WTFMove(identifier), WTFMove(configuration));
 }
 
 void RemoteCDM::loadAndInitialize()
@@ -132,10 +139,10 @@ void RemoteCDM::loadAndInitialize()
 RefPtr<SharedBuffer> RemoteCDM::sanitizeResponse(const SharedBuffer& response) const
 {
     // This check will be done, later, inside RemoteCDMInstanceSessionProxy::updateLicense().
-    return response.copy();
+    return response.makeContiguous();
 }
 
-Optional<String> RemoteCDM::sanitizeSessionId(const String& sessionId) const
+std::optional<String> RemoteCDM::sanitizeSessionId(const String& sessionId) const
 {
     // This check will be done, later, inside RemoteCDMInstanceSessionProxy::loadSession().
     return sessionId;
