@@ -30,6 +30,7 @@
 #include <WebCore/Editor.h>
 #include <WebCore/EventNames.h>
 #include <WebCore/Frame.h>
+#include <WebCore/FrameDestructionObserverInlines.h>
 #include <WebCore/KeyboardEvent.h>
 #include <WebCore/Node.h>
 #include <WebCore/PlatformKeyboardEvent.h>
@@ -168,7 +169,7 @@ static const char* interpretKeyEvent(const KeyboardEvent& event)
 
 static void handleKeyPress(Frame& frame, KeyboardEvent& event, const PlatformKeyboardEvent& platformEvent)
 {
-    String commandName = interpretKeyEvent(event);
+    auto commandName = String::fromLatin1(interpretKeyEvent(event));
 
     if (!commandName.isEmpty()) {
         frame.editor().command(commandName).execute();
@@ -190,7 +191,7 @@ static void handleKeyPress(Frame& frame, KeyboardEvent& event, const PlatformKey
 
 static void handleKeyDown(Frame& frame, KeyboardEvent& event, const PlatformKeyboardEvent&)
 {
-    String commandName = interpretKeyEvent(event);
+    auto commandName = String::fromLatin1(interpretKeyEvent(event));
     if (commandName.isEmpty())
         return;
 
