@@ -461,6 +461,15 @@ void DrawingAreaCoordinatedGraphics::updateBackingStoreState(uint64_t stateID, b
         forceRepaint();
 }
 
+void DrawingAreaCoordinatedGraphics::targetRefreshRateDidChange(unsigned rate)
+{
+    UNUSED_PARAM(rate);
+#if !USE(GRAPHICS_LAYER_TEXTURE_MAPPER)
+    if (m_layerTreeHost)
+        m_layerTreeHost->targetRefreshRateDidChange(rate);
+#endif
+}
+
 void DrawingAreaCoordinatedGraphics::didUpdate()
 {
     // We might get didUpdate messages from the UI process even after we've
@@ -624,7 +633,7 @@ void DrawingAreaCoordinatedGraphics::enterAcceleratedCompositingMode(GraphicsLay
 
     auto changeWindowScreen = [&] {
         // In order to ensure that we get a unique DisplayRefreshMonitor per-DrawingArea (necessary because ThreadedDisplayRefreshMonitor
-        // is driven by the ThreadedCompositor of the drawing area), give each page a unique DisplayID derived from WebPage's unique ID.
+        // is driven by the ThreadedCompositor of the drawing area), give each page a unique DisplayID derived from DrawingArea's unique ID.
         m_webPage.windowScreenDidChange(m_layerTreeHost->displayID(), std::nullopt);
     };
 
@@ -820,7 +829,7 @@ void DrawingAreaCoordinatedGraphics::display(UpdateInfo& updateInfo)
     IntSize bitmapSize = bounds.size();
     float deviceScaleFactor = m_webPage.corePage()->deviceScaleFactor();
     bitmapSize.scale(deviceScaleFactor);
-    auto bitmap = ShareableBitmap::createShareable(bitmapSize, { });
+    auto bitmap = ShareableBitmap::create(bitmapSize, { });
     if (!bitmap)
         return;
 
