@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010, 2015 Apple Inc. All rights reserved.
+ * Copyright (C) 2010-2022 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,6 +31,8 @@
 #include "ShareableBitmap.h"
 #include "WKPage.h"
 #include "WebCoreArgumentCoders.h"
+#include "WebFrame.h"
+#include "WebImage.h"
 #include "WebPage.h"
 #include "WebPageProxyMessages.h"
 #include <WebCore/DocumentMarkerController.h>
@@ -41,6 +43,7 @@
 #include <WebCore/FrameView.h>
 #include <WebCore/GeometryUtilities.h>
 #include <WebCore/GraphicsContext.h>
+#include <WebCore/ImageAnalysisQueue.h>
 #include <WebCore/ImageOverlay.h>
 #include <WebCore/Page.h>
 #include <WebCore/PageOverlayController.h>
@@ -66,17 +69,27 @@ FindController::~FindController()
 {
 }
 
+#if ENABLE(PDFKIT_PLUGIN)
+
+PluginView* FindController::mainFramePlugIn()
+{
+    return m_webPage->mainFramePlugIn();
+}
+
+#endif
+
 void FindController::countStringMatches(const String& string, OptionSet<FindOptions> options, unsigned maxMatchCount)
 {
     if (maxMatchCount == std::numeric_limits<unsigned>::max())
         --maxMatchCount;
-    
-    auto* pluginView = WebPage::pluginViewForFrame(m_webPage->mainFrame());
-    
+
     unsigned matchCount;
-    if (pluginView)
+#if ENABLE(PDFKIT_PLUGIN)
+    if (auto* pluginView = mainFramePlugIn())
         matchCount = pluginView->countFindMatches(string, core(options), maxMatchCount + 1);
-    else {
+    else
+#endif
+    {
         matchCount = m_webPage->corePage()->countFindMatches(string, core(options), maxMatchCount + 1);
         m_webPage->corePage()->unmarkAllTextMatches();
     }
@@ -121,13 +134,17 @@ static Frame* frameWithSelection(Page* page)
 void FindController::updateFindUIAfterPageScroll(bool found, const String& string, OptionSet<FindOptions> options, unsigned maxMatchCount, DidWrap didWrap, FindUIOriginator originator)
 {
     Frame* selectedFrame = frameWithSelection(m_webPage->corePage());
-    
-    auto* pluginView = WebPage::pluginViewForFrame(m_webPage->mainFrame());
+
+#if ENABLE(PDFKIT_PLUGIN)
+    auto* pluginView = mainFramePlugIn();
+#endif
 
     bool shouldShowOverlay = false;
 
     if (!found) {
+#if ENABLE(PDFKIT_PLUGIN)
         if (!pluginView)
+#endif
             m_webPage->corePage()->unmarkAllTextMatches();
 
         if (selectedFrame)
@@ -145,9 +162,11 @@ void FindController::updateFindUIAfterPageScroll(bool found, const String& strin
         unsigned matchCount = 1;
 
         if (shouldDetermineMatchIndex) {
+#if ENABLE(PDFKIT_PLUGIN)
             if (pluginView)
                 matchCount = pluginView->countFindMatches(string, core(options), maxMatchCount + 1);
             else
+#endif
                 matchCount = m_webPage->corePage()->countFindMatches(string, core(options), maxMatchCount + 1);
         }
 
@@ -155,11 +174,14 @@ void FindController::updateFindUIAfterPageScroll(bool found, const String& strin
             if (maxMatchCount == std::numeric_limits<unsigned>::max())
                 --maxMatchCount;
 
+#if ENABLE(PDFKIT_PLUGIN)
             if (pluginView) {
                 if (!shouldDetermineMatchIndex)
                     matchCount = pluginView->countFindMatches(string, core(options), maxMatchCount + 1);
                 shouldShowOverlay = false;
-            } else {
+            } else
+#endif
+            {
                 m_webPage->corePage()->unmarkAllTextMatches();
                 matchCount = m_webPage->corePage()->markAllMatchesForText(string, core(options), shouldShowHighlight, maxMatchCount + 1);
             }
@@ -213,7 +235,13 @@ void FindController::updateFindUIAfterPageScroll(bool found, const String& strin
 
 void FindController::findString(const String& string, OptionSet<FindOptions> options, unsigned maxMatchCount, CompletionHandler<void(bool)>&& completionHandler)
 {
-    auto* pluginView = WebPage::pluginViewForFrame(m_webPage->mainFrame());
+#if ENABLE(PDFKIT_PLUGIN)
+    auto* pluginView = mainFramePlugIn();
+#endif
+
+#if ENABLE(IMAGE_ANALYSIS)
+    m_webPage->corePage()->analyzeImagesForFindInPage();
+#endif
 
     WebCore::FindOptions coreOptions = core(options);
 
@@ -228,7 +256,10 @@ void FindController::findString(const String& string, OptionSet<FindOptions> opt
     willFindString();
 
     bool foundStringStartsAfterSelection = false;
-    if (!pluginView) {
+#if ENABLE(PDFKIT_PLUGIN)
+    if (!pluginView)
+#endif
+    {
         if (Frame* selectedFrame = frameWithSelection(m_webPage->corePage())) {
             if (selectedFrame->selection().selectionBounds().isEmpty()) {
                 auto result = m_webPage->corePage()->findTextMatches(string, coreOptions, maxMatchCount);
@@ -243,9 +274,11 @@ void FindController::findString(const String& string, OptionSet<FindOptions> opt
 
     bool found;
     DidWrap didWrap = DidWrap::No;
+#if ENABLE(PDFKIT_PLUGIN)
     if (pluginView)
         found = pluginView->findString(string, coreOptions, maxMatchCount);
     else
+#endif
         found = m_webPage->corePage()->findString(string, coreOptions, &didWrap);
 
     if (found) {
@@ -272,10 +305,9 @@ void FindController::findStringMatches(const String& string, OptionSet<FindOptio
     auto result = m_webPage->corePage()->findTextMatches(string, core(options), maxMatchCount);
     m_findMatches = WTFMove(result.ranges);
 
-    Vector<Vector<IntRect>> matchRects;
-    for (auto& range : m_findMatches)
-        matchRects.append(RenderObject::absoluteTextRects(range));
-
+    auto matchRects = m_findMatches.map([](auto& range) {
+        return RenderObject::absoluteTextRects(range);
+    });
     m_webPage->send(Messages::WebPageProxy::DidFindStringMatches(string, matchRects, result.indexForSelection));
 
     if (!options.contains(FindOptions::ShowOverlay) && !options.contains(FindOptions::ShowFindIndicator))
@@ -319,20 +351,18 @@ void FindController::getImageForFindMatch(uint32_t matchIndex)
     VisibleSelection oldSelection = frame->selection().selection();
     frame->selection().setSelection(m_findMatches[matchIndex]);
 
-    RefPtr<ShareableBitmap> selectionSnapshot = WebFrame::fromCoreFrame(*frame)->createSelectionSnapshot();
+    auto selectionSnapshot = WebFrame::fromCoreFrame(*frame)->createSelectionSnapshot();
 
     frame->selection().setSelection(oldSelection);
 
     if (!selectionSnapshot)
         return;
 
-    ShareableBitmap::Handle handle;
-    selectionSnapshot->createHandle(handle);
-
+    auto handle = selectionSnapshot->createHandle();
     if (handle.isNull())
         return;
 
-    m_webPage->send(Messages::WebPageProxy::DidGetImageForFindMatch(handle, matchIndex));
+    m_webPage->send(Messages::WebPageProxy::DidGetImageForFindMatch(selectionSnapshot->parameters(), handle, matchIndex));
 }
 
 void FindController::selectFindMatch(uint32_t matchIndex)
@@ -366,9 +396,11 @@ void FindController::hideFindUI()
     if (m_findPageOverlay)
         m_webPage->corePage()->pageOverlayController().uninstallPageOverlay(*m_findPageOverlay, PageOverlay::FadeMode::Fade);
 
-    if (auto* pluginView = WebPage::pluginViewForFrame(m_webPage->mainFrame()))
+#if ENABLE(PDFKIT_PLUGIN)
+    if (auto* pluginView = mainFramePlugIn())
         pluginView->findString(emptyString(), { }, 0);
     else
+#endif
         m_webPage->corePage()->unmarkAllTextMatches();
     
     hideFindIndicator();
