@@ -34,6 +34,7 @@
 #include <string>
 #include <wtf/HashMap.h>
 #include <wtf/Ref.h>
+#include <wtf/UniqueRef.h>
 #include <wtf/text/StringConcatenateNumbers.h>
 #include <wtf/text/StringHash.h>
 
@@ -306,52 +307,79 @@ TEST(WTF_HashMap, UniquePtrKey_TakeUsingRawPointer)
     EXPECT_EQ(1u, ConstructorDestructorCounter::destructionCount);
 }
 
+TEST(WTF_HashMap, UniqueRefValue)
+{
+    HashMap<int, UniqueRef<int>> map;
+    UniqueRef<int> five = makeUniqueRefWithoutFastMallocCheck<int>(5);
+    map.add(5, WTFMove(five));
+    EXPECT_TRUE(map.contains(5));
+    int* shouldBeFive = map.get(5);
+    EXPECT_EQ(*shouldBeFive, 5);
+    std::unique_ptr<int> takenFive = map.take(5);
+    EXPECT_EQ(*takenFive, 5);
+    map.ensure(6, [] {
+        return makeUniqueRefWithoutFastMallocCheck<int>(6);
+    });
+    EXPECT_FALSE(map.contains(5));
+    EXPECT_TRUE(map.contains(6));
+    for (UniqueRef<int>& a : map.values())
+        EXPECT_EQ(a.get(), 6);
+}
+
 TEST(WTF_HashMap, RefPtrKey_Add)
 {
-    DerivedRefLogger a("a");
+    {
+        DerivedRefLogger a("a");
 
-    HashMap<RefPtr<RefLogger>, int> map;
+        HashMap<RefPtr<RefLogger>, int> map;
 
-    RefPtr<RefLogger> ptr(&a);
-    map.add(ptr, 0);
+        RefPtr<RefLogger> ptr(&a);
+        map.add(ptr, 0);
+    }
 
-    ASSERT_STREQ("ref(a) ref(a) ", takeLogStr().c_str());
+    ASSERT_STREQ("ref(a) ref(a) deref(a) deref(a) ", takeLogStr().c_str());
 }
 
 TEST(WTF_HashMap, RefPtrKey_AddUsingRelease)
 {
-    DerivedRefLogger a("a");
+    {
+        DerivedRefLogger a("a");
 
-    HashMap<RefPtr<RefLogger>, int> map;
+        HashMap<RefPtr<RefLogger>, int> map;
 
-    RefPtr<RefLogger> ptr(&a);
-    map.add(WTFMove(ptr), 0);
+        RefPtr<RefLogger> ptr(&a);
+        map.add(WTFMove(ptr), 0);
+    }
 
-    EXPECT_STREQ("ref(a) ", takeLogStr().c_str());
+    EXPECT_STREQ("ref(a) deref(a) ", takeLogStr().c_str());
 }
 
 TEST(WTF_HashMap, RefPtrKey_AddUsingMove)
 {
-    DerivedRefLogger a("a");
+    {
+        DerivedRefLogger a("a");
 
-    HashMap<RefPtr<RefLogger>, int> map;
+        HashMap<RefPtr<RefLogger>, int> map;
 
-    RefPtr<RefLogger> ptr(&a);
-    map.add(WTFMove(ptr), 0);
+        RefPtr<RefLogger> ptr(&a);
+        map.add(WTFMove(ptr), 0);
+    }
 
-    EXPECT_STREQ("ref(a) ", takeLogStr().c_str());
+    EXPECT_STREQ("ref(a) deref(a) ", takeLogStr().c_str());
 }
 
 TEST(WTF_HashMap, RefPtrKey_AddUsingRaw)
 {
-    DerivedRefLogger a("a");
+    {
+        DerivedRefLogger a("a");
 
-    HashMap<RefPtr<RefLogger>, int> map;
+        HashMap<RefPtr<RefLogger>, int> map;
 
-    RefPtr<RefLogger> ptr(&a);
-    map.add(ptr.get(), 0);
+        RefPtr<RefLogger> ptr(&a);
+        map.add(ptr.get(), 0);
+    }
 
-    EXPECT_STREQ("ref(a) ref(a) ", takeLogStr().c_str());
+    EXPECT_STREQ("ref(a) ref(a) deref(a) deref(a) ", takeLogStr().c_str());
 }
 
 TEST(WTF_HashMap, RefPtrKey_AddKeyAlreadyPresent)
@@ -374,6 +402,10 @@ TEST(WTF_HashMap, RefPtrKey_AddKeyAlreadyPresent)
     }
 
     EXPECT_STREQ("ref(a) deref(a) ", takeLogStr().c_str());
+
+    map.clear();
+
+    EXPECT_STREQ("deref(a) ", takeLogStr().c_str());
 }
 
 TEST(WTF_HashMap, RefPtrKey_AddUsingReleaseKeyAlreadyPresent)
@@ -396,6 +428,10 @@ TEST(WTF_HashMap, RefPtrKey_AddUsingReleaseKeyAlreadyPresent)
     }
 
     EXPECT_STREQ("ref(a) deref(a) ", takeLogStr().c_str());
+
+    map.clear();
+
+    EXPECT_STREQ("deref(a) ", takeLogStr().c_str());
 }
 
 TEST(WTF_HashMap, RefPtrKey_AddUsingMoveKeyAlreadyPresent)
@@ -418,118 +454,142 @@ TEST(WTF_HashMap, RefPtrKey_AddUsingMoveKeyAlreadyPresent)
     }
 
     EXPECT_STREQ("ref(a) deref(a) ", takeLogStr().c_str());
+
+    map.clear();
+
+    EXPECT_STREQ("deref(a) ", takeLogStr().c_str());
 }
 
 TEST(WTF_HashMap, RefPtrKey_Set)
 {
-    DerivedRefLogger a("a");
+    {
+        DerivedRefLogger a("a");
 
-    HashMap<RefPtr<RefLogger>, int> map;
+        HashMap<RefPtr<RefLogger>, int> map;
 
-    RefPtr<RefLogger> ptr(&a);
-    map.set(ptr, 0);
+        RefPtr<RefLogger> ptr(&a);
+        map.set(ptr, 0);
+    }
 
-    ASSERT_STREQ("ref(a) ref(a) ", takeLogStr().c_str());
+    ASSERT_STREQ("ref(a) ref(a) deref(a) deref(a) ", takeLogStr().c_str());
 }
 
 TEST(WTF_HashMap, RefPtrKey_SetUsingRelease)
 {
-    DerivedRefLogger a("a");
+    {
+        DerivedRefLogger a("a");
 
-    HashMap<RefPtr<RefLogger>, int> map;
+        HashMap<RefPtr<RefLogger>, int> map;
 
-    RefPtr<RefLogger> ptr(&a);
-    map.set(WTFMove(ptr), 0);
+        RefPtr<RefLogger> ptr(&a);
+        map.set(WTFMove(ptr), 0);
+    }
 
-    EXPECT_STREQ("ref(a) ", takeLogStr().c_str());
+    EXPECT_STREQ("ref(a) deref(a) ", takeLogStr().c_str());
 }
 
 
 TEST(WTF_HashMap, RefPtrKey_SetUsingMove)
 {
-    DerivedRefLogger a("a");
+    {
+        DerivedRefLogger a("a");
 
-    HashMap<RefPtr<RefLogger>, int> map;
+        HashMap<RefPtr<RefLogger>, int> map;
 
-    RefPtr<RefLogger> ptr(&a);
-    map.set(WTFMove(ptr), 0);
+        RefPtr<RefLogger> ptr(&a);
+        map.set(WTFMove(ptr), 0);
+    }
 
-    EXPECT_STREQ("ref(a) ", takeLogStr().c_str());
+    EXPECT_STREQ("ref(a) deref(a) ", takeLogStr().c_str());
 }
 
 TEST(WTF_HashMap, RefPtrKey_SetUsingRaw)
 {
-    DerivedRefLogger a("a");
+    {
+        DerivedRefLogger a("a");
 
-    HashMap<RefPtr<RefLogger>, int> map;
+        HashMap<RefPtr<RefLogger>, int> map;
 
-    RefPtr<RefLogger> ptr(&a);
-    map.set(ptr.get(), 0);
+        RefPtr<RefLogger> ptr(&a);
+        map.set(ptr.get(), 0);
+    }
 
-    EXPECT_STREQ("ref(a) ref(a) ", takeLogStr().c_str());
+    EXPECT_STREQ("ref(a) ref(a) deref(a) deref(a) ", takeLogStr().c_str());
 }
 
 TEST(WTF_HashMap, RefPtrKey_SetKeyAlreadyPresent)
 {
-    DerivedRefLogger a("a");
-
-    HashMap<RefPtr<RefLogger>, int> map;
-
-    RefPtr<RefLogger> ptr(&a);
-    map.set(ptr, 0);
-
-    EXPECT_STREQ("ref(a) ref(a) ", takeLogStr().c_str());
-
     {
-        RefPtr<RefLogger> ptr2(&a);
-        auto addResult = map.set(ptr2, 1);
-        EXPECT_FALSE(addResult.isNewEntry);
-        EXPECT_EQ(1, map.get(ptr.get()));
+        DerivedRefLogger a("a");
+
+        HashMap<RefPtr<RefLogger>, int> map;
+
+        RefPtr<RefLogger> ptr(&a);
+        map.set(ptr, 0);
+
+        EXPECT_STREQ("ref(a) ref(a) ", takeLogStr().c_str());
+
+        {
+            RefPtr<RefLogger> ptr2(&a);
+            auto addResult = map.set(ptr2, 1);
+            EXPECT_FALSE(addResult.isNewEntry);
+            EXPECT_EQ(1, map.get(ptr.get()));
+        }
+
+        EXPECT_STREQ("ref(a) deref(a) ", takeLogStr().c_str());
     }
 
-    EXPECT_STREQ("ref(a) deref(a) ", takeLogStr().c_str());
+    EXPECT_STREQ("deref(a) deref(a) ", takeLogStr().c_str());
 }
 
 TEST(WTF_HashMap, RefPtrKey_SetUsingReleaseKeyAlreadyPresent)
 {
-    DerivedRefLogger a("a");
-
-    HashMap<RefPtr<RefLogger>, int> map;
-
-    RefPtr<RefLogger> ptr(&a);
-    map.set(ptr, 0);
-
-    EXPECT_STREQ("ref(a) ref(a) ", takeLogStr().c_str());
-
     {
-        RefPtr<RefLogger> ptr2(&a);
-        auto addResult = map.set(WTFMove(ptr2), 1);
-        EXPECT_FALSE(addResult.isNewEntry);
-        EXPECT_EQ(1, map.get(ptr.get()));
+        DerivedRefLogger a("a");
+
+        HashMap<RefPtr<RefLogger>, int> map;
+
+        RefPtr<RefLogger> ptr(&a);
+        map.set(ptr, 0);
+
+        EXPECT_STREQ("ref(a) ref(a) ", takeLogStr().c_str());
+
+        {
+            RefPtr<RefLogger> ptr2(&a);
+            auto addResult = map.set(WTFMove(ptr2), 1);
+            EXPECT_FALSE(addResult.isNewEntry);
+            EXPECT_EQ(1, map.get(ptr.get()));
+        }
+
+        EXPECT_STREQ("ref(a) deref(a) ", takeLogStr().c_str());
     }
 
-    EXPECT_STREQ("ref(a) deref(a) ", takeLogStr().c_str());
+    EXPECT_STREQ("deref(a) deref(a) ", takeLogStr().c_str());
 }
 
 TEST(WTF_HashMap, RefPtrKey_SetUsingMoveKeyAlreadyPresent)
 {
-    DerivedRefLogger a("a");
-
-    HashMap<RefPtr<RefLogger>, int> map;
-
-    RefPtr<RefLogger> ptr(&a);
-    map.set(ptr, 0);
-
-    EXPECT_STREQ("ref(a) ref(a) ", takeLogStr().c_str());
-
     {
-        RefPtr<RefLogger> ptr2(&a);
-        auto addResult = map.set(WTFMove(ptr2), 1);
-        EXPECT_FALSE(addResult.isNewEntry);
-        EXPECT_EQ(1, map.get(ptr.get()));
+        DerivedRefLogger a("a");
+
+        HashMap<RefPtr<RefLogger>, int> map;
+
+        RefPtr<RefLogger> ptr(&a);
+        map.set(ptr, 0);
+
+        EXPECT_STREQ("ref(a) ref(a) ", takeLogStr().c_str());
+
+        {
+            RefPtr<RefLogger> ptr2(&a);
+            auto addResult = map.set(WTFMove(ptr2), 1);
+            EXPECT_FALSE(addResult.isNewEntry);
+            EXPECT_EQ(1, map.get(ptr.get()));
+        }
+
+        EXPECT_STREQ("ref(a) deref(a) ", takeLogStr().c_str());
     }
 
-    EXPECT_STREQ("ref(a) deref(a) ", takeLogStr().c_str());
+    EXPECT_STREQ("deref(a) deref(a) ", takeLogStr().c_str());
 }
 
 TEST(WTF_HashMap, Ensure)
@@ -590,6 +650,10 @@ TEST(WTF_HashMap, Ensure_RefPtr)
 
     map.ensure(1, [&] { return RefPtr<RefLogger>(&a); });
     EXPECT_STREQ("", takeLogStr().c_str());
+
+    map.clear();
+
+    EXPECT_STREQ("deref(a) ", takeLogStr().c_str());
 }
 
 class ObjectWithRefLogger {
@@ -604,14 +668,14 @@ public:
 };
 
 
-void testMovingUsingEnsure(Ref<RefLogger>&& logger)
+static void testMovingUsingEnsure(Ref<RefLogger>&& logger)
 {
     HashMap<unsigned, std::unique_ptr<ObjectWithRefLogger>> map;
     
     map.ensure(1, [&] { return makeUnique<ObjectWithRefLogger>(WTFMove(logger)); });
 }
 
-void testMovingUsingAdd(Ref<RefLogger>&& logger)
+static void testMovingUsingAdd(Ref<RefLogger>&& logger)
 {
     HashMap<unsigned, std::unique_ptr<ObjectWithRefLogger>> map;
 
@@ -902,7 +966,7 @@ TEST(WTF_HashMap, Ref_Value)
         
         auto aOut = map.take(1);
         ASSERT_TRUE(static_cast<bool>(aOut));
-        ASSERT_EQ(&a, aOut.value().ptr());
+        ASSERT_EQ(&a, aOut.get());
     }
 
     ASSERT_STREQ("ref(a) deref(a) ", takeLogStr().c_str());
@@ -984,7 +1048,7 @@ TEST(WTF_HashMap, RefMappedToNonZeroEmptyValue)
         static Ref<Key> create() { return adoptRef(*new Key); }
     };
 
-    static_assert(!WTF::HashTraits<Value>::emptyValueIsZero, "");
+    static_assert(!WTF::HashTraits<Value>::emptyValueIsZero);
 
     HashMap<Ref<Key>, Value> map;
     Vector<std::pair<Ref<Key>, int32_t>> vectorMap;
@@ -1119,4 +1183,36 @@ TEST(WTF_HashMap, Random_IsEvenlyDistributedAfterRemove)
     }
 }
 
+class TestObjectWithCustomDestructor {
+    WTF_MAKE_FAST_ALLOCATED;
+public:
+    TestObjectWithCustomDestructor(Function<void()>&& runInDestructor)
+        : m_runInDestructor(WTFMove(runInDestructor))
+    { }
+
+    ~TestObjectWithCustomDestructor()
+    {
+        m_runInDestructor();
+    }
+
+private:
+    Function<void()> m_runInDestructor;
+};
+
+TEST(WTF_HashMap, Clear_Reenter)
+{
+    HashMap<uint64_t, std::unique_ptr<TestObjectWithCustomDestructor>> map;
+    for (unsigned i = 1; i <= 10; ++i) {
+        map.add(i, makeUnique<TestObjectWithCustomDestructor>([&map] {
+            map.clear();
+            EXPECT_EQ(0U, map.size());
+            EXPECT_TRUE(map.isEmpty());
+        }));
+    }
+    EXPECT_EQ(10U, map.size());
+    map.clear();
+    EXPECT_EQ(0U, map.size());
+    EXPECT_TRUE(map.isEmpty());
+}
+
 } // namespace TestWebKitAPI
