@@ -62,7 +62,6 @@ public:
 
     void setConnection(Ref<SocketConnection>&& connection)
     {
-        g_assert_true(connection.ptr());
         m_connection = WTFMove(connection);
         g_main_loop_quit(m_mainLoop.get());
     }
@@ -89,15 +88,9 @@ public:
     {
         static long sequenceID = 0;
         StringBuilder messageBuilder;
-        messageBuilder.appendLiteral("{\"id\":");
-        messageBuilder.appendNumber(++sequenceID);
-        messageBuilder.appendLiteral(",\"method\":\"Automation.");
-        messageBuilder.append(command);
-        messageBuilder.append('"');
-        if (!parameters.isNull()) {
-            messageBuilder.appendLiteral(",\"params\":");
-            messageBuilder.append(parameters);
-        }
+        messageBuilder.append("{\"id\":", ++sequenceID, ",\"method\":\"Automation.", command, '"');
+        if (!parameters.isNull())
+            messageBuilder.append(",\"params\":", parameters);
         messageBuilder.append('}');
         m_connection->sendMessage("SendMessageToBackend", g_variant_new("(tts)", m_connectionID, m_target.id, messageBuilder.toString().utf8().data()));
     }
@@ -199,7 +192,7 @@ public:
         m_createWebViewWasCalled = false;
         m_message = CString();
         auto signalID = g_signal_connect(m_session, "create-web-view", G_CALLBACK(createWebViewCallback), this);
-        sendCommandToBackend("createBrowsingContext");
+        sendCommandToBackend("createBrowsingContext"_s);
         g_main_loop_run(m_mainLoop.get());
         g_signal_handler_disconnect(m_session, signalID);
         g_assert_true(m_createWebViewWasCalled);
@@ -219,7 +212,7 @@ public:
         m_createWebViewInWindowWasCalled = false;
         m_message = { };
         auto signalID = g_signal_connect(m_session, "create-web-view::window", G_CALLBACK(createWebViewInWindowCallback), this);
-        sendCommandToBackend("createBrowsingContext", "{\"presentationHint\":\"Window\"}");
+        sendCommandToBackend("createBrowsingContext"_s, "{\"presentationHint\":\"Window\"}"_s);
         g_main_loop_run(m_mainLoop.get());
         g_signal_handler_disconnect(m_session, signalID);
         g_assert_true(m_createWebViewInWindowWasCalled);
@@ -237,7 +230,7 @@ public:
         m_createWebViewInTabWasCalled = false;
         m_message = { };
         auto signalID = g_signal_connect(m_session, "create-web-view::tab", G_CALLBACK(createWebViewInTabCallback), this);
-        sendCommandToBackend("createBrowsingContext", "{\"presentationHint\":\"Tab\"}");
+        sendCommandToBackend("createBrowsingContext"_s, "{\"presentationHint\":\"Tab\"}"_s);
         g_main_loop_run(m_mainLoop.get());
         g_signal_handler_disconnect(m_session, signalID);
         g_assert_true(m_createWebViewInTabWasCalled);
@@ -307,7 +300,7 @@ const SocketConnection::MessageHandlers AutomationTest::s_messageHandlers = {
 
 static void testAutomationSessionRequestSession(AutomationTest* test, gconstpointer)
 {
-    String sessionID = createCanonicalUUIDString();
+    String sessionID = createVersion4UUIDString();
     // WebKitAutomationSession::automation-started is never emitted if automation is not enabled.
     g_assert_false(webkit_web_context_is_automation_allowed(test->m_webContext.get()));
     auto* session = test->requestSession(sessionID.utf8().data());
