@@ -189,6 +189,11 @@ static void emitURIChanged(GDBusConnection* connection, const char* uri)
 
 static void uriChangedCallback(WebKitWebPage* webPage, GParamSpec* pspec, WebKitWebExtension* extension)
 {
+    WebKitFrame* frame = webkit_web_page_get_main_frame(webPage);
+    g_assert_true(WEBKIT_IS_FRAME(frame));
+    g_assert_true(webkit_frame_is_main_frame(frame));
+    g_assert_cmpstr(webkit_web_page_get_uri(webPage), ==, webkit_frame_get_uri(frame));
+
     gpointer data = g_object_get_data(G_OBJECT(extension), "dbus-connection");
     if (data)
         emitURIChanged(G_DBUS_CONNECTION(data), webkit_web_page_get_uri(webPage));
@@ -549,6 +554,7 @@ static char* echoCallback(const char* message)
 
 static void windowObjectCleared(WebKitScriptWorld* world, WebKitWebPage* page, WebKitFrame* frame, gpointer)
 {
+    webkit_web_page_send_message_to_view(page, webkit_user_message_new("WindowObjectCleared", nullptr), nullptr, nullptr, nullptr);
     GRefPtr<JSCContext> jsContext = adoptGRef(webkit_frame_get_js_context_for_script_world(frame, world));
     g_assert_true(JSC_IS_CONTEXT(jsContext.get()));
     GRefPtr<JSCValue> function = adoptGRef(jsc_value_new_function(jsContext.get(), "echo", G_CALLBACK(echoCallback), NULL, NULL, G_TYPE_STRING, 1, G_TYPE_STRING));
@@ -560,6 +566,11 @@ static void windowObjectCleared(WebKitScriptWorld* world, WebKitWebPage* page, W
     jsc_context_set_value(jsContext.get(), "GFile", constructor.get());
 }
 
+static void isolatedWorldWindowObjectCleared(WebKitScriptWorld* world, WebKitWebPage* page, WebKitFrame* frame, gpointer)
+{
+    webkit_web_page_send_message_to_view(page, webkit_user_message_new("WindowObjectClearedIsolatedWorld", nullptr), nullptr, nullptr, nullptr);
+}
+
 static WebKitWebPage* getWebPage(WebKitWebExtension* extension, uint64_t pageID, GDBusMethodInvocation* invocation)
 {
     WebKitWebPage* page = webkit_web_extension_get_page(extension, pageID);
@@ -669,6 +680,9 @@ static void dbusConnectionCreated(GObject*, GAsyncResult* result, gpointer userD
         g_error("Failed to register object: %s\n", error->message);
 
     g_object_set_data_full(G_OBJECT(userData), "dbus-connection", connection, g_object_unref);
+    g_signal_connect_object(connection, "closed", G_CALLBACK(+[](GDBusConnection*, gboolean, GError*, gpointer userData) {
+        g_object_set_data_full(G_OBJECT(userData), "dbus-connection", nullptr, nullptr);
+    }), userData, static_cast<GConnectFlags>(0));
     while (delayedSignalsQueue.size()) {
         DelayedSignal delayedSignal = delayedSignalsQueue.takeFirst();
         switch (delayedSignal.type) {
@@ -704,7 +718,7 @@ static void registerGResource(void)
     g_resource_unref(resource);
 }
 
-extern "C" void webkit_web_extension_initialize_with_user_data(WebKitWebExtension* extension, GVariant* userData)
+extern "C" WTF_EXPORT_DECLARATION void webkit_web_extension_initialize_with_user_data(WebKitWebExtension* extension, GVariant* userData)
 {
     WebKitScriptWorld* isolatedWorld = webkit_script_world_new_with_name("WebExtensionTestScriptWorld");
     g_assert_true(WEBKIT_IS_SCRIPT_WORLD(isolatedWorld));
@@ -714,6 +728,7 @@ extern "C" void webkit_web_extension_initialize_with_user_data(WebKitWebExtensio
     g_signal_connect(extension, "user-message-received", G_CALLBACK(extensionMessageReceivedCallback), nullptr);
     g_signal_connect(extension, "page-created", G_CALLBACK(pageCreatedCallback), extension);
     g_signal_connect(webkit_script_world_get_default(), "window-object-cleared", G_CALLBACK(windowObjectCleared), nullptr);
+    g_signal_connect(isolatedWorld, "window-object-cleared", G_CALLBACK(isolatedWorldWindowObjectCleared), nullptr);
 
     registerGResource();
 
